/* IBM_PROLOG_BEGIN_TAG                                                   */
/* This is an automatically generated prolog.                             */
/*                                                                        */
/* $Source: src/import/chips/p9/common/include/p9a_mc_scom_addresses.H $  */
/*                                                                        */
/* OpenPOWER sbe Project                                                  */
/*                                                                        */
/* Contributors Listed Below - COPYRIGHT 2018,2019                        */
/* [+] International Business Machines Corp.                              */
/*                                                                        */
/*                                                                        */
/* Licensed under the Apache License, Version 2.0 (the "License");        */
/* you may not use this file except in compliance with the License.       */
/* You may obtain a copy of the License at                                */
/*                                                                        */
/*     http://www.apache.org/licenses/LICENSE-2.0                         */
/*                                                                        */
/* Unless required by applicable law or agreed to in writing, software    */
/* distributed under the License is distributed on an "AS IS" BASIS,      */
/* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or        */
/* implied. See the License for the specific language governing           */
/* permissions and limitations under the License.                         */
/*                                                                        */
/* IBM_PROLOG_END_TAG                                                     */
///
/// @file p9a_mc_scom_addresses.H
/// @brief  Defines constants for scom addresses
///
// *HWP HWP Owner: Ben Gass <bgass@us.ibm.com>
// *HWP FW Owner: Thi Tran <thi@us.ibm.com>
// *HWP Team: SOA
// *HWP Level: 1
// *HWP Consumed by: FSP:HB:HS:OCC:SBE:CME:SGPE:PGPE:FPPE:IPPE

/*---------------------------------------------------------------
 *
 *---------------------------------------------------------------
 *
 *  Issues:
 *
 * Closed
 *    TOD reg same address.             HW323439
 *       - Issue was closed with the explaination "same as p8"
 *    IO0 registers need fixed.         HW320437
 *    PHB registers need fixed.         HW320416  ( all regs commented out now )
 *    OSC/perv regs same address.       HW323437
 *    MC regs with same address.        HW323435 (matteo)
 *         Duplicate IOM registers.          HW320456 (designers)
 *    PEC Sat_id issue                  HW329652
 *    PB.PB_PPE registers need fixed.   HW320435
 *    EX05 registers need fixed.        HW320427  (9020) - L2 ring id's are incorrect
 *    IOFPPE registers need fixed.      HW320424  (9020) - Investigate NULL scope
 *    PEC addresses are wrong.          HW322598  (9020)
 *    MC registers need fixed.          HW320433
 *    VA.VA_NORTH registers need fixed. HW320436
 *
 * Format:
 *
 *  <UNIT>_<SUBUNIT>_<INSTANCE>_<REGISTER>_<ACCESS/TYPE>
 *
 *  Notes: Subunits are only added to make names unique when
 *  there are name collisions.
 *  Only units with more than one instance has instance numbers.
 *  If there is only one, the instance number is omitted.
 *
 *  Instance numbers are chiplet id's for the PERV unit.  The
 *  chiplet id's are mapped to their name and used instead of
 *  instance numbers.  See bellow.
 *
 *  For registers with a single access type the type and access
 *  methods are omitted.
 *
 *  For access types where all bits have the same access methods, the
 *  access method is appended to the name.  If the access methods
 *  are different for some bits, the access type is appended to the
 *  name _SCOM instead of _RO.  The _RW(X) access method is omitted
 *  and assumed to be default.
 *
 *  Valid units / subunits
 *    PU                    : No unit chip level
 *        MCD0[0..1]        : mcd subunit
 *        PIB2OPB[0..1]     : PIB2OPB subunit
 *        OTPROM[0..1]      : otprom subunit
 *        NPU               : common npu subunit
 *        NPU[0..2]         : Npu stacks 0 to 2
 *          CTL             : Npu CTL subunit
 *          DAT             : Npu DAT subunit
 *          SM[0..3]        : Npu SM subunits
 *          NTL[0..1]       : Npu NTL subunit
 *    PERV                  : Pervasive
 *        FSI2PIB           : subunit
 *        FSISHIFT          : subunit
 *        FSII2C            : subunit
 *        FSB               : subunit
 *    EX                    : Ex unit (1/2 quad, 2 cores)
 *        L2                : L2 subunit
 *        L3                : L3 subunit
 *    PEC                   : PCI Pec unit
 *        STACK0            : subunit
 *        STACK1            : subunit
 *        STACK2            : subunit
 *    C                     : core
 *    EQ                    : quad
 *    OBUS                  : obus
 *    CAPP                  : capp
 *    MCBIST                : mcbist
 *    MCA                   : mca
 *    NVBUS                 : (not implemented yet)
 *    PHB                   : (not implemented yet)
 *    MI                    : (not implemented yet)
 *    DMI                   : (not implemented yet)
 *    MCS                   : (not implemented yet)
 *    OCC                   : (not implemented yet)
 *    PPE                   : (not implemented yet)
 *    SBE                   : (not implemented yet)
 *    XBUS                  : (not implemented yet)
 *
 * Pervasive instance names follow chiplet id.
 *
 * Instance/  | Chiplet
 * Chiplet    | name
 * -----------+-----------
 * 0x00       | PIB
 * 0x01       | TP
 * 0x02       | N0
 * 0x03       | N1
 * 0x04       | N2
 * 0x05       | N3
 * 0x06       | XB
 * 0x07       | MC01
 * 0x08       | MC23
 * 0x09       | OB0
 * 0x0A       | OB1
 * 0x0B       | OB2
 * 0x0C       | OB3
 * 0x0D       | PCI0
 * 0x0E       | PCI1
 * 0x0F       | PCI2
 * 0x10       | EP00
 * 0x11       | EP01
 * 0x12       | EP02
 * 0x13       | EP03
 * 0x14       | EP04
 * 0x15       | EP05
 * 0x20       | EC00
 * 0x21       | EC01
 * 0x22       | EC02
 * 0x23       | EC03
 * 0x24       | EC04
 * 0x25       | EC05
 * 0x26       | EC06
 * 0x27       | EC07
 * 0x28       | EC08
 * 0x29       | EC09
 * 0x2A       | EC10
 * 0x2B       | EC11
 * 0x2C       | EC12
 * 0x2D       | EC13
 * 0x2E       | EC14
 * 0x2F       | EC15
 * 0x30       | EC16
 * 0x31       | EC17
 * 0x32       | EC18
 * 0x33       | EC19
 * 0x34       | EC20
 * 0x35       | EC21
 * 0x36       | EC22
 * 0x37       | EC23
 *
 *
 *---------------------------------------------------------------
 *
 * NOTES:
 *
 * there is a SPR ring that goes around the chip with an
 * address(0:9)/tid(0:1) (thread id)/mfspr_data(0:63) and return mfspr_data_v/mfspr_data(0:63)
 *
 *    Add PU_<SUBUNITS>  (only if there are conflicts on these registers)
 *       0x0001XXXX OTPROM
 *       0x0002XXXX FSIM0
 *       0x0003XXXX FSIM1
 *       0x0004XXXX TOD
 *       0x0005XXXX FSI_MBOX
 *       0x0006XXXX OCI_BRIDGE
 *       0x0007XXXX SPI_ADC
 *       0x0008XXXX PIBMEM
 *       0x0009XXXX ADU
 *       0x000AXXXX I2CM
 *       0x000BXXXX SBE_FIFO
 *       0x000DXXXX PSU
 *       0x000EXXXX SBE
 *
 * 0x0000100A for FSI2PIB  => PERV_FSI2PIB
 * 0x00000Cxx for FSISHIFT =>  PERV_FSISHIFT
 * 0x000018xx for FSI I2C =>  PERV_FSII2C
 * 0x000024xx for FSI SBEFIFO => PERV_FSB
 *
 * 0x00000400 PEEK_TABLE
 * 0x00000800 FSI_SLAVE
 * 0x00000C00 FSI_SHIFT
 * 0x00001000 FSI2PIB
 * 0x00001400 FSI_SCRATCHPAD
 * 0x00001800 FSI_I2CM
 * 0x00002400 FSI_SBE_FIFO
 *
 * address fields
 * 0xCCRPxxxx
 *
 * CC=chiplet
 * R=always 0?
 * P=port
 *   0=gpregs
 *   1=normal unit scom ring (exclude)
 *   3=clock controller
 *   4=firs
 *   5=cpm
 *
 * =============================================================================
 * Compiling
 *
 * Precompile the header to save time on subsquent compiles:
 * g++ -I. -c scom_addresses.H
 *
 * Use these options to help reduce the binary size
 * g++ -I. -Os -fdata-sections -ffunction-sections <file>.C -o <output> -Wl,--gc-sections
 *
 *
 *---------------------------------------------------------------
 */



#ifndef __P9A_MC_SCOM_ADDRESSES_H
#define __P9A_MC_SCOM_ADDRESSES_H




static const uint64_t P9A_MC_AACR                                             = 0x07012329ull;

static const uint64_t P9A_MC_0_AACR                                           = 0x07012329ull;

static const uint64_t P9A_MC_1_AACR                                           = 0x08012329ull;


static const uint64_t P9A_MC_AADR                                             = 0x0701232Aull;

static const uint64_t P9A_MC_0_AADR                                           = 0x0701232Aull;

static const uint64_t P9A_MC_1_AADR                                           = 0x0801232Aull;


static const uint64_t P9A_MC_AAER                                             = 0x0701232Bull;

static const uint64_t P9A_MC_0_AAER                                           = 0x0701232Bull;

static const uint64_t P9A_MC_1_AAER                                           = 0x0801232Bull;


static const uint64_t P9A_MC_ADDR_TRAP_REG                                    = 0x07010003ull;

static const uint64_t P9A_MC_0_ADDR_TRAP_REG                                  = 0x07010003ull;

static const uint64_t P9A_MC_1_ADDR_TRAP_REG                                  = 0x08010003ull;


static const uint64_t P9A_MC_ASSIST_INTERRUPT_REG                             = 0x070F0011ull;

static const uint64_t P9A_MC_0_ASSIST_INTERRUPT_REG                           = 0x070F0011ull;

static const uint64_t P9A_MC_1_ASSIST_INTERRUPT_REG                           = 0x080F0011ull;


static const uint64_t P9A_MC_ATOMIC_LOCK_MASK_LATCH_REG                       = 0x07010007ull;

static const uint64_t P9A_MC_0_ATOMIC_LOCK_MASK_LATCH_REG                     = 0x07010007ull;

static const uint64_t P9A_MC_1_ATOMIC_LOCK_MASK_LATCH_REG                     = 0x08010007ull;


static const uint64_t P9A_MC_ATOMIC_LOCK_REG                                  = 0x070F03FFull;

static const uint64_t P9A_MC_0_ATOMIC_LOCK_REG                                = 0x070F03FFull;

static const uint64_t P9A_MC_1_ATOMIC_LOCK_REG                                = 0x080F03FFull;


static const uint64_t P9A_MC_ATTN_INTERRUPT_REG                               = 0x070F001Aull;

static const uint64_t P9A_MC_0_ATTN_INTERRUPT_REG                             = 0x070F001Aull;

static const uint64_t P9A_MC_1_ATTN_INTERRUPT_REG                             = 0x080F001Aull;


static const uint64_t P9A_MC_BIST                                             = 0x0703000Bull;

static const uint64_t P9A_MC_0_BIST                                           = 0x0703000Bull;

static const uint64_t P9A_MC_1_BIST                                           = 0x0803000Bull;


static const uint64_t P9A_MC_CC_ATOMIC_LOCK_REG                               = 0x070303FFull;

static const uint64_t P9A_MC_0_CC_ATOMIC_LOCK_REG                             = 0x070303FFull;

static const uint64_t P9A_MC_1_CC_ATOMIC_LOCK_REG                             = 0x080303FFull;


static const uint64_t P9A_MC_CC_PROTECT_MODE_REG                              = 0x070303FEull;

static const uint64_t P9A_MC_0_CC_PROTECT_MODE_REG                            = 0x070303FEull;

static const uint64_t P9A_MC_1_CC_PROTECT_MODE_REG                            = 0x080303FEull;


static const uint64_t P9A_MC_CLKRATIO                                         = 0x070123F0ull;

static const uint64_t P9A_MC_0_CLKRATIO                                       = 0x070123F0ull;

static const uint64_t P9A_MC_1_CLKRATIO                                       = 0x080123F0ull;


static const uint64_t P9A_MC_CLK_REGION                                       = 0x07030006ull;

static const uint64_t P9A_MC_0_CLK_REGION                                     = 0x07030006ull;

static const uint64_t P9A_MC_1_CLK_REGION                                     = 0x08030006ull;


static const uint64_t P9A_MC_CLOCK_STAT_ARY                                   = 0x0703000Aull;

static const uint64_t P9A_MC_0_CLOCK_STAT_ARY                                 = 0x0703000Aull;

static const uint64_t P9A_MC_1_CLOCK_STAT_ARY                                 = 0x0803000Aull;


static const uint64_t P9A_MC_CLOCK_STAT_NSL                                   = 0x07030009ull;

static const uint64_t P9A_MC_0_CLOCK_STAT_NSL                                 = 0x07030009ull;

static const uint64_t P9A_MC_1_CLOCK_STAT_NSL                                 = 0x08030009ull;


static const uint64_t P9A_MC_CLOCK_STAT_SL                                    = 0x07030008ull;

static const uint64_t P9A_MC_0_CLOCK_STAT_SL                                  = 0x07030008ull;

static const uint64_t P9A_MC_1_CLOCK_STAT_SL                                  = 0x08030008ull;


static const uint64_t P9A_MC_CMN_CONFIG                                       = 0x070133CEull;
//DUPS: 070133CE, 0701338E,
static const uint64_t P9A_MC_0_CMN_CONFIG                                     = 0x070133CEull;
//DUPS: 070133CE, 0701338E,
static const uint64_t P9A_MC_1_CMN_CONFIG                                     = 0x080133CEull;
//DUPS: 080133CE, 0801338E,

static const uint64_t P9A_MC_CONTROL_REG                                      = 0x07050012ull;

static const uint64_t P9A_MC_0_CONTROL_REG                                    = 0x07050012ull;

static const uint64_t P9A_MC_1_CONTROL_REG                                    = 0x08050012ull;


static const uint64_t P9A_MC_CPLT_CONF0                                       = 0x07000008ull;

static const uint64_t P9A_MC_CPLT_CONF0_OR                                    = 0x07000018ull;

static const uint64_t P9A_MC_CPLT_CONF0_CLEAR                                 = 0x07000028ull;

static const uint64_t P9A_MC_0_CPLT_CONF0                                     = 0x07000008ull;

static const uint64_t P9A_MC_0_CPLT_CONF0_OR                                  = 0x07000018ull;

static const uint64_t P9A_MC_0_CPLT_CONF0_CLEAR                               = 0x07000028ull;

static const uint64_t P9A_MC_1_CPLT_CONF0                                     = 0x08000008ull;

static const uint64_t P9A_MC_1_CPLT_CONF0_OR                                  = 0x08000018ull;

static const uint64_t P9A_MC_1_CPLT_CONF0_CLEAR                               = 0x08000028ull;


static const uint64_t P9A_MC_CPLT_CONF1                                       = 0x07000009ull;

static const uint64_t P9A_MC_CPLT_CONF1_OR                                    = 0x07000019ull;

static const uint64_t P9A_MC_CPLT_CONF1_CLEAR                                 = 0x07000029ull;

static const uint64_t P9A_MC_0_CPLT_CONF1                                     = 0x07000009ull;

static const uint64_t P9A_MC_0_CPLT_CONF1_OR                                  = 0x07000019ull;

static const uint64_t P9A_MC_0_CPLT_CONF1_CLEAR                               = 0x07000029ull;

static const uint64_t P9A_MC_1_CPLT_CONF1                                     = 0x08000009ull;

static const uint64_t P9A_MC_1_CPLT_CONF1_OR                                  = 0x08000019ull;

static const uint64_t P9A_MC_1_CPLT_CONF1_CLEAR                               = 0x08000029ull;


static const uint64_t P9A_MC_CPLT_CTRL0                                       = 0x07000000ull;

static const uint64_t P9A_MC_CPLT_CTRL0_OR                                    = 0x07000010ull;

static const uint64_t P9A_MC_CPLT_CTRL0_CLEAR                                 = 0x07000020ull;

static const uint64_t P9A_MC_0_CPLT_CTRL0                                     = 0x07000000ull;

static const uint64_t P9A_MC_0_CPLT_CTRL0_OR                                  = 0x07000010ull;

static const uint64_t P9A_MC_0_CPLT_CTRL0_CLEAR                               = 0x07000020ull;

static const uint64_t P9A_MC_1_CPLT_CTRL0                                     = 0x08000000ull;

static const uint64_t P9A_MC_1_CPLT_CTRL0_OR                                  = 0x08000010ull;

static const uint64_t P9A_MC_1_CPLT_CTRL0_CLEAR                               = 0x08000020ull;


static const uint64_t P9A_MC_CPLT_CTRL1                                       = 0x07000001ull;

static const uint64_t P9A_MC_CPLT_CTRL1_OR                                    = 0x07000011ull;

static const uint64_t P9A_MC_CPLT_CTRL1_CLEAR                                 = 0x07000021ull;

static const uint64_t P9A_MC_0_CPLT_CTRL1                                     = 0x07000001ull;

static const uint64_t P9A_MC_0_CPLT_CTRL1_OR                                  = 0x07000011ull;

static const uint64_t P9A_MC_0_CPLT_CTRL1_CLEAR                               = 0x07000021ull;

static const uint64_t P9A_MC_1_CPLT_CTRL1                                     = 0x08000001ull;

static const uint64_t P9A_MC_1_CPLT_CTRL1_OR                                  = 0x08000011ull;

static const uint64_t P9A_MC_1_CPLT_CTRL1_CLEAR                               = 0x08000021ull;


static const uint64_t P9A_MC_CPLT_MASK0                                       = 0x07000101ull;

static const uint64_t P9A_MC_0_CPLT_MASK0                                     = 0x07000101ull;

static const uint64_t P9A_MC_1_CPLT_MASK0                                     = 0x08000101ull;


static const uint64_t P9A_MC_CPLT_STAT0                                       = 0x07000100ull;

static const uint64_t P9A_MC_0_CPLT_STAT0                                     = 0x07000100ull;

static const uint64_t P9A_MC_1_CPLT_STAT0                                     = 0x08000100ull;


static const uint64_t P9A_MC_CSAR                                             = 0x07012C4Dull;
//DUPS: 0701284D, 07012C4D,
static const uint64_t P9A_MC_0_CSAR                                           = 0x07012C4Dull;
//DUPS: 0701284D, 07012C4D,
static const uint64_t P9A_MC_1_CSAR                                           = 0x08012C4Dull;
//DUPS: 0801284D, 08012C4D,

static const uint64_t P9A_OMIC_CSAR                                           = 0x0701244Dull;

static const uint64_t P9A_OMIC_0_CSAR                                         = 0x0701244Dull;

static const uint64_t P9A_OMIC_1_CSAR                                         = 0x0701284Dull;

static const uint64_t P9A_OMIC_3_CSAR                                         = 0x0801244Dull;

static const uint64_t P9A_OMIC_4_CSAR                                         = 0x0801284Dull;


static const uint64_t P9A_MC_CSCR                                             = 0x07012C4Aull;
//DUPS: 0701284A, 07012C4A,
static const uint64_t P9A_MC_CSCR_CLEAR                                       = 0x07012C4Bull;
//DUPS: 0701284B, 07012C4B,
static const uint64_t P9A_MC_CSCR_OR                                          = 0x07012C4Cull;
//DUPS: 0701284C, 07012C4C,
static const uint64_t P9A_MC_0_CSCR                                           = 0x07012C4Aull;
//DUPS: 0701284A, 07012C4A,
static const uint64_t P9A_MC_0_CSCR_CLEAR                                     = 0x07012C4Bull;
//DUPS: 0701284B, 07012C4B,
static const uint64_t P9A_MC_0_CSCR_OR                                        = 0x07012C4Cull;
//DUPS: 0701284C, 07012C4C,
static const uint64_t P9A_MC_1_CSCR                                           = 0x08012C4Aull;
//DUPS: 0801284A, 08012C4A,
static const uint64_t P9A_MC_1_CSCR_CLEAR                                     = 0x08012C4Bull;
//DUPS: 0801284B, 08012C4B,
static const uint64_t P9A_MC_1_CSCR_OR                                        = 0x08012C4Cull;
//DUPS: 0801284C, 08012C4C,

static const uint64_t P9A_OMIC_CSCR                                           = 0x0701244Aull;

static const uint64_t P9A_OMIC_CSCR_CLEAR                                     = 0x0701244Bull;

static const uint64_t P9A_OMIC_CSCR_OR                                        = 0x0701244Cull;

static const uint64_t P9A_OMIC_0_CSCR                                         = 0x0701244Aull;

static const uint64_t P9A_OMIC_0_CSCR_CLEAR                                   = 0x0701244Bull;

static const uint64_t P9A_OMIC_0_CSCR_OR                                      = 0x0701244Cull;

static const uint64_t P9A_OMIC_1_CSCR                                         = 0x0701284Aull;

static const uint64_t P9A_OMIC_1_CSCR_CLEAR                                   = 0x0701284Bull;

static const uint64_t P9A_OMIC_1_CSCR_OR                                      = 0x0701284Cull;

static const uint64_t P9A_OMIC_3_CSCR                                         = 0x0801244Aull;

static const uint64_t P9A_OMIC_3_CSCR_CLEAR                                   = 0x0801244Bull;

static const uint64_t P9A_OMIC_3_CSCR_OR                                      = 0x0801244Cull;

static const uint64_t P9A_OMIC_4_CSCR                                         = 0x0801284Aull;

static const uint64_t P9A_OMIC_4_CSCR_CLEAR                                   = 0x0801284Bull;

static const uint64_t P9A_OMIC_4_CSCR_OR                                      = 0x0801284Cull;


static const uint64_t P9A_MC_CSDR                                             = 0x07012C4Eull;
//DUPS: 0701284E, 07012C4E,
static const uint64_t P9A_MC_0_CSDR                                           = 0x07012C4Eull;
//DUPS: 0701284E, 07012C4E,
static const uint64_t P9A_MC_1_CSDR                                           = 0x08012C4Eull;
//DUPS: 0801284E, 08012C4E,

static const uint64_t P9A_OMIC_CSDR                                           = 0x0701244Eull;

static const uint64_t P9A_OMIC_0_CSDR                                         = 0x0701244Eull;

static const uint64_t P9A_OMIC_1_CSDR                                         = 0x0701284Eull;

static const uint64_t P9A_OMIC_3_CSDR                                         = 0x0801244Eull;

static const uint64_t P9A_OMIC_4_CSDR                                         = 0x0801284Eull;


static const uint64_t P9A_MC_CTRL_ATOMIC_LOCK_REG                             = 0x070003FFull;

static const uint64_t P9A_MC_0_CTRL_ATOMIC_LOCK_REG                           = 0x070003FFull;

static const uint64_t P9A_MC_1_CTRL_ATOMIC_LOCK_REG                           = 0x080003FFull;


static const uint64_t P9A_MC_CTRL_PROTECT_MODE_REG                            = 0x070003FEull;

static const uint64_t P9A_MC_0_CTRL_PROTECT_MODE_REG                          = 0x070003FEull;

static const uint64_t P9A_MC_1_CTRL_PROTECT_MODE_REG                          = 0x080003FEull;


static const uint64_t P9A_MC_DBG01_SCOM0Q                                     = 0x07012394ull;

static const uint64_t P9A_MC_0_DBG01_SCOM0Q                                   = 0x07012394ull;

static const uint64_t P9A_MC_1_DBG01_SCOM0Q                                   = 0x08012394ull;


static const uint64_t P9A_MC_DBG01_SCOM1Q                                     = 0x07012395ull;

static const uint64_t P9A_MC_0_DBG01_SCOM1Q                                   = 0x07012395ull;

static const uint64_t P9A_MC_1_DBG01_SCOM1Q                                   = 0x08012395ull;


static const uint64_t P9A_MC_DBG23_SCOM0Q                                     = 0x07012396ull;

static const uint64_t P9A_MC_0_DBG23_SCOM0Q                                   = 0x07012396ull;

static const uint64_t P9A_MC_1_DBG23_SCOM0Q                                   = 0x08012396ull;


static const uint64_t P9A_MC_DBG23_SCOM1Q                                     = 0x07012397ull;

static const uint64_t P9A_MC_0_DBG23_SCOM1Q                                   = 0x07012397ull;

static const uint64_t P9A_MC_1_DBG23_SCOM1Q                                   = 0x08012397ull;


static const uint64_t P9A_MC_DBGCFG0Q                                         = 0x070123E8ull;

static const uint64_t P9A_MC_0_DBGCFG0Q                                       = 0x070123E8ull;

static const uint64_t P9A_MC_1_DBGCFG0Q                                       = 0x080123E8ull;


static const uint64_t P9A_MC_DBGCFG1Q                                         = 0x070123E9ull;

static const uint64_t P9A_MC_0_DBGCFG1Q                                       = 0x070123E9ull;

static const uint64_t P9A_MC_1_DBGCFG1Q                                       = 0x080123E9ull;


static const uint64_t P9A_MC_DBGCFG2Q                                         = 0x070123EAull;

static const uint64_t P9A_MC_0_DBGCFG2Q                                       = 0x070123EAull;

static const uint64_t P9A_MC_1_DBGCFG2Q                                       = 0x080123EAull;


static const uint64_t P9A_MC_DBGCFG3Q                                         = 0x070123EBull;

static const uint64_t P9A_MC_0_DBGCFG3Q                                       = 0x070123EBull;

static const uint64_t P9A_MC_1_DBGCFG3Q                                       = 0x080123EBull;


static const uint64_t P9A_MC_DBG_CBS_CC                                       = 0x07030013ull;

static const uint64_t P9A_MC_0_DBG_CBS_CC                                     = 0x07030013ull;

static const uint64_t P9A_MC_1_DBG_CBS_CC                                     = 0x08030013ull;


static const uint64_t P9A_MC_DBG_INST1_COND_REG_1                             = 0x070107C1ull;

static const uint64_t P9A_MC_0_DBG_INST1_COND_REG_1                           = 0x070107C1ull;

static const uint64_t P9A_MC_1_DBG_INST1_COND_REG_1                           = 0x080107C1ull;


static const uint64_t P9A_MC_DBG_INST1_COND_REG_2                             = 0x070107C2ull;

static const uint64_t P9A_MC_0_DBG_INST1_COND_REG_2                           = 0x070107C2ull;

static const uint64_t P9A_MC_1_DBG_INST1_COND_REG_2                           = 0x080107C2ull;


static const uint64_t P9A_MC_DBG_INST1_COND_REG_3                             = 0x070107C3ull;

static const uint64_t P9A_MC_0_DBG_INST1_COND_REG_3                           = 0x070107C3ull;

static const uint64_t P9A_MC_1_DBG_INST1_COND_REG_3                           = 0x080107C3ull;


static const uint64_t P9A_MC_DBG_INST2_COND_REG_1                             = 0x070107C4ull;

static const uint64_t P9A_MC_0_DBG_INST2_COND_REG_1                           = 0x070107C4ull;

static const uint64_t P9A_MC_1_DBG_INST2_COND_REG_1                           = 0x080107C4ull;


static const uint64_t P9A_MC_DBG_INST2_COND_REG_2                             = 0x070107C5ull;

static const uint64_t P9A_MC_0_DBG_INST2_COND_REG_2                           = 0x070107C5ull;

static const uint64_t P9A_MC_1_DBG_INST2_COND_REG_2                           = 0x080107C5ull;


static const uint64_t P9A_MC_DBG_INST2_COND_REG_3                             = 0x070107C6ull;

static const uint64_t P9A_MC_0_DBG_INST2_COND_REG_3                           = 0x070107C6ull;

static const uint64_t P9A_MC_1_DBG_INST2_COND_REG_3                           = 0x080107C6ull;


static const uint64_t P9A_MC_DBG_MODE_REG                                     = 0x070107C0ull;

static const uint64_t P9A_MC_0_DBG_MODE_REG                                   = 0x070107C0ull;

static const uint64_t P9A_MC_1_DBG_MODE_REG                                   = 0x080107C0ull;


static const uint64_t P9A_MC_DBG_TRACE_MODE_REG_2                             = 0x070107CFull;

static const uint64_t P9A_MC_0_DBG_TRACE_MODE_REG_2                           = 0x070107CFull;

static const uint64_t P9A_MC_1_DBG_TRACE_MODE_REG_2                           = 0x080107CFull;


static const uint64_t P9A_MC_DBG_TRACE_REG_0                                  = 0x070107CDull;

static const uint64_t P9A_MC_0_DBG_TRACE_REG_0                                = 0x070107CDull;

static const uint64_t P9A_MC_1_DBG_TRACE_REG_0                                = 0x080107CDull;


static const uint64_t P9A_MC_DBG_TRACE_REG_1                                  = 0x070107CEull;

static const uint64_t P9A_MC_0_DBG_TRACE_REG_1                                = 0x070107CEull;

static const uint64_t P9A_MC_1_DBG_TRACE_REG_1                                = 0x080107CEull;


static const uint64_t P9A_MC_DEBUG_TRACE_CONTROL                              = 0x070107D0ull;

static const uint64_t P9A_MC_0_DEBUG_TRACE_CONTROL                            = 0x070107D0ull;

static const uint64_t P9A_MC_1_DEBUG_TRACE_CONTROL                            = 0x080107D0ull;


static const uint64_t P9A_MC_DSTLCFG                                          = 0x070109CBull;
//DUPS: 070109CB, 0701090B, 0701094B,
static const uint64_t P9A_MCC_DSTLCFG                                         = 0x0701090Bull;

static const uint64_t P9A_MCC_0_DSTLCFG                                       = 0x0701090Bull;

static const uint64_t P9A_MCC_1_DSTLCFG                                       = 0x0701094Bull;

static const uint64_t P9A_MCC_2_DSTLCFG                                       = 0x0701098Bull;

static const uint64_t P9A_MCC_3_DSTLCFG                                       = 0x070109CBull;

static const uint64_t P9A_MCC_4_DSTLCFG                                       = 0x0801090Bull;

static const uint64_t P9A_MCC_5_DSTLCFG                                       = 0x0801094Bull;

static const uint64_t P9A_MCC_6_DSTLCFG                                       = 0x0801098Bull;

static const uint64_t P9A_MCC_7_DSTLCFG                                       = 0x080109CBull;

static const uint64_t P9A_MC_0_DSTLCFG                                        = 0x070109CBull;
//DUPS: 070109CB, 0701090B, 0701094B,
static const uint64_t P9A_MC_1_DSTLCFG                                        = 0x080109CBull;
//DUPS: 080109CB, 0801090B, 0801094B,

static const uint64_t P9A_MC_DSTLCREDIT                                       = 0x070109CAull;
//DUPS: 070109CA, 0701090A, 0701094A,
static const uint64_t P9A_MCC_DSTLCREDIT                                      = 0x0701090Aull;

static const uint64_t P9A_MCC_0_DSTLCREDIT                                    = 0x0701090Aull;

static const uint64_t P9A_MCC_1_DSTLCREDIT                                    = 0x0701094Aull;

static const uint64_t P9A_MCC_2_DSTLCREDIT                                    = 0x0701098Aull;

static const uint64_t P9A_MCC_3_DSTLCREDIT                                    = 0x070109CAull;

static const uint64_t P9A_MCC_4_DSTLCREDIT                                    = 0x0801090Aull;

static const uint64_t P9A_MCC_5_DSTLCREDIT                                    = 0x0801094Aull;

static const uint64_t P9A_MCC_6_DSTLCREDIT                                    = 0x0801098Aull;

static const uint64_t P9A_MCC_7_DSTLCREDIT                                    = 0x080109CAull;

static const uint64_t P9A_MC_0_DSTLCREDIT                                     = 0x070109CAull;
//DUPS: 070109CA, 0701090A, 0701094A,
static const uint64_t P9A_MC_1_DSTLCREDIT                                     = 0x080109CAull;
//DUPS: 080109CA, 0801090A, 0801094A,

static const uint64_t P9A_MC_DSTLERRRPT                                       = 0x070109CCull;
//DUPS: 070109CC, 0701090C, 0701094C,
static const uint64_t P9A_MCC_DSTLERRRPT                                      = 0x0701090Cull;

static const uint64_t P9A_MCC_0_DSTLERRRPT                                    = 0x0701090Cull;

static const uint64_t P9A_MCC_1_DSTLERRRPT                                    = 0x0701094Cull;

static const uint64_t P9A_MCC_2_DSTLERRRPT                                    = 0x0701098Cull;

static const uint64_t P9A_MCC_3_DSTLERRRPT                                    = 0x070109CCull;

static const uint64_t P9A_MCC_4_DSTLERRRPT                                    = 0x0801090Cull;

static const uint64_t P9A_MCC_5_DSTLERRRPT                                    = 0x0801094Cull;

static const uint64_t P9A_MCC_6_DSTLERRRPT                                    = 0x0801098Cull;

static const uint64_t P9A_MCC_7_DSTLERRRPT                                    = 0x080109CCull;

static const uint64_t P9A_MC_0_DSTLERRRPT                                     = 0x070109CCull;
//DUPS: 070109CC, 0701090C, 0701094C,
static const uint64_t P9A_MC_1_DSTLERRRPT                                     = 0x080109CCull;
//DUPS: 080109CC, 0801090C, 0801094C,

static const uint64_t P9A_MC_DSTLFIR                                          = 0x070109C0ull;
//DUPS: 070109C0, 07010900, 07010940,
static const uint64_t P9A_MC_DSTLFIR_AND                                      = 0x070109C1ull;
//DUPS: 070109C1, 07010901, 07010941,
static const uint64_t P9A_MC_DSTLFIR_OR                                       = 0x070109C2ull;
//DUPS: 070109C2, 07010902, 07010942,
static const uint64_t P9A_MCC_DSTLFIR                                         = 0x07010900ull;

static const uint64_t P9A_MCC_DSTLFIR_AND                                     = 0x07010901ull;

static const uint64_t P9A_MCC_DSTLFIR_OR                                      = 0x07010902ull;

static const uint64_t P9A_MCC_0_DSTLFIR                                       = 0x07010900ull;

static const uint64_t P9A_MCC_0_DSTLFIR_AND                                   = 0x07010901ull;

static const uint64_t P9A_MCC_0_DSTLFIR_OR                                    = 0x07010902ull;

static const uint64_t P9A_MCC_1_DSTLFIR                                       = 0x07010940ull;

static const uint64_t P9A_MCC_1_DSTLFIR_AND                                   = 0x07010941ull;

static const uint64_t P9A_MCC_1_DSTLFIR_OR                                    = 0x07010942ull;

static const uint64_t P9A_MCC_2_DSTLFIR                                       = 0x07010980ull;

static const uint64_t P9A_MCC_2_DSTLFIR_AND                                   = 0x07010981ull;

static const uint64_t P9A_MCC_2_DSTLFIR_OR                                    = 0x07010982ull;

static const uint64_t P9A_MCC_3_DSTLFIR                                       = 0x070109C0ull;

static const uint64_t P9A_MCC_3_DSTLFIR_AND                                   = 0x070109C1ull;

static const uint64_t P9A_MCC_3_DSTLFIR_OR                                    = 0x070109C2ull;

static const uint64_t P9A_MCC_4_DSTLFIR                                       = 0x08010900ull;

static const uint64_t P9A_MCC_4_DSTLFIR_AND                                   = 0x08010901ull;

static const uint64_t P9A_MCC_4_DSTLFIR_OR                                    = 0x08010902ull;

static const uint64_t P9A_MCC_5_DSTLFIR                                       = 0x08010940ull;

static const uint64_t P9A_MCC_5_DSTLFIR_AND                                   = 0x08010941ull;

static const uint64_t P9A_MCC_5_DSTLFIR_OR                                    = 0x08010942ull;

static const uint64_t P9A_MCC_6_DSTLFIR                                       = 0x08010980ull;

static const uint64_t P9A_MCC_6_DSTLFIR_AND                                   = 0x08010981ull;

static const uint64_t P9A_MCC_6_DSTLFIR_OR                                    = 0x08010982ull;

static const uint64_t P9A_MCC_7_DSTLFIR                                       = 0x080109C0ull;

static const uint64_t P9A_MCC_7_DSTLFIR_AND                                   = 0x080109C1ull;

static const uint64_t P9A_MCC_7_DSTLFIR_OR                                    = 0x080109C2ull;

static const uint64_t P9A_MC_0_DSTLFIR                                        = 0x070109C0ull;
//DUPS: 070109C0, 07010900, 07010940,
static const uint64_t P9A_MC_0_DSTLFIR_AND                                    = 0x070109C1ull;
//DUPS: 070109C1, 07010901, 07010941,
static const uint64_t P9A_MC_0_DSTLFIR_OR                                     = 0x070109C2ull;
//DUPS: 070109C2, 07010902, 07010942,
static const uint64_t P9A_MC_1_DSTLFIR                                        = 0x080109C0ull;
//DUPS: 080109C0, 08010900, 08010940,
static const uint64_t P9A_MC_1_DSTLFIR_AND                                    = 0x080109C1ull;
//DUPS: 080109C1, 08010901, 08010941,
static const uint64_t P9A_MC_1_DSTLFIR_OR                                     = 0x080109C2ull;
//DUPS: 080109C2, 08010902, 08010942,

static const uint64_t P9A_MC_DSTLFIRACT0                                      = 0x070109C6ull;
//DUPS: 070109C6, 07010906, 07010946,
static const uint64_t P9A_MCC_DSTLFIRACT0                                     = 0x07010906ull;

static const uint64_t P9A_MCC_0_DSTLFIRACT0                                   = 0x07010906ull;

static const uint64_t P9A_MCC_1_DSTLFIRACT0                                   = 0x07010946ull;

static const uint64_t P9A_MCC_2_DSTLFIRACT0                                   = 0x07010986ull;

static const uint64_t P9A_MCC_3_DSTLFIRACT0                                   = 0x070109C6ull;

static const uint64_t P9A_MCC_4_DSTLFIRACT0                                   = 0x08010906ull;

static const uint64_t P9A_MCC_5_DSTLFIRACT0                                   = 0x08010946ull;

static const uint64_t P9A_MCC_6_DSTLFIRACT0                                   = 0x08010986ull;

static const uint64_t P9A_MCC_7_DSTLFIRACT0                                   = 0x080109C6ull;

static const uint64_t P9A_MC_0_DSTLFIRACT0                                    = 0x070109C6ull;
//DUPS: 070109C6, 07010906, 07010946,
static const uint64_t P9A_MC_1_DSTLFIRACT0                                    = 0x080109C6ull;
//DUPS: 080109C6, 08010906, 08010946,

static const uint64_t P9A_MC_DSTLFIRACT1                                      = 0x070109C7ull;
//DUPS: 070109C7, 07010907, 07010947,
static const uint64_t P9A_MCC_DSTLFIRACT1                                     = 0x07010907ull;

static const uint64_t P9A_MCC_0_DSTLFIRACT1                                   = 0x07010907ull;

static const uint64_t P9A_MCC_1_DSTLFIRACT1                                   = 0x07010947ull;

static const uint64_t P9A_MCC_2_DSTLFIRACT1                                   = 0x07010987ull;

static const uint64_t P9A_MCC_3_DSTLFIRACT1                                   = 0x070109C7ull;

static const uint64_t P9A_MCC_4_DSTLFIRACT1                                   = 0x08010907ull;

static const uint64_t P9A_MCC_5_DSTLFIRACT1                                   = 0x08010947ull;

static const uint64_t P9A_MCC_6_DSTLFIRACT1                                   = 0x08010987ull;

static const uint64_t P9A_MCC_7_DSTLFIRACT1                                   = 0x080109C7ull;

static const uint64_t P9A_MC_0_DSTLFIRACT1                                    = 0x070109C7ull;
//DUPS: 070109C7, 07010907, 07010947,
static const uint64_t P9A_MC_1_DSTLFIRACT1                                    = 0x080109C7ull;
//DUPS: 080109C7, 08010907, 08010947,

static const uint64_t P9A_MC_DSTLFIRMASK                                      = 0x070109C3ull;
//DUPS: 070109C3, 07010903, 07010943,
static const uint64_t P9A_MC_DSTLFIRMASK_AND                                  = 0x070109C4ull;
//DUPS: 070109C4, 07010904, 07010944,
static const uint64_t P9A_MC_DSTLFIRMASK_OR                                   = 0x070109C5ull;
//DUPS: 070109C5, 07010905, 07010945,
static const uint64_t P9A_MCC_DSTLFIRMASK                                     = 0x07010903ull;

static const uint64_t P9A_MCC_DSTLFIRMASK_AND                                 = 0x07010904ull;

static const uint64_t P9A_MCC_DSTLFIRMASK_OR                                  = 0x07010905ull;

static const uint64_t P9A_MCC_0_DSTLFIRMASK                                   = 0x07010903ull;

static const uint64_t P9A_MCC_0_DSTLFIRMASK_AND                               = 0x07010904ull;

static const uint64_t P9A_MCC_0_DSTLFIRMASK_OR                                = 0x07010905ull;

static const uint64_t P9A_MCC_1_DSTLFIRMASK                                   = 0x07010943ull;

static const uint64_t P9A_MCC_1_DSTLFIRMASK_AND                               = 0x07010944ull;

static const uint64_t P9A_MCC_1_DSTLFIRMASK_OR                                = 0x07010945ull;

static const uint64_t P9A_MCC_2_DSTLFIRMASK                                   = 0x07010983ull;

static const uint64_t P9A_MCC_2_DSTLFIRMASK_AND                               = 0x07010984ull;

static const uint64_t P9A_MCC_2_DSTLFIRMASK_OR                                = 0x07010985ull;

static const uint64_t P9A_MCC_3_DSTLFIRMASK                                   = 0x070109C3ull;

static const uint64_t P9A_MCC_3_DSTLFIRMASK_AND                               = 0x070109C4ull;

static const uint64_t P9A_MCC_3_DSTLFIRMASK_OR                                = 0x070109C5ull;

static const uint64_t P9A_MCC_4_DSTLFIRMASK                                   = 0x08010903ull;

static const uint64_t P9A_MCC_4_DSTLFIRMASK_AND                               = 0x08010904ull;

static const uint64_t P9A_MCC_4_DSTLFIRMASK_OR                                = 0x08010905ull;

static const uint64_t P9A_MCC_5_DSTLFIRMASK                                   = 0x08010943ull;

static const uint64_t P9A_MCC_5_DSTLFIRMASK_AND                               = 0x08010944ull;

static const uint64_t P9A_MCC_5_DSTLFIRMASK_OR                                = 0x08010945ull;

static const uint64_t P9A_MCC_6_DSTLFIRMASK                                   = 0x08010983ull;

static const uint64_t P9A_MCC_6_DSTLFIRMASK_AND                               = 0x08010984ull;

static const uint64_t P9A_MCC_6_DSTLFIRMASK_OR                                = 0x08010985ull;

static const uint64_t P9A_MCC_7_DSTLFIRMASK                                   = 0x080109C3ull;

static const uint64_t P9A_MCC_7_DSTLFIRMASK_AND                               = 0x080109C4ull;

static const uint64_t P9A_MCC_7_DSTLFIRMASK_OR                                = 0x080109C5ull;

static const uint64_t P9A_MC_0_DSTLFIRMASK                                    = 0x070109C3ull;
//DUPS: 070109C3, 07010903, 07010943,
static const uint64_t P9A_MC_0_DSTLFIRMASK_AND                                = 0x070109C4ull;
//DUPS: 070109C4, 07010904, 07010944,
static const uint64_t P9A_MC_0_DSTLFIRMASK_OR                                 = 0x070109C5ull;
//DUPS: 070109C5, 07010905, 07010945,
static const uint64_t P9A_MC_1_DSTLFIRMASK                                    = 0x080109C3ull;
//DUPS: 080109C3, 08010903, 08010943,
static const uint64_t P9A_MC_1_DSTLFIRMASK_AND                                = 0x080109C4ull;
//DUPS: 080109C4, 08010904, 08010944,
static const uint64_t P9A_MC_1_DSTLFIRMASK_OR                                 = 0x080109C5ull;
//DUPS: 080109C5, 08010905, 08010945,

static const uint64_t P9A_MC_DSTLFIRWOF                                       = 0x070109C8ull;
//DUPS: 070109C8, 07010908, 07010948,
static const uint64_t P9A_MCC_DSTLFIRWOF                                      = 0x07010908ull;

static const uint64_t P9A_MCC_0_DSTLFIRWOF                                    = 0x07010908ull;

static const uint64_t P9A_MCC_1_DSTLFIRWOF                                    = 0x07010948ull;

static const uint64_t P9A_MCC_2_DSTLFIRWOF                                    = 0x07010988ull;

static const uint64_t P9A_MCC_3_DSTLFIRWOF                                    = 0x070109C8ull;

static const uint64_t P9A_MCC_4_DSTLFIRWOF                                    = 0x08010908ull;

static const uint64_t P9A_MCC_5_DSTLFIRWOF                                    = 0x08010948ull;

static const uint64_t P9A_MCC_6_DSTLFIRWOF                                    = 0x08010988ull;

static const uint64_t P9A_MCC_7_DSTLFIRWOF                                    = 0x080109C8ull;

static const uint64_t P9A_MC_0_DSTLFIRWOF                                     = 0x070109C8ull;
//DUPS: 070109C8, 07010908, 07010948,
static const uint64_t P9A_MC_1_DSTLFIRWOF                                     = 0x080109C8ull;
//DUPS: 080109C8, 08010908, 08010948,

static const uint64_t P9A_MC_DSTLINJ                                          = 0x070109CDull;
//DUPS: 070109CD, 0701090D, 0701094D,
static const uint64_t P9A_MCC_DSTLINJ                                         = 0x0701090Dull;

static const uint64_t P9A_MCC_0_DSTLINJ                                       = 0x0701090Dull;

static const uint64_t P9A_MCC_1_DSTLINJ                                       = 0x0701094Dull;

static const uint64_t P9A_MCC_2_DSTLINJ                                       = 0x0701098Dull;

static const uint64_t P9A_MCC_3_DSTLINJ                                       = 0x070109CDull;

static const uint64_t P9A_MCC_4_DSTLINJ                                       = 0x0801090Dull;

static const uint64_t P9A_MCC_5_DSTLINJ                                       = 0x0801094Dull;

static const uint64_t P9A_MCC_6_DSTLINJ                                       = 0x0801098Dull;

static const uint64_t P9A_MCC_7_DSTLINJ                                       = 0x080109CDull;

static const uint64_t P9A_MC_0_DSTLINJ                                        = 0x070109CDull;
//DUPS: 070109CD, 0701090D, 0701094D,
static const uint64_t P9A_MC_1_DSTLINJ                                        = 0x080109CDull;
//DUPS: 080109CD, 0801090D, 0801094D,

static const uint64_t P9A_MC_DTS_RESULT0                                      = 0x07050000ull;

static const uint64_t P9A_MC_0_DTS_RESULT0                                    = 0x07050000ull;

static const uint64_t P9A_MC_1_DTS_RESULT0                                    = 0x08050000ull;


static const uint64_t P9A_MC_DTS_TRC_RESULT                                   = 0x07050003ull;

static const uint64_t P9A_MC_0_DTS_TRC_RESULT                                 = 0x07050003ull;

static const uint64_t P9A_MC_1_DTS_TRC_RESULT                                 = 0x08050003ull;


static const uint64_t P9A_MC_EDRAM_STATUS                                     = 0x070F0029ull;

static const uint64_t P9A_MC_0_EDRAM_STATUS                                   = 0x070F0029ull;

static const uint64_t P9A_MC_1_EDRAM_STATUS                                   = 0x080F0029ull;


static const uint64_t P9A_MC_ERROR_REG                                        = 0x070F001Full;

static const uint64_t P9A_MC_0_ERROR_REG                                      = 0x070F001Full;

static const uint64_t P9A_MC_1_ERROR_REG                                      = 0x080F001Full;


static const uint64_t P9A_MC_ERROR_STATUS                                     = 0x0703000Full;

static const uint64_t P9A_MC_0_ERROR_STATUS                                   = 0x0703000Full;

static const uint64_t P9A_MC_1_ERROR_STATUS                                   = 0x0803000Full;


static const uint64_t P9A_MC_ERR_STATUS_REG                                   = 0x07050013ull;

static const uint64_t P9A_MC_0_ERR_STATUS_REG                                 = 0x07050013ull;

static const uint64_t P9A_MC_1_ERR_STATUS_REG                                 = 0x08050013ull;


static const uint64_t P9A_MC_FIR_ACTION0_REG                                  = 0x07011006ull;
//DUPS: 07011406, 07011806,
static const uint64_t P9A_MC_0_FIR_ACTION0_REG                                = 0x07011006ull;
//DUPS: 07011406, 07011806,
static const uint64_t P9A_MC_1_FIR_ACTION0_REG                                = 0x08011006ull;
//DUPS: 08011406, 08011806,

static const uint64_t P9A_OMIC_FIR_ACTION0_REG                                = 0x07011006ull;

static const uint64_t P9A_OMIC_0_FIR_ACTION0_REG                              = 0x07011006ull;

static const uint64_t P9A_OMIC_1_FIR_ACTION0_REG                              = 0x07011406ull;

static const uint64_t P9A_OMIC_2_FIR_ACTION0_REG                              = 0x07011806ull;

static const uint64_t P9A_OMIC_3_FIR_ACTION0_REG                              = 0x08011006ull;

static const uint64_t P9A_OMIC_4_FIR_ACTION0_REG                              = 0x08011406ull;

static const uint64_t P9A_OMIC_5_FIR_ACTION0_REG                              = 0x08011806ull;


static const uint64_t P9A_MC_FIR_ACTION1_REG                                  = 0x07011007ull;
//DUPS: 07011407, 07011807,
static const uint64_t P9A_MC_0_FIR_ACTION1_REG                                = 0x07011007ull;
//DUPS: 07011407, 07011807,
static const uint64_t P9A_MC_1_FIR_ACTION1_REG                                = 0x08011007ull;
//DUPS: 08011407, 08011807,

static const uint64_t P9A_OMIC_FIR_ACTION1_REG                                = 0x07011007ull;

static const uint64_t P9A_OMIC_0_FIR_ACTION1_REG                              = 0x07011007ull;

static const uint64_t P9A_OMIC_1_FIR_ACTION1_REG                              = 0x07011407ull;

static const uint64_t P9A_OMIC_2_FIR_ACTION1_REG                              = 0x07011807ull;

static const uint64_t P9A_OMIC_3_FIR_ACTION1_REG                              = 0x08011007ull;

static const uint64_t P9A_OMIC_4_FIR_ACTION1_REG                              = 0x08011407ull;

static const uint64_t P9A_OMIC_5_FIR_ACTION1_REG                              = 0x08011807ull;


static const uint64_t P9A_MC_FIR_MASK                                         = 0x07040002ull;

static const uint64_t P9A_MC_0_FIR_MASK                                       = 0x07040002ull;

static const uint64_t P9A_MC_1_FIR_MASK                                       = 0x08040002ull;


static const uint64_t P9A_MC_FIR_MASK_REG                                     = 0x07011003ull;
//DUPS: 07011403, 07011803,
static const uint64_t P9A_MC_FIR_MASK_REG_AND                                 = 0x07011004ull;
//DUPS: 07011404, 07011804,
static const uint64_t P9A_MC_FIR_MASK_REG_OR                                  = 0x07011005ull;
//DUPS: 07011405, 07011805,
static const uint64_t P9A_MC_0_FIR_MASK_REG                                   = 0x07011003ull;
//DUPS: 07011403, 07011803,
static const uint64_t P9A_MC_0_FIR_MASK_REG_AND                               = 0x07011004ull;
//DUPS: 07011404, 07011804,
static const uint64_t P9A_MC_0_FIR_MASK_REG_OR                                = 0x07011005ull;
//DUPS: 07011405, 07011805,
static const uint64_t P9A_MC_1_FIR_MASK_REG                                   = 0x08011003ull;
//DUPS: 08011403, 08011803,
static const uint64_t P9A_MC_1_FIR_MASK_REG_AND                               = 0x08011004ull;
//DUPS: 08011404, 08011804,
static const uint64_t P9A_MC_1_FIR_MASK_REG_OR                                = 0x08011005ull;
//DUPS: 08011405, 08011805,

static const uint64_t P9A_OMIC_FIR_MASK_REG                                   = 0x07011003ull;

static const uint64_t P9A_OMIC_FIR_MASK_REG_AND                               = 0x07011004ull;

static const uint64_t P9A_OMIC_FIR_MASK_REG_OR                                = 0x07011005ull;

static const uint64_t P9A_OMIC_0_FIR_MASK_REG                                 = 0x07011003ull;

static const uint64_t P9A_OMIC_0_FIR_MASK_REG_AND                             = 0x07011004ull;

static const uint64_t P9A_OMIC_0_FIR_MASK_REG_OR                              = 0x07011005ull;

static const uint64_t P9A_OMIC_1_FIR_MASK_REG                                 = 0x07011403ull;

static const uint64_t P9A_OMIC_1_FIR_MASK_REG_AND                             = 0x07011404ull;

static const uint64_t P9A_OMIC_1_FIR_MASK_REG_OR                              = 0x07011405ull;

static const uint64_t P9A_OMIC_2_FIR_MASK_REG                                 = 0x07011803ull;

static const uint64_t P9A_OMIC_2_FIR_MASK_REG_AND                             = 0x07011804ull;

static const uint64_t P9A_OMIC_2_FIR_MASK_REG_OR                              = 0x07011805ull;

static const uint64_t P9A_OMIC_3_FIR_MASK_REG                                 = 0x08011003ull;

static const uint64_t P9A_OMIC_3_FIR_MASK_REG_AND                             = 0x08011004ull;

static const uint64_t P9A_OMIC_3_FIR_MASK_REG_OR                              = 0x08011005ull;

static const uint64_t P9A_OMIC_4_FIR_MASK_REG                                 = 0x08011403ull;

static const uint64_t P9A_OMIC_4_FIR_MASK_REG_AND                             = 0x08011404ull;

static const uint64_t P9A_OMIC_4_FIR_MASK_REG_OR                              = 0x08011405ull;

static const uint64_t P9A_OMIC_5_FIR_MASK_REG                                 = 0x08011803ull;

static const uint64_t P9A_OMIC_5_FIR_MASK_REG_AND                             = 0x08011804ull;

static const uint64_t P9A_OMIC_5_FIR_MASK_REG_OR                              = 0x08011805ull;


static const uint64_t P9A_MC_FIR_REG                                          = 0x07011000ull;
//DUPS: 07011400, 07011800,
static const uint64_t P9A_MC_FIR_REG_AND                                      = 0x07011001ull;
//DUPS: 07011401, 07011801,
static const uint64_t P9A_MC_FIR_REG_OR                                       = 0x07011002ull;
//DUPS: 07011402, 07011802,
static const uint64_t P9A_MC_0_FIR_REG                                        = 0x07011000ull;
//DUPS: 07011400, 07011800,
static const uint64_t P9A_MC_0_FIR_REG_AND                                    = 0x07011001ull;
//DUPS: 07011401, 07011801,
static const uint64_t P9A_MC_0_FIR_REG_OR                                     = 0x07011002ull;
//DUPS: 07011402, 07011802,
static const uint64_t P9A_MC_1_FIR_REG                                        = 0x08011000ull;
//DUPS: 08011400, 08011800,
static const uint64_t P9A_MC_1_FIR_REG_AND                                    = 0x08011001ull;
//DUPS: 08011401, 08011801,
static const uint64_t P9A_MC_1_FIR_REG_OR                                     = 0x08011002ull;
//DUPS: 08011402, 08011802,

static const uint64_t P9A_OMIC_FIR_REG                                        = 0x07011000ull;

static const uint64_t P9A_OMIC_FIR_REG_AND                                    = 0x07011001ull;

static const uint64_t P9A_OMIC_FIR_REG_OR                                     = 0x07011002ull;

static const uint64_t P9A_OMIC_0_FIR_REG                                      = 0x07011000ull;

static const uint64_t P9A_OMIC_0_FIR_REG_AND                                  = 0x07011001ull;

static const uint64_t P9A_OMIC_0_FIR_REG_OR                                   = 0x07011002ull;

static const uint64_t P9A_OMIC_1_FIR_REG                                      = 0x07011400ull;

static const uint64_t P9A_OMIC_1_FIR_REG_AND                                  = 0x07011401ull;

static const uint64_t P9A_OMIC_1_FIR_REG_OR                                   = 0x07011402ull;

static const uint64_t P9A_OMIC_2_FIR_REG                                      = 0x07011800ull;

static const uint64_t P9A_OMIC_2_FIR_REG_AND                                  = 0x07011801ull;

static const uint64_t P9A_OMIC_2_FIR_REG_OR                                   = 0x07011802ull;

static const uint64_t P9A_OMIC_3_FIR_REG                                      = 0x08011000ull;

static const uint64_t P9A_OMIC_3_FIR_REG_AND                                  = 0x08011001ull;

static const uint64_t P9A_OMIC_3_FIR_REG_OR                                   = 0x08011002ull;

static const uint64_t P9A_OMIC_4_FIR_REG                                      = 0x08011400ull;

static const uint64_t P9A_OMIC_4_FIR_REG_AND                                  = 0x08011401ull;

static const uint64_t P9A_OMIC_4_FIR_REG_OR                                   = 0x08011402ull;

static const uint64_t P9A_OMIC_5_FIR_REG                                      = 0x08011800ull;

static const uint64_t P9A_OMIC_5_FIR_REG_AND                                  = 0x08011801ull;

static const uint64_t P9A_OMIC_5_FIR_REG_OR                                   = 0x08011802ull;


static const uint64_t P9A_MC_FIR_WOF_REG                                      = 0x07011008ull;
//DUPS: 07011408, 07011808,
static const uint64_t P9A_MC_0_FIR_WOF_REG                                    = 0x07011008ull;
//DUPS: 07011408, 07011808,
static const uint64_t P9A_MC_1_FIR_WOF_REG                                    = 0x08011008ull;
//DUPS: 08011408, 08011808,

static const uint64_t P9A_OMIC_FIR_WOF_REG                                    = 0x07011008ull;

static const uint64_t P9A_OMIC_0_FIR_WOF_REG                                  = 0x07011008ull;

static const uint64_t P9A_OMIC_1_FIR_WOF_REG                                  = 0x07011408ull;

static const uint64_t P9A_OMIC_2_FIR_WOF_REG                                  = 0x07011808ull;

static const uint64_t P9A_OMIC_3_FIR_WOF_REG                                  = 0x08011008ull;

static const uint64_t P9A_OMIC_4_FIR_WOF_REG                                  = 0x08011408ull;

static const uint64_t P9A_OMIC_5_FIR_WOF_REG                                  = 0x08011808ull;


static const uint64_t P9A_MC_GXSTOP0_MASK_REG                                 = 0x07040014ull;

static const uint64_t P9A_MC_0_GXSTOP0_MASK_REG                               = 0x07040014ull;

static const uint64_t P9A_MC_1_GXSTOP0_MASK_REG                               = 0x08040014ull;


static const uint64_t P9A_MC_GXSTOP1_MASK_REG                                 = 0x07040015ull;

static const uint64_t P9A_MC_0_GXSTOP1_MASK_REG                               = 0x07040015ull;

static const uint64_t P9A_MC_1_GXSTOP1_MASK_REG                               = 0x08040015ull;


static const uint64_t P9A_MC_GXSTOP2_MASK_REG                                 = 0x07040016ull;

static const uint64_t P9A_MC_0_GXSTOP2_MASK_REG                               = 0x07040016ull;

static const uint64_t P9A_MC_1_GXSTOP2_MASK_REG                               = 0x08040016ull;


static const uint64_t P9A_MC_GXSTOP_TRIG_REG                                  = 0x07040013ull;

static const uint64_t P9A_MC_0_GXSTOP_TRIG_REG                                = 0x07040013ull;

static const uint64_t P9A_MC_1_GXSTOP_TRIG_REG                                = 0x08040013ull;


static const uint64_t P9A_MC_HANG_PULSE_0_REG                                 = 0x070F0020ull;

static const uint64_t P9A_MC_0_HANG_PULSE_0_REG                               = 0x070F0020ull;

static const uint64_t P9A_MC_1_HANG_PULSE_0_REG                               = 0x080F0020ull;


static const uint64_t P9A_MC_HANG_PULSE_1_REG                                 = 0x070F0021ull;

static const uint64_t P9A_MC_0_HANG_PULSE_1_REG                               = 0x070F0021ull;

static const uint64_t P9A_MC_1_HANG_PULSE_1_REG                               = 0x080F0021ull;


static const uint64_t P9A_MC_HANG_PULSE_2_REG                                 = 0x070F0022ull;

static const uint64_t P9A_MC_0_HANG_PULSE_2_REG                               = 0x070F0022ull;

static const uint64_t P9A_MC_1_HANG_PULSE_2_REG                               = 0x080F0022ull;


static const uint64_t P9A_MC_HANG_PULSE_3_REG                                 = 0x070F0023ull;

static const uint64_t P9A_MC_0_HANG_PULSE_3_REG                               = 0x070F0023ull;

static const uint64_t P9A_MC_1_HANG_PULSE_3_REG                               = 0x080F0023ull;


static const uint64_t P9A_MC_HANG_PULSE_4_REG                                 = 0x070F0024ull;

static const uint64_t P9A_MC_0_HANG_PULSE_4_REG                               = 0x070F0024ull;

static const uint64_t P9A_MC_1_HANG_PULSE_4_REG                               = 0x080F0024ull;


static const uint64_t P9A_MC_HANG_PULSE_5_REG                                 = 0x070F0025ull;

static const uint64_t P9A_MC_0_HANG_PULSE_5_REG                               = 0x070F0025ull;

static const uint64_t P9A_MC_1_HANG_PULSE_5_REG                               = 0x080F0025ull;


static const uint64_t P9A_MC_HANG_PULSE_6_REG                                 = 0x070F0026ull;

static const uint64_t P9A_MC_0_HANG_PULSE_6_REG                               = 0x070F0026ull;

static const uint64_t P9A_MC_1_HANG_PULSE_6_REG                               = 0x080F0026ull;


static const uint64_t P9A_MC_HEARTBEAT_REG                                    = 0x070F0018ull;

static const uint64_t P9A_MC_0_HEARTBEAT_REG                                  = 0x070F0018ull;

static const uint64_t P9A_MC_1_HEARTBEAT_REG                                  = 0x080F0018ull;


static const uint64_t P9A_MC_HOSTATTN                                         = 0x07040009ull;

static const uint64_t P9A_MC_0_HOSTATTN                                       = 0x07040009ull;

static const uint64_t P9A_MC_1_HOSTATTN                                       = 0x08040009ull;


static const uint64_t P9A_MC_HOSTATTN_MASK                                    = 0x0704001Aull;

static const uint64_t P9A_MC_0_HOSTATTN_MASK                                  = 0x0704001Aull;

static const uint64_t P9A_MC_1_HOSTATTN_MASK                                  = 0x0804001Aull;


static const uint64_t P9A_MC_INJECT_REG                                       = 0x07050011ull;

static const uint64_t P9A_MC_0_INJECT_REG                                     = 0x07050011ull;

static const uint64_t P9A_MC_1_INJECT_REG                                     = 0x08050011ull;


static const uint64_t P9A_MC_LOCAL_FIR                                        = 0x0704000Aull;

static const uint64_t P9A_MC_LOCAL_FIR_AND                                    = 0x0704000Bull;

static const uint64_t P9A_MC_LOCAL_FIR_OR                                     = 0x0704000Cull;

static const uint64_t P9A_MC_0_LOCAL_FIR                                      = 0x0704000Aull;

static const uint64_t P9A_MC_0_LOCAL_FIR_AND                                  = 0x0704000Bull;

static const uint64_t P9A_MC_0_LOCAL_FIR_OR                                   = 0x0704000Cull;

static const uint64_t P9A_MC_1_LOCAL_FIR                                      = 0x0804000Aull;

static const uint64_t P9A_MC_1_LOCAL_FIR_AND                                  = 0x0804000Bull;

static const uint64_t P9A_MC_1_LOCAL_FIR_OR                                   = 0x0804000Cull;


static const uint64_t P9A_MC_LOCAL_FIR_ACTION0                                = 0x07040010ull;

static const uint64_t P9A_MC_0_LOCAL_FIR_ACTION0                              = 0x07040010ull;

static const uint64_t P9A_MC_1_LOCAL_FIR_ACTION0                              = 0x08040010ull;


static const uint64_t P9A_MC_LOCAL_FIR_ACTION1                                = 0x07040011ull;

static const uint64_t P9A_MC_0_LOCAL_FIR_ACTION1                              = 0x07040011ull;

static const uint64_t P9A_MC_1_LOCAL_FIR_ACTION1                              = 0x08040011ull;


static const uint64_t P9A_MC_LOCAL_FIR_MASK                                   = 0x0704000Dull;

static const uint64_t P9A_MC_LOCAL_FIR_MASK_AND                               = 0x0704000Eull;

static const uint64_t P9A_MC_LOCAL_FIR_MASK_OR                                = 0x0704000Full;

static const uint64_t P9A_MC_0_LOCAL_FIR_MASK                                 = 0x0704000Dull;

static const uint64_t P9A_MC_0_LOCAL_FIR_MASK_AND                             = 0x0704000Eull;

static const uint64_t P9A_MC_0_LOCAL_FIR_MASK_OR                              = 0x0704000Full;

static const uint64_t P9A_MC_1_LOCAL_FIR_MASK                                 = 0x0804000Dull;

static const uint64_t P9A_MC_1_LOCAL_FIR_MASK_AND                             = 0x0804000Eull;

static const uint64_t P9A_MC_1_LOCAL_FIR_MASK_OR                              = 0x0804000Full;


static const uint64_t P9A_MC_LOCAL_XSTOP_ERR                                  = 0x07040018ull;

static const uint64_t P9A_MC_0_LOCAL_XSTOP_ERR                                = 0x07040018ull;

static const uint64_t P9A_MC_1_LOCAL_XSTOP_ERR                                = 0x08040018ull;


static const uint64_t P9A_MC_LOCAL_XSTOP_MASK                                 = 0x07040019ull;

static const uint64_t P9A_MC_0_LOCAL_XSTOP_MASK                               = 0x07040019ull;

static const uint64_t P9A_MC_1_LOCAL_XSTOP_MASK                               = 0x08040019ull;


static const uint64_t P9A_MC_MBA_MCBERRPTQ                                    = 0x070123E7ull;

static const uint64_t P9A_MC_0_MBA_MCBERRPTQ                                  = 0x070123E7ull;

static const uint64_t P9A_MC_1_MBA_MCBERRPTQ                                  = 0x080123E7ull;


static const uint64_t P9A_MCC_MCAMOC                                          = 0x05010825ull;

static const uint64_t P9A_MCC_0_MCAMOC                                        = 0x05010825ull;

static const uint64_t P9A_MCC_1_MCAMOC                                        = 0x05010835ull;

static const uint64_t P9A_MCC_2_MCAMOC                                        = 0x050108A5ull;

static const uint64_t P9A_MCC_3_MCAMOC                                        = 0x050108B5ull;

static const uint64_t P9A_MCC_4_MCAMOC                                        = 0x03010825ull;

static const uint64_t P9A_MCC_5_MCAMOC                                        = 0x03010835ull;

static const uint64_t P9A_MCC_6_MCAMOC                                        = 0x030108A5ull;

static const uint64_t P9A_MCC_7_MCAMOC                                        = 0x030108B5ull;


static const uint64_t P9A_MC_MCBCFGQ                                          = 0x070123E0ull;

static const uint64_t P9A_MC_0_MCBCFGQ                                        = 0x070123E0ull;

static const uint64_t P9A_MC_1_MCBCFGQ                                        = 0x080123E0ull;


static const uint64_t P9A_MC_MCBISTFIRACT0                                    = 0x07012306ull;

static const uint64_t P9A_MC_0_MCBISTFIRACT0                                  = 0x07012306ull;

static const uint64_t P9A_MC_1_MCBISTFIRACT0                                  = 0x08012306ull;


static const uint64_t P9A_MC_MCBISTFIRACT1                                    = 0x07012307ull;

static const uint64_t P9A_MC_0_MCBISTFIRACT1                                  = 0x07012307ull;

static const uint64_t P9A_MC_1_MCBISTFIRACT1                                  = 0x08012307ull;


static const uint64_t P9A_MC_MCBISTFIRMASK                                    = 0x07012303ull;

static const uint64_t P9A_MC_MCBISTFIRMASK_AND                                = 0x07012304ull;

static const uint64_t P9A_MC_MCBISTFIRMASK_OR                                 = 0x07012305ull;

static const uint64_t P9A_MC_0_MCBISTFIRMASK                                  = 0x07012303ull;

static const uint64_t P9A_MC_0_MCBISTFIRMASK_AND                              = 0x07012304ull;

static const uint64_t P9A_MC_0_MCBISTFIRMASK_OR                               = 0x07012305ull;

static const uint64_t P9A_MC_1_MCBISTFIRMASK                                  = 0x08012303ull;

static const uint64_t P9A_MC_1_MCBISTFIRMASK_AND                              = 0x08012304ull;

static const uint64_t P9A_MC_1_MCBISTFIRMASK_OR                               = 0x08012305ull;


static const uint64_t P9A_MC_MCBISTFIRQ                                       = 0x07012300ull;

static const uint64_t P9A_MC_MCBISTFIRQ_AND                                   = 0x07012301ull;

static const uint64_t P9A_MC_MCBISTFIRQ_OR                                    = 0x07012302ull;

static const uint64_t P9A_MC_0_MCBISTFIRQ                                     = 0x07012300ull;

static const uint64_t P9A_MC_0_MCBISTFIRQ_AND                                 = 0x07012301ull;

static const uint64_t P9A_MC_0_MCBISTFIRQ_OR                                  = 0x07012302ull;

static const uint64_t P9A_MC_1_MCBISTFIRQ                                     = 0x08012300ull;

static const uint64_t P9A_MC_1_MCBISTFIRQ_AND                                 = 0x08012301ull;

static const uint64_t P9A_MC_1_MCBISTFIRQ_OR                                  = 0x08012302ull;


static const uint64_t P9A_MC_MCBISTFIRWOF                                     = 0x07012308ull;

static const uint64_t P9A_MC_0_MCBISTFIRWOF                                   = 0x07012308ull;

static const uint64_t P9A_MC_1_MCBISTFIRWOF                                   = 0x08012308ull;


static const uint64_t P9A_MC_MCBPARMQ                                         = 0x070123AFull;

static const uint64_t P9A_MC_0_MCBPARMQ                                       = 0x070123AFull;

static const uint64_t P9A_MC_1_MCBPARMQ                                       = 0x080123AFull;


static const uint64_t P9A_MCC_MCBUSYQ                                         = 0x05010827ull;

static const uint64_t P9A_MCC_0_MCBUSYQ                                       = 0x05010827ull;

static const uint64_t P9A_MCC_1_MCBUSYQ                                       = 0x05010837ull;

static const uint64_t P9A_MCC_2_MCBUSYQ                                       = 0x050108A7ull;

static const uint64_t P9A_MCC_3_MCBUSYQ                                       = 0x050108B7ull;

static const uint64_t P9A_MCC_4_MCBUSYQ                                       = 0x03010827ull;

static const uint64_t P9A_MCC_5_MCBUSYQ                                       = 0x03010837ull;

static const uint64_t P9A_MCC_6_MCBUSYQ                                       = 0x030108A7ull;

static const uint64_t P9A_MCC_7_MCBUSYQ                                       = 0x030108B7ull;


static const uint64_t P9A_MC_MCDBG_SCOM_CFG                                   = 0x07012398ull;

static const uint64_t P9A_MC_0_MCDBG_SCOM_CFG                                 = 0x07012398ull;

static const uint64_t P9A_MC_1_MCDBG_SCOM_CFG                                 = 0x08012398ull;


static const uint64_t P9A_MCC_MCEBUSCL                                        = 0x05010829ull;

static const uint64_t P9A_MCC_0_MCEBUSCL                                      = 0x05010829ull;

static const uint64_t P9A_MCC_1_MCEBUSCL                                      = 0x05010839ull;

static const uint64_t P9A_MCC_2_MCEBUSCL                                      = 0x050108A9ull;

static const uint64_t P9A_MCC_3_MCEBUSCL                                      = 0x050108B9ull;

static const uint64_t P9A_MCC_4_MCEBUSCL                                      = 0x03010829ull;

static const uint64_t P9A_MCC_5_MCEBUSCL                                      = 0x03010839ull;

static const uint64_t P9A_MCC_6_MCEBUSCL                                      = 0x030108A9ull;

static const uint64_t P9A_MCC_7_MCEBUSCL                                      = 0x030108B9ull;


static const uint64_t P9A_MCC_MCEPSQ                                          = 0x05010826ull;

static const uint64_t P9A_MCC_0_MCEPSQ                                        = 0x05010826ull;

static const uint64_t P9A_MCC_1_MCEPSQ                                        = 0x05010836ull;

static const uint64_t P9A_MCC_2_MCEPSQ                                        = 0x050108A6ull;

static const uint64_t P9A_MCC_3_MCEPSQ                                        = 0x050108B6ull;

static const uint64_t P9A_MCC_4_MCEPSQ                                        = 0x03010826ull;

static const uint64_t P9A_MCC_5_MCEPSQ                                        = 0x03010836ull;

static const uint64_t P9A_MCC_6_MCEPSQ                                        = 0x030108A6ull;

static const uint64_t P9A_MCC_7_MCEPSQ                                        = 0x030108B6ull;


static const uint64_t P9A_MCC_MCERRINJ                                        = 0x05010828ull;

static const uint64_t P9A_MCC_0_MCERRINJ                                      = 0x05010828ull;

static const uint64_t P9A_MCC_1_MCERRINJ                                      = 0x05010838ull;

static const uint64_t P9A_MCC_2_MCERRINJ                                      = 0x050108A8ull;

static const uint64_t P9A_MCC_3_MCERRINJ                                      = 0x050108B8ull;

static const uint64_t P9A_MCC_4_MCERRINJ                                      = 0x03010828ull;

static const uint64_t P9A_MCC_5_MCERRINJ                                      = 0x03010838ull;

static const uint64_t P9A_MCC_6_MCERRINJ                                      = 0x030108A8ull;

static const uint64_t P9A_MCC_7_MCERRINJ                                      = 0x030108B8ull;


static const uint64_t P9A_MCC_MCPERF0                                         = 0x05010823ull;

static const uint64_t P9A_MCC_0_MCPERF0                                       = 0x05010823ull;

static const uint64_t P9A_MCC_1_MCPERF0                                       = 0x05010833ull;

static const uint64_t P9A_MCC_2_MCPERF0                                       = 0x050108A3ull;

static const uint64_t P9A_MCC_3_MCPERF0                                       = 0x050108B3ull;

static const uint64_t P9A_MCC_4_MCPERF0                                       = 0x03010823ull;

static const uint64_t P9A_MCC_5_MCPERF0                                       = 0x03010833ull;

static const uint64_t P9A_MCC_6_MCPERF0                                       = 0x030108A3ull;

static const uint64_t P9A_MCC_7_MCPERF0                                       = 0x030108B3ull;


static const uint64_t P9A_MCC_MCPERF2                                         = 0x05010824ull;

static const uint64_t P9A_MCC_0_MCPERF2                                       = 0x05010824ull;

static const uint64_t P9A_MCC_1_MCPERF2                                       = 0x05010834ull;

static const uint64_t P9A_MCC_2_MCPERF2                                       = 0x050108A4ull;

static const uint64_t P9A_MCC_3_MCPERF2                                       = 0x050108B4ull;

static const uint64_t P9A_MCC_4_MCPERF2                                       = 0x03010824ull;

static const uint64_t P9A_MCC_5_MCPERF2                                       = 0x03010834ull;

static const uint64_t P9A_MCC_6_MCPERF2                                       = 0x030108A4ull;

static const uint64_t P9A_MCC_7_MCPERF2                                       = 0x030108B4ull;


static const uint64_t P9A_MCC_MCPERF3                                         = 0x0501082Bull;

static const uint64_t P9A_MCC_0_MCPERF3                                       = 0x0501082Bull;

static const uint64_t P9A_MCC_1_MCPERF3                                       = 0x0501083Bull;

static const uint64_t P9A_MCC_2_MCPERF3                                       = 0x050108ABull;

static const uint64_t P9A_MCC_3_MCPERF3                                       = 0x050108BBull;

static const uint64_t P9A_MCC_4_MCPERF3                                       = 0x0301082Bull;

static const uint64_t P9A_MCC_5_MCPERF3                                       = 0x0301083Bull;

static const uint64_t P9A_MCC_6_MCPERF3                                       = 0x030108ABull;

static const uint64_t P9A_MCC_7_MCPERF3                                       = 0x030108BBull;


static const uint64_t P9A_MCC_MCWAT                                           = 0x0501082Aull;

static const uint64_t P9A_MCC_0_MCWAT                                         = 0x0501082Aull;

static const uint64_t P9A_MCC_1_MCWAT                                         = 0x0501083Aull;

static const uint64_t P9A_MCC_2_MCWAT                                         = 0x050108AAull;

static const uint64_t P9A_MCC_3_MCWAT                                         = 0x050108BAull;

static const uint64_t P9A_MCC_4_MCWAT                                         = 0x0301082Aull;

static const uint64_t P9A_MCC_5_MCWAT                                         = 0x0301083Aull;

static const uint64_t P9A_MCC_6_MCWAT                                         = 0x030108AAull;

static const uint64_t P9A_MCC_7_MCWAT                                         = 0x030108BAull;


static const uint64_t P9A_MC_MC_OMI_FIR_ACTION0_REG                           = 0x070133C6ull;
//DUPS: 070133C6, 07013386,
static const uint64_t P9A_MC_0_MC_OMI_FIR_ACTION0_REG                         = 0x070133C6ull;
//DUPS: 070133C6, 07013386,
static const uint64_t P9A_MC_1_MC_OMI_FIR_ACTION0_REG                         = 0x080133C6ull;
//DUPS: 080133C6, 08013386,

static const uint64_t P9A_MC_MC_OMI_FIR_ACTION1_REG                           = 0x070133C7ull;
//DUPS: 070133C7, 07013387,
static const uint64_t P9A_MC_0_MC_OMI_FIR_ACTION1_REG                         = 0x070133C7ull;
//DUPS: 070133C7, 07013387,
static const uint64_t P9A_MC_1_MC_OMI_FIR_ACTION1_REG                         = 0x080133C7ull;
//DUPS: 080133C7, 08013387,

static const uint64_t P9A_MC_MC_OMI_FIR_MASK_REG                              = 0x070133C3ull;
//DUPS: 070133C3, 07013383,
static const uint64_t P9A_MC_MC_OMI_FIR_MASK_REG_AND                          = 0x070133C4ull;
//DUPS: 070133C4, 07013384,
static const uint64_t P9A_MC_MC_OMI_FIR_MASK_REG_OR                           = 0x070133C5ull;
//DUPS: 070133C5, 07013385,
static const uint64_t P9A_MC_0_MC_OMI_FIR_MASK_REG                            = 0x070133C3ull;
//DUPS: 070133C3, 07013383,
static const uint64_t P9A_MC_0_MC_OMI_FIR_MASK_REG_AND                        = 0x070133C4ull;
//DUPS: 070133C4, 07013384,
static const uint64_t P9A_MC_0_MC_OMI_FIR_MASK_REG_OR                         = 0x070133C5ull;
//DUPS: 070133C5, 07013385,
static const uint64_t P9A_MC_1_MC_OMI_FIR_MASK_REG                            = 0x080133C3ull;
//DUPS: 080133C3, 08013383,
static const uint64_t P9A_MC_1_MC_OMI_FIR_MASK_REG_AND                        = 0x080133C4ull;
//DUPS: 080133C4, 08013384,
static const uint64_t P9A_MC_1_MC_OMI_FIR_MASK_REG_OR                         = 0x080133C5ull;
//DUPS: 080133C5, 08013385,

static const uint64_t P9A_MC_MC_OMI_FIR_REG                                   = 0x070133C0ull;
//DUPS: 070133C0, 07013380,
static const uint64_t P9A_MC_MC_OMI_FIR_REG_AND                               = 0x070133C1ull;
//DUPS: 070133C1, 07013381,
static const uint64_t P9A_MC_MC_OMI_FIR_REG_OR                                = 0x070133C2ull;
//DUPS: 070133C2, 07013382,
static const uint64_t P9A_MC_0_MC_OMI_FIR_REG                                 = 0x070133C0ull;
//DUPS: 070133C0, 07013380,
static const uint64_t P9A_MC_0_MC_OMI_FIR_REG_AND                             = 0x070133C1ull;
//DUPS: 070133C1, 07013381,
static const uint64_t P9A_MC_0_MC_OMI_FIR_REG_OR                              = 0x070133C2ull;
//DUPS: 070133C2, 07013382,
static const uint64_t P9A_MC_1_MC_OMI_FIR_REG                                 = 0x080133C0ull;
//DUPS: 080133C0, 08013380,
static const uint64_t P9A_MC_1_MC_OMI_FIR_REG_AND                             = 0x080133C1ull;
//DUPS: 080133C1, 08013381,
static const uint64_t P9A_MC_1_MC_OMI_FIR_REG_OR                              = 0x080133C2ull;
//DUPS: 080133C2, 08013382,

static const uint64_t P9A_MC_MC_OMI_FIR_WOF_REG                               = 0x070133C8ull;
//DUPS: 070133C8, 07013388,
static const uint64_t P9A_MC_0_MC_OMI_FIR_WOF_REG                             = 0x070133C8ull;
//DUPS: 070133C8, 07013388,
static const uint64_t P9A_MC_1_MC_OMI_FIR_WOF_REG                             = 0x080133C8ull;
//DUPS: 080133C8, 08013388,

static const uint64_t P9A_MC_MIB_XIICAC                                       = 0x07012C59ull;
//DUPS: 07012859, 07012C59,
static const uint64_t P9A_MC_0_MIB_XIICAC                                     = 0x07012C59ull;
//DUPS: 07012859, 07012C59,
static const uint64_t P9A_MC_1_MIB_XIICAC                                     = 0x08012C59ull;
//DUPS: 08012859, 08012C59,

static const uint64_t P9A_OMIC_MIB_XIICAC                                     = 0x07012459ull;

static const uint64_t P9A_OMIC_0_MIB_XIICAC                                   = 0x07012459ull;

static const uint64_t P9A_OMIC_1_MIB_XIICAC                                   = 0x07012859ull;

static const uint64_t P9A_OMIC_3_MIB_XIICAC                                   = 0x08012459ull;

static const uint64_t P9A_OMIC_4_MIB_XIICAC                                   = 0x08012859ull;


static const uint64_t P9A_MC_MIB_XIMEM                                        = 0x07012C57ull;
//DUPS: 07012857, 07012C57,
static const uint64_t P9A_MC_0_MIB_XIMEM                                      = 0x07012C57ull;
//DUPS: 07012857, 07012C57,
static const uint64_t P9A_MC_1_MIB_XIMEM                                      = 0x08012C57ull;
//DUPS: 08012857, 08012C57,

static const uint64_t P9A_OMIC_MIB_XIMEM                                      = 0x07012457ull;

static const uint64_t P9A_OMIC_0_MIB_XIMEM                                    = 0x07012457ull;

static const uint64_t P9A_OMIC_1_MIB_XIMEM                                    = 0x07012857ull;

static const uint64_t P9A_OMIC_3_MIB_XIMEM                                    = 0x08012457ull;

static const uint64_t P9A_OMIC_4_MIB_XIMEM                                    = 0x08012857ull;


static const uint64_t P9A_MC_MIB_XISGB                                        = 0x07012C58ull;
//DUPS: 07012858, 07012C58,
static const uint64_t P9A_MC_0_MIB_XISGB                                      = 0x07012C58ull;
//DUPS: 07012858, 07012C58,
static const uint64_t P9A_MC_1_MIB_XISGB                                      = 0x08012C58ull;
//DUPS: 08012858, 08012C58,

static const uint64_t P9A_OMIC_MIB_XISGB                                      = 0x07012458ull;

static const uint64_t P9A_OMIC_0_MIB_XISGB                                    = 0x07012458ull;

static const uint64_t P9A_OMIC_1_MIB_XISGB                                    = 0x07012858ull;

static const uint64_t P9A_OMIC_3_MIB_XISGB                                    = 0x08012458ull;

static const uint64_t P9A_OMIC_4_MIB_XISGB                                    = 0x08012858ull;


static const uint64_t P9A_MC_MODE_REG                                         = 0x07040008ull;

static const uint64_t P9A_MC_0_MODE_REG                                       = 0x07040008ull;

static const uint64_t P9A_MC_1_MODE_REG                                       = 0x08040008ull;


static const uint64_t P9A_MC_MULTICAST_GROUP_1                                = 0x070F0001ull;

static const uint64_t P9A_MC_0_MULTICAST_GROUP_1                              = 0x070F0001ull;

static const uint64_t P9A_MC_1_MULTICAST_GROUP_1                              = 0x080F0001ull;


static const uint64_t P9A_MC_MULTICAST_GROUP_2                                = 0x070F0002ull;

static const uint64_t P9A_MC_0_MULTICAST_GROUP_2                              = 0x070F0002ull;

static const uint64_t P9A_MC_1_MULTICAST_GROUP_2                              = 0x080F0002ull;


static const uint64_t P9A_MC_MULTICAST_GROUP_3                                = 0x070F0003ull;

static const uint64_t P9A_MC_0_MULTICAST_GROUP_3                              = 0x070F0003ull;

static const uint64_t P9A_MC_1_MULTICAST_GROUP_3                              = 0x080F0003ull;


static const uint64_t P9A_MC_MULTICAST_GROUP_4                                = 0x070F0004ull;

static const uint64_t P9A_MC_0_MULTICAST_GROUP_4                              = 0x070F0004ull;

static const uint64_t P9A_MC_1_MULTICAST_GROUP_4                              = 0x080F0004ull;


static const uint64_t P9A_MC_NET_CTRL0                                        = 0x070F0040ull;

static const uint64_t P9A_MC_NET_CTRL0_WAND                                   = 0x070F0041ull;

static const uint64_t P9A_MC_NET_CTRL0_WOR                                    = 0x070F0042ull;

static const uint64_t P9A_MC_0_NET_CTRL0                                      = 0x070F0040ull;

static const uint64_t P9A_MC_0_NET_CTRL0_WAND                                 = 0x070F0041ull;

static const uint64_t P9A_MC_0_NET_CTRL0_WOR                                  = 0x070F0042ull;

static const uint64_t P9A_MC_1_NET_CTRL0                                      = 0x080F0040ull;

static const uint64_t P9A_MC_1_NET_CTRL0_WAND                                 = 0x080F0041ull;

static const uint64_t P9A_MC_1_NET_CTRL0_WOR                                  = 0x080F0042ull;


static const uint64_t P9A_MC_NET_CTRL1                                        = 0x070F0044ull;

static const uint64_t P9A_MC_NET_CTRL1_WAND                                   = 0x070F0045ull;

static const uint64_t P9A_MC_NET_CTRL1_WOR                                    = 0x070F0046ull;

static const uint64_t P9A_MC_0_NET_CTRL1                                      = 0x070F0044ull;

static const uint64_t P9A_MC_0_NET_CTRL1_WAND                                 = 0x070F0045ull;

static const uint64_t P9A_MC_0_NET_CTRL1_WOR                                  = 0x070F0046ull;

static const uint64_t P9A_MC_1_NET_CTRL1                                      = 0x080F0044ull;

static const uint64_t P9A_MC_1_NET_CTRL1_WAND                                 = 0x080F0045ull;

static const uint64_t P9A_MC_1_NET_CTRL1_WOR                                  = 0x080F0046ull;


static const uint64_t P9A_MC_OPCG_ALIGN                                       = 0x07030001ull;

static const uint64_t P9A_MC_0_OPCG_ALIGN                                     = 0x07030001ull;

static const uint64_t P9A_MC_1_OPCG_ALIGN                                     = 0x08030001ull;


static const uint64_t P9A_MC_OPCG_CAPT1                                       = 0x07030010ull;

static const uint64_t P9A_MC_0_OPCG_CAPT1                                     = 0x07030010ull;

static const uint64_t P9A_MC_1_OPCG_CAPT1                                     = 0x08030010ull;


static const uint64_t P9A_MC_OPCG_CAPT2                                       = 0x07030011ull;

static const uint64_t P9A_MC_0_OPCG_CAPT2                                     = 0x07030011ull;

static const uint64_t P9A_MC_1_OPCG_CAPT2                                     = 0x08030011ull;


static const uint64_t P9A_MC_OPCG_CAPT3                                       = 0x07030012ull;

static const uint64_t P9A_MC_0_OPCG_CAPT3                                     = 0x07030012ull;

static const uint64_t P9A_MC_1_OPCG_CAPT3                                     = 0x08030012ull;


static const uint64_t P9A_MC_OPCG_REG0                                        = 0x07030002ull;

static const uint64_t P9A_MC_0_OPCG_REG0                                      = 0x07030002ull;

static const uint64_t P9A_MC_1_OPCG_REG0                                      = 0x08030002ull;


static const uint64_t P9A_MC_OPCG_REG1                                        = 0x07030003ull;

static const uint64_t P9A_MC_0_OPCG_REG1                                      = 0x07030003ull;

static const uint64_t P9A_MC_1_OPCG_REG1                                      = 0x08030003ull;


static const uint64_t P9A_MC_OPCG_REG2                                        = 0x07030004ull;

static const uint64_t P9A_MC_0_OPCG_REG2                                      = 0x07030004ull;

static const uint64_t P9A_MC_1_OPCG_REG2                                      = 0x08030004ull;


static const uint64_t P9A_MC_PCB_OPCG_GO                                      = 0x07030020ull;

static const uint64_t P9A_MC_0_PCB_OPCG_GO                                    = 0x07030020ull;

static const uint64_t P9A_MC_1_PCB_OPCG_GO                                    = 0x08030020ull;


static const uint64_t P9A_MC_PCB_OPCG_STOP                                    = 0x07030030ull;

static const uint64_t P9A_MC_0_PCB_OPCG_STOP                                  = 0x07030030ull;

static const uint64_t P9A_MC_1_PCB_OPCG_STOP                                  = 0x08030030ull;


static const uint64_t P9A_MC_PLL_LOCK_REG                                     = 0x070F0019ull;

static const uint64_t P9A_MC_0_PLL_LOCK_REG                                   = 0x070F0019ull;

static const uint64_t P9A_MC_1_PLL_LOCK_REG                                   = 0x080F0019ull;


static const uint64_t P9A_MC_PMU_CNTR                                         = 0x070133CFull;
//DUPS: 070133CF, 0701338F,
static const uint64_t P9A_MC_0_PMU_CNTR                                       = 0x070133CFull;
//DUPS: 070133CF, 0701338F,
static const uint64_t P9A_MC_1_PMU_CNTR                                       = 0x080133CFull;
//DUPS: 080133CF, 0801338F,

static const uint64_t P9A_MC_PPE_FIR_ACTION0_REG                              = 0x07012C46ull;
//DUPS: 07012846, 07012C46,
static const uint64_t P9A_MC_0_PPE_FIR_ACTION0_REG                            = 0x07012C46ull;
//DUPS: 07012846, 07012C46,
static const uint64_t P9A_MC_1_PPE_FIR_ACTION0_REG                            = 0x08012C46ull;
//DUPS: 08012846, 08012C46,

static const uint64_t P9A_OMIC_PPE_FIR_ACTION0_REG                            = 0x07012446ull;

static const uint64_t P9A_OMIC_0_PPE_FIR_ACTION0_REG                          = 0x07012446ull;

static const uint64_t P9A_OMIC_1_PPE_FIR_ACTION0_REG                          = 0x07012846ull;

static const uint64_t P9A_OMIC_3_PPE_FIR_ACTION0_REG                          = 0x08012446ull;

static const uint64_t P9A_OMIC_4_PPE_FIR_ACTION0_REG                          = 0x08012846ull;


static const uint64_t P9A_MC_PPE_FIR_ACTION1_REG                              = 0x07012C47ull;
//DUPS: 07012847, 07012C47,
static const uint64_t P9A_MC_0_PPE_FIR_ACTION1_REG                            = 0x07012C47ull;
//DUPS: 07012847, 07012C47,
static const uint64_t P9A_MC_1_PPE_FIR_ACTION1_REG                            = 0x08012C47ull;
//DUPS: 08012847, 08012C47,

static const uint64_t P9A_OMIC_PPE_FIR_ACTION1_REG                            = 0x07012447ull;

static const uint64_t P9A_OMIC_0_PPE_FIR_ACTION1_REG                          = 0x07012447ull;

static const uint64_t P9A_OMIC_1_PPE_FIR_ACTION1_REG                          = 0x07012847ull;

static const uint64_t P9A_OMIC_3_PPE_FIR_ACTION1_REG                          = 0x08012447ull;

static const uint64_t P9A_OMIC_4_PPE_FIR_ACTION1_REG                          = 0x08012847ull;


static const uint64_t P9A_MC_PPE_FIR_MASK_REG                                 = 0x07012C43ull;
//DUPS: 07012843, 07012C43,
static const uint64_t P9A_MC_PPE_FIR_MASK_REG_AND                             = 0x07012C44ull;
//DUPS: 07012844, 07012C44,
static const uint64_t P9A_MC_PPE_FIR_MASK_REG_OR                              = 0x07012C45ull;
//DUPS: 07012845, 07012C45,
static const uint64_t P9A_MC_0_PPE_FIR_MASK_REG                               = 0x07012C43ull;
//DUPS: 07012843, 07012C43,
static const uint64_t P9A_MC_0_PPE_FIR_MASK_REG_AND                           = 0x07012C44ull;
//DUPS: 07012844, 07012C44,
static const uint64_t P9A_MC_0_PPE_FIR_MASK_REG_OR                            = 0x07012C45ull;
//DUPS: 07012845, 07012C45,
static const uint64_t P9A_MC_1_PPE_FIR_MASK_REG                               = 0x08012C43ull;
//DUPS: 08012843, 08012C43,
static const uint64_t P9A_MC_1_PPE_FIR_MASK_REG_AND                           = 0x08012C44ull;
//DUPS: 08012844, 08012C44,
static const uint64_t P9A_MC_1_PPE_FIR_MASK_REG_OR                            = 0x08012C45ull;
//DUPS: 08012845, 08012C45,

static const uint64_t P9A_OMIC_PPE_FIR_MASK_REG                               = 0x07012443ull;

static const uint64_t P9A_OMIC_PPE_FIR_MASK_REG_AND                           = 0x07012444ull;

static const uint64_t P9A_OMIC_PPE_FIR_MASK_REG_OR                            = 0x07012445ull;

static const uint64_t P9A_OMIC_0_PPE_FIR_MASK_REG                             = 0x07012443ull;

static const uint64_t P9A_OMIC_0_PPE_FIR_MASK_REG_AND                         = 0x07012444ull;

static const uint64_t P9A_OMIC_0_PPE_FIR_MASK_REG_OR                          = 0x07012445ull;

static const uint64_t P9A_OMIC_1_PPE_FIR_MASK_REG                             = 0x07012843ull;

static const uint64_t P9A_OMIC_1_PPE_FIR_MASK_REG_AND                         = 0x07012844ull;

static const uint64_t P9A_OMIC_1_PPE_FIR_MASK_REG_OR                          = 0x07012845ull;

static const uint64_t P9A_OMIC_3_PPE_FIR_MASK_REG                             = 0x08012443ull;

static const uint64_t P9A_OMIC_3_PPE_FIR_MASK_REG_AND                         = 0x08012444ull;

static const uint64_t P9A_OMIC_3_PPE_FIR_MASK_REG_OR                          = 0x08012445ull;

static const uint64_t P9A_OMIC_4_PPE_FIR_MASK_REG                             = 0x08012843ull;

static const uint64_t P9A_OMIC_4_PPE_FIR_MASK_REG_AND                         = 0x08012844ull;

static const uint64_t P9A_OMIC_4_PPE_FIR_MASK_REG_OR                          = 0x08012845ull;


static const uint64_t P9A_MC_PPE_FIR_REG                                      = 0x07012C40ull;
//DUPS: 07012840, 07012C40,
static const uint64_t P9A_MC_PPE_FIR_REG_AND                                  = 0x07012C41ull;
//DUPS: 07012841, 07012C41,
static const uint64_t P9A_MC_PPE_FIR_REG_OR                                   = 0x07012C42ull;
//DUPS: 07012842, 07012C42,
static const uint64_t P9A_MC_0_PPE_FIR_REG                                    = 0x07012C40ull;
//DUPS: 07012840, 07012C40,
static const uint64_t P9A_MC_0_PPE_FIR_REG_AND                                = 0x07012C41ull;
//DUPS: 07012841, 07012C41,
static const uint64_t P9A_MC_0_PPE_FIR_REG_OR                                 = 0x07012C42ull;
//DUPS: 07012842, 07012C42,
static const uint64_t P9A_MC_1_PPE_FIR_REG                                    = 0x08012C40ull;
//DUPS: 08012840, 08012C40,
static const uint64_t P9A_MC_1_PPE_FIR_REG_AND                                = 0x08012C41ull;
//DUPS: 08012841, 08012C41,
static const uint64_t P9A_MC_1_PPE_FIR_REG_OR                                 = 0x08012C42ull;
//DUPS: 08012842, 08012C42,

static const uint64_t P9A_OMIC_PPE_FIR_REG                                    = 0x07012440ull;

static const uint64_t P9A_OMIC_PPE_FIR_REG_AND                                = 0x07012441ull;

static const uint64_t P9A_OMIC_PPE_FIR_REG_OR                                 = 0x07012442ull;

static const uint64_t P9A_OMIC_0_PPE_FIR_REG                                  = 0x07012440ull;

static const uint64_t P9A_OMIC_0_PPE_FIR_REG_AND                              = 0x07012441ull;

static const uint64_t P9A_OMIC_0_PPE_FIR_REG_OR                               = 0x07012442ull;

static const uint64_t P9A_OMIC_1_PPE_FIR_REG                                  = 0x07012840ull;

static const uint64_t P9A_OMIC_1_PPE_FIR_REG_AND                              = 0x07012841ull;

static const uint64_t P9A_OMIC_1_PPE_FIR_REG_OR                               = 0x07012842ull;

static const uint64_t P9A_OMIC_3_PPE_FIR_REG                                  = 0x08012440ull;

static const uint64_t P9A_OMIC_3_PPE_FIR_REG_AND                              = 0x08012441ull;

static const uint64_t P9A_OMIC_3_PPE_FIR_REG_OR                               = 0x08012442ull;

static const uint64_t P9A_OMIC_4_PPE_FIR_REG                                  = 0x08012840ull;

static const uint64_t P9A_OMIC_4_PPE_FIR_REG_AND                              = 0x08012841ull;

static const uint64_t P9A_OMIC_4_PPE_FIR_REG_OR                               = 0x08012842ull;


static const uint64_t P9A_MC_PPE_FIR_WOF_REG                                  = 0x07012C48ull;
//DUPS: 07012848, 07012C48,
static const uint64_t P9A_MC_0_PPE_FIR_WOF_REG                                = 0x07012C48ull;
//DUPS: 07012848, 07012C48,
static const uint64_t P9A_MC_1_PPE_FIR_WOF_REG                                = 0x08012C48ull;
//DUPS: 08012848, 08012C48,

static const uint64_t P9A_OMIC_PPE_FIR_WOF_REG                                = 0x07012448ull;

static const uint64_t P9A_OMIC_0_PPE_FIR_WOF_REG                              = 0x07012448ull;

static const uint64_t P9A_OMIC_1_PPE_FIR_WOF_REG                              = 0x07012848ull;

static const uint64_t P9A_OMIC_3_PPE_FIR_WOF_REG                              = 0x08012448ull;

static const uint64_t P9A_OMIC_4_PPE_FIR_WOF_REG                              = 0x08012848ull;


static const uint64_t P9A_MC_PPE_XIDBGPRO                                     = 0x07012C55ull;
//DUPS: 07012855, 07012C55,
static const uint64_t P9A_MC_0_PPE_XIDBGPRO                                   = 0x07012C55ull;
//DUPS: 07012855, 07012C55,
static const uint64_t P9A_MC_1_PPE_XIDBGPRO                                   = 0x08012C55ull;
//DUPS: 08012855, 08012C55,

static const uint64_t P9A_OMIC_PPE_XIDBGPRO                                   = 0x07012455ull;

static const uint64_t P9A_OMIC_0_PPE_XIDBGPRO                                 = 0x07012455ull;

static const uint64_t P9A_OMIC_1_PPE_XIDBGPRO                                 = 0x07012855ull;

static const uint64_t P9A_OMIC_3_PPE_XIDBGPRO                                 = 0x08012455ull;

static const uint64_t P9A_OMIC_4_PPE_XIDBGPRO                                 = 0x08012855ull;


static const uint64_t P9A_MC_PPE_XIRAMDBG                                     = 0x07012C53ull;
//DUPS: 07012853, 07012C53,
static const uint64_t P9A_MC_0_PPE_XIRAMDBG                                   = 0x07012C53ull;
//DUPS: 07012853, 07012C53,
static const uint64_t P9A_MC_1_PPE_XIRAMDBG                                   = 0x08012C53ull;
//DUPS: 08012853, 08012C53,

static const uint64_t P9A_OMIC_PPE_XIRAMDBG                                   = 0x07012453ull;

static const uint64_t P9A_OMIC_0_PPE_XIRAMDBG                                 = 0x07012453ull;

static const uint64_t P9A_OMIC_1_PPE_XIRAMDBG                                 = 0x07012853ull;

static const uint64_t P9A_OMIC_3_PPE_XIRAMDBG                                 = 0x08012453ull;

static const uint64_t P9A_OMIC_4_PPE_XIRAMDBG                                 = 0x08012853ull;


static const uint64_t P9A_MC_PPE_XIRAMEDR                                     = 0x07012C54ull;
//DUPS: 07012854, 07012C54,
static const uint64_t P9A_MC_0_PPE_XIRAMEDR                                   = 0x07012C54ull;
//DUPS: 07012854, 07012C54,
static const uint64_t P9A_MC_1_PPE_XIRAMEDR                                   = 0x08012C54ull;
//DUPS: 08012854, 08012C54,

static const uint64_t P9A_OMIC_PPE_XIRAMEDR                                   = 0x07012454ull;

static const uint64_t P9A_OMIC_0_PPE_XIRAMEDR                                 = 0x07012454ull;

static const uint64_t P9A_OMIC_1_PPE_XIRAMEDR                                 = 0x07012854ull;

static const uint64_t P9A_OMIC_3_PPE_XIRAMEDR                                 = 0x08012454ull;

static const uint64_t P9A_OMIC_4_PPE_XIRAMEDR                                 = 0x08012854ull;


static const uint64_t P9A_MC_PPE_XIRAMGA                                      = 0x07012C52ull;
//DUPS: 07012852, 07012C52,
static const uint64_t P9A_MC_0_PPE_XIRAMGA                                    = 0x07012C52ull;
//DUPS: 07012852, 07012C52,
static const uint64_t P9A_MC_1_PPE_XIRAMGA                                    = 0x08012C52ull;
//DUPS: 08012852, 08012C52,

static const uint64_t P9A_OMIC_PPE_XIRAMGA                                    = 0x07012452ull;

static const uint64_t P9A_OMIC_0_PPE_XIRAMGA                                  = 0x07012452ull;

static const uint64_t P9A_OMIC_1_PPE_XIRAMGA                                  = 0x07012852ull;

static const uint64_t P9A_OMIC_3_PPE_XIRAMGA                                  = 0x08012452ull;

static const uint64_t P9A_OMIC_4_PPE_XIRAMGA                                  = 0x08012852ull;


static const uint64_t P9A_MC_PPE_XIRAMRA                                      = 0x07012C51ull;
//DUPS: 07012851, 07012C51,
static const uint64_t P9A_MC_0_PPE_XIRAMRA                                    = 0x07012C51ull;
//DUPS: 07012851, 07012C51,
static const uint64_t P9A_MC_1_PPE_XIRAMRA                                    = 0x08012C51ull;
//DUPS: 08012851, 08012C51,

static const uint64_t P9A_OMIC_PPE_XIRAMRA                                    = 0x07012451ull;

static const uint64_t P9A_OMIC_0_PPE_XIRAMRA                                  = 0x07012451ull;

static const uint64_t P9A_OMIC_1_PPE_XIRAMRA                                  = 0x07012851ull;

static const uint64_t P9A_OMIC_3_PPE_XIRAMRA                                  = 0x08012451ull;

static const uint64_t P9A_OMIC_4_PPE_XIRAMRA                                  = 0x08012851ull;


static const uint64_t P9A_MC_PPE_XIXCR                                        = 0x07012C50ull;
//DUPS: 07012850, 07012C50,
static const uint64_t P9A_MC_0_PPE_XIXCR                                      = 0x07012C50ull;
//DUPS: 07012850, 07012C50,
static const uint64_t P9A_MC_1_PPE_XIXCR                                      = 0x08012C50ull;
//DUPS: 08012850, 08012C50,

static const uint64_t P9A_OMIC_PPE_XIXCR                                      = 0x07012450ull;

static const uint64_t P9A_OMIC_0_PPE_XIXCR                                    = 0x07012450ull;

static const uint64_t P9A_OMIC_1_PPE_XIXCR                                    = 0x07012850ull;

static const uint64_t P9A_OMIC_3_PPE_XIXCR                                    = 0x08012450ull;

static const uint64_t P9A_OMIC_4_PPE_XIXCR                                    = 0x08012850ull;


static const uint64_t P9A_MC_PRE_COUNTER_REG                                  = 0x070F0028ull;

static const uint64_t P9A_MC_0_PRE_COUNTER_REG                                = 0x070F0028ull;

static const uint64_t P9A_MC_1_PRE_COUNTER_REG                                = 0x080F0028ull;


static const uint64_t P9A_MC_PRIMARY_ADDRESS_REG                              = 0x070F0000ull;

static const uint64_t P9A_MC_0_PRIMARY_ADDRESS_REG                            = 0x070F0000ull;

static const uint64_t P9A_MC_1_PRIMARY_ADDRESS_REG                            = 0x080F0000ull;


static const uint64_t P9A_MC_PROTECT_MODE_REG                                 = 0x070F03FEull;

static const uint64_t P9A_MC_0_PROTECT_MODE_REG                               = 0x070F03FEull;

static const uint64_t P9A_MC_1_PROTECT_MODE_REG                               = 0x080F03FEull;


static const uint64_t P9A_MC_PSCOM_ERROR_MASK                                 = 0x07010002ull;

static const uint64_t P9A_MC_0_PSCOM_ERROR_MASK                               = 0x07010002ull;

static const uint64_t P9A_MC_1_PSCOM_ERROR_MASK                               = 0x08010002ull;


static const uint64_t P9A_MC_PSCOM_MODE_REG                                   = 0x07010000ull;

static const uint64_t P9A_MC_0_PSCOM_MODE_REG                                 = 0x07010000ull;

static const uint64_t P9A_MC_1_PSCOM_MODE_REG                                 = 0x08010000ull;


static const uint64_t P9A_MC_PSCOM_STATUS_ERROR_REG                           = 0x07010001ull;

static const uint64_t P9A_MC_0_PSCOM_STATUS_ERROR_REG                         = 0x07010001ull;

static const uint64_t P9A_MC_1_PSCOM_STATUS_ERROR_REG                         = 0x08010001ull;


static const uint64_t P9A_MC_READCFG                                          = 0x0701234Dull;
//DUPS: 0701237D, 0701234D, 0701235D,
static const uint64_t P9A_MC_0_READCFG                                        = 0x0701234Dull;
//DUPS: 0701237D, 0701234D, 0701235D,
static const uint64_t P9A_MC_1_READCFG                                        = 0x0801234Dull;
//DUPS: 0801237D, 0801234D, 0801235D,

static const uint64_t P9A_MC_RECOV_INTERRUPT_REG                              = 0x070F001Bull;

static const uint64_t P9A_MC_0_RECOV_INTERRUPT_REG                            = 0x070F001Bull;

static const uint64_t P9A_MC_1_RECOV_INTERRUPT_REG                            = 0x080F001Bull;


static const uint64_t P9A_MC_REG0_DL0_CONFIG0                                 = 0x07013350ull;

static const uint64_t P9A_MC_0_REG0_DL0_CONFIG0                               = 0x07013350ull;

static const uint64_t P9A_MC_1_REG0_DL0_CONFIG0                               = 0x08013350ull;

static const uint64_t P9A_OMI_REG0_DL0_CONFIG0                                = 0x07013350ull;

static const uint64_t P9A_OMI_0_REG0_DL0_CONFIG0                              = 0x07013350ull;

static const uint64_t P9A_OMI_8_REG0_DL0_CONFIG0                              = 0x08013350ull;


static const uint64_t P9A_MC_REG0_DL0_CONFIG1                                 = 0x07013351ull;

static const uint64_t P9A_MC_0_REG0_DL0_CONFIG1                               = 0x07013351ull;

static const uint64_t P9A_MC_1_REG0_DL0_CONFIG1                               = 0x08013351ull;

static const uint64_t P9A_OMI_REG0_DL0_CONFIG1                                = 0x07013351ull;

static const uint64_t P9A_OMI_0_REG0_DL0_CONFIG1                              = 0x07013351ull;

static const uint64_t P9A_OMI_8_REG0_DL0_CONFIG1                              = 0x08013351ull;


static const uint64_t P9A_MC_REG0_DL0_CYA_BITS                                = 0x0701335Full;

static const uint64_t P9A_MC_0_REG0_DL0_CYA_BITS                              = 0x0701335Full;

static const uint64_t P9A_MC_1_REG0_DL0_CYA_BITS                              = 0x0801335Full;

static const uint64_t P9A_OMI_REG0_DL0_CYA_BITS                               = 0x0701335Full;

static const uint64_t P9A_OMI_0_REG0_DL0_CYA_BITS                             = 0x0701335Full;

static const uint64_t P9A_OMI_8_REG0_DL0_CYA_BITS                             = 0x0801335Full;


static const uint64_t P9A_MC_REG0_DL0_DLX_CONFIG                              = 0x07013358ull;

static const uint64_t P9A_MC_0_REG0_DL0_DLX_CONFIG                            = 0x07013358ull;

static const uint64_t P9A_MC_1_REG0_DL0_DLX_CONFIG                            = 0x08013358ull;

static const uint64_t P9A_OMI_REG0_DL0_DLX_CONFIG                             = 0x07013358ull;

static const uint64_t P9A_OMI_0_REG0_DL0_DLX_CONFIG                           = 0x07013358ull;

static const uint64_t P9A_OMI_8_REG0_DL0_DLX_CONFIG                           = 0x08013358ull;


static const uint64_t P9A_MC_REG0_DL0_DLX_INFO                                = 0x07013359ull;

static const uint64_t P9A_MC_0_REG0_DL0_DLX_INFO                              = 0x07013359ull;

static const uint64_t P9A_MC_1_REG0_DL0_DLX_INFO                              = 0x08013359ull;

static const uint64_t P9A_OMI_REG0_DL0_DLX_INFO                               = 0x07013359ull;

static const uint64_t P9A_OMI_0_REG0_DL0_DLX_INFO                             = 0x07013359ull;

static const uint64_t P9A_OMI_8_REG0_DL0_DLX_INFO                             = 0x08013359ull;


static const uint64_t P9A_MC_REG0_DL0_EDPL_MAX_COUNT                          = 0x07013355ull;

static const uint64_t P9A_MC_0_REG0_DL0_EDPL_MAX_COUNT                        = 0x07013355ull;

static const uint64_t P9A_MC_1_REG0_DL0_EDPL_MAX_COUNT                        = 0x08013355ull;

static const uint64_t P9A_OMI_REG0_DL0_EDPL_MAX_COUNT                         = 0x07013355ull;

static const uint64_t P9A_OMI_0_REG0_DL0_EDPL_MAX_COUNT                       = 0x07013355ull;

static const uint64_t P9A_OMI_8_REG0_DL0_EDPL_MAX_COUNT                       = 0x08013355ull;


static const uint64_t P9A_MC_REG0_DL0_ERROR_CAPTURE                           = 0x07013354ull;

static const uint64_t P9A_MC_0_REG0_DL0_ERROR_CAPTURE                         = 0x07013354ull;

static const uint64_t P9A_MC_1_REG0_DL0_ERROR_CAPTURE                         = 0x08013354ull;

static const uint64_t P9A_OMI_REG0_DL0_ERROR_CAPTURE                          = 0x07013354ull;

static const uint64_t P9A_OMI_0_REG0_DL0_ERROR_CAPTURE                        = 0x07013354ull;

static const uint64_t P9A_OMI_8_REG0_DL0_ERROR_CAPTURE                        = 0x08013354ull;


static const uint64_t P9A_MC_REG0_DL0_ERROR_HOLD                              = 0x07013353ull;

static const uint64_t P9A_MC_0_REG0_DL0_ERROR_HOLD                            = 0x07013353ull;

static const uint64_t P9A_MC_1_REG0_DL0_ERROR_HOLD                            = 0x08013353ull;

static const uint64_t P9A_OMI_REG0_DL0_ERROR_HOLD                             = 0x07013353ull;

static const uint64_t P9A_OMI_0_REG0_DL0_ERROR_HOLD                           = 0x07013353ull;

static const uint64_t P9A_OMI_8_REG0_DL0_ERROR_HOLD                           = 0x08013353ull;


static const uint64_t P9A_MC_REG0_DL0_ERROR_MASK                              = 0x07013352ull;

static const uint64_t P9A_MC_0_REG0_DL0_ERROR_MASK                            = 0x07013352ull;

static const uint64_t P9A_MC_1_REG0_DL0_ERROR_MASK                            = 0x08013352ull;

static const uint64_t P9A_OMI_REG0_DL0_ERROR_MASK                             = 0x07013352ull;

static const uint64_t P9A_OMI_0_REG0_DL0_ERROR_MASK                           = 0x07013352ull;

static const uint64_t P9A_OMI_8_REG0_DL0_ERROR_MASK                           = 0x08013352ull;


static const uint64_t P9A_MC_REG0_DL0_STATUS                                  = 0x07013356ull;

static const uint64_t P9A_MC_0_REG0_DL0_STATUS                                = 0x07013356ull;

static const uint64_t P9A_MC_1_REG0_DL0_STATUS                                = 0x08013356ull;

static const uint64_t P9A_OMI_REG0_DL0_STATUS                                 = 0x07013356ull;

static const uint64_t P9A_OMI_0_REG0_DL0_STATUS                               = 0x07013356ull;

static const uint64_t P9A_OMI_8_REG0_DL0_STATUS                               = 0x08013356ull;


static const uint64_t P9A_MC_REG0_DL0_TRAINING_STATUS                         = 0x07013357ull;

static const uint64_t P9A_MC_0_REG0_DL0_TRAINING_STATUS                       = 0x07013357ull;

static const uint64_t P9A_MC_1_REG0_DL0_TRAINING_STATUS                       = 0x08013357ull;

static const uint64_t P9A_OMI_REG0_DL0_TRAINING_STATUS                        = 0x07013357ull;

static const uint64_t P9A_OMI_0_REG0_DL0_TRAINING_STATUS                      = 0x07013357ull;

static const uint64_t P9A_OMI_8_REG0_DL0_TRAINING_STATUS                      = 0x08013357ull;


static const uint64_t P9A_MC_REG0_DL1_CONFIG0                                 = 0x07013360ull;

static const uint64_t P9A_MC_0_REG0_DL1_CONFIG0                               = 0x07013360ull;

static const uint64_t P9A_MC_1_REG0_DL1_CONFIG0                               = 0x08013360ull;

static const uint64_t P9A_OMI_1_REG0_DL1_CONFIG0                              = 0x07013360ull;

static const uint64_t P9A_OMI_9_REG0_DL1_CONFIG0                              = 0x08013360ull;


static const uint64_t P9A_MC_REG0_DL1_CONFIG1                                 = 0x07013361ull;

static const uint64_t P9A_MC_0_REG0_DL1_CONFIG1                               = 0x07013361ull;

static const uint64_t P9A_MC_1_REG0_DL1_CONFIG1                               = 0x08013361ull;

static const uint64_t P9A_OMI_1_REG0_DL1_CONFIG1                              = 0x07013361ull;

static const uint64_t P9A_OMI_9_REG0_DL1_CONFIG1                              = 0x08013361ull;


static const uint64_t P9A_MC_REG0_DL1_CYA_BITS                                = 0x0701336Full;

static const uint64_t P9A_MC_0_REG0_DL1_CYA_BITS                              = 0x0701336Full;

static const uint64_t P9A_MC_1_REG0_DL1_CYA_BITS                              = 0x0801336Full;

static const uint64_t P9A_OMI_1_REG0_DL1_CYA_BITS                             = 0x0701336Full;

static const uint64_t P9A_OMI_9_REG0_DL1_CYA_BITS                             = 0x0801336Full;


static const uint64_t P9A_MC_REG0_DL1_DLX_CONFIG                              = 0x07013368ull;

static const uint64_t P9A_MC_0_REG0_DL1_DLX_CONFIG                            = 0x07013368ull;

static const uint64_t P9A_MC_1_REG0_DL1_DLX_CONFIG                            = 0x08013368ull;

static const uint64_t P9A_OMI_1_REG0_DL1_DLX_CONFIG                           = 0x07013368ull;

static const uint64_t P9A_OMI_9_REG0_DL1_DLX_CONFIG                           = 0x08013368ull;


static const uint64_t P9A_MC_REG0_DL1_DLX_INFO                                = 0x07013369ull;

static const uint64_t P9A_MC_0_REG0_DL1_DLX_INFO                              = 0x07013369ull;

static const uint64_t P9A_MC_1_REG0_DL1_DLX_INFO                              = 0x08013369ull;

static const uint64_t P9A_OMI_1_REG0_DL1_DLX_INFO                             = 0x07013369ull;

static const uint64_t P9A_OMI_9_REG0_DL1_DLX_INFO                             = 0x08013369ull;


static const uint64_t P9A_MC_REG0_DL1_EDPL_MAX_COUNT                          = 0x07013365ull;

static const uint64_t P9A_MC_0_REG0_DL1_EDPL_MAX_COUNT                        = 0x07013365ull;

static const uint64_t P9A_MC_1_REG0_DL1_EDPL_MAX_COUNT                        = 0x08013365ull;

static const uint64_t P9A_OMI_1_REG0_DL1_EDPL_MAX_COUNT                       = 0x07013365ull;

static const uint64_t P9A_OMI_9_REG0_DL1_EDPL_MAX_COUNT                       = 0x08013365ull;


static const uint64_t P9A_MC_REG0_DL1_ERROR_CAPTURE                           = 0x07013364ull;

static const uint64_t P9A_MC_0_REG0_DL1_ERROR_CAPTURE                         = 0x07013364ull;

static const uint64_t P9A_MC_1_REG0_DL1_ERROR_CAPTURE                         = 0x08013364ull;

static const uint64_t P9A_OMI_1_REG0_DL1_ERROR_CAPTURE                        = 0x07013364ull;

static const uint64_t P9A_OMI_9_REG0_DL1_ERROR_CAPTURE                        = 0x08013364ull;


static const uint64_t P9A_MC_REG0_DL1_ERROR_HOLD                              = 0x07013363ull;

static const uint64_t P9A_MC_0_REG0_DL1_ERROR_HOLD                            = 0x07013363ull;

static const uint64_t P9A_MC_1_REG0_DL1_ERROR_HOLD                            = 0x08013363ull;

static const uint64_t P9A_OMI_1_REG0_DL1_ERROR_HOLD                           = 0x07013363ull;

static const uint64_t P9A_OMI_9_REG0_DL1_ERROR_HOLD                           = 0x08013363ull;


static const uint64_t P9A_MC_REG0_DL1_ERROR_MASK                              = 0x07013362ull;

static const uint64_t P9A_MC_0_REG0_DL1_ERROR_MASK                            = 0x07013362ull;

static const uint64_t P9A_MC_1_REG0_DL1_ERROR_MASK                            = 0x08013362ull;

static const uint64_t P9A_OMI_1_REG0_DL1_ERROR_MASK                           = 0x07013362ull;

static const uint64_t P9A_OMI_9_REG0_DL1_ERROR_MASK                           = 0x08013362ull;


static const uint64_t P9A_MC_REG0_DL1_STATUS                                  = 0x07013366ull;

static const uint64_t P9A_MC_0_REG0_DL1_STATUS                                = 0x07013366ull;

static const uint64_t P9A_MC_1_REG0_DL1_STATUS                                = 0x08013366ull;

static const uint64_t P9A_OMI_1_REG0_DL1_STATUS                               = 0x07013366ull;

static const uint64_t P9A_OMI_9_REG0_DL1_STATUS                               = 0x08013366ull;


static const uint64_t P9A_MC_REG0_DL1_TRAINING_STATUS                         = 0x07013367ull;

static const uint64_t P9A_MC_0_REG0_DL1_TRAINING_STATUS                       = 0x07013367ull;

static const uint64_t P9A_MC_1_REG0_DL1_TRAINING_STATUS                       = 0x08013367ull;

static const uint64_t P9A_OMI_1_REG0_DL1_TRAINING_STATUS                      = 0x07013367ull;

static const uint64_t P9A_OMI_9_REG0_DL1_TRAINING_STATUS                      = 0x08013367ull;


static const uint64_t P9A_MC_REG0_DL2_CONFIG0                                 = 0x07013370ull;

static const uint64_t P9A_MC_0_REG0_DL2_CONFIG0                               = 0x07013370ull;

static const uint64_t P9A_MC_1_REG0_DL2_CONFIG0                               = 0x08013370ull;

static const uint64_t P9A_OMI_2_REG0_DL2_CONFIG0                              = 0x07013370ull;

static const uint64_t P9A_OMI_10_REG0_DL2_CONFIG0                             = 0x08013370ull;


static const uint64_t P9A_MC_REG0_DL2_CONFIG1                                 = 0x07013371ull;

static const uint64_t P9A_MC_0_REG0_DL2_CONFIG1                               = 0x07013371ull;

static const uint64_t P9A_MC_1_REG0_DL2_CONFIG1                               = 0x08013371ull;

static const uint64_t P9A_OMI_2_REG0_DL2_CONFIG1                              = 0x07013371ull;

static const uint64_t P9A_OMI_10_REG0_DL2_CONFIG1                             = 0x08013371ull;


static const uint64_t P9A_MC_REG0_DL2_CYA_BITS                                = 0x0701337Full;

static const uint64_t P9A_MC_0_REG0_DL2_CYA_BITS                              = 0x0701337Full;

static const uint64_t P9A_MC_1_REG0_DL2_CYA_BITS                              = 0x0801337Full;

static const uint64_t P9A_OMI_2_REG0_DL2_CYA_BITS                             = 0x0701337Full;

static const uint64_t P9A_OMI_10_REG0_DL2_CYA_BITS                            = 0x0801337Full;


static const uint64_t P9A_MC_REG0_DL2_DLX_CONFIG                              = 0x07013378ull;

static const uint64_t P9A_MC_0_REG0_DL2_DLX_CONFIG                            = 0x07013378ull;

static const uint64_t P9A_MC_1_REG0_DL2_DLX_CONFIG                            = 0x08013378ull;

static const uint64_t P9A_OMI_2_REG0_DL2_DLX_CONFIG                           = 0x07013378ull;

static const uint64_t P9A_OMI_10_REG0_DL2_DLX_CONFIG                          = 0x08013378ull;


static const uint64_t P9A_MC_REG0_DL2_DLX_INFO                                = 0x07013379ull;

static const uint64_t P9A_MC_0_REG0_DL2_DLX_INFO                              = 0x07013379ull;

static const uint64_t P9A_MC_1_REG0_DL2_DLX_INFO                              = 0x08013379ull;

static const uint64_t P9A_OMI_2_REG0_DL2_DLX_INFO                             = 0x07013379ull;

static const uint64_t P9A_OMI_10_REG0_DL2_DLX_INFO                            = 0x08013379ull;


static const uint64_t P9A_MC_REG0_DL2_EDPL_MAX_COUNT                          = 0x07013375ull;

static const uint64_t P9A_MC_0_REG0_DL2_EDPL_MAX_COUNT                        = 0x07013375ull;

static const uint64_t P9A_MC_1_REG0_DL2_EDPL_MAX_COUNT                        = 0x08013375ull;

static const uint64_t P9A_OMI_2_REG0_DL2_EDPL_MAX_COUNT                       = 0x07013375ull;

static const uint64_t P9A_OMI_10_REG0_DL2_EDPL_MAX_COUNT                      = 0x08013375ull;


static const uint64_t P9A_MC_REG0_DL2_ERROR_CAPTURE                           = 0x07013374ull;

static const uint64_t P9A_MC_0_REG0_DL2_ERROR_CAPTURE                         = 0x07013374ull;

static const uint64_t P9A_MC_1_REG0_DL2_ERROR_CAPTURE                         = 0x08013374ull;

static const uint64_t P9A_OMI_2_REG0_DL2_ERROR_CAPTURE                        = 0x07013374ull;

static const uint64_t P9A_OMI_10_REG0_DL2_ERROR_CAPTURE                       = 0x08013374ull;


static const uint64_t P9A_MC_REG0_DL2_ERROR_HOLD                              = 0x07013373ull;

static const uint64_t P9A_MC_0_REG0_DL2_ERROR_HOLD                            = 0x07013373ull;

static const uint64_t P9A_MC_1_REG0_DL2_ERROR_HOLD                            = 0x08013373ull;

static const uint64_t P9A_OMI_2_REG0_DL2_ERROR_HOLD                           = 0x07013373ull;

static const uint64_t P9A_OMI_10_REG0_DL2_ERROR_HOLD                          = 0x08013373ull;


static const uint64_t P9A_MC_REG0_DL2_ERROR_MASK                              = 0x07013372ull;

static const uint64_t P9A_MC_0_REG0_DL2_ERROR_MASK                            = 0x07013372ull;

static const uint64_t P9A_MC_1_REG0_DL2_ERROR_MASK                            = 0x08013372ull;

static const uint64_t P9A_OMI_2_REG0_DL2_ERROR_MASK                           = 0x07013372ull;

static const uint64_t P9A_OMI_10_REG0_DL2_ERROR_MASK                          = 0x08013372ull;


static const uint64_t P9A_MC_REG0_DL2_STATUS                                  = 0x07013376ull;

static const uint64_t P9A_MC_0_REG0_DL2_STATUS                                = 0x07013376ull;

static const uint64_t P9A_MC_1_REG0_DL2_STATUS                                = 0x08013376ull;

static const uint64_t P9A_OMI_2_REG0_DL2_STATUS                               = 0x07013376ull;

static const uint64_t P9A_OMI_10_REG0_DL2_STATUS                              = 0x08013376ull;


static const uint64_t P9A_MC_REG0_DL2_TRAINING_STATUS                         = 0x07013377ull;

static const uint64_t P9A_MC_0_REG0_DL2_TRAINING_STATUS                       = 0x07013377ull;

static const uint64_t P9A_MC_1_REG0_DL2_TRAINING_STATUS                       = 0x08013377ull;

static const uint64_t P9A_OMI_2_REG0_DL2_TRAINING_STATUS                      = 0x07013377ull;

static const uint64_t P9A_OMI_10_REG0_DL2_TRAINING_STATUS                     = 0x08013377ull;


static const uint64_t P9A_MC_REG1_DL0_CONFIG0                                 = 0x07013390ull;

static const uint64_t P9A_MC_0_REG1_DL0_CONFIG0                               = 0x07013390ull;

static const uint64_t P9A_MC_1_REG1_DL0_CONFIG0                               = 0x08013390ull;

static const uint64_t P9A_OMI_3_REG1_DL0_CONFIG0                              = 0x07013390ull;

static const uint64_t P9A_OMI_11_REG1_DL0_CONFIG0                             = 0x08013390ull;


static const uint64_t P9A_MC_REG1_DL0_CONFIG1                                 = 0x07013391ull;

static const uint64_t P9A_MC_0_REG1_DL0_CONFIG1                               = 0x07013391ull;

static const uint64_t P9A_MC_1_REG1_DL0_CONFIG1                               = 0x08013391ull;

static const uint64_t P9A_OMI_3_REG1_DL0_CONFIG1                              = 0x07013391ull;

static const uint64_t P9A_OMI_11_REG1_DL0_CONFIG1                             = 0x08013391ull;


static const uint64_t P9A_MC_REG1_DL0_CYA_BITS                                = 0x0701339Full;

static const uint64_t P9A_MC_0_REG1_DL0_CYA_BITS                              = 0x0701339Full;

static const uint64_t P9A_MC_1_REG1_DL0_CYA_BITS                              = 0x0801339Full;

static const uint64_t P9A_OMI_3_REG1_DL0_CYA_BITS                             = 0x0701339Full;

static const uint64_t P9A_OMI_11_REG1_DL0_CYA_BITS                            = 0x0801339Full;


static const uint64_t P9A_MC_REG1_DL0_DLX_CONFIG                              = 0x07013398ull;

static const uint64_t P9A_MC_0_REG1_DL0_DLX_CONFIG                            = 0x07013398ull;

static const uint64_t P9A_MC_1_REG1_DL0_DLX_CONFIG                            = 0x08013398ull;

static const uint64_t P9A_OMI_3_REG1_DL0_DLX_CONFIG                           = 0x07013398ull;

static const uint64_t P9A_OMI_11_REG1_DL0_DLX_CONFIG                          = 0x08013398ull;


static const uint64_t P9A_MC_REG1_DL0_DLX_INFO                                = 0x07013399ull;

static const uint64_t P9A_MC_0_REG1_DL0_DLX_INFO                              = 0x07013399ull;

static const uint64_t P9A_MC_1_REG1_DL0_DLX_INFO                              = 0x08013399ull;

static const uint64_t P9A_OMI_3_REG1_DL0_DLX_INFO                             = 0x07013399ull;

static const uint64_t P9A_OMI_11_REG1_DL0_DLX_INFO                            = 0x08013399ull;


static const uint64_t P9A_MC_REG1_DL0_EDPL_MAX_COUNT                          = 0x07013395ull;

static const uint64_t P9A_MC_0_REG1_DL0_EDPL_MAX_COUNT                        = 0x07013395ull;

static const uint64_t P9A_MC_1_REG1_DL0_EDPL_MAX_COUNT                        = 0x08013395ull;

static const uint64_t P9A_OMI_3_REG1_DL0_EDPL_MAX_COUNT                       = 0x07013395ull;

static const uint64_t P9A_OMI_11_REG1_DL0_EDPL_MAX_COUNT                      = 0x08013395ull;


static const uint64_t P9A_MC_REG1_DL0_ERROR_CAPTURE                           = 0x07013394ull;

static const uint64_t P9A_MC_0_REG1_DL0_ERROR_CAPTURE                         = 0x07013394ull;

static const uint64_t P9A_MC_1_REG1_DL0_ERROR_CAPTURE                         = 0x08013394ull;

static const uint64_t P9A_OMI_3_REG1_DL0_ERROR_CAPTURE                        = 0x07013394ull;

static const uint64_t P9A_OMI_11_REG1_DL0_ERROR_CAPTURE                       = 0x08013394ull;


static const uint64_t P9A_MC_REG1_DL0_ERROR_HOLD                              = 0x07013393ull;

static const uint64_t P9A_MC_0_REG1_DL0_ERROR_HOLD                            = 0x07013393ull;

static const uint64_t P9A_MC_1_REG1_DL0_ERROR_HOLD                            = 0x08013393ull;

static const uint64_t P9A_OMI_3_REG1_DL0_ERROR_HOLD                           = 0x07013393ull;

static const uint64_t P9A_OMI_11_REG1_DL0_ERROR_HOLD                          = 0x08013393ull;


static const uint64_t P9A_MC_REG1_DL0_ERROR_MASK                              = 0x07013392ull;

static const uint64_t P9A_MC_0_REG1_DL0_ERROR_MASK                            = 0x07013392ull;

static const uint64_t P9A_MC_1_REG1_DL0_ERROR_MASK                            = 0x08013392ull;

static const uint64_t P9A_OMI_3_REG1_DL0_ERROR_MASK                           = 0x07013392ull;

static const uint64_t P9A_OMI_11_REG1_DL0_ERROR_MASK                          = 0x08013392ull;


static const uint64_t P9A_MC_REG1_DL0_STATUS                                  = 0x07013396ull;

static const uint64_t P9A_MC_0_REG1_DL0_STATUS                                = 0x07013396ull;

static const uint64_t P9A_MC_1_REG1_DL0_STATUS                                = 0x08013396ull;

static const uint64_t P9A_OMI_3_REG1_DL0_STATUS                               = 0x07013396ull;

static const uint64_t P9A_OMI_11_REG1_DL0_STATUS                              = 0x08013396ull;


static const uint64_t P9A_MC_REG1_DL0_TRAINING_STATUS                         = 0x07013397ull;

static const uint64_t P9A_MC_0_REG1_DL0_TRAINING_STATUS                       = 0x07013397ull;

static const uint64_t P9A_MC_1_REG1_DL0_TRAINING_STATUS                       = 0x08013397ull;

static const uint64_t P9A_OMI_3_REG1_DL0_TRAINING_STATUS                      = 0x07013397ull;

static const uint64_t P9A_OMI_11_REG1_DL0_TRAINING_STATUS                     = 0x08013397ull;


static const uint64_t P9A_MC_REG1_DL1_CONFIG0                                 = 0x070133A0ull;

static const uint64_t P9A_MC_0_REG1_DL1_CONFIG0                               = 0x070133A0ull;

static const uint64_t P9A_MC_1_REG1_DL1_CONFIG0                               = 0x080133A0ull;

static const uint64_t P9A_OMI_4_REG1_DL1_CONFIG0                              = 0x070133A0ull;

static const uint64_t P9A_OMI_12_REG1_DL1_CONFIG0                             = 0x080133A0ull;


static const uint64_t P9A_MC_REG1_DL1_CONFIG1                                 = 0x070133A1ull;

static const uint64_t P9A_MC_0_REG1_DL1_CONFIG1                               = 0x070133A1ull;

static const uint64_t P9A_MC_1_REG1_DL1_CONFIG1                               = 0x080133A1ull;

static const uint64_t P9A_OMI_4_REG1_DL1_CONFIG1                              = 0x070133A1ull;

static const uint64_t P9A_OMI_12_REG1_DL1_CONFIG1                             = 0x080133A1ull;


static const uint64_t P9A_MC_REG1_DL1_CYA_BITS                                = 0x070133AFull;

static const uint64_t P9A_MC_0_REG1_DL1_CYA_BITS                              = 0x070133AFull;

static const uint64_t P9A_MC_1_REG1_DL1_CYA_BITS                              = 0x080133AFull;

static const uint64_t P9A_OMI_4_REG1_DL1_CYA_BITS                             = 0x070133AFull;

static const uint64_t P9A_OMI_12_REG1_DL1_CYA_BITS                            = 0x080133AFull;


static const uint64_t P9A_MC_REG1_DL1_DLX_CONFIG                              = 0x070133A8ull;

static const uint64_t P9A_MC_0_REG1_DL1_DLX_CONFIG                            = 0x070133A8ull;

static const uint64_t P9A_MC_1_REG1_DL1_DLX_CONFIG                            = 0x080133A8ull;

static const uint64_t P9A_OMI_4_REG1_DL1_DLX_CONFIG                           = 0x070133A8ull;

static const uint64_t P9A_OMI_12_REG1_DL1_DLX_CONFIG                          = 0x080133A8ull;


static const uint64_t P9A_MC_REG1_DL1_DLX_INFO                                = 0x070133A9ull;

static const uint64_t P9A_MC_0_REG1_DL1_DLX_INFO                              = 0x070133A9ull;

static const uint64_t P9A_MC_1_REG1_DL1_DLX_INFO                              = 0x080133A9ull;

static const uint64_t P9A_OMI_4_REG1_DL1_DLX_INFO                             = 0x070133A9ull;

static const uint64_t P9A_OMI_12_REG1_DL1_DLX_INFO                            = 0x080133A9ull;


static const uint64_t P9A_MC_REG1_DL1_EDPL_MAX_COUNT                          = 0x070133A5ull;

static const uint64_t P9A_MC_0_REG1_DL1_EDPL_MAX_COUNT                        = 0x070133A5ull;

static const uint64_t P9A_MC_1_REG1_DL1_EDPL_MAX_COUNT                        = 0x080133A5ull;

static const uint64_t P9A_OMI_4_REG1_DL1_EDPL_MAX_COUNT                       = 0x070133A5ull;

static const uint64_t P9A_OMI_12_REG1_DL1_EDPL_MAX_COUNT                      = 0x080133A5ull;


static const uint64_t P9A_MC_REG1_DL1_ERROR_CAPTURE                           = 0x070133A4ull;

static const uint64_t P9A_MC_0_REG1_DL1_ERROR_CAPTURE                         = 0x070133A4ull;

static const uint64_t P9A_MC_1_REG1_DL1_ERROR_CAPTURE                         = 0x080133A4ull;

static const uint64_t P9A_OMI_4_REG1_DL1_ERROR_CAPTURE                        = 0x070133A4ull;

static const uint64_t P9A_OMI_12_REG1_DL1_ERROR_CAPTURE                       = 0x080133A4ull;


static const uint64_t P9A_MC_REG1_DL1_ERROR_HOLD                              = 0x070133A3ull;

static const uint64_t P9A_MC_0_REG1_DL1_ERROR_HOLD                            = 0x070133A3ull;

static const uint64_t P9A_MC_1_REG1_DL1_ERROR_HOLD                            = 0x080133A3ull;

static const uint64_t P9A_OMI_4_REG1_DL1_ERROR_HOLD                           = 0x070133A3ull;

static const uint64_t P9A_OMI_12_REG1_DL1_ERROR_HOLD                          = 0x080133A3ull;


static const uint64_t P9A_MC_REG1_DL1_ERROR_MASK                              = 0x070133A2ull;

static const uint64_t P9A_MC_0_REG1_DL1_ERROR_MASK                            = 0x070133A2ull;

static const uint64_t P9A_MC_1_REG1_DL1_ERROR_MASK                            = 0x080133A2ull;

static const uint64_t P9A_OMI_4_REG1_DL1_ERROR_MASK                           = 0x070133A2ull;

static const uint64_t P9A_OMI_12_REG1_DL1_ERROR_MASK                          = 0x080133A2ull;


static const uint64_t P9A_MC_REG1_DL1_STATUS                                  = 0x070133A6ull;

static const uint64_t P9A_MC_0_REG1_DL1_STATUS                                = 0x070133A6ull;

static const uint64_t P9A_MC_1_REG1_DL1_STATUS                                = 0x080133A6ull;

static const uint64_t P9A_OMI_4_REG1_DL1_STATUS                               = 0x070133A6ull;

static const uint64_t P9A_OMI_12_REG1_DL1_STATUS                              = 0x080133A6ull;


static const uint64_t P9A_MC_REG1_DL1_TRAINING_STATUS                         = 0x070133A7ull;

static const uint64_t P9A_MC_0_REG1_DL1_TRAINING_STATUS                       = 0x070133A7ull;

static const uint64_t P9A_MC_1_REG1_DL1_TRAINING_STATUS                       = 0x080133A7ull;

static const uint64_t P9A_OMI_4_REG1_DL1_TRAINING_STATUS                      = 0x070133A7ull;

static const uint64_t P9A_OMI_12_REG1_DL1_TRAINING_STATUS                     = 0x080133A7ull;


static const uint64_t P9A_MC_REG1_DL2_CONFIG0                                 = 0x070133B0ull;

static const uint64_t P9A_MC_0_REG1_DL2_CONFIG0                               = 0x070133B0ull;

static const uint64_t P9A_MC_1_REG1_DL2_CONFIG0                               = 0x080133B0ull;

static const uint64_t P9A_OMI_5_REG1_DL2_CONFIG0                              = 0x070133B0ull;

static const uint64_t P9A_OMI_13_REG1_DL2_CONFIG0                             = 0x080133B0ull;


static const uint64_t P9A_MC_REG1_DL2_CONFIG1                                 = 0x070133B1ull;

static const uint64_t P9A_MC_0_REG1_DL2_CONFIG1                               = 0x070133B1ull;

static const uint64_t P9A_MC_1_REG1_DL2_CONFIG1                               = 0x080133B1ull;

static const uint64_t P9A_OMI_5_REG1_DL2_CONFIG1                              = 0x070133B1ull;

static const uint64_t P9A_OMI_13_REG1_DL2_CONFIG1                             = 0x080133B1ull;


static const uint64_t P9A_MC_REG1_DL2_CYA_BITS                                = 0x070133BFull;

static const uint64_t P9A_MC_0_REG1_DL2_CYA_BITS                              = 0x070133BFull;

static const uint64_t P9A_MC_1_REG1_DL2_CYA_BITS                              = 0x080133BFull;

static const uint64_t P9A_OMI_5_REG1_DL2_CYA_BITS                             = 0x070133BFull;

static const uint64_t P9A_OMI_13_REG1_DL2_CYA_BITS                            = 0x080133BFull;


static const uint64_t P9A_MC_REG1_DL2_DLX_CONFIG                              = 0x070133B8ull;

static const uint64_t P9A_MC_0_REG1_DL2_DLX_CONFIG                            = 0x070133B8ull;

static const uint64_t P9A_MC_1_REG1_DL2_DLX_CONFIG                            = 0x080133B8ull;

static const uint64_t P9A_OMI_5_REG1_DL2_DLX_CONFIG                           = 0x070133B8ull;

static const uint64_t P9A_OMI_13_REG1_DL2_DLX_CONFIG                          = 0x080133B8ull;


static const uint64_t P9A_MC_REG1_DL2_DLX_INFO                                = 0x070133B9ull;

static const uint64_t P9A_MC_0_REG1_DL2_DLX_INFO                              = 0x070133B9ull;

static const uint64_t P9A_MC_1_REG1_DL2_DLX_INFO                              = 0x080133B9ull;

static const uint64_t P9A_OMI_5_REG1_DL2_DLX_INFO                             = 0x070133B9ull;

static const uint64_t P9A_OMI_13_REG1_DL2_DLX_INFO                            = 0x080133B9ull;


static const uint64_t P9A_MC_REG1_DL2_EDPL_MAX_COUNT                          = 0x070133B5ull;

static const uint64_t P9A_MC_0_REG1_DL2_EDPL_MAX_COUNT                        = 0x070133B5ull;

static const uint64_t P9A_MC_1_REG1_DL2_EDPL_MAX_COUNT                        = 0x080133B5ull;

static const uint64_t P9A_OMI_5_REG1_DL2_EDPL_MAX_COUNT                       = 0x070133B5ull;

static const uint64_t P9A_OMI_13_REG1_DL2_EDPL_MAX_COUNT                      = 0x080133B5ull;


static const uint64_t P9A_MC_REG1_DL2_ERROR_CAPTURE                           = 0x070133B4ull;

static const uint64_t P9A_MC_0_REG1_DL2_ERROR_CAPTURE                         = 0x070133B4ull;

static const uint64_t P9A_MC_1_REG1_DL2_ERROR_CAPTURE                         = 0x080133B4ull;

static const uint64_t P9A_OMI_5_REG1_DL2_ERROR_CAPTURE                        = 0x070133B4ull;

static const uint64_t P9A_OMI_13_REG1_DL2_ERROR_CAPTURE                       = 0x080133B4ull;


static const uint64_t P9A_MC_REG1_DL2_ERROR_HOLD                              = 0x070133B3ull;

static const uint64_t P9A_MC_0_REG1_DL2_ERROR_HOLD                            = 0x070133B3ull;

static const uint64_t P9A_MC_1_REG1_DL2_ERROR_HOLD                            = 0x080133B3ull;

static const uint64_t P9A_OMI_5_REG1_DL2_ERROR_HOLD                           = 0x070133B3ull;

static const uint64_t P9A_OMI_13_REG1_DL2_ERROR_HOLD                          = 0x080133B3ull;


static const uint64_t P9A_MC_REG1_DL2_ERROR_MASK                              = 0x070133B2ull;

static const uint64_t P9A_MC_0_REG1_DL2_ERROR_MASK                            = 0x070133B2ull;

static const uint64_t P9A_MC_1_REG1_DL2_ERROR_MASK                            = 0x080133B2ull;

static const uint64_t P9A_OMI_5_REG1_DL2_ERROR_MASK                           = 0x070133B2ull;

static const uint64_t P9A_OMI_13_REG1_DL2_ERROR_MASK                          = 0x080133B2ull;


static const uint64_t P9A_MC_REG1_DL2_STATUS                                  = 0x070133B6ull;

static const uint64_t P9A_MC_0_REG1_DL2_STATUS                                = 0x070133B6ull;

static const uint64_t P9A_MC_1_REG1_DL2_STATUS                                = 0x080133B6ull;

static const uint64_t P9A_OMI_5_REG1_DL2_STATUS                               = 0x070133B6ull;

static const uint64_t P9A_OMI_13_REG1_DL2_STATUS                              = 0x080133B6ull;


static const uint64_t P9A_MC_REG1_DL2_TRAINING_STATUS                         = 0x070133B7ull;

static const uint64_t P9A_MC_0_REG1_DL2_TRAINING_STATUS                       = 0x070133B7ull;

static const uint64_t P9A_MC_1_REG1_DL2_TRAINING_STATUS                       = 0x080133B7ull;

static const uint64_t P9A_OMI_5_REG1_DL2_TRAINING_STATUS                      = 0x070133B7ull;

static const uint64_t P9A_OMI_13_REG1_DL2_TRAINING_STATUS                     = 0x080133B7ull;


static const uint64_t P9A_MC_REG2_DL0_CONFIG0                                 = 0x070133D0ull;

static const uint64_t P9A_MC_0_REG2_DL0_CONFIG0                               = 0x070133D0ull;

static const uint64_t P9A_MC_1_REG2_DL0_CONFIG0                               = 0x080133D0ull;

static const uint64_t P9A_OMI_6_REG2_DL0_CONFIG0                              = 0x070133D0ull;

static const uint64_t P9A_OMI_14_REG2_DL0_CONFIG0                             = 0x080133D0ull;


static const uint64_t P9A_MC_REG2_DL0_CONFIG1                                 = 0x070133D1ull;

static const uint64_t P9A_MC_0_REG2_DL0_CONFIG1                               = 0x070133D1ull;

static const uint64_t P9A_MC_1_REG2_DL0_CONFIG1                               = 0x080133D1ull;

static const uint64_t P9A_OMI_6_REG2_DL0_CONFIG1                              = 0x070133D1ull;

static const uint64_t P9A_OMI_14_REG2_DL0_CONFIG1                             = 0x080133D1ull;


static const uint64_t P9A_MC_REG2_DL0_CYA_BITS                                = 0x070133DFull;

static const uint64_t P9A_MC_0_REG2_DL0_CYA_BITS                              = 0x070133DFull;

static const uint64_t P9A_MC_1_REG2_DL0_CYA_BITS                              = 0x080133DFull;

static const uint64_t P9A_OMI_6_REG2_DL0_CYA_BITS                             = 0x070133DFull;

static const uint64_t P9A_OMI_14_REG2_DL0_CYA_BITS                            = 0x080133DFull;


static const uint64_t P9A_MC_REG2_DL0_DLX_CONFIG                              = 0x070133D8ull;

static const uint64_t P9A_MC_0_REG2_DL0_DLX_CONFIG                            = 0x070133D8ull;

static const uint64_t P9A_MC_1_REG2_DL0_DLX_CONFIG                            = 0x080133D8ull;

static const uint64_t P9A_OMI_6_REG2_DL0_DLX_CONFIG                           = 0x070133D8ull;

static const uint64_t P9A_OMI_14_REG2_DL0_DLX_CONFIG                          = 0x080133D8ull;


static const uint64_t P9A_MC_REG2_DL0_DLX_INFO                                = 0x070133D9ull;

static const uint64_t P9A_MC_0_REG2_DL0_DLX_INFO                              = 0x070133D9ull;

static const uint64_t P9A_MC_1_REG2_DL0_DLX_INFO                              = 0x080133D9ull;

static const uint64_t P9A_OMI_6_REG2_DL0_DLX_INFO                             = 0x070133D9ull;

static const uint64_t P9A_OMI_14_REG2_DL0_DLX_INFO                            = 0x080133D9ull;


static const uint64_t P9A_MC_REG2_DL0_EDPL_MAX_COUNT                          = 0x070133D5ull;

static const uint64_t P9A_MC_0_REG2_DL0_EDPL_MAX_COUNT                        = 0x070133D5ull;

static const uint64_t P9A_MC_1_REG2_DL0_EDPL_MAX_COUNT                        = 0x080133D5ull;

static const uint64_t P9A_OMI_6_REG2_DL0_EDPL_MAX_COUNT                       = 0x070133D5ull;

static const uint64_t P9A_OMI_14_REG2_DL0_EDPL_MAX_COUNT                      = 0x080133D5ull;


static const uint64_t P9A_MC_REG2_DL0_ERROR_CAPTURE                           = 0x070133D4ull;

static const uint64_t P9A_MC_0_REG2_DL0_ERROR_CAPTURE                         = 0x070133D4ull;

static const uint64_t P9A_MC_1_REG2_DL0_ERROR_CAPTURE                         = 0x080133D4ull;

static const uint64_t P9A_OMI_6_REG2_DL0_ERROR_CAPTURE                        = 0x070133D4ull;

static const uint64_t P9A_OMI_14_REG2_DL0_ERROR_CAPTURE                       = 0x080133D4ull;


static const uint64_t P9A_MC_REG2_DL0_ERROR_HOLD                              = 0x070133D3ull;

static const uint64_t P9A_MC_0_REG2_DL0_ERROR_HOLD                            = 0x070133D3ull;

static const uint64_t P9A_MC_1_REG2_DL0_ERROR_HOLD                            = 0x080133D3ull;

static const uint64_t P9A_OMI_6_REG2_DL0_ERROR_HOLD                           = 0x070133D3ull;

static const uint64_t P9A_OMI_14_REG2_DL0_ERROR_HOLD                          = 0x080133D3ull;


static const uint64_t P9A_MC_REG2_DL0_ERROR_MASK                              = 0x070133D2ull;

static const uint64_t P9A_MC_0_REG2_DL0_ERROR_MASK                            = 0x070133D2ull;

static const uint64_t P9A_MC_1_REG2_DL0_ERROR_MASK                            = 0x080133D2ull;

static const uint64_t P9A_OMI_6_REG2_DL0_ERROR_MASK                           = 0x070133D2ull;

static const uint64_t P9A_OMI_14_REG2_DL0_ERROR_MASK                          = 0x080133D2ull;


static const uint64_t P9A_MC_REG2_DL0_STATUS                                  = 0x070133D6ull;

static const uint64_t P9A_MC_0_REG2_DL0_STATUS                                = 0x070133D6ull;

static const uint64_t P9A_MC_1_REG2_DL0_STATUS                                = 0x080133D6ull;

static const uint64_t P9A_OMI_6_REG2_DL0_STATUS                               = 0x070133D6ull;

static const uint64_t P9A_OMI_14_REG2_DL0_STATUS                              = 0x080133D6ull;


static const uint64_t P9A_MC_REG2_DL0_TRAINING_STATUS                         = 0x070133D7ull;

static const uint64_t P9A_MC_0_REG2_DL0_TRAINING_STATUS                       = 0x070133D7ull;

static const uint64_t P9A_MC_1_REG2_DL0_TRAINING_STATUS                       = 0x080133D7ull;

static const uint64_t P9A_OMI_6_REG2_DL0_TRAINING_STATUS                      = 0x070133D7ull;

static const uint64_t P9A_OMI_14_REG2_DL0_TRAINING_STATUS                     = 0x080133D7ull;


static const uint64_t P9A_MC_REG2_DL1_CONFIG0                                 = 0x070133E0ull;

static const uint64_t P9A_MC_0_REG2_DL1_CONFIG0                               = 0x070133E0ull;

static const uint64_t P9A_MC_1_REG2_DL1_CONFIG0                               = 0x080133E0ull;

static const uint64_t P9A_OMI_7_REG2_DL1_CONFIG0                              = 0x070133E0ull;

static const uint64_t P9A_OMI_15_REG2_DL1_CONFIG0                             = 0x080133E0ull;


static const uint64_t P9A_MC_REG2_DL1_CONFIG1                                 = 0x070133E1ull;

static const uint64_t P9A_MC_0_REG2_DL1_CONFIG1                               = 0x070133E1ull;

static const uint64_t P9A_MC_1_REG2_DL1_CONFIG1                               = 0x080133E1ull;

static const uint64_t P9A_OMI_7_REG2_DL1_CONFIG1                              = 0x070133E1ull;

static const uint64_t P9A_OMI_15_REG2_DL1_CONFIG1                             = 0x080133E1ull;


static const uint64_t P9A_MC_REG2_DL1_CYA_BITS                                = 0x070133EFull;

static const uint64_t P9A_MC_0_REG2_DL1_CYA_BITS                              = 0x070133EFull;

static const uint64_t P9A_MC_1_REG2_DL1_CYA_BITS                              = 0x080133EFull;

static const uint64_t P9A_OMI_7_REG2_DL1_CYA_BITS                             = 0x070133EFull;

static const uint64_t P9A_OMI_15_REG2_DL1_CYA_BITS                            = 0x080133EFull;


static const uint64_t P9A_MC_REG2_DL1_DLX_CONFIG                              = 0x070133E8ull;

static const uint64_t P9A_MC_0_REG2_DL1_DLX_CONFIG                            = 0x070133E8ull;

static const uint64_t P9A_MC_1_REG2_DL1_DLX_CONFIG                            = 0x080133E8ull;

static const uint64_t P9A_OMI_7_REG2_DL1_DLX_CONFIG                           = 0x070133E8ull;

static const uint64_t P9A_OMI_15_REG2_DL1_DLX_CONFIG                          = 0x080133E8ull;


static const uint64_t P9A_MC_REG2_DL1_DLX_INFO                                = 0x070133E9ull;

static const uint64_t P9A_MC_0_REG2_DL1_DLX_INFO                              = 0x070133E9ull;

static const uint64_t P9A_MC_1_REG2_DL1_DLX_INFO                              = 0x080133E9ull;

static const uint64_t P9A_OMI_7_REG2_DL1_DLX_INFO                             = 0x070133E9ull;

static const uint64_t P9A_OMI_15_REG2_DL1_DLX_INFO                            = 0x080133E9ull;


static const uint64_t P9A_MC_REG2_DL1_EDPL_MAX_COUNT                          = 0x070133E5ull;

static const uint64_t P9A_MC_0_REG2_DL1_EDPL_MAX_COUNT                        = 0x070133E5ull;

static const uint64_t P9A_MC_1_REG2_DL1_EDPL_MAX_COUNT                        = 0x080133E5ull;

static const uint64_t P9A_OMI_7_REG2_DL1_EDPL_MAX_COUNT                       = 0x070133E5ull;

static const uint64_t P9A_OMI_15_REG2_DL1_EDPL_MAX_COUNT                      = 0x080133E5ull;


static const uint64_t P9A_MC_REG2_DL1_ERROR_CAPTURE                           = 0x070133E4ull;

static const uint64_t P9A_MC_0_REG2_DL1_ERROR_CAPTURE                         = 0x070133E4ull;

static const uint64_t P9A_MC_1_REG2_DL1_ERROR_CAPTURE                         = 0x080133E4ull;

static const uint64_t P9A_OMI_7_REG2_DL1_ERROR_CAPTURE                        = 0x070133E4ull;

static const uint64_t P9A_OMI_15_REG2_DL1_ERROR_CAPTURE                       = 0x080133E4ull;


static const uint64_t P9A_MC_REG2_DL1_ERROR_HOLD                              = 0x070133E3ull;

static const uint64_t P9A_MC_0_REG2_DL1_ERROR_HOLD                            = 0x070133E3ull;

static const uint64_t P9A_MC_1_REG2_DL1_ERROR_HOLD                            = 0x080133E3ull;

static const uint64_t P9A_OMI_7_REG2_DL1_ERROR_HOLD                           = 0x070133E3ull;

static const uint64_t P9A_OMI_15_REG2_DL1_ERROR_HOLD                          = 0x080133E3ull;


static const uint64_t P9A_MC_REG2_DL1_ERROR_MASK                              = 0x070133E2ull;

static const uint64_t P9A_MC_0_REG2_DL1_ERROR_MASK                            = 0x070133E2ull;

static const uint64_t P9A_MC_1_REG2_DL1_ERROR_MASK                            = 0x080133E2ull;

static const uint64_t P9A_OMI_7_REG2_DL1_ERROR_MASK                           = 0x070133E2ull;

static const uint64_t P9A_OMI_15_REG2_DL1_ERROR_MASK                          = 0x080133E2ull;


static const uint64_t P9A_MC_REG2_DL1_STATUS                                  = 0x070133E6ull;

static const uint64_t P9A_MC_0_REG2_DL1_STATUS                                = 0x070133E6ull;

static const uint64_t P9A_MC_1_REG2_DL1_STATUS                                = 0x080133E6ull;

static const uint64_t P9A_OMI_7_REG2_DL1_STATUS                               = 0x070133E6ull;

static const uint64_t P9A_OMI_15_REG2_DL1_STATUS                              = 0x080133E6ull;


static const uint64_t P9A_MC_REG2_DL1_TRAINING_STATUS                         = 0x070133E7ull;

static const uint64_t P9A_MC_0_REG2_DL1_TRAINING_STATUS                       = 0x070133E7ull;

static const uint64_t P9A_MC_1_REG2_DL1_TRAINING_STATUS                       = 0x080133E7ull;

static const uint64_t P9A_OMI_7_REG2_DL1_TRAINING_STATUS                      = 0x070133E7ull;

static const uint64_t P9A_OMI_15_REG2_DL1_TRAINING_STATUS                     = 0x080133E7ull;


static const uint64_t P9A_MC_REG2_DL2_CONFIG0                                 = 0x070133F0ull;

static const uint64_t P9A_MC_0_REG2_DL2_CONFIG0                               = 0x070133F0ull;

static const uint64_t P9A_MC_1_REG2_DL2_CONFIG0                               = 0x080133F0ull;

static const uint64_t P9A_OMI_8_REG2_DL2_CONFIG0                              = 0x070133F0ull;

static const uint64_t P9A_OMI_16_REG2_DL2_CONFIG0                             = 0x080133F0ull;


static const uint64_t P9A_MC_REG2_DL2_CONFIG1                                 = 0x070133F1ull;

static const uint64_t P9A_MC_0_REG2_DL2_CONFIG1                               = 0x070133F1ull;

static const uint64_t P9A_MC_1_REG2_DL2_CONFIG1                               = 0x080133F1ull;

static const uint64_t P9A_OMI_8_REG2_DL2_CONFIG1                              = 0x070133F1ull;

static const uint64_t P9A_OMI_16_REG2_DL2_CONFIG1                             = 0x080133F1ull;


static const uint64_t P9A_MC_REG2_DL2_CYA_BITS                                = 0x070133FFull;

static const uint64_t P9A_MC_0_REG2_DL2_CYA_BITS                              = 0x070133FFull;

static const uint64_t P9A_MC_1_REG2_DL2_CYA_BITS                              = 0x080133FFull;

static const uint64_t P9A_OMI_8_REG2_DL2_CYA_BITS                             = 0x070133FFull;

static const uint64_t P9A_OMI_16_REG2_DL2_CYA_BITS                            = 0x080133FFull;


static const uint64_t P9A_MC_REG2_DL2_DLX_CONFIG                              = 0x070133F8ull;

static const uint64_t P9A_MC_0_REG2_DL2_DLX_CONFIG                            = 0x070133F8ull;

static const uint64_t P9A_MC_1_REG2_DL2_DLX_CONFIG                            = 0x080133F8ull;

static const uint64_t P9A_OMI_8_REG2_DL2_DLX_CONFIG                           = 0x070133F8ull;

static const uint64_t P9A_OMI_16_REG2_DL2_DLX_CONFIG                          = 0x080133F8ull;


static const uint64_t P9A_MC_REG2_DL2_DLX_INFO                                = 0x070133F9ull;

static const uint64_t P9A_MC_0_REG2_DL2_DLX_INFO                              = 0x070133F9ull;

static const uint64_t P9A_MC_1_REG2_DL2_DLX_INFO                              = 0x080133F9ull;

static const uint64_t P9A_OMI_8_REG2_DL2_DLX_INFO                             = 0x070133F9ull;

static const uint64_t P9A_OMI_16_REG2_DL2_DLX_INFO                            = 0x080133F9ull;


static const uint64_t P9A_MC_REG2_DL2_EDPL_MAX_COUNT                          = 0x070133F5ull;

static const uint64_t P9A_MC_0_REG2_DL2_EDPL_MAX_COUNT                        = 0x070133F5ull;

static const uint64_t P9A_MC_1_REG2_DL2_EDPL_MAX_COUNT                        = 0x080133F5ull;

static const uint64_t P9A_OMI_8_REG2_DL2_EDPL_MAX_COUNT                       = 0x070133F5ull;

static const uint64_t P9A_OMI_16_REG2_DL2_EDPL_MAX_COUNT                      = 0x080133F5ull;


static const uint64_t P9A_MC_REG2_DL2_ERROR_CAPTURE                           = 0x070133F4ull;

static const uint64_t P9A_MC_0_REG2_DL2_ERROR_CAPTURE                         = 0x070133F4ull;

static const uint64_t P9A_MC_1_REG2_DL2_ERROR_CAPTURE                         = 0x080133F4ull;

static const uint64_t P9A_OMI_8_REG2_DL2_ERROR_CAPTURE                        = 0x070133F4ull;

static const uint64_t P9A_OMI_16_REG2_DL2_ERROR_CAPTURE                       = 0x080133F4ull;


static const uint64_t P9A_MC_REG2_DL2_ERROR_HOLD                              = 0x070133F3ull;

static const uint64_t P9A_MC_0_REG2_DL2_ERROR_HOLD                            = 0x070133F3ull;

static const uint64_t P9A_MC_1_REG2_DL2_ERROR_HOLD                            = 0x080133F3ull;

static const uint64_t P9A_OMI_8_REG2_DL2_ERROR_HOLD                           = 0x070133F3ull;

static const uint64_t P9A_OMI_16_REG2_DL2_ERROR_HOLD                          = 0x080133F3ull;


static const uint64_t P9A_MC_REG2_DL2_ERROR_MASK                              = 0x070133F2ull;

static const uint64_t P9A_MC_0_REG2_DL2_ERROR_MASK                            = 0x070133F2ull;

static const uint64_t P9A_MC_1_REG2_DL2_ERROR_MASK                            = 0x080133F2ull;

static const uint64_t P9A_OMI_8_REG2_DL2_ERROR_MASK                           = 0x070133F2ull;

static const uint64_t P9A_OMI_16_REG2_DL2_ERROR_MASK                          = 0x080133F2ull;


static const uint64_t P9A_MC_REG2_DL2_STATUS                                  = 0x070133F6ull;

static const uint64_t P9A_MC_0_REG2_DL2_STATUS                                = 0x070133F6ull;

static const uint64_t P9A_MC_1_REG2_DL2_STATUS                                = 0x080133F6ull;

static const uint64_t P9A_OMI_8_REG2_DL2_STATUS                               = 0x070133F6ull;

static const uint64_t P9A_OMI_16_REG2_DL2_STATUS                              = 0x080133F6ull;


static const uint64_t P9A_MC_REG2_DL2_TRAINING_STATUS                         = 0x070133F7ull;

static const uint64_t P9A_MC_0_REG2_DL2_TRAINING_STATUS                       = 0x070133F7ull;

static const uint64_t P9A_MC_1_REG2_DL2_TRAINING_STATUS                       = 0x080133F7ull;

static const uint64_t P9A_OMI_8_REG2_DL2_TRAINING_STATUS                      = 0x070133F7ull;

static const uint64_t P9A_OMI_16_REG2_DL2_TRAINING_STATUS                     = 0x080133F7ull;


static const uint64_t P9A_MC_RFIR                                             = 0x07040001ull;

static const uint64_t P9A_MC_0_RFIR                                           = 0x07040001ull;

static const uint64_t P9A_MC_1_RFIR                                           = 0x08040001ull;


static const uint64_t P9A_MC_RING_FENCE_MASK_LATCH_REG                        = 0x07010008ull;

static const uint64_t P9A_MC_0_RING_FENCE_MASK_LATCH_REG                      = 0x07010008ull;

static const uint64_t P9A_MC_1_RING_FENCE_MASK_LATCH_REG                      = 0x08010008ull;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL2_EO_PL           = 0x800248000701103Full;
//DUPS: 800248000701143F, 800248000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL2_EO_PL         = 0x800248000701103Full;
//DUPS: 800248000701143F, 800248000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL2_EO_PL         = 0x800248000801103Full;
//DUPS: 800248000801143F, 800248000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL2_EO_PL          = 0x800248000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL2_EO_PL         = 0x800248000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x800248000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x800248000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x800248000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x800248000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x800248000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x800248000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL2_EO_PL        = 0x800248000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL2_EO_PL        = 0x800248000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL2_EO_PL        = 0x800248000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL2_EO_PL        = 0x800248000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x800248000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x800248000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL3_EO_PL           = 0x800250000701103Full;
//DUPS: 800250000701143F, 800250000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL3_EO_PL         = 0x800250000701103Full;
//DUPS: 800250000701143F, 800250000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL3_EO_PL         = 0x800250000801103Full;
//DUPS: 800250000801143F, 800250000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL3_EO_PL          = 0x800250000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL3_EO_PL         = 0x800250000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x800250000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x800250000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x800250000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x800250000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x800250000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x800250000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL3_EO_PL        = 0x800250000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL3_EO_PL        = 0x800250000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL3_EO_PL        = 0x800250000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL3_EO_PL        = 0x800250000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x800250000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x800250000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL4_EO_PL           = 0x800258000701103Full;
//DUPS: 800258000701143F, 800258000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL4_EO_PL         = 0x800258000701103Full;
//DUPS: 800258000701143F, 800258000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL4_EO_PL         = 0x800258000801103Full;
//DUPS: 800258000801143F, 800258000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL4_EO_PL          = 0x800258000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL4_EO_PL         = 0x800258000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x800258000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x800258000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x800258000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x800258000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x800258000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x800258000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL4_EO_PL        = 0x800258000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL4_EO_PL        = 0x800258000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL4_EO_PL        = 0x800258000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL4_EO_PL        = 0x800258000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x800258000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x800258000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL5_EO_PL           = 0x800260000701103Full;
//DUPS: 800260000701143F, 800260000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL5_EO_PL         = 0x800260000701103Full;
//DUPS: 800260000701143F, 800260000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL5_EO_PL         = 0x800260000801103Full;
//DUPS: 800260000801143F, 800260000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL5_EO_PL          = 0x800260000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL5_EO_PL         = 0x800260000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x800260000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x800260000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x800260000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x800260000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x800260000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x800260000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL5_EO_PL        = 0x800260000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL5_EO_PL        = 0x800260000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL5_EO_PL        = 0x800260000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL5_EO_PL        = 0x800260000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x800260000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x800260000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_BIT_CNTLX1_EO_PL          = 0x800240000701103Full;
//DUPS: 800240000701143F, 800240000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_BIT_CNTLX1_EO_PL        = 0x800240000701103Full;
//DUPS: 800240000701143F, 800240000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_BIT_CNTLX1_EO_PL        = 0x800240000801103Full;
//DUPS: 800240000801143F, 800240000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_BIT_CNTLX1_EO_PL         = 0x800240000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_BIT_CNTLX1_EO_PL        = 0x800240000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x800240000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x800240000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x800240000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x800240000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x800240000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x800240000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_BIT_CNTLX1_EO_PL       = 0x800240000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_BIT_CNTLX1_EO_PL       = 0x800240000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_BIT_CNTLX1_EO_PL       = 0x800240000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_BIT_CNTLX1_EO_PL       = 0x800240000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x800240000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x800240000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_BIT_MODE1_EO_PL           = 0x800220000701103Full;
//DUPS: 800220000701143F, 800220000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_BIT_MODE1_EO_PL         = 0x800220000701103Full;
//DUPS: 800220000701143F, 800220000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_BIT_MODE1_EO_PL         = 0x800220000801103Full;
//DUPS: 800220000801143F, 800220000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_BIT_MODE1_EO_PL          = 0x800220000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_BIT_MODE1_EO_PL         = 0x800220000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_BIT_MODE1_EO_PL       = 0x800220000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_BIT_MODE1_EO_PL       = 0x800220000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_BIT_MODE1_EO_PL       = 0x800220000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_BIT_MODE1_EO_PL       = 0x800220000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_BIT_MODE1_EO_PL       = 0x800220000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_BIT_MODE1_EO_PL       = 0x800220000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_BIT_MODE1_EO_PL        = 0x800220000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_BIT_MODE1_EO_PL        = 0x800220000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_BIT_MODE1_EO_PL        = 0x800220000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_BIT_MODE1_EO_PL        = 0x800220000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_BIT_MODE1_EO_PL       = 0x800220000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_BIT_MODE1_EO_PL       = 0x800220000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_BIT_MODE2_EO_PL           = 0x800228000701103Full;
//DUPS: 800228000701143F, 800228000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_BIT_MODE2_EO_PL         = 0x800228000701103Full;
//DUPS: 800228000701143F, 800228000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_BIT_MODE2_EO_PL         = 0x800228000801103Full;
//DUPS: 800228000801143F, 800228000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_BIT_MODE2_EO_PL          = 0x800228000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_BIT_MODE2_EO_PL         = 0x800228000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_BIT_MODE2_EO_PL       = 0x800228000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_BIT_MODE2_EO_PL       = 0x800228000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_BIT_MODE2_EO_PL       = 0x800228000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_BIT_MODE2_EO_PL       = 0x800228000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_BIT_MODE2_EO_PL       = 0x800228000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_BIT_MODE2_EO_PL       = 0x800228000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_BIT_MODE2_EO_PL        = 0x800228000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_BIT_MODE2_EO_PL        = 0x800228000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_BIT_MODE2_EO_PL        = 0x800228000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_BIT_MODE2_EO_PL        = 0x800228000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_BIT_MODE2_EO_PL       = 0x800228000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_BIT_MODE2_EO_PL       = 0x800228000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_BIT_MODE3_EO_PL           = 0x800230000701103Full;
//DUPS: 800230000701143F, 800230000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_BIT_MODE3_EO_PL         = 0x800230000701103Full;
//DUPS: 800230000701143F, 800230000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_BIT_MODE3_EO_PL         = 0x800230000801103Full;
//DUPS: 800230000801143F, 800230000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_BIT_MODE3_EO_PL          = 0x800230000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_BIT_MODE3_EO_PL         = 0x800230000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_BIT_MODE3_EO_PL       = 0x800230000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_BIT_MODE3_EO_PL       = 0x800230000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_BIT_MODE3_EO_PL       = 0x800230000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_BIT_MODE3_EO_PL       = 0x800230000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_BIT_MODE3_EO_PL       = 0x800230000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_BIT_MODE3_EO_PL       = 0x800230000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_BIT_MODE3_EO_PL        = 0x800230000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_BIT_MODE3_EO_PL        = 0x800230000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_BIT_MODE3_EO_PL        = 0x800230000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_BIT_MODE3_EO_PL        = 0x800230000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_BIT_MODE3_EO_PL       = 0x800230000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_BIT_MODE3_EO_PL       = 0x800230000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_BIT_MODE4_EO_PL           = 0x800238000701103Full;
//DUPS: 800238000701143F, 800238000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_BIT_MODE4_EO_PL         = 0x800238000701103Full;
//DUPS: 800238000701143F, 800238000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_BIT_MODE4_EO_PL         = 0x800238000801103Full;
//DUPS: 800238000801143F, 800238000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_BIT_MODE4_EO_PL          = 0x800238000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_BIT_MODE4_EO_PL         = 0x800238000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_BIT_MODE4_EO_PL       = 0x800238000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_BIT_MODE4_EO_PL       = 0x800238000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_BIT_MODE4_EO_PL       = 0x800238000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_BIT_MODE4_EO_PL       = 0x800238000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_BIT_MODE4_EO_PL       = 0x800238000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_BIT_MODE4_EO_PL       = 0x800238000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_BIT_MODE4_EO_PL        = 0x800238000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_BIT_MODE4_EO_PL        = 0x800238000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_BIT_MODE4_EO_PL        = 0x800238000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_BIT_MODE4_EO_PL        = 0x800238000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_BIT_MODE4_EO_PL       = 0x800238000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_BIT_MODE4_EO_PL       = 0x800238000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_EO_PL           = 0x800268000701103Full;
//DUPS: 800268000701143F, 800268000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_EO_PL         = 0x800268000701103Full;
//DUPS: 800268000701143F, 800268000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_EO_PL         = 0x800268000801103Full;
//DUPS: 800268000801143F, 800268000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_EO_PL          = 0x800268000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_EO_PL         = 0x800268000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_EO_PL       = 0x800268000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_EO_PL       = 0x800268000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_EO_PL       = 0x800268000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_EO_PL       = 0x800268000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_EO_PL       = 0x800268000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_EO_PL       = 0x800268000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_EO_PL        = 0x800268000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_EO_PL        = 0x800268000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_EO_PL        = 0x800268000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_EO_PL        = 0x800268000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_EO_PL       = 0x800268000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_EO_PL       = 0x800268000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_O_PL            = 0x800318000701103Full;
//DUPS: 800318000701143F, 800318000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_O_PL          = 0x800318000701103Full;
//DUPS: 800318000701143F, 800318000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_O_PL          = 0x800318000801103Full;
//DUPS: 800318000801143F, 800318000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_O_PL           = 0x800318000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_O_PL          = 0x800318000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_O_PL        = 0x800318000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_O_PL        = 0x800318000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_O_PL        = 0x800318000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_O_PL        = 0x800318000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_O_PL        = 0x800318000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_O_PL        = 0x800318000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_O_PL         = 0x800318000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_O_PL         = 0x800318000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_O_PL         = 0x800318000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_O_PL         = 0x800318000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_O_PL        = 0x800318000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_BIT_STAT1_O_PL        = 0x800318000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_BIT_STAT2_EO_PL           = 0x800270000701103Full;
//DUPS: 800270000701143F, 800270000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_BIT_STAT2_EO_PL         = 0x800270000701103Full;
//DUPS: 800270000701143F, 800270000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_BIT_STAT2_EO_PL         = 0x800270000801103Full;
//DUPS: 800270000801143F, 800270000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_BIT_STAT2_EO_PL          = 0x800270000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_BIT_STAT2_EO_PL         = 0x800270000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_BIT_STAT2_EO_PL       = 0x800270000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_BIT_STAT2_EO_PL       = 0x800270000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_BIT_STAT2_EO_PL       = 0x800270000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_BIT_STAT2_EO_PL       = 0x800270000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_BIT_STAT2_EO_PL       = 0x800270000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_BIT_STAT2_EO_PL       = 0x800270000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_BIT_STAT2_EO_PL        = 0x800270000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_BIT_STAT2_EO_PL        = 0x800270000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_BIT_STAT2_EO_PL        = 0x800270000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_BIT_STAT2_EO_PL        = 0x800270000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_BIT_STAT2_EO_PL       = 0x800270000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_BIT_STAT2_EO_PL       = 0x800270000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_BIT_STAT3_EO_PL           = 0x800278000701103Full;
//DUPS: 800278000701143F, 800278000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_BIT_STAT3_EO_PL         = 0x800278000701103Full;
//DUPS: 800278000701143F, 800278000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_BIT_STAT3_EO_PL         = 0x800278000801103Full;
//DUPS: 800278000801143F, 800278000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_BIT_STAT3_EO_PL          = 0x800278000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_BIT_STAT3_EO_PL         = 0x800278000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_BIT_STAT3_EO_PL       = 0x800278000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_BIT_STAT3_EO_PL       = 0x800278000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_BIT_STAT3_EO_PL       = 0x800278000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_BIT_STAT3_EO_PL       = 0x800278000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_BIT_STAT3_EO_PL       = 0x800278000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_BIT_STAT3_EO_PL       = 0x800278000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_BIT_STAT3_EO_PL        = 0x800278000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_BIT_STAT3_EO_PL        = 0x800278000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_BIT_STAT3_EO_PL        = 0x800278000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_BIT_STAT3_EO_PL        = 0x800278000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_BIT_STAT3_EO_PL       = 0x800278000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_BIT_STAT3_EO_PL       = 0x800278000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_BIT_STAT4_EO_PL           = 0x800280000701103Full;
//DUPS: 800280000701143F, 800280000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_BIT_STAT4_EO_PL         = 0x800280000701103Full;
//DUPS: 800280000701143F, 800280000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_BIT_STAT4_EO_PL         = 0x800280000801103Full;
//DUPS: 800280000801143F, 800280000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_BIT_STAT4_EO_PL          = 0x800280000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_BIT_STAT4_EO_PL         = 0x800280000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_BIT_STAT4_EO_PL       = 0x800280000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_BIT_STAT4_EO_PL       = 0x800280000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_BIT_STAT4_EO_PL       = 0x800280000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_BIT_STAT4_EO_PL       = 0x800280000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_BIT_STAT4_EO_PL       = 0x800280000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_BIT_STAT4_EO_PL       = 0x800280000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_BIT_STAT4_EO_PL        = 0x800280000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_BIT_STAT4_EO_PL        = 0x800280000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_BIT_STAT4_EO_PL        = 0x800280000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_BIT_STAT4_EO_PL        = 0x800280000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_BIT_STAT4_EO_PL       = 0x800280000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_BIT_STAT4_EO_PL       = 0x800280000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL10_O_PL           = 0x8000C8000701103Full;
//DUPS: 8000C8000701143F, 8000C8000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL10_O_PL         = 0x8000C8000701103Full;
//DUPS: 8000C8000701143F, 8000C8000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL10_O_PL         = 0x8000C8000801103Full;
//DUPS: 8000C8000801143F, 8000C8000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL10_O_PL          = 0x8000C8000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL10_O_PL         = 0x8000C8000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C8000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C8000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C8000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C8000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C8000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C8000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL10_O_PL        = 0x8000C8000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL10_O_PL        = 0x8000C8000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL10_O_PL        = 0x8000C8000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL10_O_PL        = 0x8000C8000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C8000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C8000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_EO_PL           = 0x800008000701103Full;
//DUPS: 800008000701143F, 800008000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_EO_PL         = 0x800008000701103Full;
//DUPS: 800008000701143F, 800008000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_EO_PL         = 0x800008000801103Full;
//DUPS: 800008000801143F, 800008000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_EO_PL          = 0x800008000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_EO_PL         = 0x800008000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x800008000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x800008000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x800008000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x800008000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x800008000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x800008000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_EO_PL        = 0x800008000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_EO_PL        = 0x800008000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_EO_PL        = 0x800008000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_EO_PL        = 0x800008000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x800008000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x800008000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_O_PL            = 0x800080000701103Full;
//DUPS: 800080000701143F, 800080000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_O_PL          = 0x800080000701103Full;
//DUPS: 800080000701143F, 800080000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_O_PL          = 0x800080000801103Full;
//DUPS: 800080000801143F, 800080000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_O_PL           = 0x800080000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_O_PL          = 0x800080000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_O_PL        = 0x800080000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_O_PL        = 0x800080000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_O_PL        = 0x800080000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_O_PL        = 0x800080000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_O_PL        = 0x800080000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_O_PL        = 0x800080000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_O_PL         = 0x800080000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_O_PL         = 0x800080000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_O_PL         = 0x800080000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_O_PL         = 0x800080000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_O_PL        = 0x800080000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL1_O_PL        = 0x800080000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_EO_PL           = 0x800010000701103Full;
//DUPS: 800010000701143F, 800010000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_EO_PL         = 0x800010000701103Full;
//DUPS: 800010000701143F, 800010000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_EO_PL         = 0x800010000801103Full;
//DUPS: 800010000801143F, 800010000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_EO_PL          = 0x800010000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_EO_PL         = 0x800010000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x800010000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x800010000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x800010000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x800010000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x800010000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x800010000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_EO_PL        = 0x800010000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_EO_PL        = 0x800010000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_EO_PL        = 0x800010000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_EO_PL        = 0x800010000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x800010000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x800010000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_O_PL            = 0x800088000701103Full;
//DUPS: 800088000701143F, 800088000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_O_PL          = 0x800088000701103Full;
//DUPS: 800088000701143F, 800088000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_O_PL          = 0x800088000801103Full;
//DUPS: 800088000801143F, 800088000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_O_PL           = 0x800088000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_O_PL          = 0x800088000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_O_PL        = 0x800088000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_O_PL        = 0x800088000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_O_PL        = 0x800088000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_O_PL        = 0x800088000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_O_PL        = 0x800088000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_O_PL        = 0x800088000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_O_PL         = 0x800088000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_O_PL         = 0x800088000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_O_PL         = 0x800088000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_O_PL         = 0x800088000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_O_PL        = 0x800088000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL2_O_PL        = 0x800088000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_EO_PL           = 0x800018000701103Full;
//DUPS: 800018000701143F, 800018000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_EO_PL         = 0x800018000701103Full;
//DUPS: 800018000701143F, 800018000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_EO_PL         = 0x800018000801103Full;
//DUPS: 800018000801143F, 800018000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_EO_PL          = 0x800018000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_EO_PL         = 0x800018000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x800018000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x800018000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x800018000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x800018000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x800018000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x800018000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_EO_PL        = 0x800018000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_EO_PL        = 0x800018000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_EO_PL        = 0x800018000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_EO_PL        = 0x800018000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x800018000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x800018000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_O_PL            = 0x800090000701103Full;
//DUPS: 800090000701143F, 800090000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_O_PL          = 0x800090000701103Full;
//DUPS: 800090000701143F, 800090000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_O_PL          = 0x800090000801103Full;
//DUPS: 800090000801143F, 800090000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_O_PL           = 0x800090000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_O_PL          = 0x800090000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_O_PL        = 0x800090000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_O_PL        = 0x800090000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_O_PL        = 0x800090000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_O_PL        = 0x800090000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_O_PL        = 0x800090000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_O_PL        = 0x800090000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_O_PL         = 0x800090000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_O_PL         = 0x800090000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_O_PL         = 0x800090000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_O_PL         = 0x800090000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_O_PL        = 0x800090000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL3_O_PL        = 0x800090000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_EO_PL           = 0x800020000701103Full;
//DUPS: 800020000701143F, 800020000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_EO_PL         = 0x800020000701103Full;
//DUPS: 800020000701143F, 800020000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_EO_PL         = 0x800020000801103Full;
//DUPS: 800020000801143F, 800020000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_EO_PL          = 0x800020000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_EO_PL         = 0x800020000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x800020000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x800020000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x800020000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x800020000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x800020000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x800020000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_EO_PL        = 0x800020000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_EO_PL        = 0x800020000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_EO_PL        = 0x800020000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_EO_PL        = 0x800020000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x800020000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x800020000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_O_PL            = 0x800098000701103Full;
//DUPS: 800098000701143F, 800098000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_O_PL          = 0x800098000701103Full;
//DUPS: 800098000701143F, 800098000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_O_PL          = 0x800098000801103Full;
//DUPS: 800098000801143F, 800098000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_O_PL           = 0x800098000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_O_PL          = 0x800098000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_O_PL        = 0x800098000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_O_PL        = 0x800098000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_O_PL        = 0x800098000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_O_PL        = 0x800098000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_O_PL        = 0x800098000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_O_PL        = 0x800098000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_O_PL         = 0x800098000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_O_PL         = 0x800098000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_O_PL         = 0x800098000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_O_PL         = 0x800098000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_O_PL        = 0x800098000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL4_O_PL        = 0x800098000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_EO_PL           = 0x800028000701103Full;
//DUPS: 800028000701143F, 800028000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_EO_PL         = 0x800028000701103Full;
//DUPS: 800028000701143F, 800028000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_EO_PL         = 0x800028000801103Full;
//DUPS: 800028000801143F, 800028000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_EO_PL          = 0x800028000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_EO_PL         = 0x800028000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x800028000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x800028000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x800028000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x800028000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x800028000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x800028000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_EO_PL        = 0x800028000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_EO_PL        = 0x800028000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_EO_PL        = 0x800028000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_EO_PL        = 0x800028000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x800028000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x800028000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_O_PL            = 0x8000A0000701103Full;
//DUPS: 8000A0000701143F, 8000A0000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_O_PL          = 0x8000A0000701103Full;
//DUPS: 8000A0000701143F, 8000A0000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_O_PL          = 0x8000A0000801103Full;
//DUPS: 8000A0000801143F, 8000A0000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_O_PL           = 0x8000A0000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_O_PL          = 0x8000A0000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A0000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A0000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A0000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A0000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A0000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A0000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_O_PL         = 0x8000A0000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_O_PL         = 0x8000A0000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_O_PL         = 0x8000A0000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_O_PL         = 0x8000A0000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A0000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A0000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_EO_PL           = 0x800030000701103Full;
//DUPS: 800030000701143F, 800030000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_EO_PL         = 0x800030000701103Full;
//DUPS: 800030000701143F, 800030000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_EO_PL         = 0x800030000801103Full;
//DUPS: 800030000801143F, 800030000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_EO_PL          = 0x800030000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_EO_PL         = 0x800030000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x800030000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x800030000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x800030000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x800030000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x800030000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x800030000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_EO_PL        = 0x800030000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_EO_PL        = 0x800030000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_EO_PL        = 0x800030000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_EO_PL        = 0x800030000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x800030000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x800030000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_O_PL            = 0x8000A8000701103Full;
//DUPS: 8000A8000701143F, 8000A8000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_O_PL          = 0x8000A8000701103Full;
//DUPS: 8000A8000701143F, 8000A8000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_O_PL          = 0x8000A8000801103Full;
//DUPS: 8000A8000801143F, 8000A8000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_O_PL           = 0x8000A8000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_O_PL          = 0x8000A8000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A8000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A8000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A8000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A8000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A8000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A8000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_O_PL         = 0x8000A8000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_O_PL         = 0x8000A8000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_O_PL         = 0x8000A8000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_O_PL         = 0x8000A8000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A8000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A8000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_EO_PL           = 0x800038000701103Full;
//DUPS: 800038000701143F, 800038000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_EO_PL         = 0x800038000701103Full;
//DUPS: 800038000701143F, 800038000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_EO_PL         = 0x800038000801103Full;
//DUPS: 800038000801143F, 800038000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_EO_PL          = 0x800038000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_EO_PL         = 0x800038000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x800038000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x800038000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x800038000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x800038000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x800038000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x800038000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_EO_PL        = 0x800038000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_EO_PL        = 0x800038000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_EO_PL        = 0x800038000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_EO_PL        = 0x800038000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x800038000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x800038000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_O_PL            = 0x8000B0000701103Full;
//DUPS: 8000B0000701143F, 8000B0000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_O_PL          = 0x8000B0000701103Full;
//DUPS: 8000B0000701143F, 8000B0000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_O_PL          = 0x8000B0000801103Full;
//DUPS: 8000B0000801143F, 8000B0000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_O_PL           = 0x8000B0000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_O_PL          = 0x8000B0000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B0000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B0000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B0000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B0000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B0000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B0000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_O_PL         = 0x8000B0000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_O_PL         = 0x8000B0000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_O_PL         = 0x8000B0000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_O_PL         = 0x8000B0000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B0000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B0000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_EO_PL           = 0x800040000701103Full;
//DUPS: 800040000701143F, 800040000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_EO_PL         = 0x800040000701103Full;
//DUPS: 800040000701143F, 800040000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_EO_PL         = 0x800040000801103Full;
//DUPS: 800040000801143F, 800040000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_EO_PL          = 0x800040000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_EO_PL         = 0x800040000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x800040000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x800040000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x800040000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x800040000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x800040000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x800040000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_EO_PL        = 0x800040000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_EO_PL        = 0x800040000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_EO_PL        = 0x800040000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_EO_PL        = 0x800040000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x800040000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x800040000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_O_PL            = 0x8000B8000701103Full;
//DUPS: 8000B8000701143F, 8000B8000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_O_PL          = 0x8000B8000701103Full;
//DUPS: 8000B8000701143F, 8000B8000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_O_PL          = 0x8000B8000801103Full;
//DUPS: 8000B8000801143F, 8000B8000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_O_PL           = 0x8000B8000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_O_PL          = 0x8000B8000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B8000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B8000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B8000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B8000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B8000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B8000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_O_PL         = 0x8000B8000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_O_PL         = 0x8000B8000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_O_PL         = 0x8000B8000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_O_PL         = 0x8000B8000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B8000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B8000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL9_O_PL            = 0x8000C0000701103Full;
//DUPS: 8000C0000701143F, 8000C0000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL9_O_PL          = 0x8000C0000701103Full;
//DUPS: 8000C0000701143F, 8000C0000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL9_O_PL          = 0x8000C0000801103Full;
//DUPS: 8000C0000801143F, 8000C0000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL9_O_PL           = 0x8000C0000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL9_O_PL          = 0x8000C0000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C0000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C0000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C0000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C0000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C0000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C0000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL9_O_PL         = 0x8000C0000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL9_O_PL         = 0x8000C0000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL9_O_PL         = 0x8000C0000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL9_O_PL         = 0x8000C0000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C0000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C0000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_DATA_DAC_SPARE_MODE_PL    = 0x800000000701103Full;
//DUPS: 800000000701143F, 800000000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000000701103Full;
//DUPS: 800000000701143F, 800000000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000000801103Full;
//DUPS: 800000000801143F, 800000000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_DATA_DAC_SPARE_MODE_PL   = 0x800000000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_FIR_ERROR_INJECT_PL       = 0x800218000701103Full;
//DUPS: 800218000701143F, 800218000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_FIR_ERROR_INJECT_PL     = 0x800218000701103Full;
//DUPS: 800218000701143F, 800218000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_FIR_ERROR_INJECT_PL     = 0x800218000801103Full;
//DUPS: 800218000801143F, 800218000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_FIR_ERROR_INJECT_PL      = 0x800218000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_FIR_ERROR_INJECT_PL     = 0x800218000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x800218000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x800218000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x800218000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x800218000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x800218000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x800218000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_FIR_ERROR_INJECT_PL    = 0x800218000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_FIR_ERROR_INJECT_PL    = 0x800218000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_FIR_ERROR_INJECT_PL    = 0x800218000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_FIR_ERROR_INJECT_PL    = 0x800218000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x800218000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x800218000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_FIR_MASK_PL               = 0x800210000701103Full;
//DUPS: 800210000701143F, 800210000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_FIR_MASK_PL             = 0x800210000701103Full;
//DUPS: 800210000701143F, 800210000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_FIR_MASK_PL             = 0x800210000801103Full;
//DUPS: 800210000801143F, 800210000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_FIR_MASK_PL              = 0x800210000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_FIR_MASK_PL             = 0x800210000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_FIR_MASK_PL           = 0x800210000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_FIR_MASK_PL           = 0x800210000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_FIR_MASK_PL           = 0x800210000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_FIR_MASK_PL           = 0x800210000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_FIR_MASK_PL           = 0x800210000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_FIR_MASK_PL           = 0x800210000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_FIR_MASK_PL            = 0x800210000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_FIR_MASK_PL            = 0x800210000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_FIR_MASK_PL            = 0x800210000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_FIR_MASK_PL            = 0x800210000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_FIR_MASK_PL           = 0x800210000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_FIR_MASK_PL           = 0x800210000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_FIR_PL                    = 0x800208000701103Full;
//DUPS: 800208000701143F, 800208000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_FIR_PL                  = 0x800208000701103Full;
//DUPS: 800208000701143F, 800208000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_FIR_PL                  = 0x800208000801103Full;
//DUPS: 800208000801143F, 800208000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_FIR_PL                   = 0x800208000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_FIR_PL                  = 0x800208000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_FIR_PL                = 0x800208000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_FIR_PL                = 0x800208000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_FIR_PL                = 0x800208000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_FIR_PL                = 0x800208000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_FIR_PL                = 0x800208000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_FIR_PL                = 0x800208000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_FIR_PL                 = 0x800208000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_FIR_PL                 = 0x800208000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_FIR_PL                 = 0x800208000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_FIR_PL                 = 0x800208000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_FIR_PL                = 0x800208000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_FIR_PL                = 0x800208000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE0_RX_SPARE_MODE_PL             = 0x800200000701103Full;
//DUPS: 800200000701143F, 800200000701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE0_RX_SPARE_MODE_PL           = 0x800200000701103Full;
//DUPS: 800200000701143F, 800200000701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE0_RX_SPARE_MODE_PL           = 0x800200000801103Full;
//DUPS: 800200000801143F, 800200000801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE0_RX_SPARE_MODE_PL            = 0x800200000701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE0_RX_SPARE_MODE_PL           = 0x800200000701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE0_RX_SPARE_MODE_PL         = 0x800200000701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE0_RX_SPARE_MODE_PL         = 0x800200000701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE0_RX_SPARE_MODE_PL         = 0x800200000701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE0_RX_SPARE_MODE_PL         = 0x800200000801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE0_RX_SPARE_MODE_PL         = 0x800200000801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE0_RX_SPARE_MODE_PL         = 0x800200000801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE0_RX_SPARE_MODE_PL          = 0x800200000701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE0_RX_SPARE_MODE_PL          = 0x800200000701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE0_RX_SPARE_MODE_PL          = 0x800200000701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE0_RX_SPARE_MODE_PL          = 0x800200000801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE0_RX_SPARE_MODE_PL         = 0x800200000801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE0_RX_SPARE_MODE_PL         = 0x800200000801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL2_EO_PL           = 0x800248010701103Full;
//DUPS: 800248010701143F, 800248010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL2_EO_PL         = 0x800248010701103Full;
//DUPS: 800248010701143F, 800248010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL2_EO_PL         = 0x800248010801103Full;
//DUPS: 800248010801143F, 800248010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL2_EO_PL          = 0x800248010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL2_EO_PL         = 0x800248010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x800248010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x800248010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x800248010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x800248010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x800248010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x800248010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL2_EO_PL        = 0x800248010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL2_EO_PL        = 0x800248010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL2_EO_PL        = 0x800248010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL2_EO_PL        = 0x800248010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x800248010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x800248010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL3_EO_PL           = 0x800250010701103Full;
//DUPS: 800250010701143F, 800250010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL3_EO_PL         = 0x800250010701103Full;
//DUPS: 800250010701143F, 800250010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL3_EO_PL         = 0x800250010801103Full;
//DUPS: 800250010801143F, 800250010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL3_EO_PL          = 0x800250010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL3_EO_PL         = 0x800250010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x800250010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x800250010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x800250010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x800250010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x800250010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x800250010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL3_EO_PL        = 0x800250010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL3_EO_PL        = 0x800250010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL3_EO_PL        = 0x800250010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL3_EO_PL        = 0x800250010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x800250010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x800250010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL4_EO_PL           = 0x800258010701103Full;
//DUPS: 800258010701143F, 800258010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL4_EO_PL         = 0x800258010701103Full;
//DUPS: 800258010701143F, 800258010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL4_EO_PL         = 0x800258010801103Full;
//DUPS: 800258010801143F, 800258010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL4_EO_PL          = 0x800258010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL4_EO_PL         = 0x800258010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x800258010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x800258010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x800258010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x800258010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x800258010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x800258010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL4_EO_PL        = 0x800258010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL4_EO_PL        = 0x800258010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL4_EO_PL        = 0x800258010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL4_EO_PL        = 0x800258010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x800258010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x800258010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL5_EO_PL           = 0x800260010701103Full;
//DUPS: 800260010701143F, 800260010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL5_EO_PL         = 0x800260010701103Full;
//DUPS: 800260010701143F, 800260010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL5_EO_PL         = 0x800260010801103Full;
//DUPS: 800260010801143F, 800260010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL5_EO_PL          = 0x800260010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL5_EO_PL         = 0x800260010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x800260010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x800260010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x800260010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x800260010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x800260010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x800260010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL5_EO_PL        = 0x800260010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL5_EO_PL        = 0x800260010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL5_EO_PL        = 0x800260010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL5_EO_PL        = 0x800260010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x800260010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x800260010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_BIT_CNTLX1_EO_PL          = 0x800240010701103Full;
//DUPS: 800240010701143F, 800240010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_BIT_CNTLX1_EO_PL        = 0x800240010701103Full;
//DUPS: 800240010701143F, 800240010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_BIT_CNTLX1_EO_PL        = 0x800240010801103Full;
//DUPS: 800240010801143F, 800240010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_BIT_CNTLX1_EO_PL         = 0x800240010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_BIT_CNTLX1_EO_PL        = 0x800240010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x800240010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x800240010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x800240010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x800240010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x800240010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x800240010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_BIT_CNTLX1_EO_PL       = 0x800240010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_BIT_CNTLX1_EO_PL       = 0x800240010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_BIT_CNTLX1_EO_PL       = 0x800240010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_BIT_CNTLX1_EO_PL       = 0x800240010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x800240010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x800240010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_BIT_MODE1_EO_PL           = 0x800220010701103Full;
//DUPS: 800220010701143F, 800220010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_BIT_MODE1_EO_PL         = 0x800220010701103Full;
//DUPS: 800220010701143F, 800220010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_BIT_MODE1_EO_PL         = 0x800220010801103Full;
//DUPS: 800220010801143F, 800220010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_BIT_MODE1_EO_PL          = 0x800220010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_BIT_MODE1_EO_PL         = 0x800220010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_BIT_MODE1_EO_PL       = 0x800220010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_BIT_MODE1_EO_PL       = 0x800220010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_BIT_MODE1_EO_PL       = 0x800220010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_BIT_MODE1_EO_PL       = 0x800220010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_BIT_MODE1_EO_PL       = 0x800220010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_BIT_MODE1_EO_PL       = 0x800220010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_BIT_MODE1_EO_PL        = 0x800220010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_BIT_MODE1_EO_PL        = 0x800220010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_BIT_MODE1_EO_PL        = 0x800220010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_BIT_MODE1_EO_PL        = 0x800220010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_BIT_MODE1_EO_PL       = 0x800220010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_BIT_MODE1_EO_PL       = 0x800220010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_BIT_MODE2_EO_PL           = 0x800228010701103Full;
//DUPS: 800228010701143F, 800228010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_BIT_MODE2_EO_PL         = 0x800228010701103Full;
//DUPS: 800228010701143F, 800228010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_BIT_MODE2_EO_PL         = 0x800228010801103Full;
//DUPS: 800228010801143F, 800228010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_BIT_MODE2_EO_PL          = 0x800228010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_BIT_MODE2_EO_PL         = 0x800228010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_BIT_MODE2_EO_PL       = 0x800228010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_BIT_MODE2_EO_PL       = 0x800228010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_BIT_MODE2_EO_PL       = 0x800228010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_BIT_MODE2_EO_PL       = 0x800228010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_BIT_MODE2_EO_PL       = 0x800228010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_BIT_MODE2_EO_PL       = 0x800228010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_BIT_MODE2_EO_PL        = 0x800228010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_BIT_MODE2_EO_PL        = 0x800228010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_BIT_MODE2_EO_PL        = 0x800228010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_BIT_MODE2_EO_PL        = 0x800228010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_BIT_MODE2_EO_PL       = 0x800228010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_BIT_MODE2_EO_PL       = 0x800228010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_BIT_MODE3_EO_PL           = 0x800230010701103Full;
//DUPS: 800230010701143F, 800230010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_BIT_MODE3_EO_PL         = 0x800230010701103Full;
//DUPS: 800230010701143F, 800230010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_BIT_MODE3_EO_PL         = 0x800230010801103Full;
//DUPS: 800230010801143F, 800230010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_BIT_MODE3_EO_PL          = 0x800230010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_BIT_MODE3_EO_PL         = 0x800230010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_BIT_MODE3_EO_PL       = 0x800230010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_BIT_MODE3_EO_PL       = 0x800230010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_BIT_MODE3_EO_PL       = 0x800230010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_BIT_MODE3_EO_PL       = 0x800230010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_BIT_MODE3_EO_PL       = 0x800230010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_BIT_MODE3_EO_PL       = 0x800230010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_BIT_MODE3_EO_PL        = 0x800230010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_BIT_MODE3_EO_PL        = 0x800230010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_BIT_MODE3_EO_PL        = 0x800230010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_BIT_MODE3_EO_PL        = 0x800230010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_BIT_MODE3_EO_PL       = 0x800230010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_BIT_MODE3_EO_PL       = 0x800230010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_BIT_MODE4_EO_PL           = 0x800238010701103Full;
//DUPS: 800238010701143F, 800238010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_BIT_MODE4_EO_PL         = 0x800238010701103Full;
//DUPS: 800238010701143F, 800238010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_BIT_MODE4_EO_PL         = 0x800238010801103Full;
//DUPS: 800238010801143F, 800238010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_BIT_MODE4_EO_PL          = 0x800238010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_BIT_MODE4_EO_PL         = 0x800238010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_BIT_MODE4_EO_PL       = 0x800238010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_BIT_MODE4_EO_PL       = 0x800238010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_BIT_MODE4_EO_PL       = 0x800238010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_BIT_MODE4_EO_PL       = 0x800238010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_BIT_MODE4_EO_PL       = 0x800238010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_BIT_MODE4_EO_PL       = 0x800238010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_BIT_MODE4_EO_PL        = 0x800238010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_BIT_MODE4_EO_PL        = 0x800238010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_BIT_MODE4_EO_PL        = 0x800238010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_BIT_MODE4_EO_PL        = 0x800238010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_BIT_MODE4_EO_PL       = 0x800238010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_BIT_MODE4_EO_PL       = 0x800238010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_EO_PL           = 0x800268010701103Full;
//DUPS: 800268010701143F, 800268010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_EO_PL         = 0x800268010701103Full;
//DUPS: 800268010701143F, 800268010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_EO_PL         = 0x800268010801103Full;
//DUPS: 800268010801143F, 800268010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_EO_PL          = 0x800268010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_EO_PL         = 0x800268010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_EO_PL       = 0x800268010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_EO_PL       = 0x800268010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_EO_PL       = 0x800268010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_EO_PL       = 0x800268010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_EO_PL       = 0x800268010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_EO_PL       = 0x800268010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_EO_PL        = 0x800268010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_EO_PL        = 0x800268010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_EO_PL        = 0x800268010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_EO_PL        = 0x800268010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_EO_PL       = 0x800268010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_EO_PL       = 0x800268010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_O_PL            = 0x800318010701103Full;
//DUPS: 800318010701143F, 800318010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_O_PL          = 0x800318010701103Full;
//DUPS: 800318010701143F, 800318010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_O_PL          = 0x800318010801103Full;
//DUPS: 800318010801143F, 800318010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_O_PL           = 0x800318010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_O_PL          = 0x800318010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_O_PL        = 0x800318010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_O_PL        = 0x800318010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_O_PL        = 0x800318010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_O_PL        = 0x800318010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_O_PL        = 0x800318010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_O_PL        = 0x800318010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_O_PL         = 0x800318010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_O_PL         = 0x800318010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_O_PL         = 0x800318010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_O_PL         = 0x800318010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_O_PL        = 0x800318010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_BIT_STAT1_O_PL        = 0x800318010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_BIT_STAT2_EO_PL           = 0x800270010701103Full;
//DUPS: 800270010701143F, 800270010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_BIT_STAT2_EO_PL         = 0x800270010701103Full;
//DUPS: 800270010701143F, 800270010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_BIT_STAT2_EO_PL         = 0x800270010801103Full;
//DUPS: 800270010801143F, 800270010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_BIT_STAT2_EO_PL          = 0x800270010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_BIT_STAT2_EO_PL         = 0x800270010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_BIT_STAT2_EO_PL       = 0x800270010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_BIT_STAT2_EO_PL       = 0x800270010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_BIT_STAT2_EO_PL       = 0x800270010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_BIT_STAT2_EO_PL       = 0x800270010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_BIT_STAT2_EO_PL       = 0x800270010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_BIT_STAT2_EO_PL       = 0x800270010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_BIT_STAT2_EO_PL        = 0x800270010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_BIT_STAT2_EO_PL        = 0x800270010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_BIT_STAT2_EO_PL        = 0x800270010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_BIT_STAT2_EO_PL        = 0x800270010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_BIT_STAT2_EO_PL       = 0x800270010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_BIT_STAT2_EO_PL       = 0x800270010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_BIT_STAT3_EO_PL           = 0x800278010701103Full;
//DUPS: 800278010701143F, 800278010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_BIT_STAT3_EO_PL         = 0x800278010701103Full;
//DUPS: 800278010701143F, 800278010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_BIT_STAT3_EO_PL         = 0x800278010801103Full;
//DUPS: 800278010801143F, 800278010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_BIT_STAT3_EO_PL          = 0x800278010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_BIT_STAT3_EO_PL         = 0x800278010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_BIT_STAT3_EO_PL       = 0x800278010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_BIT_STAT3_EO_PL       = 0x800278010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_BIT_STAT3_EO_PL       = 0x800278010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_BIT_STAT3_EO_PL       = 0x800278010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_BIT_STAT3_EO_PL       = 0x800278010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_BIT_STAT3_EO_PL       = 0x800278010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_BIT_STAT3_EO_PL        = 0x800278010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_BIT_STAT3_EO_PL        = 0x800278010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_BIT_STAT3_EO_PL        = 0x800278010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_BIT_STAT3_EO_PL        = 0x800278010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_BIT_STAT3_EO_PL       = 0x800278010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_BIT_STAT3_EO_PL       = 0x800278010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_BIT_STAT4_EO_PL           = 0x800280010701103Full;
//DUPS: 800280010701143F, 800280010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_BIT_STAT4_EO_PL         = 0x800280010701103Full;
//DUPS: 800280010701143F, 800280010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_BIT_STAT4_EO_PL         = 0x800280010801103Full;
//DUPS: 800280010801143F, 800280010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_BIT_STAT4_EO_PL          = 0x800280010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_BIT_STAT4_EO_PL         = 0x800280010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_BIT_STAT4_EO_PL       = 0x800280010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_BIT_STAT4_EO_PL       = 0x800280010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_BIT_STAT4_EO_PL       = 0x800280010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_BIT_STAT4_EO_PL       = 0x800280010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_BIT_STAT4_EO_PL       = 0x800280010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_BIT_STAT4_EO_PL       = 0x800280010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_BIT_STAT4_EO_PL        = 0x800280010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_BIT_STAT4_EO_PL        = 0x800280010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_BIT_STAT4_EO_PL        = 0x800280010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_BIT_STAT4_EO_PL        = 0x800280010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_BIT_STAT4_EO_PL       = 0x800280010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_BIT_STAT4_EO_PL       = 0x800280010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL10_O_PL           = 0x8000C8010701103Full;
//DUPS: 8000C8010701143F, 8000C8010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL10_O_PL         = 0x8000C8010701103Full;
//DUPS: 8000C8010701143F, 8000C8010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL10_O_PL         = 0x8000C8010801103Full;
//DUPS: 8000C8010801143F, 8000C8010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL10_O_PL          = 0x8000C8010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL10_O_PL         = 0x8000C8010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C8010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C8010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C8010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C8010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C8010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C8010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL10_O_PL        = 0x8000C8010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL10_O_PL        = 0x8000C8010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL10_O_PL        = 0x8000C8010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL10_O_PL        = 0x8000C8010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C8010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C8010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_EO_PL           = 0x800008010701103Full;
//DUPS: 800008010701143F, 800008010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_EO_PL         = 0x800008010701103Full;
//DUPS: 800008010701143F, 800008010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_EO_PL         = 0x800008010801103Full;
//DUPS: 800008010801143F, 800008010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_EO_PL          = 0x800008010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_EO_PL         = 0x800008010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x800008010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x800008010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x800008010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x800008010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x800008010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x800008010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_EO_PL        = 0x800008010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_EO_PL        = 0x800008010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_EO_PL        = 0x800008010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_EO_PL        = 0x800008010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x800008010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x800008010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_O_PL            = 0x800080010701103Full;
//DUPS: 800080010701143F, 800080010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_O_PL          = 0x800080010701103Full;
//DUPS: 800080010701143F, 800080010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_O_PL          = 0x800080010801103Full;
//DUPS: 800080010801143F, 800080010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_O_PL           = 0x800080010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_O_PL          = 0x800080010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_O_PL        = 0x800080010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_O_PL        = 0x800080010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_O_PL        = 0x800080010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_O_PL        = 0x800080010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_O_PL        = 0x800080010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_O_PL        = 0x800080010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_O_PL         = 0x800080010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_O_PL         = 0x800080010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_O_PL         = 0x800080010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_O_PL         = 0x800080010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_O_PL        = 0x800080010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL1_O_PL        = 0x800080010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_EO_PL           = 0x800010010701103Full;
//DUPS: 800010010701143F, 800010010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_EO_PL         = 0x800010010701103Full;
//DUPS: 800010010701143F, 800010010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_EO_PL         = 0x800010010801103Full;
//DUPS: 800010010801143F, 800010010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_EO_PL          = 0x800010010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_EO_PL         = 0x800010010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x800010010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x800010010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x800010010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x800010010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x800010010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x800010010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_EO_PL        = 0x800010010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_EO_PL        = 0x800010010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_EO_PL        = 0x800010010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_EO_PL        = 0x800010010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x800010010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x800010010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_O_PL            = 0x800088010701103Full;
//DUPS: 800088010701143F, 800088010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_O_PL          = 0x800088010701103Full;
//DUPS: 800088010701143F, 800088010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_O_PL          = 0x800088010801103Full;
//DUPS: 800088010801143F, 800088010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_O_PL           = 0x800088010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_O_PL          = 0x800088010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_O_PL        = 0x800088010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_O_PL        = 0x800088010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_O_PL        = 0x800088010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_O_PL        = 0x800088010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_O_PL        = 0x800088010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_O_PL        = 0x800088010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_O_PL         = 0x800088010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_O_PL         = 0x800088010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_O_PL         = 0x800088010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_O_PL         = 0x800088010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_O_PL        = 0x800088010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL2_O_PL        = 0x800088010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_EO_PL           = 0x800018010701103Full;
//DUPS: 800018010701143F, 800018010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_EO_PL         = 0x800018010701103Full;
//DUPS: 800018010701143F, 800018010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_EO_PL         = 0x800018010801103Full;
//DUPS: 800018010801143F, 800018010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_EO_PL          = 0x800018010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_EO_PL         = 0x800018010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x800018010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x800018010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x800018010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x800018010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x800018010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x800018010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_EO_PL        = 0x800018010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_EO_PL        = 0x800018010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_EO_PL        = 0x800018010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_EO_PL        = 0x800018010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x800018010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x800018010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_O_PL            = 0x800090010701103Full;
//DUPS: 800090010701143F, 800090010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_O_PL          = 0x800090010701103Full;
//DUPS: 800090010701143F, 800090010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_O_PL          = 0x800090010801103Full;
//DUPS: 800090010801143F, 800090010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_O_PL           = 0x800090010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_O_PL          = 0x800090010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_O_PL        = 0x800090010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_O_PL        = 0x800090010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_O_PL        = 0x800090010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_O_PL        = 0x800090010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_O_PL        = 0x800090010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_O_PL        = 0x800090010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_O_PL         = 0x800090010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_O_PL         = 0x800090010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_O_PL         = 0x800090010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_O_PL         = 0x800090010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_O_PL        = 0x800090010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL3_O_PL        = 0x800090010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_EO_PL           = 0x800020010701103Full;
//DUPS: 800020010701143F, 800020010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_EO_PL         = 0x800020010701103Full;
//DUPS: 800020010701143F, 800020010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_EO_PL         = 0x800020010801103Full;
//DUPS: 800020010801143F, 800020010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_EO_PL          = 0x800020010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_EO_PL         = 0x800020010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x800020010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x800020010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x800020010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x800020010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x800020010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x800020010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_EO_PL        = 0x800020010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_EO_PL        = 0x800020010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_EO_PL        = 0x800020010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_EO_PL        = 0x800020010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x800020010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x800020010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_O_PL            = 0x800098010701103Full;
//DUPS: 800098010701143F, 800098010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_O_PL          = 0x800098010701103Full;
//DUPS: 800098010701143F, 800098010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_O_PL          = 0x800098010801103Full;
//DUPS: 800098010801143F, 800098010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_O_PL           = 0x800098010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_O_PL          = 0x800098010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_O_PL        = 0x800098010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_O_PL        = 0x800098010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_O_PL        = 0x800098010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_O_PL        = 0x800098010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_O_PL        = 0x800098010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_O_PL        = 0x800098010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_O_PL         = 0x800098010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_O_PL         = 0x800098010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_O_PL         = 0x800098010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_O_PL         = 0x800098010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_O_PL        = 0x800098010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL4_O_PL        = 0x800098010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_EO_PL           = 0x800028010701103Full;
//DUPS: 800028010701143F, 800028010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_EO_PL         = 0x800028010701103Full;
//DUPS: 800028010701143F, 800028010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_EO_PL         = 0x800028010801103Full;
//DUPS: 800028010801143F, 800028010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_EO_PL          = 0x800028010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_EO_PL         = 0x800028010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x800028010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x800028010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x800028010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x800028010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x800028010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x800028010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_EO_PL        = 0x800028010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_EO_PL        = 0x800028010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_EO_PL        = 0x800028010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_EO_PL        = 0x800028010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x800028010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x800028010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_O_PL            = 0x8000A0010701103Full;
//DUPS: 8000A0010701143F, 8000A0010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_O_PL          = 0x8000A0010701103Full;
//DUPS: 8000A0010701143F, 8000A0010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_O_PL          = 0x8000A0010801103Full;
//DUPS: 8000A0010801143F, 8000A0010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_O_PL           = 0x8000A0010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_O_PL          = 0x8000A0010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A0010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A0010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A0010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A0010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A0010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A0010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_O_PL         = 0x8000A0010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_O_PL         = 0x8000A0010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_O_PL         = 0x8000A0010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_O_PL         = 0x8000A0010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A0010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A0010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_EO_PL           = 0x800030010701103Full;
//DUPS: 800030010701143F, 800030010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_EO_PL         = 0x800030010701103Full;
//DUPS: 800030010701143F, 800030010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_EO_PL         = 0x800030010801103Full;
//DUPS: 800030010801143F, 800030010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_EO_PL          = 0x800030010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_EO_PL         = 0x800030010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x800030010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x800030010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x800030010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x800030010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x800030010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x800030010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_EO_PL        = 0x800030010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_EO_PL        = 0x800030010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_EO_PL        = 0x800030010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_EO_PL        = 0x800030010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x800030010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x800030010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_O_PL            = 0x8000A8010701103Full;
//DUPS: 8000A8010701143F, 8000A8010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_O_PL          = 0x8000A8010701103Full;
//DUPS: 8000A8010701143F, 8000A8010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_O_PL          = 0x8000A8010801103Full;
//DUPS: 8000A8010801143F, 8000A8010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_O_PL           = 0x8000A8010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_O_PL          = 0x8000A8010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A8010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A8010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A8010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A8010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A8010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A8010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_O_PL         = 0x8000A8010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_O_PL         = 0x8000A8010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_O_PL         = 0x8000A8010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_O_PL         = 0x8000A8010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A8010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A8010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_EO_PL           = 0x800038010701103Full;
//DUPS: 800038010701143F, 800038010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_EO_PL         = 0x800038010701103Full;
//DUPS: 800038010701143F, 800038010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_EO_PL         = 0x800038010801103Full;
//DUPS: 800038010801143F, 800038010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_EO_PL          = 0x800038010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_EO_PL         = 0x800038010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x800038010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x800038010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x800038010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x800038010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x800038010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x800038010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_EO_PL        = 0x800038010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_EO_PL        = 0x800038010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_EO_PL        = 0x800038010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_EO_PL        = 0x800038010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x800038010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x800038010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_O_PL            = 0x8000B0010701103Full;
//DUPS: 8000B0010701143F, 8000B0010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_O_PL          = 0x8000B0010701103Full;
//DUPS: 8000B0010701143F, 8000B0010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_O_PL          = 0x8000B0010801103Full;
//DUPS: 8000B0010801143F, 8000B0010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_O_PL           = 0x8000B0010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_O_PL          = 0x8000B0010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B0010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B0010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B0010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B0010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B0010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B0010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_O_PL         = 0x8000B0010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_O_PL         = 0x8000B0010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_O_PL         = 0x8000B0010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_O_PL         = 0x8000B0010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B0010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B0010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_EO_PL           = 0x800040010701103Full;
//DUPS: 800040010701143F, 800040010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_EO_PL         = 0x800040010701103Full;
//DUPS: 800040010701143F, 800040010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_EO_PL         = 0x800040010801103Full;
//DUPS: 800040010801143F, 800040010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_EO_PL          = 0x800040010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_EO_PL         = 0x800040010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x800040010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x800040010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x800040010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x800040010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x800040010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x800040010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_EO_PL        = 0x800040010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_EO_PL        = 0x800040010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_EO_PL        = 0x800040010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_EO_PL        = 0x800040010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x800040010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x800040010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_O_PL            = 0x8000B8010701103Full;
//DUPS: 8000B8010701143F, 8000B8010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_O_PL          = 0x8000B8010701103Full;
//DUPS: 8000B8010701143F, 8000B8010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_O_PL          = 0x8000B8010801103Full;
//DUPS: 8000B8010801143F, 8000B8010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_O_PL           = 0x8000B8010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_O_PL          = 0x8000B8010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B8010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B8010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B8010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B8010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B8010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B8010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_O_PL         = 0x8000B8010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_O_PL         = 0x8000B8010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_O_PL         = 0x8000B8010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_O_PL         = 0x8000B8010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B8010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B8010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL9_O_PL            = 0x8000C0010701103Full;
//DUPS: 8000C0010701143F, 8000C0010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL9_O_PL          = 0x8000C0010701103Full;
//DUPS: 8000C0010701143F, 8000C0010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL9_O_PL          = 0x8000C0010801103Full;
//DUPS: 8000C0010801143F, 8000C0010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL9_O_PL           = 0x8000C0010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL9_O_PL          = 0x8000C0010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C0010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C0010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C0010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C0010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C0010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C0010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL9_O_PL         = 0x8000C0010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL9_O_PL         = 0x8000C0010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL9_O_PL         = 0x8000C0010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL9_O_PL         = 0x8000C0010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C0010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C0010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_DATA_DAC_SPARE_MODE_PL    = 0x800000010701103Full;
//DUPS: 800000010701143F, 800000010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000010701103Full;
//DUPS: 800000010701143F, 800000010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000010801103Full;
//DUPS: 800000010801143F, 800000010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_DATA_DAC_SPARE_MODE_PL   = 0x800000010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_FIR_ERROR_INJECT_PL       = 0x800218010701103Full;
//DUPS: 800218010701143F, 800218010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_FIR_ERROR_INJECT_PL     = 0x800218010701103Full;
//DUPS: 800218010701143F, 800218010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_FIR_ERROR_INJECT_PL     = 0x800218010801103Full;
//DUPS: 800218010801143F, 800218010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_FIR_ERROR_INJECT_PL      = 0x800218010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_FIR_ERROR_INJECT_PL     = 0x800218010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x800218010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x800218010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x800218010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x800218010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x800218010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x800218010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_FIR_ERROR_INJECT_PL    = 0x800218010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_FIR_ERROR_INJECT_PL    = 0x800218010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_FIR_ERROR_INJECT_PL    = 0x800218010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_FIR_ERROR_INJECT_PL    = 0x800218010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x800218010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x800218010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_FIR_MASK_PL               = 0x800210010701103Full;
//DUPS: 800210010701143F, 800210010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_FIR_MASK_PL             = 0x800210010701103Full;
//DUPS: 800210010701143F, 800210010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_FIR_MASK_PL             = 0x800210010801103Full;
//DUPS: 800210010801143F, 800210010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_FIR_MASK_PL              = 0x800210010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_FIR_MASK_PL             = 0x800210010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_FIR_MASK_PL           = 0x800210010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_FIR_MASK_PL           = 0x800210010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_FIR_MASK_PL           = 0x800210010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_FIR_MASK_PL           = 0x800210010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_FIR_MASK_PL           = 0x800210010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_FIR_MASK_PL           = 0x800210010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_FIR_MASK_PL            = 0x800210010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_FIR_MASK_PL            = 0x800210010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_FIR_MASK_PL            = 0x800210010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_FIR_MASK_PL            = 0x800210010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_FIR_MASK_PL           = 0x800210010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_FIR_MASK_PL           = 0x800210010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_FIR_PL                    = 0x800208010701103Full;
//DUPS: 800208010701143F, 800208010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_FIR_PL                  = 0x800208010701103Full;
//DUPS: 800208010701143F, 800208010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_FIR_PL                  = 0x800208010801103Full;
//DUPS: 800208010801143F, 800208010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_FIR_PL                   = 0x800208010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_FIR_PL                  = 0x800208010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_FIR_PL                = 0x800208010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_FIR_PL                = 0x800208010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_FIR_PL                = 0x800208010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_FIR_PL                = 0x800208010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_FIR_PL                = 0x800208010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_FIR_PL                = 0x800208010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_FIR_PL                 = 0x800208010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_FIR_PL                 = 0x800208010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_FIR_PL                 = 0x800208010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_FIR_PL                 = 0x800208010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_FIR_PL                = 0x800208010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_FIR_PL                = 0x800208010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE1_RX_SPARE_MODE_PL             = 0x800200010701103Full;
//DUPS: 800200010701143F, 800200010701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE1_RX_SPARE_MODE_PL           = 0x800200010701103Full;
//DUPS: 800200010701143F, 800200010701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE1_RX_SPARE_MODE_PL           = 0x800200010801103Full;
//DUPS: 800200010801143F, 800200010801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE1_RX_SPARE_MODE_PL            = 0x800200010701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE1_RX_SPARE_MODE_PL           = 0x800200010701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE1_RX_SPARE_MODE_PL         = 0x800200010701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE1_RX_SPARE_MODE_PL         = 0x800200010701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE1_RX_SPARE_MODE_PL         = 0x800200010701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE1_RX_SPARE_MODE_PL         = 0x800200010801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE1_RX_SPARE_MODE_PL         = 0x800200010801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE1_RX_SPARE_MODE_PL         = 0x800200010801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE1_RX_SPARE_MODE_PL          = 0x800200010701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE1_RX_SPARE_MODE_PL          = 0x800200010701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE1_RX_SPARE_MODE_PL          = 0x800200010701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE1_RX_SPARE_MODE_PL          = 0x800200010801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE1_RX_SPARE_MODE_PL         = 0x800200010801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE1_RX_SPARE_MODE_PL         = 0x800200010801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL2_EO_PL           = 0x800248020701103Full;
//DUPS: 800248020701143F, 800248020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL2_EO_PL         = 0x800248020701103Full;
//DUPS: 800248020701143F, 800248020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL2_EO_PL         = 0x800248020801103Full;
//DUPS: 800248020801143F, 800248020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL2_EO_PL          = 0x800248020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL2_EO_PL         = 0x800248020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x800248020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x800248020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x800248020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x800248020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x800248020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x800248020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL2_EO_PL        = 0x800248020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL2_EO_PL        = 0x800248020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL2_EO_PL        = 0x800248020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL2_EO_PL        = 0x800248020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x800248020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x800248020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL3_EO_PL           = 0x800250020701103Full;
//DUPS: 800250020701143F, 800250020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL3_EO_PL         = 0x800250020701103Full;
//DUPS: 800250020701143F, 800250020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL3_EO_PL         = 0x800250020801103Full;
//DUPS: 800250020801143F, 800250020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL3_EO_PL          = 0x800250020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL3_EO_PL         = 0x800250020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x800250020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x800250020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x800250020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x800250020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x800250020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x800250020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL3_EO_PL        = 0x800250020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL3_EO_PL        = 0x800250020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL3_EO_PL        = 0x800250020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL3_EO_PL        = 0x800250020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x800250020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x800250020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL4_EO_PL           = 0x800258020701103Full;
//DUPS: 800258020701143F, 800258020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL4_EO_PL         = 0x800258020701103Full;
//DUPS: 800258020701143F, 800258020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL4_EO_PL         = 0x800258020801103Full;
//DUPS: 800258020801143F, 800258020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL4_EO_PL          = 0x800258020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL4_EO_PL         = 0x800258020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x800258020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x800258020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x800258020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x800258020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x800258020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x800258020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL4_EO_PL        = 0x800258020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL4_EO_PL        = 0x800258020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL4_EO_PL        = 0x800258020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL4_EO_PL        = 0x800258020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x800258020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x800258020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL5_EO_PL           = 0x800260020701103Full;
//DUPS: 800260020701143F, 800260020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL5_EO_PL         = 0x800260020701103Full;
//DUPS: 800260020701143F, 800260020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL5_EO_PL         = 0x800260020801103Full;
//DUPS: 800260020801143F, 800260020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL5_EO_PL          = 0x800260020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL5_EO_PL         = 0x800260020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x800260020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x800260020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x800260020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x800260020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x800260020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x800260020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL5_EO_PL        = 0x800260020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL5_EO_PL        = 0x800260020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL5_EO_PL        = 0x800260020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL5_EO_PL        = 0x800260020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x800260020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x800260020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_BIT_CNTLX1_EO_PL          = 0x800240020701103Full;
//DUPS: 800240020701143F, 800240020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_BIT_CNTLX1_EO_PL        = 0x800240020701103Full;
//DUPS: 800240020701143F, 800240020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_BIT_CNTLX1_EO_PL        = 0x800240020801103Full;
//DUPS: 800240020801143F, 800240020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_BIT_CNTLX1_EO_PL         = 0x800240020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_BIT_CNTLX1_EO_PL        = 0x800240020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x800240020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x800240020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x800240020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x800240020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x800240020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x800240020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_BIT_CNTLX1_EO_PL       = 0x800240020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_BIT_CNTLX1_EO_PL       = 0x800240020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_BIT_CNTLX1_EO_PL       = 0x800240020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_BIT_CNTLX1_EO_PL       = 0x800240020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x800240020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x800240020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_BIT_MODE1_EO_PL           = 0x800220020701103Full;
//DUPS: 800220020701143F, 800220020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_BIT_MODE1_EO_PL         = 0x800220020701103Full;
//DUPS: 800220020701143F, 800220020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_BIT_MODE1_EO_PL         = 0x800220020801103Full;
//DUPS: 800220020801143F, 800220020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_BIT_MODE1_EO_PL          = 0x800220020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_BIT_MODE1_EO_PL         = 0x800220020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_BIT_MODE1_EO_PL       = 0x800220020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_BIT_MODE1_EO_PL       = 0x800220020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_BIT_MODE1_EO_PL       = 0x800220020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_BIT_MODE1_EO_PL       = 0x800220020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_BIT_MODE1_EO_PL       = 0x800220020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_BIT_MODE1_EO_PL       = 0x800220020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_BIT_MODE1_EO_PL        = 0x800220020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_BIT_MODE1_EO_PL        = 0x800220020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_BIT_MODE1_EO_PL        = 0x800220020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_BIT_MODE1_EO_PL        = 0x800220020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_BIT_MODE1_EO_PL       = 0x800220020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_BIT_MODE1_EO_PL       = 0x800220020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_BIT_MODE2_EO_PL           = 0x800228020701103Full;
//DUPS: 800228020701143F, 800228020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_BIT_MODE2_EO_PL         = 0x800228020701103Full;
//DUPS: 800228020701143F, 800228020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_BIT_MODE2_EO_PL         = 0x800228020801103Full;
//DUPS: 800228020801143F, 800228020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_BIT_MODE2_EO_PL          = 0x800228020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_BIT_MODE2_EO_PL         = 0x800228020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_BIT_MODE2_EO_PL       = 0x800228020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_BIT_MODE2_EO_PL       = 0x800228020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_BIT_MODE2_EO_PL       = 0x800228020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_BIT_MODE2_EO_PL       = 0x800228020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_BIT_MODE2_EO_PL       = 0x800228020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_BIT_MODE2_EO_PL       = 0x800228020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_BIT_MODE2_EO_PL        = 0x800228020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_BIT_MODE2_EO_PL        = 0x800228020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_BIT_MODE2_EO_PL        = 0x800228020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_BIT_MODE2_EO_PL        = 0x800228020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_BIT_MODE2_EO_PL       = 0x800228020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_BIT_MODE2_EO_PL       = 0x800228020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_BIT_MODE3_EO_PL           = 0x800230020701103Full;
//DUPS: 800230020701143F, 800230020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_BIT_MODE3_EO_PL         = 0x800230020701103Full;
//DUPS: 800230020701143F, 800230020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_BIT_MODE3_EO_PL         = 0x800230020801103Full;
//DUPS: 800230020801143F, 800230020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_BIT_MODE3_EO_PL          = 0x800230020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_BIT_MODE3_EO_PL         = 0x800230020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_BIT_MODE3_EO_PL       = 0x800230020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_BIT_MODE3_EO_PL       = 0x800230020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_BIT_MODE3_EO_PL       = 0x800230020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_BIT_MODE3_EO_PL       = 0x800230020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_BIT_MODE3_EO_PL       = 0x800230020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_BIT_MODE3_EO_PL       = 0x800230020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_BIT_MODE3_EO_PL        = 0x800230020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_BIT_MODE3_EO_PL        = 0x800230020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_BIT_MODE3_EO_PL        = 0x800230020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_BIT_MODE3_EO_PL        = 0x800230020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_BIT_MODE3_EO_PL       = 0x800230020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_BIT_MODE3_EO_PL       = 0x800230020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_BIT_MODE4_EO_PL           = 0x800238020701103Full;
//DUPS: 800238020701143F, 800238020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_BIT_MODE4_EO_PL         = 0x800238020701103Full;
//DUPS: 800238020701143F, 800238020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_BIT_MODE4_EO_PL         = 0x800238020801103Full;
//DUPS: 800238020801143F, 800238020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_BIT_MODE4_EO_PL          = 0x800238020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_BIT_MODE4_EO_PL         = 0x800238020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_BIT_MODE4_EO_PL       = 0x800238020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_BIT_MODE4_EO_PL       = 0x800238020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_BIT_MODE4_EO_PL       = 0x800238020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_BIT_MODE4_EO_PL       = 0x800238020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_BIT_MODE4_EO_PL       = 0x800238020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_BIT_MODE4_EO_PL       = 0x800238020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_BIT_MODE4_EO_PL        = 0x800238020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_BIT_MODE4_EO_PL        = 0x800238020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_BIT_MODE4_EO_PL        = 0x800238020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_BIT_MODE4_EO_PL        = 0x800238020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_BIT_MODE4_EO_PL       = 0x800238020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_BIT_MODE4_EO_PL       = 0x800238020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_EO_PL           = 0x800268020701103Full;
//DUPS: 800268020701143F, 800268020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_EO_PL         = 0x800268020701103Full;
//DUPS: 800268020701143F, 800268020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_EO_PL         = 0x800268020801103Full;
//DUPS: 800268020801143F, 800268020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_EO_PL          = 0x800268020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_EO_PL         = 0x800268020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_EO_PL       = 0x800268020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_EO_PL       = 0x800268020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_EO_PL       = 0x800268020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_EO_PL       = 0x800268020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_EO_PL       = 0x800268020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_EO_PL       = 0x800268020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_EO_PL        = 0x800268020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_EO_PL        = 0x800268020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_EO_PL        = 0x800268020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_EO_PL        = 0x800268020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_EO_PL       = 0x800268020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_EO_PL       = 0x800268020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_O_PL            = 0x800318020701103Full;
//DUPS: 800318020701143F, 800318020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_O_PL          = 0x800318020701103Full;
//DUPS: 800318020701143F, 800318020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_O_PL          = 0x800318020801103Full;
//DUPS: 800318020801143F, 800318020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_O_PL           = 0x800318020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_O_PL          = 0x800318020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_O_PL        = 0x800318020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_O_PL        = 0x800318020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_O_PL        = 0x800318020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_O_PL        = 0x800318020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_O_PL        = 0x800318020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_O_PL        = 0x800318020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_O_PL         = 0x800318020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_O_PL         = 0x800318020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_O_PL         = 0x800318020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_O_PL         = 0x800318020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_O_PL        = 0x800318020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_BIT_STAT1_O_PL        = 0x800318020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_BIT_STAT2_EO_PL           = 0x800270020701103Full;
//DUPS: 800270020701143F, 800270020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_BIT_STAT2_EO_PL         = 0x800270020701103Full;
//DUPS: 800270020701143F, 800270020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_BIT_STAT2_EO_PL         = 0x800270020801103Full;
//DUPS: 800270020801143F, 800270020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_BIT_STAT2_EO_PL          = 0x800270020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_BIT_STAT2_EO_PL         = 0x800270020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_BIT_STAT2_EO_PL       = 0x800270020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_BIT_STAT2_EO_PL       = 0x800270020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_BIT_STAT2_EO_PL       = 0x800270020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_BIT_STAT2_EO_PL       = 0x800270020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_BIT_STAT2_EO_PL       = 0x800270020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_BIT_STAT2_EO_PL       = 0x800270020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_BIT_STAT2_EO_PL        = 0x800270020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_BIT_STAT2_EO_PL        = 0x800270020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_BIT_STAT2_EO_PL        = 0x800270020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_BIT_STAT2_EO_PL        = 0x800270020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_BIT_STAT2_EO_PL       = 0x800270020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_BIT_STAT2_EO_PL       = 0x800270020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_BIT_STAT3_EO_PL           = 0x800278020701103Full;
//DUPS: 800278020701143F, 800278020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_BIT_STAT3_EO_PL         = 0x800278020701103Full;
//DUPS: 800278020701143F, 800278020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_BIT_STAT3_EO_PL         = 0x800278020801103Full;
//DUPS: 800278020801143F, 800278020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_BIT_STAT3_EO_PL          = 0x800278020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_BIT_STAT3_EO_PL         = 0x800278020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_BIT_STAT3_EO_PL       = 0x800278020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_BIT_STAT3_EO_PL       = 0x800278020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_BIT_STAT3_EO_PL       = 0x800278020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_BIT_STAT3_EO_PL       = 0x800278020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_BIT_STAT3_EO_PL       = 0x800278020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_BIT_STAT3_EO_PL       = 0x800278020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_BIT_STAT3_EO_PL        = 0x800278020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_BIT_STAT3_EO_PL        = 0x800278020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_BIT_STAT3_EO_PL        = 0x800278020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_BIT_STAT3_EO_PL        = 0x800278020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_BIT_STAT3_EO_PL       = 0x800278020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_BIT_STAT3_EO_PL       = 0x800278020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_BIT_STAT4_EO_PL           = 0x800280020701103Full;
//DUPS: 800280020701143F, 800280020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_BIT_STAT4_EO_PL         = 0x800280020701103Full;
//DUPS: 800280020701143F, 800280020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_BIT_STAT4_EO_PL         = 0x800280020801103Full;
//DUPS: 800280020801143F, 800280020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_BIT_STAT4_EO_PL          = 0x800280020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_BIT_STAT4_EO_PL         = 0x800280020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_BIT_STAT4_EO_PL       = 0x800280020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_BIT_STAT4_EO_PL       = 0x800280020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_BIT_STAT4_EO_PL       = 0x800280020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_BIT_STAT4_EO_PL       = 0x800280020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_BIT_STAT4_EO_PL       = 0x800280020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_BIT_STAT4_EO_PL       = 0x800280020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_BIT_STAT4_EO_PL        = 0x800280020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_BIT_STAT4_EO_PL        = 0x800280020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_BIT_STAT4_EO_PL        = 0x800280020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_BIT_STAT4_EO_PL        = 0x800280020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_BIT_STAT4_EO_PL       = 0x800280020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_BIT_STAT4_EO_PL       = 0x800280020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL10_O_PL           = 0x8000C8020701103Full;
//DUPS: 8000C8020701143F, 8000C8020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL10_O_PL         = 0x8000C8020701103Full;
//DUPS: 8000C8020701143F, 8000C8020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL10_O_PL         = 0x8000C8020801103Full;
//DUPS: 8000C8020801143F, 8000C8020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL10_O_PL          = 0x8000C8020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL10_O_PL         = 0x8000C8020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C8020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C8020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C8020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C8020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C8020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C8020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL10_O_PL        = 0x8000C8020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL10_O_PL        = 0x8000C8020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL10_O_PL        = 0x8000C8020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL10_O_PL        = 0x8000C8020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C8020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C8020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_EO_PL           = 0x800008020701103Full;
//DUPS: 800008020701143F, 800008020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_EO_PL         = 0x800008020701103Full;
//DUPS: 800008020701143F, 800008020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_EO_PL         = 0x800008020801103Full;
//DUPS: 800008020801143F, 800008020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_EO_PL          = 0x800008020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_EO_PL         = 0x800008020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x800008020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x800008020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x800008020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x800008020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x800008020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x800008020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_EO_PL        = 0x800008020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_EO_PL        = 0x800008020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_EO_PL        = 0x800008020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_EO_PL        = 0x800008020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x800008020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x800008020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_O_PL            = 0x800080020701103Full;
//DUPS: 800080020701143F, 800080020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_O_PL          = 0x800080020701103Full;
//DUPS: 800080020701143F, 800080020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_O_PL          = 0x800080020801103Full;
//DUPS: 800080020801143F, 800080020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_O_PL           = 0x800080020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_O_PL          = 0x800080020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_O_PL        = 0x800080020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_O_PL        = 0x800080020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_O_PL        = 0x800080020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_O_PL        = 0x800080020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_O_PL        = 0x800080020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_O_PL        = 0x800080020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_O_PL         = 0x800080020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_O_PL         = 0x800080020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_O_PL         = 0x800080020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_O_PL         = 0x800080020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_O_PL        = 0x800080020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL1_O_PL        = 0x800080020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_EO_PL           = 0x800010020701103Full;
//DUPS: 800010020701143F, 800010020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_EO_PL         = 0x800010020701103Full;
//DUPS: 800010020701143F, 800010020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_EO_PL         = 0x800010020801103Full;
//DUPS: 800010020801143F, 800010020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_EO_PL          = 0x800010020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_EO_PL         = 0x800010020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x800010020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x800010020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x800010020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x800010020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x800010020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x800010020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_EO_PL        = 0x800010020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_EO_PL        = 0x800010020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_EO_PL        = 0x800010020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_EO_PL        = 0x800010020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x800010020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x800010020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_O_PL            = 0x800088020701103Full;
//DUPS: 800088020701143F, 800088020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_O_PL          = 0x800088020701103Full;
//DUPS: 800088020701143F, 800088020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_O_PL          = 0x800088020801103Full;
//DUPS: 800088020801143F, 800088020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_O_PL           = 0x800088020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_O_PL          = 0x800088020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_O_PL        = 0x800088020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_O_PL        = 0x800088020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_O_PL        = 0x800088020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_O_PL        = 0x800088020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_O_PL        = 0x800088020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_O_PL        = 0x800088020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_O_PL         = 0x800088020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_O_PL         = 0x800088020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_O_PL         = 0x800088020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_O_PL         = 0x800088020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_O_PL        = 0x800088020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL2_O_PL        = 0x800088020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_EO_PL           = 0x800018020701103Full;
//DUPS: 800018020701143F, 800018020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_EO_PL         = 0x800018020701103Full;
//DUPS: 800018020701143F, 800018020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_EO_PL         = 0x800018020801103Full;
//DUPS: 800018020801143F, 800018020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_EO_PL          = 0x800018020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_EO_PL         = 0x800018020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x800018020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x800018020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x800018020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x800018020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x800018020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x800018020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_EO_PL        = 0x800018020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_EO_PL        = 0x800018020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_EO_PL        = 0x800018020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_EO_PL        = 0x800018020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x800018020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x800018020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_O_PL            = 0x800090020701103Full;
//DUPS: 800090020701143F, 800090020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_O_PL          = 0x800090020701103Full;
//DUPS: 800090020701143F, 800090020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_O_PL          = 0x800090020801103Full;
//DUPS: 800090020801143F, 800090020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_O_PL           = 0x800090020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_O_PL          = 0x800090020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_O_PL        = 0x800090020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_O_PL        = 0x800090020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_O_PL        = 0x800090020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_O_PL        = 0x800090020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_O_PL        = 0x800090020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_O_PL        = 0x800090020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_O_PL         = 0x800090020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_O_PL         = 0x800090020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_O_PL         = 0x800090020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_O_PL         = 0x800090020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_O_PL        = 0x800090020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL3_O_PL        = 0x800090020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_EO_PL           = 0x800020020701103Full;
//DUPS: 800020020701143F, 800020020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_EO_PL         = 0x800020020701103Full;
//DUPS: 800020020701143F, 800020020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_EO_PL         = 0x800020020801103Full;
//DUPS: 800020020801143F, 800020020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_EO_PL          = 0x800020020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_EO_PL         = 0x800020020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x800020020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x800020020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x800020020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x800020020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x800020020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x800020020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_EO_PL        = 0x800020020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_EO_PL        = 0x800020020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_EO_PL        = 0x800020020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_EO_PL        = 0x800020020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x800020020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x800020020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_O_PL            = 0x800098020701103Full;
//DUPS: 800098020701143F, 800098020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_O_PL          = 0x800098020701103Full;
//DUPS: 800098020701143F, 800098020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_O_PL          = 0x800098020801103Full;
//DUPS: 800098020801143F, 800098020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_O_PL           = 0x800098020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_O_PL          = 0x800098020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_O_PL        = 0x800098020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_O_PL        = 0x800098020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_O_PL        = 0x800098020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_O_PL        = 0x800098020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_O_PL        = 0x800098020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_O_PL        = 0x800098020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_O_PL         = 0x800098020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_O_PL         = 0x800098020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_O_PL         = 0x800098020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_O_PL         = 0x800098020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_O_PL        = 0x800098020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL4_O_PL        = 0x800098020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_EO_PL           = 0x800028020701103Full;
//DUPS: 800028020701143F, 800028020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_EO_PL         = 0x800028020701103Full;
//DUPS: 800028020701143F, 800028020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_EO_PL         = 0x800028020801103Full;
//DUPS: 800028020801143F, 800028020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_EO_PL          = 0x800028020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_EO_PL         = 0x800028020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x800028020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x800028020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x800028020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x800028020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x800028020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x800028020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_EO_PL        = 0x800028020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_EO_PL        = 0x800028020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_EO_PL        = 0x800028020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_EO_PL        = 0x800028020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x800028020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x800028020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_O_PL            = 0x8000A0020701103Full;
//DUPS: 8000A0020701143F, 8000A0020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_O_PL          = 0x8000A0020701103Full;
//DUPS: 8000A0020701143F, 8000A0020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_O_PL          = 0x8000A0020801103Full;
//DUPS: 8000A0020801143F, 8000A0020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_O_PL           = 0x8000A0020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_O_PL          = 0x8000A0020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A0020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A0020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A0020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A0020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A0020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A0020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_O_PL         = 0x8000A0020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_O_PL         = 0x8000A0020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_O_PL         = 0x8000A0020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_O_PL         = 0x8000A0020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A0020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A0020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_EO_PL           = 0x800030020701103Full;
//DUPS: 800030020701143F, 800030020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_EO_PL         = 0x800030020701103Full;
//DUPS: 800030020701143F, 800030020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_EO_PL         = 0x800030020801103Full;
//DUPS: 800030020801143F, 800030020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_EO_PL          = 0x800030020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_EO_PL         = 0x800030020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x800030020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x800030020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x800030020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x800030020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x800030020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x800030020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_EO_PL        = 0x800030020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_EO_PL        = 0x800030020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_EO_PL        = 0x800030020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_EO_PL        = 0x800030020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x800030020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x800030020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_O_PL            = 0x8000A8020701103Full;
//DUPS: 8000A8020701143F, 8000A8020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_O_PL          = 0x8000A8020701103Full;
//DUPS: 8000A8020701143F, 8000A8020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_O_PL          = 0x8000A8020801103Full;
//DUPS: 8000A8020801143F, 8000A8020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_O_PL           = 0x8000A8020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_O_PL          = 0x8000A8020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A8020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A8020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A8020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A8020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A8020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A8020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_O_PL         = 0x8000A8020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_O_PL         = 0x8000A8020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_O_PL         = 0x8000A8020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_O_PL         = 0x8000A8020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A8020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A8020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_EO_PL           = 0x800038020701103Full;
//DUPS: 800038020701143F, 800038020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_EO_PL         = 0x800038020701103Full;
//DUPS: 800038020701143F, 800038020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_EO_PL         = 0x800038020801103Full;
//DUPS: 800038020801143F, 800038020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_EO_PL          = 0x800038020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_EO_PL         = 0x800038020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x800038020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x800038020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x800038020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x800038020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x800038020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x800038020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_EO_PL        = 0x800038020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_EO_PL        = 0x800038020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_EO_PL        = 0x800038020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_EO_PL        = 0x800038020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x800038020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x800038020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_O_PL            = 0x8000B0020701103Full;
//DUPS: 8000B0020701143F, 8000B0020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_O_PL          = 0x8000B0020701103Full;
//DUPS: 8000B0020701143F, 8000B0020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_O_PL          = 0x8000B0020801103Full;
//DUPS: 8000B0020801143F, 8000B0020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_O_PL           = 0x8000B0020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_O_PL          = 0x8000B0020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B0020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B0020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B0020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B0020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B0020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B0020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_O_PL         = 0x8000B0020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_O_PL         = 0x8000B0020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_O_PL         = 0x8000B0020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_O_PL         = 0x8000B0020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B0020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B0020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_EO_PL           = 0x800040020701103Full;
//DUPS: 800040020701143F, 800040020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_EO_PL         = 0x800040020701103Full;
//DUPS: 800040020701143F, 800040020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_EO_PL         = 0x800040020801103Full;
//DUPS: 800040020801143F, 800040020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_EO_PL          = 0x800040020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_EO_PL         = 0x800040020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x800040020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x800040020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x800040020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x800040020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x800040020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x800040020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_EO_PL        = 0x800040020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_EO_PL        = 0x800040020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_EO_PL        = 0x800040020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_EO_PL        = 0x800040020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x800040020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x800040020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_O_PL            = 0x8000B8020701103Full;
//DUPS: 8000B8020701143F, 8000B8020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_O_PL          = 0x8000B8020701103Full;
//DUPS: 8000B8020701143F, 8000B8020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_O_PL          = 0x8000B8020801103Full;
//DUPS: 8000B8020801143F, 8000B8020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_O_PL           = 0x8000B8020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_O_PL          = 0x8000B8020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B8020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B8020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B8020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B8020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B8020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B8020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_O_PL         = 0x8000B8020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_O_PL         = 0x8000B8020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_O_PL         = 0x8000B8020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_O_PL         = 0x8000B8020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B8020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B8020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL9_O_PL            = 0x8000C0020701103Full;
//DUPS: 8000C0020701143F, 8000C0020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL9_O_PL          = 0x8000C0020701103Full;
//DUPS: 8000C0020701143F, 8000C0020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL9_O_PL          = 0x8000C0020801103Full;
//DUPS: 8000C0020801143F, 8000C0020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL9_O_PL           = 0x8000C0020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL9_O_PL          = 0x8000C0020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C0020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C0020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C0020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C0020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C0020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C0020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL9_O_PL         = 0x8000C0020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL9_O_PL         = 0x8000C0020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL9_O_PL         = 0x8000C0020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL9_O_PL         = 0x8000C0020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C0020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C0020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_DATA_DAC_SPARE_MODE_PL    = 0x800000020701103Full;
//DUPS: 800000020701143F, 800000020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000020701103Full;
//DUPS: 800000020701143F, 800000020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000020801103Full;
//DUPS: 800000020801143F, 800000020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_DATA_DAC_SPARE_MODE_PL   = 0x800000020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_FIR_ERROR_INJECT_PL       = 0x800218020701103Full;
//DUPS: 800218020701143F, 800218020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_FIR_ERROR_INJECT_PL     = 0x800218020701103Full;
//DUPS: 800218020701143F, 800218020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_FIR_ERROR_INJECT_PL     = 0x800218020801103Full;
//DUPS: 800218020801143F, 800218020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_FIR_ERROR_INJECT_PL      = 0x800218020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_FIR_ERROR_INJECT_PL     = 0x800218020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x800218020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x800218020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x800218020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x800218020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x800218020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x800218020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_FIR_ERROR_INJECT_PL    = 0x800218020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_FIR_ERROR_INJECT_PL    = 0x800218020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_FIR_ERROR_INJECT_PL    = 0x800218020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_FIR_ERROR_INJECT_PL    = 0x800218020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x800218020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x800218020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_FIR_MASK_PL               = 0x800210020701103Full;
//DUPS: 800210020701143F, 800210020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_FIR_MASK_PL             = 0x800210020701103Full;
//DUPS: 800210020701143F, 800210020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_FIR_MASK_PL             = 0x800210020801103Full;
//DUPS: 800210020801143F, 800210020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_FIR_MASK_PL              = 0x800210020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_FIR_MASK_PL             = 0x800210020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_FIR_MASK_PL           = 0x800210020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_FIR_MASK_PL           = 0x800210020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_FIR_MASK_PL           = 0x800210020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_FIR_MASK_PL           = 0x800210020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_FIR_MASK_PL           = 0x800210020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_FIR_MASK_PL           = 0x800210020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_FIR_MASK_PL            = 0x800210020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_FIR_MASK_PL            = 0x800210020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_FIR_MASK_PL            = 0x800210020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_FIR_MASK_PL            = 0x800210020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_FIR_MASK_PL           = 0x800210020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_FIR_MASK_PL           = 0x800210020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_FIR_PL                    = 0x800208020701103Full;
//DUPS: 800208020701143F, 800208020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_FIR_PL                  = 0x800208020701103Full;
//DUPS: 800208020701143F, 800208020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_FIR_PL                  = 0x800208020801103Full;
//DUPS: 800208020801143F, 800208020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_FIR_PL                   = 0x800208020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_FIR_PL                  = 0x800208020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_FIR_PL                = 0x800208020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_FIR_PL                = 0x800208020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_FIR_PL                = 0x800208020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_FIR_PL                = 0x800208020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_FIR_PL                = 0x800208020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_FIR_PL                = 0x800208020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_FIR_PL                 = 0x800208020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_FIR_PL                 = 0x800208020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_FIR_PL                 = 0x800208020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_FIR_PL                 = 0x800208020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_FIR_PL                = 0x800208020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_FIR_PL                = 0x800208020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE2_RX_SPARE_MODE_PL             = 0x800200020701103Full;
//DUPS: 800200020701143F, 800200020701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE2_RX_SPARE_MODE_PL           = 0x800200020701103Full;
//DUPS: 800200020701143F, 800200020701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE2_RX_SPARE_MODE_PL           = 0x800200020801103Full;
//DUPS: 800200020801143F, 800200020801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE2_RX_SPARE_MODE_PL            = 0x800200020701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE2_RX_SPARE_MODE_PL           = 0x800200020701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE2_RX_SPARE_MODE_PL         = 0x800200020701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE2_RX_SPARE_MODE_PL         = 0x800200020701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE2_RX_SPARE_MODE_PL         = 0x800200020701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE2_RX_SPARE_MODE_PL         = 0x800200020801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE2_RX_SPARE_MODE_PL         = 0x800200020801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE2_RX_SPARE_MODE_PL         = 0x800200020801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE2_RX_SPARE_MODE_PL          = 0x800200020701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE2_RX_SPARE_MODE_PL          = 0x800200020701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE2_RX_SPARE_MODE_PL          = 0x800200020701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE2_RX_SPARE_MODE_PL          = 0x800200020801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE2_RX_SPARE_MODE_PL         = 0x800200020801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE2_RX_SPARE_MODE_PL         = 0x800200020801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL2_EO_PL           = 0x800248030701103Full;
//DUPS: 800248030701143F, 800248030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL2_EO_PL         = 0x800248030701103Full;
//DUPS: 800248030701143F, 800248030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL2_EO_PL         = 0x800248030801103Full;
//DUPS: 800248030801143F, 800248030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL2_EO_PL          = 0x800248030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL2_EO_PL         = 0x800248030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x800248030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x800248030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x800248030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x800248030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x800248030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x800248030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL2_EO_PL        = 0x800248030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL2_EO_PL        = 0x800248030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL2_EO_PL        = 0x800248030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL2_EO_PL        = 0x800248030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x800248030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x800248030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL3_EO_PL           = 0x800250030701103Full;
//DUPS: 800250030701143F, 800250030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL3_EO_PL         = 0x800250030701103Full;
//DUPS: 800250030701143F, 800250030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL3_EO_PL         = 0x800250030801103Full;
//DUPS: 800250030801143F, 800250030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL3_EO_PL          = 0x800250030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL3_EO_PL         = 0x800250030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x800250030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x800250030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x800250030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x800250030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x800250030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x800250030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL3_EO_PL        = 0x800250030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL3_EO_PL        = 0x800250030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL3_EO_PL        = 0x800250030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL3_EO_PL        = 0x800250030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x800250030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x800250030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL4_EO_PL           = 0x800258030701103Full;
//DUPS: 800258030701143F, 800258030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL4_EO_PL         = 0x800258030701103Full;
//DUPS: 800258030701143F, 800258030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL4_EO_PL         = 0x800258030801103Full;
//DUPS: 800258030801143F, 800258030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL4_EO_PL          = 0x800258030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL4_EO_PL         = 0x800258030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x800258030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x800258030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x800258030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x800258030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x800258030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x800258030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL4_EO_PL        = 0x800258030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL4_EO_PL        = 0x800258030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL4_EO_PL        = 0x800258030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL4_EO_PL        = 0x800258030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x800258030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x800258030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL5_EO_PL           = 0x800260030701103Full;
//DUPS: 800260030701143F, 800260030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL5_EO_PL         = 0x800260030701103Full;
//DUPS: 800260030701143F, 800260030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL5_EO_PL         = 0x800260030801103Full;
//DUPS: 800260030801143F, 800260030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL5_EO_PL          = 0x800260030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL5_EO_PL         = 0x800260030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x800260030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x800260030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x800260030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x800260030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x800260030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x800260030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL5_EO_PL        = 0x800260030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL5_EO_PL        = 0x800260030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL5_EO_PL        = 0x800260030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL5_EO_PL        = 0x800260030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x800260030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x800260030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_BIT_CNTLX1_EO_PL          = 0x800240030701103Full;
//DUPS: 800240030701143F, 800240030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_BIT_CNTLX1_EO_PL        = 0x800240030701103Full;
//DUPS: 800240030701143F, 800240030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_BIT_CNTLX1_EO_PL        = 0x800240030801103Full;
//DUPS: 800240030801143F, 800240030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_BIT_CNTLX1_EO_PL         = 0x800240030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_BIT_CNTLX1_EO_PL        = 0x800240030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x800240030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x800240030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x800240030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x800240030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x800240030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x800240030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_BIT_CNTLX1_EO_PL       = 0x800240030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_BIT_CNTLX1_EO_PL       = 0x800240030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_BIT_CNTLX1_EO_PL       = 0x800240030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_BIT_CNTLX1_EO_PL       = 0x800240030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x800240030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x800240030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_BIT_MODE1_EO_PL           = 0x800220030701103Full;
//DUPS: 800220030701143F, 800220030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_BIT_MODE1_EO_PL         = 0x800220030701103Full;
//DUPS: 800220030701143F, 800220030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_BIT_MODE1_EO_PL         = 0x800220030801103Full;
//DUPS: 800220030801143F, 800220030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_BIT_MODE1_EO_PL          = 0x800220030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_BIT_MODE1_EO_PL         = 0x800220030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_BIT_MODE1_EO_PL       = 0x800220030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_BIT_MODE1_EO_PL       = 0x800220030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_BIT_MODE1_EO_PL       = 0x800220030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_BIT_MODE1_EO_PL       = 0x800220030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_BIT_MODE1_EO_PL       = 0x800220030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_BIT_MODE1_EO_PL       = 0x800220030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_BIT_MODE1_EO_PL        = 0x800220030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_BIT_MODE1_EO_PL        = 0x800220030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_BIT_MODE1_EO_PL        = 0x800220030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_BIT_MODE1_EO_PL        = 0x800220030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_BIT_MODE1_EO_PL       = 0x800220030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_BIT_MODE1_EO_PL       = 0x800220030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_BIT_MODE2_EO_PL           = 0x800228030701103Full;
//DUPS: 800228030701143F, 800228030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_BIT_MODE2_EO_PL         = 0x800228030701103Full;
//DUPS: 800228030701143F, 800228030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_BIT_MODE2_EO_PL         = 0x800228030801103Full;
//DUPS: 800228030801143F, 800228030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_BIT_MODE2_EO_PL          = 0x800228030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_BIT_MODE2_EO_PL         = 0x800228030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_BIT_MODE2_EO_PL       = 0x800228030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_BIT_MODE2_EO_PL       = 0x800228030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_BIT_MODE2_EO_PL       = 0x800228030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_BIT_MODE2_EO_PL       = 0x800228030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_BIT_MODE2_EO_PL       = 0x800228030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_BIT_MODE2_EO_PL       = 0x800228030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_BIT_MODE2_EO_PL        = 0x800228030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_BIT_MODE2_EO_PL        = 0x800228030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_BIT_MODE2_EO_PL        = 0x800228030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_BIT_MODE2_EO_PL        = 0x800228030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_BIT_MODE2_EO_PL       = 0x800228030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_BIT_MODE2_EO_PL       = 0x800228030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_BIT_MODE3_EO_PL           = 0x800230030701103Full;
//DUPS: 800230030701143F, 800230030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_BIT_MODE3_EO_PL         = 0x800230030701103Full;
//DUPS: 800230030701143F, 800230030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_BIT_MODE3_EO_PL         = 0x800230030801103Full;
//DUPS: 800230030801143F, 800230030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_BIT_MODE3_EO_PL          = 0x800230030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_BIT_MODE3_EO_PL         = 0x800230030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_BIT_MODE3_EO_PL       = 0x800230030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_BIT_MODE3_EO_PL       = 0x800230030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_BIT_MODE3_EO_PL       = 0x800230030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_BIT_MODE3_EO_PL       = 0x800230030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_BIT_MODE3_EO_PL       = 0x800230030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_BIT_MODE3_EO_PL       = 0x800230030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_BIT_MODE3_EO_PL        = 0x800230030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_BIT_MODE3_EO_PL        = 0x800230030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_BIT_MODE3_EO_PL        = 0x800230030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_BIT_MODE3_EO_PL        = 0x800230030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_BIT_MODE3_EO_PL       = 0x800230030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_BIT_MODE3_EO_PL       = 0x800230030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_BIT_MODE4_EO_PL           = 0x800238030701103Full;
//DUPS: 800238030701143F, 800238030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_BIT_MODE4_EO_PL         = 0x800238030701103Full;
//DUPS: 800238030701143F, 800238030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_BIT_MODE4_EO_PL         = 0x800238030801103Full;
//DUPS: 800238030801143F, 800238030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_BIT_MODE4_EO_PL          = 0x800238030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_BIT_MODE4_EO_PL         = 0x800238030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_BIT_MODE4_EO_PL       = 0x800238030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_BIT_MODE4_EO_PL       = 0x800238030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_BIT_MODE4_EO_PL       = 0x800238030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_BIT_MODE4_EO_PL       = 0x800238030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_BIT_MODE4_EO_PL       = 0x800238030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_BIT_MODE4_EO_PL       = 0x800238030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_BIT_MODE4_EO_PL        = 0x800238030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_BIT_MODE4_EO_PL        = 0x800238030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_BIT_MODE4_EO_PL        = 0x800238030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_BIT_MODE4_EO_PL        = 0x800238030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_BIT_MODE4_EO_PL       = 0x800238030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_BIT_MODE4_EO_PL       = 0x800238030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_EO_PL           = 0x800268030701103Full;
//DUPS: 800268030701143F, 800268030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_EO_PL         = 0x800268030701103Full;
//DUPS: 800268030701143F, 800268030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_EO_PL         = 0x800268030801103Full;
//DUPS: 800268030801143F, 800268030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_EO_PL          = 0x800268030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_EO_PL         = 0x800268030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_EO_PL       = 0x800268030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_EO_PL       = 0x800268030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_EO_PL       = 0x800268030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_EO_PL       = 0x800268030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_EO_PL       = 0x800268030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_EO_PL       = 0x800268030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_EO_PL        = 0x800268030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_EO_PL        = 0x800268030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_EO_PL        = 0x800268030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_EO_PL        = 0x800268030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_EO_PL       = 0x800268030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_EO_PL       = 0x800268030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_O_PL            = 0x800318030701103Full;
//DUPS: 800318030701143F, 800318030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_O_PL          = 0x800318030701103Full;
//DUPS: 800318030701143F, 800318030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_O_PL          = 0x800318030801103Full;
//DUPS: 800318030801143F, 800318030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_O_PL           = 0x800318030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_O_PL          = 0x800318030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_O_PL        = 0x800318030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_O_PL        = 0x800318030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_O_PL        = 0x800318030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_O_PL        = 0x800318030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_O_PL        = 0x800318030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_O_PL        = 0x800318030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_O_PL         = 0x800318030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_O_PL         = 0x800318030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_O_PL         = 0x800318030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_O_PL         = 0x800318030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_O_PL        = 0x800318030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_BIT_STAT1_O_PL        = 0x800318030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_BIT_STAT2_EO_PL           = 0x800270030701103Full;
//DUPS: 800270030701143F, 800270030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_BIT_STAT2_EO_PL         = 0x800270030701103Full;
//DUPS: 800270030701143F, 800270030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_BIT_STAT2_EO_PL         = 0x800270030801103Full;
//DUPS: 800270030801143F, 800270030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_BIT_STAT2_EO_PL          = 0x800270030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_BIT_STAT2_EO_PL         = 0x800270030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_BIT_STAT2_EO_PL       = 0x800270030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_BIT_STAT2_EO_PL       = 0x800270030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_BIT_STAT2_EO_PL       = 0x800270030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_BIT_STAT2_EO_PL       = 0x800270030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_BIT_STAT2_EO_PL       = 0x800270030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_BIT_STAT2_EO_PL       = 0x800270030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_BIT_STAT2_EO_PL        = 0x800270030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_BIT_STAT2_EO_PL        = 0x800270030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_BIT_STAT2_EO_PL        = 0x800270030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_BIT_STAT2_EO_PL        = 0x800270030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_BIT_STAT2_EO_PL       = 0x800270030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_BIT_STAT2_EO_PL       = 0x800270030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_BIT_STAT3_EO_PL           = 0x800278030701103Full;
//DUPS: 800278030701143F, 800278030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_BIT_STAT3_EO_PL         = 0x800278030701103Full;
//DUPS: 800278030701143F, 800278030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_BIT_STAT3_EO_PL         = 0x800278030801103Full;
//DUPS: 800278030801143F, 800278030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_BIT_STAT3_EO_PL          = 0x800278030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_BIT_STAT3_EO_PL         = 0x800278030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_BIT_STAT3_EO_PL       = 0x800278030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_BIT_STAT3_EO_PL       = 0x800278030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_BIT_STAT3_EO_PL       = 0x800278030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_BIT_STAT3_EO_PL       = 0x800278030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_BIT_STAT3_EO_PL       = 0x800278030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_BIT_STAT3_EO_PL       = 0x800278030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_BIT_STAT3_EO_PL        = 0x800278030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_BIT_STAT3_EO_PL        = 0x800278030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_BIT_STAT3_EO_PL        = 0x800278030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_BIT_STAT3_EO_PL        = 0x800278030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_BIT_STAT3_EO_PL       = 0x800278030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_BIT_STAT3_EO_PL       = 0x800278030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_BIT_STAT4_EO_PL           = 0x800280030701103Full;
//DUPS: 800280030701143F, 800280030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_BIT_STAT4_EO_PL         = 0x800280030701103Full;
//DUPS: 800280030701143F, 800280030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_BIT_STAT4_EO_PL         = 0x800280030801103Full;
//DUPS: 800280030801143F, 800280030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_BIT_STAT4_EO_PL          = 0x800280030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_BIT_STAT4_EO_PL         = 0x800280030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_BIT_STAT4_EO_PL       = 0x800280030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_BIT_STAT4_EO_PL       = 0x800280030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_BIT_STAT4_EO_PL       = 0x800280030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_BIT_STAT4_EO_PL       = 0x800280030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_BIT_STAT4_EO_PL       = 0x800280030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_BIT_STAT4_EO_PL       = 0x800280030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_BIT_STAT4_EO_PL        = 0x800280030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_BIT_STAT4_EO_PL        = 0x800280030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_BIT_STAT4_EO_PL        = 0x800280030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_BIT_STAT4_EO_PL        = 0x800280030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_BIT_STAT4_EO_PL       = 0x800280030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_BIT_STAT4_EO_PL       = 0x800280030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL10_O_PL           = 0x8000C8030701103Full;
//DUPS: 8000C8030701143F, 8000C8030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL10_O_PL         = 0x8000C8030701103Full;
//DUPS: 8000C8030701143F, 8000C8030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL10_O_PL         = 0x8000C8030801103Full;
//DUPS: 8000C8030801143F, 8000C8030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL10_O_PL          = 0x8000C8030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL10_O_PL         = 0x8000C8030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C8030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C8030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C8030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C8030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C8030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C8030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL10_O_PL        = 0x8000C8030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL10_O_PL        = 0x8000C8030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL10_O_PL        = 0x8000C8030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL10_O_PL        = 0x8000C8030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C8030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C8030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_EO_PL           = 0x800008030701103Full;
//DUPS: 800008030701143F, 800008030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_EO_PL         = 0x800008030701103Full;
//DUPS: 800008030701143F, 800008030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_EO_PL         = 0x800008030801103Full;
//DUPS: 800008030801143F, 800008030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_EO_PL          = 0x800008030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_EO_PL         = 0x800008030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x800008030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x800008030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x800008030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x800008030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x800008030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x800008030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_EO_PL        = 0x800008030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_EO_PL        = 0x800008030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_EO_PL        = 0x800008030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_EO_PL        = 0x800008030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x800008030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x800008030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_O_PL            = 0x800080030701103Full;
//DUPS: 800080030701143F, 800080030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_O_PL          = 0x800080030701103Full;
//DUPS: 800080030701143F, 800080030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_O_PL          = 0x800080030801103Full;
//DUPS: 800080030801143F, 800080030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_O_PL           = 0x800080030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_O_PL          = 0x800080030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_O_PL        = 0x800080030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_O_PL        = 0x800080030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_O_PL        = 0x800080030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_O_PL        = 0x800080030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_O_PL        = 0x800080030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_O_PL        = 0x800080030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_O_PL         = 0x800080030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_O_PL         = 0x800080030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_O_PL         = 0x800080030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_O_PL         = 0x800080030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_O_PL        = 0x800080030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL1_O_PL        = 0x800080030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_EO_PL           = 0x800010030701103Full;
//DUPS: 800010030701143F, 800010030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_EO_PL         = 0x800010030701103Full;
//DUPS: 800010030701143F, 800010030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_EO_PL         = 0x800010030801103Full;
//DUPS: 800010030801143F, 800010030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_EO_PL          = 0x800010030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_EO_PL         = 0x800010030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x800010030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x800010030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x800010030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x800010030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x800010030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x800010030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_EO_PL        = 0x800010030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_EO_PL        = 0x800010030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_EO_PL        = 0x800010030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_EO_PL        = 0x800010030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x800010030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x800010030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_O_PL            = 0x800088030701103Full;
//DUPS: 800088030701143F, 800088030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_O_PL          = 0x800088030701103Full;
//DUPS: 800088030701143F, 800088030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_O_PL          = 0x800088030801103Full;
//DUPS: 800088030801143F, 800088030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_O_PL           = 0x800088030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_O_PL          = 0x800088030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_O_PL        = 0x800088030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_O_PL        = 0x800088030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_O_PL        = 0x800088030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_O_PL        = 0x800088030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_O_PL        = 0x800088030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_O_PL        = 0x800088030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_O_PL         = 0x800088030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_O_PL         = 0x800088030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_O_PL         = 0x800088030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_O_PL         = 0x800088030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_O_PL        = 0x800088030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL2_O_PL        = 0x800088030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_EO_PL           = 0x800018030701103Full;
//DUPS: 800018030701143F, 800018030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_EO_PL         = 0x800018030701103Full;
//DUPS: 800018030701143F, 800018030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_EO_PL         = 0x800018030801103Full;
//DUPS: 800018030801143F, 800018030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_EO_PL          = 0x800018030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_EO_PL         = 0x800018030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x800018030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x800018030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x800018030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x800018030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x800018030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x800018030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_EO_PL        = 0x800018030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_EO_PL        = 0x800018030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_EO_PL        = 0x800018030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_EO_PL        = 0x800018030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x800018030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x800018030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_O_PL            = 0x800090030701103Full;
//DUPS: 800090030701143F, 800090030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_O_PL          = 0x800090030701103Full;
//DUPS: 800090030701143F, 800090030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_O_PL          = 0x800090030801103Full;
//DUPS: 800090030801143F, 800090030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_O_PL           = 0x800090030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_O_PL          = 0x800090030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_O_PL        = 0x800090030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_O_PL        = 0x800090030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_O_PL        = 0x800090030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_O_PL        = 0x800090030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_O_PL        = 0x800090030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_O_PL        = 0x800090030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_O_PL         = 0x800090030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_O_PL         = 0x800090030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_O_PL         = 0x800090030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_O_PL         = 0x800090030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_O_PL        = 0x800090030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL3_O_PL        = 0x800090030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_EO_PL           = 0x800020030701103Full;
//DUPS: 800020030701143F, 800020030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_EO_PL         = 0x800020030701103Full;
//DUPS: 800020030701143F, 800020030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_EO_PL         = 0x800020030801103Full;
//DUPS: 800020030801143F, 800020030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_EO_PL          = 0x800020030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_EO_PL         = 0x800020030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x800020030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x800020030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x800020030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x800020030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x800020030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x800020030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_EO_PL        = 0x800020030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_EO_PL        = 0x800020030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_EO_PL        = 0x800020030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_EO_PL        = 0x800020030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x800020030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x800020030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_O_PL            = 0x800098030701103Full;
//DUPS: 800098030701143F, 800098030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_O_PL          = 0x800098030701103Full;
//DUPS: 800098030701143F, 800098030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_O_PL          = 0x800098030801103Full;
//DUPS: 800098030801143F, 800098030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_O_PL           = 0x800098030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_O_PL          = 0x800098030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_O_PL        = 0x800098030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_O_PL        = 0x800098030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_O_PL        = 0x800098030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_O_PL        = 0x800098030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_O_PL        = 0x800098030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_O_PL        = 0x800098030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_O_PL         = 0x800098030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_O_PL         = 0x800098030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_O_PL         = 0x800098030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_O_PL         = 0x800098030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_O_PL        = 0x800098030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL4_O_PL        = 0x800098030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_EO_PL           = 0x800028030701103Full;
//DUPS: 800028030701143F, 800028030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_EO_PL         = 0x800028030701103Full;
//DUPS: 800028030701143F, 800028030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_EO_PL         = 0x800028030801103Full;
//DUPS: 800028030801143F, 800028030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_EO_PL          = 0x800028030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_EO_PL         = 0x800028030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x800028030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x800028030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x800028030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x800028030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x800028030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x800028030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_EO_PL        = 0x800028030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_EO_PL        = 0x800028030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_EO_PL        = 0x800028030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_EO_PL        = 0x800028030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x800028030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x800028030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_O_PL            = 0x8000A0030701103Full;
//DUPS: 8000A0030701143F, 8000A0030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_O_PL          = 0x8000A0030701103Full;
//DUPS: 8000A0030701143F, 8000A0030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_O_PL          = 0x8000A0030801103Full;
//DUPS: 8000A0030801143F, 8000A0030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_O_PL           = 0x8000A0030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_O_PL          = 0x8000A0030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A0030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A0030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A0030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A0030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A0030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A0030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_O_PL         = 0x8000A0030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_O_PL         = 0x8000A0030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_O_PL         = 0x8000A0030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_O_PL         = 0x8000A0030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A0030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A0030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_EO_PL           = 0x800030030701103Full;
//DUPS: 800030030701143F, 800030030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_EO_PL         = 0x800030030701103Full;
//DUPS: 800030030701143F, 800030030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_EO_PL         = 0x800030030801103Full;
//DUPS: 800030030801143F, 800030030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_EO_PL          = 0x800030030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_EO_PL         = 0x800030030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x800030030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x800030030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x800030030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x800030030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x800030030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x800030030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_EO_PL        = 0x800030030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_EO_PL        = 0x800030030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_EO_PL        = 0x800030030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_EO_PL        = 0x800030030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x800030030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x800030030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_O_PL            = 0x8000A8030701103Full;
//DUPS: 8000A8030701143F, 8000A8030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_O_PL          = 0x8000A8030701103Full;
//DUPS: 8000A8030701143F, 8000A8030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_O_PL          = 0x8000A8030801103Full;
//DUPS: 8000A8030801143F, 8000A8030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_O_PL           = 0x8000A8030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_O_PL          = 0x8000A8030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A8030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A8030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A8030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A8030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A8030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A8030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_O_PL         = 0x8000A8030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_O_PL         = 0x8000A8030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_O_PL         = 0x8000A8030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_O_PL         = 0x8000A8030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A8030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A8030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_EO_PL           = 0x800038030701103Full;
//DUPS: 800038030701143F, 800038030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_EO_PL         = 0x800038030701103Full;
//DUPS: 800038030701143F, 800038030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_EO_PL         = 0x800038030801103Full;
//DUPS: 800038030801143F, 800038030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_EO_PL          = 0x800038030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_EO_PL         = 0x800038030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x800038030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x800038030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x800038030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x800038030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x800038030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x800038030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_EO_PL        = 0x800038030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_EO_PL        = 0x800038030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_EO_PL        = 0x800038030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_EO_PL        = 0x800038030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x800038030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x800038030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_O_PL            = 0x8000B0030701103Full;
//DUPS: 8000B0030701143F, 8000B0030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_O_PL          = 0x8000B0030701103Full;
//DUPS: 8000B0030701143F, 8000B0030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_O_PL          = 0x8000B0030801103Full;
//DUPS: 8000B0030801143F, 8000B0030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_O_PL           = 0x8000B0030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_O_PL          = 0x8000B0030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B0030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B0030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B0030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B0030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B0030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B0030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_O_PL         = 0x8000B0030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_O_PL         = 0x8000B0030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_O_PL         = 0x8000B0030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_O_PL         = 0x8000B0030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B0030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B0030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_EO_PL           = 0x800040030701103Full;
//DUPS: 800040030701143F, 800040030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_EO_PL         = 0x800040030701103Full;
//DUPS: 800040030701143F, 800040030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_EO_PL         = 0x800040030801103Full;
//DUPS: 800040030801143F, 800040030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_EO_PL          = 0x800040030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_EO_PL         = 0x800040030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x800040030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x800040030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x800040030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x800040030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x800040030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x800040030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_EO_PL        = 0x800040030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_EO_PL        = 0x800040030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_EO_PL        = 0x800040030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_EO_PL        = 0x800040030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x800040030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x800040030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_O_PL            = 0x8000B8030701103Full;
//DUPS: 8000B8030701143F, 8000B8030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_O_PL          = 0x8000B8030701103Full;
//DUPS: 8000B8030701143F, 8000B8030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_O_PL          = 0x8000B8030801103Full;
//DUPS: 8000B8030801143F, 8000B8030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_O_PL           = 0x8000B8030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_O_PL          = 0x8000B8030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B8030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B8030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B8030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B8030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B8030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B8030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_O_PL         = 0x8000B8030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_O_PL         = 0x8000B8030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_O_PL         = 0x8000B8030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_O_PL         = 0x8000B8030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B8030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B8030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL9_O_PL            = 0x8000C0030701103Full;
//DUPS: 8000C0030701143F, 8000C0030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL9_O_PL          = 0x8000C0030701103Full;
//DUPS: 8000C0030701143F, 8000C0030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL9_O_PL          = 0x8000C0030801103Full;
//DUPS: 8000C0030801143F, 8000C0030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL9_O_PL           = 0x8000C0030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL9_O_PL          = 0x8000C0030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C0030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C0030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C0030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C0030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C0030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C0030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL9_O_PL         = 0x8000C0030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL9_O_PL         = 0x8000C0030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL9_O_PL         = 0x8000C0030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL9_O_PL         = 0x8000C0030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C0030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C0030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_DATA_DAC_SPARE_MODE_PL    = 0x800000030701103Full;
//DUPS: 800000030701143F, 800000030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000030701103Full;
//DUPS: 800000030701143F, 800000030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000030801103Full;
//DUPS: 800000030801143F, 800000030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_DATA_DAC_SPARE_MODE_PL   = 0x800000030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_FIR_ERROR_INJECT_PL       = 0x800218030701103Full;
//DUPS: 800218030701143F, 800218030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_FIR_ERROR_INJECT_PL     = 0x800218030701103Full;
//DUPS: 800218030701143F, 800218030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_FIR_ERROR_INJECT_PL     = 0x800218030801103Full;
//DUPS: 800218030801143F, 800218030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_FIR_ERROR_INJECT_PL      = 0x800218030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_FIR_ERROR_INJECT_PL     = 0x800218030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x800218030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x800218030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x800218030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x800218030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x800218030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x800218030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_FIR_ERROR_INJECT_PL    = 0x800218030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_FIR_ERROR_INJECT_PL    = 0x800218030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_FIR_ERROR_INJECT_PL    = 0x800218030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_FIR_ERROR_INJECT_PL    = 0x800218030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x800218030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x800218030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_FIR_MASK_PL               = 0x800210030701103Full;
//DUPS: 800210030701143F, 800210030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_FIR_MASK_PL             = 0x800210030701103Full;
//DUPS: 800210030701143F, 800210030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_FIR_MASK_PL             = 0x800210030801103Full;
//DUPS: 800210030801143F, 800210030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_FIR_MASK_PL              = 0x800210030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_FIR_MASK_PL             = 0x800210030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_FIR_MASK_PL           = 0x800210030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_FIR_MASK_PL           = 0x800210030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_FIR_MASK_PL           = 0x800210030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_FIR_MASK_PL           = 0x800210030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_FIR_MASK_PL           = 0x800210030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_FIR_MASK_PL           = 0x800210030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_FIR_MASK_PL            = 0x800210030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_FIR_MASK_PL            = 0x800210030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_FIR_MASK_PL            = 0x800210030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_FIR_MASK_PL            = 0x800210030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_FIR_MASK_PL           = 0x800210030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_FIR_MASK_PL           = 0x800210030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_FIR_PL                    = 0x800208030701103Full;
//DUPS: 800208030701143F, 800208030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_FIR_PL                  = 0x800208030701103Full;
//DUPS: 800208030701143F, 800208030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_FIR_PL                  = 0x800208030801103Full;
//DUPS: 800208030801143F, 800208030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_FIR_PL                   = 0x800208030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_FIR_PL                  = 0x800208030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_FIR_PL                = 0x800208030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_FIR_PL                = 0x800208030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_FIR_PL                = 0x800208030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_FIR_PL                = 0x800208030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_FIR_PL                = 0x800208030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_FIR_PL                = 0x800208030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_FIR_PL                 = 0x800208030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_FIR_PL                 = 0x800208030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_FIR_PL                 = 0x800208030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_FIR_PL                 = 0x800208030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_FIR_PL                = 0x800208030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_FIR_PL                = 0x800208030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE3_RX_SPARE_MODE_PL             = 0x800200030701103Full;
//DUPS: 800200030701143F, 800200030701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE3_RX_SPARE_MODE_PL           = 0x800200030701103Full;
//DUPS: 800200030701143F, 800200030701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE3_RX_SPARE_MODE_PL           = 0x800200030801103Full;
//DUPS: 800200030801143F, 800200030801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS0_SLICE3_RX_SPARE_MODE_PL            = 0x800200030701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS0_SLICE3_RX_SPARE_MODE_PL           = 0x800200030701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS0_SLICE3_RX_SPARE_MODE_PL         = 0x800200030701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS0_SLICE3_RX_SPARE_MODE_PL         = 0x800200030701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE3_RX_SPARE_MODE_PL         = 0x800200030701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS0_SLICE3_RX_SPARE_MODE_PL         = 0x800200030801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS0_SLICE3_RX_SPARE_MODE_PL         = 0x800200030801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE3_RX_SPARE_MODE_PL         = 0x800200030801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS0_SLICE3_RX_SPARE_MODE_PL          = 0x800200030701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS0_SLICE3_RX_SPARE_MODE_PL          = 0x800200030701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE3_RX_SPARE_MODE_PL          = 0x800200030701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS0_SLICE3_RX_SPARE_MODE_PL          = 0x800200030801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS0_SLICE3_RX_SPARE_MODE_PL         = 0x800200030801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE3_RX_SPARE_MODE_PL         = 0x800200030801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_BIT_CNTL2_EO_PL           = 0x800248040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_BIT_CNTL2_EO_PL         = 0x800248040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_BIT_CNTL2_EO_PL         = 0x800248040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_BIT_CNTL2_EO_PL       = 0x800248040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_BIT_CNTL2_EO_PL       = 0x800248040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_BIT_CNTL2_EO_PL        = 0x800248040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_BIT_CNTL2_EO_PL       = 0x800248040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_BIT_CNTL3_EO_PL           = 0x800250040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_BIT_CNTL3_EO_PL         = 0x800250040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_BIT_CNTL3_EO_PL         = 0x800250040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_BIT_CNTL3_EO_PL       = 0x800250040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_BIT_CNTL3_EO_PL       = 0x800250040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_BIT_CNTL3_EO_PL        = 0x800250040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_BIT_CNTL3_EO_PL       = 0x800250040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_BIT_CNTL4_EO_PL           = 0x800258040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_BIT_CNTL4_EO_PL         = 0x800258040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_BIT_CNTL4_EO_PL         = 0x800258040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_BIT_CNTL4_EO_PL       = 0x800258040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_BIT_CNTL4_EO_PL       = 0x800258040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_BIT_CNTL4_EO_PL        = 0x800258040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_BIT_CNTL4_EO_PL       = 0x800258040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_BIT_CNTL5_EO_PL           = 0x800260040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_BIT_CNTL5_EO_PL         = 0x800260040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_BIT_CNTL5_EO_PL         = 0x800260040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_BIT_CNTL5_EO_PL       = 0x800260040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_BIT_CNTL5_EO_PL       = 0x800260040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_BIT_CNTL5_EO_PL        = 0x800260040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_BIT_CNTL5_EO_PL       = 0x800260040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_BIT_CNTLX1_EO_PL          = 0x800240040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_BIT_CNTLX1_EO_PL        = 0x800240040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_BIT_CNTLX1_EO_PL        = 0x800240040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_BIT_CNTLX1_EO_PL      = 0x800240040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_BIT_CNTLX1_EO_PL      = 0x800240040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_BIT_CNTLX1_EO_PL       = 0x800240040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_BIT_CNTLX1_EO_PL      = 0x800240040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_BIT_MODE1_EO_PL           = 0x800220040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_BIT_MODE1_EO_PL         = 0x800220040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_BIT_MODE1_EO_PL         = 0x800220040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_BIT_MODE1_EO_PL       = 0x800220040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_BIT_MODE1_EO_PL       = 0x800220040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_BIT_MODE1_EO_PL        = 0x800220040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_BIT_MODE1_EO_PL       = 0x800220040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_BIT_MODE2_EO_PL           = 0x800228040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_BIT_MODE2_EO_PL         = 0x800228040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_BIT_MODE2_EO_PL         = 0x800228040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_BIT_MODE2_EO_PL       = 0x800228040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_BIT_MODE2_EO_PL       = 0x800228040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_BIT_MODE2_EO_PL        = 0x800228040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_BIT_MODE2_EO_PL       = 0x800228040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_BIT_MODE3_EO_PL           = 0x800230040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_BIT_MODE3_EO_PL         = 0x800230040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_BIT_MODE3_EO_PL         = 0x800230040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_BIT_MODE3_EO_PL       = 0x800230040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_BIT_MODE3_EO_PL       = 0x800230040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_BIT_MODE3_EO_PL        = 0x800230040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_BIT_MODE3_EO_PL       = 0x800230040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_BIT_MODE4_EO_PL           = 0x800238040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_BIT_MODE4_EO_PL         = 0x800238040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_BIT_MODE4_EO_PL         = 0x800238040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_BIT_MODE4_EO_PL       = 0x800238040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_BIT_MODE4_EO_PL       = 0x800238040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_BIT_MODE4_EO_PL        = 0x800238040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_BIT_MODE4_EO_PL       = 0x800238040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_BIT_STAT1_EO_PL           = 0x800268040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_BIT_STAT1_EO_PL         = 0x800268040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_BIT_STAT1_EO_PL         = 0x800268040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_BIT_STAT1_EO_PL       = 0x800268040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_BIT_STAT1_EO_PL       = 0x800268040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_BIT_STAT1_EO_PL        = 0x800268040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_BIT_STAT1_EO_PL       = 0x800268040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_BIT_STAT1_O_PL            = 0x800318040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_BIT_STAT1_O_PL          = 0x800318040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_BIT_STAT1_O_PL          = 0x800318040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_BIT_STAT1_O_PL        = 0x800318040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_BIT_STAT1_O_PL        = 0x800318040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_BIT_STAT1_O_PL         = 0x800318040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_BIT_STAT1_O_PL        = 0x800318040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_BIT_STAT2_EO_PL           = 0x800270040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_BIT_STAT2_EO_PL         = 0x800270040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_BIT_STAT2_EO_PL         = 0x800270040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_BIT_STAT2_EO_PL       = 0x800270040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_BIT_STAT2_EO_PL       = 0x800270040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_BIT_STAT2_EO_PL        = 0x800270040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_BIT_STAT2_EO_PL       = 0x800270040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_BIT_STAT3_EO_PL           = 0x800278040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_BIT_STAT3_EO_PL         = 0x800278040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_BIT_STAT3_EO_PL         = 0x800278040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_BIT_STAT3_EO_PL       = 0x800278040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_BIT_STAT3_EO_PL       = 0x800278040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_BIT_STAT3_EO_PL        = 0x800278040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_BIT_STAT3_EO_PL       = 0x800278040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_BIT_STAT4_EO_PL           = 0x800280040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_BIT_STAT4_EO_PL         = 0x800280040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_BIT_STAT4_EO_PL         = 0x800280040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_BIT_STAT4_EO_PL       = 0x800280040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_BIT_STAT4_EO_PL       = 0x800280040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_BIT_STAT4_EO_PL        = 0x800280040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_BIT_STAT4_EO_PL       = 0x800280040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL10_O_PL           = 0x8000C8040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL10_O_PL         = 0x8000C8040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL10_O_PL         = 0x8000C8040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL10_O_PL       = 0x8000C8040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL10_O_PL       = 0x8000C8040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL10_O_PL        = 0x8000C8040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL10_O_PL       = 0x8000C8040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL1_EO_PL           = 0x800008040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL1_EO_PL         = 0x800008040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL1_EO_PL         = 0x800008040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL1_EO_PL       = 0x800008040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL1_EO_PL       = 0x800008040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL1_EO_PL        = 0x800008040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL1_EO_PL       = 0x800008040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL1_O_PL            = 0x800080040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL1_O_PL          = 0x800080040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL1_O_PL          = 0x800080040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL1_O_PL        = 0x800080040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL1_O_PL        = 0x800080040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL1_O_PL         = 0x800080040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL1_O_PL        = 0x800080040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL2_EO_PL           = 0x800010040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL2_EO_PL         = 0x800010040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL2_EO_PL         = 0x800010040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL2_EO_PL       = 0x800010040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL2_EO_PL       = 0x800010040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL2_EO_PL        = 0x800010040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL2_EO_PL       = 0x800010040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL2_O_PL            = 0x800088040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL2_O_PL          = 0x800088040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL2_O_PL          = 0x800088040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL2_O_PL        = 0x800088040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL2_O_PL        = 0x800088040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL2_O_PL         = 0x800088040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL2_O_PL        = 0x800088040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL3_EO_PL           = 0x800018040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL3_EO_PL         = 0x800018040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL3_EO_PL         = 0x800018040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL3_EO_PL       = 0x800018040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL3_EO_PL       = 0x800018040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL3_EO_PL        = 0x800018040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL3_EO_PL       = 0x800018040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL3_O_PL            = 0x800090040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL3_O_PL          = 0x800090040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL3_O_PL          = 0x800090040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL3_O_PL        = 0x800090040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL3_O_PL        = 0x800090040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL3_O_PL         = 0x800090040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL3_O_PL        = 0x800090040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL4_EO_PL           = 0x800020040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL4_EO_PL         = 0x800020040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL4_EO_PL         = 0x800020040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL4_EO_PL       = 0x800020040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL4_EO_PL       = 0x800020040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL4_EO_PL        = 0x800020040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL4_EO_PL       = 0x800020040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL4_O_PL            = 0x800098040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL4_O_PL          = 0x800098040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL4_O_PL          = 0x800098040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL4_O_PL        = 0x800098040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL4_O_PL        = 0x800098040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL4_O_PL         = 0x800098040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL4_O_PL        = 0x800098040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL5_EO_PL           = 0x800028040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL5_EO_PL         = 0x800028040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL5_EO_PL         = 0x800028040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL5_EO_PL       = 0x800028040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL5_EO_PL       = 0x800028040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL5_EO_PL        = 0x800028040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL5_EO_PL       = 0x800028040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL5_O_PL            = 0x8000A0040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL5_O_PL          = 0x8000A0040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL5_O_PL          = 0x8000A0040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL5_O_PL        = 0x8000A0040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL5_O_PL        = 0x8000A0040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL5_O_PL         = 0x8000A0040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL5_O_PL        = 0x8000A0040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL6_EO_PL           = 0x800030040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL6_EO_PL         = 0x800030040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL6_EO_PL         = 0x800030040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL6_EO_PL       = 0x800030040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL6_EO_PL       = 0x800030040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL6_EO_PL        = 0x800030040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL6_EO_PL       = 0x800030040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL6_O_PL            = 0x8000A8040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL6_O_PL          = 0x8000A8040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL6_O_PL          = 0x8000A8040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL6_O_PL        = 0x8000A8040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL6_O_PL        = 0x8000A8040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL6_O_PL         = 0x8000A8040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL6_O_PL        = 0x8000A8040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL7_EO_PL           = 0x800038040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL7_EO_PL         = 0x800038040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL7_EO_PL         = 0x800038040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL7_EO_PL       = 0x800038040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL7_EO_PL       = 0x800038040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL7_EO_PL        = 0x800038040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL7_EO_PL       = 0x800038040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL7_O_PL            = 0x8000B0040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL7_O_PL          = 0x8000B0040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL7_O_PL          = 0x8000B0040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL7_O_PL        = 0x8000B0040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL7_O_PL        = 0x8000B0040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL7_O_PL         = 0x8000B0040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL7_O_PL        = 0x8000B0040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL8_EO_PL           = 0x800040040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL8_EO_PL         = 0x800040040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL8_EO_PL         = 0x800040040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL8_EO_PL       = 0x800040040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL8_EO_PL       = 0x800040040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL8_EO_PL        = 0x800040040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL8_EO_PL       = 0x800040040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL8_O_PL            = 0x8000B8040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL8_O_PL          = 0x8000B8040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL8_O_PL          = 0x8000B8040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL8_O_PL        = 0x8000B8040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL8_O_PL        = 0x8000B8040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL8_O_PL         = 0x8000B8040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL8_O_PL        = 0x8000B8040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL9_O_PL            = 0x8000C0040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL9_O_PL          = 0x8000C0040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL9_O_PL          = 0x8000C0040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL9_O_PL        = 0x8000C0040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL9_O_PL        = 0x8000C0040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL9_O_PL         = 0x8000C0040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_DAC_CNTL9_O_PL        = 0x8000C0040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_DATA_DAC_SPARE_MODE_PL    = 0x800000040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_DATA_DAC_SPARE_MODE_PL = 0x800000040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_DATA_DAC_SPARE_MODE_PL = 0x800000040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_DATA_DAC_SPARE_MODE_PL = 0x800000040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_DATA_DAC_SPARE_MODE_PL = 0x800000040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_FIR_ERROR_INJECT_PL       = 0x800218040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_FIR_ERROR_INJECT_PL     = 0x800218040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_FIR_ERROR_INJECT_PL     = 0x800218040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_FIR_ERROR_INJECT_PL   = 0x800218040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_FIR_ERROR_INJECT_PL   = 0x800218040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_FIR_ERROR_INJECT_PL    = 0x800218040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_FIR_ERROR_INJECT_PL   = 0x800218040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_FIR_MASK_PL               = 0x800210040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_FIR_MASK_PL             = 0x800210040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_FIR_MASK_PL             = 0x800210040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_FIR_MASK_PL           = 0x800210040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_FIR_MASK_PL           = 0x800210040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_FIR_MASK_PL            = 0x800210040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_FIR_MASK_PL           = 0x800210040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_FIR_PL                    = 0x800208040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_FIR_PL                  = 0x800208040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_FIR_PL                  = 0x800208040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_FIR_PL                = 0x800208040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_FIR_PL                = 0x800208040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_FIR_PL                 = 0x800208040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_FIR_PL                = 0x800208040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE4_RX_SPARE_MODE_PL             = 0x800200040701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE4_RX_SPARE_MODE_PL           = 0x800200040701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE4_RX_SPARE_MODE_PL           = 0x800200040801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE4_RX_SPARE_MODE_PL         = 0x800200040701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE4_RX_SPARE_MODE_PL         = 0x800200040801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE4_RX_SPARE_MODE_PL          = 0x800200040701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE4_RX_SPARE_MODE_PL         = 0x800200040801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_BIT_CNTL2_EO_PL           = 0x800248050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_BIT_CNTL2_EO_PL         = 0x800248050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_BIT_CNTL2_EO_PL         = 0x800248050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_BIT_CNTL2_EO_PL       = 0x800248050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_BIT_CNTL2_EO_PL       = 0x800248050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_BIT_CNTL2_EO_PL        = 0x800248050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_BIT_CNTL2_EO_PL       = 0x800248050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_BIT_CNTL3_EO_PL           = 0x800250050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_BIT_CNTL3_EO_PL         = 0x800250050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_BIT_CNTL3_EO_PL         = 0x800250050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_BIT_CNTL3_EO_PL       = 0x800250050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_BIT_CNTL3_EO_PL       = 0x800250050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_BIT_CNTL3_EO_PL        = 0x800250050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_BIT_CNTL3_EO_PL       = 0x800250050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_BIT_CNTL4_EO_PL           = 0x800258050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_BIT_CNTL4_EO_PL         = 0x800258050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_BIT_CNTL4_EO_PL         = 0x800258050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_BIT_CNTL4_EO_PL       = 0x800258050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_BIT_CNTL4_EO_PL       = 0x800258050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_BIT_CNTL4_EO_PL        = 0x800258050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_BIT_CNTL4_EO_PL       = 0x800258050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_BIT_CNTL5_EO_PL           = 0x800260050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_BIT_CNTL5_EO_PL         = 0x800260050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_BIT_CNTL5_EO_PL         = 0x800260050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_BIT_CNTL5_EO_PL       = 0x800260050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_BIT_CNTL5_EO_PL       = 0x800260050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_BIT_CNTL5_EO_PL        = 0x800260050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_BIT_CNTL5_EO_PL       = 0x800260050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_BIT_CNTLX1_EO_PL          = 0x800240050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_BIT_CNTLX1_EO_PL        = 0x800240050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_BIT_CNTLX1_EO_PL        = 0x800240050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_BIT_CNTLX1_EO_PL      = 0x800240050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_BIT_CNTLX1_EO_PL      = 0x800240050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_BIT_CNTLX1_EO_PL       = 0x800240050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_BIT_CNTLX1_EO_PL      = 0x800240050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_BIT_MODE1_EO_PL           = 0x800220050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_BIT_MODE1_EO_PL         = 0x800220050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_BIT_MODE1_EO_PL         = 0x800220050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_BIT_MODE1_EO_PL       = 0x800220050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_BIT_MODE1_EO_PL       = 0x800220050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_BIT_MODE1_EO_PL        = 0x800220050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_BIT_MODE1_EO_PL       = 0x800220050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_BIT_MODE2_EO_PL           = 0x800228050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_BIT_MODE2_EO_PL         = 0x800228050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_BIT_MODE2_EO_PL         = 0x800228050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_BIT_MODE2_EO_PL       = 0x800228050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_BIT_MODE2_EO_PL       = 0x800228050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_BIT_MODE2_EO_PL        = 0x800228050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_BIT_MODE2_EO_PL       = 0x800228050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_BIT_MODE3_EO_PL           = 0x800230050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_BIT_MODE3_EO_PL         = 0x800230050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_BIT_MODE3_EO_PL         = 0x800230050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_BIT_MODE3_EO_PL       = 0x800230050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_BIT_MODE3_EO_PL       = 0x800230050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_BIT_MODE3_EO_PL        = 0x800230050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_BIT_MODE3_EO_PL       = 0x800230050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_BIT_MODE4_EO_PL           = 0x800238050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_BIT_MODE4_EO_PL         = 0x800238050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_BIT_MODE4_EO_PL         = 0x800238050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_BIT_MODE4_EO_PL       = 0x800238050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_BIT_MODE4_EO_PL       = 0x800238050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_BIT_MODE4_EO_PL        = 0x800238050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_BIT_MODE4_EO_PL       = 0x800238050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_BIT_STAT1_EO_PL           = 0x800268050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_BIT_STAT1_EO_PL         = 0x800268050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_BIT_STAT1_EO_PL         = 0x800268050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_BIT_STAT1_EO_PL       = 0x800268050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_BIT_STAT1_EO_PL       = 0x800268050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_BIT_STAT1_EO_PL        = 0x800268050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_BIT_STAT1_EO_PL       = 0x800268050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_BIT_STAT1_O_PL            = 0x800318050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_BIT_STAT1_O_PL          = 0x800318050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_BIT_STAT1_O_PL          = 0x800318050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_BIT_STAT1_O_PL        = 0x800318050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_BIT_STAT1_O_PL        = 0x800318050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_BIT_STAT1_O_PL         = 0x800318050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_BIT_STAT1_O_PL        = 0x800318050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_BIT_STAT2_EO_PL           = 0x800270050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_BIT_STAT2_EO_PL         = 0x800270050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_BIT_STAT2_EO_PL         = 0x800270050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_BIT_STAT2_EO_PL       = 0x800270050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_BIT_STAT2_EO_PL       = 0x800270050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_BIT_STAT2_EO_PL        = 0x800270050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_BIT_STAT2_EO_PL       = 0x800270050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_BIT_STAT3_EO_PL           = 0x800278050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_BIT_STAT3_EO_PL         = 0x800278050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_BIT_STAT3_EO_PL         = 0x800278050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_BIT_STAT3_EO_PL       = 0x800278050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_BIT_STAT3_EO_PL       = 0x800278050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_BIT_STAT3_EO_PL        = 0x800278050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_BIT_STAT3_EO_PL       = 0x800278050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_BIT_STAT4_EO_PL           = 0x800280050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_BIT_STAT4_EO_PL         = 0x800280050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_BIT_STAT4_EO_PL         = 0x800280050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_BIT_STAT4_EO_PL       = 0x800280050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_BIT_STAT4_EO_PL       = 0x800280050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_BIT_STAT4_EO_PL        = 0x800280050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_BIT_STAT4_EO_PL       = 0x800280050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL10_O_PL           = 0x8000C8050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL10_O_PL         = 0x8000C8050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL10_O_PL         = 0x8000C8050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL10_O_PL       = 0x8000C8050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL10_O_PL       = 0x8000C8050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL10_O_PL        = 0x8000C8050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL10_O_PL       = 0x8000C8050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL1_EO_PL           = 0x800008050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL1_EO_PL         = 0x800008050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL1_EO_PL         = 0x800008050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL1_EO_PL       = 0x800008050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL1_EO_PL       = 0x800008050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL1_EO_PL        = 0x800008050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL1_EO_PL       = 0x800008050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL1_O_PL            = 0x800080050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL1_O_PL          = 0x800080050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL1_O_PL          = 0x800080050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL1_O_PL        = 0x800080050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL1_O_PL        = 0x800080050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL1_O_PL         = 0x800080050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL1_O_PL        = 0x800080050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL2_EO_PL           = 0x800010050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL2_EO_PL         = 0x800010050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL2_EO_PL         = 0x800010050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL2_EO_PL       = 0x800010050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL2_EO_PL       = 0x800010050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL2_EO_PL        = 0x800010050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL2_EO_PL       = 0x800010050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL2_O_PL            = 0x800088050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL2_O_PL          = 0x800088050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL2_O_PL          = 0x800088050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL2_O_PL        = 0x800088050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL2_O_PL        = 0x800088050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL2_O_PL         = 0x800088050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL2_O_PL        = 0x800088050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL3_EO_PL           = 0x800018050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL3_EO_PL         = 0x800018050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL3_EO_PL         = 0x800018050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL3_EO_PL       = 0x800018050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL3_EO_PL       = 0x800018050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL3_EO_PL        = 0x800018050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL3_EO_PL       = 0x800018050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL3_O_PL            = 0x800090050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL3_O_PL          = 0x800090050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL3_O_PL          = 0x800090050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL3_O_PL        = 0x800090050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL3_O_PL        = 0x800090050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL3_O_PL         = 0x800090050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL3_O_PL        = 0x800090050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL4_EO_PL           = 0x800020050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL4_EO_PL         = 0x800020050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL4_EO_PL         = 0x800020050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL4_EO_PL       = 0x800020050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL4_EO_PL       = 0x800020050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL4_EO_PL        = 0x800020050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL4_EO_PL       = 0x800020050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL4_O_PL            = 0x800098050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL4_O_PL          = 0x800098050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL4_O_PL          = 0x800098050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL4_O_PL        = 0x800098050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL4_O_PL        = 0x800098050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL4_O_PL         = 0x800098050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL4_O_PL        = 0x800098050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL5_EO_PL           = 0x800028050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL5_EO_PL         = 0x800028050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL5_EO_PL         = 0x800028050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL5_EO_PL       = 0x800028050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL5_EO_PL       = 0x800028050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL5_EO_PL        = 0x800028050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL5_EO_PL       = 0x800028050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL5_O_PL            = 0x8000A0050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL5_O_PL          = 0x8000A0050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL5_O_PL          = 0x8000A0050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL5_O_PL        = 0x8000A0050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL5_O_PL        = 0x8000A0050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL5_O_PL         = 0x8000A0050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL5_O_PL        = 0x8000A0050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL6_EO_PL           = 0x800030050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL6_EO_PL         = 0x800030050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL6_EO_PL         = 0x800030050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL6_EO_PL       = 0x800030050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL6_EO_PL       = 0x800030050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL6_EO_PL        = 0x800030050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL6_EO_PL       = 0x800030050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL6_O_PL            = 0x8000A8050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL6_O_PL          = 0x8000A8050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL6_O_PL          = 0x8000A8050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL6_O_PL        = 0x8000A8050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL6_O_PL        = 0x8000A8050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL6_O_PL         = 0x8000A8050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL6_O_PL        = 0x8000A8050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL7_EO_PL           = 0x800038050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL7_EO_PL         = 0x800038050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL7_EO_PL         = 0x800038050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL7_EO_PL       = 0x800038050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL7_EO_PL       = 0x800038050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL7_EO_PL        = 0x800038050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL7_EO_PL       = 0x800038050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL7_O_PL            = 0x8000B0050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL7_O_PL          = 0x8000B0050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL7_O_PL          = 0x8000B0050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL7_O_PL        = 0x8000B0050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL7_O_PL        = 0x8000B0050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL7_O_PL         = 0x8000B0050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL7_O_PL        = 0x8000B0050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL8_EO_PL           = 0x800040050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL8_EO_PL         = 0x800040050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL8_EO_PL         = 0x800040050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL8_EO_PL       = 0x800040050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL8_EO_PL       = 0x800040050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL8_EO_PL        = 0x800040050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL8_EO_PL       = 0x800040050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL8_O_PL            = 0x8000B8050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL8_O_PL          = 0x8000B8050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL8_O_PL          = 0x8000B8050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL8_O_PL        = 0x8000B8050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL8_O_PL        = 0x8000B8050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL8_O_PL         = 0x8000B8050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL8_O_PL        = 0x8000B8050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL9_O_PL            = 0x8000C0050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL9_O_PL          = 0x8000C0050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL9_O_PL          = 0x8000C0050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL9_O_PL        = 0x8000C0050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL9_O_PL        = 0x8000C0050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL9_O_PL         = 0x8000C0050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_DAC_CNTL9_O_PL        = 0x8000C0050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_DATA_DAC_SPARE_MODE_PL    = 0x800000050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_DATA_DAC_SPARE_MODE_PL = 0x800000050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_DATA_DAC_SPARE_MODE_PL = 0x800000050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_DATA_DAC_SPARE_MODE_PL = 0x800000050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_DATA_DAC_SPARE_MODE_PL = 0x800000050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_FIR_ERROR_INJECT_PL       = 0x800218050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_FIR_ERROR_INJECT_PL     = 0x800218050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_FIR_ERROR_INJECT_PL     = 0x800218050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_FIR_ERROR_INJECT_PL   = 0x800218050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_FIR_ERROR_INJECT_PL   = 0x800218050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_FIR_ERROR_INJECT_PL    = 0x800218050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_FIR_ERROR_INJECT_PL   = 0x800218050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_FIR_MASK_PL               = 0x800210050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_FIR_MASK_PL             = 0x800210050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_FIR_MASK_PL             = 0x800210050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_FIR_MASK_PL           = 0x800210050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_FIR_MASK_PL           = 0x800210050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_FIR_MASK_PL            = 0x800210050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_FIR_MASK_PL           = 0x800210050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_FIR_PL                    = 0x800208050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_FIR_PL                  = 0x800208050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_FIR_PL                  = 0x800208050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_FIR_PL                = 0x800208050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_FIR_PL                = 0x800208050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_FIR_PL                 = 0x800208050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_FIR_PL                = 0x800208050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS0_SLICE5_RX_SPARE_MODE_PL             = 0x800200050701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS0_SLICE5_RX_SPARE_MODE_PL           = 0x800200050701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS0_SLICE5_RX_SPARE_MODE_PL           = 0x800200050801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS0_SLICE5_RX_SPARE_MODE_PL         = 0x800200050701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS0_SLICE5_RX_SPARE_MODE_PL         = 0x800200050801183Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS0_SLICE5_RX_SPARE_MODE_PL          = 0x800200050701183Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS0_SLICE5_RX_SPARE_MODE_PL         = 0x800200050801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL2_EO_PL           = 0x800248040701103Full;
//DUPS: 800248040701143F, 800248060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL2_EO_PL         = 0x800248040701103Full;
//DUPS: 800248040701143F, 800248060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL2_EO_PL         = 0x800248040801103Full;
//DUPS: 800248040801143F, 800248060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL2_EO_PL          = 0x800248040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL2_EO_PL         = 0x800248040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x800248040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x800248040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x800248060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x800248040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x800248040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x800248060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL2_EO_PL        = 0x800248040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL2_EO_PL        = 0x800248040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL2_EO_PL        = 0x800248060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL2_EO_PL        = 0x800248040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x800248040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x800248060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL3_EO_PL           = 0x800250040701103Full;
//DUPS: 800250040701143F, 800250060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL3_EO_PL         = 0x800250040701103Full;
//DUPS: 800250040701143F, 800250060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL3_EO_PL         = 0x800250040801103Full;
//DUPS: 800250040801143F, 800250060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL3_EO_PL          = 0x800250040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL3_EO_PL         = 0x800250040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x800250040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x800250040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x800250060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x800250040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x800250040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x800250060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL3_EO_PL        = 0x800250040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL3_EO_PL        = 0x800250040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL3_EO_PL        = 0x800250060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL3_EO_PL        = 0x800250040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x800250040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x800250060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL4_EO_PL           = 0x800258040701103Full;
//DUPS: 800258040701143F, 800258060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL4_EO_PL         = 0x800258040701103Full;
//DUPS: 800258040701143F, 800258060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL4_EO_PL         = 0x800258040801103Full;
//DUPS: 800258040801143F, 800258060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL4_EO_PL          = 0x800258040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL4_EO_PL         = 0x800258040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x800258040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x800258040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x800258060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x800258040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x800258040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x800258060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL4_EO_PL        = 0x800258040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL4_EO_PL        = 0x800258040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL4_EO_PL        = 0x800258060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL4_EO_PL        = 0x800258040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x800258040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x800258060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL5_EO_PL           = 0x800260040701103Full;
//DUPS: 800260040701143F, 800260060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL5_EO_PL         = 0x800260040701103Full;
//DUPS: 800260040701143F, 800260060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL5_EO_PL         = 0x800260040801103Full;
//DUPS: 800260040801143F, 800260060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL5_EO_PL          = 0x800260040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL5_EO_PL         = 0x800260040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x800260040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x800260040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x800260060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x800260040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x800260040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x800260060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL5_EO_PL        = 0x800260040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL5_EO_PL        = 0x800260040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL5_EO_PL        = 0x800260060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL5_EO_PL        = 0x800260040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x800260040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x800260060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_BIT_CNTLX1_EO_PL          = 0x800240040701103Full;
//DUPS: 800240040701143F, 800240060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_BIT_CNTLX1_EO_PL        = 0x800240040701103Full;
//DUPS: 800240040701143F, 800240060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_BIT_CNTLX1_EO_PL        = 0x800240040801103Full;
//DUPS: 800240040801143F, 800240060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_BIT_CNTLX1_EO_PL         = 0x800240040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_BIT_CNTLX1_EO_PL        = 0x800240040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x800240040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x800240040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x800240060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x800240040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x800240040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x800240060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_BIT_CNTLX1_EO_PL       = 0x800240040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_BIT_CNTLX1_EO_PL       = 0x800240040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_BIT_CNTLX1_EO_PL       = 0x800240060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_BIT_CNTLX1_EO_PL       = 0x800240040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x800240040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x800240060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_BIT_MODE1_EO_PL           = 0x800220040701103Full;
//DUPS: 800220040701143F, 800220060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_BIT_MODE1_EO_PL         = 0x800220040701103Full;
//DUPS: 800220040701143F, 800220060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_BIT_MODE1_EO_PL         = 0x800220040801103Full;
//DUPS: 800220040801143F, 800220060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_BIT_MODE1_EO_PL          = 0x800220040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_BIT_MODE1_EO_PL         = 0x800220040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_BIT_MODE1_EO_PL       = 0x800220040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_BIT_MODE1_EO_PL       = 0x800220040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_BIT_MODE1_EO_PL       = 0x800220060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_BIT_MODE1_EO_PL       = 0x800220040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_BIT_MODE1_EO_PL       = 0x800220040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_BIT_MODE1_EO_PL       = 0x800220060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_BIT_MODE1_EO_PL        = 0x800220040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_BIT_MODE1_EO_PL        = 0x800220040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_BIT_MODE1_EO_PL        = 0x800220060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_BIT_MODE1_EO_PL        = 0x800220040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_BIT_MODE1_EO_PL       = 0x800220040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_BIT_MODE1_EO_PL       = 0x800220060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_BIT_MODE2_EO_PL           = 0x800228040701103Full;
//DUPS: 800228040701143F, 800228060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_BIT_MODE2_EO_PL         = 0x800228040701103Full;
//DUPS: 800228040701143F, 800228060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_BIT_MODE2_EO_PL         = 0x800228040801103Full;
//DUPS: 800228040801143F, 800228060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_BIT_MODE2_EO_PL          = 0x800228040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_BIT_MODE2_EO_PL         = 0x800228040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_BIT_MODE2_EO_PL       = 0x800228040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_BIT_MODE2_EO_PL       = 0x800228040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_BIT_MODE2_EO_PL       = 0x800228060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_BIT_MODE2_EO_PL       = 0x800228040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_BIT_MODE2_EO_PL       = 0x800228040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_BIT_MODE2_EO_PL       = 0x800228060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_BIT_MODE2_EO_PL        = 0x800228040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_BIT_MODE2_EO_PL        = 0x800228040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_BIT_MODE2_EO_PL        = 0x800228060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_BIT_MODE2_EO_PL        = 0x800228040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_BIT_MODE2_EO_PL       = 0x800228040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_BIT_MODE2_EO_PL       = 0x800228060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_BIT_MODE3_EO_PL           = 0x800230040701103Full;
//DUPS: 800230040701143F, 800230060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_BIT_MODE3_EO_PL         = 0x800230040701103Full;
//DUPS: 800230040701143F, 800230060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_BIT_MODE3_EO_PL         = 0x800230040801103Full;
//DUPS: 800230040801143F, 800230060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_BIT_MODE3_EO_PL          = 0x800230040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_BIT_MODE3_EO_PL         = 0x800230040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_BIT_MODE3_EO_PL       = 0x800230040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_BIT_MODE3_EO_PL       = 0x800230040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_BIT_MODE3_EO_PL       = 0x800230060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_BIT_MODE3_EO_PL       = 0x800230040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_BIT_MODE3_EO_PL       = 0x800230040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_BIT_MODE3_EO_PL       = 0x800230060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_BIT_MODE3_EO_PL        = 0x800230040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_BIT_MODE3_EO_PL        = 0x800230040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_BIT_MODE3_EO_PL        = 0x800230060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_BIT_MODE3_EO_PL        = 0x800230040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_BIT_MODE3_EO_PL       = 0x800230040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_BIT_MODE3_EO_PL       = 0x800230060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_BIT_MODE4_EO_PL           = 0x800238040701103Full;
//DUPS: 800238040701143F, 800238060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_BIT_MODE4_EO_PL         = 0x800238040701103Full;
//DUPS: 800238040701143F, 800238060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_BIT_MODE4_EO_PL         = 0x800238040801103Full;
//DUPS: 800238040801143F, 800238060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_BIT_MODE4_EO_PL          = 0x800238040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_BIT_MODE4_EO_PL         = 0x800238040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_BIT_MODE4_EO_PL       = 0x800238040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_BIT_MODE4_EO_PL       = 0x800238040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_BIT_MODE4_EO_PL       = 0x800238060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_BIT_MODE4_EO_PL       = 0x800238040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_BIT_MODE4_EO_PL       = 0x800238040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_BIT_MODE4_EO_PL       = 0x800238060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_BIT_MODE4_EO_PL        = 0x800238040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_BIT_MODE4_EO_PL        = 0x800238040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_BIT_MODE4_EO_PL        = 0x800238060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_BIT_MODE4_EO_PL        = 0x800238040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_BIT_MODE4_EO_PL       = 0x800238040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_BIT_MODE4_EO_PL       = 0x800238060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_EO_PL           = 0x800268040701103Full;
//DUPS: 800268040701143F, 800268060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_EO_PL         = 0x800268040701103Full;
//DUPS: 800268040701143F, 800268060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_EO_PL         = 0x800268040801103Full;
//DUPS: 800268040801143F, 800268060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_EO_PL          = 0x800268040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_EO_PL         = 0x800268040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_EO_PL       = 0x800268040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_EO_PL       = 0x800268040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_EO_PL       = 0x800268060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_EO_PL       = 0x800268040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_EO_PL       = 0x800268040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_EO_PL       = 0x800268060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_EO_PL        = 0x800268040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_EO_PL        = 0x800268040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_EO_PL        = 0x800268060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_EO_PL        = 0x800268040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_EO_PL       = 0x800268040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_EO_PL       = 0x800268060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_O_PL            = 0x800318040701103Full;
//DUPS: 800318040701143F, 800318060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_O_PL          = 0x800318040701103Full;
//DUPS: 800318040701143F, 800318060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_O_PL          = 0x800318040801103Full;
//DUPS: 800318040801143F, 800318060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_O_PL           = 0x800318040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_O_PL          = 0x800318040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_O_PL        = 0x800318040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_O_PL        = 0x800318040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_O_PL        = 0x800318060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_O_PL        = 0x800318040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_O_PL        = 0x800318040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_O_PL        = 0x800318060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_O_PL         = 0x800318040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_O_PL         = 0x800318040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_O_PL         = 0x800318060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_O_PL         = 0x800318040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_O_PL        = 0x800318040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_BIT_STAT1_O_PL        = 0x800318060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_BIT_STAT2_EO_PL           = 0x800270040701103Full;
//DUPS: 800270040701143F, 800270060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_BIT_STAT2_EO_PL         = 0x800270040701103Full;
//DUPS: 800270040701143F, 800270060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_BIT_STAT2_EO_PL         = 0x800270040801103Full;
//DUPS: 800270040801143F, 800270060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_BIT_STAT2_EO_PL          = 0x800270040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_BIT_STAT2_EO_PL         = 0x800270040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_BIT_STAT2_EO_PL       = 0x800270040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_BIT_STAT2_EO_PL       = 0x800270040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_BIT_STAT2_EO_PL       = 0x800270060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_BIT_STAT2_EO_PL       = 0x800270040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_BIT_STAT2_EO_PL       = 0x800270040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_BIT_STAT2_EO_PL       = 0x800270060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_BIT_STAT2_EO_PL        = 0x800270040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_BIT_STAT2_EO_PL        = 0x800270040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_BIT_STAT2_EO_PL        = 0x800270060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_BIT_STAT2_EO_PL        = 0x800270040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_BIT_STAT2_EO_PL       = 0x800270040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_BIT_STAT2_EO_PL       = 0x800270060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_BIT_STAT3_EO_PL           = 0x800278040701103Full;
//DUPS: 800278040701143F, 800278060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_BIT_STAT3_EO_PL         = 0x800278040701103Full;
//DUPS: 800278040701143F, 800278060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_BIT_STAT3_EO_PL         = 0x800278040801103Full;
//DUPS: 800278040801143F, 800278060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_BIT_STAT3_EO_PL          = 0x800278040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_BIT_STAT3_EO_PL         = 0x800278040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_BIT_STAT3_EO_PL       = 0x800278040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_BIT_STAT3_EO_PL       = 0x800278040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_BIT_STAT3_EO_PL       = 0x800278060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_BIT_STAT3_EO_PL       = 0x800278040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_BIT_STAT3_EO_PL       = 0x800278040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_BIT_STAT3_EO_PL       = 0x800278060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_BIT_STAT3_EO_PL        = 0x800278040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_BIT_STAT3_EO_PL        = 0x800278040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_BIT_STAT3_EO_PL        = 0x800278060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_BIT_STAT3_EO_PL        = 0x800278040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_BIT_STAT3_EO_PL       = 0x800278040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_BIT_STAT3_EO_PL       = 0x800278060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_BIT_STAT4_EO_PL           = 0x800280040701103Full;
//DUPS: 800280040701143F, 800280060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_BIT_STAT4_EO_PL         = 0x800280040701103Full;
//DUPS: 800280040701143F, 800280060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_BIT_STAT4_EO_PL         = 0x800280040801103Full;
//DUPS: 800280040801143F, 800280060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_BIT_STAT4_EO_PL          = 0x800280040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_BIT_STAT4_EO_PL         = 0x800280040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_BIT_STAT4_EO_PL       = 0x800280040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_BIT_STAT4_EO_PL       = 0x800280040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_BIT_STAT4_EO_PL       = 0x800280060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_BIT_STAT4_EO_PL       = 0x800280040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_BIT_STAT4_EO_PL       = 0x800280040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_BIT_STAT4_EO_PL       = 0x800280060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_BIT_STAT4_EO_PL        = 0x800280040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_BIT_STAT4_EO_PL        = 0x800280040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_BIT_STAT4_EO_PL        = 0x800280060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_BIT_STAT4_EO_PL        = 0x800280040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_BIT_STAT4_EO_PL       = 0x800280040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_BIT_STAT4_EO_PL       = 0x800280060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL10_O_PL           = 0x8000C8040701103Full;
//DUPS: 8000C8040701143F, 8000C8060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL10_O_PL         = 0x8000C8040701103Full;
//DUPS: 8000C8040701143F, 8000C8060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL10_O_PL         = 0x8000C8040801103Full;
//DUPS: 8000C8040801143F, 8000C8060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL10_O_PL          = 0x8000C8040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL10_O_PL         = 0x8000C8040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C8040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C8040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C8060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C8040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C8040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C8060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL10_O_PL        = 0x8000C8040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL10_O_PL        = 0x8000C8040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL10_O_PL        = 0x8000C8060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL10_O_PL        = 0x8000C8040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C8040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C8060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_EO_PL           = 0x800008040701103Full;
//DUPS: 800008040701143F, 800008060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_EO_PL         = 0x800008040701103Full;
//DUPS: 800008040701143F, 800008060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_EO_PL         = 0x800008040801103Full;
//DUPS: 800008040801143F, 800008060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_EO_PL          = 0x800008040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_EO_PL         = 0x800008040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x800008040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x800008040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x800008060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x800008040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x800008040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x800008060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_EO_PL        = 0x800008040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_EO_PL        = 0x800008040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_EO_PL        = 0x800008060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_EO_PL        = 0x800008040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x800008040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x800008060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_O_PL            = 0x800080040701103Full;
//DUPS: 800080040701143F, 800080060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_O_PL          = 0x800080040701103Full;
//DUPS: 800080040701143F, 800080060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_O_PL          = 0x800080040801103Full;
//DUPS: 800080040801143F, 800080060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_O_PL           = 0x800080040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_O_PL          = 0x800080040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_O_PL        = 0x800080040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_O_PL        = 0x800080040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_O_PL        = 0x800080060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_O_PL        = 0x800080040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_O_PL        = 0x800080040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_O_PL        = 0x800080060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_O_PL         = 0x800080040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_O_PL         = 0x800080040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_O_PL         = 0x800080060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_O_PL         = 0x800080040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_O_PL        = 0x800080040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL1_O_PL        = 0x800080060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_EO_PL           = 0x800010040701103Full;
//DUPS: 800010040701143F, 800010060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_EO_PL         = 0x800010040701103Full;
//DUPS: 800010040701143F, 800010060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_EO_PL         = 0x800010040801103Full;
//DUPS: 800010040801143F, 800010060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_EO_PL          = 0x800010040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_EO_PL         = 0x800010040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x800010040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x800010040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x800010060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x800010040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x800010040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x800010060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_EO_PL        = 0x800010040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_EO_PL        = 0x800010040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_EO_PL        = 0x800010060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_EO_PL        = 0x800010040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x800010040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x800010060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_O_PL            = 0x800088040701103Full;
//DUPS: 800088040701143F, 800088060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_O_PL          = 0x800088040701103Full;
//DUPS: 800088040701143F, 800088060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_O_PL          = 0x800088040801103Full;
//DUPS: 800088040801143F, 800088060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_O_PL           = 0x800088040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_O_PL          = 0x800088040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_O_PL        = 0x800088040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_O_PL        = 0x800088040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_O_PL        = 0x800088060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_O_PL        = 0x800088040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_O_PL        = 0x800088040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_O_PL        = 0x800088060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_O_PL         = 0x800088040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_O_PL         = 0x800088040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_O_PL         = 0x800088060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_O_PL         = 0x800088040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_O_PL        = 0x800088040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL2_O_PL        = 0x800088060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_EO_PL           = 0x800018040701103Full;
//DUPS: 800018040701143F, 800018060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_EO_PL         = 0x800018040701103Full;
//DUPS: 800018040701143F, 800018060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_EO_PL         = 0x800018040801103Full;
//DUPS: 800018040801143F, 800018060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_EO_PL          = 0x800018040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_EO_PL         = 0x800018040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x800018040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x800018040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x800018060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x800018040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x800018040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x800018060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_EO_PL        = 0x800018040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_EO_PL        = 0x800018040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_EO_PL        = 0x800018060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_EO_PL        = 0x800018040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x800018040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x800018060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_O_PL            = 0x800090040701103Full;
//DUPS: 800090040701143F, 800090060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_O_PL          = 0x800090040701103Full;
//DUPS: 800090040701143F, 800090060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_O_PL          = 0x800090040801103Full;
//DUPS: 800090040801143F, 800090060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_O_PL           = 0x800090040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_O_PL          = 0x800090040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_O_PL        = 0x800090040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_O_PL        = 0x800090040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_O_PL        = 0x800090060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_O_PL        = 0x800090040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_O_PL        = 0x800090040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_O_PL        = 0x800090060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_O_PL         = 0x800090040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_O_PL         = 0x800090040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_O_PL         = 0x800090060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_O_PL         = 0x800090040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_O_PL        = 0x800090040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL3_O_PL        = 0x800090060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_EO_PL           = 0x800020040701103Full;
//DUPS: 800020040701143F, 800020060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_EO_PL         = 0x800020040701103Full;
//DUPS: 800020040701143F, 800020060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_EO_PL         = 0x800020040801103Full;
//DUPS: 800020040801143F, 800020060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_EO_PL          = 0x800020040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_EO_PL         = 0x800020040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x800020040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x800020040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x800020060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x800020040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x800020040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x800020060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_EO_PL        = 0x800020040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_EO_PL        = 0x800020040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_EO_PL        = 0x800020060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_EO_PL        = 0x800020040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x800020040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x800020060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_O_PL            = 0x800098040701103Full;
//DUPS: 800098040701143F, 800098060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_O_PL          = 0x800098040701103Full;
//DUPS: 800098040701143F, 800098060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_O_PL          = 0x800098040801103Full;
//DUPS: 800098040801143F, 800098060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_O_PL           = 0x800098040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_O_PL          = 0x800098040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_O_PL        = 0x800098040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_O_PL        = 0x800098040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_O_PL        = 0x800098060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_O_PL        = 0x800098040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_O_PL        = 0x800098040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_O_PL        = 0x800098060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_O_PL         = 0x800098040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_O_PL         = 0x800098040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_O_PL         = 0x800098060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_O_PL         = 0x800098040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_O_PL        = 0x800098040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL4_O_PL        = 0x800098060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_EO_PL           = 0x800028040701103Full;
//DUPS: 800028040701143F, 800028060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_EO_PL         = 0x800028040701103Full;
//DUPS: 800028040701143F, 800028060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_EO_PL         = 0x800028040801103Full;
//DUPS: 800028040801143F, 800028060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_EO_PL          = 0x800028040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_EO_PL         = 0x800028040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x800028040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x800028040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x800028060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x800028040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x800028040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x800028060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_EO_PL        = 0x800028040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_EO_PL        = 0x800028040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_EO_PL        = 0x800028060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_EO_PL        = 0x800028040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x800028040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x800028060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_O_PL            = 0x8000A0040701103Full;
//DUPS: 8000A0040701143F, 8000A0060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_O_PL          = 0x8000A0040701103Full;
//DUPS: 8000A0040701143F, 8000A0060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_O_PL          = 0x8000A0040801103Full;
//DUPS: 8000A0040801143F, 8000A0060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_O_PL           = 0x8000A0040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_O_PL          = 0x8000A0040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A0040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A0040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A0060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A0040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A0040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A0060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_O_PL         = 0x8000A0040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_O_PL         = 0x8000A0040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_O_PL         = 0x8000A0060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_O_PL         = 0x8000A0040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A0040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A0060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_EO_PL           = 0x800030040701103Full;
//DUPS: 800030040701143F, 800030060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_EO_PL         = 0x800030040701103Full;
//DUPS: 800030040701143F, 800030060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_EO_PL         = 0x800030040801103Full;
//DUPS: 800030040801143F, 800030060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_EO_PL          = 0x800030040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_EO_PL         = 0x800030040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x800030040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x800030040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x800030060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x800030040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x800030040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x800030060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_EO_PL        = 0x800030040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_EO_PL        = 0x800030040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_EO_PL        = 0x800030060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_EO_PL        = 0x800030040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x800030040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x800030060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_O_PL            = 0x8000A8040701103Full;
//DUPS: 8000A8040701143F, 8000A8060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_O_PL          = 0x8000A8040701103Full;
//DUPS: 8000A8040701143F, 8000A8060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_O_PL          = 0x8000A8040801103Full;
//DUPS: 8000A8040801143F, 8000A8060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_O_PL           = 0x8000A8040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_O_PL          = 0x8000A8040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A8040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A8040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A8060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A8040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A8040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A8060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_O_PL         = 0x8000A8040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_O_PL         = 0x8000A8040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_O_PL         = 0x8000A8060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_O_PL         = 0x8000A8040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A8040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A8060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_EO_PL           = 0x800038040701103Full;
//DUPS: 800038040701143F, 800038060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_EO_PL         = 0x800038040701103Full;
//DUPS: 800038040701143F, 800038060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_EO_PL         = 0x800038040801103Full;
//DUPS: 800038040801143F, 800038060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_EO_PL          = 0x800038040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_EO_PL         = 0x800038040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x800038040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x800038040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x800038060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x800038040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x800038040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x800038060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_EO_PL        = 0x800038040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_EO_PL        = 0x800038040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_EO_PL        = 0x800038060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_EO_PL        = 0x800038040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x800038040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x800038060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_O_PL            = 0x8000B0040701103Full;
//DUPS: 8000B0040701143F, 8000B0060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_O_PL          = 0x8000B0040701103Full;
//DUPS: 8000B0040701143F, 8000B0060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_O_PL          = 0x8000B0040801103Full;
//DUPS: 8000B0040801143F, 8000B0060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_O_PL           = 0x8000B0040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_O_PL          = 0x8000B0040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B0040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B0040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B0060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B0040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B0040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B0060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_O_PL         = 0x8000B0040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_O_PL         = 0x8000B0040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_O_PL         = 0x8000B0060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_O_PL         = 0x8000B0040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B0040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B0060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_EO_PL           = 0x800040040701103Full;
//DUPS: 800040040701143F, 800040060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_EO_PL         = 0x800040040701103Full;
//DUPS: 800040040701143F, 800040060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_EO_PL         = 0x800040040801103Full;
//DUPS: 800040040801143F, 800040060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_EO_PL          = 0x800040040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_EO_PL         = 0x800040040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x800040040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x800040040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x800040060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x800040040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x800040040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x800040060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_EO_PL        = 0x800040040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_EO_PL        = 0x800040040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_EO_PL        = 0x800040060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_EO_PL        = 0x800040040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x800040040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x800040060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_O_PL            = 0x8000B8040701103Full;
//DUPS: 8000B8040701143F, 8000B8060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_O_PL          = 0x8000B8040701103Full;
//DUPS: 8000B8040701143F, 8000B8060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_O_PL          = 0x8000B8040801103Full;
//DUPS: 8000B8040801143F, 8000B8060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_O_PL           = 0x8000B8040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_O_PL          = 0x8000B8040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B8040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B8040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B8060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B8040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B8040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B8060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_O_PL         = 0x8000B8040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_O_PL         = 0x8000B8040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_O_PL         = 0x8000B8060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_O_PL         = 0x8000B8040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B8040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B8060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL9_O_PL            = 0x8000C0040701103Full;
//DUPS: 8000C0040701143F, 8000C0060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL9_O_PL          = 0x8000C0040701103Full;
//DUPS: 8000C0040701143F, 8000C0060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL9_O_PL          = 0x8000C0040801103Full;
//DUPS: 8000C0040801143F, 8000C0060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL9_O_PL           = 0x8000C0040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL9_O_PL          = 0x8000C0040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C0040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C0040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C0060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C0040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C0040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C0060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL9_O_PL         = 0x8000C0040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL9_O_PL         = 0x8000C0040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL9_O_PL         = 0x8000C0060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL9_O_PL         = 0x8000C0040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C0040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C0060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_DATA_DAC_SPARE_MODE_PL    = 0x800000040701103Full;
//DUPS: 800000040701143F, 800000060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000040701103Full;
//DUPS: 800000040701143F, 800000060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000040801103Full;
//DUPS: 800000040801143F, 800000060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_DATA_DAC_SPARE_MODE_PL   = 0x800000040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_FIR_ERROR_INJECT_PL       = 0x800218040701103Full;
//DUPS: 800218040701143F, 800218060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_FIR_ERROR_INJECT_PL     = 0x800218040701103Full;
//DUPS: 800218040701143F, 800218060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_FIR_ERROR_INJECT_PL     = 0x800218040801103Full;
//DUPS: 800218040801143F, 800218060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_FIR_ERROR_INJECT_PL      = 0x800218040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_FIR_ERROR_INJECT_PL     = 0x800218040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x800218040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x800218040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x800218060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x800218040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x800218040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x800218060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_FIR_ERROR_INJECT_PL    = 0x800218040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_FIR_ERROR_INJECT_PL    = 0x800218040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_FIR_ERROR_INJECT_PL    = 0x800218060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_FIR_ERROR_INJECT_PL    = 0x800218040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x800218040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x800218060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_FIR_MASK_PL               = 0x800210040701103Full;
//DUPS: 800210040701143F, 800210060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_FIR_MASK_PL             = 0x800210040701103Full;
//DUPS: 800210040701143F, 800210060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_FIR_MASK_PL             = 0x800210040801103Full;
//DUPS: 800210040801143F, 800210060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_FIR_MASK_PL              = 0x800210040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_FIR_MASK_PL             = 0x800210040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_FIR_MASK_PL           = 0x800210040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_FIR_MASK_PL           = 0x800210040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_FIR_MASK_PL           = 0x800210060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_FIR_MASK_PL           = 0x800210040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_FIR_MASK_PL           = 0x800210040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_FIR_MASK_PL           = 0x800210060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_FIR_MASK_PL            = 0x800210040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_FIR_MASK_PL            = 0x800210040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_FIR_MASK_PL            = 0x800210060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_FIR_MASK_PL            = 0x800210040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_FIR_MASK_PL           = 0x800210040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_FIR_MASK_PL           = 0x800210060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_FIR_PL                    = 0x800208040701103Full;
//DUPS: 800208040701143F, 800208060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_FIR_PL                  = 0x800208040701103Full;
//DUPS: 800208040701143F, 800208060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_FIR_PL                  = 0x800208040801103Full;
//DUPS: 800208040801143F, 800208060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_FIR_PL                   = 0x800208040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_FIR_PL                  = 0x800208040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_FIR_PL                = 0x800208040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_FIR_PL                = 0x800208040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_FIR_PL                = 0x800208060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_FIR_PL                = 0x800208040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_FIR_PL                = 0x800208040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_FIR_PL                = 0x800208060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_FIR_PL                 = 0x800208040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_FIR_PL                 = 0x800208040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_FIR_PL                 = 0x800208060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_FIR_PL                 = 0x800208040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_FIR_PL                = 0x800208040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_FIR_PL                = 0x800208060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE0_RX_SPARE_MODE_PL             = 0x800200040701103Full;
//DUPS: 800200040701143F, 800200060701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE0_RX_SPARE_MODE_PL           = 0x800200040701103Full;
//DUPS: 800200040701143F, 800200060701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE0_RX_SPARE_MODE_PL           = 0x800200040801103Full;
//DUPS: 800200040801143F, 800200060801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE0_RX_SPARE_MODE_PL            = 0x800200040701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE0_RX_SPARE_MODE_PL           = 0x800200040701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE0_RX_SPARE_MODE_PL         = 0x800200040701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE0_RX_SPARE_MODE_PL         = 0x800200040701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE0_RX_SPARE_MODE_PL         = 0x800200060701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE0_RX_SPARE_MODE_PL         = 0x800200040801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE0_RX_SPARE_MODE_PL         = 0x800200040801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE0_RX_SPARE_MODE_PL         = 0x800200060801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE0_RX_SPARE_MODE_PL          = 0x800200040701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE0_RX_SPARE_MODE_PL          = 0x800200040701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE0_RX_SPARE_MODE_PL          = 0x800200060701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE0_RX_SPARE_MODE_PL          = 0x800200040801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE0_RX_SPARE_MODE_PL         = 0x800200040801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE0_RX_SPARE_MODE_PL         = 0x800200060801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL2_EO_PL           = 0x800248050701103Full;
//DUPS: 800248050701143F, 800248070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL2_EO_PL         = 0x800248050701103Full;
//DUPS: 800248050701143F, 800248070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL2_EO_PL         = 0x800248050801103Full;
//DUPS: 800248050801143F, 800248070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL2_EO_PL          = 0x800248050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL2_EO_PL         = 0x800248050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x800248050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x800248050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x800248070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x800248050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x800248050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x800248070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL2_EO_PL        = 0x800248050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL2_EO_PL        = 0x800248050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL2_EO_PL        = 0x800248070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL2_EO_PL        = 0x800248050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x800248050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x800248070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL3_EO_PL           = 0x800250050701103Full;
//DUPS: 800250050701143F, 800250070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL3_EO_PL         = 0x800250050701103Full;
//DUPS: 800250050701143F, 800250070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL3_EO_PL         = 0x800250050801103Full;
//DUPS: 800250050801143F, 800250070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL3_EO_PL          = 0x800250050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL3_EO_PL         = 0x800250050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x800250050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x800250050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x800250070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x800250050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x800250050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x800250070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL3_EO_PL        = 0x800250050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL3_EO_PL        = 0x800250050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL3_EO_PL        = 0x800250070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL3_EO_PL        = 0x800250050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x800250050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x800250070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL4_EO_PL           = 0x800258050701103Full;
//DUPS: 800258050701143F, 800258070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL4_EO_PL         = 0x800258050701103Full;
//DUPS: 800258050701143F, 800258070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL4_EO_PL         = 0x800258050801103Full;
//DUPS: 800258050801143F, 800258070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL4_EO_PL          = 0x800258050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL4_EO_PL         = 0x800258050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x800258050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x800258050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x800258070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x800258050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x800258050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x800258070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL4_EO_PL        = 0x800258050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL4_EO_PL        = 0x800258050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL4_EO_PL        = 0x800258070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL4_EO_PL        = 0x800258050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x800258050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x800258070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL5_EO_PL           = 0x800260050701103Full;
//DUPS: 800260050701143F, 800260070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL5_EO_PL         = 0x800260050701103Full;
//DUPS: 800260050701143F, 800260070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL5_EO_PL         = 0x800260050801103Full;
//DUPS: 800260050801143F, 800260070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL5_EO_PL          = 0x800260050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL5_EO_PL         = 0x800260050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x800260050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x800260050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x800260070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x800260050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x800260050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x800260070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL5_EO_PL        = 0x800260050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL5_EO_PL        = 0x800260050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL5_EO_PL        = 0x800260070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL5_EO_PL        = 0x800260050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x800260050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x800260070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_BIT_CNTLX1_EO_PL          = 0x800240050701103Full;
//DUPS: 800240050701143F, 800240070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_BIT_CNTLX1_EO_PL        = 0x800240050701103Full;
//DUPS: 800240050701143F, 800240070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_BIT_CNTLX1_EO_PL        = 0x800240050801103Full;
//DUPS: 800240050801143F, 800240070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_BIT_CNTLX1_EO_PL         = 0x800240050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_BIT_CNTLX1_EO_PL        = 0x800240050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x800240050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x800240050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x800240070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x800240050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x800240050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x800240070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_BIT_CNTLX1_EO_PL       = 0x800240050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_BIT_CNTLX1_EO_PL       = 0x800240050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_BIT_CNTLX1_EO_PL       = 0x800240070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_BIT_CNTLX1_EO_PL       = 0x800240050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x800240050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x800240070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_BIT_MODE1_EO_PL           = 0x800220050701103Full;
//DUPS: 800220050701143F, 800220070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_BIT_MODE1_EO_PL         = 0x800220050701103Full;
//DUPS: 800220050701143F, 800220070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_BIT_MODE1_EO_PL         = 0x800220050801103Full;
//DUPS: 800220050801143F, 800220070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_BIT_MODE1_EO_PL          = 0x800220050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_BIT_MODE1_EO_PL         = 0x800220050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_BIT_MODE1_EO_PL       = 0x800220050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_BIT_MODE1_EO_PL       = 0x800220050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_BIT_MODE1_EO_PL       = 0x800220070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_BIT_MODE1_EO_PL       = 0x800220050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_BIT_MODE1_EO_PL       = 0x800220050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_BIT_MODE1_EO_PL       = 0x800220070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_BIT_MODE1_EO_PL        = 0x800220050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_BIT_MODE1_EO_PL        = 0x800220050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_BIT_MODE1_EO_PL        = 0x800220070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_BIT_MODE1_EO_PL        = 0x800220050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_BIT_MODE1_EO_PL       = 0x800220050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_BIT_MODE1_EO_PL       = 0x800220070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_BIT_MODE2_EO_PL           = 0x800228050701103Full;
//DUPS: 800228050701143F, 800228070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_BIT_MODE2_EO_PL         = 0x800228050701103Full;
//DUPS: 800228050701143F, 800228070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_BIT_MODE2_EO_PL         = 0x800228050801103Full;
//DUPS: 800228050801143F, 800228070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_BIT_MODE2_EO_PL          = 0x800228050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_BIT_MODE2_EO_PL         = 0x800228050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_BIT_MODE2_EO_PL       = 0x800228050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_BIT_MODE2_EO_PL       = 0x800228050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_BIT_MODE2_EO_PL       = 0x800228070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_BIT_MODE2_EO_PL       = 0x800228050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_BIT_MODE2_EO_PL       = 0x800228050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_BIT_MODE2_EO_PL       = 0x800228070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_BIT_MODE2_EO_PL        = 0x800228050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_BIT_MODE2_EO_PL        = 0x800228050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_BIT_MODE2_EO_PL        = 0x800228070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_BIT_MODE2_EO_PL        = 0x800228050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_BIT_MODE2_EO_PL       = 0x800228050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_BIT_MODE2_EO_PL       = 0x800228070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_BIT_MODE3_EO_PL           = 0x800230050701103Full;
//DUPS: 800230050701143F, 800230070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_BIT_MODE3_EO_PL         = 0x800230050701103Full;
//DUPS: 800230050701143F, 800230070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_BIT_MODE3_EO_PL         = 0x800230050801103Full;
//DUPS: 800230050801143F, 800230070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_BIT_MODE3_EO_PL          = 0x800230050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_BIT_MODE3_EO_PL         = 0x800230050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_BIT_MODE3_EO_PL       = 0x800230050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_BIT_MODE3_EO_PL       = 0x800230050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_BIT_MODE3_EO_PL       = 0x800230070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_BIT_MODE3_EO_PL       = 0x800230050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_BIT_MODE3_EO_PL       = 0x800230050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_BIT_MODE3_EO_PL       = 0x800230070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_BIT_MODE3_EO_PL        = 0x800230050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_BIT_MODE3_EO_PL        = 0x800230050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_BIT_MODE3_EO_PL        = 0x800230070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_BIT_MODE3_EO_PL        = 0x800230050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_BIT_MODE3_EO_PL       = 0x800230050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_BIT_MODE3_EO_PL       = 0x800230070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_BIT_MODE4_EO_PL           = 0x800238050701103Full;
//DUPS: 800238050701143F, 800238070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_BIT_MODE4_EO_PL         = 0x800238050701103Full;
//DUPS: 800238050701143F, 800238070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_BIT_MODE4_EO_PL         = 0x800238050801103Full;
//DUPS: 800238050801143F, 800238070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_BIT_MODE4_EO_PL          = 0x800238050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_BIT_MODE4_EO_PL         = 0x800238050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_BIT_MODE4_EO_PL       = 0x800238050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_BIT_MODE4_EO_PL       = 0x800238050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_BIT_MODE4_EO_PL       = 0x800238070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_BIT_MODE4_EO_PL       = 0x800238050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_BIT_MODE4_EO_PL       = 0x800238050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_BIT_MODE4_EO_PL       = 0x800238070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_BIT_MODE4_EO_PL        = 0x800238050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_BIT_MODE4_EO_PL        = 0x800238050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_BIT_MODE4_EO_PL        = 0x800238070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_BIT_MODE4_EO_PL        = 0x800238050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_BIT_MODE4_EO_PL       = 0x800238050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_BIT_MODE4_EO_PL       = 0x800238070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_EO_PL           = 0x800268050701103Full;
//DUPS: 800268050701143F, 800268070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_EO_PL         = 0x800268050701103Full;
//DUPS: 800268050701143F, 800268070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_EO_PL         = 0x800268050801103Full;
//DUPS: 800268050801143F, 800268070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_EO_PL          = 0x800268050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_EO_PL         = 0x800268050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_EO_PL       = 0x800268050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_EO_PL       = 0x800268050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_EO_PL       = 0x800268070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_EO_PL       = 0x800268050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_EO_PL       = 0x800268050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_EO_PL       = 0x800268070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_EO_PL        = 0x800268050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_EO_PL        = 0x800268050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_EO_PL        = 0x800268070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_EO_PL        = 0x800268050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_EO_PL       = 0x800268050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_EO_PL       = 0x800268070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_O_PL            = 0x800318050701103Full;
//DUPS: 800318050701143F, 800318070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_O_PL          = 0x800318050701103Full;
//DUPS: 800318050701143F, 800318070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_O_PL          = 0x800318050801103Full;
//DUPS: 800318050801143F, 800318070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_O_PL           = 0x800318050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_O_PL          = 0x800318050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_O_PL        = 0x800318050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_O_PL        = 0x800318050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_O_PL        = 0x800318070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_O_PL        = 0x800318050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_O_PL        = 0x800318050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_O_PL        = 0x800318070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_O_PL         = 0x800318050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_O_PL         = 0x800318050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_O_PL         = 0x800318070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_O_PL         = 0x800318050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_O_PL        = 0x800318050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_BIT_STAT1_O_PL        = 0x800318070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_BIT_STAT2_EO_PL           = 0x800270050701103Full;
//DUPS: 800270050701143F, 800270070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_BIT_STAT2_EO_PL         = 0x800270050701103Full;
//DUPS: 800270050701143F, 800270070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_BIT_STAT2_EO_PL         = 0x800270050801103Full;
//DUPS: 800270050801143F, 800270070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_BIT_STAT2_EO_PL          = 0x800270050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_BIT_STAT2_EO_PL         = 0x800270050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_BIT_STAT2_EO_PL       = 0x800270050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_BIT_STAT2_EO_PL       = 0x800270050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_BIT_STAT2_EO_PL       = 0x800270070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_BIT_STAT2_EO_PL       = 0x800270050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_BIT_STAT2_EO_PL       = 0x800270050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_BIT_STAT2_EO_PL       = 0x800270070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_BIT_STAT2_EO_PL        = 0x800270050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_BIT_STAT2_EO_PL        = 0x800270050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_BIT_STAT2_EO_PL        = 0x800270070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_BIT_STAT2_EO_PL        = 0x800270050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_BIT_STAT2_EO_PL       = 0x800270050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_BIT_STAT2_EO_PL       = 0x800270070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_BIT_STAT3_EO_PL           = 0x800278050701103Full;
//DUPS: 800278050701143F, 800278070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_BIT_STAT3_EO_PL         = 0x800278050701103Full;
//DUPS: 800278050701143F, 800278070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_BIT_STAT3_EO_PL         = 0x800278050801103Full;
//DUPS: 800278050801143F, 800278070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_BIT_STAT3_EO_PL          = 0x800278050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_BIT_STAT3_EO_PL         = 0x800278050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_BIT_STAT3_EO_PL       = 0x800278050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_BIT_STAT3_EO_PL       = 0x800278050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_BIT_STAT3_EO_PL       = 0x800278070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_BIT_STAT3_EO_PL       = 0x800278050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_BIT_STAT3_EO_PL       = 0x800278050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_BIT_STAT3_EO_PL       = 0x800278070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_BIT_STAT3_EO_PL        = 0x800278050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_BIT_STAT3_EO_PL        = 0x800278050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_BIT_STAT3_EO_PL        = 0x800278070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_BIT_STAT3_EO_PL        = 0x800278050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_BIT_STAT3_EO_PL       = 0x800278050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_BIT_STAT3_EO_PL       = 0x800278070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_BIT_STAT4_EO_PL           = 0x800280050701103Full;
//DUPS: 800280050701143F, 800280070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_BIT_STAT4_EO_PL         = 0x800280050701103Full;
//DUPS: 800280050701143F, 800280070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_BIT_STAT4_EO_PL         = 0x800280050801103Full;
//DUPS: 800280050801143F, 800280070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_BIT_STAT4_EO_PL          = 0x800280050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_BIT_STAT4_EO_PL         = 0x800280050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_BIT_STAT4_EO_PL       = 0x800280050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_BIT_STAT4_EO_PL       = 0x800280050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_BIT_STAT4_EO_PL       = 0x800280070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_BIT_STAT4_EO_PL       = 0x800280050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_BIT_STAT4_EO_PL       = 0x800280050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_BIT_STAT4_EO_PL       = 0x800280070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_BIT_STAT4_EO_PL        = 0x800280050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_BIT_STAT4_EO_PL        = 0x800280050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_BIT_STAT4_EO_PL        = 0x800280070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_BIT_STAT4_EO_PL        = 0x800280050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_BIT_STAT4_EO_PL       = 0x800280050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_BIT_STAT4_EO_PL       = 0x800280070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL10_O_PL           = 0x8000C8050701103Full;
//DUPS: 8000C8050701143F, 8000C8070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL10_O_PL         = 0x8000C8050701103Full;
//DUPS: 8000C8050701143F, 8000C8070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL10_O_PL         = 0x8000C8050801103Full;
//DUPS: 8000C8050801143F, 8000C8070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL10_O_PL          = 0x8000C8050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL10_O_PL         = 0x8000C8050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C8050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C8050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C8070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C8050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C8050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C8070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL10_O_PL        = 0x8000C8050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL10_O_PL        = 0x8000C8050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL10_O_PL        = 0x8000C8070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL10_O_PL        = 0x8000C8050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C8050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C8070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_EO_PL           = 0x800008050701103Full;
//DUPS: 800008050701143F, 800008070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_EO_PL         = 0x800008050701103Full;
//DUPS: 800008050701143F, 800008070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_EO_PL         = 0x800008050801103Full;
//DUPS: 800008050801143F, 800008070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_EO_PL          = 0x800008050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_EO_PL         = 0x800008050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x800008050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x800008050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x800008070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x800008050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x800008050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x800008070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_EO_PL        = 0x800008050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_EO_PL        = 0x800008050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_EO_PL        = 0x800008070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_EO_PL        = 0x800008050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x800008050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x800008070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_O_PL            = 0x800080050701103Full;
//DUPS: 800080050701143F, 800080070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_O_PL          = 0x800080050701103Full;
//DUPS: 800080050701143F, 800080070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_O_PL          = 0x800080050801103Full;
//DUPS: 800080050801143F, 800080070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_O_PL           = 0x800080050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_O_PL          = 0x800080050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_O_PL        = 0x800080050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_O_PL        = 0x800080050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_O_PL        = 0x800080070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_O_PL        = 0x800080050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_O_PL        = 0x800080050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_O_PL        = 0x800080070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_O_PL         = 0x800080050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_O_PL         = 0x800080050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_O_PL         = 0x800080070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_O_PL         = 0x800080050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_O_PL        = 0x800080050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL1_O_PL        = 0x800080070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_EO_PL           = 0x800010050701103Full;
//DUPS: 800010050701143F, 800010070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_EO_PL         = 0x800010050701103Full;
//DUPS: 800010050701143F, 800010070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_EO_PL         = 0x800010050801103Full;
//DUPS: 800010050801143F, 800010070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_EO_PL          = 0x800010050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_EO_PL         = 0x800010050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x800010050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x800010050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x800010070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x800010050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x800010050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x800010070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_EO_PL        = 0x800010050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_EO_PL        = 0x800010050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_EO_PL        = 0x800010070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_EO_PL        = 0x800010050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x800010050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x800010070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_O_PL            = 0x800088050701103Full;
//DUPS: 800088050701143F, 800088070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_O_PL          = 0x800088050701103Full;
//DUPS: 800088050701143F, 800088070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_O_PL          = 0x800088050801103Full;
//DUPS: 800088050801143F, 800088070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_O_PL           = 0x800088050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_O_PL          = 0x800088050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_O_PL        = 0x800088050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_O_PL        = 0x800088050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_O_PL        = 0x800088070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_O_PL        = 0x800088050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_O_PL        = 0x800088050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_O_PL        = 0x800088070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_O_PL         = 0x800088050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_O_PL         = 0x800088050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_O_PL         = 0x800088070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_O_PL         = 0x800088050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_O_PL        = 0x800088050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL2_O_PL        = 0x800088070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_EO_PL           = 0x800018050701103Full;
//DUPS: 800018050701143F, 800018070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_EO_PL         = 0x800018050701103Full;
//DUPS: 800018050701143F, 800018070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_EO_PL         = 0x800018050801103Full;
//DUPS: 800018050801143F, 800018070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_EO_PL          = 0x800018050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_EO_PL         = 0x800018050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x800018050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x800018050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x800018070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x800018050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x800018050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x800018070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_EO_PL        = 0x800018050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_EO_PL        = 0x800018050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_EO_PL        = 0x800018070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_EO_PL        = 0x800018050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x800018050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x800018070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_O_PL            = 0x800090050701103Full;
//DUPS: 800090050701143F, 800090070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_O_PL          = 0x800090050701103Full;
//DUPS: 800090050701143F, 800090070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_O_PL          = 0x800090050801103Full;
//DUPS: 800090050801143F, 800090070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_O_PL           = 0x800090050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_O_PL          = 0x800090050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_O_PL        = 0x800090050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_O_PL        = 0x800090050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_O_PL        = 0x800090070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_O_PL        = 0x800090050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_O_PL        = 0x800090050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_O_PL        = 0x800090070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_O_PL         = 0x800090050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_O_PL         = 0x800090050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_O_PL         = 0x800090070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_O_PL         = 0x800090050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_O_PL        = 0x800090050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL3_O_PL        = 0x800090070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_EO_PL           = 0x800020050701103Full;
//DUPS: 800020050701143F, 800020070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_EO_PL         = 0x800020050701103Full;
//DUPS: 800020050701143F, 800020070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_EO_PL         = 0x800020050801103Full;
//DUPS: 800020050801143F, 800020070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_EO_PL          = 0x800020050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_EO_PL         = 0x800020050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x800020050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x800020050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x800020070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x800020050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x800020050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x800020070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_EO_PL        = 0x800020050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_EO_PL        = 0x800020050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_EO_PL        = 0x800020070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_EO_PL        = 0x800020050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x800020050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x800020070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_O_PL            = 0x800098050701103Full;
//DUPS: 800098050701143F, 800098070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_O_PL          = 0x800098050701103Full;
//DUPS: 800098050701143F, 800098070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_O_PL          = 0x800098050801103Full;
//DUPS: 800098050801143F, 800098070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_O_PL           = 0x800098050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_O_PL          = 0x800098050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_O_PL        = 0x800098050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_O_PL        = 0x800098050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_O_PL        = 0x800098070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_O_PL        = 0x800098050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_O_PL        = 0x800098050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_O_PL        = 0x800098070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_O_PL         = 0x800098050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_O_PL         = 0x800098050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_O_PL         = 0x800098070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_O_PL         = 0x800098050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_O_PL        = 0x800098050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL4_O_PL        = 0x800098070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_EO_PL           = 0x800028050701103Full;
//DUPS: 800028050701143F, 800028070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_EO_PL         = 0x800028050701103Full;
//DUPS: 800028050701143F, 800028070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_EO_PL         = 0x800028050801103Full;
//DUPS: 800028050801143F, 800028070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_EO_PL          = 0x800028050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_EO_PL         = 0x800028050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x800028050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x800028050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x800028070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x800028050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x800028050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x800028070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_EO_PL        = 0x800028050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_EO_PL        = 0x800028050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_EO_PL        = 0x800028070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_EO_PL        = 0x800028050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x800028050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x800028070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_O_PL            = 0x8000A0050701103Full;
//DUPS: 8000A0050701143F, 8000A0070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_O_PL          = 0x8000A0050701103Full;
//DUPS: 8000A0050701143F, 8000A0070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_O_PL          = 0x8000A0050801103Full;
//DUPS: 8000A0050801143F, 8000A0070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_O_PL           = 0x8000A0050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_O_PL          = 0x8000A0050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A0050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A0050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A0070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A0050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A0050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A0070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_O_PL         = 0x8000A0050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_O_PL         = 0x8000A0050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_O_PL         = 0x8000A0070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_O_PL         = 0x8000A0050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A0050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A0070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_EO_PL           = 0x800030050701103Full;
//DUPS: 800030050701143F, 800030070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_EO_PL         = 0x800030050701103Full;
//DUPS: 800030050701143F, 800030070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_EO_PL         = 0x800030050801103Full;
//DUPS: 800030050801143F, 800030070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_EO_PL          = 0x800030050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_EO_PL         = 0x800030050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x800030050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x800030050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x800030070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x800030050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x800030050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x800030070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_EO_PL        = 0x800030050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_EO_PL        = 0x800030050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_EO_PL        = 0x800030070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_EO_PL        = 0x800030050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x800030050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x800030070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_O_PL            = 0x8000A8050701103Full;
//DUPS: 8000A8050701143F, 8000A8070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_O_PL          = 0x8000A8050701103Full;
//DUPS: 8000A8050701143F, 8000A8070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_O_PL          = 0x8000A8050801103Full;
//DUPS: 8000A8050801143F, 8000A8070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_O_PL           = 0x8000A8050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_O_PL          = 0x8000A8050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A8050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A8050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A8070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A8050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A8050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A8070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_O_PL         = 0x8000A8050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_O_PL         = 0x8000A8050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_O_PL         = 0x8000A8070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_O_PL         = 0x8000A8050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A8050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A8070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_EO_PL           = 0x800038050701103Full;
//DUPS: 800038050701143F, 800038070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_EO_PL         = 0x800038050701103Full;
//DUPS: 800038050701143F, 800038070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_EO_PL         = 0x800038050801103Full;
//DUPS: 800038050801143F, 800038070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_EO_PL          = 0x800038050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_EO_PL         = 0x800038050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x800038050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x800038050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x800038070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x800038050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x800038050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x800038070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_EO_PL        = 0x800038050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_EO_PL        = 0x800038050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_EO_PL        = 0x800038070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_EO_PL        = 0x800038050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x800038050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x800038070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_O_PL            = 0x8000B0050701103Full;
//DUPS: 8000B0050701143F, 8000B0070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_O_PL          = 0x8000B0050701103Full;
//DUPS: 8000B0050701143F, 8000B0070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_O_PL          = 0x8000B0050801103Full;
//DUPS: 8000B0050801143F, 8000B0070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_O_PL           = 0x8000B0050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_O_PL          = 0x8000B0050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B0050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B0050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B0070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B0050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B0050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B0070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_O_PL         = 0x8000B0050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_O_PL         = 0x8000B0050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_O_PL         = 0x8000B0070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_O_PL         = 0x8000B0050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B0050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B0070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_EO_PL           = 0x800040050701103Full;
//DUPS: 800040050701143F, 800040070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_EO_PL         = 0x800040050701103Full;
//DUPS: 800040050701143F, 800040070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_EO_PL         = 0x800040050801103Full;
//DUPS: 800040050801143F, 800040070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_EO_PL          = 0x800040050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_EO_PL         = 0x800040050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x800040050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x800040050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x800040070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x800040050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x800040050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x800040070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_EO_PL        = 0x800040050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_EO_PL        = 0x800040050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_EO_PL        = 0x800040070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_EO_PL        = 0x800040050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x800040050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x800040070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_O_PL            = 0x8000B8050701103Full;
//DUPS: 8000B8050701143F, 8000B8070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_O_PL          = 0x8000B8050701103Full;
//DUPS: 8000B8050701143F, 8000B8070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_O_PL          = 0x8000B8050801103Full;
//DUPS: 8000B8050801143F, 8000B8070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_O_PL           = 0x8000B8050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_O_PL          = 0x8000B8050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B8050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B8050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B8070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B8050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B8050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B8070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_O_PL         = 0x8000B8050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_O_PL         = 0x8000B8050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_O_PL         = 0x8000B8070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_O_PL         = 0x8000B8050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B8050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B8070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL9_O_PL            = 0x8000C0050701103Full;
//DUPS: 8000C0050701143F, 8000C0070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL9_O_PL          = 0x8000C0050701103Full;
//DUPS: 8000C0050701143F, 8000C0070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL9_O_PL          = 0x8000C0050801103Full;
//DUPS: 8000C0050801143F, 8000C0070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL9_O_PL           = 0x8000C0050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL9_O_PL          = 0x8000C0050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C0050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C0050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C0070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C0050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C0050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C0070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL9_O_PL         = 0x8000C0050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL9_O_PL         = 0x8000C0050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL9_O_PL         = 0x8000C0070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL9_O_PL         = 0x8000C0050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C0050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C0070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_DATA_DAC_SPARE_MODE_PL    = 0x800000050701103Full;
//DUPS: 800000050701143F, 800000070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000050701103Full;
//DUPS: 800000050701143F, 800000070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000050801103Full;
//DUPS: 800000050801143F, 800000070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_DATA_DAC_SPARE_MODE_PL   = 0x800000050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_FIR_ERROR_INJECT_PL       = 0x800218050701103Full;
//DUPS: 800218050701143F, 800218070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_FIR_ERROR_INJECT_PL     = 0x800218050701103Full;
//DUPS: 800218050701143F, 800218070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_FIR_ERROR_INJECT_PL     = 0x800218050801103Full;
//DUPS: 800218050801143F, 800218070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_FIR_ERROR_INJECT_PL      = 0x800218050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_FIR_ERROR_INJECT_PL     = 0x800218050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x800218050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x800218050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x800218070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x800218050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x800218050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x800218070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_FIR_ERROR_INJECT_PL    = 0x800218050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_FIR_ERROR_INJECT_PL    = 0x800218050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_FIR_ERROR_INJECT_PL    = 0x800218070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_FIR_ERROR_INJECT_PL    = 0x800218050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x800218050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x800218070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_FIR_MASK_PL               = 0x800210050701103Full;
//DUPS: 800210050701143F, 800210070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_FIR_MASK_PL             = 0x800210050701103Full;
//DUPS: 800210050701143F, 800210070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_FIR_MASK_PL             = 0x800210050801103Full;
//DUPS: 800210050801143F, 800210070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_FIR_MASK_PL              = 0x800210050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_FIR_MASK_PL             = 0x800210050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_FIR_MASK_PL           = 0x800210050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_FIR_MASK_PL           = 0x800210050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_FIR_MASK_PL           = 0x800210070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_FIR_MASK_PL           = 0x800210050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_FIR_MASK_PL           = 0x800210050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_FIR_MASK_PL           = 0x800210070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_FIR_MASK_PL            = 0x800210050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_FIR_MASK_PL            = 0x800210050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_FIR_MASK_PL            = 0x800210070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_FIR_MASK_PL            = 0x800210050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_FIR_MASK_PL           = 0x800210050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_FIR_MASK_PL           = 0x800210070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_FIR_PL                    = 0x800208050701103Full;
//DUPS: 800208050701143F, 800208070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_FIR_PL                  = 0x800208050701103Full;
//DUPS: 800208050701143F, 800208070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_FIR_PL                  = 0x800208050801103Full;
//DUPS: 800208050801143F, 800208070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_FIR_PL                   = 0x800208050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_FIR_PL                  = 0x800208050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_FIR_PL                = 0x800208050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_FIR_PL                = 0x800208050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_FIR_PL                = 0x800208070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_FIR_PL                = 0x800208050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_FIR_PL                = 0x800208050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_FIR_PL                = 0x800208070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_FIR_PL                 = 0x800208050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_FIR_PL                 = 0x800208050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_FIR_PL                 = 0x800208070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_FIR_PL                 = 0x800208050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_FIR_PL                = 0x800208050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_FIR_PL                = 0x800208070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE1_RX_SPARE_MODE_PL             = 0x800200050701103Full;
//DUPS: 800200050701143F, 800200070701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE1_RX_SPARE_MODE_PL           = 0x800200050701103Full;
//DUPS: 800200050701143F, 800200070701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE1_RX_SPARE_MODE_PL           = 0x800200050801103Full;
//DUPS: 800200050801143F, 800200070801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE1_RX_SPARE_MODE_PL            = 0x800200050701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE1_RX_SPARE_MODE_PL           = 0x800200050701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE1_RX_SPARE_MODE_PL         = 0x800200050701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE1_RX_SPARE_MODE_PL         = 0x800200050701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE1_RX_SPARE_MODE_PL         = 0x800200070701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE1_RX_SPARE_MODE_PL         = 0x800200050801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE1_RX_SPARE_MODE_PL         = 0x800200050801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE1_RX_SPARE_MODE_PL         = 0x800200070801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE1_RX_SPARE_MODE_PL          = 0x800200050701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE1_RX_SPARE_MODE_PL          = 0x800200050701143Full;

static const uint64_t P9A_OMI_6_RX0_RXPACKS1_SLICE1_RX_SPARE_MODE_PL          = 0x800200070701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE1_RX_SPARE_MODE_PL          = 0x800200050801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE1_RX_SPARE_MODE_PL         = 0x800200050801143Full;

static const uint64_t P9A_OMI_14_RX0_RXPACKS1_SLICE1_RX_SPARE_MODE_PL         = 0x800200070801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL2_EO_PL           = 0x800248060701103Full;
//DUPS: 800248060701143F, 800248080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL2_EO_PL         = 0x800248060701103Full;
//DUPS: 800248060701143F, 800248080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL2_EO_PL         = 0x800248060801103Full;
//DUPS: 800248060801143F, 800248080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL2_EO_PL          = 0x800248060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL2_EO_PL         = 0x800248060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x800248060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x800248060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x800248080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x800248060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x800248060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x800248080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL2_EO_PL        = 0x800248060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL2_EO_PL        = 0x800248060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL2_EO_PL        = 0x800248080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL2_EO_PL        = 0x800248060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x800248060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x800248080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL3_EO_PL           = 0x800250060701103Full;
//DUPS: 800250060701143F, 800250080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL3_EO_PL         = 0x800250060701103Full;
//DUPS: 800250060701143F, 800250080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL3_EO_PL         = 0x800250060801103Full;
//DUPS: 800250060801143F, 800250080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL3_EO_PL          = 0x800250060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL3_EO_PL         = 0x800250060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x800250060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x800250060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x800250080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x800250060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x800250060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x800250080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL3_EO_PL        = 0x800250060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL3_EO_PL        = 0x800250060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL3_EO_PL        = 0x800250080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL3_EO_PL        = 0x800250060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x800250060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x800250080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL4_EO_PL           = 0x800258060701103Full;
//DUPS: 800258060701143F, 800258080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL4_EO_PL         = 0x800258060701103Full;
//DUPS: 800258060701143F, 800258080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL4_EO_PL         = 0x800258060801103Full;
//DUPS: 800258060801143F, 800258080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL4_EO_PL          = 0x800258060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL4_EO_PL         = 0x800258060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x800258060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x800258060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x800258080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x800258060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x800258060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x800258080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL4_EO_PL        = 0x800258060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL4_EO_PL        = 0x800258060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL4_EO_PL        = 0x800258080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL4_EO_PL        = 0x800258060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x800258060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x800258080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL5_EO_PL           = 0x800260060701103Full;
//DUPS: 800260060701143F, 800260080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL5_EO_PL         = 0x800260060701103Full;
//DUPS: 800260060701143F, 800260080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL5_EO_PL         = 0x800260060801103Full;
//DUPS: 800260060801143F, 800260080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL5_EO_PL          = 0x800260060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL5_EO_PL         = 0x800260060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x800260060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x800260060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x800260080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x800260060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x800260060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x800260080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL5_EO_PL        = 0x800260060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL5_EO_PL        = 0x800260060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL5_EO_PL        = 0x800260080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL5_EO_PL        = 0x800260060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x800260060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x800260080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_BIT_CNTLX1_EO_PL          = 0x800240060701103Full;
//DUPS: 800240060701143F, 800240080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_BIT_CNTLX1_EO_PL        = 0x800240060701103Full;
//DUPS: 800240060701143F, 800240080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_BIT_CNTLX1_EO_PL        = 0x800240060801103Full;
//DUPS: 800240060801143F, 800240080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_BIT_CNTLX1_EO_PL         = 0x800240060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_BIT_CNTLX1_EO_PL        = 0x800240060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x800240060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x800240060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x800240080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x800240060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x800240060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x800240080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_BIT_CNTLX1_EO_PL       = 0x800240060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_BIT_CNTLX1_EO_PL       = 0x800240060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_BIT_CNTLX1_EO_PL       = 0x800240080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_BIT_CNTLX1_EO_PL       = 0x800240060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x800240060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x800240080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_BIT_MODE1_EO_PL           = 0x800220060701103Full;
//DUPS: 800220060701143F, 800220080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_BIT_MODE1_EO_PL         = 0x800220060701103Full;
//DUPS: 800220060701143F, 800220080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_BIT_MODE1_EO_PL         = 0x800220060801103Full;
//DUPS: 800220060801143F, 800220080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_BIT_MODE1_EO_PL          = 0x800220060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_BIT_MODE1_EO_PL         = 0x800220060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_BIT_MODE1_EO_PL       = 0x800220060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_BIT_MODE1_EO_PL       = 0x800220060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_BIT_MODE1_EO_PL       = 0x800220080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_BIT_MODE1_EO_PL       = 0x800220060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_BIT_MODE1_EO_PL       = 0x800220060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_BIT_MODE1_EO_PL       = 0x800220080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_BIT_MODE1_EO_PL        = 0x800220060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_BIT_MODE1_EO_PL        = 0x800220060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_BIT_MODE1_EO_PL        = 0x800220080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_BIT_MODE1_EO_PL        = 0x800220060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_BIT_MODE1_EO_PL       = 0x800220060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_BIT_MODE1_EO_PL       = 0x800220080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_BIT_MODE2_EO_PL           = 0x800228060701103Full;
//DUPS: 800228060701143F, 800228080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_BIT_MODE2_EO_PL         = 0x800228060701103Full;
//DUPS: 800228060701143F, 800228080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_BIT_MODE2_EO_PL         = 0x800228060801103Full;
//DUPS: 800228060801143F, 800228080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_BIT_MODE2_EO_PL          = 0x800228060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_BIT_MODE2_EO_PL         = 0x800228060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_BIT_MODE2_EO_PL       = 0x800228060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_BIT_MODE2_EO_PL       = 0x800228060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_BIT_MODE2_EO_PL       = 0x800228080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_BIT_MODE2_EO_PL       = 0x800228060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_BIT_MODE2_EO_PL       = 0x800228060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_BIT_MODE2_EO_PL       = 0x800228080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_BIT_MODE2_EO_PL        = 0x800228060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_BIT_MODE2_EO_PL        = 0x800228060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_BIT_MODE2_EO_PL        = 0x800228080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_BIT_MODE2_EO_PL        = 0x800228060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_BIT_MODE2_EO_PL       = 0x800228060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_BIT_MODE2_EO_PL       = 0x800228080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_BIT_MODE3_EO_PL           = 0x800230060701103Full;
//DUPS: 800230060701143F, 800230080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_BIT_MODE3_EO_PL         = 0x800230060701103Full;
//DUPS: 800230060701143F, 800230080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_BIT_MODE3_EO_PL         = 0x800230060801103Full;
//DUPS: 800230060801143F, 800230080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_BIT_MODE3_EO_PL          = 0x800230060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_BIT_MODE3_EO_PL         = 0x800230060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_BIT_MODE3_EO_PL       = 0x800230060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_BIT_MODE3_EO_PL       = 0x800230060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_BIT_MODE3_EO_PL       = 0x800230080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_BIT_MODE3_EO_PL       = 0x800230060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_BIT_MODE3_EO_PL       = 0x800230060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_BIT_MODE3_EO_PL       = 0x800230080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_BIT_MODE3_EO_PL        = 0x800230060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_BIT_MODE3_EO_PL        = 0x800230060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_BIT_MODE3_EO_PL        = 0x800230080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_BIT_MODE3_EO_PL        = 0x800230060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_BIT_MODE3_EO_PL       = 0x800230060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_BIT_MODE3_EO_PL       = 0x800230080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_BIT_MODE4_EO_PL           = 0x800238060701103Full;
//DUPS: 800238060701143F, 800238080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_BIT_MODE4_EO_PL         = 0x800238060701103Full;
//DUPS: 800238060701143F, 800238080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_BIT_MODE4_EO_PL         = 0x800238060801103Full;
//DUPS: 800238060801143F, 800238080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_BIT_MODE4_EO_PL          = 0x800238060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_BIT_MODE4_EO_PL         = 0x800238060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_BIT_MODE4_EO_PL       = 0x800238060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_BIT_MODE4_EO_PL       = 0x800238060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_BIT_MODE4_EO_PL       = 0x800238080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_BIT_MODE4_EO_PL       = 0x800238060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_BIT_MODE4_EO_PL       = 0x800238060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_BIT_MODE4_EO_PL       = 0x800238080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_BIT_MODE4_EO_PL        = 0x800238060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_BIT_MODE4_EO_PL        = 0x800238060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_BIT_MODE4_EO_PL        = 0x800238080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_BIT_MODE4_EO_PL        = 0x800238060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_BIT_MODE4_EO_PL       = 0x800238060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_BIT_MODE4_EO_PL       = 0x800238080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_EO_PL           = 0x800268060701103Full;
//DUPS: 800268060701143F, 800268080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_EO_PL         = 0x800268060701103Full;
//DUPS: 800268060701143F, 800268080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_EO_PL         = 0x800268060801103Full;
//DUPS: 800268060801143F, 800268080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_EO_PL          = 0x800268060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_EO_PL         = 0x800268060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_EO_PL       = 0x800268060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_EO_PL       = 0x800268060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_EO_PL       = 0x800268080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_EO_PL       = 0x800268060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_EO_PL       = 0x800268060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_EO_PL       = 0x800268080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_EO_PL        = 0x800268060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_EO_PL        = 0x800268060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_EO_PL        = 0x800268080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_EO_PL        = 0x800268060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_EO_PL       = 0x800268060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_EO_PL       = 0x800268080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_O_PL            = 0x800318060701103Full;
//DUPS: 800318060701143F, 800318080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_O_PL          = 0x800318060701103Full;
//DUPS: 800318060701143F, 800318080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_O_PL          = 0x800318060801103Full;
//DUPS: 800318060801143F, 800318080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_O_PL           = 0x800318060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_O_PL          = 0x800318060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_O_PL        = 0x800318060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_O_PL        = 0x800318060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_O_PL        = 0x800318080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_O_PL        = 0x800318060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_O_PL        = 0x800318060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_O_PL        = 0x800318080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_O_PL         = 0x800318060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_O_PL         = 0x800318060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_O_PL         = 0x800318080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_O_PL         = 0x800318060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_O_PL        = 0x800318060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_BIT_STAT1_O_PL        = 0x800318080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_BIT_STAT2_EO_PL           = 0x800270060701103Full;
//DUPS: 800270060701143F, 800270080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_BIT_STAT2_EO_PL         = 0x800270060701103Full;
//DUPS: 800270060701143F, 800270080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_BIT_STAT2_EO_PL         = 0x800270060801103Full;
//DUPS: 800270060801143F, 800270080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_BIT_STAT2_EO_PL          = 0x800270060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_BIT_STAT2_EO_PL         = 0x800270060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_BIT_STAT2_EO_PL       = 0x800270060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_BIT_STAT2_EO_PL       = 0x800270060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_BIT_STAT2_EO_PL       = 0x800270080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_BIT_STAT2_EO_PL       = 0x800270060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_BIT_STAT2_EO_PL       = 0x800270060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_BIT_STAT2_EO_PL       = 0x800270080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_BIT_STAT2_EO_PL        = 0x800270060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_BIT_STAT2_EO_PL        = 0x800270060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_BIT_STAT2_EO_PL        = 0x800270080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_BIT_STAT2_EO_PL        = 0x800270060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_BIT_STAT2_EO_PL       = 0x800270060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_BIT_STAT2_EO_PL       = 0x800270080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_BIT_STAT3_EO_PL           = 0x800278060701103Full;
//DUPS: 800278060701143F, 800278080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_BIT_STAT3_EO_PL         = 0x800278060701103Full;
//DUPS: 800278060701143F, 800278080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_BIT_STAT3_EO_PL         = 0x800278060801103Full;
//DUPS: 800278060801143F, 800278080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_BIT_STAT3_EO_PL          = 0x800278060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_BIT_STAT3_EO_PL         = 0x800278060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_BIT_STAT3_EO_PL       = 0x800278060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_BIT_STAT3_EO_PL       = 0x800278060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_BIT_STAT3_EO_PL       = 0x800278080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_BIT_STAT3_EO_PL       = 0x800278060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_BIT_STAT3_EO_PL       = 0x800278060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_BIT_STAT3_EO_PL       = 0x800278080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_BIT_STAT3_EO_PL        = 0x800278060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_BIT_STAT3_EO_PL        = 0x800278060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_BIT_STAT3_EO_PL        = 0x800278080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_BIT_STAT3_EO_PL        = 0x800278060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_BIT_STAT3_EO_PL       = 0x800278060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_BIT_STAT3_EO_PL       = 0x800278080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_BIT_STAT4_EO_PL           = 0x800280060701103Full;
//DUPS: 800280060701143F, 800280080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_BIT_STAT4_EO_PL         = 0x800280060701103Full;
//DUPS: 800280060701143F, 800280080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_BIT_STAT4_EO_PL         = 0x800280060801103Full;
//DUPS: 800280060801143F, 800280080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_BIT_STAT4_EO_PL          = 0x800280060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_BIT_STAT4_EO_PL         = 0x800280060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_BIT_STAT4_EO_PL       = 0x800280060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_BIT_STAT4_EO_PL       = 0x800280060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_BIT_STAT4_EO_PL       = 0x800280080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_BIT_STAT4_EO_PL       = 0x800280060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_BIT_STAT4_EO_PL       = 0x800280060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_BIT_STAT4_EO_PL       = 0x800280080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_BIT_STAT4_EO_PL        = 0x800280060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_BIT_STAT4_EO_PL        = 0x800280060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_BIT_STAT4_EO_PL        = 0x800280080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_BIT_STAT4_EO_PL        = 0x800280060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_BIT_STAT4_EO_PL       = 0x800280060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_BIT_STAT4_EO_PL       = 0x800280080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL10_O_PL           = 0x8000C8060701103Full;
//DUPS: 8000C8060701143F, 8000C8080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL10_O_PL         = 0x8000C8060701103Full;
//DUPS: 8000C8060701143F, 8000C8080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL10_O_PL         = 0x8000C8060801103Full;
//DUPS: 8000C8060801143F, 8000C8080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL10_O_PL          = 0x8000C8060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL10_O_PL         = 0x8000C8060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C8060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C8060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C8080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C8060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C8060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C8080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL10_O_PL        = 0x8000C8060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL10_O_PL        = 0x8000C8060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL10_O_PL        = 0x8000C8080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL10_O_PL        = 0x8000C8060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C8060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C8080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_EO_PL           = 0x800008060701103Full;
//DUPS: 800008060701143F, 800008080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_EO_PL         = 0x800008060701103Full;
//DUPS: 800008060701143F, 800008080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_EO_PL         = 0x800008060801103Full;
//DUPS: 800008060801143F, 800008080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_EO_PL          = 0x800008060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_EO_PL         = 0x800008060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x800008060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x800008060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x800008080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x800008060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x800008060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x800008080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_EO_PL        = 0x800008060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_EO_PL        = 0x800008060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_EO_PL        = 0x800008080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_EO_PL        = 0x800008060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x800008060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x800008080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_O_PL            = 0x800080060701103Full;
//DUPS: 800080060701143F, 800080080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_O_PL          = 0x800080060701103Full;
//DUPS: 800080060701143F, 800080080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_O_PL          = 0x800080060801103Full;
//DUPS: 800080060801143F, 800080080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_O_PL           = 0x800080060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_O_PL          = 0x800080060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_O_PL        = 0x800080060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_O_PL        = 0x800080060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_O_PL        = 0x800080080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_O_PL        = 0x800080060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_O_PL        = 0x800080060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_O_PL        = 0x800080080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_O_PL         = 0x800080060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_O_PL         = 0x800080060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_O_PL         = 0x800080080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_O_PL         = 0x800080060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_O_PL        = 0x800080060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL1_O_PL        = 0x800080080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_EO_PL           = 0x800010060701103Full;
//DUPS: 800010060701143F, 800010080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_EO_PL         = 0x800010060701103Full;
//DUPS: 800010060701143F, 800010080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_EO_PL         = 0x800010060801103Full;
//DUPS: 800010060801143F, 800010080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_EO_PL          = 0x800010060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_EO_PL         = 0x800010060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x800010060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x800010060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x800010080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x800010060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x800010060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x800010080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_EO_PL        = 0x800010060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_EO_PL        = 0x800010060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_EO_PL        = 0x800010080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_EO_PL        = 0x800010060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x800010060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x800010080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_O_PL            = 0x800088060701103Full;
//DUPS: 800088060701143F, 800088080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_O_PL          = 0x800088060701103Full;
//DUPS: 800088060701143F, 800088080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_O_PL          = 0x800088060801103Full;
//DUPS: 800088060801143F, 800088080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_O_PL           = 0x800088060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_O_PL          = 0x800088060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_O_PL        = 0x800088060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_O_PL        = 0x800088060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_O_PL        = 0x800088080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_O_PL        = 0x800088060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_O_PL        = 0x800088060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_O_PL        = 0x800088080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_O_PL         = 0x800088060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_O_PL         = 0x800088060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_O_PL         = 0x800088080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_O_PL         = 0x800088060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_O_PL        = 0x800088060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL2_O_PL        = 0x800088080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_EO_PL           = 0x800018060701103Full;
//DUPS: 800018060701143F, 800018080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_EO_PL         = 0x800018060701103Full;
//DUPS: 800018060701143F, 800018080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_EO_PL         = 0x800018060801103Full;
//DUPS: 800018060801143F, 800018080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_EO_PL          = 0x800018060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_EO_PL         = 0x800018060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x800018060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x800018060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x800018080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x800018060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x800018060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x800018080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_EO_PL        = 0x800018060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_EO_PL        = 0x800018060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_EO_PL        = 0x800018080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_EO_PL        = 0x800018060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x800018060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x800018080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_O_PL            = 0x800090060701103Full;
//DUPS: 800090060701143F, 800090080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_O_PL          = 0x800090060701103Full;
//DUPS: 800090060701143F, 800090080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_O_PL          = 0x800090060801103Full;
//DUPS: 800090060801143F, 800090080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_O_PL           = 0x800090060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_O_PL          = 0x800090060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_O_PL        = 0x800090060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_O_PL        = 0x800090060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_O_PL        = 0x800090080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_O_PL        = 0x800090060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_O_PL        = 0x800090060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_O_PL        = 0x800090080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_O_PL         = 0x800090060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_O_PL         = 0x800090060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_O_PL         = 0x800090080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_O_PL         = 0x800090060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_O_PL        = 0x800090060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL3_O_PL        = 0x800090080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_EO_PL           = 0x800020060701103Full;
//DUPS: 800020060701143F, 800020080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_EO_PL         = 0x800020060701103Full;
//DUPS: 800020060701143F, 800020080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_EO_PL         = 0x800020060801103Full;
//DUPS: 800020060801143F, 800020080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_EO_PL          = 0x800020060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_EO_PL         = 0x800020060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x800020060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x800020060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x800020080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x800020060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x800020060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x800020080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_EO_PL        = 0x800020060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_EO_PL        = 0x800020060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_EO_PL        = 0x800020080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_EO_PL        = 0x800020060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x800020060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x800020080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_O_PL            = 0x800098060701103Full;
//DUPS: 800098060701143F, 800098080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_O_PL          = 0x800098060701103Full;
//DUPS: 800098060701143F, 800098080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_O_PL          = 0x800098060801103Full;
//DUPS: 800098060801143F, 800098080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_O_PL           = 0x800098060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_O_PL          = 0x800098060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_O_PL        = 0x800098060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_O_PL        = 0x800098060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_O_PL        = 0x800098080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_O_PL        = 0x800098060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_O_PL        = 0x800098060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_O_PL        = 0x800098080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_O_PL         = 0x800098060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_O_PL         = 0x800098060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_O_PL         = 0x800098080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_O_PL         = 0x800098060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_O_PL        = 0x800098060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL4_O_PL        = 0x800098080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_EO_PL           = 0x800028060701103Full;
//DUPS: 800028060701143F, 800028080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_EO_PL         = 0x800028060701103Full;
//DUPS: 800028060701143F, 800028080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_EO_PL         = 0x800028060801103Full;
//DUPS: 800028060801143F, 800028080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_EO_PL          = 0x800028060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_EO_PL         = 0x800028060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x800028060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x800028060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x800028080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x800028060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x800028060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x800028080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_EO_PL        = 0x800028060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_EO_PL        = 0x800028060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_EO_PL        = 0x800028080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_EO_PL        = 0x800028060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x800028060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x800028080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_O_PL            = 0x8000A0060701103Full;
//DUPS: 8000A0060701143F, 8000A0080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_O_PL          = 0x8000A0060701103Full;
//DUPS: 8000A0060701143F, 8000A0080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_O_PL          = 0x8000A0060801103Full;
//DUPS: 8000A0060801143F, 8000A0080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_O_PL           = 0x8000A0060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_O_PL          = 0x8000A0060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A0060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A0060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A0080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A0060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A0060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A0080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_O_PL         = 0x8000A0060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_O_PL         = 0x8000A0060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_O_PL         = 0x8000A0080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_O_PL         = 0x8000A0060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A0060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A0080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_EO_PL           = 0x800030060701103Full;
//DUPS: 800030060701143F, 800030080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_EO_PL         = 0x800030060701103Full;
//DUPS: 800030060701143F, 800030080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_EO_PL         = 0x800030060801103Full;
//DUPS: 800030060801143F, 800030080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_EO_PL          = 0x800030060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_EO_PL         = 0x800030060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x800030060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x800030060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x800030080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x800030060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x800030060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x800030080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_EO_PL        = 0x800030060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_EO_PL        = 0x800030060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_EO_PL        = 0x800030080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_EO_PL        = 0x800030060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x800030060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x800030080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_O_PL            = 0x8000A8060701103Full;
//DUPS: 8000A8060701143F, 8000A8080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_O_PL          = 0x8000A8060701103Full;
//DUPS: 8000A8060701143F, 8000A8080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_O_PL          = 0x8000A8060801103Full;
//DUPS: 8000A8060801143F, 8000A8080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_O_PL           = 0x8000A8060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_O_PL          = 0x8000A8060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A8060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A8060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A8080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A8060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A8060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A8080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_O_PL         = 0x8000A8060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_O_PL         = 0x8000A8060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_O_PL         = 0x8000A8080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_O_PL         = 0x8000A8060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A8060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A8080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_EO_PL           = 0x800038060701103Full;
//DUPS: 800038060701143F, 800038080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_EO_PL         = 0x800038060701103Full;
//DUPS: 800038060701143F, 800038080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_EO_PL         = 0x800038060801103Full;
//DUPS: 800038060801143F, 800038080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_EO_PL          = 0x800038060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_EO_PL         = 0x800038060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x800038060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x800038060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x800038080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x800038060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x800038060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x800038080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_EO_PL        = 0x800038060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_EO_PL        = 0x800038060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_EO_PL        = 0x800038080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_EO_PL        = 0x800038060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x800038060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x800038080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_O_PL            = 0x8000B0060701103Full;
//DUPS: 8000B0060701143F, 8000B0080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_O_PL          = 0x8000B0060701103Full;
//DUPS: 8000B0060701143F, 8000B0080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_O_PL          = 0x8000B0060801103Full;
//DUPS: 8000B0060801143F, 8000B0080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_O_PL           = 0x8000B0060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_O_PL          = 0x8000B0060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B0060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B0060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B0080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B0060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B0060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B0080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_O_PL         = 0x8000B0060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_O_PL         = 0x8000B0060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_O_PL         = 0x8000B0080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_O_PL         = 0x8000B0060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B0060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B0080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_EO_PL           = 0x800040060701103Full;
//DUPS: 800040060701143F, 800040080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_EO_PL         = 0x800040060701103Full;
//DUPS: 800040060701143F, 800040080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_EO_PL         = 0x800040060801103Full;
//DUPS: 800040060801143F, 800040080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_EO_PL          = 0x800040060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_EO_PL         = 0x800040060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x800040060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x800040060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x800040080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x800040060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x800040060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x800040080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_EO_PL        = 0x800040060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_EO_PL        = 0x800040060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_EO_PL        = 0x800040080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_EO_PL        = 0x800040060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x800040060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x800040080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_O_PL            = 0x8000B8060701103Full;
//DUPS: 8000B8060701143F, 8000B8080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_O_PL          = 0x8000B8060701103Full;
//DUPS: 8000B8060701143F, 8000B8080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_O_PL          = 0x8000B8060801103Full;
//DUPS: 8000B8060801143F, 8000B8080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_O_PL           = 0x8000B8060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_O_PL          = 0x8000B8060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B8060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B8060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B8080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B8060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B8060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B8080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_O_PL         = 0x8000B8060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_O_PL         = 0x8000B8060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_O_PL         = 0x8000B8080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_O_PL         = 0x8000B8060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B8060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B8080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL9_O_PL            = 0x8000C0060701103Full;
//DUPS: 8000C0060701143F, 8000C0080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL9_O_PL          = 0x8000C0060701103Full;
//DUPS: 8000C0060701143F, 8000C0080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL9_O_PL          = 0x8000C0060801103Full;
//DUPS: 8000C0060801143F, 8000C0080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL9_O_PL           = 0x8000C0060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL9_O_PL          = 0x8000C0060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C0060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C0060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C0080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C0060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C0060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C0080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL9_O_PL         = 0x8000C0060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL9_O_PL         = 0x8000C0060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL9_O_PL         = 0x8000C0080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL9_O_PL         = 0x8000C0060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C0060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C0080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_DATA_DAC_SPARE_MODE_PL    = 0x800000060701103Full;
//DUPS: 800000060701143F, 800000080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000060701103Full;
//DUPS: 800000060701143F, 800000080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000060801103Full;
//DUPS: 800000060801143F, 800000080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_DATA_DAC_SPARE_MODE_PL   = 0x800000060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_FIR_ERROR_INJECT_PL       = 0x800218060701103Full;
//DUPS: 800218060701143F, 800218080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_FIR_ERROR_INJECT_PL     = 0x800218060701103Full;
//DUPS: 800218060701143F, 800218080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_FIR_ERROR_INJECT_PL     = 0x800218060801103Full;
//DUPS: 800218060801143F, 800218080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_FIR_ERROR_INJECT_PL      = 0x800218060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_FIR_ERROR_INJECT_PL     = 0x800218060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x800218060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x800218060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x800218080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x800218060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x800218060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x800218080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_FIR_ERROR_INJECT_PL    = 0x800218060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_FIR_ERROR_INJECT_PL    = 0x800218060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_FIR_ERROR_INJECT_PL    = 0x800218080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_FIR_ERROR_INJECT_PL    = 0x800218060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x800218060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x800218080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_FIR_MASK_PL               = 0x800210060701103Full;
//DUPS: 800210060701143F, 800210080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_FIR_MASK_PL             = 0x800210060701103Full;
//DUPS: 800210060701143F, 800210080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_FIR_MASK_PL             = 0x800210060801103Full;
//DUPS: 800210060801143F, 800210080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_FIR_MASK_PL              = 0x800210060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_FIR_MASK_PL             = 0x800210060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_FIR_MASK_PL           = 0x800210060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_FIR_MASK_PL           = 0x800210060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_FIR_MASK_PL           = 0x800210080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_FIR_MASK_PL           = 0x800210060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_FIR_MASK_PL           = 0x800210060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_FIR_MASK_PL           = 0x800210080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_FIR_MASK_PL            = 0x800210060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_FIR_MASK_PL            = 0x800210060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_FIR_MASK_PL            = 0x800210080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_FIR_MASK_PL            = 0x800210060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_FIR_MASK_PL           = 0x800210060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_FIR_MASK_PL           = 0x800210080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_FIR_PL                    = 0x800208060701103Full;
//DUPS: 800208060701143F, 800208080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_FIR_PL                  = 0x800208060701103Full;
//DUPS: 800208060701143F, 800208080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_FIR_PL                  = 0x800208060801103Full;
//DUPS: 800208060801143F, 800208080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_FIR_PL                   = 0x800208060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_FIR_PL                  = 0x800208060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_FIR_PL                = 0x800208060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_FIR_PL                = 0x800208060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_FIR_PL                = 0x800208080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_FIR_PL                = 0x800208060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_FIR_PL                = 0x800208060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_FIR_PL                = 0x800208080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_FIR_PL                 = 0x800208060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_FIR_PL                 = 0x800208060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_FIR_PL                 = 0x800208080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_FIR_PL                 = 0x800208060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_FIR_PL                = 0x800208060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_FIR_PL                = 0x800208080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE2_RX_SPARE_MODE_PL             = 0x800200060701103Full;
//DUPS: 800200060701143F, 800200080701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE2_RX_SPARE_MODE_PL           = 0x800200060701103Full;
//DUPS: 800200060701143F, 800200080701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE2_RX_SPARE_MODE_PL           = 0x800200060801103Full;
//DUPS: 800200060801143F, 800200080801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE2_RX_SPARE_MODE_PL            = 0x800200060701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE2_RX_SPARE_MODE_PL           = 0x800200060701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE2_RX_SPARE_MODE_PL         = 0x800200060701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE2_RX_SPARE_MODE_PL         = 0x800200060701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE2_RX_SPARE_MODE_PL         = 0x800200080701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE2_RX_SPARE_MODE_PL         = 0x800200060801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE2_RX_SPARE_MODE_PL         = 0x800200060801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE2_RX_SPARE_MODE_PL         = 0x800200080801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE2_RX_SPARE_MODE_PL          = 0x800200060701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE2_RX_SPARE_MODE_PL          = 0x800200060701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE2_RX_SPARE_MODE_PL          = 0x800200080701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE2_RX_SPARE_MODE_PL          = 0x800200060801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE2_RX_SPARE_MODE_PL         = 0x800200060801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE2_RX_SPARE_MODE_PL         = 0x800200080801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL2_EO_PL           = 0x800248070701103Full;
//DUPS: 800248070701143F, 800248090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL2_EO_PL         = 0x800248070701103Full;
//DUPS: 800248070701143F, 800248090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL2_EO_PL         = 0x800248070801103Full;
//DUPS: 800248070801143F, 800248090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL2_EO_PL          = 0x800248070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL2_EO_PL         = 0x800248070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x800248070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x800248070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x800248090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x800248070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x800248070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x800248090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL2_EO_PL        = 0x800248070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL2_EO_PL        = 0x800248070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL2_EO_PL        = 0x800248090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL2_EO_PL        = 0x800248070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x800248070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x800248090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL3_EO_PL           = 0x800250070701103Full;
//DUPS: 800250070701143F, 800250090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL3_EO_PL         = 0x800250070701103Full;
//DUPS: 800250070701143F, 800250090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL3_EO_PL         = 0x800250070801103Full;
//DUPS: 800250070801143F, 800250090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL3_EO_PL          = 0x800250070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL3_EO_PL         = 0x800250070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x800250070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x800250070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x800250090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x800250070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x800250070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x800250090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL3_EO_PL        = 0x800250070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL3_EO_PL        = 0x800250070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL3_EO_PL        = 0x800250090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL3_EO_PL        = 0x800250070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x800250070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x800250090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL4_EO_PL           = 0x800258070701103Full;
//DUPS: 800258070701143F, 800258090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL4_EO_PL         = 0x800258070701103Full;
//DUPS: 800258070701143F, 800258090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL4_EO_PL         = 0x800258070801103Full;
//DUPS: 800258070801143F, 800258090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL4_EO_PL          = 0x800258070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL4_EO_PL         = 0x800258070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x800258070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x800258070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x800258090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x800258070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x800258070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x800258090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL4_EO_PL        = 0x800258070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL4_EO_PL        = 0x800258070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL4_EO_PL        = 0x800258090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL4_EO_PL        = 0x800258070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x800258070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x800258090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL5_EO_PL           = 0x800260070701103Full;
//DUPS: 800260070701143F, 800260090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL5_EO_PL         = 0x800260070701103Full;
//DUPS: 800260070701143F, 800260090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL5_EO_PL         = 0x800260070801103Full;
//DUPS: 800260070801143F, 800260090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL5_EO_PL          = 0x800260070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL5_EO_PL         = 0x800260070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x800260070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x800260070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x800260090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x800260070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x800260070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x800260090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL5_EO_PL        = 0x800260070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL5_EO_PL        = 0x800260070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL5_EO_PL        = 0x800260090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL5_EO_PL        = 0x800260070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x800260070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x800260090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_BIT_CNTLX1_EO_PL          = 0x800240070701103Full;
//DUPS: 800240070701143F, 800240090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_BIT_CNTLX1_EO_PL        = 0x800240070701103Full;
//DUPS: 800240070701143F, 800240090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_BIT_CNTLX1_EO_PL        = 0x800240070801103Full;
//DUPS: 800240070801143F, 800240090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_BIT_CNTLX1_EO_PL         = 0x800240070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_BIT_CNTLX1_EO_PL        = 0x800240070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x800240070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x800240070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x800240090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x800240070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x800240070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x800240090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_BIT_CNTLX1_EO_PL       = 0x800240070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_BIT_CNTLX1_EO_PL       = 0x800240070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_BIT_CNTLX1_EO_PL       = 0x800240090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_BIT_CNTLX1_EO_PL       = 0x800240070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x800240070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x800240090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_BIT_MODE1_EO_PL           = 0x800220070701103Full;
//DUPS: 800220070701143F, 800220090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_BIT_MODE1_EO_PL         = 0x800220070701103Full;
//DUPS: 800220070701143F, 800220090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_BIT_MODE1_EO_PL         = 0x800220070801103Full;
//DUPS: 800220070801143F, 800220090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_BIT_MODE1_EO_PL          = 0x800220070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_BIT_MODE1_EO_PL         = 0x800220070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_BIT_MODE1_EO_PL       = 0x800220070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_BIT_MODE1_EO_PL       = 0x800220070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_BIT_MODE1_EO_PL       = 0x800220090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_BIT_MODE1_EO_PL       = 0x800220070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_BIT_MODE1_EO_PL       = 0x800220070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_BIT_MODE1_EO_PL       = 0x800220090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_BIT_MODE1_EO_PL        = 0x800220070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_BIT_MODE1_EO_PL        = 0x800220070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_BIT_MODE1_EO_PL        = 0x800220090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_BIT_MODE1_EO_PL        = 0x800220070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_BIT_MODE1_EO_PL       = 0x800220070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_BIT_MODE1_EO_PL       = 0x800220090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_BIT_MODE2_EO_PL           = 0x800228070701103Full;
//DUPS: 800228070701143F, 800228090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_BIT_MODE2_EO_PL         = 0x800228070701103Full;
//DUPS: 800228070701143F, 800228090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_BIT_MODE2_EO_PL         = 0x800228070801103Full;
//DUPS: 800228070801143F, 800228090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_BIT_MODE2_EO_PL          = 0x800228070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_BIT_MODE2_EO_PL         = 0x800228070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_BIT_MODE2_EO_PL       = 0x800228070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_BIT_MODE2_EO_PL       = 0x800228070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_BIT_MODE2_EO_PL       = 0x800228090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_BIT_MODE2_EO_PL       = 0x800228070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_BIT_MODE2_EO_PL       = 0x800228070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_BIT_MODE2_EO_PL       = 0x800228090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_BIT_MODE2_EO_PL        = 0x800228070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_BIT_MODE2_EO_PL        = 0x800228070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_BIT_MODE2_EO_PL        = 0x800228090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_BIT_MODE2_EO_PL        = 0x800228070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_BIT_MODE2_EO_PL       = 0x800228070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_BIT_MODE2_EO_PL       = 0x800228090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_BIT_MODE3_EO_PL           = 0x800230070701103Full;
//DUPS: 800230070701143F, 800230090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_BIT_MODE3_EO_PL         = 0x800230070701103Full;
//DUPS: 800230070701143F, 800230090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_BIT_MODE3_EO_PL         = 0x800230070801103Full;
//DUPS: 800230070801143F, 800230090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_BIT_MODE3_EO_PL          = 0x800230070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_BIT_MODE3_EO_PL         = 0x800230070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_BIT_MODE3_EO_PL       = 0x800230070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_BIT_MODE3_EO_PL       = 0x800230070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_BIT_MODE3_EO_PL       = 0x800230090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_BIT_MODE3_EO_PL       = 0x800230070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_BIT_MODE3_EO_PL       = 0x800230070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_BIT_MODE3_EO_PL       = 0x800230090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_BIT_MODE3_EO_PL        = 0x800230070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_BIT_MODE3_EO_PL        = 0x800230070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_BIT_MODE3_EO_PL        = 0x800230090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_BIT_MODE3_EO_PL        = 0x800230070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_BIT_MODE3_EO_PL       = 0x800230070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_BIT_MODE3_EO_PL       = 0x800230090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_BIT_MODE4_EO_PL           = 0x800238070701103Full;
//DUPS: 800238070701143F, 800238090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_BIT_MODE4_EO_PL         = 0x800238070701103Full;
//DUPS: 800238070701143F, 800238090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_BIT_MODE4_EO_PL         = 0x800238070801103Full;
//DUPS: 800238070801143F, 800238090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_BIT_MODE4_EO_PL          = 0x800238070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_BIT_MODE4_EO_PL         = 0x800238070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_BIT_MODE4_EO_PL       = 0x800238070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_BIT_MODE4_EO_PL       = 0x800238070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_BIT_MODE4_EO_PL       = 0x800238090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_BIT_MODE4_EO_PL       = 0x800238070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_BIT_MODE4_EO_PL       = 0x800238070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_BIT_MODE4_EO_PL       = 0x800238090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_BIT_MODE4_EO_PL        = 0x800238070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_BIT_MODE4_EO_PL        = 0x800238070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_BIT_MODE4_EO_PL        = 0x800238090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_BIT_MODE4_EO_PL        = 0x800238070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_BIT_MODE4_EO_PL       = 0x800238070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_BIT_MODE4_EO_PL       = 0x800238090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_EO_PL           = 0x800268070701103Full;
//DUPS: 800268070701143F, 800268090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_EO_PL         = 0x800268070701103Full;
//DUPS: 800268070701143F, 800268090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_EO_PL         = 0x800268070801103Full;
//DUPS: 800268070801143F, 800268090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_EO_PL          = 0x800268070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_EO_PL         = 0x800268070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_EO_PL       = 0x800268070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_EO_PL       = 0x800268070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_EO_PL       = 0x800268090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_EO_PL       = 0x800268070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_EO_PL       = 0x800268070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_EO_PL       = 0x800268090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_EO_PL        = 0x800268070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_EO_PL        = 0x800268070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_EO_PL        = 0x800268090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_EO_PL        = 0x800268070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_EO_PL       = 0x800268070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_EO_PL       = 0x800268090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_O_PL            = 0x800318070701103Full;
//DUPS: 800318070701143F, 800318090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_O_PL          = 0x800318070701103Full;
//DUPS: 800318070701143F, 800318090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_O_PL          = 0x800318070801103Full;
//DUPS: 800318070801143F, 800318090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_O_PL           = 0x800318070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_O_PL          = 0x800318070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_O_PL        = 0x800318070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_O_PL        = 0x800318070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_O_PL        = 0x800318090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_O_PL        = 0x800318070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_O_PL        = 0x800318070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_O_PL        = 0x800318090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_O_PL         = 0x800318070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_O_PL         = 0x800318070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_O_PL         = 0x800318090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_O_PL         = 0x800318070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_O_PL        = 0x800318070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_BIT_STAT1_O_PL        = 0x800318090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_BIT_STAT2_EO_PL           = 0x800270070701103Full;
//DUPS: 800270070701143F, 800270090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_BIT_STAT2_EO_PL         = 0x800270070701103Full;
//DUPS: 800270070701143F, 800270090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_BIT_STAT2_EO_PL         = 0x800270070801103Full;
//DUPS: 800270070801143F, 800270090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_BIT_STAT2_EO_PL          = 0x800270070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_BIT_STAT2_EO_PL         = 0x800270070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_BIT_STAT2_EO_PL       = 0x800270070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_BIT_STAT2_EO_PL       = 0x800270070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_BIT_STAT2_EO_PL       = 0x800270090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_BIT_STAT2_EO_PL       = 0x800270070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_BIT_STAT2_EO_PL       = 0x800270070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_BIT_STAT2_EO_PL       = 0x800270090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_BIT_STAT2_EO_PL        = 0x800270070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_BIT_STAT2_EO_PL        = 0x800270070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_BIT_STAT2_EO_PL        = 0x800270090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_BIT_STAT2_EO_PL        = 0x800270070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_BIT_STAT2_EO_PL       = 0x800270070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_BIT_STAT2_EO_PL       = 0x800270090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_BIT_STAT3_EO_PL           = 0x800278070701103Full;
//DUPS: 800278070701143F, 800278090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_BIT_STAT3_EO_PL         = 0x800278070701103Full;
//DUPS: 800278070701143F, 800278090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_BIT_STAT3_EO_PL         = 0x800278070801103Full;
//DUPS: 800278070801143F, 800278090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_BIT_STAT3_EO_PL          = 0x800278070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_BIT_STAT3_EO_PL         = 0x800278070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_BIT_STAT3_EO_PL       = 0x800278070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_BIT_STAT3_EO_PL       = 0x800278070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_BIT_STAT3_EO_PL       = 0x800278090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_BIT_STAT3_EO_PL       = 0x800278070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_BIT_STAT3_EO_PL       = 0x800278070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_BIT_STAT3_EO_PL       = 0x800278090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_BIT_STAT3_EO_PL        = 0x800278070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_BIT_STAT3_EO_PL        = 0x800278070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_BIT_STAT3_EO_PL        = 0x800278090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_BIT_STAT3_EO_PL        = 0x800278070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_BIT_STAT3_EO_PL       = 0x800278070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_BIT_STAT3_EO_PL       = 0x800278090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_BIT_STAT4_EO_PL           = 0x800280070701103Full;
//DUPS: 800280070701143F, 800280090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_BIT_STAT4_EO_PL         = 0x800280070701103Full;
//DUPS: 800280070701143F, 800280090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_BIT_STAT4_EO_PL         = 0x800280070801103Full;
//DUPS: 800280070801143F, 800280090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_BIT_STAT4_EO_PL          = 0x800280070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_BIT_STAT4_EO_PL         = 0x800280070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_BIT_STAT4_EO_PL       = 0x800280070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_BIT_STAT4_EO_PL       = 0x800280070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_BIT_STAT4_EO_PL       = 0x800280090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_BIT_STAT4_EO_PL       = 0x800280070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_BIT_STAT4_EO_PL       = 0x800280070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_BIT_STAT4_EO_PL       = 0x800280090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_BIT_STAT4_EO_PL        = 0x800280070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_BIT_STAT4_EO_PL        = 0x800280070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_BIT_STAT4_EO_PL        = 0x800280090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_BIT_STAT4_EO_PL        = 0x800280070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_BIT_STAT4_EO_PL       = 0x800280070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_BIT_STAT4_EO_PL       = 0x800280090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL10_O_PL           = 0x8000C8070701103Full;
//DUPS: 8000C8070701143F, 8000C8090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL10_O_PL         = 0x8000C8070701103Full;
//DUPS: 8000C8070701143F, 8000C8090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL10_O_PL         = 0x8000C8070801103Full;
//DUPS: 8000C8070801143F, 8000C8090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL10_O_PL          = 0x8000C8070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL10_O_PL         = 0x8000C8070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C8070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C8070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C8090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C8070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C8070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C8090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL10_O_PL        = 0x8000C8070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL10_O_PL        = 0x8000C8070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL10_O_PL        = 0x8000C8090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL10_O_PL        = 0x8000C8070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C8070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C8090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_EO_PL           = 0x800008070701103Full;
//DUPS: 800008070701143F, 800008090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_EO_PL         = 0x800008070701103Full;
//DUPS: 800008070701143F, 800008090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_EO_PL         = 0x800008070801103Full;
//DUPS: 800008070801143F, 800008090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_EO_PL          = 0x800008070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_EO_PL         = 0x800008070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x800008070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x800008070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x800008090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x800008070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x800008070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x800008090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_EO_PL        = 0x800008070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_EO_PL        = 0x800008070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_EO_PL        = 0x800008090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_EO_PL        = 0x800008070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x800008070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x800008090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_O_PL            = 0x800080070701103Full;
//DUPS: 800080070701143F, 800080090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_O_PL          = 0x800080070701103Full;
//DUPS: 800080070701143F, 800080090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_O_PL          = 0x800080070801103Full;
//DUPS: 800080070801143F, 800080090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_O_PL           = 0x800080070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_O_PL          = 0x800080070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_O_PL        = 0x800080070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_O_PL        = 0x800080070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_O_PL        = 0x800080090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_O_PL        = 0x800080070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_O_PL        = 0x800080070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_O_PL        = 0x800080090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_O_PL         = 0x800080070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_O_PL         = 0x800080070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_O_PL         = 0x800080090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_O_PL         = 0x800080070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_O_PL        = 0x800080070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL1_O_PL        = 0x800080090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_EO_PL           = 0x800010070701103Full;
//DUPS: 800010070701143F, 800010090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_EO_PL         = 0x800010070701103Full;
//DUPS: 800010070701143F, 800010090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_EO_PL         = 0x800010070801103Full;
//DUPS: 800010070801143F, 800010090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_EO_PL          = 0x800010070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_EO_PL         = 0x800010070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x800010070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x800010070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x800010090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x800010070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x800010070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x800010090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_EO_PL        = 0x800010070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_EO_PL        = 0x800010070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_EO_PL        = 0x800010090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_EO_PL        = 0x800010070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x800010070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x800010090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_O_PL            = 0x800088070701103Full;
//DUPS: 800088070701143F, 800088090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_O_PL          = 0x800088070701103Full;
//DUPS: 800088070701143F, 800088090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_O_PL          = 0x800088070801103Full;
//DUPS: 800088070801143F, 800088090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_O_PL           = 0x800088070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_O_PL          = 0x800088070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_O_PL        = 0x800088070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_O_PL        = 0x800088070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_O_PL        = 0x800088090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_O_PL        = 0x800088070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_O_PL        = 0x800088070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_O_PL        = 0x800088090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_O_PL         = 0x800088070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_O_PL         = 0x800088070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_O_PL         = 0x800088090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_O_PL         = 0x800088070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_O_PL        = 0x800088070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL2_O_PL        = 0x800088090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_EO_PL           = 0x800018070701103Full;
//DUPS: 800018070701143F, 800018090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_EO_PL         = 0x800018070701103Full;
//DUPS: 800018070701143F, 800018090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_EO_PL         = 0x800018070801103Full;
//DUPS: 800018070801143F, 800018090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_EO_PL          = 0x800018070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_EO_PL         = 0x800018070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x800018070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x800018070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x800018090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x800018070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x800018070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x800018090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_EO_PL        = 0x800018070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_EO_PL        = 0x800018070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_EO_PL        = 0x800018090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_EO_PL        = 0x800018070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x800018070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x800018090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_O_PL            = 0x800090070701103Full;
//DUPS: 800090070701143F, 800090090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_O_PL          = 0x800090070701103Full;
//DUPS: 800090070701143F, 800090090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_O_PL          = 0x800090070801103Full;
//DUPS: 800090070801143F, 800090090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_O_PL           = 0x800090070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_O_PL          = 0x800090070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_O_PL        = 0x800090070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_O_PL        = 0x800090070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_O_PL        = 0x800090090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_O_PL        = 0x800090070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_O_PL        = 0x800090070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_O_PL        = 0x800090090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_O_PL         = 0x800090070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_O_PL         = 0x800090070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_O_PL         = 0x800090090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_O_PL         = 0x800090070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_O_PL        = 0x800090070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL3_O_PL        = 0x800090090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_EO_PL           = 0x800020070701103Full;
//DUPS: 800020070701143F, 800020090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_EO_PL         = 0x800020070701103Full;
//DUPS: 800020070701143F, 800020090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_EO_PL         = 0x800020070801103Full;
//DUPS: 800020070801143F, 800020090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_EO_PL          = 0x800020070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_EO_PL         = 0x800020070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x800020070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x800020070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x800020090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x800020070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x800020070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x800020090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_EO_PL        = 0x800020070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_EO_PL        = 0x800020070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_EO_PL        = 0x800020090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_EO_PL        = 0x800020070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x800020070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x800020090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_O_PL            = 0x800098070701103Full;
//DUPS: 800098070701143F, 800098090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_O_PL          = 0x800098070701103Full;
//DUPS: 800098070701143F, 800098090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_O_PL          = 0x800098070801103Full;
//DUPS: 800098070801143F, 800098090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_O_PL           = 0x800098070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_O_PL          = 0x800098070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_O_PL        = 0x800098070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_O_PL        = 0x800098070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_O_PL        = 0x800098090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_O_PL        = 0x800098070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_O_PL        = 0x800098070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_O_PL        = 0x800098090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_O_PL         = 0x800098070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_O_PL         = 0x800098070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_O_PL         = 0x800098090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_O_PL         = 0x800098070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_O_PL        = 0x800098070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL4_O_PL        = 0x800098090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_EO_PL           = 0x800028070701103Full;
//DUPS: 800028070701143F, 800028090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_EO_PL         = 0x800028070701103Full;
//DUPS: 800028070701143F, 800028090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_EO_PL         = 0x800028070801103Full;
//DUPS: 800028070801143F, 800028090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_EO_PL          = 0x800028070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_EO_PL         = 0x800028070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x800028070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x800028070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x800028090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x800028070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x800028070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x800028090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_EO_PL        = 0x800028070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_EO_PL        = 0x800028070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_EO_PL        = 0x800028090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_EO_PL        = 0x800028070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x800028070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x800028090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_O_PL            = 0x8000A0070701103Full;
//DUPS: 8000A0070701143F, 8000A0090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_O_PL          = 0x8000A0070701103Full;
//DUPS: 8000A0070701143F, 8000A0090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_O_PL          = 0x8000A0070801103Full;
//DUPS: 8000A0070801143F, 8000A0090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_O_PL           = 0x8000A0070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_O_PL          = 0x8000A0070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A0070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A0070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A0090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A0070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A0070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A0090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_O_PL         = 0x8000A0070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_O_PL         = 0x8000A0070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_O_PL         = 0x8000A0090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_O_PL         = 0x8000A0070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A0070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A0090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_EO_PL           = 0x800030070701103Full;
//DUPS: 800030070701143F, 800030090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_EO_PL         = 0x800030070701103Full;
//DUPS: 800030070701143F, 800030090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_EO_PL         = 0x800030070801103Full;
//DUPS: 800030070801143F, 800030090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_EO_PL          = 0x800030070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_EO_PL         = 0x800030070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x800030070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x800030070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x800030090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x800030070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x800030070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x800030090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_EO_PL        = 0x800030070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_EO_PL        = 0x800030070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_EO_PL        = 0x800030090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_EO_PL        = 0x800030070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x800030070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x800030090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_O_PL            = 0x8000A8070701103Full;
//DUPS: 8000A8070701143F, 8000A8090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_O_PL          = 0x8000A8070701103Full;
//DUPS: 8000A8070701143F, 8000A8090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_O_PL          = 0x8000A8070801103Full;
//DUPS: 8000A8070801143F, 8000A8090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_O_PL           = 0x8000A8070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_O_PL          = 0x8000A8070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A8070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A8070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A8090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A8070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A8070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A8090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_O_PL         = 0x8000A8070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_O_PL         = 0x8000A8070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_O_PL         = 0x8000A8090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_O_PL         = 0x8000A8070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A8070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A8090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_EO_PL           = 0x800038070701103Full;
//DUPS: 800038070701143F, 800038090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_EO_PL         = 0x800038070701103Full;
//DUPS: 800038070701143F, 800038090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_EO_PL         = 0x800038070801103Full;
//DUPS: 800038070801143F, 800038090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_EO_PL          = 0x800038070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_EO_PL         = 0x800038070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x800038070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x800038070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x800038090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x800038070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x800038070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x800038090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_EO_PL        = 0x800038070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_EO_PL        = 0x800038070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_EO_PL        = 0x800038090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_EO_PL        = 0x800038070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x800038070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x800038090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_O_PL            = 0x8000B0070701103Full;
//DUPS: 8000B0070701143F, 8000B0090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_O_PL          = 0x8000B0070701103Full;
//DUPS: 8000B0070701143F, 8000B0090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_O_PL          = 0x8000B0070801103Full;
//DUPS: 8000B0070801143F, 8000B0090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_O_PL           = 0x8000B0070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_O_PL          = 0x8000B0070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B0070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B0070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B0090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B0070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B0070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B0090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_O_PL         = 0x8000B0070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_O_PL         = 0x8000B0070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_O_PL         = 0x8000B0090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_O_PL         = 0x8000B0070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B0070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B0090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_EO_PL           = 0x800040070701103Full;
//DUPS: 800040070701143F, 800040090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_EO_PL         = 0x800040070701103Full;
//DUPS: 800040070701143F, 800040090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_EO_PL         = 0x800040070801103Full;
//DUPS: 800040070801143F, 800040090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_EO_PL          = 0x800040070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_EO_PL         = 0x800040070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x800040070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x800040070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x800040090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x800040070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x800040070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x800040090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_EO_PL        = 0x800040070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_EO_PL        = 0x800040070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_EO_PL        = 0x800040090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_EO_PL        = 0x800040070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x800040070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x800040090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_O_PL            = 0x8000B8070701103Full;
//DUPS: 8000B8070701143F, 8000B8090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_O_PL          = 0x8000B8070701103Full;
//DUPS: 8000B8070701143F, 8000B8090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_O_PL          = 0x8000B8070801103Full;
//DUPS: 8000B8070801143F, 8000B8090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_O_PL           = 0x8000B8070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_O_PL          = 0x8000B8070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B8070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B8070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B8090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B8070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B8070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B8090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_O_PL         = 0x8000B8070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_O_PL         = 0x8000B8070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_O_PL         = 0x8000B8090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_O_PL         = 0x8000B8070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B8070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B8090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL9_O_PL            = 0x8000C0070701103Full;
//DUPS: 8000C0070701143F, 8000C0090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL9_O_PL          = 0x8000C0070701103Full;
//DUPS: 8000C0070701143F, 8000C0090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL9_O_PL          = 0x8000C0070801103Full;
//DUPS: 8000C0070801143F, 8000C0090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL9_O_PL           = 0x8000C0070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL9_O_PL          = 0x8000C0070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C0070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C0070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C0090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C0070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C0070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C0090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL9_O_PL         = 0x8000C0070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL9_O_PL         = 0x8000C0070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL9_O_PL         = 0x8000C0090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL9_O_PL         = 0x8000C0070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C0070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C0090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_DATA_DAC_SPARE_MODE_PL    = 0x800000070701103Full;
//DUPS: 800000070701143F, 800000090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000070701103Full;
//DUPS: 800000070701143F, 800000090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000070801103Full;
//DUPS: 800000070801143F, 800000090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_DATA_DAC_SPARE_MODE_PL   = 0x800000070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_FIR_ERROR_INJECT_PL       = 0x800218070701103Full;
//DUPS: 800218070701143F, 800218090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_FIR_ERROR_INJECT_PL     = 0x800218070701103Full;
//DUPS: 800218070701143F, 800218090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_FIR_ERROR_INJECT_PL     = 0x800218070801103Full;
//DUPS: 800218070801143F, 800218090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_FIR_ERROR_INJECT_PL      = 0x800218070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_FIR_ERROR_INJECT_PL     = 0x800218070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x800218070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x800218070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x800218090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x800218070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x800218070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x800218090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_FIR_ERROR_INJECT_PL    = 0x800218070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_FIR_ERROR_INJECT_PL    = 0x800218070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_FIR_ERROR_INJECT_PL    = 0x800218090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_FIR_ERROR_INJECT_PL    = 0x800218070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x800218070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x800218090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_FIR_MASK_PL               = 0x800210070701103Full;
//DUPS: 800210070701143F, 800210090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_FIR_MASK_PL             = 0x800210070701103Full;
//DUPS: 800210070701143F, 800210090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_FIR_MASK_PL             = 0x800210070801103Full;
//DUPS: 800210070801143F, 800210090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_FIR_MASK_PL              = 0x800210070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_FIR_MASK_PL             = 0x800210070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_FIR_MASK_PL           = 0x800210070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_FIR_MASK_PL           = 0x800210070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_FIR_MASK_PL           = 0x800210090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_FIR_MASK_PL           = 0x800210070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_FIR_MASK_PL           = 0x800210070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_FIR_MASK_PL           = 0x800210090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_FIR_MASK_PL            = 0x800210070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_FIR_MASK_PL            = 0x800210070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_FIR_MASK_PL            = 0x800210090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_FIR_MASK_PL            = 0x800210070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_FIR_MASK_PL           = 0x800210070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_FIR_MASK_PL           = 0x800210090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_FIR_PL                    = 0x800208070701103Full;
//DUPS: 800208070701143F, 800208090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_FIR_PL                  = 0x800208070701103Full;
//DUPS: 800208070701143F, 800208090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_FIR_PL                  = 0x800208070801103Full;
//DUPS: 800208070801143F, 800208090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_FIR_PL                   = 0x800208070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_FIR_PL                  = 0x800208070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_FIR_PL                = 0x800208070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_FIR_PL                = 0x800208070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_FIR_PL                = 0x800208090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_FIR_PL                = 0x800208070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_FIR_PL                = 0x800208070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_FIR_PL                = 0x800208090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_FIR_PL                 = 0x800208070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_FIR_PL                 = 0x800208070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_FIR_PL                 = 0x800208090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_FIR_PL                 = 0x800208070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_FIR_PL                = 0x800208070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_FIR_PL                = 0x800208090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE3_RX_SPARE_MODE_PL             = 0x800200070701103Full;
//DUPS: 800200070701143F, 800200090701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE3_RX_SPARE_MODE_PL           = 0x800200070701103Full;
//DUPS: 800200070701143F, 800200090701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE3_RX_SPARE_MODE_PL           = 0x800200070801103Full;
//DUPS: 800200070801143F, 800200090801183F,

static const uint64_t P9A_OMI_RX0_RXPACKS1_SLICE3_RX_SPARE_MODE_PL            = 0x800200070701103Full;

static const uint64_t P9A_OMIC_RX0_RXPACKS1_SLICE3_RX_SPARE_MODE_PL           = 0x800200070701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS1_SLICE3_RX_SPARE_MODE_PL         = 0x800200070701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS1_SLICE3_RX_SPARE_MODE_PL         = 0x800200070701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE3_RX_SPARE_MODE_PL         = 0x800200090701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS1_SLICE3_RX_SPARE_MODE_PL         = 0x800200070801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS1_SLICE3_RX_SPARE_MODE_PL         = 0x800200070801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE3_RX_SPARE_MODE_PL         = 0x800200090801183Full;

static const uint64_t P9A_OMI_0_RX0_RXPACKS1_SLICE3_RX_SPARE_MODE_PL          = 0x800200070701103Full;

static const uint64_t P9A_OMI_3_RX0_RXPACKS1_SLICE3_RX_SPARE_MODE_PL          = 0x800200070701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE3_RX_SPARE_MODE_PL          = 0x800200090701183Full;

static const uint64_t P9A_OMI_8_RX0_RXPACKS1_SLICE3_RX_SPARE_MODE_PL          = 0x800200070801103Full;

static const uint64_t P9A_OMI_11_RX0_RXPACKS1_SLICE3_RX_SPARE_MODE_PL         = 0x800200070801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE3_RX_SPARE_MODE_PL         = 0x800200090801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_BIT_CNTL2_EO_PL           = 0x8002480A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_BIT_CNTL2_EO_PL         = 0x8002480A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_BIT_CNTL2_EO_PL         = 0x8002480A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_BIT_CNTL2_EO_PL       = 0x8002480A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_BIT_CNTL2_EO_PL       = 0x8002480A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_BIT_CNTL2_EO_PL        = 0x8002480A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_BIT_CNTL2_EO_PL       = 0x8002480A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_BIT_CNTL3_EO_PL           = 0x8002500A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_BIT_CNTL3_EO_PL         = 0x8002500A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_BIT_CNTL3_EO_PL         = 0x8002500A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_BIT_CNTL3_EO_PL       = 0x8002500A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_BIT_CNTL3_EO_PL       = 0x8002500A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_BIT_CNTL3_EO_PL        = 0x8002500A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_BIT_CNTL3_EO_PL       = 0x8002500A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_BIT_CNTL4_EO_PL           = 0x8002580A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_BIT_CNTL4_EO_PL         = 0x8002580A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_BIT_CNTL4_EO_PL         = 0x8002580A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_BIT_CNTL4_EO_PL       = 0x8002580A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_BIT_CNTL4_EO_PL       = 0x8002580A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_BIT_CNTL4_EO_PL        = 0x8002580A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_BIT_CNTL4_EO_PL       = 0x8002580A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_BIT_CNTL5_EO_PL           = 0x8002600A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_BIT_CNTL5_EO_PL         = 0x8002600A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_BIT_CNTL5_EO_PL         = 0x8002600A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_BIT_CNTL5_EO_PL       = 0x8002600A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_BIT_CNTL5_EO_PL       = 0x8002600A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_BIT_CNTL5_EO_PL        = 0x8002600A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_BIT_CNTL5_EO_PL       = 0x8002600A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_BIT_CNTLX1_EO_PL          = 0x8002400A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_BIT_CNTLX1_EO_PL        = 0x8002400A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_BIT_CNTLX1_EO_PL        = 0x8002400A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_BIT_CNTLX1_EO_PL      = 0x8002400A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_BIT_CNTLX1_EO_PL      = 0x8002400A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_BIT_CNTLX1_EO_PL       = 0x8002400A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_BIT_CNTLX1_EO_PL      = 0x8002400A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_BIT_MODE1_EO_PL           = 0x8002200A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_BIT_MODE1_EO_PL         = 0x8002200A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_BIT_MODE1_EO_PL         = 0x8002200A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_BIT_MODE1_EO_PL       = 0x8002200A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_BIT_MODE1_EO_PL       = 0x8002200A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_BIT_MODE1_EO_PL        = 0x8002200A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_BIT_MODE1_EO_PL       = 0x8002200A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_BIT_MODE2_EO_PL           = 0x8002280A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_BIT_MODE2_EO_PL         = 0x8002280A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_BIT_MODE2_EO_PL         = 0x8002280A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_BIT_MODE2_EO_PL       = 0x8002280A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_BIT_MODE2_EO_PL       = 0x8002280A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_BIT_MODE2_EO_PL        = 0x8002280A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_BIT_MODE2_EO_PL       = 0x8002280A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_BIT_MODE3_EO_PL           = 0x8002300A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_BIT_MODE3_EO_PL         = 0x8002300A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_BIT_MODE3_EO_PL         = 0x8002300A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_BIT_MODE3_EO_PL       = 0x8002300A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_BIT_MODE3_EO_PL       = 0x8002300A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_BIT_MODE3_EO_PL        = 0x8002300A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_BIT_MODE3_EO_PL       = 0x8002300A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_BIT_MODE4_EO_PL           = 0x8002380A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_BIT_MODE4_EO_PL         = 0x8002380A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_BIT_MODE4_EO_PL         = 0x8002380A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_BIT_MODE4_EO_PL       = 0x8002380A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_BIT_MODE4_EO_PL       = 0x8002380A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_BIT_MODE4_EO_PL        = 0x8002380A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_BIT_MODE4_EO_PL       = 0x8002380A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_BIT_STAT1_EO_PL           = 0x8002680A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_BIT_STAT1_EO_PL         = 0x8002680A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_BIT_STAT1_EO_PL         = 0x8002680A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_BIT_STAT1_EO_PL       = 0x8002680A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_BIT_STAT1_EO_PL       = 0x8002680A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_BIT_STAT1_EO_PL        = 0x8002680A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_BIT_STAT1_EO_PL       = 0x8002680A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_BIT_STAT1_O_PL            = 0x8003180A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_BIT_STAT1_O_PL          = 0x8003180A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_BIT_STAT1_O_PL          = 0x8003180A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_BIT_STAT1_O_PL        = 0x8003180A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_BIT_STAT1_O_PL        = 0x8003180A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_BIT_STAT1_O_PL         = 0x8003180A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_BIT_STAT1_O_PL        = 0x8003180A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_BIT_STAT2_EO_PL           = 0x8002700A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_BIT_STAT2_EO_PL         = 0x8002700A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_BIT_STAT2_EO_PL         = 0x8002700A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_BIT_STAT2_EO_PL       = 0x8002700A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_BIT_STAT2_EO_PL       = 0x8002700A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_BIT_STAT2_EO_PL        = 0x8002700A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_BIT_STAT2_EO_PL       = 0x8002700A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_BIT_STAT3_EO_PL           = 0x8002780A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_BIT_STAT3_EO_PL         = 0x8002780A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_BIT_STAT3_EO_PL         = 0x8002780A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_BIT_STAT3_EO_PL       = 0x8002780A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_BIT_STAT3_EO_PL       = 0x8002780A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_BIT_STAT3_EO_PL        = 0x8002780A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_BIT_STAT3_EO_PL       = 0x8002780A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_BIT_STAT4_EO_PL           = 0x8002800A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_BIT_STAT4_EO_PL         = 0x8002800A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_BIT_STAT4_EO_PL         = 0x8002800A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_BIT_STAT4_EO_PL       = 0x8002800A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_BIT_STAT4_EO_PL       = 0x8002800A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_BIT_STAT4_EO_PL        = 0x8002800A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_BIT_STAT4_EO_PL       = 0x8002800A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL10_O_PL           = 0x8000C80A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL10_O_PL         = 0x8000C80A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL10_O_PL         = 0x8000C80A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL10_O_PL       = 0x8000C80A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL10_O_PL       = 0x8000C80A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL10_O_PL        = 0x8000C80A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL10_O_PL       = 0x8000C80A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL1_EO_PL           = 0x8000080A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL1_EO_PL         = 0x8000080A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL1_EO_PL         = 0x8000080A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL1_EO_PL       = 0x8000080A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL1_EO_PL       = 0x8000080A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL1_EO_PL        = 0x8000080A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL1_EO_PL       = 0x8000080A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL1_O_PL            = 0x8000800A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL1_O_PL          = 0x8000800A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL1_O_PL          = 0x8000800A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL1_O_PL        = 0x8000800A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL1_O_PL        = 0x8000800A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL1_O_PL         = 0x8000800A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL1_O_PL        = 0x8000800A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL2_EO_PL           = 0x8000100A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL2_EO_PL         = 0x8000100A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL2_EO_PL         = 0x8000100A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL2_EO_PL       = 0x8000100A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL2_EO_PL       = 0x8000100A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL2_EO_PL        = 0x8000100A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL2_EO_PL       = 0x8000100A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL2_O_PL            = 0x8000880A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL2_O_PL          = 0x8000880A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL2_O_PL          = 0x8000880A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL2_O_PL        = 0x8000880A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL2_O_PL        = 0x8000880A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL2_O_PL         = 0x8000880A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL2_O_PL        = 0x8000880A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL3_EO_PL           = 0x8000180A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL3_EO_PL         = 0x8000180A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL3_EO_PL         = 0x8000180A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL3_EO_PL       = 0x8000180A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL3_EO_PL       = 0x8000180A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL3_EO_PL        = 0x8000180A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL3_EO_PL       = 0x8000180A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL3_O_PL            = 0x8000900A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL3_O_PL          = 0x8000900A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL3_O_PL          = 0x8000900A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL3_O_PL        = 0x8000900A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL3_O_PL        = 0x8000900A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL3_O_PL         = 0x8000900A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL3_O_PL        = 0x8000900A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL4_EO_PL           = 0x8000200A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL4_EO_PL         = 0x8000200A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL4_EO_PL         = 0x8000200A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL4_EO_PL       = 0x8000200A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL4_EO_PL       = 0x8000200A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL4_EO_PL        = 0x8000200A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL4_EO_PL       = 0x8000200A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL4_O_PL            = 0x8000980A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL4_O_PL          = 0x8000980A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL4_O_PL          = 0x8000980A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL4_O_PL        = 0x8000980A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL4_O_PL        = 0x8000980A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL4_O_PL         = 0x8000980A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL4_O_PL        = 0x8000980A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL5_EO_PL           = 0x8000280A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL5_EO_PL         = 0x8000280A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL5_EO_PL         = 0x8000280A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL5_EO_PL       = 0x8000280A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL5_EO_PL       = 0x8000280A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL5_EO_PL        = 0x8000280A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL5_EO_PL       = 0x8000280A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL5_O_PL            = 0x8000A00A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL5_O_PL          = 0x8000A00A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL5_O_PL          = 0x8000A00A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL5_O_PL        = 0x8000A00A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL5_O_PL        = 0x8000A00A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL5_O_PL         = 0x8000A00A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL5_O_PL        = 0x8000A00A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL6_EO_PL           = 0x8000300A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL6_EO_PL         = 0x8000300A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL6_EO_PL         = 0x8000300A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL6_EO_PL       = 0x8000300A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL6_EO_PL       = 0x8000300A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL6_EO_PL        = 0x8000300A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL6_EO_PL       = 0x8000300A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL6_O_PL            = 0x8000A80A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL6_O_PL          = 0x8000A80A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL6_O_PL          = 0x8000A80A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL6_O_PL        = 0x8000A80A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL6_O_PL        = 0x8000A80A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL6_O_PL         = 0x8000A80A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL6_O_PL        = 0x8000A80A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL7_EO_PL           = 0x8000380A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL7_EO_PL         = 0x8000380A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL7_EO_PL         = 0x8000380A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL7_EO_PL       = 0x8000380A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL7_EO_PL       = 0x8000380A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL7_EO_PL        = 0x8000380A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL7_EO_PL       = 0x8000380A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL7_O_PL            = 0x8000B00A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL7_O_PL          = 0x8000B00A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL7_O_PL          = 0x8000B00A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL7_O_PL        = 0x8000B00A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL7_O_PL        = 0x8000B00A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL7_O_PL         = 0x8000B00A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL7_O_PL        = 0x8000B00A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL8_EO_PL           = 0x8000400A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL8_EO_PL         = 0x8000400A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL8_EO_PL         = 0x8000400A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL8_EO_PL       = 0x8000400A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL8_EO_PL       = 0x8000400A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL8_EO_PL        = 0x8000400A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL8_EO_PL       = 0x8000400A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL8_O_PL            = 0x8000B80A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL8_O_PL          = 0x8000B80A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL8_O_PL          = 0x8000B80A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL8_O_PL        = 0x8000B80A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL8_O_PL        = 0x8000B80A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL8_O_PL         = 0x8000B80A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL8_O_PL        = 0x8000B80A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL9_O_PL            = 0x8000C00A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL9_O_PL          = 0x8000C00A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL9_O_PL          = 0x8000C00A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL9_O_PL        = 0x8000C00A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL9_O_PL        = 0x8000C00A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL9_O_PL         = 0x8000C00A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_DAC_CNTL9_O_PL        = 0x8000C00A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_DATA_DAC_SPARE_MODE_PL    = 0x8000000A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_DATA_DAC_SPARE_MODE_PL  = 0x8000000A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_DATA_DAC_SPARE_MODE_PL  = 0x8000000A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_FIR_ERROR_INJECT_PL       = 0x8002180A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_FIR_ERROR_INJECT_PL     = 0x8002180A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_FIR_ERROR_INJECT_PL     = 0x8002180A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_FIR_ERROR_INJECT_PL   = 0x8002180A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_FIR_ERROR_INJECT_PL   = 0x8002180A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_FIR_ERROR_INJECT_PL    = 0x8002180A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_FIR_ERROR_INJECT_PL   = 0x8002180A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_FIR_MASK_PL               = 0x8002100A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_FIR_MASK_PL             = 0x8002100A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_FIR_MASK_PL             = 0x8002100A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_FIR_MASK_PL           = 0x8002100A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_FIR_MASK_PL           = 0x8002100A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_FIR_MASK_PL            = 0x8002100A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_FIR_MASK_PL           = 0x8002100A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_FIR_PL                    = 0x8002080A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_FIR_PL                  = 0x8002080A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_FIR_PL                  = 0x8002080A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_FIR_PL                = 0x8002080A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_FIR_PL                = 0x8002080A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_FIR_PL                 = 0x8002080A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_FIR_PL                = 0x8002080A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE4_RX_SPARE_MODE_PL             = 0x8002000A0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE4_RX_SPARE_MODE_PL           = 0x8002000A0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE4_RX_SPARE_MODE_PL           = 0x8002000A0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE4_RX_SPARE_MODE_PL         = 0x8002000A0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE4_RX_SPARE_MODE_PL         = 0x8002000A0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE4_RX_SPARE_MODE_PL          = 0x8002000A0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE4_RX_SPARE_MODE_PL         = 0x8002000A0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_BIT_CNTL2_EO_PL           = 0x8002480B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_BIT_CNTL2_EO_PL         = 0x8002480B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_BIT_CNTL2_EO_PL         = 0x8002480B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_BIT_CNTL2_EO_PL       = 0x8002480B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_BIT_CNTL2_EO_PL       = 0x8002480B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_BIT_CNTL2_EO_PL        = 0x8002480B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_BIT_CNTL2_EO_PL       = 0x8002480B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_BIT_CNTL3_EO_PL           = 0x8002500B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_BIT_CNTL3_EO_PL         = 0x8002500B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_BIT_CNTL3_EO_PL         = 0x8002500B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_BIT_CNTL3_EO_PL       = 0x8002500B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_BIT_CNTL3_EO_PL       = 0x8002500B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_BIT_CNTL3_EO_PL        = 0x8002500B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_BIT_CNTL3_EO_PL       = 0x8002500B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_BIT_CNTL4_EO_PL           = 0x8002580B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_BIT_CNTL4_EO_PL         = 0x8002580B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_BIT_CNTL4_EO_PL         = 0x8002580B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_BIT_CNTL4_EO_PL       = 0x8002580B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_BIT_CNTL4_EO_PL       = 0x8002580B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_BIT_CNTL4_EO_PL        = 0x8002580B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_BIT_CNTL4_EO_PL       = 0x8002580B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_BIT_CNTL5_EO_PL           = 0x8002600B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_BIT_CNTL5_EO_PL         = 0x8002600B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_BIT_CNTL5_EO_PL         = 0x8002600B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_BIT_CNTL5_EO_PL       = 0x8002600B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_BIT_CNTL5_EO_PL       = 0x8002600B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_BIT_CNTL5_EO_PL        = 0x8002600B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_BIT_CNTL5_EO_PL       = 0x8002600B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_BIT_CNTLX1_EO_PL          = 0x8002400B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_BIT_CNTLX1_EO_PL        = 0x8002400B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_BIT_CNTLX1_EO_PL        = 0x8002400B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_BIT_CNTLX1_EO_PL      = 0x8002400B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_BIT_CNTLX1_EO_PL      = 0x8002400B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_BIT_CNTLX1_EO_PL       = 0x8002400B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_BIT_CNTLX1_EO_PL      = 0x8002400B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_BIT_MODE1_EO_PL           = 0x8002200B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_BIT_MODE1_EO_PL         = 0x8002200B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_BIT_MODE1_EO_PL         = 0x8002200B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_BIT_MODE1_EO_PL       = 0x8002200B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_BIT_MODE1_EO_PL       = 0x8002200B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_BIT_MODE1_EO_PL        = 0x8002200B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_BIT_MODE1_EO_PL       = 0x8002200B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_BIT_MODE2_EO_PL           = 0x8002280B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_BIT_MODE2_EO_PL         = 0x8002280B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_BIT_MODE2_EO_PL         = 0x8002280B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_BIT_MODE2_EO_PL       = 0x8002280B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_BIT_MODE2_EO_PL       = 0x8002280B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_BIT_MODE2_EO_PL        = 0x8002280B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_BIT_MODE2_EO_PL       = 0x8002280B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_BIT_MODE3_EO_PL           = 0x8002300B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_BIT_MODE3_EO_PL         = 0x8002300B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_BIT_MODE3_EO_PL         = 0x8002300B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_BIT_MODE3_EO_PL       = 0x8002300B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_BIT_MODE3_EO_PL       = 0x8002300B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_BIT_MODE3_EO_PL        = 0x8002300B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_BIT_MODE3_EO_PL       = 0x8002300B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_BIT_MODE4_EO_PL           = 0x8002380B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_BIT_MODE4_EO_PL         = 0x8002380B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_BIT_MODE4_EO_PL         = 0x8002380B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_BIT_MODE4_EO_PL       = 0x8002380B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_BIT_MODE4_EO_PL       = 0x8002380B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_BIT_MODE4_EO_PL        = 0x8002380B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_BIT_MODE4_EO_PL       = 0x8002380B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_BIT_STAT1_EO_PL           = 0x8002680B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_BIT_STAT1_EO_PL         = 0x8002680B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_BIT_STAT1_EO_PL         = 0x8002680B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_BIT_STAT1_EO_PL       = 0x8002680B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_BIT_STAT1_EO_PL       = 0x8002680B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_BIT_STAT1_EO_PL        = 0x8002680B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_BIT_STAT1_EO_PL       = 0x8002680B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_BIT_STAT1_O_PL            = 0x8003180B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_BIT_STAT1_O_PL          = 0x8003180B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_BIT_STAT1_O_PL          = 0x8003180B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_BIT_STAT1_O_PL        = 0x8003180B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_BIT_STAT1_O_PL        = 0x8003180B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_BIT_STAT1_O_PL         = 0x8003180B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_BIT_STAT1_O_PL        = 0x8003180B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_BIT_STAT2_EO_PL           = 0x8002700B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_BIT_STAT2_EO_PL         = 0x8002700B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_BIT_STAT2_EO_PL         = 0x8002700B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_BIT_STAT2_EO_PL       = 0x8002700B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_BIT_STAT2_EO_PL       = 0x8002700B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_BIT_STAT2_EO_PL        = 0x8002700B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_BIT_STAT2_EO_PL       = 0x8002700B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_BIT_STAT3_EO_PL           = 0x8002780B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_BIT_STAT3_EO_PL         = 0x8002780B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_BIT_STAT3_EO_PL         = 0x8002780B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_BIT_STAT3_EO_PL       = 0x8002780B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_BIT_STAT3_EO_PL       = 0x8002780B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_BIT_STAT3_EO_PL        = 0x8002780B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_BIT_STAT3_EO_PL       = 0x8002780B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_BIT_STAT4_EO_PL           = 0x8002800B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_BIT_STAT4_EO_PL         = 0x8002800B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_BIT_STAT4_EO_PL         = 0x8002800B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_BIT_STAT4_EO_PL       = 0x8002800B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_BIT_STAT4_EO_PL       = 0x8002800B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_BIT_STAT4_EO_PL        = 0x8002800B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_BIT_STAT4_EO_PL       = 0x8002800B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL10_O_PL           = 0x8000C80B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL10_O_PL         = 0x8000C80B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL10_O_PL         = 0x8000C80B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL10_O_PL       = 0x8000C80B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL10_O_PL       = 0x8000C80B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL10_O_PL        = 0x8000C80B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL10_O_PL       = 0x8000C80B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL1_EO_PL           = 0x8000080B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL1_EO_PL         = 0x8000080B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL1_EO_PL         = 0x8000080B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL1_EO_PL       = 0x8000080B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL1_EO_PL       = 0x8000080B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL1_EO_PL        = 0x8000080B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL1_EO_PL       = 0x8000080B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL1_O_PL            = 0x8000800B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL1_O_PL          = 0x8000800B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL1_O_PL          = 0x8000800B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL1_O_PL        = 0x8000800B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL1_O_PL        = 0x8000800B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL1_O_PL         = 0x8000800B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL1_O_PL        = 0x8000800B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL2_EO_PL           = 0x8000100B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL2_EO_PL         = 0x8000100B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL2_EO_PL         = 0x8000100B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL2_EO_PL       = 0x8000100B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL2_EO_PL       = 0x8000100B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL2_EO_PL        = 0x8000100B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL2_EO_PL       = 0x8000100B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL2_O_PL            = 0x8000880B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL2_O_PL          = 0x8000880B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL2_O_PL          = 0x8000880B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL2_O_PL        = 0x8000880B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL2_O_PL        = 0x8000880B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL2_O_PL         = 0x8000880B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL2_O_PL        = 0x8000880B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL3_EO_PL           = 0x8000180B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL3_EO_PL         = 0x8000180B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL3_EO_PL         = 0x8000180B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL3_EO_PL       = 0x8000180B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL3_EO_PL       = 0x8000180B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL3_EO_PL        = 0x8000180B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL3_EO_PL       = 0x8000180B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL3_O_PL            = 0x8000900B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL3_O_PL          = 0x8000900B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL3_O_PL          = 0x8000900B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL3_O_PL        = 0x8000900B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL3_O_PL        = 0x8000900B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL3_O_PL         = 0x8000900B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL3_O_PL        = 0x8000900B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL4_EO_PL           = 0x8000200B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL4_EO_PL         = 0x8000200B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL4_EO_PL         = 0x8000200B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL4_EO_PL       = 0x8000200B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL4_EO_PL       = 0x8000200B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL4_EO_PL        = 0x8000200B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL4_EO_PL       = 0x8000200B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL4_O_PL            = 0x8000980B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL4_O_PL          = 0x8000980B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL4_O_PL          = 0x8000980B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL4_O_PL        = 0x8000980B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL4_O_PL        = 0x8000980B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL4_O_PL         = 0x8000980B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL4_O_PL        = 0x8000980B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL5_EO_PL           = 0x8000280B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL5_EO_PL         = 0x8000280B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL5_EO_PL         = 0x8000280B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL5_EO_PL       = 0x8000280B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL5_EO_PL       = 0x8000280B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL5_EO_PL        = 0x8000280B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL5_EO_PL       = 0x8000280B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL5_O_PL            = 0x8000A00B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL5_O_PL          = 0x8000A00B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL5_O_PL          = 0x8000A00B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL5_O_PL        = 0x8000A00B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL5_O_PL        = 0x8000A00B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL5_O_PL         = 0x8000A00B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL5_O_PL        = 0x8000A00B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL6_EO_PL           = 0x8000300B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL6_EO_PL         = 0x8000300B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL6_EO_PL         = 0x8000300B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL6_EO_PL       = 0x8000300B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL6_EO_PL       = 0x8000300B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL6_EO_PL        = 0x8000300B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL6_EO_PL       = 0x8000300B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL6_O_PL            = 0x8000A80B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL6_O_PL          = 0x8000A80B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL6_O_PL          = 0x8000A80B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL6_O_PL        = 0x8000A80B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL6_O_PL        = 0x8000A80B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL6_O_PL         = 0x8000A80B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL6_O_PL        = 0x8000A80B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL7_EO_PL           = 0x8000380B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL7_EO_PL         = 0x8000380B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL7_EO_PL         = 0x8000380B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL7_EO_PL       = 0x8000380B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL7_EO_PL       = 0x8000380B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL7_EO_PL        = 0x8000380B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL7_EO_PL       = 0x8000380B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL7_O_PL            = 0x8000B00B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL7_O_PL          = 0x8000B00B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL7_O_PL          = 0x8000B00B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL7_O_PL        = 0x8000B00B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL7_O_PL        = 0x8000B00B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL7_O_PL         = 0x8000B00B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL7_O_PL        = 0x8000B00B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL8_EO_PL           = 0x8000400B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL8_EO_PL         = 0x8000400B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL8_EO_PL         = 0x8000400B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL8_EO_PL       = 0x8000400B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL8_EO_PL       = 0x8000400B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL8_EO_PL        = 0x8000400B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL8_EO_PL       = 0x8000400B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL8_O_PL            = 0x8000B80B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL8_O_PL          = 0x8000B80B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL8_O_PL          = 0x8000B80B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL8_O_PL        = 0x8000B80B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL8_O_PL        = 0x8000B80B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL8_O_PL         = 0x8000B80B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL8_O_PL        = 0x8000B80B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL9_O_PL            = 0x8000C00B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL9_O_PL          = 0x8000C00B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL9_O_PL          = 0x8000C00B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL9_O_PL        = 0x8000C00B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL9_O_PL        = 0x8000C00B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL9_O_PL         = 0x8000C00B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_DAC_CNTL9_O_PL        = 0x8000C00B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_DATA_DAC_SPARE_MODE_PL    = 0x8000000B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_DATA_DAC_SPARE_MODE_PL  = 0x8000000B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_DATA_DAC_SPARE_MODE_PL  = 0x8000000B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_FIR_ERROR_INJECT_PL       = 0x8002180B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_FIR_ERROR_INJECT_PL     = 0x8002180B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_FIR_ERROR_INJECT_PL     = 0x8002180B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_FIR_ERROR_INJECT_PL   = 0x8002180B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_FIR_ERROR_INJECT_PL   = 0x8002180B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_FIR_ERROR_INJECT_PL    = 0x8002180B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_FIR_ERROR_INJECT_PL   = 0x8002180B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_FIR_MASK_PL               = 0x8002100B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_FIR_MASK_PL             = 0x8002100B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_FIR_MASK_PL             = 0x8002100B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_FIR_MASK_PL           = 0x8002100B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_FIR_MASK_PL           = 0x8002100B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_FIR_MASK_PL            = 0x8002100B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_FIR_MASK_PL           = 0x8002100B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_FIR_PL                    = 0x8002080B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_FIR_PL                  = 0x8002080B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_FIR_PL                  = 0x8002080B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_FIR_PL                = 0x8002080B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_FIR_PL                = 0x8002080B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_FIR_PL                 = 0x8002080B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_FIR_PL                = 0x8002080B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS1_SLICE5_RX_SPARE_MODE_PL             = 0x8002000B0701183Full;

static const uint64_t P9A_MC_0_RX0_RXPACKS1_SLICE5_RX_SPARE_MODE_PL           = 0x8002000B0701183Full;

static const uint64_t P9A_MC_1_RX0_RXPACKS1_SLICE5_RX_SPARE_MODE_PL           = 0x8002000B0801183Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS1_SLICE5_RX_SPARE_MODE_PL         = 0x8002000B0701183Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS1_SLICE5_RX_SPARE_MODE_PL         = 0x8002000B0801183Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS1_SLICE5_RX_SPARE_MODE_PL          = 0x8002000B0701183Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS1_SLICE5_RX_SPARE_MODE_PL         = 0x8002000B0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL2_EO_PL           = 0x8002480C0701183Full;
//DUPS: 800248080701143F, 8002480C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL2_EO_PL         = 0x8002480C0701183Full;
//DUPS: 800248080701143F, 8002480C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL2_EO_PL         = 0x8002480C0801183Full;
//DUPS: 800248080801143F, 8002480C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL2_EO_PL         = 0x800248080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x800248080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x800248080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x8002480C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x800248080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x800248080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x8002480C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL2_EO_PL        = 0x800248080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL2_EO_PL        = 0x800248080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL2_EO_PL        = 0x8002480C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL2_EO_PL        = 0x800248080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x800248080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x8002480C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL3_EO_PL           = 0x8002500C0701183Full;
//DUPS: 800250080701143F, 8002500C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL3_EO_PL         = 0x8002500C0701183Full;
//DUPS: 800250080701143F, 8002500C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL3_EO_PL         = 0x8002500C0801183Full;
//DUPS: 800250080801143F, 8002500C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL3_EO_PL         = 0x800250080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x800250080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x800250080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x8002500C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x800250080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x800250080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x8002500C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL3_EO_PL        = 0x800250080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL3_EO_PL        = 0x800250080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL3_EO_PL        = 0x8002500C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL3_EO_PL        = 0x800250080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x800250080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x8002500C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL4_EO_PL           = 0x8002580C0701183Full;
//DUPS: 800258080701143F, 8002580C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL4_EO_PL         = 0x8002580C0701183Full;
//DUPS: 800258080701143F, 8002580C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL4_EO_PL         = 0x8002580C0801183Full;
//DUPS: 800258080801143F, 8002580C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL4_EO_PL         = 0x800258080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x800258080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x800258080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x8002580C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x800258080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x800258080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x8002580C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL4_EO_PL        = 0x800258080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL4_EO_PL        = 0x800258080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL4_EO_PL        = 0x8002580C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL4_EO_PL        = 0x800258080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x800258080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x8002580C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL5_EO_PL           = 0x8002600C0701183Full;
//DUPS: 800260080701143F, 8002600C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL5_EO_PL         = 0x8002600C0701183Full;
//DUPS: 800260080701143F, 8002600C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL5_EO_PL         = 0x8002600C0801183Full;
//DUPS: 800260080801143F, 8002600C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL5_EO_PL         = 0x800260080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x800260080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x800260080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x8002600C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x800260080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x800260080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x8002600C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL5_EO_PL        = 0x800260080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL5_EO_PL        = 0x800260080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL5_EO_PL        = 0x8002600C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL5_EO_PL        = 0x800260080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x800260080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x8002600C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_BIT_CNTLX1_EO_PL          = 0x8002400C0701183Full;
//DUPS: 800240080701143F, 8002400C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_BIT_CNTLX1_EO_PL        = 0x8002400C0701183Full;
//DUPS: 800240080701143F, 8002400C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_BIT_CNTLX1_EO_PL        = 0x8002400C0801183Full;
//DUPS: 800240080801143F, 8002400C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_BIT_CNTLX1_EO_PL        = 0x800240080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x800240080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x800240080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x8002400C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x800240080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x800240080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x8002400C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_BIT_CNTLX1_EO_PL       = 0x800240080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_BIT_CNTLX1_EO_PL       = 0x800240080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_BIT_CNTLX1_EO_PL       = 0x8002400C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_BIT_CNTLX1_EO_PL       = 0x800240080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x800240080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x8002400C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_BIT_MODE1_EO_PL           = 0x8002200C0701183Full;
//DUPS: 800220080701143F, 8002200C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_BIT_MODE1_EO_PL         = 0x8002200C0701183Full;
//DUPS: 800220080701143F, 8002200C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_BIT_MODE1_EO_PL         = 0x8002200C0801183Full;
//DUPS: 800220080801143F, 8002200C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_BIT_MODE1_EO_PL         = 0x800220080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_BIT_MODE1_EO_PL       = 0x800220080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_BIT_MODE1_EO_PL       = 0x800220080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_BIT_MODE1_EO_PL       = 0x8002200C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_BIT_MODE1_EO_PL       = 0x800220080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_BIT_MODE1_EO_PL       = 0x800220080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_BIT_MODE1_EO_PL       = 0x8002200C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_BIT_MODE1_EO_PL        = 0x800220080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_BIT_MODE1_EO_PL        = 0x800220080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_BIT_MODE1_EO_PL        = 0x8002200C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_BIT_MODE1_EO_PL        = 0x800220080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_BIT_MODE1_EO_PL       = 0x800220080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_BIT_MODE1_EO_PL       = 0x8002200C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_BIT_MODE2_EO_PL           = 0x8002280C0701183Full;
//DUPS: 800228080701143F, 8002280C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_BIT_MODE2_EO_PL         = 0x8002280C0701183Full;
//DUPS: 800228080701143F, 8002280C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_BIT_MODE2_EO_PL         = 0x8002280C0801183Full;
//DUPS: 800228080801143F, 8002280C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_BIT_MODE2_EO_PL         = 0x800228080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_BIT_MODE2_EO_PL       = 0x800228080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_BIT_MODE2_EO_PL       = 0x800228080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_BIT_MODE2_EO_PL       = 0x8002280C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_BIT_MODE2_EO_PL       = 0x800228080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_BIT_MODE2_EO_PL       = 0x800228080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_BIT_MODE2_EO_PL       = 0x8002280C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_BIT_MODE2_EO_PL        = 0x800228080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_BIT_MODE2_EO_PL        = 0x800228080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_BIT_MODE2_EO_PL        = 0x8002280C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_BIT_MODE2_EO_PL        = 0x800228080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_BIT_MODE2_EO_PL       = 0x800228080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_BIT_MODE2_EO_PL       = 0x8002280C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_BIT_MODE3_EO_PL           = 0x8002300C0701183Full;
//DUPS: 800230080701143F, 8002300C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_BIT_MODE3_EO_PL         = 0x8002300C0701183Full;
//DUPS: 800230080701143F, 8002300C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_BIT_MODE3_EO_PL         = 0x8002300C0801183Full;
//DUPS: 800230080801143F, 8002300C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_BIT_MODE3_EO_PL         = 0x800230080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_BIT_MODE3_EO_PL       = 0x800230080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_BIT_MODE3_EO_PL       = 0x800230080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_BIT_MODE3_EO_PL       = 0x8002300C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_BIT_MODE3_EO_PL       = 0x800230080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_BIT_MODE3_EO_PL       = 0x800230080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_BIT_MODE3_EO_PL       = 0x8002300C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_BIT_MODE3_EO_PL        = 0x800230080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_BIT_MODE3_EO_PL        = 0x800230080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_BIT_MODE3_EO_PL        = 0x8002300C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_BIT_MODE3_EO_PL        = 0x800230080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_BIT_MODE3_EO_PL       = 0x800230080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_BIT_MODE3_EO_PL       = 0x8002300C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_BIT_MODE4_EO_PL           = 0x8002380C0701183Full;
//DUPS: 800238080701143F, 8002380C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_BIT_MODE4_EO_PL         = 0x8002380C0701183Full;
//DUPS: 800238080701143F, 8002380C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_BIT_MODE4_EO_PL         = 0x8002380C0801183Full;
//DUPS: 800238080801143F, 8002380C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_BIT_MODE4_EO_PL         = 0x800238080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_BIT_MODE4_EO_PL       = 0x800238080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_BIT_MODE4_EO_PL       = 0x800238080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_BIT_MODE4_EO_PL       = 0x8002380C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_BIT_MODE4_EO_PL       = 0x800238080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_BIT_MODE4_EO_PL       = 0x800238080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_BIT_MODE4_EO_PL       = 0x8002380C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_BIT_MODE4_EO_PL        = 0x800238080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_BIT_MODE4_EO_PL        = 0x800238080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_BIT_MODE4_EO_PL        = 0x8002380C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_BIT_MODE4_EO_PL        = 0x800238080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_BIT_MODE4_EO_PL       = 0x800238080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_BIT_MODE4_EO_PL       = 0x8002380C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_BIT_STAT1_EO_PL           = 0x8002680C0701183Full;
//DUPS: 800268080701143F, 8002680C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_BIT_STAT1_EO_PL         = 0x8002680C0701183Full;
//DUPS: 800268080701143F, 8002680C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_BIT_STAT1_EO_PL         = 0x8002680C0801183Full;
//DUPS: 800268080801143F, 8002680C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_BIT_STAT1_EO_PL         = 0x800268080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_BIT_STAT1_EO_PL       = 0x800268080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_BIT_STAT1_EO_PL       = 0x800268080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_BIT_STAT1_EO_PL       = 0x8002680C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_BIT_STAT1_EO_PL       = 0x800268080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_BIT_STAT1_EO_PL       = 0x800268080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_BIT_STAT1_EO_PL       = 0x8002680C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_BIT_STAT1_EO_PL        = 0x800268080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_BIT_STAT1_EO_PL        = 0x800268080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_BIT_STAT1_EO_PL        = 0x8002680C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_BIT_STAT1_EO_PL        = 0x800268080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_BIT_STAT1_EO_PL       = 0x800268080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_BIT_STAT1_EO_PL       = 0x8002680C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_BIT_STAT1_O_PL            = 0x8003180C0701183Full;
//DUPS: 800318080701143F, 8003180C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_BIT_STAT1_O_PL          = 0x8003180C0701183Full;
//DUPS: 800318080701143F, 8003180C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_BIT_STAT1_O_PL          = 0x8003180C0801183Full;
//DUPS: 800318080801143F, 8003180C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_BIT_STAT1_O_PL          = 0x800318080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_BIT_STAT1_O_PL        = 0x800318080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_BIT_STAT1_O_PL        = 0x800318080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_BIT_STAT1_O_PL        = 0x8003180C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_BIT_STAT1_O_PL        = 0x800318080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_BIT_STAT1_O_PL        = 0x800318080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_BIT_STAT1_O_PL        = 0x8003180C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_BIT_STAT1_O_PL         = 0x800318080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_BIT_STAT1_O_PL         = 0x800318080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_BIT_STAT1_O_PL         = 0x8003180C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_BIT_STAT1_O_PL         = 0x800318080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_BIT_STAT1_O_PL        = 0x800318080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_BIT_STAT1_O_PL        = 0x8003180C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_BIT_STAT2_EO_PL           = 0x8002700C0701183Full;
//DUPS: 800270080701143F, 8002700C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_BIT_STAT2_EO_PL         = 0x8002700C0701183Full;
//DUPS: 800270080701143F, 8002700C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_BIT_STAT2_EO_PL         = 0x8002700C0801183Full;
//DUPS: 800270080801143F, 8002700C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_BIT_STAT2_EO_PL         = 0x800270080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_BIT_STAT2_EO_PL       = 0x800270080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_BIT_STAT2_EO_PL       = 0x800270080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_BIT_STAT2_EO_PL       = 0x8002700C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_BIT_STAT2_EO_PL       = 0x800270080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_BIT_STAT2_EO_PL       = 0x800270080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_BIT_STAT2_EO_PL       = 0x8002700C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_BIT_STAT2_EO_PL        = 0x800270080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_BIT_STAT2_EO_PL        = 0x800270080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_BIT_STAT2_EO_PL        = 0x8002700C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_BIT_STAT2_EO_PL        = 0x800270080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_BIT_STAT2_EO_PL       = 0x800270080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_BIT_STAT2_EO_PL       = 0x8002700C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_BIT_STAT3_EO_PL           = 0x8002780C0701183Full;
//DUPS: 800278080701143F, 8002780C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_BIT_STAT3_EO_PL         = 0x8002780C0701183Full;
//DUPS: 800278080701143F, 8002780C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_BIT_STAT3_EO_PL         = 0x8002780C0801183Full;
//DUPS: 800278080801143F, 8002780C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_BIT_STAT3_EO_PL         = 0x800278080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_BIT_STAT3_EO_PL       = 0x800278080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_BIT_STAT3_EO_PL       = 0x800278080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_BIT_STAT3_EO_PL       = 0x8002780C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_BIT_STAT3_EO_PL       = 0x800278080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_BIT_STAT3_EO_PL       = 0x800278080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_BIT_STAT3_EO_PL       = 0x8002780C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_BIT_STAT3_EO_PL        = 0x800278080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_BIT_STAT3_EO_PL        = 0x800278080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_BIT_STAT3_EO_PL        = 0x8002780C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_BIT_STAT3_EO_PL        = 0x800278080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_BIT_STAT3_EO_PL       = 0x800278080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_BIT_STAT3_EO_PL       = 0x8002780C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_BIT_STAT4_EO_PL           = 0x8002800C0701183Full;
//DUPS: 800280080701143F, 8002800C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_BIT_STAT4_EO_PL         = 0x8002800C0701183Full;
//DUPS: 800280080701143F, 8002800C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_BIT_STAT4_EO_PL         = 0x8002800C0801183Full;
//DUPS: 800280080801143F, 8002800C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_BIT_STAT4_EO_PL         = 0x800280080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_BIT_STAT4_EO_PL       = 0x800280080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_BIT_STAT4_EO_PL       = 0x800280080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_BIT_STAT4_EO_PL       = 0x8002800C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_BIT_STAT4_EO_PL       = 0x800280080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_BIT_STAT4_EO_PL       = 0x800280080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_BIT_STAT4_EO_PL       = 0x8002800C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_BIT_STAT4_EO_PL        = 0x800280080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_BIT_STAT4_EO_PL        = 0x800280080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_BIT_STAT4_EO_PL        = 0x8002800C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_BIT_STAT4_EO_PL        = 0x800280080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_BIT_STAT4_EO_PL       = 0x800280080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_BIT_STAT4_EO_PL       = 0x8002800C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL10_O_PL           = 0x8000C8080701103Full;
//DUPS: 8000C8080701143F, 8000C80C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL10_O_PL         = 0x8000C8080701103Full;
//DUPS: 8000C8080701143F, 8000C80C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL10_O_PL         = 0x8000C8080801103Full;
//DUPS: 8000C8080801143F, 8000C80C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL10_O_PL         = 0x8000C8080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C8080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C8080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C80C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C8080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C8080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C80C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL10_O_PL        = 0x8000C8080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL10_O_PL        = 0x8000C8080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL10_O_PL        = 0x8000C80C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL10_O_PL        = 0x8000C8080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C8080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C80C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL1_EO_PL           = 0x8000080C0701183Full;
//DUPS: 800008080701143F, 8000080C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL1_EO_PL         = 0x8000080C0701183Full;
//DUPS: 800008080701143F, 8000080C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL1_EO_PL         = 0x8000080C0801183Full;
//DUPS: 800008080801143F, 8000080C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL1_EO_PL         = 0x800008080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x800008080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x800008080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x8000080C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x800008080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x800008080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x8000080C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL1_EO_PL        = 0x800008080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL1_EO_PL        = 0x800008080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL1_EO_PL        = 0x8000080C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL1_EO_PL        = 0x800008080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x800008080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x8000080C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL1_O_PL            = 0x8000800C0701183Full;
//DUPS: 800080080701143F, 8000800C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL1_O_PL          = 0x8000800C0701183Full;
//DUPS: 800080080701143F, 8000800C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL1_O_PL          = 0x8000800C0801183Full;
//DUPS: 800080080801143F, 8000800C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL1_O_PL          = 0x800080080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL1_O_PL        = 0x800080080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL1_O_PL        = 0x800080080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL1_O_PL        = 0x8000800C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL1_O_PL        = 0x800080080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL1_O_PL        = 0x800080080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL1_O_PL        = 0x8000800C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL1_O_PL         = 0x800080080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL1_O_PL         = 0x800080080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL1_O_PL         = 0x8000800C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL1_O_PL         = 0x800080080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL1_O_PL        = 0x800080080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL1_O_PL        = 0x8000800C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL2_EO_PL           = 0x8000100C0701183Full;
//DUPS: 800010080701143F, 8000100C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL2_EO_PL         = 0x8000100C0701183Full;
//DUPS: 800010080701143F, 8000100C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL2_EO_PL         = 0x8000100C0801183Full;
//DUPS: 800010080801143F, 8000100C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL2_EO_PL         = 0x800010080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x800010080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x800010080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x8000100C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x800010080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x800010080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x8000100C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL2_EO_PL        = 0x800010080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL2_EO_PL        = 0x800010080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL2_EO_PL        = 0x8000100C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL2_EO_PL        = 0x800010080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x800010080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x8000100C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL2_O_PL            = 0x8000880C0701183Full;
//DUPS: 800088080701143F, 8000880C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL2_O_PL          = 0x8000880C0701183Full;
//DUPS: 800088080701143F, 8000880C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL2_O_PL          = 0x8000880C0801183Full;
//DUPS: 800088080801143F, 8000880C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL2_O_PL          = 0x800088080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL2_O_PL        = 0x800088080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL2_O_PL        = 0x800088080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL2_O_PL        = 0x8000880C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL2_O_PL        = 0x800088080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL2_O_PL        = 0x800088080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL2_O_PL        = 0x8000880C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL2_O_PL         = 0x800088080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL2_O_PL         = 0x800088080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL2_O_PL         = 0x8000880C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL2_O_PL         = 0x800088080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL2_O_PL        = 0x800088080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL2_O_PL        = 0x8000880C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL3_EO_PL           = 0x8000180C0701183Full;
//DUPS: 800018080701143F, 8000180C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL3_EO_PL         = 0x8000180C0701183Full;
//DUPS: 800018080701143F, 8000180C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL3_EO_PL         = 0x8000180C0801183Full;
//DUPS: 800018080801143F, 8000180C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL3_EO_PL         = 0x800018080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x800018080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x800018080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x8000180C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x800018080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x800018080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x8000180C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL3_EO_PL        = 0x800018080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL3_EO_PL        = 0x800018080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL3_EO_PL        = 0x8000180C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL3_EO_PL        = 0x800018080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x800018080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x8000180C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL3_O_PL            = 0x8000900C0701183Full;
//DUPS: 800090080701143F, 8000900C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL3_O_PL          = 0x8000900C0701183Full;
//DUPS: 800090080701143F, 8000900C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL3_O_PL          = 0x8000900C0801183Full;
//DUPS: 800090080801143F, 8000900C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL3_O_PL          = 0x800090080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL3_O_PL        = 0x800090080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL3_O_PL        = 0x800090080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL3_O_PL        = 0x8000900C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL3_O_PL        = 0x800090080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL3_O_PL        = 0x800090080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL3_O_PL        = 0x8000900C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL3_O_PL         = 0x800090080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL3_O_PL         = 0x800090080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL3_O_PL         = 0x8000900C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL3_O_PL         = 0x800090080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL3_O_PL        = 0x800090080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL3_O_PL        = 0x8000900C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL4_EO_PL           = 0x8000200C0701183Full;
//DUPS: 800020080701143F, 8000200C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL4_EO_PL         = 0x8000200C0701183Full;
//DUPS: 800020080701143F, 8000200C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL4_EO_PL         = 0x8000200C0801183Full;
//DUPS: 800020080801143F, 8000200C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL4_EO_PL         = 0x800020080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x800020080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x800020080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x8000200C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x800020080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x800020080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x8000200C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL4_EO_PL        = 0x800020080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL4_EO_PL        = 0x800020080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL4_EO_PL        = 0x8000200C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL4_EO_PL        = 0x800020080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x800020080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x8000200C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL4_O_PL            = 0x8000980C0701183Full;
//DUPS: 800098080701143F, 8000980C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL4_O_PL          = 0x8000980C0701183Full;
//DUPS: 800098080701143F, 8000980C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL4_O_PL          = 0x8000980C0801183Full;
//DUPS: 800098080801143F, 8000980C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL4_O_PL          = 0x800098080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL4_O_PL        = 0x800098080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL4_O_PL        = 0x800098080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL4_O_PL        = 0x8000980C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL4_O_PL        = 0x800098080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL4_O_PL        = 0x800098080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL4_O_PL        = 0x8000980C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL4_O_PL         = 0x800098080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL4_O_PL         = 0x800098080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL4_O_PL         = 0x8000980C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL4_O_PL         = 0x800098080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL4_O_PL        = 0x800098080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL4_O_PL        = 0x8000980C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL5_EO_PL           = 0x8000280C0701183Full;
//DUPS: 800028080701143F, 8000280C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL5_EO_PL         = 0x8000280C0701183Full;
//DUPS: 800028080701143F, 8000280C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL5_EO_PL         = 0x8000280C0801183Full;
//DUPS: 800028080801143F, 8000280C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL5_EO_PL         = 0x800028080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x800028080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x800028080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x8000280C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x800028080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x800028080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x8000280C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL5_EO_PL        = 0x800028080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL5_EO_PL        = 0x800028080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL5_EO_PL        = 0x8000280C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL5_EO_PL        = 0x800028080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x800028080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x8000280C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL5_O_PL            = 0x8000A0080701103Full;
//DUPS: 8000A0080701143F, 8000A00C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL5_O_PL          = 0x8000A0080701103Full;
//DUPS: 8000A0080701143F, 8000A00C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL5_O_PL          = 0x8000A0080801103Full;
//DUPS: 8000A0080801143F, 8000A00C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL5_O_PL          = 0x8000A0080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A0080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A0080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A00C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A0080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A0080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A00C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL5_O_PL         = 0x8000A0080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL5_O_PL         = 0x8000A0080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL5_O_PL         = 0x8000A00C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL5_O_PL         = 0x8000A0080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A0080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A00C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL6_EO_PL           = 0x8000300C0701183Full;
//DUPS: 800030080701143F, 8000300C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL6_EO_PL         = 0x8000300C0701183Full;
//DUPS: 800030080701143F, 8000300C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL6_EO_PL         = 0x8000300C0801183Full;
//DUPS: 800030080801143F, 8000300C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL6_EO_PL         = 0x800030080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x800030080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x800030080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x8000300C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x800030080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x800030080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x8000300C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL6_EO_PL        = 0x800030080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL6_EO_PL        = 0x800030080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL6_EO_PL        = 0x8000300C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL6_EO_PL        = 0x800030080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x800030080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x8000300C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL6_O_PL            = 0x8000A8080701103Full;
//DUPS: 8000A8080701143F, 8000A80C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL6_O_PL          = 0x8000A8080701103Full;
//DUPS: 8000A8080701143F, 8000A80C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL6_O_PL          = 0x8000A8080801103Full;
//DUPS: 8000A8080801143F, 8000A80C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL6_O_PL          = 0x8000A8080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A8080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A8080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A80C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A8080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A8080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A80C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL6_O_PL         = 0x8000A8080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL6_O_PL         = 0x8000A8080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL6_O_PL         = 0x8000A80C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL6_O_PL         = 0x8000A8080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A8080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A80C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL7_EO_PL           = 0x8000380C0701183Full;
//DUPS: 800038080701143F, 8000380C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL7_EO_PL         = 0x8000380C0701183Full;
//DUPS: 800038080701143F, 8000380C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL7_EO_PL         = 0x8000380C0801183Full;
//DUPS: 800038080801143F, 8000380C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL7_EO_PL         = 0x800038080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x800038080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x800038080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x8000380C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x800038080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x800038080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x8000380C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL7_EO_PL        = 0x800038080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL7_EO_PL        = 0x800038080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL7_EO_PL        = 0x8000380C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL7_EO_PL        = 0x800038080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x800038080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x8000380C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL7_O_PL            = 0x8000B0080701103Full;
//DUPS: 8000B0080701143F, 8000B00C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL7_O_PL          = 0x8000B0080701103Full;
//DUPS: 8000B0080701143F, 8000B00C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL7_O_PL          = 0x8000B0080801103Full;
//DUPS: 8000B0080801143F, 8000B00C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL7_O_PL          = 0x8000B0080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B0080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B0080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B00C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B0080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B0080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B00C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL7_O_PL         = 0x8000B0080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL7_O_PL         = 0x8000B0080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL7_O_PL         = 0x8000B00C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL7_O_PL         = 0x8000B0080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B0080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B00C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL8_EO_PL           = 0x8000400C0701183Full;
//DUPS: 800040080701143F, 8000400C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL8_EO_PL         = 0x8000400C0701183Full;
//DUPS: 800040080701143F, 8000400C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL8_EO_PL         = 0x8000400C0801183Full;
//DUPS: 800040080801143F, 8000400C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL8_EO_PL         = 0x800040080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x800040080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x800040080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x8000400C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x800040080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x800040080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x8000400C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL8_EO_PL        = 0x800040080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL8_EO_PL        = 0x800040080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL8_EO_PL        = 0x8000400C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL8_EO_PL        = 0x800040080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x800040080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x8000400C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL8_O_PL            = 0x8000B8080701103Full;
//DUPS: 8000B8080701143F, 8000B80C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL8_O_PL          = 0x8000B8080701103Full;
//DUPS: 8000B8080701143F, 8000B80C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL8_O_PL          = 0x8000B8080801103Full;
//DUPS: 8000B8080801143F, 8000B80C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL8_O_PL          = 0x8000B8080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B8080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B8080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B80C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B8080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B8080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B80C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL8_O_PL         = 0x8000B8080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL8_O_PL         = 0x8000B8080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL8_O_PL         = 0x8000B80C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL8_O_PL         = 0x8000B8080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B8080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B80C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL9_O_PL            = 0x8000C0080701103Full;
//DUPS: 8000C0080701143F, 8000C00C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL9_O_PL          = 0x8000C0080701103Full;
//DUPS: 8000C0080701143F, 8000C00C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL9_O_PL          = 0x8000C0080801103Full;
//DUPS: 8000C0080801143F, 8000C00C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL9_O_PL          = 0x8000C0080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C0080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C0080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C00C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C0080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C0080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C00C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL9_O_PL         = 0x8000C0080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL9_O_PL         = 0x8000C0080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL9_O_PL         = 0x8000C00C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL9_O_PL         = 0x8000C0080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C0080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C00C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_DATA_DAC_SPARE_MODE_PL    = 0x8000000C0701183Full;
//DUPS: 800000080701143F, 8000000C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_DATA_DAC_SPARE_MODE_PL  = 0x8000000C0701183Full;
//DUPS: 800000080701143F, 8000000C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_DATA_DAC_SPARE_MODE_PL  = 0x8000000C0801183Full;
//DUPS: 800000080801143F, 8000000C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_FIR_ERROR_INJECT_PL       = 0x8002180C0701183Full;
//DUPS: 800218080701143F, 8002180C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_FIR_ERROR_INJECT_PL     = 0x8002180C0701183Full;
//DUPS: 800218080701143F, 8002180C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_FIR_ERROR_INJECT_PL     = 0x8002180C0801183Full;
//DUPS: 800218080801143F, 8002180C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_FIR_ERROR_INJECT_PL     = 0x800218080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x800218080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x800218080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x8002180C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x800218080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x800218080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x8002180C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_FIR_ERROR_INJECT_PL    = 0x800218080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_FIR_ERROR_INJECT_PL    = 0x800218080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_FIR_ERROR_INJECT_PL    = 0x8002180C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_FIR_ERROR_INJECT_PL    = 0x800218080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x800218080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x8002180C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_FIR_MASK_PL               = 0x8002100C0701183Full;
//DUPS: 800210080701143F, 8002100C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_FIR_MASK_PL             = 0x8002100C0701183Full;
//DUPS: 800210080701143F, 8002100C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_FIR_MASK_PL             = 0x8002100C0801183Full;
//DUPS: 800210080801143F, 8002100C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_FIR_MASK_PL             = 0x800210080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_FIR_MASK_PL           = 0x800210080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_FIR_MASK_PL           = 0x800210080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_FIR_MASK_PL           = 0x8002100C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_FIR_MASK_PL           = 0x800210080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_FIR_MASK_PL           = 0x800210080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_FIR_MASK_PL           = 0x8002100C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_FIR_MASK_PL            = 0x800210080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_FIR_MASK_PL            = 0x800210080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_FIR_MASK_PL            = 0x8002100C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_FIR_MASK_PL            = 0x800210080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_FIR_MASK_PL           = 0x800210080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_FIR_MASK_PL           = 0x8002100C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_FIR_PL                    = 0x8002080C0701183Full;
//DUPS: 800208080701143F, 8002080C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_FIR_PL                  = 0x8002080C0701183Full;
//DUPS: 800208080701143F, 8002080C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_FIR_PL                  = 0x8002080C0801183Full;
//DUPS: 800208080801143F, 8002080C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_FIR_PL                  = 0x800208080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_FIR_PL                = 0x800208080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_FIR_PL                = 0x800208080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_FIR_PL                = 0x8002080C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_FIR_PL                = 0x800208080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_FIR_PL                = 0x800208080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_FIR_PL                = 0x8002080C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_FIR_PL                 = 0x800208080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_FIR_PL                 = 0x800208080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_FIR_PL                 = 0x8002080C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_FIR_PL                 = 0x800208080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_FIR_PL                = 0x800208080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_FIR_PL                = 0x8002080C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE0_RX_SPARE_MODE_PL             = 0x8002000C0701183Full;
//DUPS: 800200080701143F, 8002000C0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE0_RX_SPARE_MODE_PL           = 0x8002000C0701183Full;
//DUPS: 800200080701143F, 8002000C0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE0_RX_SPARE_MODE_PL           = 0x8002000C0801183Full;
//DUPS: 800200080801143F, 8002000C0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE0_RX_SPARE_MODE_PL           = 0x800200080701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE0_RX_SPARE_MODE_PL         = 0x800200080701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE0_RX_SPARE_MODE_PL         = 0x800200080701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE0_RX_SPARE_MODE_PL         = 0x8002000C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE0_RX_SPARE_MODE_PL         = 0x800200080801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE0_RX_SPARE_MODE_PL         = 0x800200080801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE0_RX_SPARE_MODE_PL         = 0x8002000C0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE0_RX_SPARE_MODE_PL          = 0x800200080701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE0_RX_SPARE_MODE_PL          = 0x800200080701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE0_RX_SPARE_MODE_PL          = 0x8002000C0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE0_RX_SPARE_MODE_PL          = 0x800200080801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE0_RX_SPARE_MODE_PL         = 0x800200080801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE0_RX_SPARE_MODE_PL         = 0x8002000C0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL2_EO_PL           = 0x8002480D0701183Full;
//DUPS: 800248090701143F, 8002480D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL2_EO_PL         = 0x8002480D0701183Full;
//DUPS: 800248090701143F, 8002480D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL2_EO_PL         = 0x8002480D0801183Full;
//DUPS: 800248090801143F, 8002480D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL2_EO_PL         = 0x800248090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x800248090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x800248090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x8002480D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x800248090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x800248090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x8002480D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL2_EO_PL        = 0x800248090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL2_EO_PL        = 0x800248090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL2_EO_PL        = 0x8002480D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL2_EO_PL        = 0x800248090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x800248090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x8002480D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL3_EO_PL           = 0x8002500D0701183Full;
//DUPS: 800250090701143F, 8002500D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL3_EO_PL         = 0x8002500D0701183Full;
//DUPS: 800250090701143F, 8002500D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL3_EO_PL         = 0x8002500D0801183Full;
//DUPS: 800250090801143F, 8002500D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL3_EO_PL         = 0x800250090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x800250090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x800250090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x8002500D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x800250090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x800250090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x8002500D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL3_EO_PL        = 0x800250090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL3_EO_PL        = 0x800250090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL3_EO_PL        = 0x8002500D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL3_EO_PL        = 0x800250090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x800250090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x8002500D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL4_EO_PL           = 0x8002580D0701183Full;
//DUPS: 800258090701143F, 8002580D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL4_EO_PL         = 0x8002580D0701183Full;
//DUPS: 800258090701143F, 8002580D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL4_EO_PL         = 0x8002580D0801183Full;
//DUPS: 800258090801143F, 8002580D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL4_EO_PL         = 0x800258090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x800258090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x800258090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x8002580D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x800258090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x800258090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x8002580D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL4_EO_PL        = 0x800258090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL4_EO_PL        = 0x800258090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL4_EO_PL        = 0x8002580D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL4_EO_PL        = 0x800258090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x800258090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x8002580D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL5_EO_PL           = 0x8002600D0701183Full;
//DUPS: 800260090701143F, 8002600D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL5_EO_PL         = 0x8002600D0701183Full;
//DUPS: 800260090701143F, 8002600D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL5_EO_PL         = 0x8002600D0801183Full;
//DUPS: 800260090801143F, 8002600D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL5_EO_PL         = 0x800260090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x800260090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x800260090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x8002600D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x800260090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x800260090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x8002600D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL5_EO_PL        = 0x800260090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL5_EO_PL        = 0x800260090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL5_EO_PL        = 0x8002600D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL5_EO_PL        = 0x800260090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x800260090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x8002600D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_BIT_CNTLX1_EO_PL          = 0x8002400D0701183Full;
//DUPS: 800240090701143F, 8002400D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_BIT_CNTLX1_EO_PL        = 0x8002400D0701183Full;
//DUPS: 800240090701143F, 8002400D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_BIT_CNTLX1_EO_PL        = 0x8002400D0801183Full;
//DUPS: 800240090801143F, 8002400D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_BIT_CNTLX1_EO_PL        = 0x800240090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x800240090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x800240090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x8002400D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x800240090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x800240090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x8002400D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_BIT_CNTLX1_EO_PL       = 0x800240090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_BIT_CNTLX1_EO_PL       = 0x800240090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_BIT_CNTLX1_EO_PL       = 0x8002400D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_BIT_CNTLX1_EO_PL       = 0x800240090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x800240090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x8002400D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_BIT_MODE1_EO_PL           = 0x8002200D0701183Full;
//DUPS: 800220090701143F, 8002200D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_BIT_MODE1_EO_PL         = 0x8002200D0701183Full;
//DUPS: 800220090701143F, 8002200D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_BIT_MODE1_EO_PL         = 0x8002200D0801183Full;
//DUPS: 800220090801143F, 8002200D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_BIT_MODE1_EO_PL         = 0x800220090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_BIT_MODE1_EO_PL       = 0x800220090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_BIT_MODE1_EO_PL       = 0x800220090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_BIT_MODE1_EO_PL       = 0x8002200D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_BIT_MODE1_EO_PL       = 0x800220090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_BIT_MODE1_EO_PL       = 0x800220090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_BIT_MODE1_EO_PL       = 0x8002200D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_BIT_MODE1_EO_PL        = 0x800220090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_BIT_MODE1_EO_PL        = 0x800220090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_BIT_MODE1_EO_PL        = 0x8002200D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_BIT_MODE1_EO_PL        = 0x800220090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_BIT_MODE1_EO_PL       = 0x800220090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_BIT_MODE1_EO_PL       = 0x8002200D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_BIT_MODE2_EO_PL           = 0x8002280D0701183Full;
//DUPS: 800228090701143F, 8002280D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_BIT_MODE2_EO_PL         = 0x8002280D0701183Full;
//DUPS: 800228090701143F, 8002280D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_BIT_MODE2_EO_PL         = 0x8002280D0801183Full;
//DUPS: 800228090801143F, 8002280D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_BIT_MODE2_EO_PL         = 0x800228090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_BIT_MODE2_EO_PL       = 0x800228090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_BIT_MODE2_EO_PL       = 0x800228090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_BIT_MODE2_EO_PL       = 0x8002280D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_BIT_MODE2_EO_PL       = 0x800228090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_BIT_MODE2_EO_PL       = 0x800228090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_BIT_MODE2_EO_PL       = 0x8002280D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_BIT_MODE2_EO_PL        = 0x800228090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_BIT_MODE2_EO_PL        = 0x800228090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_BIT_MODE2_EO_PL        = 0x8002280D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_BIT_MODE2_EO_PL        = 0x800228090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_BIT_MODE2_EO_PL       = 0x800228090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_BIT_MODE2_EO_PL       = 0x8002280D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_BIT_MODE3_EO_PL           = 0x8002300D0701183Full;
//DUPS: 800230090701143F, 8002300D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_BIT_MODE3_EO_PL         = 0x8002300D0701183Full;
//DUPS: 800230090701143F, 8002300D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_BIT_MODE3_EO_PL         = 0x8002300D0801183Full;
//DUPS: 800230090801143F, 8002300D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_BIT_MODE3_EO_PL         = 0x800230090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_BIT_MODE3_EO_PL       = 0x800230090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_BIT_MODE3_EO_PL       = 0x800230090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_BIT_MODE3_EO_PL       = 0x8002300D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_BIT_MODE3_EO_PL       = 0x800230090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_BIT_MODE3_EO_PL       = 0x800230090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_BIT_MODE3_EO_PL       = 0x8002300D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_BIT_MODE3_EO_PL        = 0x800230090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_BIT_MODE3_EO_PL        = 0x800230090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_BIT_MODE3_EO_PL        = 0x8002300D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_BIT_MODE3_EO_PL        = 0x800230090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_BIT_MODE3_EO_PL       = 0x800230090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_BIT_MODE3_EO_PL       = 0x8002300D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_BIT_MODE4_EO_PL           = 0x8002380D0701183Full;
//DUPS: 800238090701143F, 8002380D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_BIT_MODE4_EO_PL         = 0x8002380D0701183Full;
//DUPS: 800238090701143F, 8002380D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_BIT_MODE4_EO_PL         = 0x8002380D0801183Full;
//DUPS: 800238090801143F, 8002380D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_BIT_MODE4_EO_PL         = 0x800238090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_BIT_MODE4_EO_PL       = 0x800238090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_BIT_MODE4_EO_PL       = 0x800238090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_BIT_MODE4_EO_PL       = 0x8002380D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_BIT_MODE4_EO_PL       = 0x800238090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_BIT_MODE4_EO_PL       = 0x800238090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_BIT_MODE4_EO_PL       = 0x8002380D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_BIT_MODE4_EO_PL        = 0x800238090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_BIT_MODE4_EO_PL        = 0x800238090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_BIT_MODE4_EO_PL        = 0x8002380D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_BIT_MODE4_EO_PL        = 0x800238090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_BIT_MODE4_EO_PL       = 0x800238090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_BIT_MODE4_EO_PL       = 0x8002380D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_BIT_STAT1_EO_PL           = 0x8002680D0701183Full;
//DUPS: 800268090701143F, 8002680D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_BIT_STAT1_EO_PL         = 0x8002680D0701183Full;
//DUPS: 800268090701143F, 8002680D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_BIT_STAT1_EO_PL         = 0x8002680D0801183Full;
//DUPS: 800268090801143F, 8002680D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_BIT_STAT1_EO_PL         = 0x800268090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_BIT_STAT1_EO_PL       = 0x800268090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_BIT_STAT1_EO_PL       = 0x800268090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_BIT_STAT1_EO_PL       = 0x8002680D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_BIT_STAT1_EO_PL       = 0x800268090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_BIT_STAT1_EO_PL       = 0x800268090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_BIT_STAT1_EO_PL       = 0x8002680D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_BIT_STAT1_EO_PL        = 0x800268090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_BIT_STAT1_EO_PL        = 0x800268090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_BIT_STAT1_EO_PL        = 0x8002680D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_BIT_STAT1_EO_PL        = 0x800268090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_BIT_STAT1_EO_PL       = 0x800268090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_BIT_STAT1_EO_PL       = 0x8002680D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_BIT_STAT1_O_PL            = 0x8003180D0701183Full;
//DUPS: 800318090701143F, 8003180D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_BIT_STAT1_O_PL          = 0x8003180D0701183Full;
//DUPS: 800318090701143F, 8003180D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_BIT_STAT1_O_PL          = 0x8003180D0801183Full;
//DUPS: 800318090801143F, 8003180D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_BIT_STAT1_O_PL          = 0x800318090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_BIT_STAT1_O_PL        = 0x800318090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_BIT_STAT1_O_PL        = 0x800318090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_BIT_STAT1_O_PL        = 0x8003180D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_BIT_STAT1_O_PL        = 0x800318090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_BIT_STAT1_O_PL        = 0x800318090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_BIT_STAT1_O_PL        = 0x8003180D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_BIT_STAT1_O_PL         = 0x800318090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_BIT_STAT1_O_PL         = 0x800318090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_BIT_STAT1_O_PL         = 0x8003180D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_BIT_STAT1_O_PL         = 0x800318090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_BIT_STAT1_O_PL        = 0x800318090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_BIT_STAT1_O_PL        = 0x8003180D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_BIT_STAT2_EO_PL           = 0x8002700D0701183Full;
//DUPS: 800270090701143F, 8002700D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_BIT_STAT2_EO_PL         = 0x8002700D0701183Full;
//DUPS: 800270090701143F, 8002700D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_BIT_STAT2_EO_PL         = 0x8002700D0801183Full;
//DUPS: 800270090801143F, 8002700D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_BIT_STAT2_EO_PL         = 0x800270090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_BIT_STAT2_EO_PL       = 0x800270090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_BIT_STAT2_EO_PL       = 0x800270090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_BIT_STAT2_EO_PL       = 0x8002700D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_BIT_STAT2_EO_PL       = 0x800270090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_BIT_STAT2_EO_PL       = 0x800270090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_BIT_STAT2_EO_PL       = 0x8002700D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_BIT_STAT2_EO_PL        = 0x800270090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_BIT_STAT2_EO_PL        = 0x800270090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_BIT_STAT2_EO_PL        = 0x8002700D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_BIT_STAT2_EO_PL        = 0x800270090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_BIT_STAT2_EO_PL       = 0x800270090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_BIT_STAT2_EO_PL       = 0x8002700D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_BIT_STAT3_EO_PL           = 0x8002780D0701183Full;
//DUPS: 800278090701143F, 8002780D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_BIT_STAT3_EO_PL         = 0x8002780D0701183Full;
//DUPS: 800278090701143F, 8002780D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_BIT_STAT3_EO_PL         = 0x8002780D0801183Full;
//DUPS: 800278090801143F, 8002780D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_BIT_STAT3_EO_PL         = 0x800278090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_BIT_STAT3_EO_PL       = 0x800278090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_BIT_STAT3_EO_PL       = 0x800278090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_BIT_STAT3_EO_PL       = 0x8002780D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_BIT_STAT3_EO_PL       = 0x800278090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_BIT_STAT3_EO_PL       = 0x800278090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_BIT_STAT3_EO_PL       = 0x8002780D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_BIT_STAT3_EO_PL        = 0x800278090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_BIT_STAT3_EO_PL        = 0x800278090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_BIT_STAT3_EO_PL        = 0x8002780D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_BIT_STAT3_EO_PL        = 0x800278090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_BIT_STAT3_EO_PL       = 0x800278090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_BIT_STAT3_EO_PL       = 0x8002780D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_BIT_STAT4_EO_PL           = 0x8002800D0701183Full;
//DUPS: 800280090701143F, 8002800D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_BIT_STAT4_EO_PL         = 0x8002800D0701183Full;
//DUPS: 800280090701143F, 8002800D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_BIT_STAT4_EO_PL         = 0x8002800D0801183Full;
//DUPS: 800280090801143F, 8002800D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_BIT_STAT4_EO_PL         = 0x800280090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_BIT_STAT4_EO_PL       = 0x800280090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_BIT_STAT4_EO_PL       = 0x800280090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_BIT_STAT4_EO_PL       = 0x8002800D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_BIT_STAT4_EO_PL       = 0x800280090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_BIT_STAT4_EO_PL       = 0x800280090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_BIT_STAT4_EO_PL       = 0x8002800D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_BIT_STAT4_EO_PL        = 0x800280090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_BIT_STAT4_EO_PL        = 0x800280090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_BIT_STAT4_EO_PL        = 0x8002800D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_BIT_STAT4_EO_PL        = 0x800280090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_BIT_STAT4_EO_PL       = 0x800280090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_BIT_STAT4_EO_PL       = 0x8002800D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL10_O_PL           = 0x8000C8090701103Full;
//DUPS: 8000C8090701143F, 8000C80D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL10_O_PL         = 0x8000C8090701103Full;
//DUPS: 8000C8090701143F, 8000C80D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL10_O_PL         = 0x8000C8090801103Full;
//DUPS: 8000C8090801143F, 8000C80D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL10_O_PL         = 0x8000C8090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C8090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C8090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C80D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C8090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C8090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C80D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL10_O_PL        = 0x8000C8090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL10_O_PL        = 0x8000C8090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL10_O_PL        = 0x8000C80D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL10_O_PL        = 0x8000C8090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C8090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C80D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL1_EO_PL           = 0x8000080D0701183Full;
//DUPS: 800008090701143F, 8000080D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL1_EO_PL         = 0x8000080D0701183Full;
//DUPS: 800008090701143F, 8000080D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL1_EO_PL         = 0x8000080D0801183Full;
//DUPS: 800008090801143F, 8000080D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL1_EO_PL         = 0x800008090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x800008090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x800008090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x8000080D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x800008090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x800008090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x8000080D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL1_EO_PL        = 0x800008090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL1_EO_PL        = 0x800008090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL1_EO_PL        = 0x8000080D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL1_EO_PL        = 0x800008090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x800008090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x8000080D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL1_O_PL            = 0x8000800D0701183Full;
//DUPS: 800080090701143F, 8000800D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL1_O_PL          = 0x8000800D0701183Full;
//DUPS: 800080090701143F, 8000800D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL1_O_PL          = 0x8000800D0801183Full;
//DUPS: 800080090801143F, 8000800D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL1_O_PL          = 0x800080090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL1_O_PL        = 0x800080090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL1_O_PL        = 0x800080090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL1_O_PL        = 0x8000800D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL1_O_PL        = 0x800080090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL1_O_PL        = 0x800080090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL1_O_PL        = 0x8000800D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL1_O_PL         = 0x800080090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL1_O_PL         = 0x800080090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL1_O_PL         = 0x8000800D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL1_O_PL         = 0x800080090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL1_O_PL        = 0x800080090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL1_O_PL        = 0x8000800D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL2_EO_PL           = 0x8000100D0701183Full;
//DUPS: 800010090701143F, 8000100D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL2_EO_PL         = 0x8000100D0701183Full;
//DUPS: 800010090701143F, 8000100D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL2_EO_PL         = 0x8000100D0801183Full;
//DUPS: 800010090801143F, 8000100D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL2_EO_PL         = 0x800010090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x800010090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x800010090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x8000100D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x800010090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x800010090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x8000100D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL2_EO_PL        = 0x800010090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL2_EO_PL        = 0x800010090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL2_EO_PL        = 0x8000100D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL2_EO_PL        = 0x800010090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x800010090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x8000100D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL2_O_PL            = 0x8000880D0701183Full;
//DUPS: 800088090701143F, 8000880D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL2_O_PL          = 0x8000880D0701183Full;
//DUPS: 800088090701143F, 8000880D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL2_O_PL          = 0x8000880D0801183Full;
//DUPS: 800088090801143F, 8000880D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL2_O_PL          = 0x800088090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL2_O_PL        = 0x800088090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL2_O_PL        = 0x800088090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL2_O_PL        = 0x8000880D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL2_O_PL        = 0x800088090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL2_O_PL        = 0x800088090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL2_O_PL        = 0x8000880D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL2_O_PL         = 0x800088090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL2_O_PL         = 0x800088090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL2_O_PL         = 0x8000880D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL2_O_PL         = 0x800088090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL2_O_PL        = 0x800088090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL2_O_PL        = 0x8000880D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL3_EO_PL           = 0x8000180D0701183Full;
//DUPS: 800018090701143F, 8000180D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL3_EO_PL         = 0x8000180D0701183Full;
//DUPS: 800018090701143F, 8000180D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL3_EO_PL         = 0x8000180D0801183Full;
//DUPS: 800018090801143F, 8000180D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL3_EO_PL         = 0x800018090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x800018090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x800018090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x8000180D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x800018090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x800018090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x8000180D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL3_EO_PL        = 0x800018090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL3_EO_PL        = 0x800018090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL3_EO_PL        = 0x8000180D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL3_EO_PL        = 0x800018090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x800018090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x8000180D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL3_O_PL            = 0x8000900D0701183Full;
//DUPS: 800090090701143F, 8000900D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL3_O_PL          = 0x8000900D0701183Full;
//DUPS: 800090090701143F, 8000900D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL3_O_PL          = 0x8000900D0801183Full;
//DUPS: 800090090801143F, 8000900D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL3_O_PL          = 0x800090090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL3_O_PL        = 0x800090090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL3_O_PL        = 0x800090090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL3_O_PL        = 0x8000900D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL3_O_PL        = 0x800090090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL3_O_PL        = 0x800090090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL3_O_PL        = 0x8000900D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL3_O_PL         = 0x800090090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL3_O_PL         = 0x800090090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL3_O_PL         = 0x8000900D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL3_O_PL         = 0x800090090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL3_O_PL        = 0x800090090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL3_O_PL        = 0x8000900D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL4_EO_PL           = 0x8000200D0701183Full;
//DUPS: 800020090701143F, 8000200D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL4_EO_PL         = 0x8000200D0701183Full;
//DUPS: 800020090701143F, 8000200D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL4_EO_PL         = 0x8000200D0801183Full;
//DUPS: 800020090801143F, 8000200D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL4_EO_PL         = 0x800020090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x800020090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x800020090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x8000200D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x800020090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x800020090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x8000200D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL4_EO_PL        = 0x800020090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL4_EO_PL        = 0x800020090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL4_EO_PL        = 0x8000200D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL4_EO_PL        = 0x800020090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x800020090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x8000200D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL4_O_PL            = 0x8000980D0701183Full;
//DUPS: 800098090701143F, 8000980D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL4_O_PL          = 0x8000980D0701183Full;
//DUPS: 800098090701143F, 8000980D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL4_O_PL          = 0x8000980D0801183Full;
//DUPS: 800098090801143F, 8000980D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL4_O_PL          = 0x800098090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL4_O_PL        = 0x800098090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL4_O_PL        = 0x800098090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL4_O_PL        = 0x8000980D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL4_O_PL        = 0x800098090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL4_O_PL        = 0x800098090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL4_O_PL        = 0x8000980D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL4_O_PL         = 0x800098090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL4_O_PL         = 0x800098090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL4_O_PL         = 0x8000980D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL4_O_PL         = 0x800098090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL4_O_PL        = 0x800098090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL4_O_PL        = 0x8000980D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL5_EO_PL           = 0x8000280D0701183Full;
//DUPS: 800028090701143F, 8000280D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL5_EO_PL         = 0x8000280D0701183Full;
//DUPS: 800028090701143F, 8000280D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL5_EO_PL         = 0x8000280D0801183Full;
//DUPS: 800028090801143F, 8000280D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL5_EO_PL         = 0x800028090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x800028090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x800028090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x8000280D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x800028090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x800028090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x8000280D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL5_EO_PL        = 0x800028090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL5_EO_PL        = 0x800028090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL5_EO_PL        = 0x8000280D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL5_EO_PL        = 0x800028090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x800028090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x8000280D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL5_O_PL            = 0x8000A0090701103Full;
//DUPS: 8000A0090701143F, 8000A00D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL5_O_PL          = 0x8000A0090701103Full;
//DUPS: 8000A0090701143F, 8000A00D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL5_O_PL          = 0x8000A0090801103Full;
//DUPS: 8000A0090801143F, 8000A00D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL5_O_PL          = 0x8000A0090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A0090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A0090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A00D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A0090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A0090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A00D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL5_O_PL         = 0x8000A0090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL5_O_PL         = 0x8000A0090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL5_O_PL         = 0x8000A00D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL5_O_PL         = 0x8000A0090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A0090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A00D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL6_EO_PL           = 0x8000300D0701183Full;
//DUPS: 800030090701143F, 8000300D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL6_EO_PL         = 0x8000300D0701183Full;
//DUPS: 800030090701143F, 8000300D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL6_EO_PL         = 0x8000300D0801183Full;
//DUPS: 800030090801143F, 8000300D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL6_EO_PL         = 0x800030090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x800030090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x800030090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x8000300D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x800030090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x800030090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x8000300D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL6_EO_PL        = 0x800030090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL6_EO_PL        = 0x800030090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL6_EO_PL        = 0x8000300D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL6_EO_PL        = 0x800030090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x800030090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x8000300D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL6_O_PL            = 0x8000A8090701103Full;
//DUPS: 8000A8090701143F, 8000A80D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL6_O_PL          = 0x8000A8090701103Full;
//DUPS: 8000A8090701143F, 8000A80D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL6_O_PL          = 0x8000A8090801103Full;
//DUPS: 8000A8090801143F, 8000A80D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL6_O_PL          = 0x8000A8090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A8090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A8090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A80D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A8090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A8090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A80D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL6_O_PL         = 0x8000A8090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL6_O_PL         = 0x8000A8090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL6_O_PL         = 0x8000A80D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL6_O_PL         = 0x8000A8090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A8090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A80D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL7_EO_PL           = 0x8000380D0701183Full;
//DUPS: 800038090701143F, 8000380D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL7_EO_PL         = 0x8000380D0701183Full;
//DUPS: 800038090701143F, 8000380D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL7_EO_PL         = 0x8000380D0801183Full;
//DUPS: 800038090801143F, 8000380D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL7_EO_PL         = 0x800038090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x800038090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x800038090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x8000380D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x800038090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x800038090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x8000380D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL7_EO_PL        = 0x800038090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL7_EO_PL        = 0x800038090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL7_EO_PL        = 0x8000380D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL7_EO_PL        = 0x800038090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x800038090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x8000380D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL7_O_PL            = 0x8000B0090701103Full;
//DUPS: 8000B0090701143F, 8000B00D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL7_O_PL          = 0x8000B0090701103Full;
//DUPS: 8000B0090701143F, 8000B00D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL7_O_PL          = 0x8000B0090801103Full;
//DUPS: 8000B0090801143F, 8000B00D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL7_O_PL          = 0x8000B0090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B0090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B0090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B00D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B0090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B0090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B00D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL7_O_PL         = 0x8000B0090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL7_O_PL         = 0x8000B0090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL7_O_PL         = 0x8000B00D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL7_O_PL         = 0x8000B0090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B0090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B00D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL8_EO_PL           = 0x8000400D0701183Full;
//DUPS: 800040090701143F, 8000400D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL8_EO_PL         = 0x8000400D0701183Full;
//DUPS: 800040090701143F, 8000400D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL8_EO_PL         = 0x8000400D0801183Full;
//DUPS: 800040090801143F, 8000400D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL8_EO_PL         = 0x800040090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x800040090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x800040090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x8000400D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x800040090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x800040090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x8000400D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL8_EO_PL        = 0x800040090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL8_EO_PL        = 0x800040090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL8_EO_PL        = 0x8000400D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL8_EO_PL        = 0x800040090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x800040090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x8000400D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL8_O_PL            = 0x8000B8090701103Full;
//DUPS: 8000B8090701143F, 8000B80D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL8_O_PL          = 0x8000B8090701103Full;
//DUPS: 8000B8090701143F, 8000B80D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL8_O_PL          = 0x8000B8090801103Full;
//DUPS: 8000B8090801143F, 8000B80D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL8_O_PL          = 0x8000B8090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B8090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B8090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B80D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B8090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B8090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B80D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL8_O_PL         = 0x8000B8090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL8_O_PL         = 0x8000B8090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL8_O_PL         = 0x8000B80D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL8_O_PL         = 0x8000B8090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B8090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B80D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL9_O_PL            = 0x8000C0090701103Full;
//DUPS: 8000C0090701143F, 8000C00D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL9_O_PL          = 0x8000C0090701103Full;
//DUPS: 8000C0090701143F, 8000C00D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL9_O_PL          = 0x8000C0090801103Full;
//DUPS: 8000C0090801143F, 8000C00D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL9_O_PL          = 0x8000C0090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C0090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C0090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C00D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C0090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C0090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C00D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL9_O_PL         = 0x8000C0090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL9_O_PL         = 0x8000C0090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL9_O_PL         = 0x8000C00D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL9_O_PL         = 0x8000C0090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C0090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C00D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_DATA_DAC_SPARE_MODE_PL    = 0x8000000D0701183Full;
//DUPS: 800000090701143F, 8000000D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_DATA_DAC_SPARE_MODE_PL  = 0x8000000D0701183Full;
//DUPS: 800000090701143F, 8000000D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_DATA_DAC_SPARE_MODE_PL  = 0x8000000D0801183Full;
//DUPS: 800000090801143F, 8000000D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_FIR_ERROR_INJECT_PL       = 0x8002180D0701183Full;
//DUPS: 800218090701143F, 8002180D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_FIR_ERROR_INJECT_PL     = 0x8002180D0701183Full;
//DUPS: 800218090701143F, 8002180D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_FIR_ERROR_INJECT_PL     = 0x8002180D0801183Full;
//DUPS: 800218090801143F, 8002180D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_FIR_ERROR_INJECT_PL     = 0x800218090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x800218090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x800218090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x8002180D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x800218090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x800218090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x8002180D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_FIR_ERROR_INJECT_PL    = 0x800218090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_FIR_ERROR_INJECT_PL    = 0x800218090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_FIR_ERROR_INJECT_PL    = 0x8002180D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_FIR_ERROR_INJECT_PL    = 0x800218090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x800218090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x8002180D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_FIR_MASK_PL               = 0x8002100D0701183Full;
//DUPS: 800210090701143F, 8002100D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_FIR_MASK_PL             = 0x8002100D0701183Full;
//DUPS: 800210090701143F, 8002100D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_FIR_MASK_PL             = 0x8002100D0801183Full;
//DUPS: 800210090801143F, 8002100D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_FIR_MASK_PL             = 0x800210090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_FIR_MASK_PL           = 0x800210090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_FIR_MASK_PL           = 0x800210090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_FIR_MASK_PL           = 0x8002100D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_FIR_MASK_PL           = 0x800210090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_FIR_MASK_PL           = 0x800210090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_FIR_MASK_PL           = 0x8002100D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_FIR_MASK_PL            = 0x800210090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_FIR_MASK_PL            = 0x800210090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_FIR_MASK_PL            = 0x8002100D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_FIR_MASK_PL            = 0x800210090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_FIR_MASK_PL           = 0x800210090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_FIR_MASK_PL           = 0x8002100D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_FIR_PL                    = 0x8002080D0701183Full;
//DUPS: 800208090701143F, 8002080D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_FIR_PL                  = 0x8002080D0701183Full;
//DUPS: 800208090701143F, 8002080D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_FIR_PL                  = 0x8002080D0801183Full;
//DUPS: 800208090801143F, 8002080D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_FIR_PL                  = 0x800208090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_FIR_PL                = 0x800208090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_FIR_PL                = 0x800208090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_FIR_PL                = 0x8002080D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_FIR_PL                = 0x800208090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_FIR_PL                = 0x800208090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_FIR_PL                = 0x8002080D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_FIR_PL                 = 0x800208090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_FIR_PL                 = 0x800208090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_FIR_PL                 = 0x8002080D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_FIR_PL                 = 0x800208090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_FIR_PL                = 0x800208090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_FIR_PL                = 0x8002080D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE1_RX_SPARE_MODE_PL             = 0x8002000D0701183Full;
//DUPS: 800200090701143F, 8002000D0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE1_RX_SPARE_MODE_PL           = 0x8002000D0701183Full;
//DUPS: 800200090701143F, 8002000D0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE1_RX_SPARE_MODE_PL           = 0x8002000D0801183Full;
//DUPS: 800200090801143F, 8002000D0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE1_RX_SPARE_MODE_PL           = 0x800200090701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE1_RX_SPARE_MODE_PL         = 0x800200090701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE1_RX_SPARE_MODE_PL         = 0x800200090701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE1_RX_SPARE_MODE_PL         = 0x8002000D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE1_RX_SPARE_MODE_PL         = 0x800200090801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE1_RX_SPARE_MODE_PL         = 0x800200090801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE1_RX_SPARE_MODE_PL         = 0x8002000D0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE1_RX_SPARE_MODE_PL          = 0x800200090701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE1_RX_SPARE_MODE_PL          = 0x800200090701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE1_RX_SPARE_MODE_PL          = 0x8002000D0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE1_RX_SPARE_MODE_PL          = 0x800200090801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE1_RX_SPARE_MODE_PL         = 0x800200090801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE1_RX_SPARE_MODE_PL         = 0x8002000D0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL2_EO_PL           = 0x8002480A0701103Full;
//DUPS: 8002480A0701143F, 8002480E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL2_EO_PL         = 0x8002480A0701103Full;
//DUPS: 8002480A0701143F, 8002480E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL2_EO_PL         = 0x8002480A0801103Full;
//DUPS: 8002480A0801143F, 8002480E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL2_EO_PL         = 0x8002480A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x8002480A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x8002480A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x8002480E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x8002480A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x8002480A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x8002480E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL2_EO_PL        = 0x8002480A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL2_EO_PL        = 0x8002480A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL2_EO_PL        = 0x8002480E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL2_EO_PL        = 0x8002480A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x8002480A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x8002480E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL3_EO_PL           = 0x8002500A0701103Full;
//DUPS: 8002500A0701143F, 8002500E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL3_EO_PL         = 0x8002500A0701103Full;
//DUPS: 8002500A0701143F, 8002500E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL3_EO_PL         = 0x8002500A0801103Full;
//DUPS: 8002500A0801143F, 8002500E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL3_EO_PL         = 0x8002500A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x8002500A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x8002500A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x8002500E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x8002500A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x8002500A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x8002500E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL3_EO_PL        = 0x8002500A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL3_EO_PL        = 0x8002500A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL3_EO_PL        = 0x8002500E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL3_EO_PL        = 0x8002500A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x8002500A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x8002500E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL4_EO_PL           = 0x8002580A0701103Full;
//DUPS: 8002580A0701143F, 8002580E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL4_EO_PL         = 0x8002580A0701103Full;
//DUPS: 8002580A0701143F, 8002580E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL4_EO_PL         = 0x8002580A0801103Full;
//DUPS: 8002580A0801143F, 8002580E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL4_EO_PL         = 0x8002580A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x8002580A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x8002580A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x8002580E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x8002580A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x8002580A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x8002580E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL4_EO_PL        = 0x8002580A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL4_EO_PL        = 0x8002580A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL4_EO_PL        = 0x8002580E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL4_EO_PL        = 0x8002580A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x8002580A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x8002580E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL5_EO_PL           = 0x8002600A0701103Full;
//DUPS: 8002600A0701143F, 8002600E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL5_EO_PL         = 0x8002600A0701103Full;
//DUPS: 8002600A0701143F, 8002600E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL5_EO_PL         = 0x8002600A0801103Full;
//DUPS: 8002600A0801143F, 8002600E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL5_EO_PL         = 0x8002600A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x8002600A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x8002600A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x8002600E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x8002600A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x8002600A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x8002600E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL5_EO_PL        = 0x8002600A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL5_EO_PL        = 0x8002600A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL5_EO_PL        = 0x8002600E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL5_EO_PL        = 0x8002600A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x8002600A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x8002600E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_BIT_CNTLX1_EO_PL          = 0x8002400A0701103Full;
//DUPS: 8002400A0701143F, 8002400E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_BIT_CNTLX1_EO_PL        = 0x8002400A0701103Full;
//DUPS: 8002400A0701143F, 8002400E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_BIT_CNTLX1_EO_PL        = 0x8002400A0801103Full;
//DUPS: 8002400A0801143F, 8002400E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_BIT_CNTLX1_EO_PL        = 0x8002400A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x8002400A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x8002400A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x8002400E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x8002400A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x8002400A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x8002400E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_BIT_CNTLX1_EO_PL       = 0x8002400A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_BIT_CNTLX1_EO_PL       = 0x8002400A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_BIT_CNTLX1_EO_PL       = 0x8002400E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_BIT_CNTLX1_EO_PL       = 0x8002400A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x8002400A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x8002400E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_BIT_MODE1_EO_PL           = 0x8002200A0701103Full;
//DUPS: 8002200A0701143F, 8002200E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_BIT_MODE1_EO_PL         = 0x8002200A0701103Full;
//DUPS: 8002200A0701143F, 8002200E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_BIT_MODE1_EO_PL         = 0x8002200A0801103Full;
//DUPS: 8002200A0801143F, 8002200E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_BIT_MODE1_EO_PL         = 0x8002200A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_BIT_MODE1_EO_PL       = 0x8002200A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_BIT_MODE1_EO_PL       = 0x8002200A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_BIT_MODE1_EO_PL       = 0x8002200E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_BIT_MODE1_EO_PL       = 0x8002200A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_BIT_MODE1_EO_PL       = 0x8002200A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_BIT_MODE1_EO_PL       = 0x8002200E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_BIT_MODE1_EO_PL        = 0x8002200A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_BIT_MODE1_EO_PL        = 0x8002200A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_BIT_MODE1_EO_PL        = 0x8002200E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_BIT_MODE1_EO_PL        = 0x8002200A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_BIT_MODE1_EO_PL       = 0x8002200A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_BIT_MODE1_EO_PL       = 0x8002200E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_BIT_MODE2_EO_PL           = 0x8002280A0701103Full;
//DUPS: 8002280A0701143F, 8002280E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_BIT_MODE2_EO_PL         = 0x8002280A0701103Full;
//DUPS: 8002280A0701143F, 8002280E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_BIT_MODE2_EO_PL         = 0x8002280A0801103Full;
//DUPS: 8002280A0801143F, 8002280E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_BIT_MODE2_EO_PL         = 0x8002280A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_BIT_MODE2_EO_PL       = 0x8002280A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_BIT_MODE2_EO_PL       = 0x8002280A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_BIT_MODE2_EO_PL       = 0x8002280E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_BIT_MODE2_EO_PL       = 0x8002280A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_BIT_MODE2_EO_PL       = 0x8002280A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_BIT_MODE2_EO_PL       = 0x8002280E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_BIT_MODE2_EO_PL        = 0x8002280A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_BIT_MODE2_EO_PL        = 0x8002280A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_BIT_MODE2_EO_PL        = 0x8002280E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_BIT_MODE2_EO_PL        = 0x8002280A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_BIT_MODE2_EO_PL       = 0x8002280A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_BIT_MODE2_EO_PL       = 0x8002280E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_BIT_MODE3_EO_PL           = 0x8002300A0701103Full;
//DUPS: 8002300A0701143F, 8002300E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_BIT_MODE3_EO_PL         = 0x8002300A0701103Full;
//DUPS: 8002300A0701143F, 8002300E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_BIT_MODE3_EO_PL         = 0x8002300A0801103Full;
//DUPS: 8002300A0801143F, 8002300E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_BIT_MODE3_EO_PL         = 0x8002300A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_BIT_MODE3_EO_PL       = 0x8002300A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_BIT_MODE3_EO_PL       = 0x8002300A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_BIT_MODE3_EO_PL       = 0x8002300E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_BIT_MODE3_EO_PL       = 0x8002300A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_BIT_MODE3_EO_PL       = 0x8002300A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_BIT_MODE3_EO_PL       = 0x8002300E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_BIT_MODE3_EO_PL        = 0x8002300A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_BIT_MODE3_EO_PL        = 0x8002300A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_BIT_MODE3_EO_PL        = 0x8002300E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_BIT_MODE3_EO_PL        = 0x8002300A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_BIT_MODE3_EO_PL       = 0x8002300A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_BIT_MODE3_EO_PL       = 0x8002300E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_BIT_MODE4_EO_PL           = 0x8002380A0701103Full;
//DUPS: 8002380A0701143F, 8002380E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_BIT_MODE4_EO_PL         = 0x8002380A0701103Full;
//DUPS: 8002380A0701143F, 8002380E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_BIT_MODE4_EO_PL         = 0x8002380A0801103Full;
//DUPS: 8002380A0801143F, 8002380E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_BIT_MODE4_EO_PL         = 0x8002380A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_BIT_MODE4_EO_PL       = 0x8002380A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_BIT_MODE4_EO_PL       = 0x8002380A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_BIT_MODE4_EO_PL       = 0x8002380E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_BIT_MODE4_EO_PL       = 0x8002380A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_BIT_MODE4_EO_PL       = 0x8002380A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_BIT_MODE4_EO_PL       = 0x8002380E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_BIT_MODE4_EO_PL        = 0x8002380A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_BIT_MODE4_EO_PL        = 0x8002380A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_BIT_MODE4_EO_PL        = 0x8002380E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_BIT_MODE4_EO_PL        = 0x8002380A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_BIT_MODE4_EO_PL       = 0x8002380A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_BIT_MODE4_EO_PL       = 0x8002380E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_BIT_STAT1_EO_PL           = 0x8002680A0701103Full;
//DUPS: 8002680A0701143F, 8002680E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_BIT_STAT1_EO_PL         = 0x8002680A0701103Full;
//DUPS: 8002680A0701143F, 8002680E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_BIT_STAT1_EO_PL         = 0x8002680A0801103Full;
//DUPS: 8002680A0801143F, 8002680E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_BIT_STAT1_EO_PL         = 0x8002680A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_BIT_STAT1_EO_PL       = 0x8002680A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_BIT_STAT1_EO_PL       = 0x8002680A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_BIT_STAT1_EO_PL       = 0x8002680E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_BIT_STAT1_EO_PL       = 0x8002680A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_BIT_STAT1_EO_PL       = 0x8002680A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_BIT_STAT1_EO_PL       = 0x8002680E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_BIT_STAT1_EO_PL        = 0x8002680A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_BIT_STAT1_EO_PL        = 0x8002680A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_BIT_STAT1_EO_PL        = 0x8002680E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_BIT_STAT1_EO_PL        = 0x8002680A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_BIT_STAT1_EO_PL       = 0x8002680A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_BIT_STAT1_EO_PL       = 0x8002680E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_BIT_STAT1_O_PL            = 0x8003180A0701103Full;
//DUPS: 8003180A0701143F, 8003180E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_BIT_STAT1_O_PL          = 0x8003180A0701103Full;
//DUPS: 8003180A0701143F, 8003180E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_BIT_STAT1_O_PL          = 0x8003180A0801103Full;
//DUPS: 8003180A0801143F, 8003180E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_BIT_STAT1_O_PL          = 0x8003180A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_BIT_STAT1_O_PL        = 0x8003180A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_BIT_STAT1_O_PL        = 0x8003180A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_BIT_STAT1_O_PL        = 0x8003180E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_BIT_STAT1_O_PL        = 0x8003180A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_BIT_STAT1_O_PL        = 0x8003180A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_BIT_STAT1_O_PL        = 0x8003180E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_BIT_STAT1_O_PL         = 0x8003180A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_BIT_STAT1_O_PL         = 0x8003180A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_BIT_STAT1_O_PL         = 0x8003180E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_BIT_STAT1_O_PL         = 0x8003180A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_BIT_STAT1_O_PL        = 0x8003180A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_BIT_STAT1_O_PL        = 0x8003180E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_BIT_STAT2_EO_PL           = 0x8002700A0701103Full;
//DUPS: 8002700A0701143F, 8002700E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_BIT_STAT2_EO_PL         = 0x8002700A0701103Full;
//DUPS: 8002700A0701143F, 8002700E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_BIT_STAT2_EO_PL         = 0x8002700A0801103Full;
//DUPS: 8002700A0801143F, 8002700E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_BIT_STAT2_EO_PL         = 0x8002700A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_BIT_STAT2_EO_PL       = 0x8002700A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_BIT_STAT2_EO_PL       = 0x8002700A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_BIT_STAT2_EO_PL       = 0x8002700E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_BIT_STAT2_EO_PL       = 0x8002700A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_BIT_STAT2_EO_PL       = 0x8002700A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_BIT_STAT2_EO_PL       = 0x8002700E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_BIT_STAT2_EO_PL        = 0x8002700A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_BIT_STAT2_EO_PL        = 0x8002700A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_BIT_STAT2_EO_PL        = 0x8002700E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_BIT_STAT2_EO_PL        = 0x8002700A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_BIT_STAT2_EO_PL       = 0x8002700A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_BIT_STAT2_EO_PL       = 0x8002700E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_BIT_STAT3_EO_PL           = 0x8002780A0701103Full;
//DUPS: 8002780A0701143F, 8002780E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_BIT_STAT3_EO_PL         = 0x8002780A0701103Full;
//DUPS: 8002780A0701143F, 8002780E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_BIT_STAT3_EO_PL         = 0x8002780A0801103Full;
//DUPS: 8002780A0801143F, 8002780E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_BIT_STAT3_EO_PL         = 0x8002780A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_BIT_STAT3_EO_PL       = 0x8002780A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_BIT_STAT3_EO_PL       = 0x8002780A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_BIT_STAT3_EO_PL       = 0x8002780E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_BIT_STAT3_EO_PL       = 0x8002780A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_BIT_STAT3_EO_PL       = 0x8002780A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_BIT_STAT3_EO_PL       = 0x8002780E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_BIT_STAT3_EO_PL        = 0x8002780A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_BIT_STAT3_EO_PL        = 0x8002780A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_BIT_STAT3_EO_PL        = 0x8002780E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_BIT_STAT3_EO_PL        = 0x8002780A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_BIT_STAT3_EO_PL       = 0x8002780A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_BIT_STAT3_EO_PL       = 0x8002780E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_BIT_STAT4_EO_PL           = 0x8002800A0701103Full;
//DUPS: 8002800A0701143F, 8002800E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_BIT_STAT4_EO_PL         = 0x8002800A0701103Full;
//DUPS: 8002800A0701143F, 8002800E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_BIT_STAT4_EO_PL         = 0x8002800A0801103Full;
//DUPS: 8002800A0801143F, 8002800E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_BIT_STAT4_EO_PL         = 0x8002800A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_BIT_STAT4_EO_PL       = 0x8002800A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_BIT_STAT4_EO_PL       = 0x8002800A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_BIT_STAT4_EO_PL       = 0x8002800E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_BIT_STAT4_EO_PL       = 0x8002800A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_BIT_STAT4_EO_PL       = 0x8002800A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_BIT_STAT4_EO_PL       = 0x8002800E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_BIT_STAT4_EO_PL        = 0x8002800A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_BIT_STAT4_EO_PL        = 0x8002800A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_BIT_STAT4_EO_PL        = 0x8002800E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_BIT_STAT4_EO_PL        = 0x8002800A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_BIT_STAT4_EO_PL       = 0x8002800A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_BIT_STAT4_EO_PL       = 0x8002800E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL10_O_PL           = 0x8000C80A0701103Full;
//DUPS: 8000C80A0701143F, 8000C80E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL10_O_PL         = 0x8000C80A0701103Full;
//DUPS: 8000C80A0701143F, 8000C80E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL10_O_PL         = 0x8000C80A0801103Full;
//DUPS: 8000C80A0801143F, 8000C80E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL10_O_PL         = 0x8000C80A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C80A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C80A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C80E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C80A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C80A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C80E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL10_O_PL        = 0x8000C80A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL10_O_PL        = 0x8000C80A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL10_O_PL        = 0x8000C80E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL10_O_PL        = 0x8000C80A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C80A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C80E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL1_EO_PL           = 0x8000080A0701103Full;
//DUPS: 8000080A0701143F, 8000080E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL1_EO_PL         = 0x8000080A0701103Full;
//DUPS: 8000080A0701143F, 8000080E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL1_EO_PL         = 0x8000080A0801103Full;
//DUPS: 8000080A0801143F, 8000080E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL1_EO_PL         = 0x8000080A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x8000080A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x8000080A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x8000080E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x8000080A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x8000080A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x8000080E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL1_EO_PL        = 0x8000080A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL1_EO_PL        = 0x8000080A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL1_EO_PL        = 0x8000080E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL1_EO_PL        = 0x8000080A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x8000080A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x8000080E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL1_O_PL            = 0x8000800A0701103Full;
//DUPS: 8000800A0701143F, 8000800E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL1_O_PL          = 0x8000800A0701103Full;
//DUPS: 8000800A0701143F, 8000800E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL1_O_PL          = 0x8000800A0801103Full;
//DUPS: 8000800A0801143F, 8000800E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL1_O_PL          = 0x8000800A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL1_O_PL        = 0x8000800A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL1_O_PL        = 0x8000800A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL1_O_PL        = 0x8000800E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL1_O_PL        = 0x8000800A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL1_O_PL        = 0x8000800A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL1_O_PL        = 0x8000800E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL1_O_PL         = 0x8000800A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL1_O_PL         = 0x8000800A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL1_O_PL         = 0x8000800E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL1_O_PL         = 0x8000800A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL1_O_PL        = 0x8000800A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL1_O_PL        = 0x8000800E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL2_EO_PL           = 0x8000100A0701103Full;
//DUPS: 8000100A0701143F, 8000100E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL2_EO_PL         = 0x8000100A0701103Full;
//DUPS: 8000100A0701143F, 8000100E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL2_EO_PL         = 0x8000100A0801103Full;
//DUPS: 8000100A0801143F, 8000100E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL2_EO_PL         = 0x8000100A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x8000100A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x8000100A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x8000100E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x8000100A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x8000100A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x8000100E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL2_EO_PL        = 0x8000100A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL2_EO_PL        = 0x8000100A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL2_EO_PL        = 0x8000100E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL2_EO_PL        = 0x8000100A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x8000100A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x8000100E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL2_O_PL            = 0x8000880A0701103Full;
//DUPS: 8000880A0701143F, 8000880E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL2_O_PL          = 0x8000880A0701103Full;
//DUPS: 8000880A0701143F, 8000880E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL2_O_PL          = 0x8000880A0801103Full;
//DUPS: 8000880A0801143F, 8000880E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL2_O_PL          = 0x8000880A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL2_O_PL        = 0x8000880A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL2_O_PL        = 0x8000880A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL2_O_PL        = 0x8000880E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL2_O_PL        = 0x8000880A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL2_O_PL        = 0x8000880A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL2_O_PL        = 0x8000880E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL2_O_PL         = 0x8000880A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL2_O_PL         = 0x8000880A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL2_O_PL         = 0x8000880E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL2_O_PL         = 0x8000880A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL2_O_PL        = 0x8000880A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL2_O_PL        = 0x8000880E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL3_EO_PL           = 0x8000180A0701103Full;
//DUPS: 8000180A0701143F, 8000180E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL3_EO_PL         = 0x8000180A0701103Full;
//DUPS: 8000180A0701143F, 8000180E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL3_EO_PL         = 0x8000180A0801103Full;
//DUPS: 8000180A0801143F, 8000180E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL3_EO_PL         = 0x8000180A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x8000180A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x8000180A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x8000180E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x8000180A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x8000180A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x8000180E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL3_EO_PL        = 0x8000180A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL3_EO_PL        = 0x8000180A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL3_EO_PL        = 0x8000180E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL3_EO_PL        = 0x8000180A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x8000180A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x8000180E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL3_O_PL            = 0x8000900A0701103Full;
//DUPS: 8000900A0701143F, 8000900E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL3_O_PL          = 0x8000900A0701103Full;
//DUPS: 8000900A0701143F, 8000900E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL3_O_PL          = 0x8000900A0801103Full;
//DUPS: 8000900A0801143F, 8000900E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL3_O_PL          = 0x8000900A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL3_O_PL        = 0x8000900A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL3_O_PL        = 0x8000900A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL3_O_PL        = 0x8000900E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL3_O_PL        = 0x8000900A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL3_O_PL        = 0x8000900A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL3_O_PL        = 0x8000900E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL3_O_PL         = 0x8000900A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL3_O_PL         = 0x8000900A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL3_O_PL         = 0x8000900E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL3_O_PL         = 0x8000900A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL3_O_PL        = 0x8000900A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL3_O_PL        = 0x8000900E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL4_EO_PL           = 0x8000200A0701103Full;
//DUPS: 8000200A0701143F, 8000200E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL4_EO_PL         = 0x8000200A0701103Full;
//DUPS: 8000200A0701143F, 8000200E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL4_EO_PL         = 0x8000200A0801103Full;
//DUPS: 8000200A0801143F, 8000200E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL4_EO_PL         = 0x8000200A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x8000200A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x8000200A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x8000200E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x8000200A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x8000200A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x8000200E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL4_EO_PL        = 0x8000200A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL4_EO_PL        = 0x8000200A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL4_EO_PL        = 0x8000200E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL4_EO_PL        = 0x8000200A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x8000200A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x8000200E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL4_O_PL            = 0x8000980A0701103Full;
//DUPS: 8000980A0701143F, 8000980E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL4_O_PL          = 0x8000980A0701103Full;
//DUPS: 8000980A0701143F, 8000980E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL4_O_PL          = 0x8000980A0801103Full;
//DUPS: 8000980A0801143F, 8000980E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL4_O_PL          = 0x8000980A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL4_O_PL        = 0x8000980A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL4_O_PL        = 0x8000980A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL4_O_PL        = 0x8000980E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL4_O_PL        = 0x8000980A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL4_O_PL        = 0x8000980A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL4_O_PL        = 0x8000980E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL4_O_PL         = 0x8000980A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL4_O_PL         = 0x8000980A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL4_O_PL         = 0x8000980E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL4_O_PL         = 0x8000980A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL4_O_PL        = 0x8000980A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL4_O_PL        = 0x8000980E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL5_EO_PL           = 0x8000280A0701103Full;
//DUPS: 8000280A0701143F, 8000280E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL5_EO_PL         = 0x8000280A0701103Full;
//DUPS: 8000280A0701143F, 8000280E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL5_EO_PL         = 0x8000280A0801103Full;
//DUPS: 8000280A0801143F, 8000280E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL5_EO_PL         = 0x8000280A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x8000280A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x8000280A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x8000280E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x8000280A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x8000280A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x8000280E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL5_EO_PL        = 0x8000280A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL5_EO_PL        = 0x8000280A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL5_EO_PL        = 0x8000280E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL5_EO_PL        = 0x8000280A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x8000280A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x8000280E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL5_O_PL            = 0x8000A00A0701103Full;
//DUPS: 8000A00A0701143F, 8000A00E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL5_O_PL          = 0x8000A00A0701103Full;
//DUPS: 8000A00A0701143F, 8000A00E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL5_O_PL          = 0x8000A00A0801103Full;
//DUPS: 8000A00A0801143F, 8000A00E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL5_O_PL          = 0x8000A00A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A00A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A00A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A00E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A00A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A00A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A00E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL5_O_PL         = 0x8000A00A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL5_O_PL         = 0x8000A00A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL5_O_PL         = 0x8000A00E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL5_O_PL         = 0x8000A00A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A00A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A00E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL6_EO_PL           = 0x8000300A0701103Full;
//DUPS: 8000300A0701143F, 8000300E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL6_EO_PL         = 0x8000300A0701103Full;
//DUPS: 8000300A0701143F, 8000300E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL6_EO_PL         = 0x8000300A0801103Full;
//DUPS: 8000300A0801143F, 8000300E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL6_EO_PL         = 0x8000300A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x8000300A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x8000300A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x8000300E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x8000300A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x8000300A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x8000300E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL6_EO_PL        = 0x8000300A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL6_EO_PL        = 0x8000300A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL6_EO_PL        = 0x8000300E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL6_EO_PL        = 0x8000300A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x8000300A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x8000300E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL6_O_PL            = 0x8000A80A0701103Full;
//DUPS: 8000A80A0701143F, 8000A80E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL6_O_PL          = 0x8000A80A0701103Full;
//DUPS: 8000A80A0701143F, 8000A80E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL6_O_PL          = 0x8000A80A0801103Full;
//DUPS: 8000A80A0801143F, 8000A80E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL6_O_PL          = 0x8000A80A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A80A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A80A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A80E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A80A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A80A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A80E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL6_O_PL         = 0x8000A80A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL6_O_PL         = 0x8000A80A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL6_O_PL         = 0x8000A80E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL6_O_PL         = 0x8000A80A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A80A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A80E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL7_EO_PL           = 0x8000380A0701103Full;
//DUPS: 8000380A0701143F, 8000380E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL7_EO_PL         = 0x8000380A0701103Full;
//DUPS: 8000380A0701143F, 8000380E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL7_EO_PL         = 0x8000380A0801103Full;
//DUPS: 8000380A0801143F, 8000380E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL7_EO_PL         = 0x8000380A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x8000380A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x8000380A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x8000380E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x8000380A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x8000380A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x8000380E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL7_EO_PL        = 0x8000380A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL7_EO_PL        = 0x8000380A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL7_EO_PL        = 0x8000380E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL7_EO_PL        = 0x8000380A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x8000380A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x8000380E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL7_O_PL            = 0x8000B00A0701103Full;
//DUPS: 8000B00A0701143F, 8000B00E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL7_O_PL          = 0x8000B00A0701103Full;
//DUPS: 8000B00A0701143F, 8000B00E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL7_O_PL          = 0x8000B00A0801103Full;
//DUPS: 8000B00A0801143F, 8000B00E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL7_O_PL          = 0x8000B00A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B00A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B00A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B00E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B00A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B00A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B00E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL7_O_PL         = 0x8000B00A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL7_O_PL         = 0x8000B00A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL7_O_PL         = 0x8000B00E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL7_O_PL         = 0x8000B00A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B00A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B00E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL8_EO_PL           = 0x8000400A0701103Full;
//DUPS: 8000400A0701143F, 8000400E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL8_EO_PL         = 0x8000400A0701103Full;
//DUPS: 8000400A0701143F, 8000400E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL8_EO_PL         = 0x8000400A0801103Full;
//DUPS: 8000400A0801143F, 8000400E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL8_EO_PL         = 0x8000400A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x8000400A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x8000400A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x8000400E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x8000400A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x8000400A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x8000400E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL8_EO_PL        = 0x8000400A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL8_EO_PL        = 0x8000400A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL8_EO_PL        = 0x8000400E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL8_EO_PL        = 0x8000400A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x8000400A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x8000400E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL8_O_PL            = 0x8000B80A0701103Full;
//DUPS: 8000B80A0701143F, 8000B80E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL8_O_PL          = 0x8000B80A0701103Full;
//DUPS: 8000B80A0701143F, 8000B80E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL8_O_PL          = 0x8000B80A0801103Full;
//DUPS: 8000B80A0801143F, 8000B80E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL8_O_PL          = 0x8000B80A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B80A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B80A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B80E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B80A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B80A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B80E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL8_O_PL         = 0x8000B80A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL8_O_PL         = 0x8000B80A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL8_O_PL         = 0x8000B80E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL8_O_PL         = 0x8000B80A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B80A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B80E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL9_O_PL            = 0x8000C00A0701103Full;
//DUPS: 8000C00A0701143F, 8000C00E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL9_O_PL          = 0x8000C00A0701103Full;
//DUPS: 8000C00A0701143F, 8000C00E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL9_O_PL          = 0x8000C00A0801103Full;
//DUPS: 8000C00A0801143F, 8000C00E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL9_O_PL          = 0x8000C00A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C00A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C00A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C00E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C00A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C00A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C00E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL9_O_PL         = 0x8000C00A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL9_O_PL         = 0x8000C00A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL9_O_PL         = 0x8000C00E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL9_O_PL         = 0x8000C00A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C00A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C00E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_DATA_DAC_SPARE_MODE_PL    = 0x8000000A0701103Full;
//DUPS: 8000000A0701143F, 8000000E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_DATA_DAC_SPARE_MODE_PL  = 0x8000000A0701103Full;
//DUPS: 8000000A0701143F, 8000000E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_DATA_DAC_SPARE_MODE_PL  = 0x8000000A0801103Full;
//DUPS: 8000000A0801143F, 8000000E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_DATA_DAC_SPARE_MODE_PL  = 0x8000000A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_FIR_ERROR_INJECT_PL       = 0x8002180A0701103Full;
//DUPS: 8002180A0701143F, 8002180E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_FIR_ERROR_INJECT_PL     = 0x8002180A0701103Full;
//DUPS: 8002180A0701143F, 8002180E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_FIR_ERROR_INJECT_PL     = 0x8002180A0801103Full;
//DUPS: 8002180A0801143F, 8002180E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_FIR_ERROR_INJECT_PL     = 0x8002180A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x8002180A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x8002180A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x8002180E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x8002180A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x8002180A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x8002180E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_FIR_ERROR_INJECT_PL    = 0x8002180A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_FIR_ERROR_INJECT_PL    = 0x8002180A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_FIR_ERROR_INJECT_PL    = 0x8002180E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_FIR_ERROR_INJECT_PL    = 0x8002180A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x8002180A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x8002180E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_FIR_MASK_PL               = 0x8002100A0701103Full;
//DUPS: 8002100A0701143F, 8002100E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_FIR_MASK_PL             = 0x8002100A0701103Full;
//DUPS: 8002100A0701143F, 8002100E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_FIR_MASK_PL             = 0x8002100A0801103Full;
//DUPS: 8002100A0801143F, 8002100E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_FIR_MASK_PL             = 0x8002100A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_FIR_MASK_PL           = 0x8002100A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_FIR_MASK_PL           = 0x8002100A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_FIR_MASK_PL           = 0x8002100E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_FIR_MASK_PL           = 0x8002100A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_FIR_MASK_PL           = 0x8002100A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_FIR_MASK_PL           = 0x8002100E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_FIR_MASK_PL            = 0x8002100A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_FIR_MASK_PL            = 0x8002100A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_FIR_MASK_PL            = 0x8002100E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_FIR_MASK_PL            = 0x8002100A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_FIR_MASK_PL           = 0x8002100A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_FIR_MASK_PL           = 0x8002100E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_FIR_PL                    = 0x8002080A0701103Full;
//DUPS: 8002080A0701143F, 8002080E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_FIR_PL                  = 0x8002080A0701103Full;
//DUPS: 8002080A0701143F, 8002080E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_FIR_PL                  = 0x8002080A0801103Full;
//DUPS: 8002080A0801143F, 8002080E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_FIR_PL                  = 0x8002080A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_FIR_PL                = 0x8002080A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_FIR_PL                = 0x8002080A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_FIR_PL                = 0x8002080E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_FIR_PL                = 0x8002080A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_FIR_PL                = 0x8002080A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_FIR_PL                = 0x8002080E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_FIR_PL                 = 0x8002080A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_FIR_PL                 = 0x8002080A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_FIR_PL                 = 0x8002080E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_FIR_PL                 = 0x8002080A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_FIR_PL                = 0x8002080A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_FIR_PL                = 0x8002080E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE2_RX_SPARE_MODE_PL             = 0x8002000A0701103Full;
//DUPS: 8002000A0701143F, 8002000E0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE2_RX_SPARE_MODE_PL           = 0x8002000A0701103Full;
//DUPS: 8002000A0701143F, 8002000E0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE2_RX_SPARE_MODE_PL           = 0x8002000A0801103Full;
//DUPS: 8002000A0801143F, 8002000E0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE2_RX_SPARE_MODE_PL           = 0x8002000A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE2_RX_SPARE_MODE_PL         = 0x8002000A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE2_RX_SPARE_MODE_PL         = 0x8002000A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE2_RX_SPARE_MODE_PL         = 0x8002000E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE2_RX_SPARE_MODE_PL         = 0x8002000A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE2_RX_SPARE_MODE_PL         = 0x8002000A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE2_RX_SPARE_MODE_PL         = 0x8002000E0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE2_RX_SPARE_MODE_PL          = 0x8002000A0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE2_RX_SPARE_MODE_PL          = 0x8002000A0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE2_RX_SPARE_MODE_PL          = 0x8002000E0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE2_RX_SPARE_MODE_PL          = 0x8002000A0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE2_RX_SPARE_MODE_PL         = 0x8002000A0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE2_RX_SPARE_MODE_PL         = 0x8002000E0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL2_EO_PL           = 0x8002480B0701103Full;
//DUPS: 8002480B0701143F, 8002480F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL2_EO_PL         = 0x8002480B0701103Full;
//DUPS: 8002480B0701143F, 8002480F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL2_EO_PL         = 0x8002480B0801103Full;
//DUPS: 8002480B0801143F, 8002480F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL2_EO_PL         = 0x8002480B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x8002480B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x8002480B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x8002480F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x8002480B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x8002480B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x8002480F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL2_EO_PL        = 0x8002480B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL2_EO_PL        = 0x8002480B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL2_EO_PL        = 0x8002480F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL2_EO_PL        = 0x8002480B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x8002480B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x8002480F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL3_EO_PL           = 0x8002500B0701103Full;
//DUPS: 8002500B0701143F, 8002500F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL3_EO_PL         = 0x8002500B0701103Full;
//DUPS: 8002500B0701143F, 8002500F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL3_EO_PL         = 0x8002500B0801103Full;
//DUPS: 8002500B0801143F, 8002500F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL3_EO_PL         = 0x8002500B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x8002500B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x8002500B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x8002500F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x8002500B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x8002500B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x8002500F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL3_EO_PL        = 0x8002500B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL3_EO_PL        = 0x8002500B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL3_EO_PL        = 0x8002500F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL3_EO_PL        = 0x8002500B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x8002500B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x8002500F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL4_EO_PL           = 0x8002580B0701103Full;
//DUPS: 8002580B0701143F, 8002580F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL4_EO_PL         = 0x8002580B0701103Full;
//DUPS: 8002580B0701143F, 8002580F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL4_EO_PL         = 0x8002580B0801103Full;
//DUPS: 8002580B0801143F, 8002580F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL4_EO_PL         = 0x8002580B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x8002580B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x8002580B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x8002580F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x8002580B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x8002580B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x8002580F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL4_EO_PL        = 0x8002580B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL4_EO_PL        = 0x8002580B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL4_EO_PL        = 0x8002580F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL4_EO_PL        = 0x8002580B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x8002580B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x8002580F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL5_EO_PL           = 0x8002600B0701103Full;
//DUPS: 8002600B0701143F, 8002600F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL5_EO_PL         = 0x8002600B0701103Full;
//DUPS: 8002600B0701143F, 8002600F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL5_EO_PL         = 0x8002600B0801103Full;
//DUPS: 8002600B0801143F, 8002600F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL5_EO_PL         = 0x8002600B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x8002600B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x8002600B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x8002600F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x8002600B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x8002600B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x8002600F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL5_EO_PL        = 0x8002600B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL5_EO_PL        = 0x8002600B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL5_EO_PL        = 0x8002600F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL5_EO_PL        = 0x8002600B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x8002600B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x8002600F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_BIT_CNTLX1_EO_PL          = 0x8002400B0701103Full;
//DUPS: 8002400B0701143F, 8002400F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_BIT_CNTLX1_EO_PL        = 0x8002400B0701103Full;
//DUPS: 8002400B0701143F, 8002400F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_BIT_CNTLX1_EO_PL        = 0x8002400B0801103Full;
//DUPS: 8002400B0801143F, 8002400F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_BIT_CNTLX1_EO_PL        = 0x8002400B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x8002400B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x8002400B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x8002400F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x8002400B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x8002400B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x8002400F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_BIT_CNTLX1_EO_PL       = 0x8002400B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_BIT_CNTLX1_EO_PL       = 0x8002400B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_BIT_CNTLX1_EO_PL       = 0x8002400F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_BIT_CNTLX1_EO_PL       = 0x8002400B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x8002400B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x8002400F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_BIT_MODE1_EO_PL           = 0x8002200B0701103Full;
//DUPS: 8002200B0701143F, 8002200F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_BIT_MODE1_EO_PL         = 0x8002200B0701103Full;
//DUPS: 8002200B0701143F, 8002200F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_BIT_MODE1_EO_PL         = 0x8002200B0801103Full;
//DUPS: 8002200B0801143F, 8002200F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_BIT_MODE1_EO_PL         = 0x8002200B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_BIT_MODE1_EO_PL       = 0x8002200B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_BIT_MODE1_EO_PL       = 0x8002200B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_BIT_MODE1_EO_PL       = 0x8002200F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_BIT_MODE1_EO_PL       = 0x8002200B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_BIT_MODE1_EO_PL       = 0x8002200B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_BIT_MODE1_EO_PL       = 0x8002200F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_BIT_MODE1_EO_PL        = 0x8002200B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_BIT_MODE1_EO_PL        = 0x8002200B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_BIT_MODE1_EO_PL        = 0x8002200F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_BIT_MODE1_EO_PL        = 0x8002200B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_BIT_MODE1_EO_PL       = 0x8002200B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_BIT_MODE1_EO_PL       = 0x8002200F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_BIT_MODE2_EO_PL           = 0x8002280B0701103Full;
//DUPS: 8002280B0701143F, 8002280F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_BIT_MODE2_EO_PL         = 0x8002280B0701103Full;
//DUPS: 8002280B0701143F, 8002280F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_BIT_MODE2_EO_PL         = 0x8002280B0801103Full;
//DUPS: 8002280B0801143F, 8002280F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_BIT_MODE2_EO_PL         = 0x8002280B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_BIT_MODE2_EO_PL       = 0x8002280B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_BIT_MODE2_EO_PL       = 0x8002280B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_BIT_MODE2_EO_PL       = 0x8002280F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_BIT_MODE2_EO_PL       = 0x8002280B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_BIT_MODE2_EO_PL       = 0x8002280B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_BIT_MODE2_EO_PL       = 0x8002280F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_BIT_MODE2_EO_PL        = 0x8002280B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_BIT_MODE2_EO_PL        = 0x8002280B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_BIT_MODE2_EO_PL        = 0x8002280F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_BIT_MODE2_EO_PL        = 0x8002280B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_BIT_MODE2_EO_PL       = 0x8002280B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_BIT_MODE2_EO_PL       = 0x8002280F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_BIT_MODE3_EO_PL           = 0x8002300B0701103Full;
//DUPS: 8002300B0701143F, 8002300F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_BIT_MODE3_EO_PL         = 0x8002300B0701103Full;
//DUPS: 8002300B0701143F, 8002300F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_BIT_MODE3_EO_PL         = 0x8002300B0801103Full;
//DUPS: 8002300B0801143F, 8002300F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_BIT_MODE3_EO_PL         = 0x8002300B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_BIT_MODE3_EO_PL       = 0x8002300B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_BIT_MODE3_EO_PL       = 0x8002300B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_BIT_MODE3_EO_PL       = 0x8002300F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_BIT_MODE3_EO_PL       = 0x8002300B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_BIT_MODE3_EO_PL       = 0x8002300B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_BIT_MODE3_EO_PL       = 0x8002300F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_BIT_MODE3_EO_PL        = 0x8002300B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_BIT_MODE3_EO_PL        = 0x8002300B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_BIT_MODE3_EO_PL        = 0x8002300F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_BIT_MODE3_EO_PL        = 0x8002300B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_BIT_MODE3_EO_PL       = 0x8002300B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_BIT_MODE3_EO_PL       = 0x8002300F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_BIT_MODE4_EO_PL           = 0x8002380B0701103Full;
//DUPS: 8002380B0701143F, 8002380F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_BIT_MODE4_EO_PL         = 0x8002380B0701103Full;
//DUPS: 8002380B0701143F, 8002380F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_BIT_MODE4_EO_PL         = 0x8002380B0801103Full;
//DUPS: 8002380B0801143F, 8002380F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_BIT_MODE4_EO_PL         = 0x8002380B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_BIT_MODE4_EO_PL       = 0x8002380B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_BIT_MODE4_EO_PL       = 0x8002380B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_BIT_MODE4_EO_PL       = 0x8002380F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_BIT_MODE4_EO_PL       = 0x8002380B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_BIT_MODE4_EO_PL       = 0x8002380B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_BIT_MODE4_EO_PL       = 0x8002380F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_BIT_MODE4_EO_PL        = 0x8002380B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_BIT_MODE4_EO_PL        = 0x8002380B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_BIT_MODE4_EO_PL        = 0x8002380F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_BIT_MODE4_EO_PL        = 0x8002380B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_BIT_MODE4_EO_PL       = 0x8002380B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_BIT_MODE4_EO_PL       = 0x8002380F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_BIT_STAT1_EO_PL           = 0x8002680B0701103Full;
//DUPS: 8002680B0701143F, 8002680F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_BIT_STAT1_EO_PL         = 0x8002680B0701103Full;
//DUPS: 8002680B0701143F, 8002680F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_BIT_STAT1_EO_PL         = 0x8002680B0801103Full;
//DUPS: 8002680B0801143F, 8002680F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_BIT_STAT1_EO_PL         = 0x8002680B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_BIT_STAT1_EO_PL       = 0x8002680B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_BIT_STAT1_EO_PL       = 0x8002680B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_BIT_STAT1_EO_PL       = 0x8002680F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_BIT_STAT1_EO_PL       = 0x8002680B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_BIT_STAT1_EO_PL       = 0x8002680B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_BIT_STAT1_EO_PL       = 0x8002680F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_BIT_STAT1_EO_PL        = 0x8002680B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_BIT_STAT1_EO_PL        = 0x8002680B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_BIT_STAT1_EO_PL        = 0x8002680F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_BIT_STAT1_EO_PL        = 0x8002680B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_BIT_STAT1_EO_PL       = 0x8002680B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_BIT_STAT1_EO_PL       = 0x8002680F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_BIT_STAT1_O_PL            = 0x8003180B0701103Full;
//DUPS: 8003180B0701143F, 8003180F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_BIT_STAT1_O_PL          = 0x8003180B0701103Full;
//DUPS: 8003180B0701143F, 8003180F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_BIT_STAT1_O_PL          = 0x8003180B0801103Full;
//DUPS: 8003180B0801143F, 8003180F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_BIT_STAT1_O_PL          = 0x8003180B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_BIT_STAT1_O_PL        = 0x8003180B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_BIT_STAT1_O_PL        = 0x8003180B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_BIT_STAT1_O_PL        = 0x8003180F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_BIT_STAT1_O_PL        = 0x8003180B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_BIT_STAT1_O_PL        = 0x8003180B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_BIT_STAT1_O_PL        = 0x8003180F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_BIT_STAT1_O_PL         = 0x8003180B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_BIT_STAT1_O_PL         = 0x8003180B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_BIT_STAT1_O_PL         = 0x8003180F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_BIT_STAT1_O_PL         = 0x8003180B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_BIT_STAT1_O_PL        = 0x8003180B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_BIT_STAT1_O_PL        = 0x8003180F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_BIT_STAT2_EO_PL           = 0x8002700B0701103Full;
//DUPS: 8002700B0701143F, 8002700F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_BIT_STAT2_EO_PL         = 0x8002700B0701103Full;
//DUPS: 8002700B0701143F, 8002700F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_BIT_STAT2_EO_PL         = 0x8002700B0801103Full;
//DUPS: 8002700B0801143F, 8002700F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_BIT_STAT2_EO_PL         = 0x8002700B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_BIT_STAT2_EO_PL       = 0x8002700B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_BIT_STAT2_EO_PL       = 0x8002700B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_BIT_STAT2_EO_PL       = 0x8002700F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_BIT_STAT2_EO_PL       = 0x8002700B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_BIT_STAT2_EO_PL       = 0x8002700B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_BIT_STAT2_EO_PL       = 0x8002700F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_BIT_STAT2_EO_PL        = 0x8002700B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_BIT_STAT2_EO_PL        = 0x8002700B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_BIT_STAT2_EO_PL        = 0x8002700F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_BIT_STAT2_EO_PL        = 0x8002700B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_BIT_STAT2_EO_PL       = 0x8002700B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_BIT_STAT2_EO_PL       = 0x8002700F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_BIT_STAT3_EO_PL           = 0x8002780B0701103Full;
//DUPS: 8002780B0701143F, 8002780F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_BIT_STAT3_EO_PL         = 0x8002780B0701103Full;
//DUPS: 8002780B0701143F, 8002780F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_BIT_STAT3_EO_PL         = 0x8002780B0801103Full;
//DUPS: 8002780B0801143F, 8002780F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_BIT_STAT3_EO_PL         = 0x8002780B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_BIT_STAT3_EO_PL       = 0x8002780B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_BIT_STAT3_EO_PL       = 0x8002780B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_BIT_STAT3_EO_PL       = 0x8002780F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_BIT_STAT3_EO_PL       = 0x8002780B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_BIT_STAT3_EO_PL       = 0x8002780B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_BIT_STAT3_EO_PL       = 0x8002780F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_BIT_STAT3_EO_PL        = 0x8002780B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_BIT_STAT3_EO_PL        = 0x8002780B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_BIT_STAT3_EO_PL        = 0x8002780F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_BIT_STAT3_EO_PL        = 0x8002780B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_BIT_STAT3_EO_PL       = 0x8002780B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_BIT_STAT3_EO_PL       = 0x8002780F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_BIT_STAT4_EO_PL           = 0x8002800B0701103Full;
//DUPS: 8002800B0701143F, 8002800F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_BIT_STAT4_EO_PL         = 0x8002800B0701103Full;
//DUPS: 8002800B0701143F, 8002800F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_BIT_STAT4_EO_PL         = 0x8002800B0801103Full;
//DUPS: 8002800B0801143F, 8002800F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_BIT_STAT4_EO_PL         = 0x8002800B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_BIT_STAT4_EO_PL       = 0x8002800B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_BIT_STAT4_EO_PL       = 0x8002800B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_BIT_STAT4_EO_PL       = 0x8002800F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_BIT_STAT4_EO_PL       = 0x8002800B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_BIT_STAT4_EO_PL       = 0x8002800B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_BIT_STAT4_EO_PL       = 0x8002800F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_BIT_STAT4_EO_PL        = 0x8002800B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_BIT_STAT4_EO_PL        = 0x8002800B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_BIT_STAT4_EO_PL        = 0x8002800F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_BIT_STAT4_EO_PL        = 0x8002800B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_BIT_STAT4_EO_PL       = 0x8002800B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_BIT_STAT4_EO_PL       = 0x8002800F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL10_O_PL           = 0x8000C80B0701103Full;
//DUPS: 8000C80B0701143F, 8000C80F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL10_O_PL         = 0x8000C80B0701103Full;
//DUPS: 8000C80B0701143F, 8000C80F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL10_O_PL         = 0x8000C80B0801103Full;
//DUPS: 8000C80B0801143F, 8000C80F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL10_O_PL         = 0x8000C80B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C80B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C80B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C80F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C80B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C80B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C80F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL10_O_PL        = 0x8000C80B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL10_O_PL        = 0x8000C80B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL10_O_PL        = 0x8000C80F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL10_O_PL        = 0x8000C80B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C80B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C80F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL1_EO_PL           = 0x8000080B0701103Full;
//DUPS: 8000080B0701143F, 8000080F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL1_EO_PL         = 0x8000080B0701103Full;
//DUPS: 8000080B0701143F, 8000080F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL1_EO_PL         = 0x8000080B0801103Full;
//DUPS: 8000080B0801143F, 8000080F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL1_EO_PL         = 0x8000080B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x8000080B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x8000080B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x8000080F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x8000080B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x8000080B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x8000080F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL1_EO_PL        = 0x8000080B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL1_EO_PL        = 0x8000080B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL1_EO_PL        = 0x8000080F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL1_EO_PL        = 0x8000080B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x8000080B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x8000080F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL1_O_PL            = 0x8000800B0701103Full;
//DUPS: 8000800B0701143F, 8000800F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL1_O_PL          = 0x8000800B0701103Full;
//DUPS: 8000800B0701143F, 8000800F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL1_O_PL          = 0x8000800B0801103Full;
//DUPS: 8000800B0801143F, 8000800F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL1_O_PL          = 0x8000800B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL1_O_PL        = 0x8000800B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL1_O_PL        = 0x8000800B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL1_O_PL        = 0x8000800F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL1_O_PL        = 0x8000800B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL1_O_PL        = 0x8000800B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL1_O_PL        = 0x8000800F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL1_O_PL         = 0x8000800B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL1_O_PL         = 0x8000800B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL1_O_PL         = 0x8000800F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL1_O_PL         = 0x8000800B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL1_O_PL        = 0x8000800B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL1_O_PL        = 0x8000800F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL2_EO_PL           = 0x8000100B0701103Full;
//DUPS: 8000100B0701143F, 8000100F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL2_EO_PL         = 0x8000100B0701103Full;
//DUPS: 8000100B0701143F, 8000100F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL2_EO_PL         = 0x8000100B0801103Full;
//DUPS: 8000100B0801143F, 8000100F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL2_EO_PL         = 0x8000100B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x8000100B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x8000100B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x8000100F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x8000100B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x8000100B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x8000100F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL2_EO_PL        = 0x8000100B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL2_EO_PL        = 0x8000100B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL2_EO_PL        = 0x8000100F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL2_EO_PL        = 0x8000100B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x8000100B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x8000100F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL2_O_PL            = 0x8000880B0701103Full;
//DUPS: 8000880B0701143F, 8000880F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL2_O_PL          = 0x8000880B0701103Full;
//DUPS: 8000880B0701143F, 8000880F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL2_O_PL          = 0x8000880B0801103Full;
//DUPS: 8000880B0801143F, 8000880F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL2_O_PL          = 0x8000880B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL2_O_PL        = 0x8000880B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL2_O_PL        = 0x8000880B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL2_O_PL        = 0x8000880F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL2_O_PL        = 0x8000880B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL2_O_PL        = 0x8000880B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL2_O_PL        = 0x8000880F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL2_O_PL         = 0x8000880B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL2_O_PL         = 0x8000880B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL2_O_PL         = 0x8000880F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL2_O_PL         = 0x8000880B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL2_O_PL        = 0x8000880B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL2_O_PL        = 0x8000880F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL3_EO_PL           = 0x8000180B0701103Full;
//DUPS: 8000180B0701143F, 8000180F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL3_EO_PL         = 0x8000180B0701103Full;
//DUPS: 8000180B0701143F, 8000180F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL3_EO_PL         = 0x8000180B0801103Full;
//DUPS: 8000180B0801143F, 8000180F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL3_EO_PL         = 0x8000180B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x8000180B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x8000180B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x8000180F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x8000180B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x8000180B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x8000180F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL3_EO_PL        = 0x8000180B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL3_EO_PL        = 0x8000180B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL3_EO_PL        = 0x8000180F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL3_EO_PL        = 0x8000180B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x8000180B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x8000180F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL3_O_PL            = 0x8000900B0701103Full;
//DUPS: 8000900B0701143F, 8000900F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL3_O_PL          = 0x8000900B0701103Full;
//DUPS: 8000900B0701143F, 8000900F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL3_O_PL          = 0x8000900B0801103Full;
//DUPS: 8000900B0801143F, 8000900F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL3_O_PL          = 0x8000900B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL3_O_PL        = 0x8000900B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL3_O_PL        = 0x8000900B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL3_O_PL        = 0x8000900F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL3_O_PL        = 0x8000900B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL3_O_PL        = 0x8000900B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL3_O_PL        = 0x8000900F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL3_O_PL         = 0x8000900B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL3_O_PL         = 0x8000900B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL3_O_PL         = 0x8000900F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL3_O_PL         = 0x8000900B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL3_O_PL        = 0x8000900B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL3_O_PL        = 0x8000900F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL4_EO_PL           = 0x8000200B0701103Full;
//DUPS: 8000200B0701143F, 8000200F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL4_EO_PL         = 0x8000200B0701103Full;
//DUPS: 8000200B0701143F, 8000200F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL4_EO_PL         = 0x8000200B0801103Full;
//DUPS: 8000200B0801143F, 8000200F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL4_EO_PL         = 0x8000200B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x8000200B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x8000200B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x8000200F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x8000200B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x8000200B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x8000200F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL4_EO_PL        = 0x8000200B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL4_EO_PL        = 0x8000200B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL4_EO_PL        = 0x8000200F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL4_EO_PL        = 0x8000200B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x8000200B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x8000200F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL4_O_PL            = 0x8000980B0701103Full;
//DUPS: 8000980B0701143F, 8000980F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL4_O_PL          = 0x8000980B0701103Full;
//DUPS: 8000980B0701143F, 8000980F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL4_O_PL          = 0x8000980B0801103Full;
//DUPS: 8000980B0801143F, 8000980F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL4_O_PL          = 0x8000980B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL4_O_PL        = 0x8000980B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL4_O_PL        = 0x8000980B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL4_O_PL        = 0x8000980F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL4_O_PL        = 0x8000980B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL4_O_PL        = 0x8000980B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL4_O_PL        = 0x8000980F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL4_O_PL         = 0x8000980B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL4_O_PL         = 0x8000980B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL4_O_PL         = 0x8000980F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL4_O_PL         = 0x8000980B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL4_O_PL        = 0x8000980B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL4_O_PL        = 0x8000980F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL5_EO_PL           = 0x8000280B0701103Full;
//DUPS: 8000280B0701143F, 8000280F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL5_EO_PL         = 0x8000280B0701103Full;
//DUPS: 8000280B0701143F, 8000280F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL5_EO_PL         = 0x8000280B0801103Full;
//DUPS: 8000280B0801143F, 8000280F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL5_EO_PL         = 0x8000280B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x8000280B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x8000280B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x8000280F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x8000280B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x8000280B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x8000280F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL5_EO_PL        = 0x8000280B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL5_EO_PL        = 0x8000280B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL5_EO_PL        = 0x8000280F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL5_EO_PL        = 0x8000280B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x8000280B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x8000280F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL5_O_PL            = 0x8000A00B0701103Full;
//DUPS: 8000A00B0701143F, 8000A00F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL5_O_PL          = 0x8000A00B0701103Full;
//DUPS: 8000A00B0701143F, 8000A00F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL5_O_PL          = 0x8000A00B0801103Full;
//DUPS: 8000A00B0801143F, 8000A00F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL5_O_PL          = 0x8000A00B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A00B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A00B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A00F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A00B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A00B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A00F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL5_O_PL         = 0x8000A00B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL5_O_PL         = 0x8000A00B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL5_O_PL         = 0x8000A00F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL5_O_PL         = 0x8000A00B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A00B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A00F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL6_EO_PL           = 0x8000300B0701103Full;
//DUPS: 8000300B0701143F, 8000300F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL6_EO_PL         = 0x8000300B0701103Full;
//DUPS: 8000300B0701143F, 8000300F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL6_EO_PL         = 0x8000300B0801103Full;
//DUPS: 8000300B0801143F, 8000300F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL6_EO_PL         = 0x8000300B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x8000300B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x8000300B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x8000300F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x8000300B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x8000300B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x8000300F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL6_EO_PL        = 0x8000300B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL6_EO_PL        = 0x8000300B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL6_EO_PL        = 0x8000300F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL6_EO_PL        = 0x8000300B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x8000300B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x8000300F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL6_O_PL            = 0x8000A80B0701103Full;
//DUPS: 8000A80B0701143F, 8000A80F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL6_O_PL          = 0x8000A80B0701103Full;
//DUPS: 8000A80B0701143F, 8000A80F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL6_O_PL          = 0x8000A80B0801103Full;
//DUPS: 8000A80B0801143F, 8000A80F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL6_O_PL          = 0x8000A80B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A80B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A80B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A80F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A80B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A80B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A80F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL6_O_PL         = 0x8000A80B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL6_O_PL         = 0x8000A80B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL6_O_PL         = 0x8000A80F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL6_O_PL         = 0x8000A80B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A80B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A80F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL7_EO_PL           = 0x8000380B0701103Full;
//DUPS: 8000380B0701143F, 8000380F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL7_EO_PL         = 0x8000380B0701103Full;
//DUPS: 8000380B0701143F, 8000380F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL7_EO_PL         = 0x8000380B0801103Full;
//DUPS: 8000380B0801143F, 8000380F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL7_EO_PL         = 0x8000380B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x8000380B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x8000380B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x8000380F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x8000380B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x8000380B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x8000380F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL7_EO_PL        = 0x8000380B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL7_EO_PL        = 0x8000380B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL7_EO_PL        = 0x8000380F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL7_EO_PL        = 0x8000380B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x8000380B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x8000380F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL7_O_PL            = 0x8000B00B0701103Full;
//DUPS: 8000B00B0701143F, 8000B00F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL7_O_PL          = 0x8000B00B0701103Full;
//DUPS: 8000B00B0701143F, 8000B00F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL7_O_PL          = 0x8000B00B0801103Full;
//DUPS: 8000B00B0801143F, 8000B00F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL7_O_PL          = 0x8000B00B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B00B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B00B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B00F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B00B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B00B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B00F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL7_O_PL         = 0x8000B00B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL7_O_PL         = 0x8000B00B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL7_O_PL         = 0x8000B00F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL7_O_PL         = 0x8000B00B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B00B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B00F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL8_EO_PL           = 0x8000400B0701103Full;
//DUPS: 8000400B0701143F, 8000400F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL8_EO_PL         = 0x8000400B0701103Full;
//DUPS: 8000400B0701143F, 8000400F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL8_EO_PL         = 0x8000400B0801103Full;
//DUPS: 8000400B0801143F, 8000400F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL8_EO_PL         = 0x8000400B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x8000400B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x8000400B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x8000400F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x8000400B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x8000400B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x8000400F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL8_EO_PL        = 0x8000400B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL8_EO_PL        = 0x8000400B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL8_EO_PL        = 0x8000400F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL8_EO_PL        = 0x8000400B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x8000400B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x8000400F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL8_O_PL            = 0x8000B80B0701103Full;
//DUPS: 8000B80B0701143F, 8000B80F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL8_O_PL          = 0x8000B80B0701103Full;
//DUPS: 8000B80B0701143F, 8000B80F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL8_O_PL          = 0x8000B80B0801103Full;
//DUPS: 8000B80B0801143F, 8000B80F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL8_O_PL          = 0x8000B80B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B80B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B80B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B80F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B80B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B80B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B80F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL8_O_PL         = 0x8000B80B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL8_O_PL         = 0x8000B80B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL8_O_PL         = 0x8000B80F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL8_O_PL         = 0x8000B80B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B80B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B80F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL9_O_PL            = 0x8000C00B0701103Full;
//DUPS: 8000C00B0701143F, 8000C00F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL9_O_PL          = 0x8000C00B0701103Full;
//DUPS: 8000C00B0701143F, 8000C00F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL9_O_PL          = 0x8000C00B0801103Full;
//DUPS: 8000C00B0801143F, 8000C00F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL9_O_PL          = 0x8000C00B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C00B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C00B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C00F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C00B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C00B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C00F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL9_O_PL         = 0x8000C00B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL9_O_PL         = 0x8000C00B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL9_O_PL         = 0x8000C00F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL9_O_PL         = 0x8000C00B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C00B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C00F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_DATA_DAC_SPARE_MODE_PL    = 0x8000000B0701103Full;
//DUPS: 8000000B0701143F, 8000000F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_DATA_DAC_SPARE_MODE_PL  = 0x8000000B0701103Full;
//DUPS: 8000000B0701143F, 8000000F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_DATA_DAC_SPARE_MODE_PL  = 0x8000000B0801103Full;
//DUPS: 8000000B0801143F, 8000000F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_DATA_DAC_SPARE_MODE_PL  = 0x8000000B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_FIR_ERROR_INJECT_PL       = 0x8002180B0701103Full;
//DUPS: 8002180B0701143F, 8002180F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_FIR_ERROR_INJECT_PL     = 0x8002180B0701103Full;
//DUPS: 8002180B0701143F, 8002180F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_FIR_ERROR_INJECT_PL     = 0x8002180B0801103Full;
//DUPS: 8002180B0801143F, 8002180F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_FIR_ERROR_INJECT_PL     = 0x8002180B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x8002180B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x8002180B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x8002180F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x8002180B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x8002180B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x8002180F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_FIR_ERROR_INJECT_PL    = 0x8002180B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_FIR_ERROR_INJECT_PL    = 0x8002180B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_FIR_ERROR_INJECT_PL    = 0x8002180F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_FIR_ERROR_INJECT_PL    = 0x8002180B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x8002180B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x8002180F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_FIR_MASK_PL               = 0x8002100B0701103Full;
//DUPS: 8002100B0701143F, 8002100F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_FIR_MASK_PL             = 0x8002100B0701103Full;
//DUPS: 8002100B0701143F, 8002100F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_FIR_MASK_PL             = 0x8002100B0801103Full;
//DUPS: 8002100B0801143F, 8002100F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_FIR_MASK_PL             = 0x8002100B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_FIR_MASK_PL           = 0x8002100B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_FIR_MASK_PL           = 0x8002100B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_FIR_MASK_PL           = 0x8002100F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_FIR_MASK_PL           = 0x8002100B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_FIR_MASK_PL           = 0x8002100B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_FIR_MASK_PL           = 0x8002100F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_FIR_MASK_PL            = 0x8002100B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_FIR_MASK_PL            = 0x8002100B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_FIR_MASK_PL            = 0x8002100F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_FIR_MASK_PL            = 0x8002100B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_FIR_MASK_PL           = 0x8002100B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_FIR_MASK_PL           = 0x8002100F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_FIR_PL                    = 0x8002080B0701103Full;
//DUPS: 8002080B0701143F, 8002080F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_FIR_PL                  = 0x8002080B0701103Full;
//DUPS: 8002080B0701143F, 8002080F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_FIR_PL                  = 0x8002080B0801103Full;
//DUPS: 8002080B0801143F, 8002080F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_FIR_PL                  = 0x8002080B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_FIR_PL                = 0x8002080B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_FIR_PL                = 0x8002080B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_FIR_PL                = 0x8002080F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_FIR_PL                = 0x8002080B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_FIR_PL                = 0x8002080B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_FIR_PL                = 0x8002080F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_FIR_PL                 = 0x8002080B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_FIR_PL                 = 0x8002080B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_FIR_PL                 = 0x8002080F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_FIR_PL                 = 0x8002080B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_FIR_PL                = 0x8002080B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_FIR_PL                = 0x8002080F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS2_SLICE3_RX_SPARE_MODE_PL             = 0x8002000B0701103Full;
//DUPS: 8002000B0701143F, 8002000F0701183F,
static const uint64_t P9A_MC_0_RX0_RXPACKS2_SLICE3_RX_SPARE_MODE_PL           = 0x8002000B0701103Full;
//DUPS: 8002000B0701143F, 8002000F0701183F,
static const uint64_t P9A_MC_1_RX0_RXPACKS2_SLICE3_RX_SPARE_MODE_PL           = 0x8002000B0801103Full;
//DUPS: 8002000B0801143F, 8002000F0801183F,

static const uint64_t P9A_OMIC_RX0_RXPACKS2_SLICE3_RX_SPARE_MODE_PL           = 0x8002000B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS2_SLICE3_RX_SPARE_MODE_PL         = 0x8002000B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS2_SLICE3_RX_SPARE_MODE_PL         = 0x8002000B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_RXPACKS2_SLICE3_RX_SPARE_MODE_PL         = 0x8002000F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS2_SLICE3_RX_SPARE_MODE_PL         = 0x8002000B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS2_SLICE3_RX_SPARE_MODE_PL         = 0x8002000B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_RXPACKS2_SLICE3_RX_SPARE_MODE_PL         = 0x8002000F0801183Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS2_SLICE3_RX_SPARE_MODE_PL          = 0x8002000B0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS2_SLICE3_RX_SPARE_MODE_PL          = 0x8002000B0701143Full;

static const uint64_t P9A_OMI_7_RX0_RXPACKS2_SLICE3_RX_SPARE_MODE_PL          = 0x8002000F0701183Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS2_SLICE3_RX_SPARE_MODE_PL          = 0x8002000B0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS2_SLICE3_RX_SPARE_MODE_PL         = 0x8002000B0801143Full;

static const uint64_t P9A_OMI_15_RX0_RXPACKS2_SLICE3_RX_SPARE_MODE_PL         = 0x8002000F0801183Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL2_EO_PL           = 0x8002480C0701103Full;
//DUPS: 8002480C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL2_EO_PL         = 0x8002480C0701103Full;
//DUPS: 8002480C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL2_EO_PL         = 0x8002480C0801103Full;
//DUPS: 8002480C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL2_EO_PL         = 0x8002480C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x8002480C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x8002480C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x8002480C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x8002480C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL2_EO_PL        = 0x8002480C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL2_EO_PL        = 0x8002480C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL2_EO_PL        = 0x8002480C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x8002480C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL3_EO_PL           = 0x8002500C0701103Full;
//DUPS: 8002500C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL3_EO_PL         = 0x8002500C0701103Full;
//DUPS: 8002500C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL3_EO_PL         = 0x8002500C0801103Full;
//DUPS: 8002500C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL3_EO_PL         = 0x8002500C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x8002500C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x8002500C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x8002500C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x8002500C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL3_EO_PL        = 0x8002500C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL3_EO_PL        = 0x8002500C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL3_EO_PL        = 0x8002500C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x8002500C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL4_EO_PL           = 0x8002580C0701103Full;
//DUPS: 8002580C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL4_EO_PL         = 0x8002580C0701103Full;
//DUPS: 8002580C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL4_EO_PL         = 0x8002580C0801103Full;
//DUPS: 8002580C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL4_EO_PL         = 0x8002580C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x8002580C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x8002580C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x8002580C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x8002580C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL4_EO_PL        = 0x8002580C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL4_EO_PL        = 0x8002580C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL4_EO_PL        = 0x8002580C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x8002580C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL5_EO_PL           = 0x8002600C0701103Full;
//DUPS: 8002600C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL5_EO_PL         = 0x8002600C0701103Full;
//DUPS: 8002600C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL5_EO_PL         = 0x8002600C0801103Full;
//DUPS: 8002600C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL5_EO_PL         = 0x8002600C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x8002600C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x8002600C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x8002600C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x8002600C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL5_EO_PL        = 0x8002600C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL5_EO_PL        = 0x8002600C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL5_EO_PL        = 0x8002600C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x8002600C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_BIT_CNTLX1_EO_PL          = 0x8002400C0701103Full;
//DUPS: 8002400C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_BIT_CNTLX1_EO_PL        = 0x8002400C0701103Full;
//DUPS: 8002400C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_BIT_CNTLX1_EO_PL        = 0x8002400C0801103Full;
//DUPS: 8002400C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_BIT_CNTLX1_EO_PL        = 0x8002400C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x8002400C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x8002400C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x8002400C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x8002400C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_BIT_CNTLX1_EO_PL       = 0x8002400C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_BIT_CNTLX1_EO_PL       = 0x8002400C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_BIT_CNTLX1_EO_PL       = 0x8002400C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x8002400C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_BIT_MODE1_EO_PL           = 0x8002200C0701103Full;
//DUPS: 8002200C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_BIT_MODE1_EO_PL         = 0x8002200C0701103Full;
//DUPS: 8002200C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_BIT_MODE1_EO_PL         = 0x8002200C0801103Full;
//DUPS: 8002200C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_BIT_MODE1_EO_PL         = 0x8002200C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_BIT_MODE1_EO_PL       = 0x8002200C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_BIT_MODE1_EO_PL       = 0x8002200C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_BIT_MODE1_EO_PL       = 0x8002200C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_BIT_MODE1_EO_PL       = 0x8002200C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_BIT_MODE1_EO_PL        = 0x8002200C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_BIT_MODE1_EO_PL        = 0x8002200C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_BIT_MODE1_EO_PL        = 0x8002200C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_BIT_MODE1_EO_PL       = 0x8002200C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_BIT_MODE2_EO_PL           = 0x8002280C0701103Full;
//DUPS: 8002280C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_BIT_MODE2_EO_PL         = 0x8002280C0701103Full;
//DUPS: 8002280C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_BIT_MODE2_EO_PL         = 0x8002280C0801103Full;
//DUPS: 8002280C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_BIT_MODE2_EO_PL         = 0x8002280C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_BIT_MODE2_EO_PL       = 0x8002280C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_BIT_MODE2_EO_PL       = 0x8002280C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_BIT_MODE2_EO_PL       = 0x8002280C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_BIT_MODE2_EO_PL       = 0x8002280C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_BIT_MODE2_EO_PL        = 0x8002280C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_BIT_MODE2_EO_PL        = 0x8002280C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_BIT_MODE2_EO_PL        = 0x8002280C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_BIT_MODE2_EO_PL       = 0x8002280C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_BIT_MODE3_EO_PL           = 0x8002300C0701103Full;
//DUPS: 8002300C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_BIT_MODE3_EO_PL         = 0x8002300C0701103Full;
//DUPS: 8002300C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_BIT_MODE3_EO_PL         = 0x8002300C0801103Full;
//DUPS: 8002300C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_BIT_MODE3_EO_PL         = 0x8002300C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_BIT_MODE3_EO_PL       = 0x8002300C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_BIT_MODE3_EO_PL       = 0x8002300C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_BIT_MODE3_EO_PL       = 0x8002300C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_BIT_MODE3_EO_PL       = 0x8002300C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_BIT_MODE3_EO_PL        = 0x8002300C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_BIT_MODE3_EO_PL        = 0x8002300C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_BIT_MODE3_EO_PL        = 0x8002300C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_BIT_MODE3_EO_PL       = 0x8002300C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_BIT_MODE4_EO_PL           = 0x8002380C0701103Full;
//DUPS: 8002380C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_BIT_MODE4_EO_PL         = 0x8002380C0701103Full;
//DUPS: 8002380C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_BIT_MODE4_EO_PL         = 0x8002380C0801103Full;
//DUPS: 8002380C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_BIT_MODE4_EO_PL         = 0x8002380C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_BIT_MODE4_EO_PL       = 0x8002380C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_BIT_MODE4_EO_PL       = 0x8002380C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_BIT_MODE4_EO_PL       = 0x8002380C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_BIT_MODE4_EO_PL       = 0x8002380C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_BIT_MODE4_EO_PL        = 0x8002380C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_BIT_MODE4_EO_PL        = 0x8002380C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_BIT_MODE4_EO_PL        = 0x8002380C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_BIT_MODE4_EO_PL       = 0x8002380C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_BIT_STAT1_EO_PL           = 0x8002680C0701103Full;
//DUPS: 8002680C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_BIT_STAT1_EO_PL         = 0x8002680C0701103Full;
//DUPS: 8002680C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_BIT_STAT1_EO_PL         = 0x8002680C0801103Full;
//DUPS: 8002680C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_BIT_STAT1_EO_PL         = 0x8002680C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_BIT_STAT1_EO_PL       = 0x8002680C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_BIT_STAT1_EO_PL       = 0x8002680C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_BIT_STAT1_EO_PL       = 0x8002680C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_BIT_STAT1_EO_PL       = 0x8002680C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_BIT_STAT1_EO_PL        = 0x8002680C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_BIT_STAT1_EO_PL        = 0x8002680C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_BIT_STAT1_EO_PL        = 0x8002680C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_BIT_STAT1_EO_PL       = 0x8002680C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_BIT_STAT1_O_PL            = 0x8003180C0701103Full;
//DUPS: 8003180C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_BIT_STAT1_O_PL          = 0x8003180C0701103Full;
//DUPS: 8003180C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_BIT_STAT1_O_PL          = 0x8003180C0801103Full;
//DUPS: 8003180C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_BIT_STAT1_O_PL          = 0x8003180C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_BIT_STAT1_O_PL        = 0x8003180C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_BIT_STAT1_O_PL        = 0x8003180C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_BIT_STAT1_O_PL        = 0x8003180C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_BIT_STAT1_O_PL        = 0x8003180C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_BIT_STAT1_O_PL         = 0x8003180C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_BIT_STAT1_O_PL         = 0x8003180C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_BIT_STAT1_O_PL         = 0x8003180C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_BIT_STAT1_O_PL        = 0x8003180C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_BIT_STAT2_EO_PL           = 0x8002700C0701103Full;
//DUPS: 8002700C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_BIT_STAT2_EO_PL         = 0x8002700C0701103Full;
//DUPS: 8002700C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_BIT_STAT2_EO_PL         = 0x8002700C0801103Full;
//DUPS: 8002700C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_BIT_STAT2_EO_PL         = 0x8002700C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_BIT_STAT2_EO_PL       = 0x8002700C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_BIT_STAT2_EO_PL       = 0x8002700C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_BIT_STAT2_EO_PL       = 0x8002700C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_BIT_STAT2_EO_PL       = 0x8002700C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_BIT_STAT2_EO_PL        = 0x8002700C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_BIT_STAT2_EO_PL        = 0x8002700C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_BIT_STAT2_EO_PL        = 0x8002700C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_BIT_STAT2_EO_PL       = 0x8002700C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_BIT_STAT3_EO_PL           = 0x8002780C0701103Full;
//DUPS: 8002780C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_BIT_STAT3_EO_PL         = 0x8002780C0701103Full;
//DUPS: 8002780C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_BIT_STAT3_EO_PL         = 0x8002780C0801103Full;
//DUPS: 8002780C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_BIT_STAT3_EO_PL         = 0x8002780C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_BIT_STAT3_EO_PL       = 0x8002780C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_BIT_STAT3_EO_PL       = 0x8002780C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_BIT_STAT3_EO_PL       = 0x8002780C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_BIT_STAT3_EO_PL       = 0x8002780C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_BIT_STAT3_EO_PL        = 0x8002780C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_BIT_STAT3_EO_PL        = 0x8002780C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_BIT_STAT3_EO_PL        = 0x8002780C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_BIT_STAT3_EO_PL       = 0x8002780C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_BIT_STAT4_EO_PL           = 0x8002800C0701103Full;
//DUPS: 8002800C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_BIT_STAT4_EO_PL         = 0x8002800C0701103Full;
//DUPS: 8002800C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_BIT_STAT4_EO_PL         = 0x8002800C0801103Full;
//DUPS: 8002800C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_BIT_STAT4_EO_PL         = 0x8002800C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_BIT_STAT4_EO_PL       = 0x8002800C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_BIT_STAT4_EO_PL       = 0x8002800C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_BIT_STAT4_EO_PL       = 0x8002800C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_BIT_STAT4_EO_PL       = 0x8002800C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_BIT_STAT4_EO_PL        = 0x8002800C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_BIT_STAT4_EO_PL        = 0x8002800C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_BIT_STAT4_EO_PL        = 0x8002800C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_BIT_STAT4_EO_PL       = 0x8002800C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL10_O_PL           = 0x8000C80C0701103Full;
//DUPS: 8000C80C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL10_O_PL         = 0x8000C80C0701103Full;
//DUPS: 8000C80C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL10_O_PL         = 0x8000C80C0801103Full;
//DUPS: 8000C80C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL10_O_PL         = 0x8000C80C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C80C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C80C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C80C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C80C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL10_O_PL        = 0x8000C80C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL10_O_PL        = 0x8000C80C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL10_O_PL        = 0x8000C80C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C80C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL1_EO_PL           = 0x8000080C0701103Full;
//DUPS: 8000080C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL1_EO_PL         = 0x8000080C0701103Full;
//DUPS: 8000080C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL1_EO_PL         = 0x8000080C0801103Full;
//DUPS: 8000080C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL1_EO_PL         = 0x8000080C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x8000080C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x8000080C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x8000080C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x8000080C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL1_EO_PL        = 0x8000080C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL1_EO_PL        = 0x8000080C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL1_EO_PL        = 0x8000080C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x8000080C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL1_O_PL            = 0x8000800C0701103Full;
//DUPS: 8000800C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL1_O_PL          = 0x8000800C0701103Full;
//DUPS: 8000800C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL1_O_PL          = 0x8000800C0801103Full;
//DUPS: 8000800C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL1_O_PL          = 0x8000800C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL1_O_PL        = 0x8000800C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL1_O_PL        = 0x8000800C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL1_O_PL        = 0x8000800C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL1_O_PL        = 0x8000800C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL1_O_PL         = 0x8000800C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL1_O_PL         = 0x8000800C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL1_O_PL         = 0x8000800C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL1_O_PL        = 0x8000800C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL2_EO_PL           = 0x8000100C0701103Full;
//DUPS: 8000100C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL2_EO_PL         = 0x8000100C0701103Full;
//DUPS: 8000100C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL2_EO_PL         = 0x8000100C0801103Full;
//DUPS: 8000100C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL2_EO_PL         = 0x8000100C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x8000100C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x8000100C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x8000100C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x8000100C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL2_EO_PL        = 0x8000100C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL2_EO_PL        = 0x8000100C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL2_EO_PL        = 0x8000100C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x8000100C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL2_O_PL            = 0x8000880C0701103Full;
//DUPS: 8000880C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL2_O_PL          = 0x8000880C0701103Full;
//DUPS: 8000880C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL2_O_PL          = 0x8000880C0801103Full;
//DUPS: 8000880C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL2_O_PL          = 0x8000880C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL2_O_PL        = 0x8000880C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL2_O_PL        = 0x8000880C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL2_O_PL        = 0x8000880C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL2_O_PL        = 0x8000880C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL2_O_PL         = 0x8000880C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL2_O_PL         = 0x8000880C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL2_O_PL         = 0x8000880C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL2_O_PL        = 0x8000880C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL3_EO_PL           = 0x8000180C0701103Full;
//DUPS: 8000180C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL3_EO_PL         = 0x8000180C0701103Full;
//DUPS: 8000180C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL3_EO_PL         = 0x8000180C0801103Full;
//DUPS: 8000180C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL3_EO_PL         = 0x8000180C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x8000180C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x8000180C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x8000180C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x8000180C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL3_EO_PL        = 0x8000180C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL3_EO_PL        = 0x8000180C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL3_EO_PL        = 0x8000180C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x8000180C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL3_O_PL            = 0x8000900C0701103Full;
//DUPS: 8000900C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL3_O_PL          = 0x8000900C0701103Full;
//DUPS: 8000900C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL3_O_PL          = 0x8000900C0801103Full;
//DUPS: 8000900C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL3_O_PL          = 0x8000900C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL3_O_PL        = 0x8000900C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL3_O_PL        = 0x8000900C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL3_O_PL        = 0x8000900C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL3_O_PL        = 0x8000900C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL3_O_PL         = 0x8000900C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL3_O_PL         = 0x8000900C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL3_O_PL         = 0x8000900C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL3_O_PL        = 0x8000900C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL4_EO_PL           = 0x8000200C0701103Full;
//DUPS: 8000200C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL4_EO_PL         = 0x8000200C0701103Full;
//DUPS: 8000200C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL4_EO_PL         = 0x8000200C0801103Full;
//DUPS: 8000200C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL4_EO_PL         = 0x8000200C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x8000200C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x8000200C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x8000200C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x8000200C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL4_EO_PL        = 0x8000200C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL4_EO_PL        = 0x8000200C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL4_EO_PL        = 0x8000200C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x8000200C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL4_O_PL            = 0x8000980C0701103Full;
//DUPS: 8000980C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL4_O_PL          = 0x8000980C0701103Full;
//DUPS: 8000980C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL4_O_PL          = 0x8000980C0801103Full;
//DUPS: 8000980C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL4_O_PL          = 0x8000980C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL4_O_PL        = 0x8000980C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL4_O_PL        = 0x8000980C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL4_O_PL        = 0x8000980C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL4_O_PL        = 0x8000980C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL4_O_PL         = 0x8000980C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL4_O_PL         = 0x8000980C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL4_O_PL         = 0x8000980C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL4_O_PL        = 0x8000980C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL5_EO_PL           = 0x8000280C0701103Full;
//DUPS: 8000280C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL5_EO_PL         = 0x8000280C0701103Full;
//DUPS: 8000280C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL5_EO_PL         = 0x8000280C0801103Full;
//DUPS: 8000280C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL5_EO_PL         = 0x8000280C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x8000280C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x8000280C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x8000280C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x8000280C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL5_EO_PL        = 0x8000280C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL5_EO_PL        = 0x8000280C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL5_EO_PL        = 0x8000280C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x8000280C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL5_O_PL            = 0x8000A00C0701103Full;
//DUPS: 8000A00C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL5_O_PL          = 0x8000A00C0701103Full;
//DUPS: 8000A00C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL5_O_PL          = 0x8000A00C0801103Full;
//DUPS: 8000A00C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL5_O_PL          = 0x8000A00C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A00C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A00C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A00C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A00C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL5_O_PL         = 0x8000A00C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL5_O_PL         = 0x8000A00C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL5_O_PL         = 0x8000A00C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A00C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL6_EO_PL           = 0x8000300C0701103Full;
//DUPS: 8000300C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL6_EO_PL         = 0x8000300C0701103Full;
//DUPS: 8000300C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL6_EO_PL         = 0x8000300C0801103Full;
//DUPS: 8000300C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL6_EO_PL         = 0x8000300C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x8000300C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x8000300C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x8000300C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x8000300C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL6_EO_PL        = 0x8000300C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL6_EO_PL        = 0x8000300C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL6_EO_PL        = 0x8000300C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x8000300C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL6_O_PL            = 0x8000A80C0701103Full;
//DUPS: 8000A80C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL6_O_PL          = 0x8000A80C0701103Full;
//DUPS: 8000A80C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL6_O_PL          = 0x8000A80C0801103Full;
//DUPS: 8000A80C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL6_O_PL          = 0x8000A80C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A80C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A80C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A80C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A80C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL6_O_PL         = 0x8000A80C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL6_O_PL         = 0x8000A80C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL6_O_PL         = 0x8000A80C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A80C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL7_EO_PL           = 0x8000380C0701103Full;
//DUPS: 8000380C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL7_EO_PL         = 0x8000380C0701103Full;
//DUPS: 8000380C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL7_EO_PL         = 0x8000380C0801103Full;
//DUPS: 8000380C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL7_EO_PL         = 0x8000380C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x8000380C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x8000380C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x8000380C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x8000380C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL7_EO_PL        = 0x8000380C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL7_EO_PL        = 0x8000380C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL7_EO_PL        = 0x8000380C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x8000380C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL7_O_PL            = 0x8000B00C0701103Full;
//DUPS: 8000B00C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL7_O_PL          = 0x8000B00C0701103Full;
//DUPS: 8000B00C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL7_O_PL          = 0x8000B00C0801103Full;
//DUPS: 8000B00C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL7_O_PL          = 0x8000B00C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B00C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B00C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B00C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B00C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL7_O_PL         = 0x8000B00C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL7_O_PL         = 0x8000B00C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL7_O_PL         = 0x8000B00C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B00C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL8_EO_PL           = 0x8000400C0701103Full;
//DUPS: 8000400C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL8_EO_PL         = 0x8000400C0701103Full;
//DUPS: 8000400C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL8_EO_PL         = 0x8000400C0801103Full;
//DUPS: 8000400C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL8_EO_PL         = 0x8000400C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x8000400C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x8000400C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x8000400C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x8000400C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL8_EO_PL        = 0x8000400C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL8_EO_PL        = 0x8000400C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL8_EO_PL        = 0x8000400C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x8000400C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL8_O_PL            = 0x8000B80C0701103Full;
//DUPS: 8000B80C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL8_O_PL          = 0x8000B80C0701103Full;
//DUPS: 8000B80C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL8_O_PL          = 0x8000B80C0801103Full;
//DUPS: 8000B80C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL8_O_PL          = 0x8000B80C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B80C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B80C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B80C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B80C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL8_O_PL         = 0x8000B80C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL8_O_PL         = 0x8000B80C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL8_O_PL         = 0x8000B80C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B80C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL9_O_PL            = 0x8000C00C0701103Full;
//DUPS: 8000C00C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL9_O_PL          = 0x8000C00C0701103Full;
//DUPS: 8000C00C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL9_O_PL          = 0x8000C00C0801103Full;
//DUPS: 8000C00C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL9_O_PL          = 0x8000C00C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C00C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C00C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C00C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C00C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL9_O_PL         = 0x8000C00C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL9_O_PL         = 0x8000C00C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL9_O_PL         = 0x8000C00C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C00C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_DATA_DAC_SPARE_MODE_PL    = 0x8000000C0701103Full;
//DUPS: 8000000C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_DATA_DAC_SPARE_MODE_PL  = 0x8000000C0701103Full;
//DUPS: 8000000C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_DATA_DAC_SPARE_MODE_PL  = 0x8000000C0801103Full;
//DUPS: 8000000C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_DATA_DAC_SPARE_MODE_PL  = 0x8000000C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_FIR_ERROR_INJECT_PL       = 0x8002180C0701103Full;
//DUPS: 8002180C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_FIR_ERROR_INJECT_PL     = 0x8002180C0701103Full;
//DUPS: 8002180C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_FIR_ERROR_INJECT_PL     = 0x8002180C0801103Full;
//DUPS: 8002180C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_FIR_ERROR_INJECT_PL     = 0x8002180C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x8002180C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x8002180C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x8002180C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x8002180C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_FIR_ERROR_INJECT_PL    = 0x8002180C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_FIR_ERROR_INJECT_PL    = 0x8002180C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_FIR_ERROR_INJECT_PL    = 0x8002180C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x8002180C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_FIR_MASK_PL               = 0x8002100C0701103Full;
//DUPS: 8002100C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_FIR_MASK_PL             = 0x8002100C0701103Full;
//DUPS: 8002100C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_FIR_MASK_PL             = 0x8002100C0801103Full;
//DUPS: 8002100C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_FIR_MASK_PL             = 0x8002100C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_FIR_MASK_PL           = 0x8002100C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_FIR_MASK_PL           = 0x8002100C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_FIR_MASK_PL           = 0x8002100C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_FIR_MASK_PL           = 0x8002100C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_FIR_MASK_PL            = 0x8002100C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_FIR_MASK_PL            = 0x8002100C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_FIR_MASK_PL            = 0x8002100C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_FIR_MASK_PL           = 0x8002100C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_FIR_PL                    = 0x8002080C0701103Full;
//DUPS: 8002080C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_FIR_PL                  = 0x8002080C0701103Full;
//DUPS: 8002080C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_FIR_PL                  = 0x8002080C0801103Full;
//DUPS: 8002080C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_FIR_PL                  = 0x8002080C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_FIR_PL                = 0x8002080C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_FIR_PL                = 0x8002080C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_FIR_PL                = 0x8002080C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_FIR_PL                = 0x8002080C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_FIR_PL                 = 0x8002080C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_FIR_PL                 = 0x8002080C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_FIR_PL                 = 0x8002080C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_FIR_PL                = 0x8002080C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE0_RX_SPARE_MODE_PL             = 0x8002000C0701103Full;
//DUPS: 8002000C0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE0_RX_SPARE_MODE_PL           = 0x8002000C0701103Full;
//DUPS: 8002000C0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE0_RX_SPARE_MODE_PL           = 0x8002000C0801103Full;
//DUPS: 8002000C0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE0_RX_SPARE_MODE_PL           = 0x8002000C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE0_RX_SPARE_MODE_PL         = 0x8002000C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE0_RX_SPARE_MODE_PL         = 0x8002000C0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE0_RX_SPARE_MODE_PL         = 0x8002000C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE0_RX_SPARE_MODE_PL         = 0x8002000C0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE0_RX_SPARE_MODE_PL          = 0x8002000C0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE0_RX_SPARE_MODE_PL          = 0x8002000C0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE0_RX_SPARE_MODE_PL          = 0x8002000C0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE0_RX_SPARE_MODE_PL         = 0x8002000C0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL2_EO_PL           = 0x8002480D0701103Full;
//DUPS: 8002480D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL2_EO_PL         = 0x8002480D0701103Full;
//DUPS: 8002480D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL2_EO_PL         = 0x8002480D0801103Full;
//DUPS: 8002480D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL2_EO_PL         = 0x8002480D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x8002480D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x8002480D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x8002480D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x8002480D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL2_EO_PL        = 0x8002480D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL2_EO_PL        = 0x8002480D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL2_EO_PL        = 0x8002480D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x8002480D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL3_EO_PL           = 0x8002500D0701103Full;
//DUPS: 8002500D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL3_EO_PL         = 0x8002500D0701103Full;
//DUPS: 8002500D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL3_EO_PL         = 0x8002500D0801103Full;
//DUPS: 8002500D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL3_EO_PL         = 0x8002500D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x8002500D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x8002500D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x8002500D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x8002500D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL3_EO_PL        = 0x8002500D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL3_EO_PL        = 0x8002500D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL3_EO_PL        = 0x8002500D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x8002500D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL4_EO_PL           = 0x8002580D0701103Full;
//DUPS: 8002580D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL4_EO_PL         = 0x8002580D0701103Full;
//DUPS: 8002580D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL4_EO_PL         = 0x8002580D0801103Full;
//DUPS: 8002580D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL4_EO_PL         = 0x8002580D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x8002580D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x8002580D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x8002580D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x8002580D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL4_EO_PL        = 0x8002580D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL4_EO_PL        = 0x8002580D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL4_EO_PL        = 0x8002580D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x8002580D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL5_EO_PL           = 0x8002600D0701103Full;
//DUPS: 8002600D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL5_EO_PL         = 0x8002600D0701103Full;
//DUPS: 8002600D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL5_EO_PL         = 0x8002600D0801103Full;
//DUPS: 8002600D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL5_EO_PL         = 0x8002600D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x8002600D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x8002600D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x8002600D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x8002600D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL5_EO_PL        = 0x8002600D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL5_EO_PL        = 0x8002600D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL5_EO_PL        = 0x8002600D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x8002600D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_BIT_CNTLX1_EO_PL          = 0x8002400D0701103Full;
//DUPS: 8002400D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_BIT_CNTLX1_EO_PL        = 0x8002400D0701103Full;
//DUPS: 8002400D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_BIT_CNTLX1_EO_PL        = 0x8002400D0801103Full;
//DUPS: 8002400D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_BIT_CNTLX1_EO_PL        = 0x8002400D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x8002400D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x8002400D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x8002400D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x8002400D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_BIT_CNTLX1_EO_PL       = 0x8002400D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_BIT_CNTLX1_EO_PL       = 0x8002400D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_BIT_CNTLX1_EO_PL       = 0x8002400D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x8002400D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_BIT_MODE1_EO_PL           = 0x8002200D0701103Full;
//DUPS: 8002200D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_BIT_MODE1_EO_PL         = 0x8002200D0701103Full;
//DUPS: 8002200D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_BIT_MODE1_EO_PL         = 0x8002200D0801103Full;
//DUPS: 8002200D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_BIT_MODE1_EO_PL         = 0x8002200D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_BIT_MODE1_EO_PL       = 0x8002200D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_BIT_MODE1_EO_PL       = 0x8002200D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_BIT_MODE1_EO_PL       = 0x8002200D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_BIT_MODE1_EO_PL       = 0x8002200D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_BIT_MODE1_EO_PL        = 0x8002200D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_BIT_MODE1_EO_PL        = 0x8002200D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_BIT_MODE1_EO_PL        = 0x8002200D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_BIT_MODE1_EO_PL       = 0x8002200D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_BIT_MODE2_EO_PL           = 0x8002280D0701103Full;
//DUPS: 8002280D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_BIT_MODE2_EO_PL         = 0x8002280D0701103Full;
//DUPS: 8002280D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_BIT_MODE2_EO_PL         = 0x8002280D0801103Full;
//DUPS: 8002280D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_BIT_MODE2_EO_PL         = 0x8002280D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_BIT_MODE2_EO_PL       = 0x8002280D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_BIT_MODE2_EO_PL       = 0x8002280D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_BIT_MODE2_EO_PL       = 0x8002280D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_BIT_MODE2_EO_PL       = 0x8002280D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_BIT_MODE2_EO_PL        = 0x8002280D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_BIT_MODE2_EO_PL        = 0x8002280D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_BIT_MODE2_EO_PL        = 0x8002280D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_BIT_MODE2_EO_PL       = 0x8002280D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_BIT_MODE3_EO_PL           = 0x8002300D0701103Full;
//DUPS: 8002300D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_BIT_MODE3_EO_PL         = 0x8002300D0701103Full;
//DUPS: 8002300D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_BIT_MODE3_EO_PL         = 0x8002300D0801103Full;
//DUPS: 8002300D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_BIT_MODE3_EO_PL         = 0x8002300D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_BIT_MODE3_EO_PL       = 0x8002300D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_BIT_MODE3_EO_PL       = 0x8002300D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_BIT_MODE3_EO_PL       = 0x8002300D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_BIT_MODE3_EO_PL       = 0x8002300D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_BIT_MODE3_EO_PL        = 0x8002300D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_BIT_MODE3_EO_PL        = 0x8002300D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_BIT_MODE3_EO_PL        = 0x8002300D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_BIT_MODE3_EO_PL       = 0x8002300D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_BIT_MODE4_EO_PL           = 0x8002380D0701103Full;
//DUPS: 8002380D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_BIT_MODE4_EO_PL         = 0x8002380D0701103Full;
//DUPS: 8002380D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_BIT_MODE4_EO_PL         = 0x8002380D0801103Full;
//DUPS: 8002380D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_BIT_MODE4_EO_PL         = 0x8002380D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_BIT_MODE4_EO_PL       = 0x8002380D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_BIT_MODE4_EO_PL       = 0x8002380D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_BIT_MODE4_EO_PL       = 0x8002380D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_BIT_MODE4_EO_PL       = 0x8002380D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_BIT_MODE4_EO_PL        = 0x8002380D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_BIT_MODE4_EO_PL        = 0x8002380D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_BIT_MODE4_EO_PL        = 0x8002380D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_BIT_MODE4_EO_PL       = 0x8002380D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_BIT_STAT1_EO_PL           = 0x8002680D0701103Full;
//DUPS: 8002680D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_BIT_STAT1_EO_PL         = 0x8002680D0701103Full;
//DUPS: 8002680D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_BIT_STAT1_EO_PL         = 0x8002680D0801103Full;
//DUPS: 8002680D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_BIT_STAT1_EO_PL         = 0x8002680D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_BIT_STAT1_EO_PL       = 0x8002680D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_BIT_STAT1_EO_PL       = 0x8002680D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_BIT_STAT1_EO_PL       = 0x8002680D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_BIT_STAT1_EO_PL       = 0x8002680D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_BIT_STAT1_EO_PL        = 0x8002680D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_BIT_STAT1_EO_PL        = 0x8002680D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_BIT_STAT1_EO_PL        = 0x8002680D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_BIT_STAT1_EO_PL       = 0x8002680D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_BIT_STAT1_O_PL            = 0x8003180D0701103Full;
//DUPS: 8003180D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_BIT_STAT1_O_PL          = 0x8003180D0701103Full;
//DUPS: 8003180D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_BIT_STAT1_O_PL          = 0x8003180D0801103Full;
//DUPS: 8003180D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_BIT_STAT1_O_PL          = 0x8003180D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_BIT_STAT1_O_PL        = 0x8003180D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_BIT_STAT1_O_PL        = 0x8003180D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_BIT_STAT1_O_PL        = 0x8003180D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_BIT_STAT1_O_PL        = 0x8003180D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_BIT_STAT1_O_PL         = 0x8003180D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_BIT_STAT1_O_PL         = 0x8003180D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_BIT_STAT1_O_PL         = 0x8003180D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_BIT_STAT1_O_PL        = 0x8003180D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_BIT_STAT2_EO_PL           = 0x8002700D0701103Full;
//DUPS: 8002700D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_BIT_STAT2_EO_PL         = 0x8002700D0701103Full;
//DUPS: 8002700D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_BIT_STAT2_EO_PL         = 0x8002700D0801103Full;
//DUPS: 8002700D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_BIT_STAT2_EO_PL         = 0x8002700D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_BIT_STAT2_EO_PL       = 0x8002700D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_BIT_STAT2_EO_PL       = 0x8002700D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_BIT_STAT2_EO_PL       = 0x8002700D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_BIT_STAT2_EO_PL       = 0x8002700D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_BIT_STAT2_EO_PL        = 0x8002700D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_BIT_STAT2_EO_PL        = 0x8002700D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_BIT_STAT2_EO_PL        = 0x8002700D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_BIT_STAT2_EO_PL       = 0x8002700D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_BIT_STAT3_EO_PL           = 0x8002780D0701103Full;
//DUPS: 8002780D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_BIT_STAT3_EO_PL         = 0x8002780D0701103Full;
//DUPS: 8002780D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_BIT_STAT3_EO_PL         = 0x8002780D0801103Full;
//DUPS: 8002780D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_BIT_STAT3_EO_PL         = 0x8002780D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_BIT_STAT3_EO_PL       = 0x8002780D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_BIT_STAT3_EO_PL       = 0x8002780D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_BIT_STAT3_EO_PL       = 0x8002780D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_BIT_STAT3_EO_PL       = 0x8002780D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_BIT_STAT3_EO_PL        = 0x8002780D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_BIT_STAT3_EO_PL        = 0x8002780D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_BIT_STAT3_EO_PL        = 0x8002780D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_BIT_STAT3_EO_PL       = 0x8002780D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_BIT_STAT4_EO_PL           = 0x8002800D0701103Full;
//DUPS: 8002800D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_BIT_STAT4_EO_PL         = 0x8002800D0701103Full;
//DUPS: 8002800D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_BIT_STAT4_EO_PL         = 0x8002800D0801103Full;
//DUPS: 8002800D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_BIT_STAT4_EO_PL         = 0x8002800D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_BIT_STAT4_EO_PL       = 0x8002800D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_BIT_STAT4_EO_PL       = 0x8002800D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_BIT_STAT4_EO_PL       = 0x8002800D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_BIT_STAT4_EO_PL       = 0x8002800D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_BIT_STAT4_EO_PL        = 0x8002800D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_BIT_STAT4_EO_PL        = 0x8002800D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_BIT_STAT4_EO_PL        = 0x8002800D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_BIT_STAT4_EO_PL       = 0x8002800D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL10_O_PL           = 0x8000C80D0701103Full;
//DUPS: 8000C80D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL10_O_PL         = 0x8000C80D0701103Full;
//DUPS: 8000C80D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL10_O_PL         = 0x8000C80D0801103Full;
//DUPS: 8000C80D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL10_O_PL         = 0x8000C80D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C80D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C80D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C80D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C80D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL10_O_PL        = 0x8000C80D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL10_O_PL        = 0x8000C80D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL10_O_PL        = 0x8000C80D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C80D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL1_EO_PL           = 0x8000080D0701103Full;
//DUPS: 8000080D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL1_EO_PL         = 0x8000080D0701103Full;
//DUPS: 8000080D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL1_EO_PL         = 0x8000080D0801103Full;
//DUPS: 8000080D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL1_EO_PL         = 0x8000080D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x8000080D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x8000080D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x8000080D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x8000080D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL1_EO_PL        = 0x8000080D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL1_EO_PL        = 0x8000080D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL1_EO_PL        = 0x8000080D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x8000080D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL1_O_PL            = 0x8000800D0701103Full;
//DUPS: 8000800D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL1_O_PL          = 0x8000800D0701103Full;
//DUPS: 8000800D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL1_O_PL          = 0x8000800D0801103Full;
//DUPS: 8000800D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL1_O_PL          = 0x8000800D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL1_O_PL        = 0x8000800D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL1_O_PL        = 0x8000800D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL1_O_PL        = 0x8000800D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL1_O_PL        = 0x8000800D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL1_O_PL         = 0x8000800D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL1_O_PL         = 0x8000800D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL1_O_PL         = 0x8000800D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL1_O_PL        = 0x8000800D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL2_EO_PL           = 0x8000100D0701103Full;
//DUPS: 8000100D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL2_EO_PL         = 0x8000100D0701103Full;
//DUPS: 8000100D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL2_EO_PL         = 0x8000100D0801103Full;
//DUPS: 8000100D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL2_EO_PL         = 0x8000100D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x8000100D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x8000100D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x8000100D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x8000100D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL2_EO_PL        = 0x8000100D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL2_EO_PL        = 0x8000100D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL2_EO_PL        = 0x8000100D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x8000100D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL2_O_PL            = 0x8000880D0701103Full;
//DUPS: 8000880D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL2_O_PL          = 0x8000880D0701103Full;
//DUPS: 8000880D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL2_O_PL          = 0x8000880D0801103Full;
//DUPS: 8000880D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL2_O_PL          = 0x8000880D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL2_O_PL        = 0x8000880D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL2_O_PL        = 0x8000880D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL2_O_PL        = 0x8000880D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL2_O_PL        = 0x8000880D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL2_O_PL         = 0x8000880D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL2_O_PL         = 0x8000880D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL2_O_PL         = 0x8000880D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL2_O_PL        = 0x8000880D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL3_EO_PL           = 0x8000180D0701103Full;
//DUPS: 8000180D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL3_EO_PL         = 0x8000180D0701103Full;
//DUPS: 8000180D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL3_EO_PL         = 0x8000180D0801103Full;
//DUPS: 8000180D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL3_EO_PL         = 0x8000180D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x8000180D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x8000180D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x8000180D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x8000180D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL3_EO_PL        = 0x8000180D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL3_EO_PL        = 0x8000180D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL3_EO_PL        = 0x8000180D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x8000180D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL3_O_PL            = 0x8000900D0701103Full;
//DUPS: 8000900D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL3_O_PL          = 0x8000900D0701103Full;
//DUPS: 8000900D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL3_O_PL          = 0x8000900D0801103Full;
//DUPS: 8000900D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL3_O_PL          = 0x8000900D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL3_O_PL        = 0x8000900D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL3_O_PL        = 0x8000900D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL3_O_PL        = 0x8000900D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL3_O_PL        = 0x8000900D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL3_O_PL         = 0x8000900D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL3_O_PL         = 0x8000900D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL3_O_PL         = 0x8000900D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL3_O_PL        = 0x8000900D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL4_EO_PL           = 0x8000200D0701103Full;
//DUPS: 8000200D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL4_EO_PL         = 0x8000200D0701103Full;
//DUPS: 8000200D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL4_EO_PL         = 0x8000200D0801103Full;
//DUPS: 8000200D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL4_EO_PL         = 0x8000200D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x8000200D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x8000200D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x8000200D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x8000200D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL4_EO_PL        = 0x8000200D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL4_EO_PL        = 0x8000200D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL4_EO_PL        = 0x8000200D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x8000200D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL4_O_PL            = 0x8000980D0701103Full;
//DUPS: 8000980D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL4_O_PL          = 0x8000980D0701103Full;
//DUPS: 8000980D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL4_O_PL          = 0x8000980D0801103Full;
//DUPS: 8000980D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL4_O_PL          = 0x8000980D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL4_O_PL        = 0x8000980D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL4_O_PL        = 0x8000980D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL4_O_PL        = 0x8000980D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL4_O_PL        = 0x8000980D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL4_O_PL         = 0x8000980D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL4_O_PL         = 0x8000980D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL4_O_PL         = 0x8000980D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL4_O_PL        = 0x8000980D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL5_EO_PL           = 0x8000280D0701103Full;
//DUPS: 8000280D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL5_EO_PL         = 0x8000280D0701103Full;
//DUPS: 8000280D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL5_EO_PL         = 0x8000280D0801103Full;
//DUPS: 8000280D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL5_EO_PL         = 0x8000280D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x8000280D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x8000280D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x8000280D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x8000280D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL5_EO_PL        = 0x8000280D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL5_EO_PL        = 0x8000280D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL5_EO_PL        = 0x8000280D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x8000280D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL5_O_PL            = 0x8000A00D0701103Full;
//DUPS: 8000A00D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL5_O_PL          = 0x8000A00D0701103Full;
//DUPS: 8000A00D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL5_O_PL          = 0x8000A00D0801103Full;
//DUPS: 8000A00D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL5_O_PL          = 0x8000A00D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A00D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A00D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A00D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A00D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL5_O_PL         = 0x8000A00D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL5_O_PL         = 0x8000A00D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL5_O_PL         = 0x8000A00D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A00D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL6_EO_PL           = 0x8000300D0701103Full;
//DUPS: 8000300D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL6_EO_PL         = 0x8000300D0701103Full;
//DUPS: 8000300D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL6_EO_PL         = 0x8000300D0801103Full;
//DUPS: 8000300D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL6_EO_PL         = 0x8000300D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x8000300D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x8000300D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x8000300D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x8000300D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL6_EO_PL        = 0x8000300D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL6_EO_PL        = 0x8000300D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL6_EO_PL        = 0x8000300D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x8000300D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL6_O_PL            = 0x8000A80D0701103Full;
//DUPS: 8000A80D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL6_O_PL          = 0x8000A80D0701103Full;
//DUPS: 8000A80D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL6_O_PL          = 0x8000A80D0801103Full;
//DUPS: 8000A80D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL6_O_PL          = 0x8000A80D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A80D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A80D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A80D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A80D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL6_O_PL         = 0x8000A80D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL6_O_PL         = 0x8000A80D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL6_O_PL         = 0x8000A80D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A80D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL7_EO_PL           = 0x8000380D0701103Full;
//DUPS: 8000380D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL7_EO_PL         = 0x8000380D0701103Full;
//DUPS: 8000380D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL7_EO_PL         = 0x8000380D0801103Full;
//DUPS: 8000380D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL7_EO_PL         = 0x8000380D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x8000380D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x8000380D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x8000380D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x8000380D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL7_EO_PL        = 0x8000380D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL7_EO_PL        = 0x8000380D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL7_EO_PL        = 0x8000380D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x8000380D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL7_O_PL            = 0x8000B00D0701103Full;
//DUPS: 8000B00D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL7_O_PL          = 0x8000B00D0701103Full;
//DUPS: 8000B00D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL7_O_PL          = 0x8000B00D0801103Full;
//DUPS: 8000B00D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL7_O_PL          = 0x8000B00D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B00D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B00D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B00D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B00D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL7_O_PL         = 0x8000B00D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL7_O_PL         = 0x8000B00D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL7_O_PL         = 0x8000B00D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B00D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL8_EO_PL           = 0x8000400D0701103Full;
//DUPS: 8000400D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL8_EO_PL         = 0x8000400D0701103Full;
//DUPS: 8000400D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL8_EO_PL         = 0x8000400D0801103Full;
//DUPS: 8000400D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL8_EO_PL         = 0x8000400D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x8000400D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x8000400D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x8000400D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x8000400D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL8_EO_PL        = 0x8000400D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL8_EO_PL        = 0x8000400D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL8_EO_PL        = 0x8000400D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x8000400D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL8_O_PL            = 0x8000B80D0701103Full;
//DUPS: 8000B80D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL8_O_PL          = 0x8000B80D0701103Full;
//DUPS: 8000B80D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL8_O_PL          = 0x8000B80D0801103Full;
//DUPS: 8000B80D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL8_O_PL          = 0x8000B80D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B80D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B80D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B80D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B80D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL8_O_PL         = 0x8000B80D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL8_O_PL         = 0x8000B80D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL8_O_PL         = 0x8000B80D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B80D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL9_O_PL            = 0x8000C00D0701103Full;
//DUPS: 8000C00D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL9_O_PL          = 0x8000C00D0701103Full;
//DUPS: 8000C00D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL9_O_PL          = 0x8000C00D0801103Full;
//DUPS: 8000C00D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL9_O_PL          = 0x8000C00D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C00D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C00D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C00D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C00D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL9_O_PL         = 0x8000C00D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL9_O_PL         = 0x8000C00D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL9_O_PL         = 0x8000C00D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C00D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_DATA_DAC_SPARE_MODE_PL    = 0x8000000D0701103Full;
//DUPS: 8000000D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_DATA_DAC_SPARE_MODE_PL  = 0x8000000D0701103Full;
//DUPS: 8000000D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_DATA_DAC_SPARE_MODE_PL  = 0x8000000D0801103Full;
//DUPS: 8000000D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_DATA_DAC_SPARE_MODE_PL  = 0x8000000D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_FIR_ERROR_INJECT_PL       = 0x8002180D0701103Full;
//DUPS: 8002180D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_FIR_ERROR_INJECT_PL     = 0x8002180D0701103Full;
//DUPS: 8002180D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_FIR_ERROR_INJECT_PL     = 0x8002180D0801103Full;
//DUPS: 8002180D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_FIR_ERROR_INJECT_PL     = 0x8002180D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x8002180D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x8002180D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x8002180D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x8002180D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_FIR_ERROR_INJECT_PL    = 0x8002180D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_FIR_ERROR_INJECT_PL    = 0x8002180D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_FIR_ERROR_INJECT_PL    = 0x8002180D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x8002180D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_FIR_MASK_PL               = 0x8002100D0701103Full;
//DUPS: 8002100D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_FIR_MASK_PL             = 0x8002100D0701103Full;
//DUPS: 8002100D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_FIR_MASK_PL             = 0x8002100D0801103Full;
//DUPS: 8002100D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_FIR_MASK_PL             = 0x8002100D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_FIR_MASK_PL           = 0x8002100D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_FIR_MASK_PL           = 0x8002100D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_FIR_MASK_PL           = 0x8002100D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_FIR_MASK_PL           = 0x8002100D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_FIR_MASK_PL            = 0x8002100D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_FIR_MASK_PL            = 0x8002100D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_FIR_MASK_PL            = 0x8002100D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_FIR_MASK_PL           = 0x8002100D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_FIR_PL                    = 0x8002080D0701103Full;
//DUPS: 8002080D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_FIR_PL                  = 0x8002080D0701103Full;
//DUPS: 8002080D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_FIR_PL                  = 0x8002080D0801103Full;
//DUPS: 8002080D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_FIR_PL                  = 0x8002080D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_FIR_PL                = 0x8002080D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_FIR_PL                = 0x8002080D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_FIR_PL                = 0x8002080D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_FIR_PL                = 0x8002080D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_FIR_PL                 = 0x8002080D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_FIR_PL                 = 0x8002080D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_FIR_PL                 = 0x8002080D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_FIR_PL                = 0x8002080D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE1_RX_SPARE_MODE_PL             = 0x8002000D0701103Full;
//DUPS: 8002000D0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE1_RX_SPARE_MODE_PL           = 0x8002000D0701103Full;
//DUPS: 8002000D0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE1_RX_SPARE_MODE_PL           = 0x8002000D0801103Full;
//DUPS: 8002000D0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE1_RX_SPARE_MODE_PL           = 0x8002000D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE1_RX_SPARE_MODE_PL         = 0x8002000D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE1_RX_SPARE_MODE_PL         = 0x8002000D0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE1_RX_SPARE_MODE_PL         = 0x8002000D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE1_RX_SPARE_MODE_PL         = 0x8002000D0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE1_RX_SPARE_MODE_PL          = 0x8002000D0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE1_RX_SPARE_MODE_PL          = 0x8002000D0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE1_RX_SPARE_MODE_PL          = 0x8002000D0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE1_RX_SPARE_MODE_PL         = 0x8002000D0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL2_EO_PL           = 0x8002480E0701103Full;
//DUPS: 8002480E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL2_EO_PL         = 0x8002480E0701103Full;
//DUPS: 8002480E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL2_EO_PL         = 0x8002480E0801103Full;
//DUPS: 8002480E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL2_EO_PL         = 0x8002480E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x8002480E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x8002480E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x8002480E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x8002480E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL2_EO_PL        = 0x8002480E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL2_EO_PL        = 0x8002480E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL2_EO_PL        = 0x8002480E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x8002480E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL3_EO_PL           = 0x8002500E0701103Full;
//DUPS: 8002500E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL3_EO_PL         = 0x8002500E0701103Full;
//DUPS: 8002500E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL3_EO_PL         = 0x8002500E0801103Full;
//DUPS: 8002500E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL3_EO_PL         = 0x8002500E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x8002500E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x8002500E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x8002500E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x8002500E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL3_EO_PL        = 0x8002500E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL3_EO_PL        = 0x8002500E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL3_EO_PL        = 0x8002500E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x8002500E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL4_EO_PL           = 0x8002580E0701103Full;
//DUPS: 8002580E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL4_EO_PL         = 0x8002580E0701103Full;
//DUPS: 8002580E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL4_EO_PL         = 0x8002580E0801103Full;
//DUPS: 8002580E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL4_EO_PL         = 0x8002580E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x8002580E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x8002580E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x8002580E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x8002580E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL4_EO_PL        = 0x8002580E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL4_EO_PL        = 0x8002580E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL4_EO_PL        = 0x8002580E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x8002580E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL5_EO_PL           = 0x8002600E0701103Full;
//DUPS: 8002600E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL5_EO_PL         = 0x8002600E0701103Full;
//DUPS: 8002600E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL5_EO_PL         = 0x8002600E0801103Full;
//DUPS: 8002600E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL5_EO_PL         = 0x8002600E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x8002600E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x8002600E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x8002600E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x8002600E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL5_EO_PL        = 0x8002600E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL5_EO_PL        = 0x8002600E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL5_EO_PL        = 0x8002600E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x8002600E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_BIT_CNTLX1_EO_PL          = 0x8002400E0701103Full;
//DUPS: 8002400E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_BIT_CNTLX1_EO_PL        = 0x8002400E0701103Full;
//DUPS: 8002400E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_BIT_CNTLX1_EO_PL        = 0x8002400E0801103Full;
//DUPS: 8002400E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_BIT_CNTLX1_EO_PL        = 0x8002400E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x8002400E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x8002400E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x8002400E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x8002400E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_BIT_CNTLX1_EO_PL       = 0x8002400E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_BIT_CNTLX1_EO_PL       = 0x8002400E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_BIT_CNTLX1_EO_PL       = 0x8002400E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x8002400E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_BIT_MODE1_EO_PL           = 0x8002200E0701103Full;
//DUPS: 8002200E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_BIT_MODE1_EO_PL         = 0x8002200E0701103Full;
//DUPS: 8002200E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_BIT_MODE1_EO_PL         = 0x8002200E0801103Full;
//DUPS: 8002200E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_BIT_MODE1_EO_PL         = 0x8002200E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_BIT_MODE1_EO_PL       = 0x8002200E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_BIT_MODE1_EO_PL       = 0x8002200E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_BIT_MODE1_EO_PL       = 0x8002200E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_BIT_MODE1_EO_PL       = 0x8002200E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_BIT_MODE1_EO_PL        = 0x8002200E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_BIT_MODE1_EO_PL        = 0x8002200E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_BIT_MODE1_EO_PL        = 0x8002200E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_BIT_MODE1_EO_PL       = 0x8002200E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_BIT_MODE2_EO_PL           = 0x8002280E0701103Full;
//DUPS: 8002280E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_BIT_MODE2_EO_PL         = 0x8002280E0701103Full;
//DUPS: 8002280E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_BIT_MODE2_EO_PL         = 0x8002280E0801103Full;
//DUPS: 8002280E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_BIT_MODE2_EO_PL         = 0x8002280E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_BIT_MODE2_EO_PL       = 0x8002280E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_BIT_MODE2_EO_PL       = 0x8002280E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_BIT_MODE2_EO_PL       = 0x8002280E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_BIT_MODE2_EO_PL       = 0x8002280E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_BIT_MODE2_EO_PL        = 0x8002280E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_BIT_MODE2_EO_PL        = 0x8002280E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_BIT_MODE2_EO_PL        = 0x8002280E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_BIT_MODE2_EO_PL       = 0x8002280E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_BIT_MODE3_EO_PL           = 0x8002300E0701103Full;
//DUPS: 8002300E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_BIT_MODE3_EO_PL         = 0x8002300E0701103Full;
//DUPS: 8002300E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_BIT_MODE3_EO_PL         = 0x8002300E0801103Full;
//DUPS: 8002300E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_BIT_MODE3_EO_PL         = 0x8002300E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_BIT_MODE3_EO_PL       = 0x8002300E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_BIT_MODE3_EO_PL       = 0x8002300E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_BIT_MODE3_EO_PL       = 0x8002300E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_BIT_MODE3_EO_PL       = 0x8002300E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_BIT_MODE3_EO_PL        = 0x8002300E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_BIT_MODE3_EO_PL        = 0x8002300E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_BIT_MODE3_EO_PL        = 0x8002300E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_BIT_MODE3_EO_PL       = 0x8002300E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_BIT_MODE4_EO_PL           = 0x8002380E0701103Full;
//DUPS: 8002380E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_BIT_MODE4_EO_PL         = 0x8002380E0701103Full;
//DUPS: 8002380E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_BIT_MODE4_EO_PL         = 0x8002380E0801103Full;
//DUPS: 8002380E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_BIT_MODE4_EO_PL         = 0x8002380E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_BIT_MODE4_EO_PL       = 0x8002380E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_BIT_MODE4_EO_PL       = 0x8002380E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_BIT_MODE4_EO_PL       = 0x8002380E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_BIT_MODE4_EO_PL       = 0x8002380E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_BIT_MODE4_EO_PL        = 0x8002380E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_BIT_MODE4_EO_PL        = 0x8002380E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_BIT_MODE4_EO_PL        = 0x8002380E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_BIT_MODE4_EO_PL       = 0x8002380E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_BIT_STAT1_EO_PL           = 0x8002680E0701103Full;
//DUPS: 8002680E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_BIT_STAT1_EO_PL         = 0x8002680E0701103Full;
//DUPS: 8002680E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_BIT_STAT1_EO_PL         = 0x8002680E0801103Full;
//DUPS: 8002680E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_BIT_STAT1_EO_PL         = 0x8002680E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_BIT_STAT1_EO_PL       = 0x8002680E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_BIT_STAT1_EO_PL       = 0x8002680E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_BIT_STAT1_EO_PL       = 0x8002680E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_BIT_STAT1_EO_PL       = 0x8002680E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_BIT_STAT1_EO_PL        = 0x8002680E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_BIT_STAT1_EO_PL        = 0x8002680E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_BIT_STAT1_EO_PL        = 0x8002680E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_BIT_STAT1_EO_PL       = 0x8002680E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_BIT_STAT1_O_PL            = 0x8003180E0701103Full;
//DUPS: 8003180E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_BIT_STAT1_O_PL          = 0x8003180E0701103Full;
//DUPS: 8003180E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_BIT_STAT1_O_PL          = 0x8003180E0801103Full;
//DUPS: 8003180E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_BIT_STAT1_O_PL          = 0x8003180E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_BIT_STAT1_O_PL        = 0x8003180E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_BIT_STAT1_O_PL        = 0x8003180E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_BIT_STAT1_O_PL        = 0x8003180E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_BIT_STAT1_O_PL        = 0x8003180E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_BIT_STAT1_O_PL         = 0x8003180E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_BIT_STAT1_O_PL         = 0x8003180E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_BIT_STAT1_O_PL         = 0x8003180E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_BIT_STAT1_O_PL        = 0x8003180E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_BIT_STAT2_EO_PL           = 0x8002700E0701103Full;
//DUPS: 8002700E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_BIT_STAT2_EO_PL         = 0x8002700E0701103Full;
//DUPS: 8002700E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_BIT_STAT2_EO_PL         = 0x8002700E0801103Full;
//DUPS: 8002700E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_BIT_STAT2_EO_PL         = 0x8002700E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_BIT_STAT2_EO_PL       = 0x8002700E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_BIT_STAT2_EO_PL       = 0x8002700E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_BIT_STAT2_EO_PL       = 0x8002700E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_BIT_STAT2_EO_PL       = 0x8002700E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_BIT_STAT2_EO_PL        = 0x8002700E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_BIT_STAT2_EO_PL        = 0x8002700E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_BIT_STAT2_EO_PL        = 0x8002700E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_BIT_STAT2_EO_PL       = 0x8002700E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_BIT_STAT3_EO_PL           = 0x8002780E0701103Full;
//DUPS: 8002780E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_BIT_STAT3_EO_PL         = 0x8002780E0701103Full;
//DUPS: 8002780E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_BIT_STAT3_EO_PL         = 0x8002780E0801103Full;
//DUPS: 8002780E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_BIT_STAT3_EO_PL         = 0x8002780E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_BIT_STAT3_EO_PL       = 0x8002780E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_BIT_STAT3_EO_PL       = 0x8002780E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_BIT_STAT3_EO_PL       = 0x8002780E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_BIT_STAT3_EO_PL       = 0x8002780E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_BIT_STAT3_EO_PL        = 0x8002780E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_BIT_STAT3_EO_PL        = 0x8002780E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_BIT_STAT3_EO_PL        = 0x8002780E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_BIT_STAT3_EO_PL       = 0x8002780E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_BIT_STAT4_EO_PL           = 0x8002800E0701103Full;
//DUPS: 8002800E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_BIT_STAT4_EO_PL         = 0x8002800E0701103Full;
//DUPS: 8002800E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_BIT_STAT4_EO_PL         = 0x8002800E0801103Full;
//DUPS: 8002800E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_BIT_STAT4_EO_PL         = 0x8002800E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_BIT_STAT4_EO_PL       = 0x8002800E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_BIT_STAT4_EO_PL       = 0x8002800E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_BIT_STAT4_EO_PL       = 0x8002800E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_BIT_STAT4_EO_PL       = 0x8002800E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_BIT_STAT4_EO_PL        = 0x8002800E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_BIT_STAT4_EO_PL        = 0x8002800E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_BIT_STAT4_EO_PL        = 0x8002800E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_BIT_STAT4_EO_PL       = 0x8002800E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL10_O_PL           = 0x8000C80E0701103Full;
//DUPS: 8000C80E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL10_O_PL         = 0x8000C80E0701103Full;
//DUPS: 8000C80E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL10_O_PL         = 0x8000C80E0801103Full;
//DUPS: 8000C80E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL10_O_PL         = 0x8000C80E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C80E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C80E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C80E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C80E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL10_O_PL        = 0x8000C80E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL10_O_PL        = 0x8000C80E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL10_O_PL        = 0x8000C80E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C80E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL1_EO_PL           = 0x8000080E0701103Full;
//DUPS: 8000080E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL1_EO_PL         = 0x8000080E0701103Full;
//DUPS: 8000080E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL1_EO_PL         = 0x8000080E0801103Full;
//DUPS: 8000080E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL1_EO_PL         = 0x8000080E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x8000080E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x8000080E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x8000080E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x8000080E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL1_EO_PL        = 0x8000080E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL1_EO_PL        = 0x8000080E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL1_EO_PL        = 0x8000080E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x8000080E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL1_O_PL            = 0x8000800E0701103Full;
//DUPS: 8000800E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL1_O_PL          = 0x8000800E0701103Full;
//DUPS: 8000800E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL1_O_PL          = 0x8000800E0801103Full;
//DUPS: 8000800E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL1_O_PL          = 0x8000800E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL1_O_PL        = 0x8000800E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL1_O_PL        = 0x8000800E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL1_O_PL        = 0x8000800E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL1_O_PL        = 0x8000800E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL1_O_PL         = 0x8000800E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL1_O_PL         = 0x8000800E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL1_O_PL         = 0x8000800E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL1_O_PL        = 0x8000800E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL2_EO_PL           = 0x8000100E0701103Full;
//DUPS: 8000100E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL2_EO_PL         = 0x8000100E0701103Full;
//DUPS: 8000100E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL2_EO_PL         = 0x8000100E0801103Full;
//DUPS: 8000100E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL2_EO_PL         = 0x8000100E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x8000100E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x8000100E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x8000100E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x8000100E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL2_EO_PL        = 0x8000100E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL2_EO_PL        = 0x8000100E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL2_EO_PL        = 0x8000100E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x8000100E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL2_O_PL            = 0x8000880E0701103Full;
//DUPS: 8000880E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL2_O_PL          = 0x8000880E0701103Full;
//DUPS: 8000880E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL2_O_PL          = 0x8000880E0801103Full;
//DUPS: 8000880E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL2_O_PL          = 0x8000880E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL2_O_PL        = 0x8000880E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL2_O_PL        = 0x8000880E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL2_O_PL        = 0x8000880E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL2_O_PL        = 0x8000880E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL2_O_PL         = 0x8000880E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL2_O_PL         = 0x8000880E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL2_O_PL         = 0x8000880E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL2_O_PL        = 0x8000880E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL3_EO_PL           = 0x8000180E0701103Full;
//DUPS: 8000180E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL3_EO_PL         = 0x8000180E0701103Full;
//DUPS: 8000180E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL3_EO_PL         = 0x8000180E0801103Full;
//DUPS: 8000180E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL3_EO_PL         = 0x8000180E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x8000180E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x8000180E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x8000180E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x8000180E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL3_EO_PL        = 0x8000180E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL3_EO_PL        = 0x8000180E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL3_EO_PL        = 0x8000180E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x8000180E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL3_O_PL            = 0x8000900E0701103Full;
//DUPS: 8000900E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL3_O_PL          = 0x8000900E0701103Full;
//DUPS: 8000900E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL3_O_PL          = 0x8000900E0801103Full;
//DUPS: 8000900E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL3_O_PL          = 0x8000900E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL3_O_PL        = 0x8000900E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL3_O_PL        = 0x8000900E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL3_O_PL        = 0x8000900E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL3_O_PL        = 0x8000900E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL3_O_PL         = 0x8000900E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL3_O_PL         = 0x8000900E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL3_O_PL         = 0x8000900E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL3_O_PL        = 0x8000900E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL4_EO_PL           = 0x8000200E0701103Full;
//DUPS: 8000200E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL4_EO_PL         = 0x8000200E0701103Full;
//DUPS: 8000200E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL4_EO_PL         = 0x8000200E0801103Full;
//DUPS: 8000200E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL4_EO_PL         = 0x8000200E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x8000200E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x8000200E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x8000200E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x8000200E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL4_EO_PL        = 0x8000200E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL4_EO_PL        = 0x8000200E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL4_EO_PL        = 0x8000200E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x8000200E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL4_O_PL            = 0x8000980E0701103Full;
//DUPS: 8000980E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL4_O_PL          = 0x8000980E0701103Full;
//DUPS: 8000980E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL4_O_PL          = 0x8000980E0801103Full;
//DUPS: 8000980E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL4_O_PL          = 0x8000980E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL4_O_PL        = 0x8000980E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL4_O_PL        = 0x8000980E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL4_O_PL        = 0x8000980E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL4_O_PL        = 0x8000980E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL4_O_PL         = 0x8000980E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL4_O_PL         = 0x8000980E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL4_O_PL         = 0x8000980E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL4_O_PL        = 0x8000980E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL5_EO_PL           = 0x8000280E0701103Full;
//DUPS: 8000280E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL5_EO_PL         = 0x8000280E0701103Full;
//DUPS: 8000280E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL5_EO_PL         = 0x8000280E0801103Full;
//DUPS: 8000280E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL5_EO_PL         = 0x8000280E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x8000280E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x8000280E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x8000280E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x8000280E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL5_EO_PL        = 0x8000280E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL5_EO_PL        = 0x8000280E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL5_EO_PL        = 0x8000280E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x8000280E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL5_O_PL            = 0x8000A00E0701103Full;
//DUPS: 8000A00E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL5_O_PL          = 0x8000A00E0701103Full;
//DUPS: 8000A00E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL5_O_PL          = 0x8000A00E0801103Full;
//DUPS: 8000A00E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL5_O_PL          = 0x8000A00E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A00E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A00E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A00E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A00E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL5_O_PL         = 0x8000A00E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL5_O_PL         = 0x8000A00E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL5_O_PL         = 0x8000A00E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A00E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL6_EO_PL           = 0x8000300E0701103Full;
//DUPS: 8000300E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL6_EO_PL         = 0x8000300E0701103Full;
//DUPS: 8000300E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL6_EO_PL         = 0x8000300E0801103Full;
//DUPS: 8000300E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL6_EO_PL         = 0x8000300E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x8000300E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x8000300E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x8000300E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x8000300E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL6_EO_PL        = 0x8000300E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL6_EO_PL        = 0x8000300E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL6_EO_PL        = 0x8000300E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x8000300E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL6_O_PL            = 0x8000A80E0701103Full;
//DUPS: 8000A80E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL6_O_PL          = 0x8000A80E0701103Full;
//DUPS: 8000A80E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL6_O_PL          = 0x8000A80E0801103Full;
//DUPS: 8000A80E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL6_O_PL          = 0x8000A80E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A80E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A80E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A80E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A80E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL6_O_PL         = 0x8000A80E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL6_O_PL         = 0x8000A80E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL6_O_PL         = 0x8000A80E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A80E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL7_EO_PL           = 0x8000380E0701103Full;
//DUPS: 8000380E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL7_EO_PL         = 0x8000380E0701103Full;
//DUPS: 8000380E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL7_EO_PL         = 0x8000380E0801103Full;
//DUPS: 8000380E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL7_EO_PL         = 0x8000380E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x8000380E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x8000380E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x8000380E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x8000380E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL7_EO_PL        = 0x8000380E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL7_EO_PL        = 0x8000380E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL7_EO_PL        = 0x8000380E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x8000380E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL7_O_PL            = 0x8000B00E0701103Full;
//DUPS: 8000B00E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL7_O_PL          = 0x8000B00E0701103Full;
//DUPS: 8000B00E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL7_O_PL          = 0x8000B00E0801103Full;
//DUPS: 8000B00E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL7_O_PL          = 0x8000B00E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B00E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B00E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B00E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B00E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL7_O_PL         = 0x8000B00E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL7_O_PL         = 0x8000B00E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL7_O_PL         = 0x8000B00E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B00E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL8_EO_PL           = 0x8000400E0701103Full;
//DUPS: 8000400E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL8_EO_PL         = 0x8000400E0701103Full;
//DUPS: 8000400E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL8_EO_PL         = 0x8000400E0801103Full;
//DUPS: 8000400E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL8_EO_PL         = 0x8000400E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x8000400E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x8000400E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x8000400E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x8000400E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL8_EO_PL        = 0x8000400E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL8_EO_PL        = 0x8000400E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL8_EO_PL        = 0x8000400E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x8000400E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL8_O_PL            = 0x8000B80E0701103Full;
//DUPS: 8000B80E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL8_O_PL          = 0x8000B80E0701103Full;
//DUPS: 8000B80E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL8_O_PL          = 0x8000B80E0801103Full;
//DUPS: 8000B80E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL8_O_PL          = 0x8000B80E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B80E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B80E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B80E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B80E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL8_O_PL         = 0x8000B80E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL8_O_PL         = 0x8000B80E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL8_O_PL         = 0x8000B80E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B80E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL9_O_PL            = 0x8000C00E0701103Full;
//DUPS: 8000C00E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL9_O_PL          = 0x8000C00E0701103Full;
//DUPS: 8000C00E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL9_O_PL          = 0x8000C00E0801103Full;
//DUPS: 8000C00E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL9_O_PL          = 0x8000C00E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C00E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C00E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C00E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C00E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL9_O_PL         = 0x8000C00E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL9_O_PL         = 0x8000C00E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL9_O_PL         = 0x8000C00E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C00E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_DATA_DAC_SPARE_MODE_PL    = 0x8000000E0701103Full;
//DUPS: 8000000E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_DATA_DAC_SPARE_MODE_PL  = 0x8000000E0701103Full;
//DUPS: 8000000E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_DATA_DAC_SPARE_MODE_PL  = 0x8000000E0801103Full;
//DUPS: 8000000E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_DATA_DAC_SPARE_MODE_PL  = 0x8000000E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_FIR_ERROR_INJECT_PL       = 0x8002180E0701103Full;
//DUPS: 8002180E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_FIR_ERROR_INJECT_PL     = 0x8002180E0701103Full;
//DUPS: 8002180E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_FIR_ERROR_INJECT_PL     = 0x8002180E0801103Full;
//DUPS: 8002180E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_FIR_ERROR_INJECT_PL     = 0x8002180E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x8002180E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x8002180E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x8002180E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x8002180E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_FIR_ERROR_INJECT_PL    = 0x8002180E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_FIR_ERROR_INJECT_PL    = 0x8002180E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_FIR_ERROR_INJECT_PL    = 0x8002180E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x8002180E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_FIR_MASK_PL               = 0x8002100E0701103Full;
//DUPS: 8002100E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_FIR_MASK_PL             = 0x8002100E0701103Full;
//DUPS: 8002100E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_FIR_MASK_PL             = 0x8002100E0801103Full;
//DUPS: 8002100E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_FIR_MASK_PL             = 0x8002100E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_FIR_MASK_PL           = 0x8002100E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_FIR_MASK_PL           = 0x8002100E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_FIR_MASK_PL           = 0x8002100E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_FIR_MASK_PL           = 0x8002100E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_FIR_MASK_PL            = 0x8002100E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_FIR_MASK_PL            = 0x8002100E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_FIR_MASK_PL            = 0x8002100E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_FIR_MASK_PL           = 0x8002100E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_FIR_PL                    = 0x8002080E0701103Full;
//DUPS: 8002080E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_FIR_PL                  = 0x8002080E0701103Full;
//DUPS: 8002080E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_FIR_PL                  = 0x8002080E0801103Full;
//DUPS: 8002080E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_FIR_PL                  = 0x8002080E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_FIR_PL                = 0x8002080E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_FIR_PL                = 0x8002080E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_FIR_PL                = 0x8002080E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_FIR_PL                = 0x8002080E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_FIR_PL                 = 0x8002080E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_FIR_PL                 = 0x8002080E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_FIR_PL                 = 0x8002080E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_FIR_PL                = 0x8002080E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE2_RX_SPARE_MODE_PL             = 0x8002000E0701103Full;
//DUPS: 8002000E0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE2_RX_SPARE_MODE_PL           = 0x8002000E0701103Full;
//DUPS: 8002000E0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE2_RX_SPARE_MODE_PL           = 0x8002000E0801103Full;
//DUPS: 8002000E0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE2_RX_SPARE_MODE_PL           = 0x8002000E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE2_RX_SPARE_MODE_PL         = 0x8002000E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE2_RX_SPARE_MODE_PL         = 0x8002000E0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE2_RX_SPARE_MODE_PL         = 0x8002000E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE2_RX_SPARE_MODE_PL         = 0x8002000E0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE2_RX_SPARE_MODE_PL          = 0x8002000E0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE2_RX_SPARE_MODE_PL          = 0x8002000E0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE2_RX_SPARE_MODE_PL          = 0x8002000E0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE2_RX_SPARE_MODE_PL         = 0x8002000E0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL2_EO_PL           = 0x8002480F0701103Full;
//DUPS: 8002480F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL2_EO_PL         = 0x8002480F0701103Full;
//DUPS: 8002480F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL2_EO_PL         = 0x8002480F0801103Full;
//DUPS: 8002480F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL2_EO_PL         = 0x8002480F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x8002480F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x8002480F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x8002480F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x8002480F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL2_EO_PL        = 0x8002480F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL2_EO_PL        = 0x8002480F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL2_EO_PL        = 0x8002480F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x8002480F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL3_EO_PL           = 0x8002500F0701103Full;
//DUPS: 8002500F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL3_EO_PL         = 0x8002500F0701103Full;
//DUPS: 8002500F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL3_EO_PL         = 0x8002500F0801103Full;
//DUPS: 8002500F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL3_EO_PL         = 0x8002500F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x8002500F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x8002500F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x8002500F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x8002500F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL3_EO_PL        = 0x8002500F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL3_EO_PL        = 0x8002500F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL3_EO_PL        = 0x8002500F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x8002500F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL4_EO_PL           = 0x8002580F0701103Full;
//DUPS: 8002580F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL4_EO_PL         = 0x8002580F0701103Full;
//DUPS: 8002580F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL4_EO_PL         = 0x8002580F0801103Full;
//DUPS: 8002580F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL4_EO_PL         = 0x8002580F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x8002580F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x8002580F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x8002580F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x8002580F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL4_EO_PL        = 0x8002580F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL4_EO_PL        = 0x8002580F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL4_EO_PL        = 0x8002580F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x8002580F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL5_EO_PL           = 0x8002600F0701103Full;
//DUPS: 8002600F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL5_EO_PL         = 0x8002600F0701103Full;
//DUPS: 8002600F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL5_EO_PL         = 0x8002600F0801103Full;
//DUPS: 8002600F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL5_EO_PL         = 0x8002600F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x8002600F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x8002600F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x8002600F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x8002600F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL5_EO_PL        = 0x8002600F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL5_EO_PL        = 0x8002600F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL5_EO_PL        = 0x8002600F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x8002600F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_BIT_CNTLX1_EO_PL          = 0x8002400F0701103Full;
//DUPS: 8002400F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_BIT_CNTLX1_EO_PL        = 0x8002400F0701103Full;
//DUPS: 8002400F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_BIT_CNTLX1_EO_PL        = 0x8002400F0801103Full;
//DUPS: 8002400F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_BIT_CNTLX1_EO_PL        = 0x8002400F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x8002400F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x8002400F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x8002400F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x8002400F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_BIT_CNTLX1_EO_PL       = 0x8002400F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_BIT_CNTLX1_EO_PL       = 0x8002400F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_BIT_CNTLX1_EO_PL       = 0x8002400F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x8002400F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_BIT_MODE1_EO_PL           = 0x8002200F0701103Full;
//DUPS: 8002200F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_BIT_MODE1_EO_PL         = 0x8002200F0701103Full;
//DUPS: 8002200F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_BIT_MODE1_EO_PL         = 0x8002200F0801103Full;
//DUPS: 8002200F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_BIT_MODE1_EO_PL         = 0x8002200F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_BIT_MODE1_EO_PL       = 0x8002200F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_BIT_MODE1_EO_PL       = 0x8002200F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_BIT_MODE1_EO_PL       = 0x8002200F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_BIT_MODE1_EO_PL       = 0x8002200F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_BIT_MODE1_EO_PL        = 0x8002200F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_BIT_MODE1_EO_PL        = 0x8002200F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_BIT_MODE1_EO_PL        = 0x8002200F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_BIT_MODE1_EO_PL       = 0x8002200F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_BIT_MODE2_EO_PL           = 0x8002280F0701103Full;
//DUPS: 8002280F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_BIT_MODE2_EO_PL         = 0x8002280F0701103Full;
//DUPS: 8002280F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_BIT_MODE2_EO_PL         = 0x8002280F0801103Full;
//DUPS: 8002280F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_BIT_MODE2_EO_PL         = 0x8002280F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_BIT_MODE2_EO_PL       = 0x8002280F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_BIT_MODE2_EO_PL       = 0x8002280F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_BIT_MODE2_EO_PL       = 0x8002280F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_BIT_MODE2_EO_PL       = 0x8002280F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_BIT_MODE2_EO_PL        = 0x8002280F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_BIT_MODE2_EO_PL        = 0x8002280F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_BIT_MODE2_EO_PL        = 0x8002280F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_BIT_MODE2_EO_PL       = 0x8002280F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_BIT_MODE3_EO_PL           = 0x8002300F0701103Full;
//DUPS: 8002300F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_BIT_MODE3_EO_PL         = 0x8002300F0701103Full;
//DUPS: 8002300F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_BIT_MODE3_EO_PL         = 0x8002300F0801103Full;
//DUPS: 8002300F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_BIT_MODE3_EO_PL         = 0x8002300F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_BIT_MODE3_EO_PL       = 0x8002300F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_BIT_MODE3_EO_PL       = 0x8002300F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_BIT_MODE3_EO_PL       = 0x8002300F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_BIT_MODE3_EO_PL       = 0x8002300F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_BIT_MODE3_EO_PL        = 0x8002300F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_BIT_MODE3_EO_PL        = 0x8002300F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_BIT_MODE3_EO_PL        = 0x8002300F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_BIT_MODE3_EO_PL       = 0x8002300F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_BIT_MODE4_EO_PL           = 0x8002380F0701103Full;
//DUPS: 8002380F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_BIT_MODE4_EO_PL         = 0x8002380F0701103Full;
//DUPS: 8002380F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_BIT_MODE4_EO_PL         = 0x8002380F0801103Full;
//DUPS: 8002380F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_BIT_MODE4_EO_PL         = 0x8002380F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_BIT_MODE4_EO_PL       = 0x8002380F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_BIT_MODE4_EO_PL       = 0x8002380F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_BIT_MODE4_EO_PL       = 0x8002380F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_BIT_MODE4_EO_PL       = 0x8002380F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_BIT_MODE4_EO_PL        = 0x8002380F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_BIT_MODE4_EO_PL        = 0x8002380F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_BIT_MODE4_EO_PL        = 0x8002380F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_BIT_MODE4_EO_PL       = 0x8002380F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_BIT_STAT1_EO_PL           = 0x8002680F0701103Full;
//DUPS: 8002680F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_BIT_STAT1_EO_PL         = 0x8002680F0701103Full;
//DUPS: 8002680F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_BIT_STAT1_EO_PL         = 0x8002680F0801103Full;
//DUPS: 8002680F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_BIT_STAT1_EO_PL         = 0x8002680F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_BIT_STAT1_EO_PL       = 0x8002680F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_BIT_STAT1_EO_PL       = 0x8002680F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_BIT_STAT1_EO_PL       = 0x8002680F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_BIT_STAT1_EO_PL       = 0x8002680F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_BIT_STAT1_EO_PL        = 0x8002680F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_BIT_STAT1_EO_PL        = 0x8002680F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_BIT_STAT1_EO_PL        = 0x8002680F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_BIT_STAT1_EO_PL       = 0x8002680F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_BIT_STAT1_O_PL            = 0x8003180F0701103Full;
//DUPS: 8003180F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_BIT_STAT1_O_PL          = 0x8003180F0701103Full;
//DUPS: 8003180F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_BIT_STAT1_O_PL          = 0x8003180F0801103Full;
//DUPS: 8003180F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_BIT_STAT1_O_PL          = 0x8003180F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_BIT_STAT1_O_PL        = 0x8003180F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_BIT_STAT1_O_PL        = 0x8003180F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_BIT_STAT1_O_PL        = 0x8003180F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_BIT_STAT1_O_PL        = 0x8003180F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_BIT_STAT1_O_PL         = 0x8003180F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_BIT_STAT1_O_PL         = 0x8003180F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_BIT_STAT1_O_PL         = 0x8003180F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_BIT_STAT1_O_PL        = 0x8003180F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_BIT_STAT2_EO_PL           = 0x8002700F0701103Full;
//DUPS: 8002700F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_BIT_STAT2_EO_PL         = 0x8002700F0701103Full;
//DUPS: 8002700F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_BIT_STAT2_EO_PL         = 0x8002700F0801103Full;
//DUPS: 8002700F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_BIT_STAT2_EO_PL         = 0x8002700F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_BIT_STAT2_EO_PL       = 0x8002700F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_BIT_STAT2_EO_PL       = 0x8002700F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_BIT_STAT2_EO_PL       = 0x8002700F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_BIT_STAT2_EO_PL       = 0x8002700F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_BIT_STAT2_EO_PL        = 0x8002700F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_BIT_STAT2_EO_PL        = 0x8002700F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_BIT_STAT2_EO_PL        = 0x8002700F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_BIT_STAT2_EO_PL       = 0x8002700F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_BIT_STAT3_EO_PL           = 0x8002780F0701103Full;
//DUPS: 8002780F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_BIT_STAT3_EO_PL         = 0x8002780F0701103Full;
//DUPS: 8002780F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_BIT_STAT3_EO_PL         = 0x8002780F0801103Full;
//DUPS: 8002780F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_BIT_STAT3_EO_PL         = 0x8002780F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_BIT_STAT3_EO_PL       = 0x8002780F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_BIT_STAT3_EO_PL       = 0x8002780F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_BIT_STAT3_EO_PL       = 0x8002780F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_BIT_STAT3_EO_PL       = 0x8002780F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_BIT_STAT3_EO_PL        = 0x8002780F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_BIT_STAT3_EO_PL        = 0x8002780F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_BIT_STAT3_EO_PL        = 0x8002780F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_BIT_STAT3_EO_PL       = 0x8002780F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_BIT_STAT4_EO_PL           = 0x8002800F0701103Full;
//DUPS: 8002800F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_BIT_STAT4_EO_PL         = 0x8002800F0701103Full;
//DUPS: 8002800F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_BIT_STAT4_EO_PL         = 0x8002800F0801103Full;
//DUPS: 8002800F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_BIT_STAT4_EO_PL         = 0x8002800F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_BIT_STAT4_EO_PL       = 0x8002800F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_BIT_STAT4_EO_PL       = 0x8002800F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_BIT_STAT4_EO_PL       = 0x8002800F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_BIT_STAT4_EO_PL       = 0x8002800F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_BIT_STAT4_EO_PL        = 0x8002800F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_BIT_STAT4_EO_PL        = 0x8002800F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_BIT_STAT4_EO_PL        = 0x8002800F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_BIT_STAT4_EO_PL       = 0x8002800F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL10_O_PL           = 0x8000C80F0701103Full;
//DUPS: 8000C80F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL10_O_PL         = 0x8000C80F0701103Full;
//DUPS: 8000C80F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL10_O_PL         = 0x8000C80F0801103Full;
//DUPS: 8000C80F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL10_O_PL         = 0x8000C80F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C80F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C80F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C80F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C80F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL10_O_PL        = 0x8000C80F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL10_O_PL        = 0x8000C80F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL10_O_PL        = 0x8000C80F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C80F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL1_EO_PL           = 0x8000080F0701103Full;
//DUPS: 8000080F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL1_EO_PL         = 0x8000080F0701103Full;
//DUPS: 8000080F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL1_EO_PL         = 0x8000080F0801103Full;
//DUPS: 8000080F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL1_EO_PL         = 0x8000080F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x8000080F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x8000080F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x8000080F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x8000080F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL1_EO_PL        = 0x8000080F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL1_EO_PL        = 0x8000080F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL1_EO_PL        = 0x8000080F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x8000080F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL1_O_PL            = 0x8000800F0701103Full;
//DUPS: 8000800F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL1_O_PL          = 0x8000800F0701103Full;
//DUPS: 8000800F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL1_O_PL          = 0x8000800F0801103Full;
//DUPS: 8000800F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL1_O_PL          = 0x8000800F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL1_O_PL        = 0x8000800F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL1_O_PL        = 0x8000800F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL1_O_PL        = 0x8000800F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL1_O_PL        = 0x8000800F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL1_O_PL         = 0x8000800F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL1_O_PL         = 0x8000800F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL1_O_PL         = 0x8000800F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL1_O_PL        = 0x8000800F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL2_EO_PL           = 0x8000100F0701103Full;
//DUPS: 8000100F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL2_EO_PL         = 0x8000100F0701103Full;
//DUPS: 8000100F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL2_EO_PL         = 0x8000100F0801103Full;
//DUPS: 8000100F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL2_EO_PL         = 0x8000100F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x8000100F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x8000100F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x8000100F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x8000100F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL2_EO_PL        = 0x8000100F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL2_EO_PL        = 0x8000100F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL2_EO_PL        = 0x8000100F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x8000100F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL2_O_PL            = 0x8000880F0701103Full;
//DUPS: 8000880F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL2_O_PL          = 0x8000880F0701103Full;
//DUPS: 8000880F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL2_O_PL          = 0x8000880F0801103Full;
//DUPS: 8000880F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL2_O_PL          = 0x8000880F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL2_O_PL        = 0x8000880F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL2_O_PL        = 0x8000880F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL2_O_PL        = 0x8000880F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL2_O_PL        = 0x8000880F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL2_O_PL         = 0x8000880F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL2_O_PL         = 0x8000880F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL2_O_PL         = 0x8000880F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL2_O_PL        = 0x8000880F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL3_EO_PL           = 0x8000180F0701103Full;
//DUPS: 8000180F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL3_EO_PL         = 0x8000180F0701103Full;
//DUPS: 8000180F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL3_EO_PL         = 0x8000180F0801103Full;
//DUPS: 8000180F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL3_EO_PL         = 0x8000180F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x8000180F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x8000180F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x8000180F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x8000180F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL3_EO_PL        = 0x8000180F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL3_EO_PL        = 0x8000180F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL3_EO_PL        = 0x8000180F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x8000180F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL3_O_PL            = 0x8000900F0701103Full;
//DUPS: 8000900F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL3_O_PL          = 0x8000900F0701103Full;
//DUPS: 8000900F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL3_O_PL          = 0x8000900F0801103Full;
//DUPS: 8000900F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL3_O_PL          = 0x8000900F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL3_O_PL        = 0x8000900F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL3_O_PL        = 0x8000900F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL3_O_PL        = 0x8000900F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL3_O_PL        = 0x8000900F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL3_O_PL         = 0x8000900F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL3_O_PL         = 0x8000900F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL3_O_PL         = 0x8000900F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL3_O_PL        = 0x8000900F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL4_EO_PL           = 0x8000200F0701103Full;
//DUPS: 8000200F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL4_EO_PL         = 0x8000200F0701103Full;
//DUPS: 8000200F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL4_EO_PL         = 0x8000200F0801103Full;
//DUPS: 8000200F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL4_EO_PL         = 0x8000200F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x8000200F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x8000200F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x8000200F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x8000200F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL4_EO_PL        = 0x8000200F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL4_EO_PL        = 0x8000200F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL4_EO_PL        = 0x8000200F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x8000200F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL4_O_PL            = 0x8000980F0701103Full;
//DUPS: 8000980F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL4_O_PL          = 0x8000980F0701103Full;
//DUPS: 8000980F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL4_O_PL          = 0x8000980F0801103Full;
//DUPS: 8000980F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL4_O_PL          = 0x8000980F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL4_O_PL        = 0x8000980F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL4_O_PL        = 0x8000980F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL4_O_PL        = 0x8000980F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL4_O_PL        = 0x8000980F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL4_O_PL         = 0x8000980F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL4_O_PL         = 0x8000980F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL4_O_PL         = 0x8000980F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL4_O_PL        = 0x8000980F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL5_EO_PL           = 0x8000280F0701103Full;
//DUPS: 8000280F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL5_EO_PL         = 0x8000280F0701103Full;
//DUPS: 8000280F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL5_EO_PL         = 0x8000280F0801103Full;
//DUPS: 8000280F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL5_EO_PL         = 0x8000280F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x8000280F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x8000280F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x8000280F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x8000280F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL5_EO_PL        = 0x8000280F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL5_EO_PL        = 0x8000280F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL5_EO_PL        = 0x8000280F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x8000280F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL5_O_PL            = 0x8000A00F0701103Full;
//DUPS: 8000A00F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL5_O_PL          = 0x8000A00F0701103Full;
//DUPS: 8000A00F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL5_O_PL          = 0x8000A00F0801103Full;
//DUPS: 8000A00F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL5_O_PL          = 0x8000A00F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A00F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A00F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A00F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A00F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL5_O_PL         = 0x8000A00F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL5_O_PL         = 0x8000A00F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL5_O_PL         = 0x8000A00F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A00F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL6_EO_PL           = 0x8000300F0701103Full;
//DUPS: 8000300F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL6_EO_PL         = 0x8000300F0701103Full;
//DUPS: 8000300F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL6_EO_PL         = 0x8000300F0801103Full;
//DUPS: 8000300F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL6_EO_PL         = 0x8000300F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x8000300F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x8000300F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x8000300F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x8000300F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL6_EO_PL        = 0x8000300F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL6_EO_PL        = 0x8000300F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL6_EO_PL        = 0x8000300F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x8000300F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL6_O_PL            = 0x8000A80F0701103Full;
//DUPS: 8000A80F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL6_O_PL          = 0x8000A80F0701103Full;
//DUPS: 8000A80F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL6_O_PL          = 0x8000A80F0801103Full;
//DUPS: 8000A80F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL6_O_PL          = 0x8000A80F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A80F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A80F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A80F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A80F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL6_O_PL         = 0x8000A80F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL6_O_PL         = 0x8000A80F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL6_O_PL         = 0x8000A80F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A80F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL7_EO_PL           = 0x8000380F0701103Full;
//DUPS: 8000380F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL7_EO_PL         = 0x8000380F0701103Full;
//DUPS: 8000380F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL7_EO_PL         = 0x8000380F0801103Full;
//DUPS: 8000380F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL7_EO_PL         = 0x8000380F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x8000380F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x8000380F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x8000380F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x8000380F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL7_EO_PL        = 0x8000380F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL7_EO_PL        = 0x8000380F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL7_EO_PL        = 0x8000380F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x8000380F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL7_O_PL            = 0x8000B00F0701103Full;
//DUPS: 8000B00F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL7_O_PL          = 0x8000B00F0701103Full;
//DUPS: 8000B00F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL7_O_PL          = 0x8000B00F0801103Full;
//DUPS: 8000B00F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL7_O_PL          = 0x8000B00F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B00F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B00F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B00F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B00F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL7_O_PL         = 0x8000B00F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL7_O_PL         = 0x8000B00F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL7_O_PL         = 0x8000B00F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B00F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL8_EO_PL           = 0x8000400F0701103Full;
//DUPS: 8000400F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL8_EO_PL         = 0x8000400F0701103Full;
//DUPS: 8000400F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL8_EO_PL         = 0x8000400F0801103Full;
//DUPS: 8000400F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL8_EO_PL         = 0x8000400F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x8000400F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x8000400F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x8000400F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x8000400F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL8_EO_PL        = 0x8000400F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL8_EO_PL        = 0x8000400F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL8_EO_PL        = 0x8000400F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x8000400F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL8_O_PL            = 0x8000B80F0701103Full;
//DUPS: 8000B80F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL8_O_PL          = 0x8000B80F0701103Full;
//DUPS: 8000B80F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL8_O_PL          = 0x8000B80F0801103Full;
//DUPS: 8000B80F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL8_O_PL          = 0x8000B80F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B80F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B80F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B80F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B80F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL8_O_PL         = 0x8000B80F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL8_O_PL         = 0x8000B80F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL8_O_PL         = 0x8000B80F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B80F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL9_O_PL            = 0x8000C00F0701103Full;
//DUPS: 8000C00F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL9_O_PL          = 0x8000C00F0701103Full;
//DUPS: 8000C00F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL9_O_PL          = 0x8000C00F0801103Full;
//DUPS: 8000C00F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL9_O_PL          = 0x8000C00F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C00F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C00F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C00F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C00F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL9_O_PL         = 0x8000C00F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL9_O_PL         = 0x8000C00F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL9_O_PL         = 0x8000C00F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C00F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_DATA_DAC_SPARE_MODE_PL    = 0x8000000F0701103Full;
//DUPS: 8000000F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_DATA_DAC_SPARE_MODE_PL  = 0x8000000F0701103Full;
//DUPS: 8000000F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_DATA_DAC_SPARE_MODE_PL  = 0x8000000F0801103Full;
//DUPS: 8000000F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_DATA_DAC_SPARE_MODE_PL  = 0x8000000F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x8000000F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_FIR_ERROR_INJECT_PL       = 0x8002180F0701103Full;
//DUPS: 8002180F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_FIR_ERROR_INJECT_PL     = 0x8002180F0701103Full;
//DUPS: 8002180F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_FIR_ERROR_INJECT_PL     = 0x8002180F0801103Full;
//DUPS: 8002180F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_FIR_ERROR_INJECT_PL     = 0x8002180F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x8002180F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x8002180F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x8002180F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x8002180F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_FIR_ERROR_INJECT_PL    = 0x8002180F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_FIR_ERROR_INJECT_PL    = 0x8002180F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_FIR_ERROR_INJECT_PL    = 0x8002180F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x8002180F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_FIR_MASK_PL               = 0x8002100F0701103Full;
//DUPS: 8002100F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_FIR_MASK_PL             = 0x8002100F0701103Full;
//DUPS: 8002100F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_FIR_MASK_PL             = 0x8002100F0801103Full;
//DUPS: 8002100F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_FIR_MASK_PL             = 0x8002100F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_FIR_MASK_PL           = 0x8002100F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_FIR_MASK_PL           = 0x8002100F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_FIR_MASK_PL           = 0x8002100F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_FIR_MASK_PL           = 0x8002100F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_FIR_MASK_PL            = 0x8002100F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_FIR_MASK_PL            = 0x8002100F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_FIR_MASK_PL            = 0x8002100F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_FIR_MASK_PL           = 0x8002100F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_FIR_PL                    = 0x8002080F0701103Full;
//DUPS: 8002080F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_FIR_PL                  = 0x8002080F0701103Full;
//DUPS: 8002080F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_FIR_PL                  = 0x8002080F0801103Full;
//DUPS: 8002080F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_FIR_PL                  = 0x8002080F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_FIR_PL                = 0x8002080F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_FIR_PL                = 0x8002080F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_FIR_PL                = 0x8002080F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_FIR_PL                = 0x8002080F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_FIR_PL                 = 0x8002080F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_FIR_PL                 = 0x8002080F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_FIR_PL                 = 0x8002080F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_FIR_PL                = 0x8002080F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS3_SLICE3_RX_SPARE_MODE_PL             = 0x8002000F0701103Full;
//DUPS: 8002000F0701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS3_SLICE3_RX_SPARE_MODE_PL           = 0x8002000F0701103Full;
//DUPS: 8002000F0701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS3_SLICE3_RX_SPARE_MODE_PL           = 0x8002000F0801103Full;
//DUPS: 8002000F0801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS3_SLICE3_RX_SPARE_MODE_PL           = 0x8002000F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS3_SLICE3_RX_SPARE_MODE_PL         = 0x8002000F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS3_SLICE3_RX_SPARE_MODE_PL         = 0x8002000F0701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS3_SLICE3_RX_SPARE_MODE_PL         = 0x8002000F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS3_SLICE3_RX_SPARE_MODE_PL         = 0x8002000F0801143Full;

static const uint64_t P9A_OMI_1_RX0_RXPACKS3_SLICE3_RX_SPARE_MODE_PL          = 0x8002000F0701103Full;

static const uint64_t P9A_OMI_4_RX0_RXPACKS3_SLICE3_RX_SPARE_MODE_PL          = 0x8002000F0701143Full;

static const uint64_t P9A_OMI_9_RX0_RXPACKS3_SLICE3_RX_SPARE_MODE_PL          = 0x8002000F0801103Full;

static const uint64_t P9A_OMI_12_RX0_RXPACKS3_SLICE3_RX_SPARE_MODE_PL         = 0x8002000F0801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL2_EO_PL           = 0x800248100701103Full;
//DUPS: 800248100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL2_EO_PL         = 0x800248100701103Full;
//DUPS: 800248100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL2_EO_PL         = 0x800248100801103Full;
//DUPS: 800248100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL2_EO_PL         = 0x800248100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x800248100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x800248100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x800248100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x800248100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL2_EO_PL        = 0x800248100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL2_EO_PL        = 0x800248100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x800248100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x800248100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL3_EO_PL           = 0x800250100701103Full;
//DUPS: 800250100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL3_EO_PL         = 0x800250100701103Full;
//DUPS: 800250100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL3_EO_PL         = 0x800250100801103Full;
//DUPS: 800250100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL3_EO_PL         = 0x800250100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x800250100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x800250100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x800250100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x800250100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL3_EO_PL        = 0x800250100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL3_EO_PL        = 0x800250100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x800250100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x800250100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL4_EO_PL           = 0x800258100701103Full;
//DUPS: 800258100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL4_EO_PL         = 0x800258100701103Full;
//DUPS: 800258100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL4_EO_PL         = 0x800258100801103Full;
//DUPS: 800258100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL4_EO_PL         = 0x800258100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x800258100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x800258100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x800258100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x800258100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL4_EO_PL        = 0x800258100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL4_EO_PL        = 0x800258100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x800258100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x800258100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL5_EO_PL           = 0x800260100701103Full;
//DUPS: 800260100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL5_EO_PL         = 0x800260100701103Full;
//DUPS: 800260100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL5_EO_PL         = 0x800260100801103Full;
//DUPS: 800260100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL5_EO_PL         = 0x800260100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x800260100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x800260100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x800260100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x800260100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL5_EO_PL        = 0x800260100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL5_EO_PL        = 0x800260100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x800260100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x800260100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_BIT_CNTLX1_EO_PL          = 0x800240100701103Full;
//DUPS: 800240100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_BIT_CNTLX1_EO_PL        = 0x800240100701103Full;
//DUPS: 800240100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_BIT_CNTLX1_EO_PL        = 0x800240100801103Full;
//DUPS: 800240100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_BIT_CNTLX1_EO_PL        = 0x800240100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x800240100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x800240100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x800240100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x800240100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_BIT_CNTLX1_EO_PL       = 0x800240100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_BIT_CNTLX1_EO_PL       = 0x800240100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x800240100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x800240100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_BIT_MODE1_EO_PL           = 0x800220100701103Full;
//DUPS: 800220100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_BIT_MODE1_EO_PL         = 0x800220100701103Full;
//DUPS: 800220100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_BIT_MODE1_EO_PL         = 0x800220100801103Full;
//DUPS: 800220100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_BIT_MODE1_EO_PL         = 0x800220100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_BIT_MODE1_EO_PL       = 0x800220100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_BIT_MODE1_EO_PL       = 0x800220100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_BIT_MODE1_EO_PL       = 0x800220100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_BIT_MODE1_EO_PL       = 0x800220100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_BIT_MODE1_EO_PL        = 0x800220100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_BIT_MODE1_EO_PL        = 0x800220100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_BIT_MODE1_EO_PL       = 0x800220100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_BIT_MODE1_EO_PL       = 0x800220100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_BIT_MODE2_EO_PL           = 0x800228100701103Full;
//DUPS: 800228100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_BIT_MODE2_EO_PL         = 0x800228100701103Full;
//DUPS: 800228100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_BIT_MODE2_EO_PL         = 0x800228100801103Full;
//DUPS: 800228100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_BIT_MODE2_EO_PL         = 0x800228100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_BIT_MODE2_EO_PL       = 0x800228100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_BIT_MODE2_EO_PL       = 0x800228100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_BIT_MODE2_EO_PL       = 0x800228100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_BIT_MODE2_EO_PL       = 0x800228100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_BIT_MODE2_EO_PL        = 0x800228100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_BIT_MODE2_EO_PL        = 0x800228100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_BIT_MODE2_EO_PL       = 0x800228100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_BIT_MODE2_EO_PL       = 0x800228100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_BIT_MODE3_EO_PL           = 0x800230100701103Full;
//DUPS: 800230100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_BIT_MODE3_EO_PL         = 0x800230100701103Full;
//DUPS: 800230100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_BIT_MODE3_EO_PL         = 0x800230100801103Full;
//DUPS: 800230100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_BIT_MODE3_EO_PL         = 0x800230100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_BIT_MODE3_EO_PL       = 0x800230100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_BIT_MODE3_EO_PL       = 0x800230100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_BIT_MODE3_EO_PL       = 0x800230100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_BIT_MODE3_EO_PL       = 0x800230100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_BIT_MODE3_EO_PL        = 0x800230100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_BIT_MODE3_EO_PL        = 0x800230100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_BIT_MODE3_EO_PL       = 0x800230100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_BIT_MODE3_EO_PL       = 0x800230100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_BIT_MODE4_EO_PL           = 0x800238100701103Full;
//DUPS: 800238100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_BIT_MODE4_EO_PL         = 0x800238100701103Full;
//DUPS: 800238100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_BIT_MODE4_EO_PL         = 0x800238100801103Full;
//DUPS: 800238100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_BIT_MODE4_EO_PL         = 0x800238100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_BIT_MODE4_EO_PL       = 0x800238100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_BIT_MODE4_EO_PL       = 0x800238100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_BIT_MODE4_EO_PL       = 0x800238100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_BIT_MODE4_EO_PL       = 0x800238100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_BIT_MODE4_EO_PL        = 0x800238100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_BIT_MODE4_EO_PL        = 0x800238100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_BIT_MODE4_EO_PL       = 0x800238100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_BIT_MODE4_EO_PL       = 0x800238100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_BIT_STAT1_EO_PL           = 0x800268100701103Full;
//DUPS: 800268100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_BIT_STAT1_EO_PL         = 0x800268100701103Full;
//DUPS: 800268100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_BIT_STAT1_EO_PL         = 0x800268100801103Full;
//DUPS: 800268100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_BIT_STAT1_EO_PL         = 0x800268100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_BIT_STAT1_EO_PL       = 0x800268100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_BIT_STAT1_EO_PL       = 0x800268100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_BIT_STAT1_EO_PL       = 0x800268100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_BIT_STAT1_EO_PL       = 0x800268100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_BIT_STAT1_EO_PL        = 0x800268100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_BIT_STAT1_EO_PL        = 0x800268100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_BIT_STAT1_EO_PL       = 0x800268100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_BIT_STAT1_EO_PL       = 0x800268100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_BIT_STAT1_O_PL            = 0x800318100701103Full;
//DUPS: 800318100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_BIT_STAT1_O_PL          = 0x800318100701103Full;
//DUPS: 800318100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_BIT_STAT1_O_PL          = 0x800318100801103Full;
//DUPS: 800318100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_BIT_STAT1_O_PL          = 0x800318100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_BIT_STAT1_O_PL        = 0x800318100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_BIT_STAT1_O_PL        = 0x800318100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_BIT_STAT1_O_PL        = 0x800318100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_BIT_STAT1_O_PL        = 0x800318100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_BIT_STAT1_O_PL         = 0x800318100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_BIT_STAT1_O_PL         = 0x800318100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_BIT_STAT1_O_PL        = 0x800318100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_BIT_STAT1_O_PL        = 0x800318100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_BIT_STAT2_EO_PL           = 0x800270100701103Full;
//DUPS: 800270100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_BIT_STAT2_EO_PL         = 0x800270100701103Full;
//DUPS: 800270100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_BIT_STAT2_EO_PL         = 0x800270100801103Full;
//DUPS: 800270100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_BIT_STAT2_EO_PL         = 0x800270100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_BIT_STAT2_EO_PL       = 0x800270100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_BIT_STAT2_EO_PL       = 0x800270100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_BIT_STAT2_EO_PL       = 0x800270100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_BIT_STAT2_EO_PL       = 0x800270100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_BIT_STAT2_EO_PL        = 0x800270100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_BIT_STAT2_EO_PL        = 0x800270100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_BIT_STAT2_EO_PL       = 0x800270100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_BIT_STAT2_EO_PL       = 0x800270100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_BIT_STAT3_EO_PL           = 0x800278100701103Full;
//DUPS: 800278100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_BIT_STAT3_EO_PL         = 0x800278100701103Full;
//DUPS: 800278100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_BIT_STAT3_EO_PL         = 0x800278100801103Full;
//DUPS: 800278100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_BIT_STAT3_EO_PL         = 0x800278100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_BIT_STAT3_EO_PL       = 0x800278100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_BIT_STAT3_EO_PL       = 0x800278100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_BIT_STAT3_EO_PL       = 0x800278100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_BIT_STAT3_EO_PL       = 0x800278100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_BIT_STAT3_EO_PL        = 0x800278100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_BIT_STAT3_EO_PL        = 0x800278100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_BIT_STAT3_EO_PL       = 0x800278100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_BIT_STAT3_EO_PL       = 0x800278100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_BIT_STAT4_EO_PL           = 0x800280100701103Full;
//DUPS: 800280100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_BIT_STAT4_EO_PL         = 0x800280100701103Full;
//DUPS: 800280100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_BIT_STAT4_EO_PL         = 0x800280100801103Full;
//DUPS: 800280100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_BIT_STAT4_EO_PL         = 0x800280100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_BIT_STAT4_EO_PL       = 0x800280100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_BIT_STAT4_EO_PL       = 0x800280100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_BIT_STAT4_EO_PL       = 0x800280100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_BIT_STAT4_EO_PL       = 0x800280100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_BIT_STAT4_EO_PL        = 0x800280100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_BIT_STAT4_EO_PL        = 0x800280100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_BIT_STAT4_EO_PL       = 0x800280100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_BIT_STAT4_EO_PL       = 0x800280100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL10_O_PL           = 0x8000C8100701103Full;
//DUPS: 8000C8100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL10_O_PL         = 0x8000C8100701103Full;
//DUPS: 8000C8100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL10_O_PL         = 0x8000C8100801103Full;
//DUPS: 8000C8100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL10_O_PL         = 0x8000C8100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C8100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C8100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C8100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C8100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL10_O_PL        = 0x8000C8100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL10_O_PL        = 0x8000C8100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C8100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C8100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL1_EO_PL           = 0x800008100701103Full;
//DUPS: 800008100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL1_EO_PL         = 0x800008100701103Full;
//DUPS: 800008100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL1_EO_PL         = 0x800008100801103Full;
//DUPS: 800008100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL1_EO_PL         = 0x800008100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x800008100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x800008100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x800008100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x800008100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL1_EO_PL        = 0x800008100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL1_EO_PL        = 0x800008100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x800008100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x800008100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL1_O_PL            = 0x800080100701103Full;
//DUPS: 800080100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL1_O_PL          = 0x800080100701103Full;
//DUPS: 800080100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL1_O_PL          = 0x800080100801103Full;
//DUPS: 800080100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL1_O_PL          = 0x800080100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL1_O_PL        = 0x800080100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL1_O_PL        = 0x800080100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL1_O_PL        = 0x800080100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL1_O_PL        = 0x800080100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL1_O_PL         = 0x800080100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL1_O_PL         = 0x800080100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL1_O_PL        = 0x800080100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL1_O_PL        = 0x800080100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL2_EO_PL           = 0x800010100701103Full;
//DUPS: 800010100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL2_EO_PL         = 0x800010100701103Full;
//DUPS: 800010100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL2_EO_PL         = 0x800010100801103Full;
//DUPS: 800010100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL2_EO_PL         = 0x800010100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x800010100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x800010100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x800010100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x800010100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL2_EO_PL        = 0x800010100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL2_EO_PL        = 0x800010100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x800010100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x800010100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL2_O_PL            = 0x800088100701103Full;
//DUPS: 800088100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL2_O_PL          = 0x800088100701103Full;
//DUPS: 800088100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL2_O_PL          = 0x800088100801103Full;
//DUPS: 800088100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL2_O_PL          = 0x800088100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL2_O_PL        = 0x800088100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL2_O_PL        = 0x800088100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL2_O_PL        = 0x800088100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL2_O_PL        = 0x800088100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL2_O_PL         = 0x800088100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL2_O_PL         = 0x800088100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL2_O_PL        = 0x800088100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL2_O_PL        = 0x800088100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL3_EO_PL           = 0x800018100701103Full;
//DUPS: 800018100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL3_EO_PL         = 0x800018100701103Full;
//DUPS: 800018100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL3_EO_PL         = 0x800018100801103Full;
//DUPS: 800018100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL3_EO_PL         = 0x800018100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x800018100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x800018100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x800018100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x800018100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL3_EO_PL        = 0x800018100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL3_EO_PL        = 0x800018100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x800018100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x800018100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL3_O_PL            = 0x800090100701103Full;
//DUPS: 800090100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL3_O_PL          = 0x800090100701103Full;
//DUPS: 800090100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL3_O_PL          = 0x800090100801103Full;
//DUPS: 800090100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL3_O_PL          = 0x800090100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL3_O_PL        = 0x800090100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL3_O_PL        = 0x800090100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL3_O_PL        = 0x800090100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL3_O_PL        = 0x800090100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL3_O_PL         = 0x800090100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL3_O_PL         = 0x800090100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL3_O_PL        = 0x800090100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL3_O_PL        = 0x800090100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL4_EO_PL           = 0x800020100701103Full;
//DUPS: 800020100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL4_EO_PL         = 0x800020100701103Full;
//DUPS: 800020100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL4_EO_PL         = 0x800020100801103Full;
//DUPS: 800020100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL4_EO_PL         = 0x800020100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x800020100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x800020100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x800020100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x800020100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL4_EO_PL        = 0x800020100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL4_EO_PL        = 0x800020100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x800020100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x800020100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL4_O_PL            = 0x800098100701103Full;
//DUPS: 800098100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL4_O_PL          = 0x800098100701103Full;
//DUPS: 800098100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL4_O_PL          = 0x800098100801103Full;
//DUPS: 800098100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL4_O_PL          = 0x800098100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL4_O_PL        = 0x800098100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL4_O_PL        = 0x800098100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL4_O_PL        = 0x800098100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL4_O_PL        = 0x800098100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL4_O_PL         = 0x800098100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL4_O_PL         = 0x800098100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL4_O_PL        = 0x800098100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL4_O_PL        = 0x800098100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL5_EO_PL           = 0x800028100701103Full;
//DUPS: 800028100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL5_EO_PL         = 0x800028100701103Full;
//DUPS: 800028100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL5_EO_PL         = 0x800028100801103Full;
//DUPS: 800028100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL5_EO_PL         = 0x800028100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x800028100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x800028100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x800028100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x800028100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL5_EO_PL        = 0x800028100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL5_EO_PL        = 0x800028100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x800028100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x800028100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL5_O_PL            = 0x8000A0100701103Full;
//DUPS: 8000A0100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL5_O_PL          = 0x8000A0100701103Full;
//DUPS: 8000A0100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL5_O_PL          = 0x8000A0100801103Full;
//DUPS: 8000A0100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL5_O_PL          = 0x8000A0100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A0100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A0100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A0100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A0100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL5_O_PL         = 0x8000A0100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL5_O_PL         = 0x8000A0100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A0100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A0100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL6_EO_PL           = 0x800030100701103Full;
//DUPS: 800030100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL6_EO_PL         = 0x800030100701103Full;
//DUPS: 800030100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL6_EO_PL         = 0x800030100801103Full;
//DUPS: 800030100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL6_EO_PL         = 0x800030100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x800030100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x800030100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x800030100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x800030100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL6_EO_PL        = 0x800030100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL6_EO_PL        = 0x800030100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x800030100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x800030100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL6_O_PL            = 0x8000A8100701103Full;
//DUPS: 8000A8100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL6_O_PL          = 0x8000A8100701103Full;
//DUPS: 8000A8100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL6_O_PL          = 0x8000A8100801103Full;
//DUPS: 8000A8100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL6_O_PL          = 0x8000A8100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A8100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A8100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A8100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A8100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL6_O_PL         = 0x8000A8100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL6_O_PL         = 0x8000A8100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A8100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A8100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL7_EO_PL           = 0x800038100701103Full;
//DUPS: 800038100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL7_EO_PL         = 0x800038100701103Full;
//DUPS: 800038100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL7_EO_PL         = 0x800038100801103Full;
//DUPS: 800038100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL7_EO_PL         = 0x800038100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x800038100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x800038100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x800038100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x800038100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL7_EO_PL        = 0x800038100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL7_EO_PL        = 0x800038100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x800038100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x800038100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL7_O_PL            = 0x8000B0100701103Full;
//DUPS: 8000B0100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL7_O_PL          = 0x8000B0100701103Full;
//DUPS: 8000B0100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL7_O_PL          = 0x8000B0100801103Full;
//DUPS: 8000B0100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL7_O_PL          = 0x8000B0100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B0100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B0100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B0100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B0100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL7_O_PL         = 0x8000B0100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL7_O_PL         = 0x8000B0100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B0100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B0100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL8_EO_PL           = 0x800040100701103Full;
//DUPS: 800040100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL8_EO_PL         = 0x800040100701103Full;
//DUPS: 800040100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL8_EO_PL         = 0x800040100801103Full;
//DUPS: 800040100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL8_EO_PL         = 0x800040100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x800040100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x800040100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x800040100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x800040100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL8_EO_PL        = 0x800040100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL8_EO_PL        = 0x800040100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x800040100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x800040100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL8_O_PL            = 0x8000B8100701103Full;
//DUPS: 8000B8100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL8_O_PL          = 0x8000B8100701103Full;
//DUPS: 8000B8100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL8_O_PL          = 0x8000B8100801103Full;
//DUPS: 8000B8100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL8_O_PL          = 0x8000B8100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B8100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B8100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B8100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B8100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL8_O_PL         = 0x8000B8100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL8_O_PL         = 0x8000B8100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B8100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B8100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL9_O_PL            = 0x8000C0100701103Full;
//DUPS: 8000C0100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL9_O_PL          = 0x8000C0100701103Full;
//DUPS: 8000C0100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL9_O_PL          = 0x8000C0100801103Full;
//DUPS: 8000C0100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL9_O_PL          = 0x8000C0100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C0100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C0100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C0100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C0100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL9_O_PL         = 0x8000C0100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL9_O_PL         = 0x8000C0100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C0100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C0100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_DATA_DAC_SPARE_MODE_PL    = 0x800000100701103Full;
//DUPS: 800000100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000100701103Full;
//DUPS: 800000100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000100801103Full;
//DUPS: 800000100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_FIR_ERROR_INJECT_PL       = 0x800218100701103Full;
//DUPS: 800218100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_FIR_ERROR_INJECT_PL     = 0x800218100701103Full;
//DUPS: 800218100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_FIR_ERROR_INJECT_PL     = 0x800218100801103Full;
//DUPS: 800218100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_FIR_ERROR_INJECT_PL     = 0x800218100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x800218100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x800218100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x800218100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x800218100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_FIR_ERROR_INJECT_PL    = 0x800218100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_FIR_ERROR_INJECT_PL    = 0x800218100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x800218100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x800218100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_FIR_MASK_PL               = 0x800210100701103Full;
//DUPS: 800210100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_FIR_MASK_PL             = 0x800210100701103Full;
//DUPS: 800210100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_FIR_MASK_PL             = 0x800210100801103Full;
//DUPS: 800210100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_FIR_MASK_PL             = 0x800210100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_FIR_MASK_PL           = 0x800210100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_FIR_MASK_PL           = 0x800210100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_FIR_MASK_PL           = 0x800210100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_FIR_MASK_PL           = 0x800210100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_FIR_MASK_PL            = 0x800210100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_FIR_MASK_PL            = 0x800210100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_FIR_MASK_PL           = 0x800210100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_FIR_MASK_PL           = 0x800210100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_FIR_PL                    = 0x800208100701103Full;
//DUPS: 800208100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_FIR_PL                  = 0x800208100701103Full;
//DUPS: 800208100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_FIR_PL                  = 0x800208100801103Full;
//DUPS: 800208100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_FIR_PL                  = 0x800208100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_FIR_PL                = 0x800208100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_FIR_PL                = 0x800208100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_FIR_PL                = 0x800208100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_FIR_PL                = 0x800208100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_FIR_PL                 = 0x800208100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_FIR_PL                 = 0x800208100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_FIR_PL                = 0x800208100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_FIR_PL                = 0x800208100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE0_RX_SPARE_MODE_PL             = 0x800200100701103Full;
//DUPS: 800200100701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE0_RX_SPARE_MODE_PL           = 0x800200100701103Full;
//DUPS: 800200100701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE0_RX_SPARE_MODE_PL           = 0x800200100801103Full;
//DUPS: 800200100801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE0_RX_SPARE_MODE_PL           = 0x800200100701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE0_RX_SPARE_MODE_PL         = 0x800200100701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE0_RX_SPARE_MODE_PL         = 0x800200100701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE0_RX_SPARE_MODE_PL         = 0x800200100801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE0_RX_SPARE_MODE_PL         = 0x800200100801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE0_RX_SPARE_MODE_PL          = 0x800200100701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE0_RX_SPARE_MODE_PL          = 0x800200100701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE0_RX_SPARE_MODE_PL         = 0x800200100801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE0_RX_SPARE_MODE_PL         = 0x800200100801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL2_EO_PL           = 0x800248110701103Full;
//DUPS: 800248110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL2_EO_PL         = 0x800248110701103Full;
//DUPS: 800248110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL2_EO_PL         = 0x800248110801103Full;
//DUPS: 800248110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL2_EO_PL         = 0x800248110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x800248110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x800248110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x800248110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x800248110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL2_EO_PL        = 0x800248110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL2_EO_PL        = 0x800248110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x800248110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x800248110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL3_EO_PL           = 0x800250110701103Full;
//DUPS: 800250110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL3_EO_PL         = 0x800250110701103Full;
//DUPS: 800250110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL3_EO_PL         = 0x800250110801103Full;
//DUPS: 800250110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL3_EO_PL         = 0x800250110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x800250110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x800250110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x800250110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x800250110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL3_EO_PL        = 0x800250110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL3_EO_PL        = 0x800250110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x800250110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x800250110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL4_EO_PL           = 0x800258110701103Full;
//DUPS: 800258110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL4_EO_PL         = 0x800258110701103Full;
//DUPS: 800258110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL4_EO_PL         = 0x800258110801103Full;
//DUPS: 800258110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL4_EO_PL         = 0x800258110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x800258110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x800258110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x800258110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x800258110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL4_EO_PL        = 0x800258110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL4_EO_PL        = 0x800258110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x800258110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x800258110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL5_EO_PL           = 0x800260110701103Full;
//DUPS: 800260110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL5_EO_PL         = 0x800260110701103Full;
//DUPS: 800260110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL5_EO_PL         = 0x800260110801103Full;
//DUPS: 800260110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL5_EO_PL         = 0x800260110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x800260110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x800260110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x800260110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x800260110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL5_EO_PL        = 0x800260110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL5_EO_PL        = 0x800260110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x800260110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x800260110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_BIT_CNTLX1_EO_PL          = 0x800240110701103Full;
//DUPS: 800240110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_BIT_CNTLX1_EO_PL        = 0x800240110701103Full;
//DUPS: 800240110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_BIT_CNTLX1_EO_PL        = 0x800240110801103Full;
//DUPS: 800240110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_BIT_CNTLX1_EO_PL        = 0x800240110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x800240110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x800240110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x800240110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x800240110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_BIT_CNTLX1_EO_PL       = 0x800240110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_BIT_CNTLX1_EO_PL       = 0x800240110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x800240110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x800240110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_BIT_MODE1_EO_PL           = 0x800220110701103Full;
//DUPS: 800220110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_BIT_MODE1_EO_PL         = 0x800220110701103Full;
//DUPS: 800220110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_BIT_MODE1_EO_PL         = 0x800220110801103Full;
//DUPS: 800220110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_BIT_MODE1_EO_PL         = 0x800220110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_BIT_MODE1_EO_PL       = 0x800220110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_BIT_MODE1_EO_PL       = 0x800220110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_BIT_MODE1_EO_PL       = 0x800220110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_BIT_MODE1_EO_PL       = 0x800220110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_BIT_MODE1_EO_PL        = 0x800220110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_BIT_MODE1_EO_PL        = 0x800220110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_BIT_MODE1_EO_PL       = 0x800220110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_BIT_MODE1_EO_PL       = 0x800220110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_BIT_MODE2_EO_PL           = 0x800228110701103Full;
//DUPS: 800228110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_BIT_MODE2_EO_PL         = 0x800228110701103Full;
//DUPS: 800228110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_BIT_MODE2_EO_PL         = 0x800228110801103Full;
//DUPS: 800228110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_BIT_MODE2_EO_PL         = 0x800228110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_BIT_MODE2_EO_PL       = 0x800228110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_BIT_MODE2_EO_PL       = 0x800228110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_BIT_MODE2_EO_PL       = 0x800228110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_BIT_MODE2_EO_PL       = 0x800228110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_BIT_MODE2_EO_PL        = 0x800228110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_BIT_MODE2_EO_PL        = 0x800228110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_BIT_MODE2_EO_PL       = 0x800228110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_BIT_MODE2_EO_PL       = 0x800228110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_BIT_MODE3_EO_PL           = 0x800230110701103Full;
//DUPS: 800230110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_BIT_MODE3_EO_PL         = 0x800230110701103Full;
//DUPS: 800230110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_BIT_MODE3_EO_PL         = 0x800230110801103Full;
//DUPS: 800230110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_BIT_MODE3_EO_PL         = 0x800230110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_BIT_MODE3_EO_PL       = 0x800230110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_BIT_MODE3_EO_PL       = 0x800230110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_BIT_MODE3_EO_PL       = 0x800230110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_BIT_MODE3_EO_PL       = 0x800230110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_BIT_MODE3_EO_PL        = 0x800230110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_BIT_MODE3_EO_PL        = 0x800230110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_BIT_MODE3_EO_PL       = 0x800230110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_BIT_MODE3_EO_PL       = 0x800230110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_BIT_MODE4_EO_PL           = 0x800238110701103Full;
//DUPS: 800238110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_BIT_MODE4_EO_PL         = 0x800238110701103Full;
//DUPS: 800238110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_BIT_MODE4_EO_PL         = 0x800238110801103Full;
//DUPS: 800238110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_BIT_MODE4_EO_PL         = 0x800238110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_BIT_MODE4_EO_PL       = 0x800238110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_BIT_MODE4_EO_PL       = 0x800238110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_BIT_MODE4_EO_PL       = 0x800238110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_BIT_MODE4_EO_PL       = 0x800238110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_BIT_MODE4_EO_PL        = 0x800238110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_BIT_MODE4_EO_PL        = 0x800238110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_BIT_MODE4_EO_PL       = 0x800238110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_BIT_MODE4_EO_PL       = 0x800238110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_BIT_STAT1_EO_PL           = 0x800268110701103Full;
//DUPS: 800268110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_BIT_STAT1_EO_PL         = 0x800268110701103Full;
//DUPS: 800268110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_BIT_STAT1_EO_PL         = 0x800268110801103Full;
//DUPS: 800268110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_BIT_STAT1_EO_PL         = 0x800268110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_BIT_STAT1_EO_PL       = 0x800268110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_BIT_STAT1_EO_PL       = 0x800268110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_BIT_STAT1_EO_PL       = 0x800268110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_BIT_STAT1_EO_PL       = 0x800268110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_BIT_STAT1_EO_PL        = 0x800268110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_BIT_STAT1_EO_PL        = 0x800268110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_BIT_STAT1_EO_PL       = 0x800268110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_BIT_STAT1_EO_PL       = 0x800268110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_BIT_STAT1_O_PL            = 0x800318110701103Full;
//DUPS: 800318110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_BIT_STAT1_O_PL          = 0x800318110701103Full;
//DUPS: 800318110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_BIT_STAT1_O_PL          = 0x800318110801103Full;
//DUPS: 800318110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_BIT_STAT1_O_PL          = 0x800318110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_BIT_STAT1_O_PL        = 0x800318110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_BIT_STAT1_O_PL        = 0x800318110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_BIT_STAT1_O_PL        = 0x800318110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_BIT_STAT1_O_PL        = 0x800318110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_BIT_STAT1_O_PL         = 0x800318110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_BIT_STAT1_O_PL         = 0x800318110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_BIT_STAT1_O_PL        = 0x800318110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_BIT_STAT1_O_PL        = 0x800318110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_BIT_STAT2_EO_PL           = 0x800270110701103Full;
//DUPS: 800270110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_BIT_STAT2_EO_PL         = 0x800270110701103Full;
//DUPS: 800270110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_BIT_STAT2_EO_PL         = 0x800270110801103Full;
//DUPS: 800270110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_BIT_STAT2_EO_PL         = 0x800270110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_BIT_STAT2_EO_PL       = 0x800270110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_BIT_STAT2_EO_PL       = 0x800270110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_BIT_STAT2_EO_PL       = 0x800270110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_BIT_STAT2_EO_PL       = 0x800270110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_BIT_STAT2_EO_PL        = 0x800270110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_BIT_STAT2_EO_PL        = 0x800270110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_BIT_STAT2_EO_PL       = 0x800270110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_BIT_STAT2_EO_PL       = 0x800270110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_BIT_STAT3_EO_PL           = 0x800278110701103Full;
//DUPS: 800278110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_BIT_STAT3_EO_PL         = 0x800278110701103Full;
//DUPS: 800278110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_BIT_STAT3_EO_PL         = 0x800278110801103Full;
//DUPS: 800278110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_BIT_STAT3_EO_PL         = 0x800278110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_BIT_STAT3_EO_PL       = 0x800278110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_BIT_STAT3_EO_PL       = 0x800278110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_BIT_STAT3_EO_PL       = 0x800278110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_BIT_STAT3_EO_PL       = 0x800278110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_BIT_STAT3_EO_PL        = 0x800278110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_BIT_STAT3_EO_PL        = 0x800278110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_BIT_STAT3_EO_PL       = 0x800278110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_BIT_STAT3_EO_PL       = 0x800278110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_BIT_STAT4_EO_PL           = 0x800280110701103Full;
//DUPS: 800280110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_BIT_STAT4_EO_PL         = 0x800280110701103Full;
//DUPS: 800280110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_BIT_STAT4_EO_PL         = 0x800280110801103Full;
//DUPS: 800280110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_BIT_STAT4_EO_PL         = 0x800280110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_BIT_STAT4_EO_PL       = 0x800280110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_BIT_STAT4_EO_PL       = 0x800280110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_BIT_STAT4_EO_PL       = 0x800280110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_BIT_STAT4_EO_PL       = 0x800280110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_BIT_STAT4_EO_PL        = 0x800280110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_BIT_STAT4_EO_PL        = 0x800280110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_BIT_STAT4_EO_PL       = 0x800280110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_BIT_STAT4_EO_PL       = 0x800280110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL10_O_PL           = 0x8000C8110701103Full;
//DUPS: 8000C8110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL10_O_PL         = 0x8000C8110701103Full;
//DUPS: 8000C8110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL10_O_PL         = 0x8000C8110801103Full;
//DUPS: 8000C8110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL10_O_PL         = 0x8000C8110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C8110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C8110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C8110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C8110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL10_O_PL        = 0x8000C8110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL10_O_PL        = 0x8000C8110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C8110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C8110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL1_EO_PL           = 0x800008110701103Full;
//DUPS: 800008110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL1_EO_PL         = 0x800008110701103Full;
//DUPS: 800008110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL1_EO_PL         = 0x800008110801103Full;
//DUPS: 800008110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL1_EO_PL         = 0x800008110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x800008110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x800008110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x800008110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x800008110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL1_EO_PL        = 0x800008110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL1_EO_PL        = 0x800008110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x800008110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x800008110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL1_O_PL            = 0x800080110701103Full;
//DUPS: 800080110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL1_O_PL          = 0x800080110701103Full;
//DUPS: 800080110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL1_O_PL          = 0x800080110801103Full;
//DUPS: 800080110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL1_O_PL          = 0x800080110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL1_O_PL        = 0x800080110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL1_O_PL        = 0x800080110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL1_O_PL        = 0x800080110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL1_O_PL        = 0x800080110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL1_O_PL         = 0x800080110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL1_O_PL         = 0x800080110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL1_O_PL        = 0x800080110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL1_O_PL        = 0x800080110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL2_EO_PL           = 0x800010110701103Full;
//DUPS: 800010110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL2_EO_PL         = 0x800010110701103Full;
//DUPS: 800010110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL2_EO_PL         = 0x800010110801103Full;
//DUPS: 800010110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL2_EO_PL         = 0x800010110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x800010110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x800010110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x800010110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x800010110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL2_EO_PL        = 0x800010110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL2_EO_PL        = 0x800010110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x800010110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x800010110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL2_O_PL            = 0x800088110701103Full;
//DUPS: 800088110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL2_O_PL          = 0x800088110701103Full;
//DUPS: 800088110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL2_O_PL          = 0x800088110801103Full;
//DUPS: 800088110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL2_O_PL          = 0x800088110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL2_O_PL        = 0x800088110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL2_O_PL        = 0x800088110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL2_O_PL        = 0x800088110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL2_O_PL        = 0x800088110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL2_O_PL         = 0x800088110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL2_O_PL         = 0x800088110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL2_O_PL        = 0x800088110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL2_O_PL        = 0x800088110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL3_EO_PL           = 0x800018110701103Full;
//DUPS: 800018110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL3_EO_PL         = 0x800018110701103Full;
//DUPS: 800018110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL3_EO_PL         = 0x800018110801103Full;
//DUPS: 800018110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL3_EO_PL         = 0x800018110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x800018110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x800018110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x800018110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x800018110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL3_EO_PL        = 0x800018110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL3_EO_PL        = 0x800018110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x800018110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x800018110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL3_O_PL            = 0x800090110701103Full;
//DUPS: 800090110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL3_O_PL          = 0x800090110701103Full;
//DUPS: 800090110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL3_O_PL          = 0x800090110801103Full;
//DUPS: 800090110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL3_O_PL          = 0x800090110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL3_O_PL        = 0x800090110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL3_O_PL        = 0x800090110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL3_O_PL        = 0x800090110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL3_O_PL        = 0x800090110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL3_O_PL         = 0x800090110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL3_O_PL         = 0x800090110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL3_O_PL        = 0x800090110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL3_O_PL        = 0x800090110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL4_EO_PL           = 0x800020110701103Full;
//DUPS: 800020110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL4_EO_PL         = 0x800020110701103Full;
//DUPS: 800020110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL4_EO_PL         = 0x800020110801103Full;
//DUPS: 800020110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL4_EO_PL         = 0x800020110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x800020110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x800020110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x800020110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x800020110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL4_EO_PL        = 0x800020110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL4_EO_PL        = 0x800020110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x800020110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x800020110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL4_O_PL            = 0x800098110701103Full;
//DUPS: 800098110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL4_O_PL          = 0x800098110701103Full;
//DUPS: 800098110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL4_O_PL          = 0x800098110801103Full;
//DUPS: 800098110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL4_O_PL          = 0x800098110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL4_O_PL        = 0x800098110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL4_O_PL        = 0x800098110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL4_O_PL        = 0x800098110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL4_O_PL        = 0x800098110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL4_O_PL         = 0x800098110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL4_O_PL         = 0x800098110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL4_O_PL        = 0x800098110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL4_O_PL        = 0x800098110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL5_EO_PL           = 0x800028110701103Full;
//DUPS: 800028110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL5_EO_PL         = 0x800028110701103Full;
//DUPS: 800028110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL5_EO_PL         = 0x800028110801103Full;
//DUPS: 800028110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL5_EO_PL         = 0x800028110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x800028110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x800028110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x800028110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x800028110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL5_EO_PL        = 0x800028110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL5_EO_PL        = 0x800028110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x800028110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x800028110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL5_O_PL            = 0x8000A0110701103Full;
//DUPS: 8000A0110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL5_O_PL          = 0x8000A0110701103Full;
//DUPS: 8000A0110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL5_O_PL          = 0x8000A0110801103Full;
//DUPS: 8000A0110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL5_O_PL          = 0x8000A0110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A0110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A0110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A0110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A0110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL5_O_PL         = 0x8000A0110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL5_O_PL         = 0x8000A0110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A0110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A0110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL6_EO_PL           = 0x800030110701103Full;
//DUPS: 800030110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL6_EO_PL         = 0x800030110701103Full;
//DUPS: 800030110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL6_EO_PL         = 0x800030110801103Full;
//DUPS: 800030110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL6_EO_PL         = 0x800030110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x800030110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x800030110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x800030110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x800030110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL6_EO_PL        = 0x800030110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL6_EO_PL        = 0x800030110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x800030110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x800030110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL6_O_PL            = 0x8000A8110701103Full;
//DUPS: 8000A8110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL6_O_PL          = 0x8000A8110701103Full;
//DUPS: 8000A8110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL6_O_PL          = 0x8000A8110801103Full;
//DUPS: 8000A8110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL6_O_PL          = 0x8000A8110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A8110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A8110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A8110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A8110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL6_O_PL         = 0x8000A8110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL6_O_PL         = 0x8000A8110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A8110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A8110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL7_EO_PL           = 0x800038110701103Full;
//DUPS: 800038110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL7_EO_PL         = 0x800038110701103Full;
//DUPS: 800038110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL7_EO_PL         = 0x800038110801103Full;
//DUPS: 800038110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL7_EO_PL         = 0x800038110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x800038110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x800038110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x800038110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x800038110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL7_EO_PL        = 0x800038110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL7_EO_PL        = 0x800038110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x800038110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x800038110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL7_O_PL            = 0x8000B0110701103Full;
//DUPS: 8000B0110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL7_O_PL          = 0x8000B0110701103Full;
//DUPS: 8000B0110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL7_O_PL          = 0x8000B0110801103Full;
//DUPS: 8000B0110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL7_O_PL          = 0x8000B0110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B0110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B0110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B0110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B0110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL7_O_PL         = 0x8000B0110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL7_O_PL         = 0x8000B0110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B0110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B0110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL8_EO_PL           = 0x800040110701103Full;
//DUPS: 800040110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL8_EO_PL         = 0x800040110701103Full;
//DUPS: 800040110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL8_EO_PL         = 0x800040110801103Full;
//DUPS: 800040110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL8_EO_PL         = 0x800040110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x800040110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x800040110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x800040110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x800040110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL8_EO_PL        = 0x800040110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL8_EO_PL        = 0x800040110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x800040110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x800040110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL8_O_PL            = 0x8000B8110701103Full;
//DUPS: 8000B8110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL8_O_PL          = 0x8000B8110701103Full;
//DUPS: 8000B8110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL8_O_PL          = 0x8000B8110801103Full;
//DUPS: 8000B8110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL8_O_PL          = 0x8000B8110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B8110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B8110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B8110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B8110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL8_O_PL         = 0x8000B8110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL8_O_PL         = 0x8000B8110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B8110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B8110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL9_O_PL            = 0x8000C0110701103Full;
//DUPS: 8000C0110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL9_O_PL          = 0x8000C0110701103Full;
//DUPS: 8000C0110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL9_O_PL          = 0x8000C0110801103Full;
//DUPS: 8000C0110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL9_O_PL          = 0x8000C0110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C0110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C0110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C0110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C0110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL9_O_PL         = 0x8000C0110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL9_O_PL         = 0x8000C0110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C0110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C0110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_DATA_DAC_SPARE_MODE_PL    = 0x800000110701103Full;
//DUPS: 800000110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000110701103Full;
//DUPS: 800000110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000110801103Full;
//DUPS: 800000110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_FIR_ERROR_INJECT_PL       = 0x800218110701103Full;
//DUPS: 800218110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_FIR_ERROR_INJECT_PL     = 0x800218110701103Full;
//DUPS: 800218110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_FIR_ERROR_INJECT_PL     = 0x800218110801103Full;
//DUPS: 800218110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_FIR_ERROR_INJECT_PL     = 0x800218110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x800218110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x800218110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x800218110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x800218110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_FIR_ERROR_INJECT_PL    = 0x800218110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_FIR_ERROR_INJECT_PL    = 0x800218110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x800218110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x800218110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_FIR_MASK_PL               = 0x800210110701103Full;
//DUPS: 800210110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_FIR_MASK_PL             = 0x800210110701103Full;
//DUPS: 800210110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_FIR_MASK_PL             = 0x800210110801103Full;
//DUPS: 800210110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_FIR_MASK_PL             = 0x800210110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_FIR_MASK_PL           = 0x800210110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_FIR_MASK_PL           = 0x800210110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_FIR_MASK_PL           = 0x800210110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_FIR_MASK_PL           = 0x800210110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_FIR_MASK_PL            = 0x800210110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_FIR_MASK_PL            = 0x800210110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_FIR_MASK_PL           = 0x800210110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_FIR_MASK_PL           = 0x800210110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_FIR_PL                    = 0x800208110701103Full;
//DUPS: 800208110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_FIR_PL                  = 0x800208110701103Full;
//DUPS: 800208110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_FIR_PL                  = 0x800208110801103Full;
//DUPS: 800208110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_FIR_PL                  = 0x800208110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_FIR_PL                = 0x800208110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_FIR_PL                = 0x800208110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_FIR_PL                = 0x800208110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_FIR_PL                = 0x800208110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_FIR_PL                 = 0x800208110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_FIR_PL                 = 0x800208110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_FIR_PL                = 0x800208110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_FIR_PL                = 0x800208110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE1_RX_SPARE_MODE_PL             = 0x800200110701103Full;
//DUPS: 800200110701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE1_RX_SPARE_MODE_PL           = 0x800200110701103Full;
//DUPS: 800200110701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE1_RX_SPARE_MODE_PL           = 0x800200110801103Full;
//DUPS: 800200110801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE1_RX_SPARE_MODE_PL           = 0x800200110701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE1_RX_SPARE_MODE_PL         = 0x800200110701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE1_RX_SPARE_MODE_PL         = 0x800200110701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE1_RX_SPARE_MODE_PL         = 0x800200110801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE1_RX_SPARE_MODE_PL         = 0x800200110801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE1_RX_SPARE_MODE_PL          = 0x800200110701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE1_RX_SPARE_MODE_PL          = 0x800200110701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE1_RX_SPARE_MODE_PL         = 0x800200110801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE1_RX_SPARE_MODE_PL         = 0x800200110801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL2_EO_PL           = 0x800248120701103Full;
//DUPS: 800248120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL2_EO_PL         = 0x800248120701103Full;
//DUPS: 800248120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL2_EO_PL         = 0x800248120801103Full;
//DUPS: 800248120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL2_EO_PL         = 0x800248120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x800248120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x800248120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x800248120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x800248120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL2_EO_PL        = 0x800248120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL2_EO_PL        = 0x800248120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x800248120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x800248120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL3_EO_PL           = 0x800250120701103Full;
//DUPS: 800250120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL3_EO_PL         = 0x800250120701103Full;
//DUPS: 800250120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL3_EO_PL         = 0x800250120801103Full;
//DUPS: 800250120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL3_EO_PL         = 0x800250120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x800250120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x800250120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x800250120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x800250120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL3_EO_PL        = 0x800250120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL3_EO_PL        = 0x800250120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x800250120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x800250120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL4_EO_PL           = 0x800258120701103Full;
//DUPS: 800258120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL4_EO_PL         = 0x800258120701103Full;
//DUPS: 800258120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL4_EO_PL         = 0x800258120801103Full;
//DUPS: 800258120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL4_EO_PL         = 0x800258120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x800258120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x800258120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x800258120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x800258120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL4_EO_PL        = 0x800258120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL4_EO_PL        = 0x800258120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x800258120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x800258120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL5_EO_PL           = 0x800260120701103Full;
//DUPS: 800260120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL5_EO_PL         = 0x800260120701103Full;
//DUPS: 800260120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL5_EO_PL         = 0x800260120801103Full;
//DUPS: 800260120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL5_EO_PL         = 0x800260120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x800260120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x800260120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x800260120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x800260120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL5_EO_PL        = 0x800260120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL5_EO_PL        = 0x800260120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x800260120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x800260120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_BIT_CNTLX1_EO_PL          = 0x800240120701103Full;
//DUPS: 800240120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_BIT_CNTLX1_EO_PL        = 0x800240120701103Full;
//DUPS: 800240120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_BIT_CNTLX1_EO_PL        = 0x800240120801103Full;
//DUPS: 800240120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_BIT_CNTLX1_EO_PL        = 0x800240120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x800240120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x800240120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x800240120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x800240120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_BIT_CNTLX1_EO_PL       = 0x800240120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_BIT_CNTLX1_EO_PL       = 0x800240120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x800240120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x800240120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_BIT_MODE1_EO_PL           = 0x800220120701103Full;
//DUPS: 800220120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_BIT_MODE1_EO_PL         = 0x800220120701103Full;
//DUPS: 800220120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_BIT_MODE1_EO_PL         = 0x800220120801103Full;
//DUPS: 800220120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_BIT_MODE1_EO_PL         = 0x800220120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_BIT_MODE1_EO_PL       = 0x800220120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_BIT_MODE1_EO_PL       = 0x800220120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_BIT_MODE1_EO_PL       = 0x800220120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_BIT_MODE1_EO_PL       = 0x800220120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_BIT_MODE1_EO_PL        = 0x800220120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_BIT_MODE1_EO_PL        = 0x800220120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_BIT_MODE1_EO_PL       = 0x800220120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_BIT_MODE1_EO_PL       = 0x800220120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_BIT_MODE2_EO_PL           = 0x800228120701103Full;
//DUPS: 800228120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_BIT_MODE2_EO_PL         = 0x800228120701103Full;
//DUPS: 800228120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_BIT_MODE2_EO_PL         = 0x800228120801103Full;
//DUPS: 800228120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_BIT_MODE2_EO_PL         = 0x800228120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_BIT_MODE2_EO_PL       = 0x800228120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_BIT_MODE2_EO_PL       = 0x800228120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_BIT_MODE2_EO_PL       = 0x800228120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_BIT_MODE2_EO_PL       = 0x800228120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_BIT_MODE2_EO_PL        = 0x800228120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_BIT_MODE2_EO_PL        = 0x800228120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_BIT_MODE2_EO_PL       = 0x800228120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_BIT_MODE2_EO_PL       = 0x800228120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_BIT_MODE3_EO_PL           = 0x800230120701103Full;
//DUPS: 800230120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_BIT_MODE3_EO_PL         = 0x800230120701103Full;
//DUPS: 800230120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_BIT_MODE3_EO_PL         = 0x800230120801103Full;
//DUPS: 800230120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_BIT_MODE3_EO_PL         = 0x800230120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_BIT_MODE3_EO_PL       = 0x800230120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_BIT_MODE3_EO_PL       = 0x800230120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_BIT_MODE3_EO_PL       = 0x800230120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_BIT_MODE3_EO_PL       = 0x800230120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_BIT_MODE3_EO_PL        = 0x800230120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_BIT_MODE3_EO_PL        = 0x800230120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_BIT_MODE3_EO_PL       = 0x800230120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_BIT_MODE3_EO_PL       = 0x800230120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_BIT_MODE4_EO_PL           = 0x800238120701103Full;
//DUPS: 800238120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_BIT_MODE4_EO_PL         = 0x800238120701103Full;
//DUPS: 800238120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_BIT_MODE4_EO_PL         = 0x800238120801103Full;
//DUPS: 800238120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_BIT_MODE4_EO_PL         = 0x800238120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_BIT_MODE4_EO_PL       = 0x800238120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_BIT_MODE4_EO_PL       = 0x800238120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_BIT_MODE4_EO_PL       = 0x800238120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_BIT_MODE4_EO_PL       = 0x800238120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_BIT_MODE4_EO_PL        = 0x800238120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_BIT_MODE4_EO_PL        = 0x800238120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_BIT_MODE4_EO_PL       = 0x800238120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_BIT_MODE4_EO_PL       = 0x800238120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_BIT_STAT1_EO_PL           = 0x800268120701103Full;
//DUPS: 800268120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_BIT_STAT1_EO_PL         = 0x800268120701103Full;
//DUPS: 800268120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_BIT_STAT1_EO_PL         = 0x800268120801103Full;
//DUPS: 800268120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_BIT_STAT1_EO_PL         = 0x800268120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_BIT_STAT1_EO_PL       = 0x800268120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_BIT_STAT1_EO_PL       = 0x800268120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_BIT_STAT1_EO_PL       = 0x800268120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_BIT_STAT1_EO_PL       = 0x800268120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_BIT_STAT1_EO_PL        = 0x800268120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_BIT_STAT1_EO_PL        = 0x800268120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_BIT_STAT1_EO_PL       = 0x800268120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_BIT_STAT1_EO_PL       = 0x800268120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_BIT_STAT1_O_PL            = 0x800318120701103Full;
//DUPS: 800318120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_BIT_STAT1_O_PL          = 0x800318120701103Full;
//DUPS: 800318120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_BIT_STAT1_O_PL          = 0x800318120801103Full;
//DUPS: 800318120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_BIT_STAT1_O_PL          = 0x800318120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_BIT_STAT1_O_PL        = 0x800318120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_BIT_STAT1_O_PL        = 0x800318120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_BIT_STAT1_O_PL        = 0x800318120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_BIT_STAT1_O_PL        = 0x800318120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_BIT_STAT1_O_PL         = 0x800318120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_BIT_STAT1_O_PL         = 0x800318120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_BIT_STAT1_O_PL        = 0x800318120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_BIT_STAT1_O_PL        = 0x800318120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_BIT_STAT2_EO_PL           = 0x800270120701103Full;
//DUPS: 800270120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_BIT_STAT2_EO_PL         = 0x800270120701103Full;
//DUPS: 800270120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_BIT_STAT2_EO_PL         = 0x800270120801103Full;
//DUPS: 800270120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_BIT_STAT2_EO_PL         = 0x800270120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_BIT_STAT2_EO_PL       = 0x800270120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_BIT_STAT2_EO_PL       = 0x800270120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_BIT_STAT2_EO_PL       = 0x800270120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_BIT_STAT2_EO_PL       = 0x800270120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_BIT_STAT2_EO_PL        = 0x800270120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_BIT_STAT2_EO_PL        = 0x800270120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_BIT_STAT2_EO_PL       = 0x800270120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_BIT_STAT2_EO_PL       = 0x800270120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_BIT_STAT3_EO_PL           = 0x800278120701103Full;
//DUPS: 800278120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_BIT_STAT3_EO_PL         = 0x800278120701103Full;
//DUPS: 800278120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_BIT_STAT3_EO_PL         = 0x800278120801103Full;
//DUPS: 800278120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_BIT_STAT3_EO_PL         = 0x800278120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_BIT_STAT3_EO_PL       = 0x800278120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_BIT_STAT3_EO_PL       = 0x800278120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_BIT_STAT3_EO_PL       = 0x800278120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_BIT_STAT3_EO_PL       = 0x800278120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_BIT_STAT3_EO_PL        = 0x800278120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_BIT_STAT3_EO_PL        = 0x800278120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_BIT_STAT3_EO_PL       = 0x800278120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_BIT_STAT3_EO_PL       = 0x800278120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_BIT_STAT4_EO_PL           = 0x800280120701103Full;
//DUPS: 800280120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_BIT_STAT4_EO_PL         = 0x800280120701103Full;
//DUPS: 800280120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_BIT_STAT4_EO_PL         = 0x800280120801103Full;
//DUPS: 800280120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_BIT_STAT4_EO_PL         = 0x800280120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_BIT_STAT4_EO_PL       = 0x800280120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_BIT_STAT4_EO_PL       = 0x800280120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_BIT_STAT4_EO_PL       = 0x800280120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_BIT_STAT4_EO_PL       = 0x800280120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_BIT_STAT4_EO_PL        = 0x800280120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_BIT_STAT4_EO_PL        = 0x800280120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_BIT_STAT4_EO_PL       = 0x800280120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_BIT_STAT4_EO_PL       = 0x800280120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL10_O_PL           = 0x8000C8120701103Full;
//DUPS: 8000C8120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL10_O_PL         = 0x8000C8120701103Full;
//DUPS: 8000C8120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL10_O_PL         = 0x8000C8120801103Full;
//DUPS: 8000C8120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL10_O_PL         = 0x8000C8120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C8120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C8120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C8120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C8120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL10_O_PL        = 0x8000C8120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL10_O_PL        = 0x8000C8120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C8120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C8120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL1_EO_PL           = 0x800008120701103Full;
//DUPS: 800008120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL1_EO_PL         = 0x800008120701103Full;
//DUPS: 800008120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL1_EO_PL         = 0x800008120801103Full;
//DUPS: 800008120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL1_EO_PL         = 0x800008120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x800008120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x800008120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x800008120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x800008120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL1_EO_PL        = 0x800008120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL1_EO_PL        = 0x800008120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x800008120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x800008120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL1_O_PL            = 0x800080120701103Full;
//DUPS: 800080120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL1_O_PL          = 0x800080120701103Full;
//DUPS: 800080120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL1_O_PL          = 0x800080120801103Full;
//DUPS: 800080120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL1_O_PL          = 0x800080120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL1_O_PL        = 0x800080120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL1_O_PL        = 0x800080120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL1_O_PL        = 0x800080120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL1_O_PL        = 0x800080120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL1_O_PL         = 0x800080120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL1_O_PL         = 0x800080120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL1_O_PL        = 0x800080120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL1_O_PL        = 0x800080120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL2_EO_PL           = 0x800010120701103Full;
//DUPS: 800010120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL2_EO_PL         = 0x800010120701103Full;
//DUPS: 800010120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL2_EO_PL         = 0x800010120801103Full;
//DUPS: 800010120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL2_EO_PL         = 0x800010120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x800010120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x800010120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x800010120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x800010120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL2_EO_PL        = 0x800010120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL2_EO_PL        = 0x800010120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x800010120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x800010120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL2_O_PL            = 0x800088120701103Full;
//DUPS: 800088120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL2_O_PL          = 0x800088120701103Full;
//DUPS: 800088120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL2_O_PL          = 0x800088120801103Full;
//DUPS: 800088120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL2_O_PL          = 0x800088120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL2_O_PL        = 0x800088120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL2_O_PL        = 0x800088120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL2_O_PL        = 0x800088120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL2_O_PL        = 0x800088120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL2_O_PL         = 0x800088120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL2_O_PL         = 0x800088120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL2_O_PL        = 0x800088120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL2_O_PL        = 0x800088120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL3_EO_PL           = 0x800018120701103Full;
//DUPS: 800018120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL3_EO_PL         = 0x800018120701103Full;
//DUPS: 800018120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL3_EO_PL         = 0x800018120801103Full;
//DUPS: 800018120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL3_EO_PL         = 0x800018120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x800018120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x800018120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x800018120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x800018120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL3_EO_PL        = 0x800018120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL3_EO_PL        = 0x800018120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x800018120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x800018120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL3_O_PL            = 0x800090120701103Full;
//DUPS: 800090120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL3_O_PL          = 0x800090120701103Full;
//DUPS: 800090120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL3_O_PL          = 0x800090120801103Full;
//DUPS: 800090120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL3_O_PL          = 0x800090120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL3_O_PL        = 0x800090120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL3_O_PL        = 0x800090120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL3_O_PL        = 0x800090120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL3_O_PL        = 0x800090120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL3_O_PL         = 0x800090120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL3_O_PL         = 0x800090120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL3_O_PL        = 0x800090120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL3_O_PL        = 0x800090120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL4_EO_PL           = 0x800020120701103Full;
//DUPS: 800020120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL4_EO_PL         = 0x800020120701103Full;
//DUPS: 800020120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL4_EO_PL         = 0x800020120801103Full;
//DUPS: 800020120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL4_EO_PL         = 0x800020120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x800020120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x800020120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x800020120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x800020120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL4_EO_PL        = 0x800020120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL4_EO_PL        = 0x800020120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x800020120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x800020120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL4_O_PL            = 0x800098120701103Full;
//DUPS: 800098120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL4_O_PL          = 0x800098120701103Full;
//DUPS: 800098120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL4_O_PL          = 0x800098120801103Full;
//DUPS: 800098120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL4_O_PL          = 0x800098120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL4_O_PL        = 0x800098120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL4_O_PL        = 0x800098120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL4_O_PL        = 0x800098120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL4_O_PL        = 0x800098120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL4_O_PL         = 0x800098120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL4_O_PL         = 0x800098120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL4_O_PL        = 0x800098120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL4_O_PL        = 0x800098120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL5_EO_PL           = 0x800028120701103Full;
//DUPS: 800028120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL5_EO_PL         = 0x800028120701103Full;
//DUPS: 800028120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL5_EO_PL         = 0x800028120801103Full;
//DUPS: 800028120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL5_EO_PL         = 0x800028120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x800028120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x800028120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x800028120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x800028120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL5_EO_PL        = 0x800028120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL5_EO_PL        = 0x800028120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x800028120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x800028120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL5_O_PL            = 0x8000A0120701103Full;
//DUPS: 8000A0120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL5_O_PL          = 0x8000A0120701103Full;
//DUPS: 8000A0120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL5_O_PL          = 0x8000A0120801103Full;
//DUPS: 8000A0120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL5_O_PL          = 0x8000A0120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A0120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A0120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A0120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A0120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL5_O_PL         = 0x8000A0120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL5_O_PL         = 0x8000A0120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A0120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A0120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL6_EO_PL           = 0x800030120701103Full;
//DUPS: 800030120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL6_EO_PL         = 0x800030120701103Full;
//DUPS: 800030120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL6_EO_PL         = 0x800030120801103Full;
//DUPS: 800030120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL6_EO_PL         = 0x800030120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x800030120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x800030120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x800030120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x800030120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL6_EO_PL        = 0x800030120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL6_EO_PL        = 0x800030120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x800030120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x800030120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL6_O_PL            = 0x8000A8120701103Full;
//DUPS: 8000A8120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL6_O_PL          = 0x8000A8120701103Full;
//DUPS: 8000A8120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL6_O_PL          = 0x8000A8120801103Full;
//DUPS: 8000A8120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL6_O_PL          = 0x8000A8120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A8120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A8120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A8120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A8120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL6_O_PL         = 0x8000A8120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL6_O_PL         = 0x8000A8120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A8120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A8120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL7_EO_PL           = 0x800038120701103Full;
//DUPS: 800038120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL7_EO_PL         = 0x800038120701103Full;
//DUPS: 800038120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL7_EO_PL         = 0x800038120801103Full;
//DUPS: 800038120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL7_EO_PL         = 0x800038120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x800038120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x800038120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x800038120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x800038120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL7_EO_PL        = 0x800038120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL7_EO_PL        = 0x800038120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x800038120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x800038120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL7_O_PL            = 0x8000B0120701103Full;
//DUPS: 8000B0120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL7_O_PL          = 0x8000B0120701103Full;
//DUPS: 8000B0120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL7_O_PL          = 0x8000B0120801103Full;
//DUPS: 8000B0120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL7_O_PL          = 0x8000B0120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B0120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B0120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B0120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B0120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL7_O_PL         = 0x8000B0120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL7_O_PL         = 0x8000B0120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B0120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B0120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL8_EO_PL           = 0x800040120701103Full;
//DUPS: 800040120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL8_EO_PL         = 0x800040120701103Full;
//DUPS: 800040120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL8_EO_PL         = 0x800040120801103Full;
//DUPS: 800040120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL8_EO_PL         = 0x800040120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x800040120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x800040120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x800040120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x800040120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL8_EO_PL        = 0x800040120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL8_EO_PL        = 0x800040120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x800040120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x800040120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL8_O_PL            = 0x8000B8120701103Full;
//DUPS: 8000B8120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL8_O_PL          = 0x8000B8120701103Full;
//DUPS: 8000B8120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL8_O_PL          = 0x8000B8120801103Full;
//DUPS: 8000B8120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL8_O_PL          = 0x8000B8120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B8120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B8120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B8120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B8120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL8_O_PL         = 0x8000B8120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL8_O_PL         = 0x8000B8120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B8120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B8120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL9_O_PL            = 0x8000C0120701103Full;
//DUPS: 8000C0120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL9_O_PL          = 0x8000C0120701103Full;
//DUPS: 8000C0120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL9_O_PL          = 0x8000C0120801103Full;
//DUPS: 8000C0120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL9_O_PL          = 0x8000C0120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C0120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C0120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C0120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C0120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL9_O_PL         = 0x8000C0120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL9_O_PL         = 0x8000C0120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C0120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C0120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_DATA_DAC_SPARE_MODE_PL    = 0x800000120701103Full;
//DUPS: 800000120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000120701103Full;
//DUPS: 800000120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000120801103Full;
//DUPS: 800000120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_FIR_ERROR_INJECT_PL       = 0x800218120701103Full;
//DUPS: 800218120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_FIR_ERROR_INJECT_PL     = 0x800218120701103Full;
//DUPS: 800218120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_FIR_ERROR_INJECT_PL     = 0x800218120801103Full;
//DUPS: 800218120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_FIR_ERROR_INJECT_PL     = 0x800218120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x800218120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x800218120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x800218120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x800218120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_FIR_ERROR_INJECT_PL    = 0x800218120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_FIR_ERROR_INJECT_PL    = 0x800218120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x800218120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x800218120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_FIR_MASK_PL               = 0x800210120701103Full;
//DUPS: 800210120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_FIR_MASK_PL             = 0x800210120701103Full;
//DUPS: 800210120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_FIR_MASK_PL             = 0x800210120801103Full;
//DUPS: 800210120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_FIR_MASK_PL             = 0x800210120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_FIR_MASK_PL           = 0x800210120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_FIR_MASK_PL           = 0x800210120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_FIR_MASK_PL           = 0x800210120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_FIR_MASK_PL           = 0x800210120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_FIR_MASK_PL            = 0x800210120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_FIR_MASK_PL            = 0x800210120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_FIR_MASK_PL           = 0x800210120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_FIR_MASK_PL           = 0x800210120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_FIR_PL                    = 0x800208120701103Full;
//DUPS: 800208120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_FIR_PL                  = 0x800208120701103Full;
//DUPS: 800208120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_FIR_PL                  = 0x800208120801103Full;
//DUPS: 800208120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_FIR_PL                  = 0x800208120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_FIR_PL                = 0x800208120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_FIR_PL                = 0x800208120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_FIR_PL                = 0x800208120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_FIR_PL                = 0x800208120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_FIR_PL                 = 0x800208120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_FIR_PL                 = 0x800208120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_FIR_PL                = 0x800208120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_FIR_PL                = 0x800208120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE2_RX_SPARE_MODE_PL             = 0x800200120701103Full;
//DUPS: 800200120701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE2_RX_SPARE_MODE_PL           = 0x800200120701103Full;
//DUPS: 800200120701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE2_RX_SPARE_MODE_PL           = 0x800200120801103Full;
//DUPS: 800200120801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE2_RX_SPARE_MODE_PL           = 0x800200120701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE2_RX_SPARE_MODE_PL         = 0x800200120701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE2_RX_SPARE_MODE_PL         = 0x800200120701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE2_RX_SPARE_MODE_PL         = 0x800200120801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE2_RX_SPARE_MODE_PL         = 0x800200120801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE2_RX_SPARE_MODE_PL          = 0x800200120701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE2_RX_SPARE_MODE_PL          = 0x800200120701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE2_RX_SPARE_MODE_PL         = 0x800200120801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE2_RX_SPARE_MODE_PL         = 0x800200120801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL2_EO_PL           = 0x800248130701103Full;
//DUPS: 800248130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL2_EO_PL         = 0x800248130701103Full;
//DUPS: 800248130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL2_EO_PL         = 0x800248130801103Full;
//DUPS: 800248130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL2_EO_PL         = 0x800248130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x800248130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x800248130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x800248130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x800248130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL2_EO_PL        = 0x800248130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL2_EO_PL        = 0x800248130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x800248130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x800248130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL3_EO_PL           = 0x800250130701103Full;
//DUPS: 800250130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL3_EO_PL         = 0x800250130701103Full;
//DUPS: 800250130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL3_EO_PL         = 0x800250130801103Full;
//DUPS: 800250130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL3_EO_PL         = 0x800250130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x800250130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x800250130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x800250130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x800250130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL3_EO_PL        = 0x800250130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL3_EO_PL        = 0x800250130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x800250130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x800250130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL4_EO_PL           = 0x800258130701103Full;
//DUPS: 800258130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL4_EO_PL         = 0x800258130701103Full;
//DUPS: 800258130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL4_EO_PL         = 0x800258130801103Full;
//DUPS: 800258130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL4_EO_PL         = 0x800258130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x800258130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x800258130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x800258130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x800258130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL4_EO_PL        = 0x800258130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL4_EO_PL        = 0x800258130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x800258130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x800258130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL5_EO_PL           = 0x800260130701103Full;
//DUPS: 800260130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL5_EO_PL         = 0x800260130701103Full;
//DUPS: 800260130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL5_EO_PL         = 0x800260130801103Full;
//DUPS: 800260130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL5_EO_PL         = 0x800260130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x800260130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x800260130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x800260130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x800260130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL5_EO_PL        = 0x800260130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL5_EO_PL        = 0x800260130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x800260130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x800260130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_BIT_CNTLX1_EO_PL          = 0x800240130701103Full;
//DUPS: 800240130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_BIT_CNTLX1_EO_PL        = 0x800240130701103Full;
//DUPS: 800240130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_BIT_CNTLX1_EO_PL        = 0x800240130801103Full;
//DUPS: 800240130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_BIT_CNTLX1_EO_PL        = 0x800240130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x800240130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x800240130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x800240130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x800240130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_BIT_CNTLX1_EO_PL       = 0x800240130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_BIT_CNTLX1_EO_PL       = 0x800240130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x800240130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x800240130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_BIT_MODE1_EO_PL           = 0x800220130701103Full;
//DUPS: 800220130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_BIT_MODE1_EO_PL         = 0x800220130701103Full;
//DUPS: 800220130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_BIT_MODE1_EO_PL         = 0x800220130801103Full;
//DUPS: 800220130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_BIT_MODE1_EO_PL         = 0x800220130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_BIT_MODE1_EO_PL       = 0x800220130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_BIT_MODE1_EO_PL       = 0x800220130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_BIT_MODE1_EO_PL       = 0x800220130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_BIT_MODE1_EO_PL       = 0x800220130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_BIT_MODE1_EO_PL        = 0x800220130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_BIT_MODE1_EO_PL        = 0x800220130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_BIT_MODE1_EO_PL       = 0x800220130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_BIT_MODE1_EO_PL       = 0x800220130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_BIT_MODE2_EO_PL           = 0x800228130701103Full;
//DUPS: 800228130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_BIT_MODE2_EO_PL         = 0x800228130701103Full;
//DUPS: 800228130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_BIT_MODE2_EO_PL         = 0x800228130801103Full;
//DUPS: 800228130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_BIT_MODE2_EO_PL         = 0x800228130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_BIT_MODE2_EO_PL       = 0x800228130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_BIT_MODE2_EO_PL       = 0x800228130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_BIT_MODE2_EO_PL       = 0x800228130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_BIT_MODE2_EO_PL       = 0x800228130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_BIT_MODE2_EO_PL        = 0x800228130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_BIT_MODE2_EO_PL        = 0x800228130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_BIT_MODE2_EO_PL       = 0x800228130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_BIT_MODE2_EO_PL       = 0x800228130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_BIT_MODE3_EO_PL           = 0x800230130701103Full;
//DUPS: 800230130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_BIT_MODE3_EO_PL         = 0x800230130701103Full;
//DUPS: 800230130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_BIT_MODE3_EO_PL         = 0x800230130801103Full;
//DUPS: 800230130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_BIT_MODE3_EO_PL         = 0x800230130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_BIT_MODE3_EO_PL       = 0x800230130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_BIT_MODE3_EO_PL       = 0x800230130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_BIT_MODE3_EO_PL       = 0x800230130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_BIT_MODE3_EO_PL       = 0x800230130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_BIT_MODE3_EO_PL        = 0x800230130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_BIT_MODE3_EO_PL        = 0x800230130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_BIT_MODE3_EO_PL       = 0x800230130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_BIT_MODE3_EO_PL       = 0x800230130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_BIT_MODE4_EO_PL           = 0x800238130701103Full;
//DUPS: 800238130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_BIT_MODE4_EO_PL         = 0x800238130701103Full;
//DUPS: 800238130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_BIT_MODE4_EO_PL         = 0x800238130801103Full;
//DUPS: 800238130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_BIT_MODE4_EO_PL         = 0x800238130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_BIT_MODE4_EO_PL       = 0x800238130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_BIT_MODE4_EO_PL       = 0x800238130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_BIT_MODE4_EO_PL       = 0x800238130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_BIT_MODE4_EO_PL       = 0x800238130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_BIT_MODE4_EO_PL        = 0x800238130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_BIT_MODE4_EO_PL        = 0x800238130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_BIT_MODE4_EO_PL       = 0x800238130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_BIT_MODE4_EO_PL       = 0x800238130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_BIT_STAT1_EO_PL           = 0x800268130701103Full;
//DUPS: 800268130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_BIT_STAT1_EO_PL         = 0x800268130701103Full;
//DUPS: 800268130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_BIT_STAT1_EO_PL         = 0x800268130801103Full;
//DUPS: 800268130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_BIT_STAT1_EO_PL         = 0x800268130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_BIT_STAT1_EO_PL       = 0x800268130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_BIT_STAT1_EO_PL       = 0x800268130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_BIT_STAT1_EO_PL       = 0x800268130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_BIT_STAT1_EO_PL       = 0x800268130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_BIT_STAT1_EO_PL        = 0x800268130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_BIT_STAT1_EO_PL        = 0x800268130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_BIT_STAT1_EO_PL       = 0x800268130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_BIT_STAT1_EO_PL       = 0x800268130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_BIT_STAT1_O_PL            = 0x800318130701103Full;
//DUPS: 800318130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_BIT_STAT1_O_PL          = 0x800318130701103Full;
//DUPS: 800318130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_BIT_STAT1_O_PL          = 0x800318130801103Full;
//DUPS: 800318130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_BIT_STAT1_O_PL          = 0x800318130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_BIT_STAT1_O_PL        = 0x800318130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_BIT_STAT1_O_PL        = 0x800318130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_BIT_STAT1_O_PL        = 0x800318130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_BIT_STAT1_O_PL        = 0x800318130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_BIT_STAT1_O_PL         = 0x800318130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_BIT_STAT1_O_PL         = 0x800318130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_BIT_STAT1_O_PL        = 0x800318130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_BIT_STAT1_O_PL        = 0x800318130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_BIT_STAT2_EO_PL           = 0x800270130701103Full;
//DUPS: 800270130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_BIT_STAT2_EO_PL         = 0x800270130701103Full;
//DUPS: 800270130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_BIT_STAT2_EO_PL         = 0x800270130801103Full;
//DUPS: 800270130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_BIT_STAT2_EO_PL         = 0x800270130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_BIT_STAT2_EO_PL       = 0x800270130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_BIT_STAT2_EO_PL       = 0x800270130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_BIT_STAT2_EO_PL       = 0x800270130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_BIT_STAT2_EO_PL       = 0x800270130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_BIT_STAT2_EO_PL        = 0x800270130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_BIT_STAT2_EO_PL        = 0x800270130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_BIT_STAT2_EO_PL       = 0x800270130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_BIT_STAT2_EO_PL       = 0x800270130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_BIT_STAT3_EO_PL           = 0x800278130701103Full;
//DUPS: 800278130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_BIT_STAT3_EO_PL         = 0x800278130701103Full;
//DUPS: 800278130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_BIT_STAT3_EO_PL         = 0x800278130801103Full;
//DUPS: 800278130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_BIT_STAT3_EO_PL         = 0x800278130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_BIT_STAT3_EO_PL       = 0x800278130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_BIT_STAT3_EO_PL       = 0x800278130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_BIT_STAT3_EO_PL       = 0x800278130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_BIT_STAT3_EO_PL       = 0x800278130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_BIT_STAT3_EO_PL        = 0x800278130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_BIT_STAT3_EO_PL        = 0x800278130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_BIT_STAT3_EO_PL       = 0x800278130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_BIT_STAT3_EO_PL       = 0x800278130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_BIT_STAT4_EO_PL           = 0x800280130701103Full;
//DUPS: 800280130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_BIT_STAT4_EO_PL         = 0x800280130701103Full;
//DUPS: 800280130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_BIT_STAT4_EO_PL         = 0x800280130801103Full;
//DUPS: 800280130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_BIT_STAT4_EO_PL         = 0x800280130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_BIT_STAT4_EO_PL       = 0x800280130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_BIT_STAT4_EO_PL       = 0x800280130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_BIT_STAT4_EO_PL       = 0x800280130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_BIT_STAT4_EO_PL       = 0x800280130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_BIT_STAT4_EO_PL        = 0x800280130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_BIT_STAT4_EO_PL        = 0x800280130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_BIT_STAT4_EO_PL       = 0x800280130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_BIT_STAT4_EO_PL       = 0x800280130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL10_O_PL           = 0x8000C8130701103Full;
//DUPS: 8000C8130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL10_O_PL         = 0x8000C8130701103Full;
//DUPS: 8000C8130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL10_O_PL         = 0x8000C8130801103Full;
//DUPS: 8000C8130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL10_O_PL         = 0x8000C8130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C8130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C8130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C8130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C8130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL10_O_PL        = 0x8000C8130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL10_O_PL        = 0x8000C8130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C8130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C8130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL1_EO_PL           = 0x800008130701103Full;
//DUPS: 800008130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL1_EO_PL         = 0x800008130701103Full;
//DUPS: 800008130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL1_EO_PL         = 0x800008130801103Full;
//DUPS: 800008130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL1_EO_PL         = 0x800008130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x800008130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x800008130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x800008130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x800008130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL1_EO_PL        = 0x800008130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL1_EO_PL        = 0x800008130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x800008130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x800008130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL1_O_PL            = 0x800080130701103Full;
//DUPS: 800080130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL1_O_PL          = 0x800080130701103Full;
//DUPS: 800080130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL1_O_PL          = 0x800080130801103Full;
//DUPS: 800080130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL1_O_PL          = 0x800080130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL1_O_PL        = 0x800080130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL1_O_PL        = 0x800080130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL1_O_PL        = 0x800080130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL1_O_PL        = 0x800080130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL1_O_PL         = 0x800080130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL1_O_PL         = 0x800080130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL1_O_PL        = 0x800080130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL1_O_PL        = 0x800080130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL2_EO_PL           = 0x800010130701103Full;
//DUPS: 800010130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL2_EO_PL         = 0x800010130701103Full;
//DUPS: 800010130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL2_EO_PL         = 0x800010130801103Full;
//DUPS: 800010130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL2_EO_PL         = 0x800010130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x800010130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x800010130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x800010130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x800010130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL2_EO_PL        = 0x800010130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL2_EO_PL        = 0x800010130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x800010130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x800010130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL2_O_PL            = 0x800088130701103Full;
//DUPS: 800088130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL2_O_PL          = 0x800088130701103Full;
//DUPS: 800088130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL2_O_PL          = 0x800088130801103Full;
//DUPS: 800088130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL2_O_PL          = 0x800088130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL2_O_PL        = 0x800088130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL2_O_PL        = 0x800088130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL2_O_PL        = 0x800088130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL2_O_PL        = 0x800088130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL2_O_PL         = 0x800088130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL2_O_PL         = 0x800088130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL2_O_PL        = 0x800088130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL2_O_PL        = 0x800088130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL3_EO_PL           = 0x800018130701103Full;
//DUPS: 800018130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL3_EO_PL         = 0x800018130701103Full;
//DUPS: 800018130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL3_EO_PL         = 0x800018130801103Full;
//DUPS: 800018130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL3_EO_PL         = 0x800018130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x800018130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x800018130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x800018130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x800018130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL3_EO_PL        = 0x800018130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL3_EO_PL        = 0x800018130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x800018130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x800018130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL3_O_PL            = 0x800090130701103Full;
//DUPS: 800090130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL3_O_PL          = 0x800090130701103Full;
//DUPS: 800090130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL3_O_PL          = 0x800090130801103Full;
//DUPS: 800090130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL3_O_PL          = 0x800090130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL3_O_PL        = 0x800090130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL3_O_PL        = 0x800090130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL3_O_PL        = 0x800090130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL3_O_PL        = 0x800090130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL3_O_PL         = 0x800090130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL3_O_PL         = 0x800090130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL3_O_PL        = 0x800090130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL3_O_PL        = 0x800090130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL4_EO_PL           = 0x800020130701103Full;
//DUPS: 800020130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL4_EO_PL         = 0x800020130701103Full;
//DUPS: 800020130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL4_EO_PL         = 0x800020130801103Full;
//DUPS: 800020130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL4_EO_PL         = 0x800020130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x800020130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x800020130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x800020130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x800020130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL4_EO_PL        = 0x800020130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL4_EO_PL        = 0x800020130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x800020130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x800020130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL4_O_PL            = 0x800098130701103Full;
//DUPS: 800098130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL4_O_PL          = 0x800098130701103Full;
//DUPS: 800098130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL4_O_PL          = 0x800098130801103Full;
//DUPS: 800098130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL4_O_PL          = 0x800098130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL4_O_PL        = 0x800098130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL4_O_PL        = 0x800098130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL4_O_PL        = 0x800098130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL4_O_PL        = 0x800098130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL4_O_PL         = 0x800098130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL4_O_PL         = 0x800098130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL4_O_PL        = 0x800098130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL4_O_PL        = 0x800098130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL5_EO_PL           = 0x800028130701103Full;
//DUPS: 800028130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL5_EO_PL         = 0x800028130701103Full;
//DUPS: 800028130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL5_EO_PL         = 0x800028130801103Full;
//DUPS: 800028130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL5_EO_PL         = 0x800028130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x800028130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x800028130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x800028130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x800028130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL5_EO_PL        = 0x800028130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL5_EO_PL        = 0x800028130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x800028130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x800028130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL5_O_PL            = 0x8000A0130701103Full;
//DUPS: 8000A0130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL5_O_PL          = 0x8000A0130701103Full;
//DUPS: 8000A0130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL5_O_PL          = 0x8000A0130801103Full;
//DUPS: 8000A0130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL5_O_PL          = 0x8000A0130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A0130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A0130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A0130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A0130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL5_O_PL         = 0x8000A0130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL5_O_PL         = 0x8000A0130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A0130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A0130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL6_EO_PL           = 0x800030130701103Full;
//DUPS: 800030130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL6_EO_PL         = 0x800030130701103Full;
//DUPS: 800030130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL6_EO_PL         = 0x800030130801103Full;
//DUPS: 800030130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL6_EO_PL         = 0x800030130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x800030130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x800030130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x800030130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x800030130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL6_EO_PL        = 0x800030130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL6_EO_PL        = 0x800030130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x800030130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x800030130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL6_O_PL            = 0x8000A8130701103Full;
//DUPS: 8000A8130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL6_O_PL          = 0x8000A8130701103Full;
//DUPS: 8000A8130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL6_O_PL          = 0x8000A8130801103Full;
//DUPS: 8000A8130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL6_O_PL          = 0x8000A8130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A8130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A8130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A8130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A8130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL6_O_PL         = 0x8000A8130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL6_O_PL         = 0x8000A8130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A8130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A8130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL7_EO_PL           = 0x800038130701103Full;
//DUPS: 800038130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL7_EO_PL         = 0x800038130701103Full;
//DUPS: 800038130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL7_EO_PL         = 0x800038130801103Full;
//DUPS: 800038130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL7_EO_PL         = 0x800038130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x800038130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x800038130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x800038130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x800038130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL7_EO_PL        = 0x800038130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL7_EO_PL        = 0x800038130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x800038130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x800038130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL7_O_PL            = 0x8000B0130701103Full;
//DUPS: 8000B0130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL7_O_PL          = 0x8000B0130701103Full;
//DUPS: 8000B0130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL7_O_PL          = 0x8000B0130801103Full;
//DUPS: 8000B0130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL7_O_PL          = 0x8000B0130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B0130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B0130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B0130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B0130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL7_O_PL         = 0x8000B0130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL7_O_PL         = 0x8000B0130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B0130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B0130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL8_EO_PL           = 0x800040130701103Full;
//DUPS: 800040130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL8_EO_PL         = 0x800040130701103Full;
//DUPS: 800040130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL8_EO_PL         = 0x800040130801103Full;
//DUPS: 800040130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL8_EO_PL         = 0x800040130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x800040130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x800040130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x800040130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x800040130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL8_EO_PL        = 0x800040130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL8_EO_PL        = 0x800040130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x800040130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x800040130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL8_O_PL            = 0x8000B8130701103Full;
//DUPS: 8000B8130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL8_O_PL          = 0x8000B8130701103Full;
//DUPS: 8000B8130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL8_O_PL          = 0x8000B8130801103Full;
//DUPS: 8000B8130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL8_O_PL          = 0x8000B8130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B8130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B8130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B8130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B8130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL8_O_PL         = 0x8000B8130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL8_O_PL         = 0x8000B8130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B8130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B8130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL9_O_PL            = 0x8000C0130701103Full;
//DUPS: 8000C0130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL9_O_PL          = 0x8000C0130701103Full;
//DUPS: 8000C0130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL9_O_PL          = 0x8000C0130801103Full;
//DUPS: 8000C0130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL9_O_PL          = 0x8000C0130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C0130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C0130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C0130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C0130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL9_O_PL         = 0x8000C0130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL9_O_PL         = 0x8000C0130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C0130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C0130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_DATA_DAC_SPARE_MODE_PL    = 0x800000130701103Full;
//DUPS: 800000130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000130701103Full;
//DUPS: 800000130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000130801103Full;
//DUPS: 800000130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_FIR_ERROR_INJECT_PL       = 0x800218130701103Full;
//DUPS: 800218130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_FIR_ERROR_INJECT_PL     = 0x800218130701103Full;
//DUPS: 800218130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_FIR_ERROR_INJECT_PL     = 0x800218130801103Full;
//DUPS: 800218130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_FIR_ERROR_INJECT_PL     = 0x800218130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x800218130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x800218130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x800218130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x800218130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_FIR_ERROR_INJECT_PL    = 0x800218130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_FIR_ERROR_INJECT_PL    = 0x800218130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x800218130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x800218130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_FIR_MASK_PL               = 0x800210130701103Full;
//DUPS: 800210130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_FIR_MASK_PL             = 0x800210130701103Full;
//DUPS: 800210130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_FIR_MASK_PL             = 0x800210130801103Full;
//DUPS: 800210130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_FIR_MASK_PL             = 0x800210130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_FIR_MASK_PL           = 0x800210130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_FIR_MASK_PL           = 0x800210130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_FIR_MASK_PL           = 0x800210130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_FIR_MASK_PL           = 0x800210130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_FIR_MASK_PL            = 0x800210130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_FIR_MASK_PL            = 0x800210130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_FIR_MASK_PL           = 0x800210130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_FIR_MASK_PL           = 0x800210130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_FIR_PL                    = 0x800208130701103Full;
//DUPS: 800208130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_FIR_PL                  = 0x800208130701103Full;
//DUPS: 800208130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_FIR_PL                  = 0x800208130801103Full;
//DUPS: 800208130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_FIR_PL                  = 0x800208130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_FIR_PL                = 0x800208130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_FIR_PL                = 0x800208130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_FIR_PL                = 0x800208130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_FIR_PL                = 0x800208130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_FIR_PL                 = 0x800208130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_FIR_PL                 = 0x800208130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_FIR_PL                = 0x800208130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_FIR_PL                = 0x800208130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS4_SLICE3_RX_SPARE_MODE_PL             = 0x800200130701103Full;
//DUPS: 800200130701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS4_SLICE3_RX_SPARE_MODE_PL           = 0x800200130701103Full;
//DUPS: 800200130701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS4_SLICE3_RX_SPARE_MODE_PL           = 0x800200130801103Full;
//DUPS: 800200130801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS4_SLICE3_RX_SPARE_MODE_PL           = 0x800200130701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS4_SLICE3_RX_SPARE_MODE_PL         = 0x800200130701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS4_SLICE3_RX_SPARE_MODE_PL         = 0x800200130701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS4_SLICE3_RX_SPARE_MODE_PL         = 0x800200130801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS4_SLICE3_RX_SPARE_MODE_PL         = 0x800200130801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS4_SLICE3_RX_SPARE_MODE_PL          = 0x800200130701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS4_SLICE3_RX_SPARE_MODE_PL          = 0x800200130701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS4_SLICE3_RX_SPARE_MODE_PL         = 0x800200130801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS4_SLICE3_RX_SPARE_MODE_PL         = 0x800200130801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL2_EO_PL           = 0x800248140701103Full;
//DUPS: 800248140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL2_EO_PL         = 0x800248140701103Full;
//DUPS: 800248140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL2_EO_PL         = 0x800248140801103Full;
//DUPS: 800248140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL2_EO_PL         = 0x800248140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x800248140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x800248140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x800248140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x800248140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL2_EO_PL        = 0x800248140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL2_EO_PL        = 0x800248140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x800248140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL2_EO_PL       = 0x800248140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL3_EO_PL           = 0x800250140701103Full;
//DUPS: 800250140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL3_EO_PL         = 0x800250140701103Full;
//DUPS: 800250140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL3_EO_PL         = 0x800250140801103Full;
//DUPS: 800250140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL3_EO_PL         = 0x800250140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x800250140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x800250140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x800250140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x800250140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL3_EO_PL        = 0x800250140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL3_EO_PL        = 0x800250140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x800250140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL3_EO_PL       = 0x800250140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL4_EO_PL           = 0x800258140701103Full;
//DUPS: 800258140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL4_EO_PL         = 0x800258140701103Full;
//DUPS: 800258140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL4_EO_PL         = 0x800258140801103Full;
//DUPS: 800258140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL4_EO_PL         = 0x800258140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x800258140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x800258140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x800258140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x800258140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL4_EO_PL        = 0x800258140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL4_EO_PL        = 0x800258140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x800258140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL4_EO_PL       = 0x800258140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL5_EO_PL           = 0x800260140701103Full;
//DUPS: 800260140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL5_EO_PL         = 0x800260140701103Full;
//DUPS: 800260140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL5_EO_PL         = 0x800260140801103Full;
//DUPS: 800260140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL5_EO_PL         = 0x800260140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x800260140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x800260140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x800260140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x800260140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL5_EO_PL        = 0x800260140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL5_EO_PL        = 0x800260140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x800260140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_BIT_CNTL5_EO_PL       = 0x800260140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_BIT_CNTLX1_EO_PL          = 0x800240140701103Full;
//DUPS: 800240140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_BIT_CNTLX1_EO_PL        = 0x800240140701103Full;
//DUPS: 800240140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_BIT_CNTLX1_EO_PL        = 0x800240140801103Full;
//DUPS: 800240140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_BIT_CNTLX1_EO_PL        = 0x800240140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x800240140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x800240140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x800240140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x800240140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_BIT_CNTLX1_EO_PL       = 0x800240140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_BIT_CNTLX1_EO_PL       = 0x800240140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x800240140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_BIT_CNTLX1_EO_PL      = 0x800240140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_BIT_MODE1_EO_PL           = 0x800220140701103Full;
//DUPS: 800220140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_BIT_MODE1_EO_PL         = 0x800220140701103Full;
//DUPS: 800220140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_BIT_MODE1_EO_PL         = 0x800220140801103Full;
//DUPS: 800220140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_BIT_MODE1_EO_PL         = 0x800220140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_BIT_MODE1_EO_PL       = 0x800220140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_BIT_MODE1_EO_PL       = 0x800220140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_BIT_MODE1_EO_PL       = 0x800220140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_BIT_MODE1_EO_PL       = 0x800220140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_BIT_MODE1_EO_PL        = 0x800220140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_BIT_MODE1_EO_PL        = 0x800220140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_BIT_MODE1_EO_PL       = 0x800220140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_BIT_MODE1_EO_PL       = 0x800220140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_BIT_MODE2_EO_PL           = 0x800228140701103Full;
//DUPS: 800228140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_BIT_MODE2_EO_PL         = 0x800228140701103Full;
//DUPS: 800228140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_BIT_MODE2_EO_PL         = 0x800228140801103Full;
//DUPS: 800228140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_BIT_MODE2_EO_PL         = 0x800228140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_BIT_MODE2_EO_PL       = 0x800228140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_BIT_MODE2_EO_PL       = 0x800228140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_BIT_MODE2_EO_PL       = 0x800228140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_BIT_MODE2_EO_PL       = 0x800228140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_BIT_MODE2_EO_PL        = 0x800228140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_BIT_MODE2_EO_PL        = 0x800228140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_BIT_MODE2_EO_PL       = 0x800228140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_BIT_MODE2_EO_PL       = 0x800228140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_BIT_MODE3_EO_PL           = 0x800230140701103Full;
//DUPS: 800230140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_BIT_MODE3_EO_PL         = 0x800230140701103Full;
//DUPS: 800230140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_BIT_MODE3_EO_PL         = 0x800230140801103Full;
//DUPS: 800230140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_BIT_MODE3_EO_PL         = 0x800230140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_BIT_MODE3_EO_PL       = 0x800230140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_BIT_MODE3_EO_PL       = 0x800230140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_BIT_MODE3_EO_PL       = 0x800230140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_BIT_MODE3_EO_PL       = 0x800230140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_BIT_MODE3_EO_PL        = 0x800230140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_BIT_MODE3_EO_PL        = 0x800230140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_BIT_MODE3_EO_PL       = 0x800230140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_BIT_MODE3_EO_PL       = 0x800230140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_BIT_MODE4_EO_PL           = 0x800238140701103Full;
//DUPS: 800238140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_BIT_MODE4_EO_PL         = 0x800238140701103Full;
//DUPS: 800238140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_BIT_MODE4_EO_PL         = 0x800238140801103Full;
//DUPS: 800238140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_BIT_MODE4_EO_PL         = 0x800238140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_BIT_MODE4_EO_PL       = 0x800238140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_BIT_MODE4_EO_PL       = 0x800238140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_BIT_MODE4_EO_PL       = 0x800238140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_BIT_MODE4_EO_PL       = 0x800238140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_BIT_MODE4_EO_PL        = 0x800238140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_BIT_MODE4_EO_PL        = 0x800238140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_BIT_MODE4_EO_PL       = 0x800238140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_BIT_MODE4_EO_PL       = 0x800238140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_BIT_STAT1_EO_PL           = 0x800268140701103Full;
//DUPS: 800268140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_BIT_STAT1_EO_PL         = 0x800268140701103Full;
//DUPS: 800268140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_BIT_STAT1_EO_PL         = 0x800268140801103Full;
//DUPS: 800268140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_BIT_STAT1_EO_PL         = 0x800268140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_BIT_STAT1_EO_PL       = 0x800268140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_BIT_STAT1_EO_PL       = 0x800268140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_BIT_STAT1_EO_PL       = 0x800268140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_BIT_STAT1_EO_PL       = 0x800268140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_BIT_STAT1_EO_PL        = 0x800268140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_BIT_STAT1_EO_PL        = 0x800268140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_BIT_STAT1_EO_PL       = 0x800268140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_BIT_STAT1_EO_PL       = 0x800268140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_BIT_STAT1_O_PL            = 0x800318140701103Full;
//DUPS: 800318140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_BIT_STAT1_O_PL          = 0x800318140701103Full;
//DUPS: 800318140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_BIT_STAT1_O_PL          = 0x800318140801103Full;
//DUPS: 800318140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_BIT_STAT1_O_PL          = 0x800318140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_BIT_STAT1_O_PL        = 0x800318140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_BIT_STAT1_O_PL        = 0x800318140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_BIT_STAT1_O_PL        = 0x800318140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_BIT_STAT1_O_PL        = 0x800318140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_BIT_STAT1_O_PL         = 0x800318140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_BIT_STAT1_O_PL         = 0x800318140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_BIT_STAT1_O_PL        = 0x800318140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_BIT_STAT1_O_PL        = 0x800318140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_BIT_STAT2_EO_PL           = 0x800270140701103Full;
//DUPS: 800270140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_BIT_STAT2_EO_PL         = 0x800270140701103Full;
//DUPS: 800270140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_BIT_STAT2_EO_PL         = 0x800270140801103Full;
//DUPS: 800270140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_BIT_STAT2_EO_PL         = 0x800270140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_BIT_STAT2_EO_PL       = 0x800270140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_BIT_STAT2_EO_PL       = 0x800270140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_BIT_STAT2_EO_PL       = 0x800270140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_BIT_STAT2_EO_PL       = 0x800270140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_BIT_STAT2_EO_PL        = 0x800270140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_BIT_STAT2_EO_PL        = 0x800270140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_BIT_STAT2_EO_PL       = 0x800270140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_BIT_STAT2_EO_PL       = 0x800270140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_BIT_STAT3_EO_PL           = 0x800278140701103Full;
//DUPS: 800278140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_BIT_STAT3_EO_PL         = 0x800278140701103Full;
//DUPS: 800278140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_BIT_STAT3_EO_PL         = 0x800278140801103Full;
//DUPS: 800278140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_BIT_STAT3_EO_PL         = 0x800278140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_BIT_STAT3_EO_PL       = 0x800278140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_BIT_STAT3_EO_PL       = 0x800278140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_BIT_STAT3_EO_PL       = 0x800278140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_BIT_STAT3_EO_PL       = 0x800278140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_BIT_STAT3_EO_PL        = 0x800278140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_BIT_STAT3_EO_PL        = 0x800278140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_BIT_STAT3_EO_PL       = 0x800278140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_BIT_STAT3_EO_PL       = 0x800278140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_BIT_STAT4_EO_PL           = 0x800280140701103Full;
//DUPS: 800280140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_BIT_STAT4_EO_PL         = 0x800280140701103Full;
//DUPS: 800280140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_BIT_STAT4_EO_PL         = 0x800280140801103Full;
//DUPS: 800280140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_BIT_STAT4_EO_PL         = 0x800280140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_BIT_STAT4_EO_PL       = 0x800280140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_BIT_STAT4_EO_PL       = 0x800280140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_BIT_STAT4_EO_PL       = 0x800280140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_BIT_STAT4_EO_PL       = 0x800280140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_BIT_STAT4_EO_PL        = 0x800280140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_BIT_STAT4_EO_PL        = 0x800280140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_BIT_STAT4_EO_PL       = 0x800280140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_BIT_STAT4_EO_PL       = 0x800280140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL10_O_PL           = 0x8000C8140701103Full;
//DUPS: 8000C8140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL10_O_PL         = 0x8000C8140701103Full;
//DUPS: 8000C8140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL10_O_PL         = 0x8000C8140801103Full;
//DUPS: 8000C8140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL10_O_PL         = 0x8000C8140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C8140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C8140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C8140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C8140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL10_O_PL        = 0x8000C8140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL10_O_PL        = 0x8000C8140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C8140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL10_O_PL       = 0x8000C8140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL1_EO_PL           = 0x800008140701103Full;
//DUPS: 800008140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL1_EO_PL         = 0x800008140701103Full;
//DUPS: 800008140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL1_EO_PL         = 0x800008140801103Full;
//DUPS: 800008140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL1_EO_PL         = 0x800008140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x800008140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x800008140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x800008140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x800008140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL1_EO_PL        = 0x800008140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL1_EO_PL        = 0x800008140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x800008140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL1_EO_PL       = 0x800008140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL1_O_PL            = 0x800080140701103Full;
//DUPS: 800080140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL1_O_PL          = 0x800080140701103Full;
//DUPS: 800080140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL1_O_PL          = 0x800080140801103Full;
//DUPS: 800080140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL1_O_PL          = 0x800080140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL1_O_PL        = 0x800080140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL1_O_PL        = 0x800080140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL1_O_PL        = 0x800080140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL1_O_PL        = 0x800080140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL1_O_PL         = 0x800080140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL1_O_PL         = 0x800080140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL1_O_PL        = 0x800080140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL1_O_PL        = 0x800080140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL2_EO_PL           = 0x800010140701103Full;
//DUPS: 800010140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL2_EO_PL         = 0x800010140701103Full;
//DUPS: 800010140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL2_EO_PL         = 0x800010140801103Full;
//DUPS: 800010140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL2_EO_PL         = 0x800010140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x800010140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x800010140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x800010140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x800010140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL2_EO_PL        = 0x800010140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL2_EO_PL        = 0x800010140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x800010140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL2_EO_PL       = 0x800010140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL2_O_PL            = 0x800088140701103Full;
//DUPS: 800088140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL2_O_PL          = 0x800088140701103Full;
//DUPS: 800088140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL2_O_PL          = 0x800088140801103Full;
//DUPS: 800088140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL2_O_PL          = 0x800088140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL2_O_PL        = 0x800088140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL2_O_PL        = 0x800088140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL2_O_PL        = 0x800088140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL2_O_PL        = 0x800088140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL2_O_PL         = 0x800088140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL2_O_PL         = 0x800088140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL2_O_PL        = 0x800088140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL2_O_PL        = 0x800088140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL3_EO_PL           = 0x800018140701103Full;
//DUPS: 800018140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL3_EO_PL         = 0x800018140701103Full;
//DUPS: 800018140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL3_EO_PL         = 0x800018140801103Full;
//DUPS: 800018140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL3_EO_PL         = 0x800018140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x800018140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x800018140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x800018140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x800018140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL3_EO_PL        = 0x800018140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL3_EO_PL        = 0x800018140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x800018140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL3_EO_PL       = 0x800018140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL3_O_PL            = 0x800090140701103Full;
//DUPS: 800090140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL3_O_PL          = 0x800090140701103Full;
//DUPS: 800090140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL3_O_PL          = 0x800090140801103Full;
//DUPS: 800090140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL3_O_PL          = 0x800090140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL3_O_PL        = 0x800090140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL3_O_PL        = 0x800090140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL3_O_PL        = 0x800090140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL3_O_PL        = 0x800090140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL3_O_PL         = 0x800090140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL3_O_PL         = 0x800090140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL3_O_PL        = 0x800090140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL3_O_PL        = 0x800090140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL4_EO_PL           = 0x800020140701103Full;
//DUPS: 800020140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL4_EO_PL         = 0x800020140701103Full;
//DUPS: 800020140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL4_EO_PL         = 0x800020140801103Full;
//DUPS: 800020140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL4_EO_PL         = 0x800020140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x800020140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x800020140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x800020140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x800020140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL4_EO_PL        = 0x800020140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL4_EO_PL        = 0x800020140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x800020140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL4_EO_PL       = 0x800020140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL4_O_PL            = 0x800098140701103Full;
//DUPS: 800098140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL4_O_PL          = 0x800098140701103Full;
//DUPS: 800098140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL4_O_PL          = 0x800098140801103Full;
//DUPS: 800098140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL4_O_PL          = 0x800098140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL4_O_PL        = 0x800098140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL4_O_PL        = 0x800098140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL4_O_PL        = 0x800098140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL4_O_PL        = 0x800098140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL4_O_PL         = 0x800098140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL4_O_PL         = 0x800098140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL4_O_PL        = 0x800098140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL4_O_PL        = 0x800098140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL5_EO_PL           = 0x800028140701103Full;
//DUPS: 800028140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL5_EO_PL         = 0x800028140701103Full;
//DUPS: 800028140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL5_EO_PL         = 0x800028140801103Full;
//DUPS: 800028140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL5_EO_PL         = 0x800028140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x800028140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x800028140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x800028140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x800028140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL5_EO_PL        = 0x800028140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL5_EO_PL        = 0x800028140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x800028140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL5_EO_PL       = 0x800028140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL5_O_PL            = 0x8000A0140701103Full;
//DUPS: 8000A0140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL5_O_PL          = 0x8000A0140701103Full;
//DUPS: 8000A0140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL5_O_PL          = 0x8000A0140801103Full;
//DUPS: 8000A0140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL5_O_PL          = 0x8000A0140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A0140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A0140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A0140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A0140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL5_O_PL         = 0x8000A0140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL5_O_PL         = 0x8000A0140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A0140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL5_O_PL        = 0x8000A0140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL6_EO_PL           = 0x800030140701103Full;
//DUPS: 800030140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL6_EO_PL         = 0x800030140701103Full;
//DUPS: 800030140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL6_EO_PL         = 0x800030140801103Full;
//DUPS: 800030140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL6_EO_PL         = 0x800030140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x800030140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x800030140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x800030140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x800030140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL6_EO_PL        = 0x800030140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL6_EO_PL        = 0x800030140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x800030140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL6_EO_PL       = 0x800030140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL6_O_PL            = 0x8000A8140701103Full;
//DUPS: 8000A8140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL6_O_PL          = 0x8000A8140701103Full;
//DUPS: 8000A8140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL6_O_PL          = 0x8000A8140801103Full;
//DUPS: 8000A8140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL6_O_PL          = 0x8000A8140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A8140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A8140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A8140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A8140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL6_O_PL         = 0x8000A8140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL6_O_PL         = 0x8000A8140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A8140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL6_O_PL        = 0x8000A8140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL7_EO_PL           = 0x800038140701103Full;
//DUPS: 800038140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL7_EO_PL         = 0x800038140701103Full;
//DUPS: 800038140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL7_EO_PL         = 0x800038140801103Full;
//DUPS: 800038140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL7_EO_PL         = 0x800038140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x800038140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x800038140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x800038140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x800038140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL7_EO_PL        = 0x800038140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL7_EO_PL        = 0x800038140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x800038140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL7_EO_PL       = 0x800038140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL7_O_PL            = 0x8000B0140701103Full;
//DUPS: 8000B0140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL7_O_PL          = 0x8000B0140701103Full;
//DUPS: 8000B0140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL7_O_PL          = 0x8000B0140801103Full;
//DUPS: 8000B0140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL7_O_PL          = 0x8000B0140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B0140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B0140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B0140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B0140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL7_O_PL         = 0x8000B0140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL7_O_PL         = 0x8000B0140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B0140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL7_O_PL        = 0x8000B0140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL8_EO_PL           = 0x800040140701103Full;
//DUPS: 800040140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL8_EO_PL         = 0x800040140701103Full;
//DUPS: 800040140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL8_EO_PL         = 0x800040140801103Full;
//DUPS: 800040140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL8_EO_PL         = 0x800040140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x800040140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x800040140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x800040140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x800040140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL8_EO_PL        = 0x800040140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL8_EO_PL        = 0x800040140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x800040140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL8_EO_PL       = 0x800040140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL8_O_PL            = 0x8000B8140701103Full;
//DUPS: 8000B8140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL8_O_PL          = 0x8000B8140701103Full;
//DUPS: 8000B8140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL8_O_PL          = 0x8000B8140801103Full;
//DUPS: 8000B8140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL8_O_PL          = 0x8000B8140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B8140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B8140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B8140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B8140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL8_O_PL         = 0x8000B8140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL8_O_PL         = 0x8000B8140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B8140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL8_O_PL        = 0x8000B8140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL9_O_PL            = 0x8000C0140701103Full;
//DUPS: 8000C0140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL9_O_PL          = 0x8000C0140701103Full;
//DUPS: 8000C0140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL9_O_PL          = 0x8000C0140801103Full;
//DUPS: 8000C0140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL9_O_PL          = 0x8000C0140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C0140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C0140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C0140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C0140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL9_O_PL         = 0x8000C0140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL9_O_PL         = 0x8000C0140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C0140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_DAC_CNTL9_O_PL        = 0x8000C0140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_DATA_DAC_SPARE_MODE_PL    = 0x800000140701103Full;
//DUPS: 800000140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000140701103Full;
//DUPS: 800000140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000140801103Full;
//DUPS: 800000140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_DATA_DAC_SPARE_MODE_PL = 0x800000140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_FIR_ERROR_INJECT_PL       = 0x800218140701103Full;
//DUPS: 800218140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_FIR_ERROR_INJECT_PL     = 0x800218140701103Full;
//DUPS: 800218140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_FIR_ERROR_INJECT_PL     = 0x800218140801103Full;
//DUPS: 800218140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_FIR_ERROR_INJECT_PL     = 0x800218140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x800218140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x800218140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x800218140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x800218140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_FIR_ERROR_INJECT_PL    = 0x800218140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_FIR_ERROR_INJECT_PL    = 0x800218140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x800218140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_FIR_ERROR_INJECT_PL   = 0x800218140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_FIR_MASK_PL               = 0x800210140701103Full;
//DUPS: 800210140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_FIR_MASK_PL             = 0x800210140701103Full;
//DUPS: 800210140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_FIR_MASK_PL             = 0x800210140801103Full;
//DUPS: 800210140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_FIR_MASK_PL             = 0x800210140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_FIR_MASK_PL           = 0x800210140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_FIR_MASK_PL           = 0x800210140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_FIR_MASK_PL           = 0x800210140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_FIR_MASK_PL           = 0x800210140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_FIR_MASK_PL            = 0x800210140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_FIR_MASK_PL            = 0x800210140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_FIR_MASK_PL           = 0x800210140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_FIR_MASK_PL           = 0x800210140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_FIR_PL                    = 0x800208140701103Full;
//DUPS: 800208140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_FIR_PL                  = 0x800208140701103Full;
//DUPS: 800208140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_FIR_PL                  = 0x800208140801103Full;
//DUPS: 800208140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_FIR_PL                  = 0x800208140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_FIR_PL                = 0x800208140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_FIR_PL                = 0x800208140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_FIR_PL                = 0x800208140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_FIR_PL                = 0x800208140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_FIR_PL                 = 0x800208140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_FIR_PL                 = 0x800208140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_FIR_PL                = 0x800208140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_FIR_PL                = 0x800208140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE0_RX_SPARE_MODE_PL             = 0x800200140701103Full;
//DUPS: 800200140701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE0_RX_SPARE_MODE_PL           = 0x800200140701103Full;
//DUPS: 800200140701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE0_RX_SPARE_MODE_PL           = 0x800200140801103Full;
//DUPS: 800200140801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE0_RX_SPARE_MODE_PL           = 0x800200140701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE0_RX_SPARE_MODE_PL         = 0x800200140701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE0_RX_SPARE_MODE_PL         = 0x800200140701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE0_RX_SPARE_MODE_PL         = 0x800200140801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE0_RX_SPARE_MODE_PL         = 0x800200140801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE0_RX_SPARE_MODE_PL          = 0x800200140701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE0_RX_SPARE_MODE_PL          = 0x800200140701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE0_RX_SPARE_MODE_PL         = 0x800200140801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE0_RX_SPARE_MODE_PL         = 0x800200140801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL2_EO_PL           = 0x800248150701103Full;
//DUPS: 800248150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL2_EO_PL         = 0x800248150701103Full;
//DUPS: 800248150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL2_EO_PL         = 0x800248150801103Full;
//DUPS: 800248150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL2_EO_PL         = 0x800248150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x800248150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x800248150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x800248150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x800248150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL2_EO_PL        = 0x800248150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL2_EO_PL        = 0x800248150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x800248150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL2_EO_PL       = 0x800248150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL3_EO_PL           = 0x800250150701103Full;
//DUPS: 800250150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL3_EO_PL         = 0x800250150701103Full;
//DUPS: 800250150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL3_EO_PL         = 0x800250150801103Full;
//DUPS: 800250150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL3_EO_PL         = 0x800250150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x800250150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x800250150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x800250150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x800250150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL3_EO_PL        = 0x800250150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL3_EO_PL        = 0x800250150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x800250150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL3_EO_PL       = 0x800250150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL4_EO_PL           = 0x800258150701103Full;
//DUPS: 800258150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL4_EO_PL         = 0x800258150701103Full;
//DUPS: 800258150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL4_EO_PL         = 0x800258150801103Full;
//DUPS: 800258150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL4_EO_PL         = 0x800258150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x800258150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x800258150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x800258150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x800258150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL4_EO_PL        = 0x800258150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL4_EO_PL        = 0x800258150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x800258150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL4_EO_PL       = 0x800258150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL5_EO_PL           = 0x800260150701103Full;
//DUPS: 800260150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL5_EO_PL         = 0x800260150701103Full;
//DUPS: 800260150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL5_EO_PL         = 0x800260150801103Full;
//DUPS: 800260150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL5_EO_PL         = 0x800260150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x800260150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x800260150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x800260150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x800260150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL5_EO_PL        = 0x800260150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL5_EO_PL        = 0x800260150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x800260150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_BIT_CNTL5_EO_PL       = 0x800260150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_BIT_CNTLX1_EO_PL          = 0x800240150701103Full;
//DUPS: 800240150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_BIT_CNTLX1_EO_PL        = 0x800240150701103Full;
//DUPS: 800240150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_BIT_CNTLX1_EO_PL        = 0x800240150801103Full;
//DUPS: 800240150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_BIT_CNTLX1_EO_PL        = 0x800240150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x800240150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x800240150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x800240150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x800240150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_BIT_CNTLX1_EO_PL       = 0x800240150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_BIT_CNTLX1_EO_PL       = 0x800240150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x800240150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_BIT_CNTLX1_EO_PL      = 0x800240150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_BIT_MODE1_EO_PL           = 0x800220150701103Full;
//DUPS: 800220150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_BIT_MODE1_EO_PL         = 0x800220150701103Full;
//DUPS: 800220150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_BIT_MODE1_EO_PL         = 0x800220150801103Full;
//DUPS: 800220150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_BIT_MODE1_EO_PL         = 0x800220150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_BIT_MODE1_EO_PL       = 0x800220150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_BIT_MODE1_EO_PL       = 0x800220150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_BIT_MODE1_EO_PL       = 0x800220150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_BIT_MODE1_EO_PL       = 0x800220150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_BIT_MODE1_EO_PL        = 0x800220150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_BIT_MODE1_EO_PL        = 0x800220150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_BIT_MODE1_EO_PL       = 0x800220150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_BIT_MODE1_EO_PL       = 0x800220150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_BIT_MODE2_EO_PL           = 0x800228150701103Full;
//DUPS: 800228150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_BIT_MODE2_EO_PL         = 0x800228150701103Full;
//DUPS: 800228150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_BIT_MODE2_EO_PL         = 0x800228150801103Full;
//DUPS: 800228150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_BIT_MODE2_EO_PL         = 0x800228150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_BIT_MODE2_EO_PL       = 0x800228150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_BIT_MODE2_EO_PL       = 0x800228150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_BIT_MODE2_EO_PL       = 0x800228150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_BIT_MODE2_EO_PL       = 0x800228150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_BIT_MODE2_EO_PL        = 0x800228150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_BIT_MODE2_EO_PL        = 0x800228150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_BIT_MODE2_EO_PL       = 0x800228150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_BIT_MODE2_EO_PL       = 0x800228150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_BIT_MODE3_EO_PL           = 0x800230150701103Full;
//DUPS: 800230150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_BIT_MODE3_EO_PL         = 0x800230150701103Full;
//DUPS: 800230150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_BIT_MODE3_EO_PL         = 0x800230150801103Full;
//DUPS: 800230150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_BIT_MODE3_EO_PL         = 0x800230150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_BIT_MODE3_EO_PL       = 0x800230150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_BIT_MODE3_EO_PL       = 0x800230150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_BIT_MODE3_EO_PL       = 0x800230150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_BIT_MODE3_EO_PL       = 0x800230150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_BIT_MODE3_EO_PL        = 0x800230150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_BIT_MODE3_EO_PL        = 0x800230150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_BIT_MODE3_EO_PL       = 0x800230150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_BIT_MODE3_EO_PL       = 0x800230150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_BIT_MODE4_EO_PL           = 0x800238150701103Full;
//DUPS: 800238150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_BIT_MODE4_EO_PL         = 0x800238150701103Full;
//DUPS: 800238150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_BIT_MODE4_EO_PL         = 0x800238150801103Full;
//DUPS: 800238150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_BIT_MODE4_EO_PL         = 0x800238150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_BIT_MODE4_EO_PL       = 0x800238150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_BIT_MODE4_EO_PL       = 0x800238150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_BIT_MODE4_EO_PL       = 0x800238150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_BIT_MODE4_EO_PL       = 0x800238150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_BIT_MODE4_EO_PL        = 0x800238150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_BIT_MODE4_EO_PL        = 0x800238150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_BIT_MODE4_EO_PL       = 0x800238150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_BIT_MODE4_EO_PL       = 0x800238150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_BIT_STAT1_EO_PL           = 0x800268150701103Full;
//DUPS: 800268150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_BIT_STAT1_EO_PL         = 0x800268150701103Full;
//DUPS: 800268150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_BIT_STAT1_EO_PL         = 0x800268150801103Full;
//DUPS: 800268150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_BIT_STAT1_EO_PL         = 0x800268150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_BIT_STAT1_EO_PL       = 0x800268150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_BIT_STAT1_EO_PL       = 0x800268150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_BIT_STAT1_EO_PL       = 0x800268150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_BIT_STAT1_EO_PL       = 0x800268150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_BIT_STAT1_EO_PL        = 0x800268150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_BIT_STAT1_EO_PL        = 0x800268150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_BIT_STAT1_EO_PL       = 0x800268150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_BIT_STAT1_EO_PL       = 0x800268150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_BIT_STAT1_O_PL            = 0x800318150701103Full;
//DUPS: 800318150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_BIT_STAT1_O_PL          = 0x800318150701103Full;
//DUPS: 800318150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_BIT_STAT1_O_PL          = 0x800318150801103Full;
//DUPS: 800318150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_BIT_STAT1_O_PL          = 0x800318150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_BIT_STAT1_O_PL        = 0x800318150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_BIT_STAT1_O_PL        = 0x800318150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_BIT_STAT1_O_PL        = 0x800318150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_BIT_STAT1_O_PL        = 0x800318150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_BIT_STAT1_O_PL         = 0x800318150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_BIT_STAT1_O_PL         = 0x800318150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_BIT_STAT1_O_PL        = 0x800318150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_BIT_STAT1_O_PL        = 0x800318150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_BIT_STAT2_EO_PL           = 0x800270150701103Full;
//DUPS: 800270150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_BIT_STAT2_EO_PL         = 0x800270150701103Full;
//DUPS: 800270150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_BIT_STAT2_EO_PL         = 0x800270150801103Full;
//DUPS: 800270150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_BIT_STAT2_EO_PL         = 0x800270150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_BIT_STAT2_EO_PL       = 0x800270150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_BIT_STAT2_EO_PL       = 0x800270150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_BIT_STAT2_EO_PL       = 0x800270150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_BIT_STAT2_EO_PL       = 0x800270150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_BIT_STAT2_EO_PL        = 0x800270150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_BIT_STAT2_EO_PL        = 0x800270150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_BIT_STAT2_EO_PL       = 0x800270150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_BIT_STAT2_EO_PL       = 0x800270150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_BIT_STAT3_EO_PL           = 0x800278150701103Full;
//DUPS: 800278150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_BIT_STAT3_EO_PL         = 0x800278150701103Full;
//DUPS: 800278150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_BIT_STAT3_EO_PL         = 0x800278150801103Full;
//DUPS: 800278150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_BIT_STAT3_EO_PL         = 0x800278150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_BIT_STAT3_EO_PL       = 0x800278150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_BIT_STAT3_EO_PL       = 0x800278150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_BIT_STAT3_EO_PL       = 0x800278150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_BIT_STAT3_EO_PL       = 0x800278150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_BIT_STAT3_EO_PL        = 0x800278150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_BIT_STAT3_EO_PL        = 0x800278150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_BIT_STAT3_EO_PL       = 0x800278150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_BIT_STAT3_EO_PL       = 0x800278150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_BIT_STAT4_EO_PL           = 0x800280150701103Full;
//DUPS: 800280150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_BIT_STAT4_EO_PL         = 0x800280150701103Full;
//DUPS: 800280150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_BIT_STAT4_EO_PL         = 0x800280150801103Full;
//DUPS: 800280150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_BIT_STAT4_EO_PL         = 0x800280150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_BIT_STAT4_EO_PL       = 0x800280150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_BIT_STAT4_EO_PL       = 0x800280150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_BIT_STAT4_EO_PL       = 0x800280150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_BIT_STAT4_EO_PL       = 0x800280150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_BIT_STAT4_EO_PL        = 0x800280150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_BIT_STAT4_EO_PL        = 0x800280150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_BIT_STAT4_EO_PL       = 0x800280150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_BIT_STAT4_EO_PL       = 0x800280150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL10_O_PL           = 0x8000C8150701103Full;
//DUPS: 8000C8150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL10_O_PL         = 0x8000C8150701103Full;
//DUPS: 8000C8150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL10_O_PL         = 0x8000C8150801103Full;
//DUPS: 8000C8150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL10_O_PL         = 0x8000C8150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C8150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C8150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C8150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C8150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL10_O_PL        = 0x8000C8150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL10_O_PL        = 0x8000C8150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C8150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL10_O_PL       = 0x8000C8150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL1_EO_PL           = 0x800008150701103Full;
//DUPS: 800008150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL1_EO_PL         = 0x800008150701103Full;
//DUPS: 800008150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL1_EO_PL         = 0x800008150801103Full;
//DUPS: 800008150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL1_EO_PL         = 0x800008150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x800008150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x800008150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x800008150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x800008150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL1_EO_PL        = 0x800008150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL1_EO_PL        = 0x800008150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x800008150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL1_EO_PL       = 0x800008150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL1_O_PL            = 0x800080150701103Full;
//DUPS: 800080150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL1_O_PL          = 0x800080150701103Full;
//DUPS: 800080150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL1_O_PL          = 0x800080150801103Full;
//DUPS: 800080150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL1_O_PL          = 0x800080150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL1_O_PL        = 0x800080150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL1_O_PL        = 0x800080150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL1_O_PL        = 0x800080150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL1_O_PL        = 0x800080150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL1_O_PL         = 0x800080150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL1_O_PL         = 0x800080150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL1_O_PL        = 0x800080150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL1_O_PL        = 0x800080150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL2_EO_PL           = 0x800010150701103Full;
//DUPS: 800010150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL2_EO_PL         = 0x800010150701103Full;
//DUPS: 800010150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL2_EO_PL         = 0x800010150801103Full;
//DUPS: 800010150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL2_EO_PL         = 0x800010150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x800010150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x800010150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x800010150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x800010150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL2_EO_PL        = 0x800010150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL2_EO_PL        = 0x800010150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x800010150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL2_EO_PL       = 0x800010150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL2_O_PL            = 0x800088150701103Full;
//DUPS: 800088150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL2_O_PL          = 0x800088150701103Full;
//DUPS: 800088150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL2_O_PL          = 0x800088150801103Full;
//DUPS: 800088150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL2_O_PL          = 0x800088150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL2_O_PL        = 0x800088150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL2_O_PL        = 0x800088150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL2_O_PL        = 0x800088150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL2_O_PL        = 0x800088150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL2_O_PL         = 0x800088150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL2_O_PL         = 0x800088150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL2_O_PL        = 0x800088150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL2_O_PL        = 0x800088150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL3_EO_PL           = 0x800018150701103Full;
//DUPS: 800018150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL3_EO_PL         = 0x800018150701103Full;
//DUPS: 800018150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL3_EO_PL         = 0x800018150801103Full;
//DUPS: 800018150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL3_EO_PL         = 0x800018150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x800018150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x800018150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x800018150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x800018150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL3_EO_PL        = 0x800018150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL3_EO_PL        = 0x800018150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x800018150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL3_EO_PL       = 0x800018150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL3_O_PL            = 0x800090150701103Full;
//DUPS: 800090150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL3_O_PL          = 0x800090150701103Full;
//DUPS: 800090150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL3_O_PL          = 0x800090150801103Full;
//DUPS: 800090150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL3_O_PL          = 0x800090150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL3_O_PL        = 0x800090150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL3_O_PL        = 0x800090150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL3_O_PL        = 0x800090150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL3_O_PL        = 0x800090150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL3_O_PL         = 0x800090150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL3_O_PL         = 0x800090150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL3_O_PL        = 0x800090150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL3_O_PL        = 0x800090150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL4_EO_PL           = 0x800020150701103Full;
//DUPS: 800020150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL4_EO_PL         = 0x800020150701103Full;
//DUPS: 800020150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL4_EO_PL         = 0x800020150801103Full;
//DUPS: 800020150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL4_EO_PL         = 0x800020150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x800020150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x800020150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x800020150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x800020150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL4_EO_PL        = 0x800020150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL4_EO_PL        = 0x800020150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x800020150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL4_EO_PL       = 0x800020150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL4_O_PL            = 0x800098150701103Full;
//DUPS: 800098150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL4_O_PL          = 0x800098150701103Full;
//DUPS: 800098150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL4_O_PL          = 0x800098150801103Full;
//DUPS: 800098150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL4_O_PL          = 0x800098150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL4_O_PL        = 0x800098150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL4_O_PL        = 0x800098150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL4_O_PL        = 0x800098150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL4_O_PL        = 0x800098150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL4_O_PL         = 0x800098150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL4_O_PL         = 0x800098150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL4_O_PL        = 0x800098150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL4_O_PL        = 0x800098150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL5_EO_PL           = 0x800028150701103Full;
//DUPS: 800028150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL5_EO_PL         = 0x800028150701103Full;
//DUPS: 800028150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL5_EO_PL         = 0x800028150801103Full;
//DUPS: 800028150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL5_EO_PL         = 0x800028150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x800028150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x800028150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x800028150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x800028150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL5_EO_PL        = 0x800028150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL5_EO_PL        = 0x800028150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x800028150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL5_EO_PL       = 0x800028150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL5_O_PL            = 0x8000A0150701103Full;
//DUPS: 8000A0150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL5_O_PL          = 0x8000A0150701103Full;
//DUPS: 8000A0150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL5_O_PL          = 0x8000A0150801103Full;
//DUPS: 8000A0150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL5_O_PL          = 0x8000A0150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A0150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A0150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A0150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A0150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL5_O_PL         = 0x8000A0150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL5_O_PL         = 0x8000A0150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A0150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL5_O_PL        = 0x8000A0150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL6_EO_PL           = 0x800030150701103Full;
//DUPS: 800030150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL6_EO_PL         = 0x800030150701103Full;
//DUPS: 800030150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL6_EO_PL         = 0x800030150801103Full;
//DUPS: 800030150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL6_EO_PL         = 0x800030150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x800030150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x800030150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x800030150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x800030150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL6_EO_PL        = 0x800030150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL6_EO_PL        = 0x800030150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x800030150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL6_EO_PL       = 0x800030150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL6_O_PL            = 0x8000A8150701103Full;
//DUPS: 8000A8150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL6_O_PL          = 0x8000A8150701103Full;
//DUPS: 8000A8150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL6_O_PL          = 0x8000A8150801103Full;
//DUPS: 8000A8150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL6_O_PL          = 0x8000A8150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A8150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A8150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A8150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A8150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL6_O_PL         = 0x8000A8150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL6_O_PL         = 0x8000A8150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A8150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL6_O_PL        = 0x8000A8150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL7_EO_PL           = 0x800038150701103Full;
//DUPS: 800038150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL7_EO_PL         = 0x800038150701103Full;
//DUPS: 800038150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL7_EO_PL         = 0x800038150801103Full;
//DUPS: 800038150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL7_EO_PL         = 0x800038150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x800038150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x800038150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x800038150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x800038150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL7_EO_PL        = 0x800038150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL7_EO_PL        = 0x800038150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x800038150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL7_EO_PL       = 0x800038150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL7_O_PL            = 0x8000B0150701103Full;
//DUPS: 8000B0150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL7_O_PL          = 0x8000B0150701103Full;
//DUPS: 8000B0150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL7_O_PL          = 0x8000B0150801103Full;
//DUPS: 8000B0150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL7_O_PL          = 0x8000B0150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B0150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B0150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B0150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B0150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL7_O_PL         = 0x8000B0150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL7_O_PL         = 0x8000B0150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B0150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL7_O_PL        = 0x8000B0150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL8_EO_PL           = 0x800040150701103Full;
//DUPS: 800040150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL8_EO_PL         = 0x800040150701103Full;
//DUPS: 800040150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL8_EO_PL         = 0x800040150801103Full;
//DUPS: 800040150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL8_EO_PL         = 0x800040150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x800040150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x800040150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x800040150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x800040150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL8_EO_PL        = 0x800040150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL8_EO_PL        = 0x800040150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x800040150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL8_EO_PL       = 0x800040150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL8_O_PL            = 0x8000B8150701103Full;
//DUPS: 8000B8150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL8_O_PL          = 0x8000B8150701103Full;
//DUPS: 8000B8150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL8_O_PL          = 0x8000B8150801103Full;
//DUPS: 8000B8150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL8_O_PL          = 0x8000B8150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B8150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B8150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B8150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B8150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL8_O_PL         = 0x8000B8150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL8_O_PL         = 0x8000B8150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B8150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL8_O_PL        = 0x8000B8150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL9_O_PL            = 0x8000C0150701103Full;
//DUPS: 8000C0150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL9_O_PL          = 0x8000C0150701103Full;
//DUPS: 8000C0150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL9_O_PL          = 0x8000C0150801103Full;
//DUPS: 8000C0150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL9_O_PL          = 0x8000C0150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C0150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C0150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C0150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C0150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL9_O_PL         = 0x8000C0150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL9_O_PL         = 0x8000C0150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C0150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_DAC_CNTL9_O_PL        = 0x8000C0150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_DATA_DAC_SPARE_MODE_PL    = 0x800000150701103Full;
//DUPS: 800000150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000150701103Full;
//DUPS: 800000150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000150801103Full;
//DUPS: 800000150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_DATA_DAC_SPARE_MODE_PL = 0x800000150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_FIR_ERROR_INJECT_PL       = 0x800218150701103Full;
//DUPS: 800218150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_FIR_ERROR_INJECT_PL     = 0x800218150701103Full;
//DUPS: 800218150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_FIR_ERROR_INJECT_PL     = 0x800218150801103Full;
//DUPS: 800218150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_FIR_ERROR_INJECT_PL     = 0x800218150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x800218150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x800218150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x800218150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x800218150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_FIR_ERROR_INJECT_PL    = 0x800218150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_FIR_ERROR_INJECT_PL    = 0x800218150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x800218150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_FIR_ERROR_INJECT_PL   = 0x800218150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_FIR_MASK_PL               = 0x800210150701103Full;
//DUPS: 800210150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_FIR_MASK_PL             = 0x800210150701103Full;
//DUPS: 800210150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_FIR_MASK_PL             = 0x800210150801103Full;
//DUPS: 800210150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_FIR_MASK_PL             = 0x800210150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_FIR_MASK_PL           = 0x800210150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_FIR_MASK_PL           = 0x800210150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_FIR_MASK_PL           = 0x800210150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_FIR_MASK_PL           = 0x800210150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_FIR_MASK_PL            = 0x800210150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_FIR_MASK_PL            = 0x800210150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_FIR_MASK_PL           = 0x800210150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_FIR_MASK_PL           = 0x800210150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_FIR_PL                    = 0x800208150701103Full;
//DUPS: 800208150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_FIR_PL                  = 0x800208150701103Full;
//DUPS: 800208150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_FIR_PL                  = 0x800208150801103Full;
//DUPS: 800208150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_FIR_PL                  = 0x800208150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_FIR_PL                = 0x800208150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_FIR_PL                = 0x800208150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_FIR_PL                = 0x800208150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_FIR_PL                = 0x800208150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_FIR_PL                 = 0x800208150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_FIR_PL                 = 0x800208150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_FIR_PL                = 0x800208150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_FIR_PL                = 0x800208150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE1_RX_SPARE_MODE_PL             = 0x800200150701103Full;
//DUPS: 800200150701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE1_RX_SPARE_MODE_PL           = 0x800200150701103Full;
//DUPS: 800200150701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE1_RX_SPARE_MODE_PL           = 0x800200150801103Full;
//DUPS: 800200150801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE1_RX_SPARE_MODE_PL           = 0x800200150701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE1_RX_SPARE_MODE_PL         = 0x800200150701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE1_RX_SPARE_MODE_PL         = 0x800200150701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE1_RX_SPARE_MODE_PL         = 0x800200150801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE1_RX_SPARE_MODE_PL         = 0x800200150801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE1_RX_SPARE_MODE_PL          = 0x800200150701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE1_RX_SPARE_MODE_PL          = 0x800200150701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE1_RX_SPARE_MODE_PL         = 0x800200150801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE1_RX_SPARE_MODE_PL         = 0x800200150801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL2_EO_PL           = 0x800248160701103Full;
//DUPS: 800248160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL2_EO_PL         = 0x800248160701103Full;
//DUPS: 800248160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL2_EO_PL         = 0x800248160801103Full;
//DUPS: 800248160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL2_EO_PL         = 0x800248160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x800248160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x800248160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x800248160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x800248160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL2_EO_PL        = 0x800248160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL2_EO_PL        = 0x800248160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x800248160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL2_EO_PL       = 0x800248160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL3_EO_PL           = 0x800250160701103Full;
//DUPS: 800250160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL3_EO_PL         = 0x800250160701103Full;
//DUPS: 800250160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL3_EO_PL         = 0x800250160801103Full;
//DUPS: 800250160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL3_EO_PL         = 0x800250160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x800250160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x800250160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x800250160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x800250160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL3_EO_PL        = 0x800250160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL3_EO_PL        = 0x800250160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x800250160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL3_EO_PL       = 0x800250160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL4_EO_PL           = 0x800258160701103Full;
//DUPS: 800258160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL4_EO_PL         = 0x800258160701103Full;
//DUPS: 800258160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL4_EO_PL         = 0x800258160801103Full;
//DUPS: 800258160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL4_EO_PL         = 0x800258160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x800258160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x800258160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x800258160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x800258160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL4_EO_PL        = 0x800258160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL4_EO_PL        = 0x800258160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x800258160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL4_EO_PL       = 0x800258160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL5_EO_PL           = 0x800260160701103Full;
//DUPS: 800260160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL5_EO_PL         = 0x800260160701103Full;
//DUPS: 800260160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL5_EO_PL         = 0x800260160801103Full;
//DUPS: 800260160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL5_EO_PL         = 0x800260160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x800260160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x800260160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x800260160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x800260160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL5_EO_PL        = 0x800260160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL5_EO_PL        = 0x800260160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x800260160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_BIT_CNTL5_EO_PL       = 0x800260160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_BIT_CNTLX1_EO_PL          = 0x800240160701103Full;
//DUPS: 800240160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_BIT_CNTLX1_EO_PL        = 0x800240160701103Full;
//DUPS: 800240160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_BIT_CNTLX1_EO_PL        = 0x800240160801103Full;
//DUPS: 800240160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_BIT_CNTLX1_EO_PL        = 0x800240160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x800240160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x800240160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x800240160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x800240160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_BIT_CNTLX1_EO_PL       = 0x800240160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_BIT_CNTLX1_EO_PL       = 0x800240160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x800240160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_BIT_CNTLX1_EO_PL      = 0x800240160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_BIT_MODE1_EO_PL           = 0x800220160701103Full;
//DUPS: 800220160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_BIT_MODE1_EO_PL         = 0x800220160701103Full;
//DUPS: 800220160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_BIT_MODE1_EO_PL         = 0x800220160801103Full;
//DUPS: 800220160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_BIT_MODE1_EO_PL         = 0x800220160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_BIT_MODE1_EO_PL       = 0x800220160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_BIT_MODE1_EO_PL       = 0x800220160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_BIT_MODE1_EO_PL       = 0x800220160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_BIT_MODE1_EO_PL       = 0x800220160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_BIT_MODE1_EO_PL        = 0x800220160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_BIT_MODE1_EO_PL        = 0x800220160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_BIT_MODE1_EO_PL       = 0x800220160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_BIT_MODE1_EO_PL       = 0x800220160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_BIT_MODE2_EO_PL           = 0x800228160701103Full;
//DUPS: 800228160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_BIT_MODE2_EO_PL         = 0x800228160701103Full;
//DUPS: 800228160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_BIT_MODE2_EO_PL         = 0x800228160801103Full;
//DUPS: 800228160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_BIT_MODE2_EO_PL         = 0x800228160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_BIT_MODE2_EO_PL       = 0x800228160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_BIT_MODE2_EO_PL       = 0x800228160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_BIT_MODE2_EO_PL       = 0x800228160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_BIT_MODE2_EO_PL       = 0x800228160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_BIT_MODE2_EO_PL        = 0x800228160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_BIT_MODE2_EO_PL        = 0x800228160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_BIT_MODE2_EO_PL       = 0x800228160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_BIT_MODE2_EO_PL       = 0x800228160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_BIT_MODE3_EO_PL           = 0x800230160701103Full;
//DUPS: 800230160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_BIT_MODE3_EO_PL         = 0x800230160701103Full;
//DUPS: 800230160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_BIT_MODE3_EO_PL         = 0x800230160801103Full;
//DUPS: 800230160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_BIT_MODE3_EO_PL         = 0x800230160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_BIT_MODE3_EO_PL       = 0x800230160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_BIT_MODE3_EO_PL       = 0x800230160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_BIT_MODE3_EO_PL       = 0x800230160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_BIT_MODE3_EO_PL       = 0x800230160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_BIT_MODE3_EO_PL        = 0x800230160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_BIT_MODE3_EO_PL        = 0x800230160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_BIT_MODE3_EO_PL       = 0x800230160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_BIT_MODE3_EO_PL       = 0x800230160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_BIT_MODE4_EO_PL           = 0x800238160701103Full;
//DUPS: 800238160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_BIT_MODE4_EO_PL         = 0x800238160701103Full;
//DUPS: 800238160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_BIT_MODE4_EO_PL         = 0x800238160801103Full;
//DUPS: 800238160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_BIT_MODE4_EO_PL         = 0x800238160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_BIT_MODE4_EO_PL       = 0x800238160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_BIT_MODE4_EO_PL       = 0x800238160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_BIT_MODE4_EO_PL       = 0x800238160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_BIT_MODE4_EO_PL       = 0x800238160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_BIT_MODE4_EO_PL        = 0x800238160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_BIT_MODE4_EO_PL        = 0x800238160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_BIT_MODE4_EO_PL       = 0x800238160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_BIT_MODE4_EO_PL       = 0x800238160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_BIT_STAT1_EO_PL           = 0x800268160701103Full;
//DUPS: 800268160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_BIT_STAT1_EO_PL         = 0x800268160701103Full;
//DUPS: 800268160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_BIT_STAT1_EO_PL         = 0x800268160801103Full;
//DUPS: 800268160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_BIT_STAT1_EO_PL         = 0x800268160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_BIT_STAT1_EO_PL       = 0x800268160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_BIT_STAT1_EO_PL       = 0x800268160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_BIT_STAT1_EO_PL       = 0x800268160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_BIT_STAT1_EO_PL       = 0x800268160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_BIT_STAT1_EO_PL        = 0x800268160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_BIT_STAT1_EO_PL        = 0x800268160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_BIT_STAT1_EO_PL       = 0x800268160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_BIT_STAT1_EO_PL       = 0x800268160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_BIT_STAT1_O_PL            = 0x800318160701103Full;
//DUPS: 800318160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_BIT_STAT1_O_PL          = 0x800318160701103Full;
//DUPS: 800318160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_BIT_STAT1_O_PL          = 0x800318160801103Full;
//DUPS: 800318160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_BIT_STAT1_O_PL          = 0x800318160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_BIT_STAT1_O_PL        = 0x800318160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_BIT_STAT1_O_PL        = 0x800318160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_BIT_STAT1_O_PL        = 0x800318160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_BIT_STAT1_O_PL        = 0x800318160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_BIT_STAT1_O_PL         = 0x800318160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_BIT_STAT1_O_PL         = 0x800318160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_BIT_STAT1_O_PL        = 0x800318160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_BIT_STAT1_O_PL        = 0x800318160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_BIT_STAT2_EO_PL           = 0x800270160701103Full;
//DUPS: 800270160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_BIT_STAT2_EO_PL         = 0x800270160701103Full;
//DUPS: 800270160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_BIT_STAT2_EO_PL         = 0x800270160801103Full;
//DUPS: 800270160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_BIT_STAT2_EO_PL         = 0x800270160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_BIT_STAT2_EO_PL       = 0x800270160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_BIT_STAT2_EO_PL       = 0x800270160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_BIT_STAT2_EO_PL       = 0x800270160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_BIT_STAT2_EO_PL       = 0x800270160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_BIT_STAT2_EO_PL        = 0x800270160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_BIT_STAT2_EO_PL        = 0x800270160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_BIT_STAT2_EO_PL       = 0x800270160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_BIT_STAT2_EO_PL       = 0x800270160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_BIT_STAT3_EO_PL           = 0x800278160701103Full;
//DUPS: 800278160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_BIT_STAT3_EO_PL         = 0x800278160701103Full;
//DUPS: 800278160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_BIT_STAT3_EO_PL         = 0x800278160801103Full;
//DUPS: 800278160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_BIT_STAT3_EO_PL         = 0x800278160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_BIT_STAT3_EO_PL       = 0x800278160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_BIT_STAT3_EO_PL       = 0x800278160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_BIT_STAT3_EO_PL       = 0x800278160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_BIT_STAT3_EO_PL       = 0x800278160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_BIT_STAT3_EO_PL        = 0x800278160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_BIT_STAT3_EO_PL        = 0x800278160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_BIT_STAT3_EO_PL       = 0x800278160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_BIT_STAT3_EO_PL       = 0x800278160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_BIT_STAT4_EO_PL           = 0x800280160701103Full;
//DUPS: 800280160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_BIT_STAT4_EO_PL         = 0x800280160701103Full;
//DUPS: 800280160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_BIT_STAT4_EO_PL         = 0x800280160801103Full;
//DUPS: 800280160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_BIT_STAT4_EO_PL         = 0x800280160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_BIT_STAT4_EO_PL       = 0x800280160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_BIT_STAT4_EO_PL       = 0x800280160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_BIT_STAT4_EO_PL       = 0x800280160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_BIT_STAT4_EO_PL       = 0x800280160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_BIT_STAT4_EO_PL        = 0x800280160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_BIT_STAT4_EO_PL        = 0x800280160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_BIT_STAT4_EO_PL       = 0x800280160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_BIT_STAT4_EO_PL       = 0x800280160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL10_O_PL           = 0x8000C8160701103Full;
//DUPS: 8000C8160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL10_O_PL         = 0x8000C8160701103Full;
//DUPS: 8000C8160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL10_O_PL         = 0x8000C8160801103Full;
//DUPS: 8000C8160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL10_O_PL         = 0x8000C8160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C8160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C8160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C8160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C8160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL10_O_PL        = 0x8000C8160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL10_O_PL        = 0x8000C8160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C8160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL10_O_PL       = 0x8000C8160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL1_EO_PL           = 0x800008160701103Full;
//DUPS: 800008160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL1_EO_PL         = 0x800008160701103Full;
//DUPS: 800008160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL1_EO_PL         = 0x800008160801103Full;
//DUPS: 800008160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL1_EO_PL         = 0x800008160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x800008160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x800008160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x800008160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x800008160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL1_EO_PL        = 0x800008160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL1_EO_PL        = 0x800008160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x800008160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL1_EO_PL       = 0x800008160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL1_O_PL            = 0x800080160701103Full;
//DUPS: 800080160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL1_O_PL          = 0x800080160701103Full;
//DUPS: 800080160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL1_O_PL          = 0x800080160801103Full;
//DUPS: 800080160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL1_O_PL          = 0x800080160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL1_O_PL        = 0x800080160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL1_O_PL        = 0x800080160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL1_O_PL        = 0x800080160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL1_O_PL        = 0x800080160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL1_O_PL         = 0x800080160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL1_O_PL         = 0x800080160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL1_O_PL        = 0x800080160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL1_O_PL        = 0x800080160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL2_EO_PL           = 0x800010160701103Full;
//DUPS: 800010160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL2_EO_PL         = 0x800010160701103Full;
//DUPS: 800010160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL2_EO_PL         = 0x800010160801103Full;
//DUPS: 800010160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL2_EO_PL         = 0x800010160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x800010160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x800010160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x800010160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x800010160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL2_EO_PL        = 0x800010160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL2_EO_PL        = 0x800010160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x800010160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL2_EO_PL       = 0x800010160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL2_O_PL            = 0x800088160701103Full;
//DUPS: 800088160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL2_O_PL          = 0x800088160701103Full;
//DUPS: 800088160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL2_O_PL          = 0x800088160801103Full;
//DUPS: 800088160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL2_O_PL          = 0x800088160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL2_O_PL        = 0x800088160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL2_O_PL        = 0x800088160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL2_O_PL        = 0x800088160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL2_O_PL        = 0x800088160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL2_O_PL         = 0x800088160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL2_O_PL         = 0x800088160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL2_O_PL        = 0x800088160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL2_O_PL        = 0x800088160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL3_EO_PL           = 0x800018160701103Full;
//DUPS: 800018160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL3_EO_PL         = 0x800018160701103Full;
//DUPS: 800018160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL3_EO_PL         = 0x800018160801103Full;
//DUPS: 800018160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL3_EO_PL         = 0x800018160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x800018160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x800018160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x800018160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x800018160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL3_EO_PL        = 0x800018160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL3_EO_PL        = 0x800018160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x800018160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL3_EO_PL       = 0x800018160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL3_O_PL            = 0x800090160701103Full;
//DUPS: 800090160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL3_O_PL          = 0x800090160701103Full;
//DUPS: 800090160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL3_O_PL          = 0x800090160801103Full;
//DUPS: 800090160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL3_O_PL          = 0x800090160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL3_O_PL        = 0x800090160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL3_O_PL        = 0x800090160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL3_O_PL        = 0x800090160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL3_O_PL        = 0x800090160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL3_O_PL         = 0x800090160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL3_O_PL         = 0x800090160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL3_O_PL        = 0x800090160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL3_O_PL        = 0x800090160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL4_EO_PL           = 0x800020160701103Full;
//DUPS: 800020160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL4_EO_PL         = 0x800020160701103Full;
//DUPS: 800020160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL4_EO_PL         = 0x800020160801103Full;
//DUPS: 800020160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL4_EO_PL         = 0x800020160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x800020160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x800020160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x800020160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x800020160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL4_EO_PL        = 0x800020160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL4_EO_PL        = 0x800020160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x800020160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL4_EO_PL       = 0x800020160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL4_O_PL            = 0x800098160701103Full;
//DUPS: 800098160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL4_O_PL          = 0x800098160701103Full;
//DUPS: 800098160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL4_O_PL          = 0x800098160801103Full;
//DUPS: 800098160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL4_O_PL          = 0x800098160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL4_O_PL        = 0x800098160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL4_O_PL        = 0x800098160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL4_O_PL        = 0x800098160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL4_O_PL        = 0x800098160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL4_O_PL         = 0x800098160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL4_O_PL         = 0x800098160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL4_O_PL        = 0x800098160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL4_O_PL        = 0x800098160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL5_EO_PL           = 0x800028160701103Full;
//DUPS: 800028160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL5_EO_PL         = 0x800028160701103Full;
//DUPS: 800028160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL5_EO_PL         = 0x800028160801103Full;
//DUPS: 800028160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL5_EO_PL         = 0x800028160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x800028160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x800028160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x800028160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x800028160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL5_EO_PL        = 0x800028160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL5_EO_PL        = 0x800028160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x800028160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL5_EO_PL       = 0x800028160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL5_O_PL            = 0x8000A0160701103Full;
//DUPS: 8000A0160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL5_O_PL          = 0x8000A0160701103Full;
//DUPS: 8000A0160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL5_O_PL          = 0x8000A0160801103Full;
//DUPS: 8000A0160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL5_O_PL          = 0x8000A0160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A0160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A0160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A0160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A0160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL5_O_PL         = 0x8000A0160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL5_O_PL         = 0x8000A0160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A0160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL5_O_PL        = 0x8000A0160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL6_EO_PL           = 0x800030160701103Full;
//DUPS: 800030160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL6_EO_PL         = 0x800030160701103Full;
//DUPS: 800030160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL6_EO_PL         = 0x800030160801103Full;
//DUPS: 800030160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL6_EO_PL         = 0x800030160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x800030160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x800030160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x800030160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x800030160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL6_EO_PL        = 0x800030160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL6_EO_PL        = 0x800030160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x800030160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL6_EO_PL       = 0x800030160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL6_O_PL            = 0x8000A8160701103Full;
//DUPS: 8000A8160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL6_O_PL          = 0x8000A8160701103Full;
//DUPS: 8000A8160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL6_O_PL          = 0x8000A8160801103Full;
//DUPS: 8000A8160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL6_O_PL          = 0x8000A8160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A8160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A8160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A8160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A8160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL6_O_PL         = 0x8000A8160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL6_O_PL         = 0x8000A8160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A8160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL6_O_PL        = 0x8000A8160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL7_EO_PL           = 0x800038160701103Full;
//DUPS: 800038160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL7_EO_PL         = 0x800038160701103Full;
//DUPS: 800038160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL7_EO_PL         = 0x800038160801103Full;
//DUPS: 800038160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL7_EO_PL         = 0x800038160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x800038160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x800038160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x800038160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x800038160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL7_EO_PL        = 0x800038160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL7_EO_PL        = 0x800038160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x800038160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL7_EO_PL       = 0x800038160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL7_O_PL            = 0x8000B0160701103Full;
//DUPS: 8000B0160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL7_O_PL          = 0x8000B0160701103Full;
//DUPS: 8000B0160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL7_O_PL          = 0x8000B0160801103Full;
//DUPS: 8000B0160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL7_O_PL          = 0x8000B0160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B0160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B0160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B0160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B0160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL7_O_PL         = 0x8000B0160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL7_O_PL         = 0x8000B0160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B0160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL7_O_PL        = 0x8000B0160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL8_EO_PL           = 0x800040160701103Full;
//DUPS: 800040160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL8_EO_PL         = 0x800040160701103Full;
//DUPS: 800040160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL8_EO_PL         = 0x800040160801103Full;
//DUPS: 800040160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL8_EO_PL         = 0x800040160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x800040160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x800040160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x800040160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x800040160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL8_EO_PL        = 0x800040160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL8_EO_PL        = 0x800040160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x800040160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL8_EO_PL       = 0x800040160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL8_O_PL            = 0x8000B8160701103Full;
//DUPS: 8000B8160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL8_O_PL          = 0x8000B8160701103Full;
//DUPS: 8000B8160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL8_O_PL          = 0x8000B8160801103Full;
//DUPS: 8000B8160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL8_O_PL          = 0x8000B8160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B8160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B8160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B8160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B8160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL8_O_PL         = 0x8000B8160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL8_O_PL         = 0x8000B8160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B8160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL8_O_PL        = 0x8000B8160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL9_O_PL            = 0x8000C0160701103Full;
//DUPS: 8000C0160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL9_O_PL          = 0x8000C0160701103Full;
//DUPS: 8000C0160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL9_O_PL          = 0x8000C0160801103Full;
//DUPS: 8000C0160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL9_O_PL          = 0x8000C0160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C0160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C0160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C0160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C0160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL9_O_PL         = 0x8000C0160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL9_O_PL         = 0x8000C0160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C0160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_DAC_CNTL9_O_PL        = 0x8000C0160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_DATA_DAC_SPARE_MODE_PL    = 0x800000160701103Full;
//DUPS: 800000160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000160701103Full;
//DUPS: 800000160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000160801103Full;
//DUPS: 800000160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_DATA_DAC_SPARE_MODE_PL = 0x800000160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_FIR_ERROR_INJECT_PL       = 0x800218160701103Full;
//DUPS: 800218160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_FIR_ERROR_INJECT_PL     = 0x800218160701103Full;
//DUPS: 800218160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_FIR_ERROR_INJECT_PL     = 0x800218160801103Full;
//DUPS: 800218160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_FIR_ERROR_INJECT_PL     = 0x800218160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x800218160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x800218160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x800218160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x800218160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_FIR_ERROR_INJECT_PL    = 0x800218160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_FIR_ERROR_INJECT_PL    = 0x800218160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x800218160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_FIR_ERROR_INJECT_PL   = 0x800218160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_FIR_MASK_PL               = 0x800210160701103Full;
//DUPS: 800210160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_FIR_MASK_PL             = 0x800210160701103Full;
//DUPS: 800210160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_FIR_MASK_PL             = 0x800210160801103Full;
//DUPS: 800210160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_FIR_MASK_PL             = 0x800210160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_FIR_MASK_PL           = 0x800210160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_FIR_MASK_PL           = 0x800210160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_FIR_MASK_PL           = 0x800210160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_FIR_MASK_PL           = 0x800210160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_FIR_MASK_PL            = 0x800210160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_FIR_MASK_PL            = 0x800210160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_FIR_MASK_PL           = 0x800210160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_FIR_MASK_PL           = 0x800210160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_FIR_PL                    = 0x800208160701103Full;
//DUPS: 800208160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_FIR_PL                  = 0x800208160701103Full;
//DUPS: 800208160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_FIR_PL                  = 0x800208160801103Full;
//DUPS: 800208160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_FIR_PL                  = 0x800208160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_FIR_PL                = 0x800208160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_FIR_PL                = 0x800208160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_FIR_PL                = 0x800208160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_FIR_PL                = 0x800208160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_FIR_PL                 = 0x800208160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_FIR_PL                 = 0x800208160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_FIR_PL                = 0x800208160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_FIR_PL                = 0x800208160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE2_RX_SPARE_MODE_PL             = 0x800200160701103Full;
//DUPS: 800200160701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE2_RX_SPARE_MODE_PL           = 0x800200160701103Full;
//DUPS: 800200160701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE2_RX_SPARE_MODE_PL           = 0x800200160801103Full;
//DUPS: 800200160801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE2_RX_SPARE_MODE_PL           = 0x800200160701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE2_RX_SPARE_MODE_PL         = 0x800200160701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE2_RX_SPARE_MODE_PL         = 0x800200160701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE2_RX_SPARE_MODE_PL         = 0x800200160801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE2_RX_SPARE_MODE_PL         = 0x800200160801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE2_RX_SPARE_MODE_PL          = 0x800200160701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE2_RX_SPARE_MODE_PL          = 0x800200160701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE2_RX_SPARE_MODE_PL         = 0x800200160801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE2_RX_SPARE_MODE_PL         = 0x800200160801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL2_EO_PL           = 0x800248170701103Full;
//DUPS: 800248170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL2_EO_PL         = 0x800248170701103Full;
//DUPS: 800248170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL2_EO_PL         = 0x800248170801103Full;
//DUPS: 800248170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL2_EO_PL         = 0x800248170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x800248170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x800248170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x800248170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x800248170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL2_EO_PL        = 0x800248170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL2_EO_PL        = 0x800248170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x800248170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL2_EO_PL       = 0x800248170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL3_EO_PL           = 0x800250170701103Full;
//DUPS: 800250170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL3_EO_PL         = 0x800250170701103Full;
//DUPS: 800250170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL3_EO_PL         = 0x800250170801103Full;
//DUPS: 800250170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL3_EO_PL         = 0x800250170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x800250170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x800250170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x800250170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x800250170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL3_EO_PL        = 0x800250170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL3_EO_PL        = 0x800250170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x800250170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL3_EO_PL       = 0x800250170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL4_EO_PL           = 0x800258170701103Full;
//DUPS: 800258170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL4_EO_PL         = 0x800258170701103Full;
//DUPS: 800258170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL4_EO_PL         = 0x800258170801103Full;
//DUPS: 800258170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL4_EO_PL         = 0x800258170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x800258170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x800258170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x800258170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x800258170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL4_EO_PL        = 0x800258170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL4_EO_PL        = 0x800258170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x800258170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL4_EO_PL       = 0x800258170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL5_EO_PL           = 0x800260170701103Full;
//DUPS: 800260170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL5_EO_PL         = 0x800260170701103Full;
//DUPS: 800260170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL5_EO_PL         = 0x800260170801103Full;
//DUPS: 800260170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL5_EO_PL         = 0x800260170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x800260170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x800260170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x800260170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x800260170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL5_EO_PL        = 0x800260170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL5_EO_PL        = 0x800260170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x800260170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_BIT_CNTL5_EO_PL       = 0x800260170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_BIT_CNTLX1_EO_PL          = 0x800240170701103Full;
//DUPS: 800240170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_BIT_CNTLX1_EO_PL        = 0x800240170701103Full;
//DUPS: 800240170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_BIT_CNTLX1_EO_PL        = 0x800240170801103Full;
//DUPS: 800240170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_BIT_CNTLX1_EO_PL        = 0x800240170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x800240170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x800240170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x800240170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x800240170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_BIT_CNTLX1_EO_PL       = 0x800240170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_BIT_CNTLX1_EO_PL       = 0x800240170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x800240170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_BIT_CNTLX1_EO_PL      = 0x800240170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_BIT_MODE1_EO_PL           = 0x800220170701103Full;
//DUPS: 800220170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_BIT_MODE1_EO_PL         = 0x800220170701103Full;
//DUPS: 800220170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_BIT_MODE1_EO_PL         = 0x800220170801103Full;
//DUPS: 800220170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_BIT_MODE1_EO_PL         = 0x800220170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_BIT_MODE1_EO_PL       = 0x800220170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_BIT_MODE1_EO_PL       = 0x800220170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_BIT_MODE1_EO_PL       = 0x800220170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_BIT_MODE1_EO_PL       = 0x800220170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_BIT_MODE1_EO_PL        = 0x800220170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_BIT_MODE1_EO_PL        = 0x800220170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_BIT_MODE1_EO_PL       = 0x800220170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_BIT_MODE1_EO_PL       = 0x800220170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_BIT_MODE2_EO_PL           = 0x800228170701103Full;
//DUPS: 800228170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_BIT_MODE2_EO_PL         = 0x800228170701103Full;
//DUPS: 800228170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_BIT_MODE2_EO_PL         = 0x800228170801103Full;
//DUPS: 800228170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_BIT_MODE2_EO_PL         = 0x800228170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_BIT_MODE2_EO_PL       = 0x800228170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_BIT_MODE2_EO_PL       = 0x800228170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_BIT_MODE2_EO_PL       = 0x800228170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_BIT_MODE2_EO_PL       = 0x800228170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_BIT_MODE2_EO_PL        = 0x800228170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_BIT_MODE2_EO_PL        = 0x800228170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_BIT_MODE2_EO_PL       = 0x800228170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_BIT_MODE2_EO_PL       = 0x800228170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_BIT_MODE3_EO_PL           = 0x800230170701103Full;
//DUPS: 800230170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_BIT_MODE3_EO_PL         = 0x800230170701103Full;
//DUPS: 800230170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_BIT_MODE3_EO_PL         = 0x800230170801103Full;
//DUPS: 800230170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_BIT_MODE3_EO_PL         = 0x800230170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_BIT_MODE3_EO_PL       = 0x800230170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_BIT_MODE3_EO_PL       = 0x800230170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_BIT_MODE3_EO_PL       = 0x800230170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_BIT_MODE3_EO_PL       = 0x800230170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_BIT_MODE3_EO_PL        = 0x800230170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_BIT_MODE3_EO_PL        = 0x800230170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_BIT_MODE3_EO_PL       = 0x800230170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_BIT_MODE3_EO_PL       = 0x800230170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_BIT_MODE4_EO_PL           = 0x800238170701103Full;
//DUPS: 800238170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_BIT_MODE4_EO_PL         = 0x800238170701103Full;
//DUPS: 800238170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_BIT_MODE4_EO_PL         = 0x800238170801103Full;
//DUPS: 800238170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_BIT_MODE4_EO_PL         = 0x800238170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_BIT_MODE4_EO_PL       = 0x800238170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_BIT_MODE4_EO_PL       = 0x800238170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_BIT_MODE4_EO_PL       = 0x800238170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_BIT_MODE4_EO_PL       = 0x800238170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_BIT_MODE4_EO_PL        = 0x800238170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_BIT_MODE4_EO_PL        = 0x800238170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_BIT_MODE4_EO_PL       = 0x800238170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_BIT_MODE4_EO_PL       = 0x800238170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_BIT_STAT1_EO_PL           = 0x800268170701103Full;
//DUPS: 800268170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_BIT_STAT1_EO_PL         = 0x800268170701103Full;
//DUPS: 800268170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_BIT_STAT1_EO_PL         = 0x800268170801103Full;
//DUPS: 800268170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_BIT_STAT1_EO_PL         = 0x800268170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_BIT_STAT1_EO_PL       = 0x800268170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_BIT_STAT1_EO_PL       = 0x800268170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_BIT_STAT1_EO_PL       = 0x800268170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_BIT_STAT1_EO_PL       = 0x800268170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_BIT_STAT1_EO_PL        = 0x800268170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_BIT_STAT1_EO_PL        = 0x800268170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_BIT_STAT1_EO_PL       = 0x800268170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_BIT_STAT1_EO_PL       = 0x800268170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_BIT_STAT1_O_PL            = 0x800318170701103Full;
//DUPS: 800318170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_BIT_STAT1_O_PL          = 0x800318170701103Full;
//DUPS: 800318170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_BIT_STAT1_O_PL          = 0x800318170801103Full;
//DUPS: 800318170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_BIT_STAT1_O_PL          = 0x800318170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_BIT_STAT1_O_PL        = 0x800318170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_BIT_STAT1_O_PL        = 0x800318170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_BIT_STAT1_O_PL        = 0x800318170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_BIT_STAT1_O_PL        = 0x800318170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_BIT_STAT1_O_PL         = 0x800318170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_BIT_STAT1_O_PL         = 0x800318170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_BIT_STAT1_O_PL        = 0x800318170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_BIT_STAT1_O_PL        = 0x800318170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_BIT_STAT2_EO_PL           = 0x800270170701103Full;
//DUPS: 800270170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_BIT_STAT2_EO_PL         = 0x800270170701103Full;
//DUPS: 800270170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_BIT_STAT2_EO_PL         = 0x800270170801103Full;
//DUPS: 800270170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_BIT_STAT2_EO_PL         = 0x800270170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_BIT_STAT2_EO_PL       = 0x800270170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_BIT_STAT2_EO_PL       = 0x800270170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_BIT_STAT2_EO_PL       = 0x800270170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_BIT_STAT2_EO_PL       = 0x800270170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_BIT_STAT2_EO_PL        = 0x800270170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_BIT_STAT2_EO_PL        = 0x800270170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_BIT_STAT2_EO_PL       = 0x800270170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_BIT_STAT2_EO_PL       = 0x800270170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_BIT_STAT3_EO_PL           = 0x800278170701103Full;
//DUPS: 800278170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_BIT_STAT3_EO_PL         = 0x800278170701103Full;
//DUPS: 800278170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_BIT_STAT3_EO_PL         = 0x800278170801103Full;
//DUPS: 800278170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_BIT_STAT3_EO_PL         = 0x800278170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_BIT_STAT3_EO_PL       = 0x800278170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_BIT_STAT3_EO_PL       = 0x800278170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_BIT_STAT3_EO_PL       = 0x800278170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_BIT_STAT3_EO_PL       = 0x800278170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_BIT_STAT3_EO_PL        = 0x800278170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_BIT_STAT3_EO_PL        = 0x800278170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_BIT_STAT3_EO_PL       = 0x800278170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_BIT_STAT3_EO_PL       = 0x800278170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_BIT_STAT4_EO_PL           = 0x800280170701103Full;
//DUPS: 800280170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_BIT_STAT4_EO_PL         = 0x800280170701103Full;
//DUPS: 800280170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_BIT_STAT4_EO_PL         = 0x800280170801103Full;
//DUPS: 800280170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_BIT_STAT4_EO_PL         = 0x800280170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_BIT_STAT4_EO_PL       = 0x800280170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_BIT_STAT4_EO_PL       = 0x800280170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_BIT_STAT4_EO_PL       = 0x800280170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_BIT_STAT4_EO_PL       = 0x800280170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_BIT_STAT4_EO_PL        = 0x800280170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_BIT_STAT4_EO_PL        = 0x800280170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_BIT_STAT4_EO_PL       = 0x800280170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_BIT_STAT4_EO_PL       = 0x800280170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL10_O_PL           = 0x8000C8170701103Full;
//DUPS: 8000C8170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL10_O_PL         = 0x8000C8170701103Full;
//DUPS: 8000C8170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL10_O_PL         = 0x8000C8170801103Full;
//DUPS: 8000C8170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL10_O_PL         = 0x8000C8170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C8170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C8170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C8170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C8170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL10_O_PL        = 0x8000C8170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL10_O_PL        = 0x8000C8170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C8170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL10_O_PL       = 0x8000C8170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL1_EO_PL           = 0x800008170701103Full;
//DUPS: 800008170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL1_EO_PL         = 0x800008170701103Full;
//DUPS: 800008170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL1_EO_PL         = 0x800008170801103Full;
//DUPS: 800008170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL1_EO_PL         = 0x800008170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x800008170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x800008170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x800008170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x800008170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL1_EO_PL        = 0x800008170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL1_EO_PL        = 0x800008170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x800008170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL1_EO_PL       = 0x800008170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL1_O_PL            = 0x800080170701103Full;
//DUPS: 800080170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL1_O_PL          = 0x800080170701103Full;
//DUPS: 800080170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL1_O_PL          = 0x800080170801103Full;
//DUPS: 800080170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL1_O_PL          = 0x800080170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL1_O_PL        = 0x800080170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL1_O_PL        = 0x800080170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL1_O_PL        = 0x800080170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL1_O_PL        = 0x800080170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL1_O_PL         = 0x800080170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL1_O_PL         = 0x800080170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL1_O_PL        = 0x800080170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL1_O_PL        = 0x800080170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL2_EO_PL           = 0x800010170701103Full;
//DUPS: 800010170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL2_EO_PL         = 0x800010170701103Full;
//DUPS: 800010170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL2_EO_PL         = 0x800010170801103Full;
//DUPS: 800010170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL2_EO_PL         = 0x800010170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x800010170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x800010170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x800010170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x800010170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL2_EO_PL        = 0x800010170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL2_EO_PL        = 0x800010170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x800010170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL2_EO_PL       = 0x800010170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL2_O_PL            = 0x800088170701103Full;
//DUPS: 800088170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL2_O_PL          = 0x800088170701103Full;
//DUPS: 800088170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL2_O_PL          = 0x800088170801103Full;
//DUPS: 800088170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL2_O_PL          = 0x800088170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL2_O_PL        = 0x800088170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL2_O_PL        = 0x800088170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL2_O_PL        = 0x800088170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL2_O_PL        = 0x800088170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL2_O_PL         = 0x800088170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL2_O_PL         = 0x800088170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL2_O_PL        = 0x800088170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL2_O_PL        = 0x800088170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL3_EO_PL           = 0x800018170701103Full;
//DUPS: 800018170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL3_EO_PL         = 0x800018170701103Full;
//DUPS: 800018170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL3_EO_PL         = 0x800018170801103Full;
//DUPS: 800018170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL3_EO_PL         = 0x800018170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x800018170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x800018170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x800018170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x800018170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL3_EO_PL        = 0x800018170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL3_EO_PL        = 0x800018170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x800018170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL3_EO_PL       = 0x800018170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL3_O_PL            = 0x800090170701103Full;
//DUPS: 800090170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL3_O_PL          = 0x800090170701103Full;
//DUPS: 800090170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL3_O_PL          = 0x800090170801103Full;
//DUPS: 800090170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL3_O_PL          = 0x800090170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL3_O_PL        = 0x800090170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL3_O_PL        = 0x800090170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL3_O_PL        = 0x800090170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL3_O_PL        = 0x800090170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL3_O_PL         = 0x800090170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL3_O_PL         = 0x800090170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL3_O_PL        = 0x800090170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL3_O_PL        = 0x800090170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL4_EO_PL           = 0x800020170701103Full;
//DUPS: 800020170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL4_EO_PL         = 0x800020170701103Full;
//DUPS: 800020170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL4_EO_PL         = 0x800020170801103Full;
//DUPS: 800020170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL4_EO_PL         = 0x800020170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x800020170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x800020170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x800020170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x800020170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL4_EO_PL        = 0x800020170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL4_EO_PL        = 0x800020170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x800020170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL4_EO_PL       = 0x800020170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL4_O_PL            = 0x800098170701103Full;
//DUPS: 800098170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL4_O_PL          = 0x800098170701103Full;
//DUPS: 800098170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL4_O_PL          = 0x800098170801103Full;
//DUPS: 800098170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL4_O_PL          = 0x800098170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL4_O_PL        = 0x800098170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL4_O_PL        = 0x800098170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL4_O_PL        = 0x800098170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL4_O_PL        = 0x800098170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL4_O_PL         = 0x800098170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL4_O_PL         = 0x800098170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL4_O_PL        = 0x800098170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL4_O_PL        = 0x800098170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL5_EO_PL           = 0x800028170701103Full;
//DUPS: 800028170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL5_EO_PL         = 0x800028170701103Full;
//DUPS: 800028170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL5_EO_PL         = 0x800028170801103Full;
//DUPS: 800028170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL5_EO_PL         = 0x800028170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x800028170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x800028170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x800028170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x800028170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL5_EO_PL        = 0x800028170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL5_EO_PL        = 0x800028170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x800028170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL5_EO_PL       = 0x800028170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL5_O_PL            = 0x8000A0170701103Full;
//DUPS: 8000A0170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL5_O_PL          = 0x8000A0170701103Full;
//DUPS: 8000A0170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL5_O_PL          = 0x8000A0170801103Full;
//DUPS: 8000A0170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL5_O_PL          = 0x8000A0170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A0170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A0170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A0170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A0170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL5_O_PL         = 0x8000A0170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL5_O_PL         = 0x8000A0170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A0170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL5_O_PL        = 0x8000A0170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL6_EO_PL           = 0x800030170701103Full;
//DUPS: 800030170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL6_EO_PL         = 0x800030170701103Full;
//DUPS: 800030170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL6_EO_PL         = 0x800030170801103Full;
//DUPS: 800030170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL6_EO_PL         = 0x800030170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x800030170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x800030170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x800030170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x800030170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL6_EO_PL        = 0x800030170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL6_EO_PL        = 0x800030170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x800030170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL6_EO_PL       = 0x800030170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL6_O_PL            = 0x8000A8170701103Full;
//DUPS: 8000A8170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL6_O_PL          = 0x8000A8170701103Full;
//DUPS: 8000A8170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL6_O_PL          = 0x8000A8170801103Full;
//DUPS: 8000A8170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL6_O_PL          = 0x8000A8170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A8170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A8170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A8170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A8170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL6_O_PL         = 0x8000A8170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL6_O_PL         = 0x8000A8170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A8170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL6_O_PL        = 0x8000A8170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL7_EO_PL           = 0x800038170701103Full;
//DUPS: 800038170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL7_EO_PL         = 0x800038170701103Full;
//DUPS: 800038170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL7_EO_PL         = 0x800038170801103Full;
//DUPS: 800038170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL7_EO_PL         = 0x800038170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x800038170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x800038170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x800038170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x800038170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL7_EO_PL        = 0x800038170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL7_EO_PL        = 0x800038170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x800038170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL7_EO_PL       = 0x800038170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL7_O_PL            = 0x8000B0170701103Full;
//DUPS: 8000B0170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL7_O_PL          = 0x8000B0170701103Full;
//DUPS: 8000B0170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL7_O_PL          = 0x8000B0170801103Full;
//DUPS: 8000B0170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL7_O_PL          = 0x8000B0170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B0170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B0170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B0170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B0170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL7_O_PL         = 0x8000B0170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL7_O_PL         = 0x8000B0170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B0170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL7_O_PL        = 0x8000B0170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL8_EO_PL           = 0x800040170701103Full;
//DUPS: 800040170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL8_EO_PL         = 0x800040170701103Full;
//DUPS: 800040170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL8_EO_PL         = 0x800040170801103Full;
//DUPS: 800040170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL8_EO_PL         = 0x800040170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x800040170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x800040170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x800040170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x800040170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL8_EO_PL        = 0x800040170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL8_EO_PL        = 0x800040170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x800040170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL8_EO_PL       = 0x800040170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL8_O_PL            = 0x8000B8170701103Full;
//DUPS: 8000B8170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL8_O_PL          = 0x8000B8170701103Full;
//DUPS: 8000B8170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL8_O_PL          = 0x8000B8170801103Full;
//DUPS: 8000B8170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL8_O_PL          = 0x8000B8170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B8170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B8170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B8170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B8170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL8_O_PL         = 0x8000B8170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL8_O_PL         = 0x8000B8170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B8170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL8_O_PL        = 0x8000B8170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL9_O_PL            = 0x8000C0170701103Full;
//DUPS: 8000C0170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL9_O_PL          = 0x8000C0170701103Full;
//DUPS: 8000C0170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL9_O_PL          = 0x8000C0170801103Full;
//DUPS: 8000C0170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL9_O_PL          = 0x8000C0170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C0170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C0170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C0170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C0170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL9_O_PL         = 0x8000C0170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL9_O_PL         = 0x8000C0170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C0170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_DAC_CNTL9_O_PL        = 0x8000C0170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_DATA_DAC_SPARE_MODE_PL    = 0x800000170701103Full;
//DUPS: 800000170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000170701103Full;
//DUPS: 800000170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000170801103Full;
//DUPS: 800000170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_DATA_DAC_SPARE_MODE_PL  = 0x800000170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_DATA_DAC_SPARE_MODE_PL = 0x800000170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_FIR_ERROR_INJECT_PL       = 0x800218170701103Full;
//DUPS: 800218170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_FIR_ERROR_INJECT_PL     = 0x800218170701103Full;
//DUPS: 800218170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_FIR_ERROR_INJECT_PL     = 0x800218170801103Full;
//DUPS: 800218170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_FIR_ERROR_INJECT_PL     = 0x800218170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x800218170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x800218170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x800218170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x800218170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_FIR_ERROR_INJECT_PL    = 0x800218170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_FIR_ERROR_INJECT_PL    = 0x800218170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x800218170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_FIR_ERROR_INJECT_PL   = 0x800218170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_FIR_MASK_PL               = 0x800210170701103Full;
//DUPS: 800210170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_FIR_MASK_PL             = 0x800210170701103Full;
//DUPS: 800210170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_FIR_MASK_PL             = 0x800210170801103Full;
//DUPS: 800210170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_FIR_MASK_PL             = 0x800210170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_FIR_MASK_PL           = 0x800210170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_FIR_MASK_PL           = 0x800210170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_FIR_MASK_PL           = 0x800210170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_FIR_MASK_PL           = 0x800210170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_FIR_MASK_PL            = 0x800210170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_FIR_MASK_PL            = 0x800210170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_FIR_MASK_PL           = 0x800210170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_FIR_MASK_PL           = 0x800210170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_FIR_PL                    = 0x800208170701103Full;
//DUPS: 800208170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_FIR_PL                  = 0x800208170701103Full;
//DUPS: 800208170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_FIR_PL                  = 0x800208170801103Full;
//DUPS: 800208170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_FIR_PL                  = 0x800208170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_FIR_PL                = 0x800208170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_FIR_PL                = 0x800208170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_FIR_PL                = 0x800208170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_FIR_PL                = 0x800208170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_FIR_PL                 = 0x800208170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_FIR_PL                 = 0x800208170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_FIR_PL                = 0x800208170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_FIR_PL                = 0x800208170801143Full;


static const uint64_t P9A_MC_RX0_RXPACKS5_SLICE3_RX_SPARE_MODE_PL             = 0x800200170701103Full;
//DUPS: 800200170701143F,
static const uint64_t P9A_MC_0_RX0_RXPACKS5_SLICE3_RX_SPARE_MODE_PL           = 0x800200170701103Full;
//DUPS: 800200170701143F,
static const uint64_t P9A_MC_1_RX0_RXPACKS5_SLICE3_RX_SPARE_MODE_PL           = 0x800200170801103Full;
//DUPS: 800200170801143F,

static const uint64_t P9A_OMIC_RX0_RXPACKS5_SLICE3_RX_SPARE_MODE_PL           = 0x800200170701103Full;

static const uint64_t P9A_OMIC_0_RX0_RXPACKS5_SLICE3_RX_SPARE_MODE_PL         = 0x800200170701103Full;

static const uint64_t P9A_OMIC_1_RX0_RXPACKS5_SLICE3_RX_SPARE_MODE_PL         = 0x800200170701143Full;

static const uint64_t P9A_OMIC_3_RX0_RXPACKS5_SLICE3_RX_SPARE_MODE_PL         = 0x800200170801103Full;

static const uint64_t P9A_OMIC_4_RX0_RXPACKS5_SLICE3_RX_SPARE_MODE_PL         = 0x800200170801143Full;

static const uint64_t P9A_OMI_2_RX0_RXPACKS5_SLICE3_RX_SPARE_MODE_PL          = 0x800200170701103Full;

static const uint64_t P9A_OMI_5_RX0_RXPACKS5_SLICE3_RX_SPARE_MODE_PL          = 0x800200170701143Full;

static const uint64_t P9A_OMI_10_RX0_RXPACKS5_SLICE3_RX_SPARE_MODE_PL         = 0x800200170801103Full;

static const uint64_t P9A_OMI_13_RX0_RXPACKS5_SLICE3_RX_SPARE_MODE_PL         = 0x800200170801143Full;


static const uint64_t P9A_MC_RX0_SLICE0_RX_DATA_WORK_SPARE_MODE_PL            = 0x800380000701103Full;
//DUPS: 800380000701143F, 800380000701183F,
static const uint64_t P9A_MC_0_RX0_SLICE0_RX_DATA_WORK_SPARE_MODE_PL          = 0x800380000701103Full;
//DUPS: 800380000701143F, 800380000701183F,
static const uint64_t P9A_MC_1_RX0_SLICE0_RX_DATA_WORK_SPARE_MODE_PL          = 0x800380000801103Full;
//DUPS: 800380000801143F, 800380000801183F,

static const uint64_t P9A_OMI_RX0_SLICE0_RX_DATA_WORK_SPARE_MODE_PL           = 0x800380000701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE0_RX_DATA_WORK_SPARE_MODE_PL          = 0x800380000701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE0_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380000701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE0_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380000701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE0_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380000701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE0_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380000801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE0_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380000801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE0_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380000801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE0_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380000701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE0_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380000701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE0_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380000701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE0_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380000801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE0_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380000801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE0_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380000801183Full;


static const uint64_t P9A_MC_RX0_SLICE0_RX_GLBSM_PL_CNTL1X_O_PL               = 0x800330000701103Full;
//DUPS: 800330000701143F, 800330000701183F,
static const uint64_t P9A_MC_0_RX0_SLICE0_RX_GLBSM_PL_CNTL1X_O_PL             = 0x800330000701103Full;
//DUPS: 800330000701143F, 800330000701183F,
static const uint64_t P9A_MC_1_RX0_SLICE0_RX_GLBSM_PL_CNTL1X_O_PL             = 0x800330000801103Full;
//DUPS: 800330000801143F, 800330000801183F,

static const uint64_t P9A_OMI_RX0_SLICE0_RX_GLBSM_PL_CNTL1X_O_PL              = 0x800330000701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE0_RX_GLBSM_PL_CNTL1X_O_PL             = 0x800330000701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE0_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330000701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE0_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330000701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE0_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330000701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE0_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330000801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE0_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330000801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE0_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330000801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE0_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330000701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE0_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330000701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE0_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330000701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE0_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330000801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE0_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330000801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE0_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330000801183Full;


static const uint64_t P9A_MC_RX0_SLICE0_RX_GLBSM_PL_CNTL1_O_PL                = 0x800320000701103Full;
//DUPS: 800320000701143F, 800320000701183F,
static const uint64_t P9A_MC_0_RX0_SLICE0_RX_GLBSM_PL_CNTL1_O_PL              = 0x800320000701103Full;
//DUPS: 800320000701143F, 800320000701183F,
static const uint64_t P9A_MC_1_RX0_SLICE0_RX_GLBSM_PL_CNTL1_O_PL              = 0x800320000801103Full;
//DUPS: 800320000801143F, 800320000801183F,

static const uint64_t P9A_OMI_RX0_SLICE0_RX_GLBSM_PL_CNTL1_O_PL               = 0x800320000701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE0_RX_GLBSM_PL_CNTL1_O_PL              = 0x800320000701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE0_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320000701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE0_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320000701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE0_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320000701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE0_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320000801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE0_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320000801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE0_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320000801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE0_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320000701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE0_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320000701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE0_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320000701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE0_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320000801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE0_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320000801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE0_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320000801183Full;


static const uint64_t P9A_MC_RX0_SLICE0_RX_GLBSM_PL_STAT1_O_PL                = 0x800328000701103Full;
//DUPS: 800328000701143F, 800328000701183F,
static const uint64_t P9A_MC_0_RX0_SLICE0_RX_GLBSM_PL_STAT1_O_PL              = 0x800328000701103Full;
//DUPS: 800328000701143F, 800328000701183F,
static const uint64_t P9A_MC_1_RX0_SLICE0_RX_GLBSM_PL_STAT1_O_PL              = 0x800328000801103Full;
//DUPS: 800328000801143F, 800328000801183F,

static const uint64_t P9A_OMI_RX0_SLICE0_RX_GLBSM_PL_STAT1_O_PL               = 0x800328000701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE0_RX_GLBSM_PL_STAT1_O_PL              = 0x800328000701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE0_RX_GLBSM_PL_STAT1_O_PL            = 0x800328000701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE0_RX_GLBSM_PL_STAT1_O_PL            = 0x800328000701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE0_RX_GLBSM_PL_STAT1_O_PL            = 0x800328000701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE0_RX_GLBSM_PL_STAT1_O_PL            = 0x800328000801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE0_RX_GLBSM_PL_STAT1_O_PL            = 0x800328000801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE0_RX_GLBSM_PL_STAT1_O_PL            = 0x800328000801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE0_RX_GLBSM_PL_STAT1_O_PL             = 0x800328000701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE0_RX_GLBSM_PL_STAT1_O_PL             = 0x800328000701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE0_RX_GLBSM_PL_STAT1_O_PL             = 0x800328000701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE0_RX_GLBSM_PL_STAT1_O_PL             = 0x800328000801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE0_RX_GLBSM_PL_STAT1_O_PL            = 0x800328000801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE0_RX_GLBSM_PL_STAT1_O_PL            = 0x800328000801183Full;


static const uint64_t P9A_MC_RX0_SLICE0_RX_WORK_CNTL1_O_PL                    = 0x8003A8000701103Full;
//DUPS: 8003A8000701143F, 8003A8000701183F,
static const uint64_t P9A_MC_0_RX0_SLICE0_RX_WORK_CNTL1_O_PL                  = 0x8003A8000701103Full;
//DUPS: 8003A8000701143F, 8003A8000701183F,
static const uint64_t P9A_MC_1_RX0_SLICE0_RX_WORK_CNTL1_O_PL                  = 0x8003A8000801103Full;
//DUPS: 8003A8000801143F, 8003A8000801183F,

static const uint64_t P9A_OMI_RX0_SLICE0_RX_WORK_CNTL1_O_PL                   = 0x8003A8000701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE0_RX_WORK_CNTL1_O_PL                  = 0x8003A8000701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE0_RX_WORK_CNTL1_O_PL                = 0x8003A8000701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE0_RX_WORK_CNTL1_O_PL                = 0x8003A8000701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE0_RX_WORK_CNTL1_O_PL                = 0x8003A8000701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE0_RX_WORK_CNTL1_O_PL                = 0x8003A8000801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE0_RX_WORK_CNTL1_O_PL                = 0x8003A8000801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE0_RX_WORK_CNTL1_O_PL                = 0x8003A8000801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE0_RX_WORK_CNTL1_O_PL                 = 0x8003A8000701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE0_RX_WORK_CNTL1_O_PL                 = 0x8003A8000701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE0_RX_WORK_CNTL1_O_PL                 = 0x8003A8000701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE0_RX_WORK_CNTL1_O_PL                 = 0x8003A8000801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE0_RX_WORK_CNTL1_O_PL                = 0x8003A8000801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE0_RX_WORK_CNTL1_O_PL                = 0x8003A8000801183Full;


static const uint64_t P9A_MC_RX0_SLICE0_RX_WORK_STAT1_EO_PL                   = 0x800388000701103Full;
//DUPS: 800388000701143F, 800388000701183F,
static const uint64_t P9A_MC_0_RX0_SLICE0_RX_WORK_STAT1_EO_PL                 = 0x800388000701103Full;
//DUPS: 800388000701143F, 800388000701183F,
static const uint64_t P9A_MC_1_RX0_SLICE0_RX_WORK_STAT1_EO_PL                 = 0x800388000801103Full;
//DUPS: 800388000801143F, 800388000801183F,

static const uint64_t P9A_OMI_RX0_SLICE0_RX_WORK_STAT1_EO_PL                  = 0x800388000701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE0_RX_WORK_STAT1_EO_PL                 = 0x800388000701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE0_RX_WORK_STAT1_EO_PL               = 0x800388000701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE0_RX_WORK_STAT1_EO_PL               = 0x800388000701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE0_RX_WORK_STAT1_EO_PL               = 0x800388000701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE0_RX_WORK_STAT1_EO_PL               = 0x800388000801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE0_RX_WORK_STAT1_EO_PL               = 0x800388000801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE0_RX_WORK_STAT1_EO_PL               = 0x800388000801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE0_RX_WORK_STAT1_EO_PL                = 0x800388000701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE0_RX_WORK_STAT1_EO_PL                = 0x800388000701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE0_RX_WORK_STAT1_EO_PL                = 0x800388000701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE0_RX_WORK_STAT1_EO_PL                = 0x800388000801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE0_RX_WORK_STAT1_EO_PL               = 0x800388000801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE0_RX_WORK_STAT1_EO_PL               = 0x800388000801183Full;


static const uint64_t P9A_MC_RX0_SLICE0_RX_WORK_STAT2_EO_PL                   = 0x800390000701103Full;
//DUPS: 800390000701143F, 800390000701183F,
static const uint64_t P9A_MC_0_RX0_SLICE0_RX_WORK_STAT2_EO_PL                 = 0x800390000701103Full;
//DUPS: 800390000701143F, 800390000701183F,
static const uint64_t P9A_MC_1_RX0_SLICE0_RX_WORK_STAT2_EO_PL                 = 0x800390000801103Full;
//DUPS: 800390000801143F, 800390000801183F,

static const uint64_t P9A_OMI_RX0_SLICE0_RX_WORK_STAT2_EO_PL                  = 0x800390000701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE0_RX_WORK_STAT2_EO_PL                 = 0x800390000701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE0_RX_WORK_STAT2_EO_PL               = 0x800390000701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE0_RX_WORK_STAT2_EO_PL               = 0x800390000701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE0_RX_WORK_STAT2_EO_PL               = 0x800390000701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE0_RX_WORK_STAT2_EO_PL               = 0x800390000801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE0_RX_WORK_STAT2_EO_PL               = 0x800390000801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE0_RX_WORK_STAT2_EO_PL               = 0x800390000801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE0_RX_WORK_STAT2_EO_PL                = 0x800390000701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE0_RX_WORK_STAT2_EO_PL                = 0x800390000701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE0_RX_WORK_STAT2_EO_PL                = 0x800390000701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE0_RX_WORK_STAT2_EO_PL                = 0x800390000801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE0_RX_WORK_STAT2_EO_PL               = 0x800390000801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE0_RX_WORK_STAT2_EO_PL               = 0x800390000801183Full;


static const uint64_t P9A_MC_RX0_SLICE0_RX_WORK_STAT3_EO_PL                   = 0x800398000701103Full;
//DUPS: 800398000701143F, 800398000701183F,
static const uint64_t P9A_MC_0_RX0_SLICE0_RX_WORK_STAT3_EO_PL                 = 0x800398000701103Full;
//DUPS: 800398000701143F, 800398000701183F,
static const uint64_t P9A_MC_1_RX0_SLICE0_RX_WORK_STAT3_EO_PL                 = 0x800398000801103Full;
//DUPS: 800398000801143F, 800398000801183F,

static const uint64_t P9A_OMI_RX0_SLICE0_RX_WORK_STAT3_EO_PL                  = 0x800398000701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE0_RX_WORK_STAT3_EO_PL                 = 0x800398000701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE0_RX_WORK_STAT3_EO_PL               = 0x800398000701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE0_RX_WORK_STAT3_EO_PL               = 0x800398000701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE0_RX_WORK_STAT3_EO_PL               = 0x800398000701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE0_RX_WORK_STAT3_EO_PL               = 0x800398000801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE0_RX_WORK_STAT3_EO_PL               = 0x800398000801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE0_RX_WORK_STAT3_EO_PL               = 0x800398000801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE0_RX_WORK_STAT3_EO_PL                = 0x800398000701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE0_RX_WORK_STAT3_EO_PL                = 0x800398000701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE0_RX_WORK_STAT3_EO_PL                = 0x800398000701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE0_RX_WORK_STAT3_EO_PL                = 0x800398000801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE0_RX_WORK_STAT3_EO_PL               = 0x800398000801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE0_RX_WORK_STAT3_EO_PL               = 0x800398000801183Full;


static const uint64_t P9A_MC_RX0_SLICE0_RX_WORK_STAT4_O_PL                    = 0x8003A0000701103Full;
//DUPS: 8003A0000701143F, 8003A0000701183F,
static const uint64_t P9A_MC_0_RX0_SLICE0_RX_WORK_STAT4_O_PL                  = 0x8003A0000701103Full;
//DUPS: 8003A0000701143F, 8003A0000701183F,
static const uint64_t P9A_MC_1_RX0_SLICE0_RX_WORK_STAT4_O_PL                  = 0x8003A0000801103Full;
//DUPS: 8003A0000801143F, 8003A0000801183F,

static const uint64_t P9A_OMI_RX0_SLICE0_RX_WORK_STAT4_O_PL                   = 0x8003A0000701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE0_RX_WORK_STAT4_O_PL                  = 0x8003A0000701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE0_RX_WORK_STAT4_O_PL                = 0x8003A0000701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE0_RX_WORK_STAT4_O_PL                = 0x8003A0000701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE0_RX_WORK_STAT4_O_PL                = 0x8003A0000701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE0_RX_WORK_STAT4_O_PL                = 0x8003A0000801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE0_RX_WORK_STAT4_O_PL                = 0x8003A0000801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE0_RX_WORK_STAT4_O_PL                = 0x8003A0000801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE0_RX_WORK_STAT4_O_PL                 = 0x8003A0000701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE0_RX_WORK_STAT4_O_PL                 = 0x8003A0000701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE0_RX_WORK_STAT4_O_PL                 = 0x8003A0000701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE0_RX_WORK_STAT4_O_PL                 = 0x8003A0000801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE0_RX_WORK_STAT4_O_PL                = 0x8003A0000801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE0_RX_WORK_STAT4_O_PL                = 0x8003A0000801183Full;


static const uint64_t P9A_MC_RX0_SLICE10_RX_DATA_WORK_SPARE_MODE_PL           = 0x8003800A0701103Full;
//DUPS: 8003800A0701143F, 8003800A0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE10_RX_DATA_WORK_SPARE_MODE_PL         = 0x8003800A0701103Full;
//DUPS: 8003800A0701143F, 8003800A0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE10_RX_DATA_WORK_SPARE_MODE_PL         = 0x8003800A0801103Full;
//DUPS: 8003800A0801143F, 8003800A0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE10_RX_DATA_WORK_SPARE_MODE_PL         = 0x8003800A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE10_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE10_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE10_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800A0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE10_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE10_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE10_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800A0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE10_RX_DATA_WORK_SPARE_MODE_PL        = 0x8003800A0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE10_RX_DATA_WORK_SPARE_MODE_PL        = 0x8003800A0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE10_RX_DATA_WORK_SPARE_MODE_PL        = 0x8003800A0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE10_RX_DATA_WORK_SPARE_MODE_PL        = 0x8003800A0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE10_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800A0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE10_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800A0801183Full;


static const uint64_t P9A_MC_RX0_SLICE10_RX_GLBSM_PL_CNTL1X_O_PL              = 0x8003300A0701103Full;
//DUPS: 8003300A0701143F, 8003300A0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE10_RX_GLBSM_PL_CNTL1X_O_PL            = 0x8003300A0701103Full;
//DUPS: 8003300A0701143F, 8003300A0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE10_RX_GLBSM_PL_CNTL1X_O_PL            = 0x8003300A0801103Full;
//DUPS: 8003300A0801143F, 8003300A0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE10_RX_GLBSM_PL_CNTL1X_O_PL            = 0x8003300A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE10_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE10_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE10_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300A0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE10_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE10_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE10_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300A0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE10_RX_GLBSM_PL_CNTL1X_O_PL           = 0x8003300A0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE10_RX_GLBSM_PL_CNTL1X_O_PL           = 0x8003300A0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE10_RX_GLBSM_PL_CNTL1X_O_PL           = 0x8003300A0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE10_RX_GLBSM_PL_CNTL1X_O_PL           = 0x8003300A0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE10_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300A0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE10_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300A0801183Full;


static const uint64_t P9A_MC_RX0_SLICE10_RX_GLBSM_PL_CNTL1_O_PL               = 0x8003200A0701103Full;
//DUPS: 8003200A0701143F, 8003200A0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE10_RX_GLBSM_PL_CNTL1_O_PL             = 0x8003200A0701103Full;
//DUPS: 8003200A0701143F, 8003200A0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE10_RX_GLBSM_PL_CNTL1_O_PL             = 0x8003200A0801103Full;
//DUPS: 8003200A0801143F, 8003200A0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE10_RX_GLBSM_PL_CNTL1_O_PL             = 0x8003200A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE10_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE10_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE10_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200A0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE10_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE10_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE10_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200A0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE10_RX_GLBSM_PL_CNTL1_O_PL            = 0x8003200A0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE10_RX_GLBSM_PL_CNTL1_O_PL            = 0x8003200A0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE10_RX_GLBSM_PL_CNTL1_O_PL            = 0x8003200A0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE10_RX_GLBSM_PL_CNTL1_O_PL            = 0x8003200A0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE10_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200A0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE10_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200A0801183Full;


static const uint64_t P9A_MC_RX0_SLICE10_RX_GLBSM_PL_STAT1_O_PL               = 0x8003280A0701103Full;
//DUPS: 8003280A0701143F, 8003280A0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE10_RX_GLBSM_PL_STAT1_O_PL             = 0x8003280A0701103Full;
//DUPS: 8003280A0701143F, 8003280A0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE10_RX_GLBSM_PL_STAT1_O_PL             = 0x8003280A0801103Full;
//DUPS: 8003280A0801143F, 8003280A0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE10_RX_GLBSM_PL_STAT1_O_PL             = 0x8003280A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE10_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE10_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE10_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280A0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE10_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE10_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE10_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280A0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE10_RX_GLBSM_PL_STAT1_O_PL            = 0x8003280A0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE10_RX_GLBSM_PL_STAT1_O_PL            = 0x8003280A0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE10_RX_GLBSM_PL_STAT1_O_PL            = 0x8003280A0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE10_RX_GLBSM_PL_STAT1_O_PL            = 0x8003280A0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE10_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280A0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE10_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280A0801183Full;


static const uint64_t P9A_MC_RX0_SLICE10_RX_WORK_CNTL1_O_PL                   = 0x8003A80A0701103Full;
//DUPS: 8003A80A0701143F, 8003A80A0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE10_RX_WORK_CNTL1_O_PL                 = 0x8003A80A0701103Full;
//DUPS: 8003A80A0701143F, 8003A80A0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE10_RX_WORK_CNTL1_O_PL                 = 0x8003A80A0801103Full;
//DUPS: 8003A80A0801143F, 8003A80A0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE10_RX_WORK_CNTL1_O_PL                 = 0x8003A80A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE10_RX_WORK_CNTL1_O_PL               = 0x8003A80A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE10_RX_WORK_CNTL1_O_PL               = 0x8003A80A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE10_RX_WORK_CNTL1_O_PL               = 0x8003A80A0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE10_RX_WORK_CNTL1_O_PL               = 0x8003A80A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE10_RX_WORK_CNTL1_O_PL               = 0x8003A80A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE10_RX_WORK_CNTL1_O_PL               = 0x8003A80A0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE10_RX_WORK_CNTL1_O_PL                = 0x8003A80A0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE10_RX_WORK_CNTL1_O_PL                = 0x8003A80A0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE10_RX_WORK_CNTL1_O_PL                = 0x8003A80A0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE10_RX_WORK_CNTL1_O_PL                = 0x8003A80A0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE10_RX_WORK_CNTL1_O_PL               = 0x8003A80A0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE10_RX_WORK_CNTL1_O_PL               = 0x8003A80A0801183Full;


static const uint64_t P9A_MC_RX0_SLICE10_RX_WORK_STAT1_EO_PL                  = 0x8003880A0701103Full;
//DUPS: 8003880A0701143F, 8003880A0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE10_RX_WORK_STAT1_EO_PL                = 0x8003880A0701103Full;
//DUPS: 8003880A0701143F, 8003880A0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE10_RX_WORK_STAT1_EO_PL                = 0x8003880A0801103Full;
//DUPS: 8003880A0801143F, 8003880A0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE10_RX_WORK_STAT1_EO_PL                = 0x8003880A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE10_RX_WORK_STAT1_EO_PL              = 0x8003880A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE10_RX_WORK_STAT1_EO_PL              = 0x8003880A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE10_RX_WORK_STAT1_EO_PL              = 0x8003880A0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE10_RX_WORK_STAT1_EO_PL              = 0x8003880A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE10_RX_WORK_STAT1_EO_PL              = 0x8003880A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE10_RX_WORK_STAT1_EO_PL              = 0x8003880A0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE10_RX_WORK_STAT1_EO_PL               = 0x8003880A0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE10_RX_WORK_STAT1_EO_PL               = 0x8003880A0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE10_RX_WORK_STAT1_EO_PL               = 0x8003880A0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE10_RX_WORK_STAT1_EO_PL               = 0x8003880A0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE10_RX_WORK_STAT1_EO_PL              = 0x8003880A0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE10_RX_WORK_STAT1_EO_PL              = 0x8003880A0801183Full;


static const uint64_t P9A_MC_RX0_SLICE10_RX_WORK_STAT2_EO_PL                  = 0x8003900A0701103Full;
//DUPS: 8003900A0701143F, 8003900A0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE10_RX_WORK_STAT2_EO_PL                = 0x8003900A0701103Full;
//DUPS: 8003900A0701143F, 8003900A0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE10_RX_WORK_STAT2_EO_PL                = 0x8003900A0801103Full;
//DUPS: 8003900A0801143F, 8003900A0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE10_RX_WORK_STAT2_EO_PL                = 0x8003900A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE10_RX_WORK_STAT2_EO_PL              = 0x8003900A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE10_RX_WORK_STAT2_EO_PL              = 0x8003900A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE10_RX_WORK_STAT2_EO_PL              = 0x8003900A0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE10_RX_WORK_STAT2_EO_PL              = 0x8003900A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE10_RX_WORK_STAT2_EO_PL              = 0x8003900A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE10_RX_WORK_STAT2_EO_PL              = 0x8003900A0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE10_RX_WORK_STAT2_EO_PL               = 0x8003900A0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE10_RX_WORK_STAT2_EO_PL               = 0x8003900A0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE10_RX_WORK_STAT2_EO_PL               = 0x8003900A0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE10_RX_WORK_STAT2_EO_PL               = 0x8003900A0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE10_RX_WORK_STAT2_EO_PL              = 0x8003900A0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE10_RX_WORK_STAT2_EO_PL              = 0x8003900A0801183Full;


static const uint64_t P9A_MC_RX0_SLICE10_RX_WORK_STAT3_EO_PL                  = 0x8003980A0701103Full;
//DUPS: 8003980A0701143F, 8003980A0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE10_RX_WORK_STAT3_EO_PL                = 0x8003980A0701103Full;
//DUPS: 8003980A0701143F, 8003980A0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE10_RX_WORK_STAT3_EO_PL                = 0x8003980A0801103Full;
//DUPS: 8003980A0801143F, 8003980A0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE10_RX_WORK_STAT3_EO_PL                = 0x8003980A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE10_RX_WORK_STAT3_EO_PL              = 0x8003980A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE10_RX_WORK_STAT3_EO_PL              = 0x8003980A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE10_RX_WORK_STAT3_EO_PL              = 0x8003980A0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE10_RX_WORK_STAT3_EO_PL              = 0x8003980A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE10_RX_WORK_STAT3_EO_PL              = 0x8003980A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE10_RX_WORK_STAT3_EO_PL              = 0x8003980A0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE10_RX_WORK_STAT3_EO_PL               = 0x8003980A0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE10_RX_WORK_STAT3_EO_PL               = 0x8003980A0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE10_RX_WORK_STAT3_EO_PL               = 0x8003980A0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE10_RX_WORK_STAT3_EO_PL               = 0x8003980A0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE10_RX_WORK_STAT3_EO_PL              = 0x8003980A0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE10_RX_WORK_STAT3_EO_PL              = 0x8003980A0801183Full;


static const uint64_t P9A_MC_RX0_SLICE10_RX_WORK_STAT4_O_PL                   = 0x8003A00A0701103Full;
//DUPS: 8003A00A0701143F, 8003A00A0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE10_RX_WORK_STAT4_O_PL                 = 0x8003A00A0701103Full;
//DUPS: 8003A00A0701143F, 8003A00A0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE10_RX_WORK_STAT4_O_PL                 = 0x8003A00A0801103Full;
//DUPS: 8003A00A0801143F, 8003A00A0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE10_RX_WORK_STAT4_O_PL                 = 0x8003A00A0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE10_RX_WORK_STAT4_O_PL               = 0x8003A00A0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE10_RX_WORK_STAT4_O_PL               = 0x8003A00A0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE10_RX_WORK_STAT4_O_PL               = 0x8003A00A0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE10_RX_WORK_STAT4_O_PL               = 0x8003A00A0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE10_RX_WORK_STAT4_O_PL               = 0x8003A00A0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE10_RX_WORK_STAT4_O_PL               = 0x8003A00A0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE10_RX_WORK_STAT4_O_PL                = 0x8003A00A0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE10_RX_WORK_STAT4_O_PL                = 0x8003A00A0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE10_RX_WORK_STAT4_O_PL                = 0x8003A00A0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE10_RX_WORK_STAT4_O_PL                = 0x8003A00A0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE10_RX_WORK_STAT4_O_PL               = 0x8003A00A0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE10_RX_WORK_STAT4_O_PL               = 0x8003A00A0801183Full;


static const uint64_t P9A_MC_RX0_SLICE11_RX_DATA_WORK_SPARE_MODE_PL           = 0x8003800B0701103Full;
//DUPS: 8003800B0701143F, 8003800B0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE11_RX_DATA_WORK_SPARE_MODE_PL         = 0x8003800B0701103Full;
//DUPS: 8003800B0701143F, 8003800B0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE11_RX_DATA_WORK_SPARE_MODE_PL         = 0x8003800B0801103Full;
//DUPS: 8003800B0801143F, 8003800B0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE11_RX_DATA_WORK_SPARE_MODE_PL         = 0x8003800B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE11_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE11_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE11_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800B0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE11_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE11_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE11_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800B0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE11_RX_DATA_WORK_SPARE_MODE_PL        = 0x8003800B0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE11_RX_DATA_WORK_SPARE_MODE_PL        = 0x8003800B0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE11_RX_DATA_WORK_SPARE_MODE_PL        = 0x8003800B0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE11_RX_DATA_WORK_SPARE_MODE_PL        = 0x8003800B0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE11_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800B0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE11_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800B0801183Full;


static const uint64_t P9A_MC_RX0_SLICE11_RX_GLBSM_PL_CNTL1X_O_PL              = 0x8003300B0701103Full;
//DUPS: 8003300B0701143F, 8003300B0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE11_RX_GLBSM_PL_CNTL1X_O_PL            = 0x8003300B0701103Full;
//DUPS: 8003300B0701143F, 8003300B0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE11_RX_GLBSM_PL_CNTL1X_O_PL            = 0x8003300B0801103Full;
//DUPS: 8003300B0801143F, 8003300B0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE11_RX_GLBSM_PL_CNTL1X_O_PL            = 0x8003300B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE11_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE11_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE11_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300B0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE11_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE11_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE11_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300B0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE11_RX_GLBSM_PL_CNTL1X_O_PL           = 0x8003300B0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE11_RX_GLBSM_PL_CNTL1X_O_PL           = 0x8003300B0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE11_RX_GLBSM_PL_CNTL1X_O_PL           = 0x8003300B0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE11_RX_GLBSM_PL_CNTL1X_O_PL           = 0x8003300B0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE11_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300B0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE11_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300B0801183Full;


static const uint64_t P9A_MC_RX0_SLICE11_RX_GLBSM_PL_CNTL1_O_PL               = 0x8003200B0701103Full;
//DUPS: 8003200B0701143F, 8003200B0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE11_RX_GLBSM_PL_CNTL1_O_PL             = 0x8003200B0701103Full;
//DUPS: 8003200B0701143F, 8003200B0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE11_RX_GLBSM_PL_CNTL1_O_PL             = 0x8003200B0801103Full;
//DUPS: 8003200B0801143F, 8003200B0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE11_RX_GLBSM_PL_CNTL1_O_PL             = 0x8003200B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE11_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE11_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE11_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200B0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE11_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE11_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE11_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200B0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE11_RX_GLBSM_PL_CNTL1_O_PL            = 0x8003200B0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE11_RX_GLBSM_PL_CNTL1_O_PL            = 0x8003200B0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE11_RX_GLBSM_PL_CNTL1_O_PL            = 0x8003200B0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE11_RX_GLBSM_PL_CNTL1_O_PL            = 0x8003200B0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE11_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200B0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE11_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200B0801183Full;


static const uint64_t P9A_MC_RX0_SLICE11_RX_GLBSM_PL_STAT1_O_PL               = 0x8003280B0701103Full;
//DUPS: 8003280B0701143F, 8003280B0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE11_RX_GLBSM_PL_STAT1_O_PL             = 0x8003280B0701103Full;
//DUPS: 8003280B0701143F, 8003280B0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE11_RX_GLBSM_PL_STAT1_O_PL             = 0x8003280B0801103Full;
//DUPS: 8003280B0801143F, 8003280B0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE11_RX_GLBSM_PL_STAT1_O_PL             = 0x8003280B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE11_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE11_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE11_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280B0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE11_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE11_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE11_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280B0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE11_RX_GLBSM_PL_STAT1_O_PL            = 0x8003280B0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE11_RX_GLBSM_PL_STAT1_O_PL            = 0x8003280B0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE11_RX_GLBSM_PL_STAT1_O_PL            = 0x8003280B0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE11_RX_GLBSM_PL_STAT1_O_PL            = 0x8003280B0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE11_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280B0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE11_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280B0801183Full;


static const uint64_t P9A_MC_RX0_SLICE11_RX_WORK_CNTL1_O_PL                   = 0x8003A80B0701103Full;
//DUPS: 8003A80B0701143F, 8003A80B0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE11_RX_WORK_CNTL1_O_PL                 = 0x8003A80B0701103Full;
//DUPS: 8003A80B0701143F, 8003A80B0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE11_RX_WORK_CNTL1_O_PL                 = 0x8003A80B0801103Full;
//DUPS: 8003A80B0801143F, 8003A80B0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE11_RX_WORK_CNTL1_O_PL                 = 0x8003A80B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE11_RX_WORK_CNTL1_O_PL               = 0x8003A80B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE11_RX_WORK_CNTL1_O_PL               = 0x8003A80B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE11_RX_WORK_CNTL1_O_PL               = 0x8003A80B0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE11_RX_WORK_CNTL1_O_PL               = 0x8003A80B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE11_RX_WORK_CNTL1_O_PL               = 0x8003A80B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE11_RX_WORK_CNTL1_O_PL               = 0x8003A80B0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE11_RX_WORK_CNTL1_O_PL                = 0x8003A80B0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE11_RX_WORK_CNTL1_O_PL                = 0x8003A80B0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE11_RX_WORK_CNTL1_O_PL                = 0x8003A80B0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE11_RX_WORK_CNTL1_O_PL                = 0x8003A80B0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE11_RX_WORK_CNTL1_O_PL               = 0x8003A80B0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE11_RX_WORK_CNTL1_O_PL               = 0x8003A80B0801183Full;


static const uint64_t P9A_MC_RX0_SLICE11_RX_WORK_STAT1_EO_PL                  = 0x8003880B0701103Full;
//DUPS: 8003880B0701143F, 8003880B0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE11_RX_WORK_STAT1_EO_PL                = 0x8003880B0701103Full;
//DUPS: 8003880B0701143F, 8003880B0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE11_RX_WORK_STAT1_EO_PL                = 0x8003880B0801103Full;
//DUPS: 8003880B0801143F, 8003880B0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE11_RX_WORK_STAT1_EO_PL                = 0x8003880B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE11_RX_WORK_STAT1_EO_PL              = 0x8003880B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE11_RX_WORK_STAT1_EO_PL              = 0x8003880B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE11_RX_WORK_STAT1_EO_PL              = 0x8003880B0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE11_RX_WORK_STAT1_EO_PL              = 0x8003880B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE11_RX_WORK_STAT1_EO_PL              = 0x8003880B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE11_RX_WORK_STAT1_EO_PL              = 0x8003880B0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE11_RX_WORK_STAT1_EO_PL               = 0x8003880B0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE11_RX_WORK_STAT1_EO_PL               = 0x8003880B0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE11_RX_WORK_STAT1_EO_PL               = 0x8003880B0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE11_RX_WORK_STAT1_EO_PL               = 0x8003880B0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE11_RX_WORK_STAT1_EO_PL              = 0x8003880B0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE11_RX_WORK_STAT1_EO_PL              = 0x8003880B0801183Full;


static const uint64_t P9A_MC_RX0_SLICE11_RX_WORK_STAT2_EO_PL                  = 0x8003900B0701103Full;
//DUPS: 8003900B0701143F, 8003900B0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE11_RX_WORK_STAT2_EO_PL                = 0x8003900B0701103Full;
//DUPS: 8003900B0701143F, 8003900B0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE11_RX_WORK_STAT2_EO_PL                = 0x8003900B0801103Full;
//DUPS: 8003900B0801143F, 8003900B0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE11_RX_WORK_STAT2_EO_PL                = 0x8003900B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE11_RX_WORK_STAT2_EO_PL              = 0x8003900B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE11_RX_WORK_STAT2_EO_PL              = 0x8003900B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE11_RX_WORK_STAT2_EO_PL              = 0x8003900B0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE11_RX_WORK_STAT2_EO_PL              = 0x8003900B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE11_RX_WORK_STAT2_EO_PL              = 0x8003900B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE11_RX_WORK_STAT2_EO_PL              = 0x8003900B0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE11_RX_WORK_STAT2_EO_PL               = 0x8003900B0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE11_RX_WORK_STAT2_EO_PL               = 0x8003900B0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE11_RX_WORK_STAT2_EO_PL               = 0x8003900B0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE11_RX_WORK_STAT2_EO_PL               = 0x8003900B0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE11_RX_WORK_STAT2_EO_PL              = 0x8003900B0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE11_RX_WORK_STAT2_EO_PL              = 0x8003900B0801183Full;


static const uint64_t P9A_MC_RX0_SLICE11_RX_WORK_STAT3_EO_PL                  = 0x8003980B0701103Full;
//DUPS: 8003980B0701143F, 8003980B0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE11_RX_WORK_STAT3_EO_PL                = 0x8003980B0701103Full;
//DUPS: 8003980B0701143F, 8003980B0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE11_RX_WORK_STAT3_EO_PL                = 0x8003980B0801103Full;
//DUPS: 8003980B0801143F, 8003980B0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE11_RX_WORK_STAT3_EO_PL                = 0x8003980B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE11_RX_WORK_STAT3_EO_PL              = 0x8003980B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE11_RX_WORK_STAT3_EO_PL              = 0x8003980B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE11_RX_WORK_STAT3_EO_PL              = 0x8003980B0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE11_RX_WORK_STAT3_EO_PL              = 0x8003980B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE11_RX_WORK_STAT3_EO_PL              = 0x8003980B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE11_RX_WORK_STAT3_EO_PL              = 0x8003980B0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE11_RX_WORK_STAT3_EO_PL               = 0x8003980B0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE11_RX_WORK_STAT3_EO_PL               = 0x8003980B0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE11_RX_WORK_STAT3_EO_PL               = 0x8003980B0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE11_RX_WORK_STAT3_EO_PL               = 0x8003980B0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE11_RX_WORK_STAT3_EO_PL              = 0x8003980B0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE11_RX_WORK_STAT3_EO_PL              = 0x8003980B0801183Full;


static const uint64_t P9A_MC_RX0_SLICE11_RX_WORK_STAT4_O_PL                   = 0x8003A00B0701103Full;
//DUPS: 8003A00B0701143F, 8003A00B0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE11_RX_WORK_STAT4_O_PL                 = 0x8003A00B0701103Full;
//DUPS: 8003A00B0701143F, 8003A00B0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE11_RX_WORK_STAT4_O_PL                 = 0x8003A00B0801103Full;
//DUPS: 8003A00B0801143F, 8003A00B0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE11_RX_WORK_STAT4_O_PL                 = 0x8003A00B0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE11_RX_WORK_STAT4_O_PL               = 0x8003A00B0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE11_RX_WORK_STAT4_O_PL               = 0x8003A00B0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE11_RX_WORK_STAT4_O_PL               = 0x8003A00B0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE11_RX_WORK_STAT4_O_PL               = 0x8003A00B0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE11_RX_WORK_STAT4_O_PL               = 0x8003A00B0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE11_RX_WORK_STAT4_O_PL               = 0x8003A00B0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE11_RX_WORK_STAT4_O_PL                = 0x8003A00B0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE11_RX_WORK_STAT4_O_PL                = 0x8003A00B0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE11_RX_WORK_STAT4_O_PL                = 0x8003A00B0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE11_RX_WORK_STAT4_O_PL                = 0x8003A00B0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE11_RX_WORK_STAT4_O_PL               = 0x8003A00B0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE11_RX_WORK_STAT4_O_PL               = 0x8003A00B0801183Full;


static const uint64_t P9A_MC_RX0_SLICE12_RX_DATA_WORK_SPARE_MODE_PL           = 0x8003800C0701103Full;
//DUPS: 8003800C0701143F, 8003800C0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE12_RX_DATA_WORK_SPARE_MODE_PL         = 0x8003800C0701103Full;
//DUPS: 8003800C0701143F, 8003800C0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE12_RX_DATA_WORK_SPARE_MODE_PL         = 0x8003800C0801103Full;
//DUPS: 8003800C0801143F, 8003800C0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE12_RX_DATA_WORK_SPARE_MODE_PL         = 0x8003800C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE12_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE12_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800C0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE12_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE12_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE12_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800C0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE12_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800C0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE12_RX_DATA_WORK_SPARE_MODE_PL        = 0x8003800C0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE12_RX_DATA_WORK_SPARE_MODE_PL        = 0x8003800C0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE12_RX_DATA_WORK_SPARE_MODE_PL        = 0x8003800C0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE12_RX_DATA_WORK_SPARE_MODE_PL        = 0x8003800C0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE12_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800C0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE12_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800C0801183Full;


static const uint64_t P9A_MC_RX0_SLICE12_RX_GLBSM_PL_CNTL1X_O_PL              = 0x8003300C0701103Full;
//DUPS: 8003300C0701143F, 8003300C0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE12_RX_GLBSM_PL_CNTL1X_O_PL            = 0x8003300C0701103Full;
//DUPS: 8003300C0701143F, 8003300C0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE12_RX_GLBSM_PL_CNTL1X_O_PL            = 0x8003300C0801103Full;
//DUPS: 8003300C0801143F, 8003300C0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE12_RX_GLBSM_PL_CNTL1X_O_PL            = 0x8003300C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE12_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE12_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300C0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE12_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE12_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE12_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300C0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE12_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300C0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE12_RX_GLBSM_PL_CNTL1X_O_PL           = 0x8003300C0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE12_RX_GLBSM_PL_CNTL1X_O_PL           = 0x8003300C0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE12_RX_GLBSM_PL_CNTL1X_O_PL           = 0x8003300C0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE12_RX_GLBSM_PL_CNTL1X_O_PL           = 0x8003300C0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE12_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300C0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE12_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300C0801183Full;


static const uint64_t P9A_MC_RX0_SLICE12_RX_GLBSM_PL_CNTL1_O_PL               = 0x8003200C0701103Full;
//DUPS: 8003200C0701143F, 8003200C0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE12_RX_GLBSM_PL_CNTL1_O_PL             = 0x8003200C0701103Full;
//DUPS: 8003200C0701143F, 8003200C0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE12_RX_GLBSM_PL_CNTL1_O_PL             = 0x8003200C0801103Full;
//DUPS: 8003200C0801143F, 8003200C0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE12_RX_GLBSM_PL_CNTL1_O_PL             = 0x8003200C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE12_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE12_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200C0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE12_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE12_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE12_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200C0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE12_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200C0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE12_RX_GLBSM_PL_CNTL1_O_PL            = 0x8003200C0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE12_RX_GLBSM_PL_CNTL1_O_PL            = 0x8003200C0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE12_RX_GLBSM_PL_CNTL1_O_PL            = 0x8003200C0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE12_RX_GLBSM_PL_CNTL1_O_PL            = 0x8003200C0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE12_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200C0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE12_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200C0801183Full;


static const uint64_t P9A_MC_RX0_SLICE12_RX_GLBSM_PL_STAT1_O_PL               = 0x8003280C0701103Full;
//DUPS: 8003280C0701143F, 8003280C0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE12_RX_GLBSM_PL_STAT1_O_PL             = 0x8003280C0701103Full;
//DUPS: 8003280C0701143F, 8003280C0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE12_RX_GLBSM_PL_STAT1_O_PL             = 0x8003280C0801103Full;
//DUPS: 8003280C0801143F, 8003280C0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE12_RX_GLBSM_PL_STAT1_O_PL             = 0x8003280C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE12_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE12_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280C0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE12_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE12_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE12_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280C0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE12_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280C0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE12_RX_GLBSM_PL_STAT1_O_PL            = 0x8003280C0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE12_RX_GLBSM_PL_STAT1_O_PL            = 0x8003280C0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE12_RX_GLBSM_PL_STAT1_O_PL            = 0x8003280C0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE12_RX_GLBSM_PL_STAT1_O_PL            = 0x8003280C0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE12_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280C0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE12_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280C0801183Full;


static const uint64_t P9A_MC_RX0_SLICE12_RX_WORK_CNTL1_O_PL                   = 0x8003A80C0701103Full;
//DUPS: 8003A80C0701143F, 8003A80C0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE12_RX_WORK_CNTL1_O_PL                 = 0x8003A80C0701103Full;
//DUPS: 8003A80C0701143F, 8003A80C0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE12_RX_WORK_CNTL1_O_PL                 = 0x8003A80C0801103Full;
//DUPS: 8003A80C0801143F, 8003A80C0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE12_RX_WORK_CNTL1_O_PL                 = 0x8003A80C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE12_RX_WORK_CNTL1_O_PL               = 0x8003A80C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE12_RX_WORK_CNTL1_O_PL               = 0x8003A80C0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE12_RX_WORK_CNTL1_O_PL               = 0x8003A80C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE12_RX_WORK_CNTL1_O_PL               = 0x8003A80C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE12_RX_WORK_CNTL1_O_PL               = 0x8003A80C0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE12_RX_WORK_CNTL1_O_PL               = 0x8003A80C0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE12_RX_WORK_CNTL1_O_PL                = 0x8003A80C0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE12_RX_WORK_CNTL1_O_PL                = 0x8003A80C0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE12_RX_WORK_CNTL1_O_PL                = 0x8003A80C0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE12_RX_WORK_CNTL1_O_PL                = 0x8003A80C0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE12_RX_WORK_CNTL1_O_PL               = 0x8003A80C0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE12_RX_WORK_CNTL1_O_PL               = 0x8003A80C0801183Full;


static const uint64_t P9A_MC_RX0_SLICE12_RX_WORK_STAT1_EO_PL                  = 0x8003880C0701103Full;
//DUPS: 8003880C0701143F, 8003880C0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE12_RX_WORK_STAT1_EO_PL                = 0x8003880C0701103Full;
//DUPS: 8003880C0701143F, 8003880C0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE12_RX_WORK_STAT1_EO_PL                = 0x8003880C0801103Full;
//DUPS: 8003880C0801143F, 8003880C0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE12_RX_WORK_STAT1_EO_PL                = 0x8003880C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE12_RX_WORK_STAT1_EO_PL              = 0x8003880C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE12_RX_WORK_STAT1_EO_PL              = 0x8003880C0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE12_RX_WORK_STAT1_EO_PL              = 0x8003880C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE12_RX_WORK_STAT1_EO_PL              = 0x8003880C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE12_RX_WORK_STAT1_EO_PL              = 0x8003880C0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE12_RX_WORK_STAT1_EO_PL              = 0x8003880C0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE12_RX_WORK_STAT1_EO_PL               = 0x8003880C0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE12_RX_WORK_STAT1_EO_PL               = 0x8003880C0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE12_RX_WORK_STAT1_EO_PL               = 0x8003880C0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE12_RX_WORK_STAT1_EO_PL               = 0x8003880C0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE12_RX_WORK_STAT1_EO_PL              = 0x8003880C0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE12_RX_WORK_STAT1_EO_PL              = 0x8003880C0801183Full;


static const uint64_t P9A_MC_RX0_SLICE12_RX_WORK_STAT2_EO_PL                  = 0x8003900C0701103Full;
//DUPS: 8003900C0701143F, 8003900C0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE12_RX_WORK_STAT2_EO_PL                = 0x8003900C0701103Full;
//DUPS: 8003900C0701143F, 8003900C0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE12_RX_WORK_STAT2_EO_PL                = 0x8003900C0801103Full;
//DUPS: 8003900C0801143F, 8003900C0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE12_RX_WORK_STAT2_EO_PL                = 0x8003900C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE12_RX_WORK_STAT2_EO_PL              = 0x8003900C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE12_RX_WORK_STAT2_EO_PL              = 0x8003900C0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE12_RX_WORK_STAT2_EO_PL              = 0x8003900C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE12_RX_WORK_STAT2_EO_PL              = 0x8003900C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE12_RX_WORK_STAT2_EO_PL              = 0x8003900C0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE12_RX_WORK_STAT2_EO_PL              = 0x8003900C0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE12_RX_WORK_STAT2_EO_PL               = 0x8003900C0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE12_RX_WORK_STAT2_EO_PL               = 0x8003900C0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE12_RX_WORK_STAT2_EO_PL               = 0x8003900C0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE12_RX_WORK_STAT2_EO_PL               = 0x8003900C0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE12_RX_WORK_STAT2_EO_PL              = 0x8003900C0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE12_RX_WORK_STAT2_EO_PL              = 0x8003900C0801183Full;


static const uint64_t P9A_MC_RX0_SLICE12_RX_WORK_STAT3_EO_PL                  = 0x8003980C0701103Full;
//DUPS: 8003980C0701143F, 8003980C0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE12_RX_WORK_STAT3_EO_PL                = 0x8003980C0701103Full;
//DUPS: 8003980C0701143F, 8003980C0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE12_RX_WORK_STAT3_EO_PL                = 0x8003980C0801103Full;
//DUPS: 8003980C0801143F, 8003980C0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE12_RX_WORK_STAT3_EO_PL                = 0x8003980C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE12_RX_WORK_STAT3_EO_PL              = 0x8003980C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE12_RX_WORK_STAT3_EO_PL              = 0x8003980C0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE12_RX_WORK_STAT3_EO_PL              = 0x8003980C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE12_RX_WORK_STAT3_EO_PL              = 0x8003980C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE12_RX_WORK_STAT3_EO_PL              = 0x8003980C0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE12_RX_WORK_STAT3_EO_PL              = 0x8003980C0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE12_RX_WORK_STAT3_EO_PL               = 0x8003980C0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE12_RX_WORK_STAT3_EO_PL               = 0x8003980C0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE12_RX_WORK_STAT3_EO_PL               = 0x8003980C0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE12_RX_WORK_STAT3_EO_PL               = 0x8003980C0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE12_RX_WORK_STAT3_EO_PL              = 0x8003980C0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE12_RX_WORK_STAT3_EO_PL              = 0x8003980C0801183Full;


static const uint64_t P9A_MC_RX0_SLICE12_RX_WORK_STAT4_O_PL                   = 0x8003A00C0701103Full;
//DUPS: 8003A00C0701143F, 8003A00C0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE12_RX_WORK_STAT4_O_PL                 = 0x8003A00C0701103Full;
//DUPS: 8003A00C0701143F, 8003A00C0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE12_RX_WORK_STAT4_O_PL                 = 0x8003A00C0801103Full;
//DUPS: 8003A00C0801143F, 8003A00C0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE12_RX_WORK_STAT4_O_PL                 = 0x8003A00C0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE12_RX_WORK_STAT4_O_PL               = 0x8003A00C0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE12_RX_WORK_STAT4_O_PL               = 0x8003A00C0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE12_RX_WORK_STAT4_O_PL               = 0x8003A00C0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE12_RX_WORK_STAT4_O_PL               = 0x8003A00C0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE12_RX_WORK_STAT4_O_PL               = 0x8003A00C0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE12_RX_WORK_STAT4_O_PL               = 0x8003A00C0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE12_RX_WORK_STAT4_O_PL                = 0x8003A00C0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE12_RX_WORK_STAT4_O_PL                = 0x8003A00C0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE12_RX_WORK_STAT4_O_PL                = 0x8003A00C0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE12_RX_WORK_STAT4_O_PL                = 0x8003A00C0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE12_RX_WORK_STAT4_O_PL               = 0x8003A00C0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE12_RX_WORK_STAT4_O_PL               = 0x8003A00C0801183Full;


static const uint64_t P9A_MC_RX0_SLICE13_RX_DATA_WORK_SPARE_MODE_PL           = 0x8003800D0701103Full;
//DUPS: 8003800D0701143F, 8003800D0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE13_RX_DATA_WORK_SPARE_MODE_PL         = 0x8003800D0701103Full;
//DUPS: 8003800D0701143F, 8003800D0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE13_RX_DATA_WORK_SPARE_MODE_PL         = 0x8003800D0801103Full;
//DUPS: 8003800D0801143F, 8003800D0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE13_RX_DATA_WORK_SPARE_MODE_PL         = 0x8003800D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE13_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE13_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800D0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE13_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE13_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE13_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800D0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE13_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800D0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE13_RX_DATA_WORK_SPARE_MODE_PL        = 0x8003800D0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE13_RX_DATA_WORK_SPARE_MODE_PL        = 0x8003800D0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE13_RX_DATA_WORK_SPARE_MODE_PL        = 0x8003800D0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE13_RX_DATA_WORK_SPARE_MODE_PL        = 0x8003800D0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE13_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800D0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE13_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800D0801183Full;


static const uint64_t P9A_MC_RX0_SLICE13_RX_GLBSM_PL_CNTL1X_O_PL              = 0x8003300D0701103Full;
//DUPS: 8003300D0701143F, 8003300D0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE13_RX_GLBSM_PL_CNTL1X_O_PL            = 0x8003300D0701103Full;
//DUPS: 8003300D0701143F, 8003300D0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE13_RX_GLBSM_PL_CNTL1X_O_PL            = 0x8003300D0801103Full;
//DUPS: 8003300D0801143F, 8003300D0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE13_RX_GLBSM_PL_CNTL1X_O_PL            = 0x8003300D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE13_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE13_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300D0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE13_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE13_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE13_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300D0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE13_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300D0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE13_RX_GLBSM_PL_CNTL1X_O_PL           = 0x8003300D0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE13_RX_GLBSM_PL_CNTL1X_O_PL           = 0x8003300D0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE13_RX_GLBSM_PL_CNTL1X_O_PL           = 0x8003300D0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE13_RX_GLBSM_PL_CNTL1X_O_PL           = 0x8003300D0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE13_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300D0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE13_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300D0801183Full;


static const uint64_t P9A_MC_RX0_SLICE13_RX_GLBSM_PL_CNTL1_O_PL               = 0x8003200D0701103Full;
//DUPS: 8003200D0701143F, 8003200D0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE13_RX_GLBSM_PL_CNTL1_O_PL             = 0x8003200D0701103Full;
//DUPS: 8003200D0701143F, 8003200D0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE13_RX_GLBSM_PL_CNTL1_O_PL             = 0x8003200D0801103Full;
//DUPS: 8003200D0801143F, 8003200D0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE13_RX_GLBSM_PL_CNTL1_O_PL             = 0x8003200D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE13_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE13_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200D0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE13_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE13_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE13_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200D0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE13_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200D0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE13_RX_GLBSM_PL_CNTL1_O_PL            = 0x8003200D0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE13_RX_GLBSM_PL_CNTL1_O_PL            = 0x8003200D0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE13_RX_GLBSM_PL_CNTL1_O_PL            = 0x8003200D0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE13_RX_GLBSM_PL_CNTL1_O_PL            = 0x8003200D0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE13_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200D0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE13_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200D0801183Full;


static const uint64_t P9A_MC_RX0_SLICE13_RX_GLBSM_PL_STAT1_O_PL               = 0x8003280D0701103Full;
//DUPS: 8003280D0701143F, 8003280D0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE13_RX_GLBSM_PL_STAT1_O_PL             = 0x8003280D0701103Full;
//DUPS: 8003280D0701143F, 8003280D0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE13_RX_GLBSM_PL_STAT1_O_PL             = 0x8003280D0801103Full;
//DUPS: 8003280D0801143F, 8003280D0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE13_RX_GLBSM_PL_STAT1_O_PL             = 0x8003280D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE13_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE13_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280D0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE13_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE13_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE13_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280D0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE13_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280D0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE13_RX_GLBSM_PL_STAT1_O_PL            = 0x8003280D0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE13_RX_GLBSM_PL_STAT1_O_PL            = 0x8003280D0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE13_RX_GLBSM_PL_STAT1_O_PL            = 0x8003280D0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE13_RX_GLBSM_PL_STAT1_O_PL            = 0x8003280D0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE13_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280D0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE13_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280D0801183Full;


static const uint64_t P9A_MC_RX0_SLICE13_RX_WORK_CNTL1_O_PL                   = 0x8003A80D0701103Full;
//DUPS: 8003A80D0701143F, 8003A80D0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE13_RX_WORK_CNTL1_O_PL                 = 0x8003A80D0701103Full;
//DUPS: 8003A80D0701143F, 8003A80D0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE13_RX_WORK_CNTL1_O_PL                 = 0x8003A80D0801103Full;
//DUPS: 8003A80D0801143F, 8003A80D0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE13_RX_WORK_CNTL1_O_PL                 = 0x8003A80D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE13_RX_WORK_CNTL1_O_PL               = 0x8003A80D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE13_RX_WORK_CNTL1_O_PL               = 0x8003A80D0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE13_RX_WORK_CNTL1_O_PL               = 0x8003A80D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE13_RX_WORK_CNTL1_O_PL               = 0x8003A80D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE13_RX_WORK_CNTL1_O_PL               = 0x8003A80D0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE13_RX_WORK_CNTL1_O_PL               = 0x8003A80D0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE13_RX_WORK_CNTL1_O_PL                = 0x8003A80D0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE13_RX_WORK_CNTL1_O_PL                = 0x8003A80D0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE13_RX_WORK_CNTL1_O_PL                = 0x8003A80D0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE13_RX_WORK_CNTL1_O_PL                = 0x8003A80D0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE13_RX_WORK_CNTL1_O_PL               = 0x8003A80D0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE13_RX_WORK_CNTL1_O_PL               = 0x8003A80D0801183Full;


static const uint64_t P9A_MC_RX0_SLICE13_RX_WORK_STAT1_EO_PL                  = 0x8003880D0701103Full;
//DUPS: 8003880D0701143F, 8003880D0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE13_RX_WORK_STAT1_EO_PL                = 0x8003880D0701103Full;
//DUPS: 8003880D0701143F, 8003880D0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE13_RX_WORK_STAT1_EO_PL                = 0x8003880D0801103Full;
//DUPS: 8003880D0801143F, 8003880D0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE13_RX_WORK_STAT1_EO_PL                = 0x8003880D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE13_RX_WORK_STAT1_EO_PL              = 0x8003880D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE13_RX_WORK_STAT1_EO_PL              = 0x8003880D0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE13_RX_WORK_STAT1_EO_PL              = 0x8003880D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE13_RX_WORK_STAT1_EO_PL              = 0x8003880D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE13_RX_WORK_STAT1_EO_PL              = 0x8003880D0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE13_RX_WORK_STAT1_EO_PL              = 0x8003880D0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE13_RX_WORK_STAT1_EO_PL               = 0x8003880D0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE13_RX_WORK_STAT1_EO_PL               = 0x8003880D0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE13_RX_WORK_STAT1_EO_PL               = 0x8003880D0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE13_RX_WORK_STAT1_EO_PL               = 0x8003880D0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE13_RX_WORK_STAT1_EO_PL              = 0x8003880D0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE13_RX_WORK_STAT1_EO_PL              = 0x8003880D0801183Full;


static const uint64_t P9A_MC_RX0_SLICE13_RX_WORK_STAT2_EO_PL                  = 0x8003900D0701103Full;
//DUPS: 8003900D0701143F, 8003900D0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE13_RX_WORK_STAT2_EO_PL                = 0x8003900D0701103Full;
//DUPS: 8003900D0701143F, 8003900D0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE13_RX_WORK_STAT2_EO_PL                = 0x8003900D0801103Full;
//DUPS: 8003900D0801143F, 8003900D0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE13_RX_WORK_STAT2_EO_PL                = 0x8003900D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE13_RX_WORK_STAT2_EO_PL              = 0x8003900D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE13_RX_WORK_STAT2_EO_PL              = 0x8003900D0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE13_RX_WORK_STAT2_EO_PL              = 0x8003900D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE13_RX_WORK_STAT2_EO_PL              = 0x8003900D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE13_RX_WORK_STAT2_EO_PL              = 0x8003900D0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE13_RX_WORK_STAT2_EO_PL              = 0x8003900D0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE13_RX_WORK_STAT2_EO_PL               = 0x8003900D0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE13_RX_WORK_STAT2_EO_PL               = 0x8003900D0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE13_RX_WORK_STAT2_EO_PL               = 0x8003900D0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE13_RX_WORK_STAT2_EO_PL               = 0x8003900D0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE13_RX_WORK_STAT2_EO_PL              = 0x8003900D0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE13_RX_WORK_STAT2_EO_PL              = 0x8003900D0801183Full;


static const uint64_t P9A_MC_RX0_SLICE13_RX_WORK_STAT3_EO_PL                  = 0x8003980D0701103Full;
//DUPS: 8003980D0701143F, 8003980D0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE13_RX_WORK_STAT3_EO_PL                = 0x8003980D0701103Full;
//DUPS: 8003980D0701143F, 8003980D0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE13_RX_WORK_STAT3_EO_PL                = 0x8003980D0801103Full;
//DUPS: 8003980D0801143F, 8003980D0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE13_RX_WORK_STAT3_EO_PL                = 0x8003980D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE13_RX_WORK_STAT3_EO_PL              = 0x8003980D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE13_RX_WORK_STAT3_EO_PL              = 0x8003980D0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE13_RX_WORK_STAT3_EO_PL              = 0x8003980D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE13_RX_WORK_STAT3_EO_PL              = 0x8003980D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE13_RX_WORK_STAT3_EO_PL              = 0x8003980D0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE13_RX_WORK_STAT3_EO_PL              = 0x8003980D0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE13_RX_WORK_STAT3_EO_PL               = 0x8003980D0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE13_RX_WORK_STAT3_EO_PL               = 0x8003980D0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE13_RX_WORK_STAT3_EO_PL               = 0x8003980D0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE13_RX_WORK_STAT3_EO_PL               = 0x8003980D0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE13_RX_WORK_STAT3_EO_PL              = 0x8003980D0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE13_RX_WORK_STAT3_EO_PL              = 0x8003980D0801183Full;


static const uint64_t P9A_MC_RX0_SLICE13_RX_WORK_STAT4_O_PL                   = 0x8003A00D0701103Full;
//DUPS: 8003A00D0701143F, 8003A00D0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE13_RX_WORK_STAT4_O_PL                 = 0x8003A00D0701103Full;
//DUPS: 8003A00D0701143F, 8003A00D0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE13_RX_WORK_STAT4_O_PL                 = 0x8003A00D0801103Full;
//DUPS: 8003A00D0801143F, 8003A00D0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE13_RX_WORK_STAT4_O_PL                 = 0x8003A00D0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE13_RX_WORK_STAT4_O_PL               = 0x8003A00D0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE13_RX_WORK_STAT4_O_PL               = 0x8003A00D0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE13_RX_WORK_STAT4_O_PL               = 0x8003A00D0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE13_RX_WORK_STAT4_O_PL               = 0x8003A00D0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE13_RX_WORK_STAT4_O_PL               = 0x8003A00D0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE13_RX_WORK_STAT4_O_PL               = 0x8003A00D0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE13_RX_WORK_STAT4_O_PL                = 0x8003A00D0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE13_RX_WORK_STAT4_O_PL                = 0x8003A00D0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE13_RX_WORK_STAT4_O_PL                = 0x8003A00D0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE13_RX_WORK_STAT4_O_PL                = 0x8003A00D0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE13_RX_WORK_STAT4_O_PL               = 0x8003A00D0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE13_RX_WORK_STAT4_O_PL               = 0x8003A00D0801183Full;


static const uint64_t P9A_MC_RX0_SLICE14_RX_DATA_WORK_SPARE_MODE_PL           = 0x8003800E0701103Full;
//DUPS: 8003800E0701143F, 8003800E0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE14_RX_DATA_WORK_SPARE_MODE_PL         = 0x8003800E0701103Full;
//DUPS: 8003800E0701143F, 8003800E0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE14_RX_DATA_WORK_SPARE_MODE_PL         = 0x8003800E0801103Full;
//DUPS: 8003800E0801143F, 8003800E0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE14_RX_DATA_WORK_SPARE_MODE_PL         = 0x8003800E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE14_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE14_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800E0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE14_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE14_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE14_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800E0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE14_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800E0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE14_RX_DATA_WORK_SPARE_MODE_PL        = 0x8003800E0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE14_RX_DATA_WORK_SPARE_MODE_PL        = 0x8003800E0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE14_RX_DATA_WORK_SPARE_MODE_PL        = 0x8003800E0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE14_RX_DATA_WORK_SPARE_MODE_PL        = 0x8003800E0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE14_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800E0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE14_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800E0801183Full;


static const uint64_t P9A_MC_RX0_SLICE14_RX_GLBSM_PL_CNTL1X_O_PL              = 0x8003300E0701103Full;
//DUPS: 8003300E0701143F, 8003300E0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE14_RX_GLBSM_PL_CNTL1X_O_PL            = 0x8003300E0701103Full;
//DUPS: 8003300E0701143F, 8003300E0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE14_RX_GLBSM_PL_CNTL1X_O_PL            = 0x8003300E0801103Full;
//DUPS: 8003300E0801143F, 8003300E0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE14_RX_GLBSM_PL_CNTL1X_O_PL            = 0x8003300E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE14_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE14_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300E0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE14_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE14_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE14_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300E0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE14_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300E0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE14_RX_GLBSM_PL_CNTL1X_O_PL           = 0x8003300E0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE14_RX_GLBSM_PL_CNTL1X_O_PL           = 0x8003300E0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE14_RX_GLBSM_PL_CNTL1X_O_PL           = 0x8003300E0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE14_RX_GLBSM_PL_CNTL1X_O_PL           = 0x8003300E0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE14_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300E0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE14_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300E0801183Full;


static const uint64_t P9A_MC_RX0_SLICE14_RX_GLBSM_PL_CNTL1_O_PL               = 0x8003200E0701103Full;
//DUPS: 8003200E0701143F, 8003200E0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE14_RX_GLBSM_PL_CNTL1_O_PL             = 0x8003200E0701103Full;
//DUPS: 8003200E0701143F, 8003200E0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE14_RX_GLBSM_PL_CNTL1_O_PL             = 0x8003200E0801103Full;
//DUPS: 8003200E0801143F, 8003200E0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE14_RX_GLBSM_PL_CNTL1_O_PL             = 0x8003200E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE14_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE14_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200E0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE14_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE14_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE14_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200E0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE14_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200E0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE14_RX_GLBSM_PL_CNTL1_O_PL            = 0x8003200E0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE14_RX_GLBSM_PL_CNTL1_O_PL            = 0x8003200E0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE14_RX_GLBSM_PL_CNTL1_O_PL            = 0x8003200E0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE14_RX_GLBSM_PL_CNTL1_O_PL            = 0x8003200E0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE14_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200E0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE14_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200E0801183Full;


static const uint64_t P9A_MC_RX0_SLICE14_RX_GLBSM_PL_STAT1_O_PL               = 0x8003280E0701103Full;
//DUPS: 8003280E0701143F, 8003280E0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE14_RX_GLBSM_PL_STAT1_O_PL             = 0x8003280E0701103Full;
//DUPS: 8003280E0701143F, 8003280E0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE14_RX_GLBSM_PL_STAT1_O_PL             = 0x8003280E0801103Full;
//DUPS: 8003280E0801143F, 8003280E0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE14_RX_GLBSM_PL_STAT1_O_PL             = 0x8003280E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE14_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE14_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280E0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE14_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE14_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE14_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280E0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE14_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280E0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE14_RX_GLBSM_PL_STAT1_O_PL            = 0x8003280E0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE14_RX_GLBSM_PL_STAT1_O_PL            = 0x8003280E0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE14_RX_GLBSM_PL_STAT1_O_PL            = 0x8003280E0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE14_RX_GLBSM_PL_STAT1_O_PL            = 0x8003280E0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE14_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280E0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE14_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280E0801183Full;


static const uint64_t P9A_MC_RX0_SLICE14_RX_WORK_CNTL1_O_PL                   = 0x8003A80E0701103Full;
//DUPS: 8003A80E0701143F, 8003A80E0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE14_RX_WORK_CNTL1_O_PL                 = 0x8003A80E0701103Full;
//DUPS: 8003A80E0701143F, 8003A80E0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE14_RX_WORK_CNTL1_O_PL                 = 0x8003A80E0801103Full;
//DUPS: 8003A80E0801143F, 8003A80E0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE14_RX_WORK_CNTL1_O_PL                 = 0x8003A80E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE14_RX_WORK_CNTL1_O_PL               = 0x8003A80E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE14_RX_WORK_CNTL1_O_PL               = 0x8003A80E0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE14_RX_WORK_CNTL1_O_PL               = 0x8003A80E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE14_RX_WORK_CNTL1_O_PL               = 0x8003A80E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE14_RX_WORK_CNTL1_O_PL               = 0x8003A80E0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE14_RX_WORK_CNTL1_O_PL               = 0x8003A80E0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE14_RX_WORK_CNTL1_O_PL                = 0x8003A80E0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE14_RX_WORK_CNTL1_O_PL                = 0x8003A80E0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE14_RX_WORK_CNTL1_O_PL                = 0x8003A80E0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE14_RX_WORK_CNTL1_O_PL                = 0x8003A80E0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE14_RX_WORK_CNTL1_O_PL               = 0x8003A80E0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE14_RX_WORK_CNTL1_O_PL               = 0x8003A80E0801183Full;


static const uint64_t P9A_MC_RX0_SLICE14_RX_WORK_STAT1_EO_PL                  = 0x8003880E0701103Full;
//DUPS: 8003880E0701143F, 8003880E0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE14_RX_WORK_STAT1_EO_PL                = 0x8003880E0701103Full;
//DUPS: 8003880E0701143F, 8003880E0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE14_RX_WORK_STAT1_EO_PL                = 0x8003880E0801103Full;
//DUPS: 8003880E0801143F, 8003880E0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE14_RX_WORK_STAT1_EO_PL                = 0x8003880E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE14_RX_WORK_STAT1_EO_PL              = 0x8003880E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE14_RX_WORK_STAT1_EO_PL              = 0x8003880E0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE14_RX_WORK_STAT1_EO_PL              = 0x8003880E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE14_RX_WORK_STAT1_EO_PL              = 0x8003880E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE14_RX_WORK_STAT1_EO_PL              = 0x8003880E0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE14_RX_WORK_STAT1_EO_PL              = 0x8003880E0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE14_RX_WORK_STAT1_EO_PL               = 0x8003880E0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE14_RX_WORK_STAT1_EO_PL               = 0x8003880E0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE14_RX_WORK_STAT1_EO_PL               = 0x8003880E0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE14_RX_WORK_STAT1_EO_PL               = 0x8003880E0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE14_RX_WORK_STAT1_EO_PL              = 0x8003880E0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE14_RX_WORK_STAT1_EO_PL              = 0x8003880E0801183Full;


static const uint64_t P9A_MC_RX0_SLICE14_RX_WORK_STAT2_EO_PL                  = 0x8003900E0701103Full;
//DUPS: 8003900E0701143F, 8003900E0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE14_RX_WORK_STAT2_EO_PL                = 0x8003900E0701103Full;
//DUPS: 8003900E0701143F, 8003900E0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE14_RX_WORK_STAT2_EO_PL                = 0x8003900E0801103Full;
//DUPS: 8003900E0801143F, 8003900E0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE14_RX_WORK_STAT2_EO_PL                = 0x8003900E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE14_RX_WORK_STAT2_EO_PL              = 0x8003900E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE14_RX_WORK_STAT2_EO_PL              = 0x8003900E0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE14_RX_WORK_STAT2_EO_PL              = 0x8003900E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE14_RX_WORK_STAT2_EO_PL              = 0x8003900E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE14_RX_WORK_STAT2_EO_PL              = 0x8003900E0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE14_RX_WORK_STAT2_EO_PL              = 0x8003900E0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE14_RX_WORK_STAT2_EO_PL               = 0x8003900E0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE14_RX_WORK_STAT2_EO_PL               = 0x8003900E0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE14_RX_WORK_STAT2_EO_PL               = 0x8003900E0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE14_RX_WORK_STAT2_EO_PL               = 0x8003900E0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE14_RX_WORK_STAT2_EO_PL              = 0x8003900E0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE14_RX_WORK_STAT2_EO_PL              = 0x8003900E0801183Full;


static const uint64_t P9A_MC_RX0_SLICE14_RX_WORK_STAT3_EO_PL                  = 0x8003980E0701103Full;
//DUPS: 8003980E0701143F, 8003980E0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE14_RX_WORK_STAT3_EO_PL                = 0x8003980E0701103Full;
//DUPS: 8003980E0701143F, 8003980E0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE14_RX_WORK_STAT3_EO_PL                = 0x8003980E0801103Full;
//DUPS: 8003980E0801143F, 8003980E0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE14_RX_WORK_STAT3_EO_PL                = 0x8003980E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE14_RX_WORK_STAT3_EO_PL              = 0x8003980E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE14_RX_WORK_STAT3_EO_PL              = 0x8003980E0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE14_RX_WORK_STAT3_EO_PL              = 0x8003980E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE14_RX_WORK_STAT3_EO_PL              = 0x8003980E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE14_RX_WORK_STAT3_EO_PL              = 0x8003980E0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE14_RX_WORK_STAT3_EO_PL              = 0x8003980E0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE14_RX_WORK_STAT3_EO_PL               = 0x8003980E0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE14_RX_WORK_STAT3_EO_PL               = 0x8003980E0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE14_RX_WORK_STAT3_EO_PL               = 0x8003980E0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE14_RX_WORK_STAT3_EO_PL               = 0x8003980E0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE14_RX_WORK_STAT3_EO_PL              = 0x8003980E0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE14_RX_WORK_STAT3_EO_PL              = 0x8003980E0801183Full;


static const uint64_t P9A_MC_RX0_SLICE14_RX_WORK_STAT4_O_PL                   = 0x8003A00E0701103Full;
//DUPS: 8003A00E0701143F, 8003A00E0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE14_RX_WORK_STAT4_O_PL                 = 0x8003A00E0701103Full;
//DUPS: 8003A00E0701143F, 8003A00E0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE14_RX_WORK_STAT4_O_PL                 = 0x8003A00E0801103Full;
//DUPS: 8003A00E0801143F, 8003A00E0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE14_RX_WORK_STAT4_O_PL                 = 0x8003A00E0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE14_RX_WORK_STAT4_O_PL               = 0x8003A00E0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE14_RX_WORK_STAT4_O_PL               = 0x8003A00E0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE14_RX_WORK_STAT4_O_PL               = 0x8003A00E0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE14_RX_WORK_STAT4_O_PL               = 0x8003A00E0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE14_RX_WORK_STAT4_O_PL               = 0x8003A00E0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE14_RX_WORK_STAT4_O_PL               = 0x8003A00E0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE14_RX_WORK_STAT4_O_PL                = 0x8003A00E0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE14_RX_WORK_STAT4_O_PL                = 0x8003A00E0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE14_RX_WORK_STAT4_O_PL                = 0x8003A00E0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE14_RX_WORK_STAT4_O_PL                = 0x8003A00E0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE14_RX_WORK_STAT4_O_PL               = 0x8003A00E0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE14_RX_WORK_STAT4_O_PL               = 0x8003A00E0801183Full;


static const uint64_t P9A_MC_RX0_SLICE15_RX_DATA_WORK_SPARE_MODE_PL           = 0x8003800F0701103Full;
//DUPS: 8003800F0701143F, 8003800F0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE15_RX_DATA_WORK_SPARE_MODE_PL         = 0x8003800F0701103Full;
//DUPS: 8003800F0701143F, 8003800F0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE15_RX_DATA_WORK_SPARE_MODE_PL         = 0x8003800F0801103Full;
//DUPS: 8003800F0801143F, 8003800F0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE15_RX_DATA_WORK_SPARE_MODE_PL         = 0x8003800F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE15_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE15_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800F0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE15_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE15_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE15_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800F0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE15_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800F0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE15_RX_DATA_WORK_SPARE_MODE_PL        = 0x8003800F0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE15_RX_DATA_WORK_SPARE_MODE_PL        = 0x8003800F0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE15_RX_DATA_WORK_SPARE_MODE_PL        = 0x8003800F0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE15_RX_DATA_WORK_SPARE_MODE_PL        = 0x8003800F0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE15_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800F0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE15_RX_DATA_WORK_SPARE_MODE_PL       = 0x8003800F0801183Full;


static const uint64_t P9A_MC_RX0_SLICE15_RX_GLBSM_PL_CNTL1X_O_PL              = 0x8003300F0701103Full;
//DUPS: 8003300F0701143F, 8003300F0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE15_RX_GLBSM_PL_CNTL1X_O_PL            = 0x8003300F0701103Full;
//DUPS: 8003300F0701143F, 8003300F0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE15_RX_GLBSM_PL_CNTL1X_O_PL            = 0x8003300F0801103Full;
//DUPS: 8003300F0801143F, 8003300F0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE15_RX_GLBSM_PL_CNTL1X_O_PL            = 0x8003300F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE15_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE15_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300F0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE15_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE15_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE15_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300F0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE15_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300F0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE15_RX_GLBSM_PL_CNTL1X_O_PL           = 0x8003300F0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE15_RX_GLBSM_PL_CNTL1X_O_PL           = 0x8003300F0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE15_RX_GLBSM_PL_CNTL1X_O_PL           = 0x8003300F0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE15_RX_GLBSM_PL_CNTL1X_O_PL           = 0x8003300F0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE15_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300F0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE15_RX_GLBSM_PL_CNTL1X_O_PL          = 0x8003300F0801183Full;


static const uint64_t P9A_MC_RX0_SLICE15_RX_GLBSM_PL_CNTL1_O_PL               = 0x8003200F0701103Full;
//DUPS: 8003200F0701143F, 8003200F0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE15_RX_GLBSM_PL_CNTL1_O_PL             = 0x8003200F0701103Full;
//DUPS: 8003200F0701143F, 8003200F0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE15_RX_GLBSM_PL_CNTL1_O_PL             = 0x8003200F0801103Full;
//DUPS: 8003200F0801143F, 8003200F0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE15_RX_GLBSM_PL_CNTL1_O_PL             = 0x8003200F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE15_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE15_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200F0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE15_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE15_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE15_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200F0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE15_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200F0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE15_RX_GLBSM_PL_CNTL1_O_PL            = 0x8003200F0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE15_RX_GLBSM_PL_CNTL1_O_PL            = 0x8003200F0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE15_RX_GLBSM_PL_CNTL1_O_PL            = 0x8003200F0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE15_RX_GLBSM_PL_CNTL1_O_PL            = 0x8003200F0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE15_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200F0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE15_RX_GLBSM_PL_CNTL1_O_PL           = 0x8003200F0801183Full;


static const uint64_t P9A_MC_RX0_SLICE15_RX_GLBSM_PL_STAT1_O_PL               = 0x8003280F0701103Full;
//DUPS: 8003280F0701143F, 8003280F0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE15_RX_GLBSM_PL_STAT1_O_PL             = 0x8003280F0701103Full;
//DUPS: 8003280F0701143F, 8003280F0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE15_RX_GLBSM_PL_STAT1_O_PL             = 0x8003280F0801103Full;
//DUPS: 8003280F0801143F, 8003280F0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE15_RX_GLBSM_PL_STAT1_O_PL             = 0x8003280F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE15_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE15_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280F0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE15_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE15_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE15_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280F0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE15_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280F0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE15_RX_GLBSM_PL_STAT1_O_PL            = 0x8003280F0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE15_RX_GLBSM_PL_STAT1_O_PL            = 0x8003280F0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE15_RX_GLBSM_PL_STAT1_O_PL            = 0x8003280F0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE15_RX_GLBSM_PL_STAT1_O_PL            = 0x8003280F0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE15_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280F0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE15_RX_GLBSM_PL_STAT1_O_PL           = 0x8003280F0801183Full;


static const uint64_t P9A_MC_RX0_SLICE15_RX_WORK_CNTL1_O_PL                   = 0x8003A80F0701103Full;
//DUPS: 8003A80F0701143F, 8003A80F0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE15_RX_WORK_CNTL1_O_PL                 = 0x8003A80F0701103Full;
//DUPS: 8003A80F0701143F, 8003A80F0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE15_RX_WORK_CNTL1_O_PL                 = 0x8003A80F0801103Full;
//DUPS: 8003A80F0801143F, 8003A80F0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE15_RX_WORK_CNTL1_O_PL                 = 0x8003A80F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE15_RX_WORK_CNTL1_O_PL               = 0x8003A80F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE15_RX_WORK_CNTL1_O_PL               = 0x8003A80F0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE15_RX_WORK_CNTL1_O_PL               = 0x8003A80F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE15_RX_WORK_CNTL1_O_PL               = 0x8003A80F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE15_RX_WORK_CNTL1_O_PL               = 0x8003A80F0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE15_RX_WORK_CNTL1_O_PL               = 0x8003A80F0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE15_RX_WORK_CNTL1_O_PL                = 0x8003A80F0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE15_RX_WORK_CNTL1_O_PL                = 0x8003A80F0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE15_RX_WORK_CNTL1_O_PL                = 0x8003A80F0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE15_RX_WORK_CNTL1_O_PL                = 0x8003A80F0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE15_RX_WORK_CNTL1_O_PL               = 0x8003A80F0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE15_RX_WORK_CNTL1_O_PL               = 0x8003A80F0801183Full;


static const uint64_t P9A_MC_RX0_SLICE15_RX_WORK_STAT1_EO_PL                  = 0x8003880F0701103Full;
//DUPS: 8003880F0701143F, 8003880F0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE15_RX_WORK_STAT1_EO_PL                = 0x8003880F0701103Full;
//DUPS: 8003880F0701143F, 8003880F0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE15_RX_WORK_STAT1_EO_PL                = 0x8003880F0801103Full;
//DUPS: 8003880F0801143F, 8003880F0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE15_RX_WORK_STAT1_EO_PL                = 0x8003880F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE15_RX_WORK_STAT1_EO_PL              = 0x8003880F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE15_RX_WORK_STAT1_EO_PL              = 0x8003880F0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE15_RX_WORK_STAT1_EO_PL              = 0x8003880F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE15_RX_WORK_STAT1_EO_PL              = 0x8003880F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE15_RX_WORK_STAT1_EO_PL              = 0x8003880F0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE15_RX_WORK_STAT1_EO_PL              = 0x8003880F0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE15_RX_WORK_STAT1_EO_PL               = 0x8003880F0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE15_RX_WORK_STAT1_EO_PL               = 0x8003880F0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE15_RX_WORK_STAT1_EO_PL               = 0x8003880F0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE15_RX_WORK_STAT1_EO_PL               = 0x8003880F0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE15_RX_WORK_STAT1_EO_PL              = 0x8003880F0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE15_RX_WORK_STAT1_EO_PL              = 0x8003880F0801183Full;


static const uint64_t P9A_MC_RX0_SLICE15_RX_WORK_STAT2_EO_PL                  = 0x8003900F0701103Full;
//DUPS: 8003900F0701143F, 8003900F0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE15_RX_WORK_STAT2_EO_PL                = 0x8003900F0701103Full;
//DUPS: 8003900F0701143F, 8003900F0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE15_RX_WORK_STAT2_EO_PL                = 0x8003900F0801103Full;
//DUPS: 8003900F0801143F, 8003900F0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE15_RX_WORK_STAT2_EO_PL                = 0x8003900F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE15_RX_WORK_STAT2_EO_PL              = 0x8003900F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE15_RX_WORK_STAT2_EO_PL              = 0x8003900F0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE15_RX_WORK_STAT2_EO_PL              = 0x8003900F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE15_RX_WORK_STAT2_EO_PL              = 0x8003900F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE15_RX_WORK_STAT2_EO_PL              = 0x8003900F0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE15_RX_WORK_STAT2_EO_PL              = 0x8003900F0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE15_RX_WORK_STAT2_EO_PL               = 0x8003900F0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE15_RX_WORK_STAT2_EO_PL               = 0x8003900F0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE15_RX_WORK_STAT2_EO_PL               = 0x8003900F0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE15_RX_WORK_STAT2_EO_PL               = 0x8003900F0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE15_RX_WORK_STAT2_EO_PL              = 0x8003900F0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE15_RX_WORK_STAT2_EO_PL              = 0x8003900F0801183Full;


static const uint64_t P9A_MC_RX0_SLICE15_RX_WORK_STAT3_EO_PL                  = 0x8003980F0701103Full;
//DUPS: 8003980F0701143F, 8003980F0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE15_RX_WORK_STAT3_EO_PL                = 0x8003980F0701103Full;
//DUPS: 8003980F0701143F, 8003980F0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE15_RX_WORK_STAT3_EO_PL                = 0x8003980F0801103Full;
//DUPS: 8003980F0801143F, 8003980F0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE15_RX_WORK_STAT3_EO_PL                = 0x8003980F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE15_RX_WORK_STAT3_EO_PL              = 0x8003980F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE15_RX_WORK_STAT3_EO_PL              = 0x8003980F0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE15_RX_WORK_STAT3_EO_PL              = 0x8003980F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE15_RX_WORK_STAT3_EO_PL              = 0x8003980F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE15_RX_WORK_STAT3_EO_PL              = 0x8003980F0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE15_RX_WORK_STAT3_EO_PL              = 0x8003980F0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE15_RX_WORK_STAT3_EO_PL               = 0x8003980F0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE15_RX_WORK_STAT3_EO_PL               = 0x8003980F0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE15_RX_WORK_STAT3_EO_PL               = 0x8003980F0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE15_RX_WORK_STAT3_EO_PL               = 0x8003980F0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE15_RX_WORK_STAT3_EO_PL              = 0x8003980F0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE15_RX_WORK_STAT3_EO_PL              = 0x8003980F0801183Full;


static const uint64_t P9A_MC_RX0_SLICE15_RX_WORK_STAT4_O_PL                   = 0x8003A00F0701103Full;
//DUPS: 8003A00F0701143F, 8003A00F0701183F,
static const uint64_t P9A_MC_0_RX0_SLICE15_RX_WORK_STAT4_O_PL                 = 0x8003A00F0701103Full;
//DUPS: 8003A00F0701143F, 8003A00F0701183F,
static const uint64_t P9A_MC_1_RX0_SLICE15_RX_WORK_STAT4_O_PL                 = 0x8003A00F0801103Full;
//DUPS: 8003A00F0801143F, 8003A00F0801183F,

static const uint64_t P9A_OMIC_RX0_SLICE15_RX_WORK_STAT4_O_PL                 = 0x8003A00F0701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE15_RX_WORK_STAT4_O_PL               = 0x8003A00F0701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE15_RX_WORK_STAT4_O_PL               = 0x8003A00F0701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE15_RX_WORK_STAT4_O_PL               = 0x8003A00F0701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE15_RX_WORK_STAT4_O_PL               = 0x8003A00F0801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE15_RX_WORK_STAT4_O_PL               = 0x8003A00F0801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE15_RX_WORK_STAT4_O_PL               = 0x8003A00F0801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE15_RX_WORK_STAT4_O_PL                = 0x8003A00F0701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE15_RX_WORK_STAT4_O_PL                = 0x8003A00F0701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE15_RX_WORK_STAT4_O_PL                = 0x8003A00F0701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE15_RX_WORK_STAT4_O_PL                = 0x8003A00F0801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE15_RX_WORK_STAT4_O_PL               = 0x8003A00F0801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE15_RX_WORK_STAT4_O_PL               = 0x8003A00F0801183Full;


static const uint64_t P9A_MC_RX0_SLICE16_RX_DATA_WORK_SPARE_MODE_PL           = 0x800380100701103Full;
//DUPS: 800380100701143F, 800380100701183F,
static const uint64_t P9A_MC_0_RX0_SLICE16_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380100701103Full;
//DUPS: 800380100701143F, 800380100701183F,
static const uint64_t P9A_MC_1_RX0_SLICE16_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380100801103Full;
//DUPS: 800380100801143F, 800380100801183F,

static const uint64_t P9A_OMIC_RX0_SLICE16_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380100701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE16_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380100701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE16_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380100701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE16_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380100701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE16_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380100801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE16_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380100801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE16_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380100801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE16_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380100701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE16_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380100701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE16_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380100701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE16_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380100801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE16_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380100801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE16_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380100801183Full;


static const uint64_t P9A_MC_RX0_SLICE16_RX_GLBSM_PL_CNTL1X_O_PL              = 0x800330100701103Full;
//DUPS: 800330100701143F, 800330100701183F,
static const uint64_t P9A_MC_0_RX0_SLICE16_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330100701103Full;
//DUPS: 800330100701143F, 800330100701183F,
static const uint64_t P9A_MC_1_RX0_SLICE16_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330100801103Full;
//DUPS: 800330100801143F, 800330100801183F,

static const uint64_t P9A_OMIC_RX0_SLICE16_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330100701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE16_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330100701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE16_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330100701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE16_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330100701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE16_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330100801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE16_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330100801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE16_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330100801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE16_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330100701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE16_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330100701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE16_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330100701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE16_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330100801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE16_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330100801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE16_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330100801183Full;


static const uint64_t P9A_MC_RX0_SLICE16_RX_GLBSM_PL_CNTL1_O_PL               = 0x800320100701103Full;
//DUPS: 800320100701143F, 800320100701183F,
static const uint64_t P9A_MC_0_RX0_SLICE16_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320100701103Full;
//DUPS: 800320100701143F, 800320100701183F,
static const uint64_t P9A_MC_1_RX0_SLICE16_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320100801103Full;
//DUPS: 800320100801143F, 800320100801183F,

static const uint64_t P9A_OMIC_RX0_SLICE16_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320100701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE16_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320100701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE16_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320100701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE16_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320100701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE16_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320100801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE16_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320100801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE16_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320100801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE16_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320100701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE16_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320100701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE16_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320100701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE16_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320100801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE16_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320100801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE16_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320100801183Full;


static const uint64_t P9A_MC_RX0_SLICE16_RX_GLBSM_PL_STAT1_O_PL               = 0x800328100701103Full;
//DUPS: 800328100701143F, 800328100701183F,
static const uint64_t P9A_MC_0_RX0_SLICE16_RX_GLBSM_PL_STAT1_O_PL             = 0x800328100701103Full;
//DUPS: 800328100701143F, 800328100701183F,
static const uint64_t P9A_MC_1_RX0_SLICE16_RX_GLBSM_PL_STAT1_O_PL             = 0x800328100801103Full;
//DUPS: 800328100801143F, 800328100801183F,

static const uint64_t P9A_OMIC_RX0_SLICE16_RX_GLBSM_PL_STAT1_O_PL             = 0x800328100701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE16_RX_GLBSM_PL_STAT1_O_PL           = 0x800328100701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE16_RX_GLBSM_PL_STAT1_O_PL           = 0x800328100701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE16_RX_GLBSM_PL_STAT1_O_PL           = 0x800328100701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE16_RX_GLBSM_PL_STAT1_O_PL           = 0x800328100801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE16_RX_GLBSM_PL_STAT1_O_PL           = 0x800328100801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE16_RX_GLBSM_PL_STAT1_O_PL           = 0x800328100801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE16_RX_GLBSM_PL_STAT1_O_PL            = 0x800328100701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE16_RX_GLBSM_PL_STAT1_O_PL            = 0x800328100701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE16_RX_GLBSM_PL_STAT1_O_PL            = 0x800328100701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE16_RX_GLBSM_PL_STAT1_O_PL           = 0x800328100801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE16_RX_GLBSM_PL_STAT1_O_PL           = 0x800328100801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE16_RX_GLBSM_PL_STAT1_O_PL           = 0x800328100801183Full;


static const uint64_t P9A_MC_RX0_SLICE16_RX_WORK_CNTL1_O_PL                   = 0x8003A8100701103Full;
//DUPS: 8003A8100701143F, 8003A8100701183F,
static const uint64_t P9A_MC_0_RX0_SLICE16_RX_WORK_CNTL1_O_PL                 = 0x8003A8100701103Full;
//DUPS: 8003A8100701143F, 8003A8100701183F,
static const uint64_t P9A_MC_1_RX0_SLICE16_RX_WORK_CNTL1_O_PL                 = 0x8003A8100801103Full;
//DUPS: 8003A8100801143F, 8003A8100801183F,

static const uint64_t P9A_OMIC_RX0_SLICE16_RX_WORK_CNTL1_O_PL                 = 0x8003A8100701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE16_RX_WORK_CNTL1_O_PL               = 0x8003A8100701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE16_RX_WORK_CNTL1_O_PL               = 0x8003A8100701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE16_RX_WORK_CNTL1_O_PL               = 0x8003A8100701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE16_RX_WORK_CNTL1_O_PL               = 0x8003A8100801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE16_RX_WORK_CNTL1_O_PL               = 0x8003A8100801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE16_RX_WORK_CNTL1_O_PL               = 0x8003A8100801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE16_RX_WORK_CNTL1_O_PL                = 0x8003A8100701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE16_RX_WORK_CNTL1_O_PL                = 0x8003A8100701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE16_RX_WORK_CNTL1_O_PL                = 0x8003A8100701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE16_RX_WORK_CNTL1_O_PL               = 0x8003A8100801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE16_RX_WORK_CNTL1_O_PL               = 0x8003A8100801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE16_RX_WORK_CNTL1_O_PL               = 0x8003A8100801183Full;


static const uint64_t P9A_MC_RX0_SLICE16_RX_WORK_STAT1_EO_PL                  = 0x800388100701103Full;
//DUPS: 800388100701143F, 800388100701183F,
static const uint64_t P9A_MC_0_RX0_SLICE16_RX_WORK_STAT1_EO_PL                = 0x800388100701103Full;
//DUPS: 800388100701143F, 800388100701183F,
static const uint64_t P9A_MC_1_RX0_SLICE16_RX_WORK_STAT1_EO_PL                = 0x800388100801103Full;
//DUPS: 800388100801143F, 800388100801183F,

static const uint64_t P9A_OMIC_RX0_SLICE16_RX_WORK_STAT1_EO_PL                = 0x800388100701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE16_RX_WORK_STAT1_EO_PL              = 0x800388100701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE16_RX_WORK_STAT1_EO_PL              = 0x800388100701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE16_RX_WORK_STAT1_EO_PL              = 0x800388100701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE16_RX_WORK_STAT1_EO_PL              = 0x800388100801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE16_RX_WORK_STAT1_EO_PL              = 0x800388100801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE16_RX_WORK_STAT1_EO_PL              = 0x800388100801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE16_RX_WORK_STAT1_EO_PL               = 0x800388100701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE16_RX_WORK_STAT1_EO_PL               = 0x800388100701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE16_RX_WORK_STAT1_EO_PL               = 0x800388100701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE16_RX_WORK_STAT1_EO_PL              = 0x800388100801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE16_RX_WORK_STAT1_EO_PL              = 0x800388100801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE16_RX_WORK_STAT1_EO_PL              = 0x800388100801183Full;


static const uint64_t P9A_MC_RX0_SLICE16_RX_WORK_STAT2_EO_PL                  = 0x800390100701103Full;
//DUPS: 800390100701143F, 800390100701183F,
static const uint64_t P9A_MC_0_RX0_SLICE16_RX_WORK_STAT2_EO_PL                = 0x800390100701103Full;
//DUPS: 800390100701143F, 800390100701183F,
static const uint64_t P9A_MC_1_RX0_SLICE16_RX_WORK_STAT2_EO_PL                = 0x800390100801103Full;
//DUPS: 800390100801143F, 800390100801183F,

static const uint64_t P9A_OMIC_RX0_SLICE16_RX_WORK_STAT2_EO_PL                = 0x800390100701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE16_RX_WORK_STAT2_EO_PL              = 0x800390100701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE16_RX_WORK_STAT2_EO_PL              = 0x800390100701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE16_RX_WORK_STAT2_EO_PL              = 0x800390100701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE16_RX_WORK_STAT2_EO_PL              = 0x800390100801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE16_RX_WORK_STAT2_EO_PL              = 0x800390100801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE16_RX_WORK_STAT2_EO_PL              = 0x800390100801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE16_RX_WORK_STAT2_EO_PL               = 0x800390100701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE16_RX_WORK_STAT2_EO_PL               = 0x800390100701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE16_RX_WORK_STAT2_EO_PL               = 0x800390100701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE16_RX_WORK_STAT2_EO_PL              = 0x800390100801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE16_RX_WORK_STAT2_EO_PL              = 0x800390100801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE16_RX_WORK_STAT2_EO_PL              = 0x800390100801183Full;


static const uint64_t P9A_MC_RX0_SLICE16_RX_WORK_STAT3_EO_PL                  = 0x800398100701103Full;
//DUPS: 800398100701143F, 800398100701183F,
static const uint64_t P9A_MC_0_RX0_SLICE16_RX_WORK_STAT3_EO_PL                = 0x800398100701103Full;
//DUPS: 800398100701143F, 800398100701183F,
static const uint64_t P9A_MC_1_RX0_SLICE16_RX_WORK_STAT3_EO_PL                = 0x800398100801103Full;
//DUPS: 800398100801143F, 800398100801183F,

static const uint64_t P9A_OMIC_RX0_SLICE16_RX_WORK_STAT3_EO_PL                = 0x800398100701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE16_RX_WORK_STAT3_EO_PL              = 0x800398100701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE16_RX_WORK_STAT3_EO_PL              = 0x800398100701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE16_RX_WORK_STAT3_EO_PL              = 0x800398100701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE16_RX_WORK_STAT3_EO_PL              = 0x800398100801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE16_RX_WORK_STAT3_EO_PL              = 0x800398100801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE16_RX_WORK_STAT3_EO_PL              = 0x800398100801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE16_RX_WORK_STAT3_EO_PL               = 0x800398100701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE16_RX_WORK_STAT3_EO_PL               = 0x800398100701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE16_RX_WORK_STAT3_EO_PL               = 0x800398100701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE16_RX_WORK_STAT3_EO_PL              = 0x800398100801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE16_RX_WORK_STAT3_EO_PL              = 0x800398100801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE16_RX_WORK_STAT3_EO_PL              = 0x800398100801183Full;


static const uint64_t P9A_MC_RX0_SLICE16_RX_WORK_STAT4_O_PL                   = 0x8003A0100701103Full;
//DUPS: 8003A0100701143F, 8003A0100701183F,
static const uint64_t P9A_MC_0_RX0_SLICE16_RX_WORK_STAT4_O_PL                 = 0x8003A0100701103Full;
//DUPS: 8003A0100701143F, 8003A0100701183F,
static const uint64_t P9A_MC_1_RX0_SLICE16_RX_WORK_STAT4_O_PL                 = 0x8003A0100801103Full;
//DUPS: 8003A0100801143F, 8003A0100801183F,

static const uint64_t P9A_OMIC_RX0_SLICE16_RX_WORK_STAT4_O_PL                 = 0x8003A0100701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE16_RX_WORK_STAT4_O_PL               = 0x8003A0100701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE16_RX_WORK_STAT4_O_PL               = 0x8003A0100701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE16_RX_WORK_STAT4_O_PL               = 0x8003A0100701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE16_RX_WORK_STAT4_O_PL               = 0x8003A0100801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE16_RX_WORK_STAT4_O_PL               = 0x8003A0100801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE16_RX_WORK_STAT4_O_PL               = 0x8003A0100801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE16_RX_WORK_STAT4_O_PL                = 0x8003A0100701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE16_RX_WORK_STAT4_O_PL                = 0x8003A0100701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE16_RX_WORK_STAT4_O_PL                = 0x8003A0100701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE16_RX_WORK_STAT4_O_PL               = 0x8003A0100801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE16_RX_WORK_STAT4_O_PL               = 0x8003A0100801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE16_RX_WORK_STAT4_O_PL               = 0x8003A0100801183Full;


static const uint64_t P9A_MC_RX0_SLICE17_RX_DATA_WORK_SPARE_MODE_PL           = 0x800380110701103Full;
//DUPS: 800380110701143F, 800380110701183F,
static const uint64_t P9A_MC_0_RX0_SLICE17_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380110701103Full;
//DUPS: 800380110701143F, 800380110701183F,
static const uint64_t P9A_MC_1_RX0_SLICE17_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380110801103Full;
//DUPS: 800380110801143F, 800380110801183F,

static const uint64_t P9A_OMIC_RX0_SLICE17_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380110701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE17_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380110701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE17_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380110701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE17_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380110701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE17_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380110801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE17_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380110801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE17_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380110801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE17_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380110701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE17_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380110701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE17_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380110701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE17_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380110801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE17_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380110801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE17_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380110801183Full;


static const uint64_t P9A_MC_RX0_SLICE17_RX_GLBSM_PL_CNTL1X_O_PL              = 0x800330110701103Full;
//DUPS: 800330110701143F, 800330110701183F,
static const uint64_t P9A_MC_0_RX0_SLICE17_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330110701103Full;
//DUPS: 800330110701143F, 800330110701183F,
static const uint64_t P9A_MC_1_RX0_SLICE17_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330110801103Full;
//DUPS: 800330110801143F, 800330110801183F,

static const uint64_t P9A_OMIC_RX0_SLICE17_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330110701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE17_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330110701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE17_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330110701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE17_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330110701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE17_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330110801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE17_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330110801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE17_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330110801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE17_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330110701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE17_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330110701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE17_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330110701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE17_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330110801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE17_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330110801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE17_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330110801183Full;


static const uint64_t P9A_MC_RX0_SLICE17_RX_GLBSM_PL_CNTL1_O_PL               = 0x800320110701103Full;
//DUPS: 800320110701143F, 800320110701183F,
static const uint64_t P9A_MC_0_RX0_SLICE17_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320110701103Full;
//DUPS: 800320110701143F, 800320110701183F,
static const uint64_t P9A_MC_1_RX0_SLICE17_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320110801103Full;
//DUPS: 800320110801143F, 800320110801183F,

static const uint64_t P9A_OMIC_RX0_SLICE17_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320110701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE17_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320110701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE17_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320110701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE17_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320110701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE17_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320110801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE17_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320110801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE17_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320110801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE17_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320110701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE17_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320110701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE17_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320110701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE17_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320110801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE17_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320110801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE17_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320110801183Full;


static const uint64_t P9A_MC_RX0_SLICE17_RX_GLBSM_PL_STAT1_O_PL               = 0x800328110701103Full;
//DUPS: 800328110701143F, 800328110701183F,
static const uint64_t P9A_MC_0_RX0_SLICE17_RX_GLBSM_PL_STAT1_O_PL             = 0x800328110701103Full;
//DUPS: 800328110701143F, 800328110701183F,
static const uint64_t P9A_MC_1_RX0_SLICE17_RX_GLBSM_PL_STAT1_O_PL             = 0x800328110801103Full;
//DUPS: 800328110801143F, 800328110801183F,

static const uint64_t P9A_OMIC_RX0_SLICE17_RX_GLBSM_PL_STAT1_O_PL             = 0x800328110701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE17_RX_GLBSM_PL_STAT1_O_PL           = 0x800328110701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE17_RX_GLBSM_PL_STAT1_O_PL           = 0x800328110701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE17_RX_GLBSM_PL_STAT1_O_PL           = 0x800328110701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE17_RX_GLBSM_PL_STAT1_O_PL           = 0x800328110801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE17_RX_GLBSM_PL_STAT1_O_PL           = 0x800328110801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE17_RX_GLBSM_PL_STAT1_O_PL           = 0x800328110801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE17_RX_GLBSM_PL_STAT1_O_PL            = 0x800328110701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE17_RX_GLBSM_PL_STAT1_O_PL            = 0x800328110701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE17_RX_GLBSM_PL_STAT1_O_PL            = 0x800328110701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE17_RX_GLBSM_PL_STAT1_O_PL           = 0x800328110801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE17_RX_GLBSM_PL_STAT1_O_PL           = 0x800328110801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE17_RX_GLBSM_PL_STAT1_O_PL           = 0x800328110801183Full;


static const uint64_t P9A_MC_RX0_SLICE17_RX_WORK_CNTL1_O_PL                   = 0x8003A8110701103Full;
//DUPS: 8003A8110701143F, 8003A8110701183F,
static const uint64_t P9A_MC_0_RX0_SLICE17_RX_WORK_CNTL1_O_PL                 = 0x8003A8110701103Full;
//DUPS: 8003A8110701143F, 8003A8110701183F,
static const uint64_t P9A_MC_1_RX0_SLICE17_RX_WORK_CNTL1_O_PL                 = 0x8003A8110801103Full;
//DUPS: 8003A8110801143F, 8003A8110801183F,

static const uint64_t P9A_OMIC_RX0_SLICE17_RX_WORK_CNTL1_O_PL                 = 0x8003A8110701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE17_RX_WORK_CNTL1_O_PL               = 0x8003A8110701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE17_RX_WORK_CNTL1_O_PL               = 0x8003A8110701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE17_RX_WORK_CNTL1_O_PL               = 0x8003A8110701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE17_RX_WORK_CNTL1_O_PL               = 0x8003A8110801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE17_RX_WORK_CNTL1_O_PL               = 0x8003A8110801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE17_RX_WORK_CNTL1_O_PL               = 0x8003A8110801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE17_RX_WORK_CNTL1_O_PL                = 0x8003A8110701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE17_RX_WORK_CNTL1_O_PL                = 0x8003A8110701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE17_RX_WORK_CNTL1_O_PL                = 0x8003A8110701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE17_RX_WORK_CNTL1_O_PL               = 0x8003A8110801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE17_RX_WORK_CNTL1_O_PL               = 0x8003A8110801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE17_RX_WORK_CNTL1_O_PL               = 0x8003A8110801183Full;


static const uint64_t P9A_MC_RX0_SLICE17_RX_WORK_STAT1_EO_PL                  = 0x800388110701103Full;
//DUPS: 800388110701143F, 800388110701183F,
static const uint64_t P9A_MC_0_RX0_SLICE17_RX_WORK_STAT1_EO_PL                = 0x800388110701103Full;
//DUPS: 800388110701143F, 800388110701183F,
static const uint64_t P9A_MC_1_RX0_SLICE17_RX_WORK_STAT1_EO_PL                = 0x800388110801103Full;
//DUPS: 800388110801143F, 800388110801183F,

static const uint64_t P9A_OMIC_RX0_SLICE17_RX_WORK_STAT1_EO_PL                = 0x800388110701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE17_RX_WORK_STAT1_EO_PL              = 0x800388110701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE17_RX_WORK_STAT1_EO_PL              = 0x800388110701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE17_RX_WORK_STAT1_EO_PL              = 0x800388110701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE17_RX_WORK_STAT1_EO_PL              = 0x800388110801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE17_RX_WORK_STAT1_EO_PL              = 0x800388110801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE17_RX_WORK_STAT1_EO_PL              = 0x800388110801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE17_RX_WORK_STAT1_EO_PL               = 0x800388110701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE17_RX_WORK_STAT1_EO_PL               = 0x800388110701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE17_RX_WORK_STAT1_EO_PL               = 0x800388110701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE17_RX_WORK_STAT1_EO_PL              = 0x800388110801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE17_RX_WORK_STAT1_EO_PL              = 0x800388110801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE17_RX_WORK_STAT1_EO_PL              = 0x800388110801183Full;


static const uint64_t P9A_MC_RX0_SLICE17_RX_WORK_STAT2_EO_PL                  = 0x800390110701103Full;
//DUPS: 800390110701143F, 800390110701183F,
static const uint64_t P9A_MC_0_RX0_SLICE17_RX_WORK_STAT2_EO_PL                = 0x800390110701103Full;
//DUPS: 800390110701143F, 800390110701183F,
static const uint64_t P9A_MC_1_RX0_SLICE17_RX_WORK_STAT2_EO_PL                = 0x800390110801103Full;
//DUPS: 800390110801143F, 800390110801183F,

static const uint64_t P9A_OMIC_RX0_SLICE17_RX_WORK_STAT2_EO_PL                = 0x800390110701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE17_RX_WORK_STAT2_EO_PL              = 0x800390110701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE17_RX_WORK_STAT2_EO_PL              = 0x800390110701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE17_RX_WORK_STAT2_EO_PL              = 0x800390110701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE17_RX_WORK_STAT2_EO_PL              = 0x800390110801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE17_RX_WORK_STAT2_EO_PL              = 0x800390110801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE17_RX_WORK_STAT2_EO_PL              = 0x800390110801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE17_RX_WORK_STAT2_EO_PL               = 0x800390110701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE17_RX_WORK_STAT2_EO_PL               = 0x800390110701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE17_RX_WORK_STAT2_EO_PL               = 0x800390110701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE17_RX_WORK_STAT2_EO_PL              = 0x800390110801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE17_RX_WORK_STAT2_EO_PL              = 0x800390110801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE17_RX_WORK_STAT2_EO_PL              = 0x800390110801183Full;


static const uint64_t P9A_MC_RX0_SLICE17_RX_WORK_STAT3_EO_PL                  = 0x800398110701103Full;
//DUPS: 800398110701143F, 800398110701183F,
static const uint64_t P9A_MC_0_RX0_SLICE17_RX_WORK_STAT3_EO_PL                = 0x800398110701103Full;
//DUPS: 800398110701143F, 800398110701183F,
static const uint64_t P9A_MC_1_RX0_SLICE17_RX_WORK_STAT3_EO_PL                = 0x800398110801103Full;
//DUPS: 800398110801143F, 800398110801183F,

static const uint64_t P9A_OMIC_RX0_SLICE17_RX_WORK_STAT3_EO_PL                = 0x800398110701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE17_RX_WORK_STAT3_EO_PL              = 0x800398110701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE17_RX_WORK_STAT3_EO_PL              = 0x800398110701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE17_RX_WORK_STAT3_EO_PL              = 0x800398110701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE17_RX_WORK_STAT3_EO_PL              = 0x800398110801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE17_RX_WORK_STAT3_EO_PL              = 0x800398110801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE17_RX_WORK_STAT3_EO_PL              = 0x800398110801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE17_RX_WORK_STAT3_EO_PL               = 0x800398110701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE17_RX_WORK_STAT3_EO_PL               = 0x800398110701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE17_RX_WORK_STAT3_EO_PL               = 0x800398110701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE17_RX_WORK_STAT3_EO_PL              = 0x800398110801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE17_RX_WORK_STAT3_EO_PL              = 0x800398110801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE17_RX_WORK_STAT3_EO_PL              = 0x800398110801183Full;


static const uint64_t P9A_MC_RX0_SLICE17_RX_WORK_STAT4_O_PL                   = 0x8003A0110701103Full;
//DUPS: 8003A0110701143F, 8003A0110701183F,
static const uint64_t P9A_MC_0_RX0_SLICE17_RX_WORK_STAT4_O_PL                 = 0x8003A0110701103Full;
//DUPS: 8003A0110701143F, 8003A0110701183F,
static const uint64_t P9A_MC_1_RX0_SLICE17_RX_WORK_STAT4_O_PL                 = 0x8003A0110801103Full;
//DUPS: 8003A0110801143F, 8003A0110801183F,

static const uint64_t P9A_OMIC_RX0_SLICE17_RX_WORK_STAT4_O_PL                 = 0x8003A0110701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE17_RX_WORK_STAT4_O_PL               = 0x8003A0110701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE17_RX_WORK_STAT4_O_PL               = 0x8003A0110701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE17_RX_WORK_STAT4_O_PL               = 0x8003A0110701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE17_RX_WORK_STAT4_O_PL               = 0x8003A0110801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE17_RX_WORK_STAT4_O_PL               = 0x8003A0110801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE17_RX_WORK_STAT4_O_PL               = 0x8003A0110801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE17_RX_WORK_STAT4_O_PL                = 0x8003A0110701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE17_RX_WORK_STAT4_O_PL                = 0x8003A0110701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE17_RX_WORK_STAT4_O_PL                = 0x8003A0110701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE17_RX_WORK_STAT4_O_PL               = 0x8003A0110801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE17_RX_WORK_STAT4_O_PL               = 0x8003A0110801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE17_RX_WORK_STAT4_O_PL               = 0x8003A0110801183Full;


static const uint64_t P9A_MC_RX0_SLICE18_RX_DATA_WORK_SPARE_MODE_PL           = 0x800380120701103Full;
//DUPS: 800380120701143F, 800380120701183F,
static const uint64_t P9A_MC_0_RX0_SLICE18_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380120701103Full;
//DUPS: 800380120701143F, 800380120701183F,
static const uint64_t P9A_MC_1_RX0_SLICE18_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380120801103Full;
//DUPS: 800380120801143F, 800380120801183F,

static const uint64_t P9A_OMIC_RX0_SLICE18_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380120701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE18_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380120701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE18_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380120701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE18_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380120701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE18_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380120801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE18_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380120801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE18_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380120801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE18_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380120701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE18_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380120701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE18_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380120701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE18_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380120801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE18_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380120801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE18_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380120801183Full;


static const uint64_t P9A_MC_RX0_SLICE18_RX_GLBSM_PL_CNTL1X_O_PL              = 0x800330120701103Full;
//DUPS: 800330120701143F, 800330120701183F,
static const uint64_t P9A_MC_0_RX0_SLICE18_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330120701103Full;
//DUPS: 800330120701143F, 800330120701183F,
static const uint64_t P9A_MC_1_RX0_SLICE18_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330120801103Full;
//DUPS: 800330120801143F, 800330120801183F,

static const uint64_t P9A_OMIC_RX0_SLICE18_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330120701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE18_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330120701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE18_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330120701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE18_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330120701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE18_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330120801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE18_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330120801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE18_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330120801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE18_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330120701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE18_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330120701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE18_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330120701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE18_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330120801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE18_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330120801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE18_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330120801183Full;


static const uint64_t P9A_MC_RX0_SLICE18_RX_GLBSM_PL_CNTL1_O_PL               = 0x800320120701103Full;
//DUPS: 800320120701143F, 800320120701183F,
static const uint64_t P9A_MC_0_RX0_SLICE18_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320120701103Full;
//DUPS: 800320120701143F, 800320120701183F,
static const uint64_t P9A_MC_1_RX0_SLICE18_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320120801103Full;
//DUPS: 800320120801143F, 800320120801183F,

static const uint64_t P9A_OMIC_RX0_SLICE18_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320120701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE18_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320120701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE18_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320120701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE18_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320120701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE18_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320120801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE18_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320120801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE18_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320120801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE18_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320120701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE18_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320120701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE18_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320120701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE18_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320120801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE18_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320120801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE18_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320120801183Full;


static const uint64_t P9A_MC_RX0_SLICE18_RX_GLBSM_PL_STAT1_O_PL               = 0x800328120701103Full;
//DUPS: 800328120701143F, 800328120701183F,
static const uint64_t P9A_MC_0_RX0_SLICE18_RX_GLBSM_PL_STAT1_O_PL             = 0x800328120701103Full;
//DUPS: 800328120701143F, 800328120701183F,
static const uint64_t P9A_MC_1_RX0_SLICE18_RX_GLBSM_PL_STAT1_O_PL             = 0x800328120801103Full;
//DUPS: 800328120801143F, 800328120801183F,

static const uint64_t P9A_OMIC_RX0_SLICE18_RX_GLBSM_PL_STAT1_O_PL             = 0x800328120701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE18_RX_GLBSM_PL_STAT1_O_PL           = 0x800328120701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE18_RX_GLBSM_PL_STAT1_O_PL           = 0x800328120701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE18_RX_GLBSM_PL_STAT1_O_PL           = 0x800328120701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE18_RX_GLBSM_PL_STAT1_O_PL           = 0x800328120801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE18_RX_GLBSM_PL_STAT1_O_PL           = 0x800328120801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE18_RX_GLBSM_PL_STAT1_O_PL           = 0x800328120801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE18_RX_GLBSM_PL_STAT1_O_PL            = 0x800328120701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE18_RX_GLBSM_PL_STAT1_O_PL            = 0x800328120701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE18_RX_GLBSM_PL_STAT1_O_PL            = 0x800328120701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE18_RX_GLBSM_PL_STAT1_O_PL           = 0x800328120801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE18_RX_GLBSM_PL_STAT1_O_PL           = 0x800328120801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE18_RX_GLBSM_PL_STAT1_O_PL           = 0x800328120801183Full;


static const uint64_t P9A_MC_RX0_SLICE18_RX_WORK_CNTL1_O_PL                   = 0x8003A8120701103Full;
//DUPS: 8003A8120701143F, 8003A8120701183F,
static const uint64_t P9A_MC_0_RX0_SLICE18_RX_WORK_CNTL1_O_PL                 = 0x8003A8120701103Full;
//DUPS: 8003A8120701143F, 8003A8120701183F,
static const uint64_t P9A_MC_1_RX0_SLICE18_RX_WORK_CNTL1_O_PL                 = 0x8003A8120801103Full;
//DUPS: 8003A8120801143F, 8003A8120801183F,

static const uint64_t P9A_OMIC_RX0_SLICE18_RX_WORK_CNTL1_O_PL                 = 0x8003A8120701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE18_RX_WORK_CNTL1_O_PL               = 0x8003A8120701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE18_RX_WORK_CNTL1_O_PL               = 0x8003A8120701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE18_RX_WORK_CNTL1_O_PL               = 0x8003A8120701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE18_RX_WORK_CNTL1_O_PL               = 0x8003A8120801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE18_RX_WORK_CNTL1_O_PL               = 0x8003A8120801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE18_RX_WORK_CNTL1_O_PL               = 0x8003A8120801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE18_RX_WORK_CNTL1_O_PL                = 0x8003A8120701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE18_RX_WORK_CNTL1_O_PL                = 0x8003A8120701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE18_RX_WORK_CNTL1_O_PL                = 0x8003A8120701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE18_RX_WORK_CNTL1_O_PL               = 0x8003A8120801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE18_RX_WORK_CNTL1_O_PL               = 0x8003A8120801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE18_RX_WORK_CNTL1_O_PL               = 0x8003A8120801183Full;


static const uint64_t P9A_MC_RX0_SLICE18_RX_WORK_STAT1_EO_PL                  = 0x800388120701103Full;
//DUPS: 800388120701143F, 800388120701183F,
static const uint64_t P9A_MC_0_RX0_SLICE18_RX_WORK_STAT1_EO_PL                = 0x800388120701103Full;
//DUPS: 800388120701143F, 800388120701183F,
static const uint64_t P9A_MC_1_RX0_SLICE18_RX_WORK_STAT1_EO_PL                = 0x800388120801103Full;
//DUPS: 800388120801143F, 800388120801183F,

static const uint64_t P9A_OMIC_RX0_SLICE18_RX_WORK_STAT1_EO_PL                = 0x800388120701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE18_RX_WORK_STAT1_EO_PL              = 0x800388120701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE18_RX_WORK_STAT1_EO_PL              = 0x800388120701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE18_RX_WORK_STAT1_EO_PL              = 0x800388120701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE18_RX_WORK_STAT1_EO_PL              = 0x800388120801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE18_RX_WORK_STAT1_EO_PL              = 0x800388120801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE18_RX_WORK_STAT1_EO_PL              = 0x800388120801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE18_RX_WORK_STAT1_EO_PL               = 0x800388120701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE18_RX_WORK_STAT1_EO_PL               = 0x800388120701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE18_RX_WORK_STAT1_EO_PL               = 0x800388120701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE18_RX_WORK_STAT1_EO_PL              = 0x800388120801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE18_RX_WORK_STAT1_EO_PL              = 0x800388120801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE18_RX_WORK_STAT1_EO_PL              = 0x800388120801183Full;


static const uint64_t P9A_MC_RX0_SLICE18_RX_WORK_STAT2_EO_PL                  = 0x800390120701103Full;
//DUPS: 800390120701143F, 800390120701183F,
static const uint64_t P9A_MC_0_RX0_SLICE18_RX_WORK_STAT2_EO_PL                = 0x800390120701103Full;
//DUPS: 800390120701143F, 800390120701183F,
static const uint64_t P9A_MC_1_RX0_SLICE18_RX_WORK_STAT2_EO_PL                = 0x800390120801103Full;
//DUPS: 800390120801143F, 800390120801183F,

static const uint64_t P9A_OMIC_RX0_SLICE18_RX_WORK_STAT2_EO_PL                = 0x800390120701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE18_RX_WORK_STAT2_EO_PL              = 0x800390120701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE18_RX_WORK_STAT2_EO_PL              = 0x800390120701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE18_RX_WORK_STAT2_EO_PL              = 0x800390120701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE18_RX_WORK_STAT2_EO_PL              = 0x800390120801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE18_RX_WORK_STAT2_EO_PL              = 0x800390120801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE18_RX_WORK_STAT2_EO_PL              = 0x800390120801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE18_RX_WORK_STAT2_EO_PL               = 0x800390120701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE18_RX_WORK_STAT2_EO_PL               = 0x800390120701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE18_RX_WORK_STAT2_EO_PL               = 0x800390120701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE18_RX_WORK_STAT2_EO_PL              = 0x800390120801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE18_RX_WORK_STAT2_EO_PL              = 0x800390120801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE18_RX_WORK_STAT2_EO_PL              = 0x800390120801183Full;


static const uint64_t P9A_MC_RX0_SLICE18_RX_WORK_STAT3_EO_PL                  = 0x800398120701103Full;
//DUPS: 800398120701143F, 800398120701183F,
static const uint64_t P9A_MC_0_RX0_SLICE18_RX_WORK_STAT3_EO_PL                = 0x800398120701103Full;
//DUPS: 800398120701143F, 800398120701183F,
static const uint64_t P9A_MC_1_RX0_SLICE18_RX_WORK_STAT3_EO_PL                = 0x800398120801103Full;
//DUPS: 800398120801143F, 800398120801183F,

static const uint64_t P9A_OMIC_RX0_SLICE18_RX_WORK_STAT3_EO_PL                = 0x800398120701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE18_RX_WORK_STAT3_EO_PL              = 0x800398120701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE18_RX_WORK_STAT3_EO_PL              = 0x800398120701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE18_RX_WORK_STAT3_EO_PL              = 0x800398120701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE18_RX_WORK_STAT3_EO_PL              = 0x800398120801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE18_RX_WORK_STAT3_EO_PL              = 0x800398120801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE18_RX_WORK_STAT3_EO_PL              = 0x800398120801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE18_RX_WORK_STAT3_EO_PL               = 0x800398120701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE18_RX_WORK_STAT3_EO_PL               = 0x800398120701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE18_RX_WORK_STAT3_EO_PL               = 0x800398120701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE18_RX_WORK_STAT3_EO_PL              = 0x800398120801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE18_RX_WORK_STAT3_EO_PL              = 0x800398120801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE18_RX_WORK_STAT3_EO_PL              = 0x800398120801183Full;


static const uint64_t P9A_MC_RX0_SLICE18_RX_WORK_STAT4_O_PL                   = 0x8003A0120701103Full;
//DUPS: 8003A0120701143F, 8003A0120701183F,
static const uint64_t P9A_MC_0_RX0_SLICE18_RX_WORK_STAT4_O_PL                 = 0x8003A0120701103Full;
//DUPS: 8003A0120701143F, 8003A0120701183F,
static const uint64_t P9A_MC_1_RX0_SLICE18_RX_WORK_STAT4_O_PL                 = 0x8003A0120801103Full;
//DUPS: 8003A0120801143F, 8003A0120801183F,

static const uint64_t P9A_OMIC_RX0_SLICE18_RX_WORK_STAT4_O_PL                 = 0x8003A0120701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE18_RX_WORK_STAT4_O_PL               = 0x8003A0120701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE18_RX_WORK_STAT4_O_PL               = 0x8003A0120701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE18_RX_WORK_STAT4_O_PL               = 0x8003A0120701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE18_RX_WORK_STAT4_O_PL               = 0x8003A0120801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE18_RX_WORK_STAT4_O_PL               = 0x8003A0120801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE18_RX_WORK_STAT4_O_PL               = 0x8003A0120801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE18_RX_WORK_STAT4_O_PL                = 0x8003A0120701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE18_RX_WORK_STAT4_O_PL                = 0x8003A0120701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE18_RX_WORK_STAT4_O_PL                = 0x8003A0120701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE18_RX_WORK_STAT4_O_PL               = 0x8003A0120801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE18_RX_WORK_STAT4_O_PL               = 0x8003A0120801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE18_RX_WORK_STAT4_O_PL               = 0x8003A0120801183Full;


static const uint64_t P9A_MC_RX0_SLICE19_RX_DATA_WORK_SPARE_MODE_PL           = 0x800380130701103Full;
//DUPS: 800380130701143F, 800380130701183F,
static const uint64_t P9A_MC_0_RX0_SLICE19_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380130701103Full;
//DUPS: 800380130701143F, 800380130701183F,
static const uint64_t P9A_MC_1_RX0_SLICE19_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380130801103Full;
//DUPS: 800380130801143F, 800380130801183F,

static const uint64_t P9A_OMIC_RX0_SLICE19_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380130701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE19_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380130701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE19_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380130701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE19_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380130701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE19_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380130801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE19_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380130801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE19_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380130801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE19_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380130701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE19_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380130701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE19_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380130701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE19_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380130801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE19_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380130801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE19_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380130801183Full;


static const uint64_t P9A_MC_RX0_SLICE19_RX_GLBSM_PL_CNTL1X_O_PL              = 0x800330130701103Full;
//DUPS: 800330130701143F, 800330130701183F,
static const uint64_t P9A_MC_0_RX0_SLICE19_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330130701103Full;
//DUPS: 800330130701143F, 800330130701183F,
static const uint64_t P9A_MC_1_RX0_SLICE19_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330130801103Full;
//DUPS: 800330130801143F, 800330130801183F,

static const uint64_t P9A_OMIC_RX0_SLICE19_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330130701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE19_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330130701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE19_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330130701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE19_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330130701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE19_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330130801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE19_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330130801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE19_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330130801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE19_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330130701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE19_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330130701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE19_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330130701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE19_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330130801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE19_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330130801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE19_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330130801183Full;


static const uint64_t P9A_MC_RX0_SLICE19_RX_GLBSM_PL_CNTL1_O_PL               = 0x800320130701103Full;
//DUPS: 800320130701143F, 800320130701183F,
static const uint64_t P9A_MC_0_RX0_SLICE19_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320130701103Full;
//DUPS: 800320130701143F, 800320130701183F,
static const uint64_t P9A_MC_1_RX0_SLICE19_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320130801103Full;
//DUPS: 800320130801143F, 800320130801183F,

static const uint64_t P9A_OMIC_RX0_SLICE19_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320130701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE19_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320130701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE19_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320130701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE19_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320130701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE19_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320130801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE19_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320130801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE19_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320130801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE19_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320130701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE19_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320130701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE19_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320130701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE19_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320130801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE19_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320130801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE19_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320130801183Full;


static const uint64_t P9A_MC_RX0_SLICE19_RX_GLBSM_PL_STAT1_O_PL               = 0x800328130701103Full;
//DUPS: 800328130701143F, 800328130701183F,
static const uint64_t P9A_MC_0_RX0_SLICE19_RX_GLBSM_PL_STAT1_O_PL             = 0x800328130701103Full;
//DUPS: 800328130701143F, 800328130701183F,
static const uint64_t P9A_MC_1_RX0_SLICE19_RX_GLBSM_PL_STAT1_O_PL             = 0x800328130801103Full;
//DUPS: 800328130801143F, 800328130801183F,

static const uint64_t P9A_OMIC_RX0_SLICE19_RX_GLBSM_PL_STAT1_O_PL             = 0x800328130701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE19_RX_GLBSM_PL_STAT1_O_PL           = 0x800328130701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE19_RX_GLBSM_PL_STAT1_O_PL           = 0x800328130701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE19_RX_GLBSM_PL_STAT1_O_PL           = 0x800328130701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE19_RX_GLBSM_PL_STAT1_O_PL           = 0x800328130801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE19_RX_GLBSM_PL_STAT1_O_PL           = 0x800328130801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE19_RX_GLBSM_PL_STAT1_O_PL           = 0x800328130801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE19_RX_GLBSM_PL_STAT1_O_PL            = 0x800328130701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE19_RX_GLBSM_PL_STAT1_O_PL            = 0x800328130701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE19_RX_GLBSM_PL_STAT1_O_PL            = 0x800328130701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE19_RX_GLBSM_PL_STAT1_O_PL           = 0x800328130801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE19_RX_GLBSM_PL_STAT1_O_PL           = 0x800328130801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE19_RX_GLBSM_PL_STAT1_O_PL           = 0x800328130801183Full;


static const uint64_t P9A_MC_RX0_SLICE19_RX_WORK_CNTL1_O_PL                   = 0x8003A8130701103Full;
//DUPS: 8003A8130701143F, 8003A8130701183F,
static const uint64_t P9A_MC_0_RX0_SLICE19_RX_WORK_CNTL1_O_PL                 = 0x8003A8130701103Full;
//DUPS: 8003A8130701143F, 8003A8130701183F,
static const uint64_t P9A_MC_1_RX0_SLICE19_RX_WORK_CNTL1_O_PL                 = 0x8003A8130801103Full;
//DUPS: 8003A8130801143F, 8003A8130801183F,

static const uint64_t P9A_OMIC_RX0_SLICE19_RX_WORK_CNTL1_O_PL                 = 0x8003A8130701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE19_RX_WORK_CNTL1_O_PL               = 0x8003A8130701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE19_RX_WORK_CNTL1_O_PL               = 0x8003A8130701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE19_RX_WORK_CNTL1_O_PL               = 0x8003A8130701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE19_RX_WORK_CNTL1_O_PL               = 0x8003A8130801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE19_RX_WORK_CNTL1_O_PL               = 0x8003A8130801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE19_RX_WORK_CNTL1_O_PL               = 0x8003A8130801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE19_RX_WORK_CNTL1_O_PL                = 0x8003A8130701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE19_RX_WORK_CNTL1_O_PL                = 0x8003A8130701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE19_RX_WORK_CNTL1_O_PL                = 0x8003A8130701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE19_RX_WORK_CNTL1_O_PL               = 0x8003A8130801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE19_RX_WORK_CNTL1_O_PL               = 0x8003A8130801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE19_RX_WORK_CNTL1_O_PL               = 0x8003A8130801183Full;


static const uint64_t P9A_MC_RX0_SLICE19_RX_WORK_STAT1_EO_PL                  = 0x800388130701103Full;
//DUPS: 800388130701143F, 800388130701183F,
static const uint64_t P9A_MC_0_RX0_SLICE19_RX_WORK_STAT1_EO_PL                = 0x800388130701103Full;
//DUPS: 800388130701143F, 800388130701183F,
static const uint64_t P9A_MC_1_RX0_SLICE19_RX_WORK_STAT1_EO_PL                = 0x800388130801103Full;
//DUPS: 800388130801143F, 800388130801183F,

static const uint64_t P9A_OMIC_RX0_SLICE19_RX_WORK_STAT1_EO_PL                = 0x800388130701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE19_RX_WORK_STAT1_EO_PL              = 0x800388130701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE19_RX_WORK_STAT1_EO_PL              = 0x800388130701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE19_RX_WORK_STAT1_EO_PL              = 0x800388130701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE19_RX_WORK_STAT1_EO_PL              = 0x800388130801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE19_RX_WORK_STAT1_EO_PL              = 0x800388130801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE19_RX_WORK_STAT1_EO_PL              = 0x800388130801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE19_RX_WORK_STAT1_EO_PL               = 0x800388130701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE19_RX_WORK_STAT1_EO_PL               = 0x800388130701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE19_RX_WORK_STAT1_EO_PL               = 0x800388130701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE19_RX_WORK_STAT1_EO_PL              = 0x800388130801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE19_RX_WORK_STAT1_EO_PL              = 0x800388130801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE19_RX_WORK_STAT1_EO_PL              = 0x800388130801183Full;


static const uint64_t P9A_MC_RX0_SLICE19_RX_WORK_STAT2_EO_PL                  = 0x800390130701103Full;
//DUPS: 800390130701143F, 800390130701183F,
static const uint64_t P9A_MC_0_RX0_SLICE19_RX_WORK_STAT2_EO_PL                = 0x800390130701103Full;
//DUPS: 800390130701143F, 800390130701183F,
static const uint64_t P9A_MC_1_RX0_SLICE19_RX_WORK_STAT2_EO_PL                = 0x800390130801103Full;
//DUPS: 800390130801143F, 800390130801183F,

static const uint64_t P9A_OMIC_RX0_SLICE19_RX_WORK_STAT2_EO_PL                = 0x800390130701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE19_RX_WORK_STAT2_EO_PL              = 0x800390130701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE19_RX_WORK_STAT2_EO_PL              = 0x800390130701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE19_RX_WORK_STAT2_EO_PL              = 0x800390130701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE19_RX_WORK_STAT2_EO_PL              = 0x800390130801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE19_RX_WORK_STAT2_EO_PL              = 0x800390130801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE19_RX_WORK_STAT2_EO_PL              = 0x800390130801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE19_RX_WORK_STAT2_EO_PL               = 0x800390130701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE19_RX_WORK_STAT2_EO_PL               = 0x800390130701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE19_RX_WORK_STAT2_EO_PL               = 0x800390130701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE19_RX_WORK_STAT2_EO_PL              = 0x800390130801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE19_RX_WORK_STAT2_EO_PL              = 0x800390130801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE19_RX_WORK_STAT2_EO_PL              = 0x800390130801183Full;


static const uint64_t P9A_MC_RX0_SLICE19_RX_WORK_STAT3_EO_PL                  = 0x800398130701103Full;
//DUPS: 800398130701143F, 800398130701183F,
static const uint64_t P9A_MC_0_RX0_SLICE19_RX_WORK_STAT3_EO_PL                = 0x800398130701103Full;
//DUPS: 800398130701143F, 800398130701183F,
static const uint64_t P9A_MC_1_RX0_SLICE19_RX_WORK_STAT3_EO_PL                = 0x800398130801103Full;
//DUPS: 800398130801143F, 800398130801183F,

static const uint64_t P9A_OMIC_RX0_SLICE19_RX_WORK_STAT3_EO_PL                = 0x800398130701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE19_RX_WORK_STAT3_EO_PL              = 0x800398130701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE19_RX_WORK_STAT3_EO_PL              = 0x800398130701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE19_RX_WORK_STAT3_EO_PL              = 0x800398130701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE19_RX_WORK_STAT3_EO_PL              = 0x800398130801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE19_RX_WORK_STAT3_EO_PL              = 0x800398130801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE19_RX_WORK_STAT3_EO_PL              = 0x800398130801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE19_RX_WORK_STAT3_EO_PL               = 0x800398130701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE19_RX_WORK_STAT3_EO_PL               = 0x800398130701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE19_RX_WORK_STAT3_EO_PL               = 0x800398130701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE19_RX_WORK_STAT3_EO_PL              = 0x800398130801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE19_RX_WORK_STAT3_EO_PL              = 0x800398130801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE19_RX_WORK_STAT3_EO_PL              = 0x800398130801183Full;


static const uint64_t P9A_MC_RX0_SLICE19_RX_WORK_STAT4_O_PL                   = 0x8003A0130701103Full;
//DUPS: 8003A0130701143F, 8003A0130701183F,
static const uint64_t P9A_MC_0_RX0_SLICE19_RX_WORK_STAT4_O_PL                 = 0x8003A0130701103Full;
//DUPS: 8003A0130701143F, 8003A0130701183F,
static const uint64_t P9A_MC_1_RX0_SLICE19_RX_WORK_STAT4_O_PL                 = 0x8003A0130801103Full;
//DUPS: 8003A0130801143F, 8003A0130801183F,

static const uint64_t P9A_OMIC_RX0_SLICE19_RX_WORK_STAT4_O_PL                 = 0x8003A0130701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE19_RX_WORK_STAT4_O_PL               = 0x8003A0130701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE19_RX_WORK_STAT4_O_PL               = 0x8003A0130701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE19_RX_WORK_STAT4_O_PL               = 0x8003A0130701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE19_RX_WORK_STAT4_O_PL               = 0x8003A0130801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE19_RX_WORK_STAT4_O_PL               = 0x8003A0130801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE19_RX_WORK_STAT4_O_PL               = 0x8003A0130801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE19_RX_WORK_STAT4_O_PL                = 0x8003A0130701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE19_RX_WORK_STAT4_O_PL                = 0x8003A0130701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE19_RX_WORK_STAT4_O_PL                = 0x8003A0130701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE19_RX_WORK_STAT4_O_PL               = 0x8003A0130801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE19_RX_WORK_STAT4_O_PL               = 0x8003A0130801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE19_RX_WORK_STAT4_O_PL               = 0x8003A0130801183Full;


static const uint64_t P9A_MC_RX0_SLICE1_RX_DATA_WORK_SPARE_MODE_PL            = 0x800380010701103Full;
//DUPS: 800380010701143F, 800380010701183F,
static const uint64_t P9A_MC_0_RX0_SLICE1_RX_DATA_WORK_SPARE_MODE_PL          = 0x800380010701103Full;
//DUPS: 800380010701143F, 800380010701183F,
static const uint64_t P9A_MC_1_RX0_SLICE1_RX_DATA_WORK_SPARE_MODE_PL          = 0x800380010801103Full;
//DUPS: 800380010801143F, 800380010801183F,

static const uint64_t P9A_OMI_RX0_SLICE1_RX_DATA_WORK_SPARE_MODE_PL           = 0x800380010701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE1_RX_DATA_WORK_SPARE_MODE_PL          = 0x800380010701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE1_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380010701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE1_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380010701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE1_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380010701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE1_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380010801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE1_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380010801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE1_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380010801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE1_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380010701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE1_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380010701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE1_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380010701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE1_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380010801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE1_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380010801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE1_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380010801183Full;


static const uint64_t P9A_MC_RX0_SLICE1_RX_GLBSM_PL_CNTL1X_O_PL               = 0x800330010701103Full;
//DUPS: 800330010701143F, 800330010701183F,
static const uint64_t P9A_MC_0_RX0_SLICE1_RX_GLBSM_PL_CNTL1X_O_PL             = 0x800330010701103Full;
//DUPS: 800330010701143F, 800330010701183F,
static const uint64_t P9A_MC_1_RX0_SLICE1_RX_GLBSM_PL_CNTL1X_O_PL             = 0x800330010801103Full;
//DUPS: 800330010801143F, 800330010801183F,

static const uint64_t P9A_OMI_RX0_SLICE1_RX_GLBSM_PL_CNTL1X_O_PL              = 0x800330010701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE1_RX_GLBSM_PL_CNTL1X_O_PL             = 0x800330010701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE1_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330010701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE1_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330010701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE1_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330010701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE1_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330010801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE1_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330010801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE1_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330010801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE1_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330010701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE1_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330010701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE1_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330010701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE1_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330010801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE1_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330010801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE1_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330010801183Full;


static const uint64_t P9A_MC_RX0_SLICE1_RX_GLBSM_PL_CNTL1_O_PL                = 0x800320010701103Full;
//DUPS: 800320010701143F, 800320010701183F,
static const uint64_t P9A_MC_0_RX0_SLICE1_RX_GLBSM_PL_CNTL1_O_PL              = 0x800320010701103Full;
//DUPS: 800320010701143F, 800320010701183F,
static const uint64_t P9A_MC_1_RX0_SLICE1_RX_GLBSM_PL_CNTL1_O_PL              = 0x800320010801103Full;
//DUPS: 800320010801143F, 800320010801183F,

static const uint64_t P9A_OMI_RX0_SLICE1_RX_GLBSM_PL_CNTL1_O_PL               = 0x800320010701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE1_RX_GLBSM_PL_CNTL1_O_PL              = 0x800320010701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE1_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320010701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE1_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320010701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE1_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320010701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE1_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320010801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE1_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320010801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE1_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320010801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE1_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320010701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE1_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320010701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE1_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320010701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE1_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320010801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE1_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320010801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE1_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320010801183Full;


static const uint64_t P9A_MC_RX0_SLICE1_RX_GLBSM_PL_STAT1_O_PL                = 0x800328010701103Full;
//DUPS: 800328010701143F, 800328010701183F,
static const uint64_t P9A_MC_0_RX0_SLICE1_RX_GLBSM_PL_STAT1_O_PL              = 0x800328010701103Full;
//DUPS: 800328010701143F, 800328010701183F,
static const uint64_t P9A_MC_1_RX0_SLICE1_RX_GLBSM_PL_STAT1_O_PL              = 0x800328010801103Full;
//DUPS: 800328010801143F, 800328010801183F,

static const uint64_t P9A_OMI_RX0_SLICE1_RX_GLBSM_PL_STAT1_O_PL               = 0x800328010701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE1_RX_GLBSM_PL_STAT1_O_PL              = 0x800328010701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE1_RX_GLBSM_PL_STAT1_O_PL            = 0x800328010701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE1_RX_GLBSM_PL_STAT1_O_PL            = 0x800328010701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE1_RX_GLBSM_PL_STAT1_O_PL            = 0x800328010701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE1_RX_GLBSM_PL_STAT1_O_PL            = 0x800328010801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE1_RX_GLBSM_PL_STAT1_O_PL            = 0x800328010801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE1_RX_GLBSM_PL_STAT1_O_PL            = 0x800328010801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE1_RX_GLBSM_PL_STAT1_O_PL             = 0x800328010701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE1_RX_GLBSM_PL_STAT1_O_PL             = 0x800328010701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE1_RX_GLBSM_PL_STAT1_O_PL             = 0x800328010701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE1_RX_GLBSM_PL_STAT1_O_PL             = 0x800328010801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE1_RX_GLBSM_PL_STAT1_O_PL            = 0x800328010801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE1_RX_GLBSM_PL_STAT1_O_PL            = 0x800328010801183Full;


static const uint64_t P9A_MC_RX0_SLICE1_RX_WORK_CNTL1_O_PL                    = 0x8003A8010701103Full;
//DUPS: 8003A8010701143F, 8003A8010701183F,
static const uint64_t P9A_MC_0_RX0_SLICE1_RX_WORK_CNTL1_O_PL                  = 0x8003A8010701103Full;
//DUPS: 8003A8010701143F, 8003A8010701183F,
static const uint64_t P9A_MC_1_RX0_SLICE1_RX_WORK_CNTL1_O_PL                  = 0x8003A8010801103Full;
//DUPS: 8003A8010801143F, 8003A8010801183F,

static const uint64_t P9A_OMI_RX0_SLICE1_RX_WORK_CNTL1_O_PL                   = 0x8003A8010701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE1_RX_WORK_CNTL1_O_PL                  = 0x8003A8010701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE1_RX_WORK_CNTL1_O_PL                = 0x8003A8010701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE1_RX_WORK_CNTL1_O_PL                = 0x8003A8010701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE1_RX_WORK_CNTL1_O_PL                = 0x8003A8010701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE1_RX_WORK_CNTL1_O_PL                = 0x8003A8010801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE1_RX_WORK_CNTL1_O_PL                = 0x8003A8010801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE1_RX_WORK_CNTL1_O_PL                = 0x8003A8010801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE1_RX_WORK_CNTL1_O_PL                 = 0x8003A8010701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE1_RX_WORK_CNTL1_O_PL                 = 0x8003A8010701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE1_RX_WORK_CNTL1_O_PL                 = 0x8003A8010701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE1_RX_WORK_CNTL1_O_PL                 = 0x8003A8010801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE1_RX_WORK_CNTL1_O_PL                = 0x8003A8010801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE1_RX_WORK_CNTL1_O_PL                = 0x8003A8010801183Full;


static const uint64_t P9A_MC_RX0_SLICE1_RX_WORK_STAT1_EO_PL                   = 0x800388010701103Full;
//DUPS: 800388010701143F, 800388010701183F,
static const uint64_t P9A_MC_0_RX0_SLICE1_RX_WORK_STAT1_EO_PL                 = 0x800388010701103Full;
//DUPS: 800388010701143F, 800388010701183F,
static const uint64_t P9A_MC_1_RX0_SLICE1_RX_WORK_STAT1_EO_PL                 = 0x800388010801103Full;
//DUPS: 800388010801143F, 800388010801183F,

static const uint64_t P9A_OMI_RX0_SLICE1_RX_WORK_STAT1_EO_PL                  = 0x800388010701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE1_RX_WORK_STAT1_EO_PL                 = 0x800388010701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE1_RX_WORK_STAT1_EO_PL               = 0x800388010701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE1_RX_WORK_STAT1_EO_PL               = 0x800388010701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE1_RX_WORK_STAT1_EO_PL               = 0x800388010701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE1_RX_WORK_STAT1_EO_PL               = 0x800388010801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE1_RX_WORK_STAT1_EO_PL               = 0x800388010801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE1_RX_WORK_STAT1_EO_PL               = 0x800388010801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE1_RX_WORK_STAT1_EO_PL                = 0x800388010701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE1_RX_WORK_STAT1_EO_PL                = 0x800388010701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE1_RX_WORK_STAT1_EO_PL                = 0x800388010701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE1_RX_WORK_STAT1_EO_PL                = 0x800388010801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE1_RX_WORK_STAT1_EO_PL               = 0x800388010801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE1_RX_WORK_STAT1_EO_PL               = 0x800388010801183Full;


static const uint64_t P9A_MC_RX0_SLICE1_RX_WORK_STAT2_EO_PL                   = 0x800390010701103Full;
//DUPS: 800390010701143F, 800390010701183F,
static const uint64_t P9A_MC_0_RX0_SLICE1_RX_WORK_STAT2_EO_PL                 = 0x800390010701103Full;
//DUPS: 800390010701143F, 800390010701183F,
static const uint64_t P9A_MC_1_RX0_SLICE1_RX_WORK_STAT2_EO_PL                 = 0x800390010801103Full;
//DUPS: 800390010801143F, 800390010801183F,

static const uint64_t P9A_OMI_RX0_SLICE1_RX_WORK_STAT2_EO_PL                  = 0x800390010701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE1_RX_WORK_STAT2_EO_PL                 = 0x800390010701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE1_RX_WORK_STAT2_EO_PL               = 0x800390010701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE1_RX_WORK_STAT2_EO_PL               = 0x800390010701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE1_RX_WORK_STAT2_EO_PL               = 0x800390010701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE1_RX_WORK_STAT2_EO_PL               = 0x800390010801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE1_RX_WORK_STAT2_EO_PL               = 0x800390010801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE1_RX_WORK_STAT2_EO_PL               = 0x800390010801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE1_RX_WORK_STAT2_EO_PL                = 0x800390010701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE1_RX_WORK_STAT2_EO_PL                = 0x800390010701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE1_RX_WORK_STAT2_EO_PL                = 0x800390010701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE1_RX_WORK_STAT2_EO_PL                = 0x800390010801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE1_RX_WORK_STAT2_EO_PL               = 0x800390010801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE1_RX_WORK_STAT2_EO_PL               = 0x800390010801183Full;


static const uint64_t P9A_MC_RX0_SLICE1_RX_WORK_STAT3_EO_PL                   = 0x800398010701103Full;
//DUPS: 800398010701143F, 800398010701183F,
static const uint64_t P9A_MC_0_RX0_SLICE1_RX_WORK_STAT3_EO_PL                 = 0x800398010701103Full;
//DUPS: 800398010701143F, 800398010701183F,
static const uint64_t P9A_MC_1_RX0_SLICE1_RX_WORK_STAT3_EO_PL                 = 0x800398010801103Full;
//DUPS: 800398010801143F, 800398010801183F,

static const uint64_t P9A_OMI_RX0_SLICE1_RX_WORK_STAT3_EO_PL                  = 0x800398010701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE1_RX_WORK_STAT3_EO_PL                 = 0x800398010701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE1_RX_WORK_STAT3_EO_PL               = 0x800398010701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE1_RX_WORK_STAT3_EO_PL               = 0x800398010701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE1_RX_WORK_STAT3_EO_PL               = 0x800398010701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE1_RX_WORK_STAT3_EO_PL               = 0x800398010801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE1_RX_WORK_STAT3_EO_PL               = 0x800398010801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE1_RX_WORK_STAT3_EO_PL               = 0x800398010801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE1_RX_WORK_STAT3_EO_PL                = 0x800398010701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE1_RX_WORK_STAT3_EO_PL                = 0x800398010701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE1_RX_WORK_STAT3_EO_PL                = 0x800398010701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE1_RX_WORK_STAT3_EO_PL                = 0x800398010801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE1_RX_WORK_STAT3_EO_PL               = 0x800398010801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE1_RX_WORK_STAT3_EO_PL               = 0x800398010801183Full;


static const uint64_t P9A_MC_RX0_SLICE1_RX_WORK_STAT4_O_PL                    = 0x8003A0010701103Full;
//DUPS: 8003A0010701143F, 8003A0010701183F,
static const uint64_t P9A_MC_0_RX0_SLICE1_RX_WORK_STAT4_O_PL                  = 0x8003A0010701103Full;
//DUPS: 8003A0010701143F, 8003A0010701183F,
static const uint64_t P9A_MC_1_RX0_SLICE1_RX_WORK_STAT4_O_PL                  = 0x8003A0010801103Full;
//DUPS: 8003A0010801143F, 8003A0010801183F,

static const uint64_t P9A_OMI_RX0_SLICE1_RX_WORK_STAT4_O_PL                   = 0x8003A0010701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE1_RX_WORK_STAT4_O_PL                  = 0x8003A0010701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE1_RX_WORK_STAT4_O_PL                = 0x8003A0010701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE1_RX_WORK_STAT4_O_PL                = 0x8003A0010701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE1_RX_WORK_STAT4_O_PL                = 0x8003A0010701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE1_RX_WORK_STAT4_O_PL                = 0x8003A0010801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE1_RX_WORK_STAT4_O_PL                = 0x8003A0010801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE1_RX_WORK_STAT4_O_PL                = 0x8003A0010801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE1_RX_WORK_STAT4_O_PL                 = 0x8003A0010701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE1_RX_WORK_STAT4_O_PL                 = 0x8003A0010701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE1_RX_WORK_STAT4_O_PL                 = 0x8003A0010701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE1_RX_WORK_STAT4_O_PL                 = 0x8003A0010801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE1_RX_WORK_STAT4_O_PL                = 0x8003A0010801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE1_RX_WORK_STAT4_O_PL                = 0x8003A0010801183Full;


static const uint64_t P9A_MC_RX0_SLICE20_RX_DATA_WORK_SPARE_MODE_PL           = 0x800380140701103Full;
//DUPS: 800380140701143F, 800380140701183F,
static const uint64_t P9A_MC_0_RX0_SLICE20_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380140701103Full;
//DUPS: 800380140701143F, 800380140701183F,
static const uint64_t P9A_MC_1_RX0_SLICE20_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380140801103Full;
//DUPS: 800380140801143F, 800380140801183F,

static const uint64_t P9A_OMIC_RX0_SLICE20_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380140701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE20_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380140701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE20_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380140701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE20_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380140701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE20_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380140801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE20_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380140801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE20_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380140801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE20_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380140701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE20_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380140701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE20_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380140701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE20_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380140801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE20_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380140801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE20_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380140801183Full;


static const uint64_t P9A_MC_RX0_SLICE20_RX_GLBSM_PL_CNTL1X_O_PL              = 0x800330140701103Full;
//DUPS: 800330140701143F, 800330140701183F,
static const uint64_t P9A_MC_0_RX0_SLICE20_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330140701103Full;
//DUPS: 800330140701143F, 800330140701183F,
static const uint64_t P9A_MC_1_RX0_SLICE20_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330140801103Full;
//DUPS: 800330140801143F, 800330140801183F,

static const uint64_t P9A_OMIC_RX0_SLICE20_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330140701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE20_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330140701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE20_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330140701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE20_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330140701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE20_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330140801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE20_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330140801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE20_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330140801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE20_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330140701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE20_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330140701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE20_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330140701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE20_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330140801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE20_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330140801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE20_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330140801183Full;


static const uint64_t P9A_MC_RX0_SLICE20_RX_GLBSM_PL_CNTL1_O_PL               = 0x800320140701103Full;
//DUPS: 800320140701143F, 800320140701183F,
static const uint64_t P9A_MC_0_RX0_SLICE20_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320140701103Full;
//DUPS: 800320140701143F, 800320140701183F,
static const uint64_t P9A_MC_1_RX0_SLICE20_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320140801103Full;
//DUPS: 800320140801143F, 800320140801183F,

static const uint64_t P9A_OMIC_RX0_SLICE20_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320140701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE20_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320140701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE20_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320140701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE20_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320140701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE20_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320140801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE20_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320140801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE20_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320140801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE20_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320140701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE20_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320140701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE20_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320140701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE20_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320140801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE20_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320140801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE20_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320140801183Full;


static const uint64_t P9A_MC_RX0_SLICE20_RX_GLBSM_PL_STAT1_O_PL               = 0x800328140701103Full;
//DUPS: 800328140701143F, 800328140701183F,
static const uint64_t P9A_MC_0_RX0_SLICE20_RX_GLBSM_PL_STAT1_O_PL             = 0x800328140701103Full;
//DUPS: 800328140701143F, 800328140701183F,
static const uint64_t P9A_MC_1_RX0_SLICE20_RX_GLBSM_PL_STAT1_O_PL             = 0x800328140801103Full;
//DUPS: 800328140801143F, 800328140801183F,

static const uint64_t P9A_OMIC_RX0_SLICE20_RX_GLBSM_PL_STAT1_O_PL             = 0x800328140701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE20_RX_GLBSM_PL_STAT1_O_PL           = 0x800328140701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE20_RX_GLBSM_PL_STAT1_O_PL           = 0x800328140701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE20_RX_GLBSM_PL_STAT1_O_PL           = 0x800328140701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE20_RX_GLBSM_PL_STAT1_O_PL           = 0x800328140801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE20_RX_GLBSM_PL_STAT1_O_PL           = 0x800328140801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE20_RX_GLBSM_PL_STAT1_O_PL           = 0x800328140801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE20_RX_GLBSM_PL_STAT1_O_PL            = 0x800328140701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE20_RX_GLBSM_PL_STAT1_O_PL            = 0x800328140701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE20_RX_GLBSM_PL_STAT1_O_PL            = 0x800328140701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE20_RX_GLBSM_PL_STAT1_O_PL           = 0x800328140801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE20_RX_GLBSM_PL_STAT1_O_PL           = 0x800328140801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE20_RX_GLBSM_PL_STAT1_O_PL           = 0x800328140801183Full;


static const uint64_t P9A_MC_RX0_SLICE20_RX_WORK_CNTL1_O_PL                   = 0x8003A8140701103Full;
//DUPS: 8003A8140701143F, 8003A8140701183F,
static const uint64_t P9A_MC_0_RX0_SLICE20_RX_WORK_CNTL1_O_PL                 = 0x8003A8140701103Full;
//DUPS: 8003A8140701143F, 8003A8140701183F,
static const uint64_t P9A_MC_1_RX0_SLICE20_RX_WORK_CNTL1_O_PL                 = 0x8003A8140801103Full;
//DUPS: 8003A8140801143F, 8003A8140801183F,

static const uint64_t P9A_OMIC_RX0_SLICE20_RX_WORK_CNTL1_O_PL                 = 0x8003A8140701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE20_RX_WORK_CNTL1_O_PL               = 0x8003A8140701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE20_RX_WORK_CNTL1_O_PL               = 0x8003A8140701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE20_RX_WORK_CNTL1_O_PL               = 0x8003A8140701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE20_RX_WORK_CNTL1_O_PL               = 0x8003A8140801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE20_RX_WORK_CNTL1_O_PL               = 0x8003A8140801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE20_RX_WORK_CNTL1_O_PL               = 0x8003A8140801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE20_RX_WORK_CNTL1_O_PL                = 0x8003A8140701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE20_RX_WORK_CNTL1_O_PL                = 0x8003A8140701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE20_RX_WORK_CNTL1_O_PL                = 0x8003A8140701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE20_RX_WORK_CNTL1_O_PL               = 0x8003A8140801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE20_RX_WORK_CNTL1_O_PL               = 0x8003A8140801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE20_RX_WORK_CNTL1_O_PL               = 0x8003A8140801183Full;


static const uint64_t P9A_MC_RX0_SLICE20_RX_WORK_STAT1_EO_PL                  = 0x800388140701103Full;
//DUPS: 800388140701143F, 800388140701183F,
static const uint64_t P9A_MC_0_RX0_SLICE20_RX_WORK_STAT1_EO_PL                = 0x800388140701103Full;
//DUPS: 800388140701143F, 800388140701183F,
static const uint64_t P9A_MC_1_RX0_SLICE20_RX_WORK_STAT1_EO_PL                = 0x800388140801103Full;
//DUPS: 800388140801143F, 800388140801183F,

static const uint64_t P9A_OMIC_RX0_SLICE20_RX_WORK_STAT1_EO_PL                = 0x800388140701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE20_RX_WORK_STAT1_EO_PL              = 0x800388140701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE20_RX_WORK_STAT1_EO_PL              = 0x800388140701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE20_RX_WORK_STAT1_EO_PL              = 0x800388140701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE20_RX_WORK_STAT1_EO_PL              = 0x800388140801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE20_RX_WORK_STAT1_EO_PL              = 0x800388140801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE20_RX_WORK_STAT1_EO_PL              = 0x800388140801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE20_RX_WORK_STAT1_EO_PL               = 0x800388140701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE20_RX_WORK_STAT1_EO_PL               = 0x800388140701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE20_RX_WORK_STAT1_EO_PL               = 0x800388140701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE20_RX_WORK_STAT1_EO_PL              = 0x800388140801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE20_RX_WORK_STAT1_EO_PL              = 0x800388140801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE20_RX_WORK_STAT1_EO_PL              = 0x800388140801183Full;


static const uint64_t P9A_MC_RX0_SLICE20_RX_WORK_STAT2_EO_PL                  = 0x800390140701103Full;
//DUPS: 800390140701143F, 800390140701183F,
static const uint64_t P9A_MC_0_RX0_SLICE20_RX_WORK_STAT2_EO_PL                = 0x800390140701103Full;
//DUPS: 800390140701143F, 800390140701183F,
static const uint64_t P9A_MC_1_RX0_SLICE20_RX_WORK_STAT2_EO_PL                = 0x800390140801103Full;
//DUPS: 800390140801143F, 800390140801183F,

static const uint64_t P9A_OMIC_RX0_SLICE20_RX_WORK_STAT2_EO_PL                = 0x800390140701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE20_RX_WORK_STAT2_EO_PL              = 0x800390140701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE20_RX_WORK_STAT2_EO_PL              = 0x800390140701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE20_RX_WORK_STAT2_EO_PL              = 0x800390140701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE20_RX_WORK_STAT2_EO_PL              = 0x800390140801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE20_RX_WORK_STAT2_EO_PL              = 0x800390140801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE20_RX_WORK_STAT2_EO_PL              = 0x800390140801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE20_RX_WORK_STAT2_EO_PL               = 0x800390140701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE20_RX_WORK_STAT2_EO_PL               = 0x800390140701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE20_RX_WORK_STAT2_EO_PL               = 0x800390140701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE20_RX_WORK_STAT2_EO_PL              = 0x800390140801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE20_RX_WORK_STAT2_EO_PL              = 0x800390140801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE20_RX_WORK_STAT2_EO_PL              = 0x800390140801183Full;


static const uint64_t P9A_MC_RX0_SLICE20_RX_WORK_STAT3_EO_PL                  = 0x800398140701103Full;
//DUPS: 800398140701143F, 800398140701183F,
static const uint64_t P9A_MC_0_RX0_SLICE20_RX_WORK_STAT3_EO_PL                = 0x800398140701103Full;
//DUPS: 800398140701143F, 800398140701183F,
static const uint64_t P9A_MC_1_RX0_SLICE20_RX_WORK_STAT3_EO_PL                = 0x800398140801103Full;
//DUPS: 800398140801143F, 800398140801183F,

static const uint64_t P9A_OMIC_RX0_SLICE20_RX_WORK_STAT3_EO_PL                = 0x800398140701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE20_RX_WORK_STAT3_EO_PL              = 0x800398140701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE20_RX_WORK_STAT3_EO_PL              = 0x800398140701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE20_RX_WORK_STAT3_EO_PL              = 0x800398140701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE20_RX_WORK_STAT3_EO_PL              = 0x800398140801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE20_RX_WORK_STAT3_EO_PL              = 0x800398140801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE20_RX_WORK_STAT3_EO_PL              = 0x800398140801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE20_RX_WORK_STAT3_EO_PL               = 0x800398140701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE20_RX_WORK_STAT3_EO_PL               = 0x800398140701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE20_RX_WORK_STAT3_EO_PL               = 0x800398140701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE20_RX_WORK_STAT3_EO_PL              = 0x800398140801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE20_RX_WORK_STAT3_EO_PL              = 0x800398140801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE20_RX_WORK_STAT3_EO_PL              = 0x800398140801183Full;


static const uint64_t P9A_MC_RX0_SLICE20_RX_WORK_STAT4_O_PL                   = 0x8003A0140701103Full;
//DUPS: 8003A0140701143F, 8003A0140701183F,
static const uint64_t P9A_MC_0_RX0_SLICE20_RX_WORK_STAT4_O_PL                 = 0x8003A0140701103Full;
//DUPS: 8003A0140701143F, 8003A0140701183F,
static const uint64_t P9A_MC_1_RX0_SLICE20_RX_WORK_STAT4_O_PL                 = 0x8003A0140801103Full;
//DUPS: 8003A0140801143F, 8003A0140801183F,

static const uint64_t P9A_OMIC_RX0_SLICE20_RX_WORK_STAT4_O_PL                 = 0x8003A0140701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE20_RX_WORK_STAT4_O_PL               = 0x8003A0140701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE20_RX_WORK_STAT4_O_PL               = 0x8003A0140701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE20_RX_WORK_STAT4_O_PL               = 0x8003A0140701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE20_RX_WORK_STAT4_O_PL               = 0x8003A0140801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE20_RX_WORK_STAT4_O_PL               = 0x8003A0140801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE20_RX_WORK_STAT4_O_PL               = 0x8003A0140801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE20_RX_WORK_STAT4_O_PL                = 0x8003A0140701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE20_RX_WORK_STAT4_O_PL                = 0x8003A0140701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE20_RX_WORK_STAT4_O_PL                = 0x8003A0140701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE20_RX_WORK_STAT4_O_PL               = 0x8003A0140801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE20_RX_WORK_STAT4_O_PL               = 0x8003A0140801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE20_RX_WORK_STAT4_O_PL               = 0x8003A0140801183Full;


static const uint64_t P9A_MC_RX0_SLICE21_RX_DATA_WORK_SPARE_MODE_PL           = 0x800380150701103Full;
//DUPS: 800380150701143F, 800380150701183F,
static const uint64_t P9A_MC_0_RX0_SLICE21_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380150701103Full;
//DUPS: 800380150701143F, 800380150701183F,
static const uint64_t P9A_MC_1_RX0_SLICE21_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380150801103Full;
//DUPS: 800380150801143F, 800380150801183F,

static const uint64_t P9A_OMIC_RX0_SLICE21_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380150701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE21_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380150701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE21_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380150701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE21_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380150701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE21_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380150801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE21_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380150801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE21_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380150801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE21_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380150701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE21_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380150701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE21_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380150701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE21_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380150801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE21_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380150801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE21_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380150801183Full;


static const uint64_t P9A_MC_RX0_SLICE21_RX_GLBSM_PL_CNTL1X_O_PL              = 0x800330150701103Full;
//DUPS: 800330150701143F, 800330150701183F,
static const uint64_t P9A_MC_0_RX0_SLICE21_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330150701103Full;
//DUPS: 800330150701143F, 800330150701183F,
static const uint64_t P9A_MC_1_RX0_SLICE21_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330150801103Full;
//DUPS: 800330150801143F, 800330150801183F,

static const uint64_t P9A_OMIC_RX0_SLICE21_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330150701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE21_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330150701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE21_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330150701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE21_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330150701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE21_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330150801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE21_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330150801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE21_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330150801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE21_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330150701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE21_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330150701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE21_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330150701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE21_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330150801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE21_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330150801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE21_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330150801183Full;


static const uint64_t P9A_MC_RX0_SLICE21_RX_GLBSM_PL_CNTL1_O_PL               = 0x800320150701103Full;
//DUPS: 800320150701143F, 800320150701183F,
static const uint64_t P9A_MC_0_RX0_SLICE21_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320150701103Full;
//DUPS: 800320150701143F, 800320150701183F,
static const uint64_t P9A_MC_1_RX0_SLICE21_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320150801103Full;
//DUPS: 800320150801143F, 800320150801183F,

static const uint64_t P9A_OMIC_RX0_SLICE21_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320150701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE21_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320150701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE21_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320150701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE21_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320150701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE21_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320150801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE21_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320150801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE21_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320150801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE21_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320150701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE21_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320150701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE21_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320150701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE21_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320150801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE21_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320150801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE21_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320150801183Full;


static const uint64_t P9A_MC_RX0_SLICE21_RX_GLBSM_PL_STAT1_O_PL               = 0x800328150701103Full;
//DUPS: 800328150701143F, 800328150701183F,
static const uint64_t P9A_MC_0_RX0_SLICE21_RX_GLBSM_PL_STAT1_O_PL             = 0x800328150701103Full;
//DUPS: 800328150701143F, 800328150701183F,
static const uint64_t P9A_MC_1_RX0_SLICE21_RX_GLBSM_PL_STAT1_O_PL             = 0x800328150801103Full;
//DUPS: 800328150801143F, 800328150801183F,

static const uint64_t P9A_OMIC_RX0_SLICE21_RX_GLBSM_PL_STAT1_O_PL             = 0x800328150701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE21_RX_GLBSM_PL_STAT1_O_PL           = 0x800328150701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE21_RX_GLBSM_PL_STAT1_O_PL           = 0x800328150701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE21_RX_GLBSM_PL_STAT1_O_PL           = 0x800328150701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE21_RX_GLBSM_PL_STAT1_O_PL           = 0x800328150801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE21_RX_GLBSM_PL_STAT1_O_PL           = 0x800328150801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE21_RX_GLBSM_PL_STAT1_O_PL           = 0x800328150801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE21_RX_GLBSM_PL_STAT1_O_PL            = 0x800328150701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE21_RX_GLBSM_PL_STAT1_O_PL            = 0x800328150701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE21_RX_GLBSM_PL_STAT1_O_PL            = 0x800328150701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE21_RX_GLBSM_PL_STAT1_O_PL           = 0x800328150801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE21_RX_GLBSM_PL_STAT1_O_PL           = 0x800328150801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE21_RX_GLBSM_PL_STAT1_O_PL           = 0x800328150801183Full;


static const uint64_t P9A_MC_RX0_SLICE21_RX_WORK_CNTL1_O_PL                   = 0x8003A8150701103Full;
//DUPS: 8003A8150701143F, 8003A8150701183F,
static const uint64_t P9A_MC_0_RX0_SLICE21_RX_WORK_CNTL1_O_PL                 = 0x8003A8150701103Full;
//DUPS: 8003A8150701143F, 8003A8150701183F,
static const uint64_t P9A_MC_1_RX0_SLICE21_RX_WORK_CNTL1_O_PL                 = 0x8003A8150801103Full;
//DUPS: 8003A8150801143F, 8003A8150801183F,

static const uint64_t P9A_OMIC_RX0_SLICE21_RX_WORK_CNTL1_O_PL                 = 0x8003A8150701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE21_RX_WORK_CNTL1_O_PL               = 0x8003A8150701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE21_RX_WORK_CNTL1_O_PL               = 0x8003A8150701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE21_RX_WORK_CNTL1_O_PL               = 0x8003A8150701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE21_RX_WORK_CNTL1_O_PL               = 0x8003A8150801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE21_RX_WORK_CNTL1_O_PL               = 0x8003A8150801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE21_RX_WORK_CNTL1_O_PL               = 0x8003A8150801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE21_RX_WORK_CNTL1_O_PL                = 0x8003A8150701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE21_RX_WORK_CNTL1_O_PL                = 0x8003A8150701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE21_RX_WORK_CNTL1_O_PL                = 0x8003A8150701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE21_RX_WORK_CNTL1_O_PL               = 0x8003A8150801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE21_RX_WORK_CNTL1_O_PL               = 0x8003A8150801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE21_RX_WORK_CNTL1_O_PL               = 0x8003A8150801183Full;


static const uint64_t P9A_MC_RX0_SLICE21_RX_WORK_STAT1_EO_PL                  = 0x800388150701103Full;
//DUPS: 800388150701143F, 800388150701183F,
static const uint64_t P9A_MC_0_RX0_SLICE21_RX_WORK_STAT1_EO_PL                = 0x800388150701103Full;
//DUPS: 800388150701143F, 800388150701183F,
static const uint64_t P9A_MC_1_RX0_SLICE21_RX_WORK_STAT1_EO_PL                = 0x800388150801103Full;
//DUPS: 800388150801143F, 800388150801183F,

static const uint64_t P9A_OMIC_RX0_SLICE21_RX_WORK_STAT1_EO_PL                = 0x800388150701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE21_RX_WORK_STAT1_EO_PL              = 0x800388150701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE21_RX_WORK_STAT1_EO_PL              = 0x800388150701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE21_RX_WORK_STAT1_EO_PL              = 0x800388150701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE21_RX_WORK_STAT1_EO_PL              = 0x800388150801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE21_RX_WORK_STAT1_EO_PL              = 0x800388150801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE21_RX_WORK_STAT1_EO_PL              = 0x800388150801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE21_RX_WORK_STAT1_EO_PL               = 0x800388150701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE21_RX_WORK_STAT1_EO_PL               = 0x800388150701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE21_RX_WORK_STAT1_EO_PL               = 0x800388150701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE21_RX_WORK_STAT1_EO_PL              = 0x800388150801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE21_RX_WORK_STAT1_EO_PL              = 0x800388150801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE21_RX_WORK_STAT1_EO_PL              = 0x800388150801183Full;


static const uint64_t P9A_MC_RX0_SLICE21_RX_WORK_STAT2_EO_PL                  = 0x800390150701103Full;
//DUPS: 800390150701143F, 800390150701183F,
static const uint64_t P9A_MC_0_RX0_SLICE21_RX_WORK_STAT2_EO_PL                = 0x800390150701103Full;
//DUPS: 800390150701143F, 800390150701183F,
static const uint64_t P9A_MC_1_RX0_SLICE21_RX_WORK_STAT2_EO_PL                = 0x800390150801103Full;
//DUPS: 800390150801143F, 800390150801183F,

static const uint64_t P9A_OMIC_RX0_SLICE21_RX_WORK_STAT2_EO_PL                = 0x800390150701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE21_RX_WORK_STAT2_EO_PL              = 0x800390150701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE21_RX_WORK_STAT2_EO_PL              = 0x800390150701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE21_RX_WORK_STAT2_EO_PL              = 0x800390150701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE21_RX_WORK_STAT2_EO_PL              = 0x800390150801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE21_RX_WORK_STAT2_EO_PL              = 0x800390150801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE21_RX_WORK_STAT2_EO_PL              = 0x800390150801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE21_RX_WORK_STAT2_EO_PL               = 0x800390150701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE21_RX_WORK_STAT2_EO_PL               = 0x800390150701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE21_RX_WORK_STAT2_EO_PL               = 0x800390150701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE21_RX_WORK_STAT2_EO_PL              = 0x800390150801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE21_RX_WORK_STAT2_EO_PL              = 0x800390150801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE21_RX_WORK_STAT2_EO_PL              = 0x800390150801183Full;


static const uint64_t P9A_MC_RX0_SLICE21_RX_WORK_STAT3_EO_PL                  = 0x800398150701103Full;
//DUPS: 800398150701143F, 800398150701183F,
static const uint64_t P9A_MC_0_RX0_SLICE21_RX_WORK_STAT3_EO_PL                = 0x800398150701103Full;
//DUPS: 800398150701143F, 800398150701183F,
static const uint64_t P9A_MC_1_RX0_SLICE21_RX_WORK_STAT3_EO_PL                = 0x800398150801103Full;
//DUPS: 800398150801143F, 800398150801183F,

static const uint64_t P9A_OMIC_RX0_SLICE21_RX_WORK_STAT3_EO_PL                = 0x800398150701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE21_RX_WORK_STAT3_EO_PL              = 0x800398150701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE21_RX_WORK_STAT3_EO_PL              = 0x800398150701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE21_RX_WORK_STAT3_EO_PL              = 0x800398150701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE21_RX_WORK_STAT3_EO_PL              = 0x800398150801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE21_RX_WORK_STAT3_EO_PL              = 0x800398150801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE21_RX_WORK_STAT3_EO_PL              = 0x800398150801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE21_RX_WORK_STAT3_EO_PL               = 0x800398150701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE21_RX_WORK_STAT3_EO_PL               = 0x800398150701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE21_RX_WORK_STAT3_EO_PL               = 0x800398150701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE21_RX_WORK_STAT3_EO_PL              = 0x800398150801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE21_RX_WORK_STAT3_EO_PL              = 0x800398150801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE21_RX_WORK_STAT3_EO_PL              = 0x800398150801183Full;


static const uint64_t P9A_MC_RX0_SLICE21_RX_WORK_STAT4_O_PL                   = 0x8003A0150701103Full;
//DUPS: 8003A0150701143F, 8003A0150701183F,
static const uint64_t P9A_MC_0_RX0_SLICE21_RX_WORK_STAT4_O_PL                 = 0x8003A0150701103Full;
//DUPS: 8003A0150701143F, 8003A0150701183F,
static const uint64_t P9A_MC_1_RX0_SLICE21_RX_WORK_STAT4_O_PL                 = 0x8003A0150801103Full;
//DUPS: 8003A0150801143F, 8003A0150801183F,

static const uint64_t P9A_OMIC_RX0_SLICE21_RX_WORK_STAT4_O_PL                 = 0x8003A0150701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE21_RX_WORK_STAT4_O_PL               = 0x8003A0150701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE21_RX_WORK_STAT4_O_PL               = 0x8003A0150701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE21_RX_WORK_STAT4_O_PL               = 0x8003A0150701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE21_RX_WORK_STAT4_O_PL               = 0x8003A0150801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE21_RX_WORK_STAT4_O_PL               = 0x8003A0150801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE21_RX_WORK_STAT4_O_PL               = 0x8003A0150801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE21_RX_WORK_STAT4_O_PL                = 0x8003A0150701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE21_RX_WORK_STAT4_O_PL                = 0x8003A0150701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE21_RX_WORK_STAT4_O_PL                = 0x8003A0150701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE21_RX_WORK_STAT4_O_PL               = 0x8003A0150801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE21_RX_WORK_STAT4_O_PL               = 0x8003A0150801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE21_RX_WORK_STAT4_O_PL               = 0x8003A0150801183Full;


static const uint64_t P9A_MC_RX0_SLICE22_RX_DATA_WORK_SPARE_MODE_PL           = 0x800380160701103Full;
//DUPS: 800380160701143F, 800380160701183F,
static const uint64_t P9A_MC_0_RX0_SLICE22_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380160701103Full;
//DUPS: 800380160701143F, 800380160701183F,
static const uint64_t P9A_MC_1_RX0_SLICE22_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380160801103Full;
//DUPS: 800380160801143F, 800380160801183F,

static const uint64_t P9A_OMIC_RX0_SLICE22_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380160701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE22_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380160701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE22_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380160701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE22_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380160701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE22_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380160801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE22_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380160801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE22_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380160801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE22_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380160701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE22_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380160701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE22_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380160701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE22_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380160801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE22_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380160801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE22_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380160801183Full;


static const uint64_t P9A_MC_RX0_SLICE22_RX_GLBSM_PL_CNTL1X_O_PL              = 0x800330160701103Full;
//DUPS: 800330160701143F, 800330160701183F,
static const uint64_t P9A_MC_0_RX0_SLICE22_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330160701103Full;
//DUPS: 800330160701143F, 800330160701183F,
static const uint64_t P9A_MC_1_RX0_SLICE22_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330160801103Full;
//DUPS: 800330160801143F, 800330160801183F,

static const uint64_t P9A_OMIC_RX0_SLICE22_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330160701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE22_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330160701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE22_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330160701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE22_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330160701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE22_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330160801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE22_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330160801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE22_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330160801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE22_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330160701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE22_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330160701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE22_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330160701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE22_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330160801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE22_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330160801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE22_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330160801183Full;


static const uint64_t P9A_MC_RX0_SLICE22_RX_GLBSM_PL_CNTL1_O_PL               = 0x800320160701103Full;
//DUPS: 800320160701143F, 800320160701183F,
static const uint64_t P9A_MC_0_RX0_SLICE22_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320160701103Full;
//DUPS: 800320160701143F, 800320160701183F,
static const uint64_t P9A_MC_1_RX0_SLICE22_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320160801103Full;
//DUPS: 800320160801143F, 800320160801183F,

static const uint64_t P9A_OMIC_RX0_SLICE22_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320160701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE22_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320160701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE22_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320160701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE22_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320160701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE22_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320160801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE22_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320160801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE22_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320160801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE22_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320160701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE22_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320160701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE22_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320160701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE22_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320160801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE22_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320160801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE22_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320160801183Full;


static const uint64_t P9A_MC_RX0_SLICE22_RX_GLBSM_PL_STAT1_O_PL               = 0x800328160701103Full;
//DUPS: 800328160701143F, 800328160701183F,
static const uint64_t P9A_MC_0_RX0_SLICE22_RX_GLBSM_PL_STAT1_O_PL             = 0x800328160701103Full;
//DUPS: 800328160701143F, 800328160701183F,
static const uint64_t P9A_MC_1_RX0_SLICE22_RX_GLBSM_PL_STAT1_O_PL             = 0x800328160801103Full;
//DUPS: 800328160801143F, 800328160801183F,

static const uint64_t P9A_OMIC_RX0_SLICE22_RX_GLBSM_PL_STAT1_O_PL             = 0x800328160701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE22_RX_GLBSM_PL_STAT1_O_PL           = 0x800328160701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE22_RX_GLBSM_PL_STAT1_O_PL           = 0x800328160701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE22_RX_GLBSM_PL_STAT1_O_PL           = 0x800328160701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE22_RX_GLBSM_PL_STAT1_O_PL           = 0x800328160801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE22_RX_GLBSM_PL_STAT1_O_PL           = 0x800328160801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE22_RX_GLBSM_PL_STAT1_O_PL           = 0x800328160801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE22_RX_GLBSM_PL_STAT1_O_PL            = 0x800328160701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE22_RX_GLBSM_PL_STAT1_O_PL            = 0x800328160701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE22_RX_GLBSM_PL_STAT1_O_PL            = 0x800328160701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE22_RX_GLBSM_PL_STAT1_O_PL           = 0x800328160801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE22_RX_GLBSM_PL_STAT1_O_PL           = 0x800328160801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE22_RX_GLBSM_PL_STAT1_O_PL           = 0x800328160801183Full;


static const uint64_t P9A_MC_RX0_SLICE22_RX_WORK_CNTL1_O_PL                   = 0x8003A8160701103Full;
//DUPS: 8003A8160701143F, 8003A8160701183F,
static const uint64_t P9A_MC_0_RX0_SLICE22_RX_WORK_CNTL1_O_PL                 = 0x8003A8160701103Full;
//DUPS: 8003A8160701143F, 8003A8160701183F,
static const uint64_t P9A_MC_1_RX0_SLICE22_RX_WORK_CNTL1_O_PL                 = 0x8003A8160801103Full;
//DUPS: 8003A8160801143F, 8003A8160801183F,

static const uint64_t P9A_OMIC_RX0_SLICE22_RX_WORK_CNTL1_O_PL                 = 0x8003A8160701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE22_RX_WORK_CNTL1_O_PL               = 0x8003A8160701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE22_RX_WORK_CNTL1_O_PL               = 0x8003A8160701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE22_RX_WORK_CNTL1_O_PL               = 0x8003A8160701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE22_RX_WORK_CNTL1_O_PL               = 0x8003A8160801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE22_RX_WORK_CNTL1_O_PL               = 0x8003A8160801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE22_RX_WORK_CNTL1_O_PL               = 0x8003A8160801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE22_RX_WORK_CNTL1_O_PL                = 0x8003A8160701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE22_RX_WORK_CNTL1_O_PL                = 0x8003A8160701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE22_RX_WORK_CNTL1_O_PL                = 0x8003A8160701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE22_RX_WORK_CNTL1_O_PL               = 0x8003A8160801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE22_RX_WORK_CNTL1_O_PL               = 0x8003A8160801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE22_RX_WORK_CNTL1_O_PL               = 0x8003A8160801183Full;


static const uint64_t P9A_MC_RX0_SLICE22_RX_WORK_STAT1_EO_PL                  = 0x800388160701103Full;
//DUPS: 800388160701143F, 800388160701183F,
static const uint64_t P9A_MC_0_RX0_SLICE22_RX_WORK_STAT1_EO_PL                = 0x800388160701103Full;
//DUPS: 800388160701143F, 800388160701183F,
static const uint64_t P9A_MC_1_RX0_SLICE22_RX_WORK_STAT1_EO_PL                = 0x800388160801103Full;
//DUPS: 800388160801143F, 800388160801183F,

static const uint64_t P9A_OMIC_RX0_SLICE22_RX_WORK_STAT1_EO_PL                = 0x800388160701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE22_RX_WORK_STAT1_EO_PL              = 0x800388160701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE22_RX_WORK_STAT1_EO_PL              = 0x800388160701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE22_RX_WORK_STAT1_EO_PL              = 0x800388160701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE22_RX_WORK_STAT1_EO_PL              = 0x800388160801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE22_RX_WORK_STAT1_EO_PL              = 0x800388160801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE22_RX_WORK_STAT1_EO_PL              = 0x800388160801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE22_RX_WORK_STAT1_EO_PL               = 0x800388160701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE22_RX_WORK_STAT1_EO_PL               = 0x800388160701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE22_RX_WORK_STAT1_EO_PL               = 0x800388160701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE22_RX_WORK_STAT1_EO_PL              = 0x800388160801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE22_RX_WORK_STAT1_EO_PL              = 0x800388160801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE22_RX_WORK_STAT1_EO_PL              = 0x800388160801183Full;


static const uint64_t P9A_MC_RX0_SLICE22_RX_WORK_STAT2_EO_PL                  = 0x800390160701103Full;
//DUPS: 800390160701143F, 800390160701183F,
static const uint64_t P9A_MC_0_RX0_SLICE22_RX_WORK_STAT2_EO_PL                = 0x800390160701103Full;
//DUPS: 800390160701143F, 800390160701183F,
static const uint64_t P9A_MC_1_RX0_SLICE22_RX_WORK_STAT2_EO_PL                = 0x800390160801103Full;
//DUPS: 800390160801143F, 800390160801183F,

static const uint64_t P9A_OMIC_RX0_SLICE22_RX_WORK_STAT2_EO_PL                = 0x800390160701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE22_RX_WORK_STAT2_EO_PL              = 0x800390160701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE22_RX_WORK_STAT2_EO_PL              = 0x800390160701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE22_RX_WORK_STAT2_EO_PL              = 0x800390160701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE22_RX_WORK_STAT2_EO_PL              = 0x800390160801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE22_RX_WORK_STAT2_EO_PL              = 0x800390160801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE22_RX_WORK_STAT2_EO_PL              = 0x800390160801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE22_RX_WORK_STAT2_EO_PL               = 0x800390160701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE22_RX_WORK_STAT2_EO_PL               = 0x800390160701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE22_RX_WORK_STAT2_EO_PL               = 0x800390160701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE22_RX_WORK_STAT2_EO_PL              = 0x800390160801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE22_RX_WORK_STAT2_EO_PL              = 0x800390160801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE22_RX_WORK_STAT2_EO_PL              = 0x800390160801183Full;


static const uint64_t P9A_MC_RX0_SLICE22_RX_WORK_STAT3_EO_PL                  = 0x800398160701103Full;
//DUPS: 800398160701143F, 800398160701183F,
static const uint64_t P9A_MC_0_RX0_SLICE22_RX_WORK_STAT3_EO_PL                = 0x800398160701103Full;
//DUPS: 800398160701143F, 800398160701183F,
static const uint64_t P9A_MC_1_RX0_SLICE22_RX_WORK_STAT3_EO_PL                = 0x800398160801103Full;
//DUPS: 800398160801143F, 800398160801183F,

static const uint64_t P9A_OMIC_RX0_SLICE22_RX_WORK_STAT3_EO_PL                = 0x800398160701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE22_RX_WORK_STAT3_EO_PL              = 0x800398160701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE22_RX_WORK_STAT3_EO_PL              = 0x800398160701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE22_RX_WORK_STAT3_EO_PL              = 0x800398160701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE22_RX_WORK_STAT3_EO_PL              = 0x800398160801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE22_RX_WORK_STAT3_EO_PL              = 0x800398160801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE22_RX_WORK_STAT3_EO_PL              = 0x800398160801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE22_RX_WORK_STAT3_EO_PL               = 0x800398160701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE22_RX_WORK_STAT3_EO_PL               = 0x800398160701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE22_RX_WORK_STAT3_EO_PL               = 0x800398160701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE22_RX_WORK_STAT3_EO_PL              = 0x800398160801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE22_RX_WORK_STAT3_EO_PL              = 0x800398160801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE22_RX_WORK_STAT3_EO_PL              = 0x800398160801183Full;


static const uint64_t P9A_MC_RX0_SLICE22_RX_WORK_STAT4_O_PL                   = 0x8003A0160701103Full;
//DUPS: 8003A0160701143F, 8003A0160701183F,
static const uint64_t P9A_MC_0_RX0_SLICE22_RX_WORK_STAT4_O_PL                 = 0x8003A0160701103Full;
//DUPS: 8003A0160701143F, 8003A0160701183F,
static const uint64_t P9A_MC_1_RX0_SLICE22_RX_WORK_STAT4_O_PL                 = 0x8003A0160801103Full;
//DUPS: 8003A0160801143F, 8003A0160801183F,

static const uint64_t P9A_OMIC_RX0_SLICE22_RX_WORK_STAT4_O_PL                 = 0x8003A0160701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE22_RX_WORK_STAT4_O_PL               = 0x8003A0160701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE22_RX_WORK_STAT4_O_PL               = 0x8003A0160701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE22_RX_WORK_STAT4_O_PL               = 0x8003A0160701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE22_RX_WORK_STAT4_O_PL               = 0x8003A0160801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE22_RX_WORK_STAT4_O_PL               = 0x8003A0160801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE22_RX_WORK_STAT4_O_PL               = 0x8003A0160801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE22_RX_WORK_STAT4_O_PL                = 0x8003A0160701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE22_RX_WORK_STAT4_O_PL                = 0x8003A0160701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE22_RX_WORK_STAT4_O_PL                = 0x8003A0160701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE22_RX_WORK_STAT4_O_PL               = 0x8003A0160801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE22_RX_WORK_STAT4_O_PL               = 0x8003A0160801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE22_RX_WORK_STAT4_O_PL               = 0x8003A0160801183Full;


static const uint64_t P9A_MC_RX0_SLICE23_RX_DATA_WORK_SPARE_MODE_PL           = 0x800380170701103Full;
//DUPS: 800380170701143F, 800380170701183F,
static const uint64_t P9A_MC_0_RX0_SLICE23_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380170701103Full;
//DUPS: 800380170701143F, 800380170701183F,
static const uint64_t P9A_MC_1_RX0_SLICE23_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380170801103Full;
//DUPS: 800380170801143F, 800380170801183F,

static const uint64_t P9A_OMIC_RX0_SLICE23_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380170701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE23_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380170701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE23_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380170701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE23_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380170701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE23_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380170801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE23_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380170801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE23_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380170801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE23_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380170701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE23_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380170701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE23_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380170701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE23_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380170801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE23_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380170801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE23_RX_DATA_WORK_SPARE_MODE_PL       = 0x800380170801183Full;


static const uint64_t P9A_MC_RX0_SLICE23_RX_GLBSM_PL_CNTL1X_O_PL              = 0x800330170701103Full;
//DUPS: 800330170701143F, 800330170701183F,
static const uint64_t P9A_MC_0_RX0_SLICE23_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330170701103Full;
//DUPS: 800330170701143F, 800330170701183F,
static const uint64_t P9A_MC_1_RX0_SLICE23_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330170801103Full;
//DUPS: 800330170801143F, 800330170801183F,

static const uint64_t P9A_OMIC_RX0_SLICE23_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330170701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE23_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330170701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE23_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330170701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE23_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330170701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE23_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330170801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE23_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330170801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE23_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330170801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE23_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330170701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE23_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330170701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE23_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330170701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE23_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330170801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE23_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330170801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE23_RX_GLBSM_PL_CNTL1X_O_PL          = 0x800330170801183Full;


static const uint64_t P9A_MC_RX0_SLICE23_RX_GLBSM_PL_CNTL1_O_PL               = 0x800320170701103Full;
//DUPS: 800320170701143F, 800320170701183F,
static const uint64_t P9A_MC_0_RX0_SLICE23_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320170701103Full;
//DUPS: 800320170701143F, 800320170701183F,
static const uint64_t P9A_MC_1_RX0_SLICE23_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320170801103Full;
//DUPS: 800320170801143F, 800320170801183F,

static const uint64_t P9A_OMIC_RX0_SLICE23_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320170701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE23_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320170701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE23_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320170701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE23_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320170701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE23_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320170801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE23_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320170801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE23_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320170801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE23_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320170701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE23_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320170701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE23_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320170701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE23_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320170801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE23_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320170801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE23_RX_GLBSM_PL_CNTL1_O_PL           = 0x800320170801183Full;


static const uint64_t P9A_MC_RX0_SLICE23_RX_GLBSM_PL_STAT1_O_PL               = 0x800328170701103Full;
//DUPS: 800328170701143F, 800328170701183F,
static const uint64_t P9A_MC_0_RX0_SLICE23_RX_GLBSM_PL_STAT1_O_PL             = 0x800328170701103Full;
//DUPS: 800328170701143F, 800328170701183F,
static const uint64_t P9A_MC_1_RX0_SLICE23_RX_GLBSM_PL_STAT1_O_PL             = 0x800328170801103Full;
//DUPS: 800328170801143F, 800328170801183F,

static const uint64_t P9A_OMIC_RX0_SLICE23_RX_GLBSM_PL_STAT1_O_PL             = 0x800328170701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE23_RX_GLBSM_PL_STAT1_O_PL           = 0x800328170701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE23_RX_GLBSM_PL_STAT1_O_PL           = 0x800328170701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE23_RX_GLBSM_PL_STAT1_O_PL           = 0x800328170701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE23_RX_GLBSM_PL_STAT1_O_PL           = 0x800328170801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE23_RX_GLBSM_PL_STAT1_O_PL           = 0x800328170801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE23_RX_GLBSM_PL_STAT1_O_PL           = 0x800328170801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE23_RX_GLBSM_PL_STAT1_O_PL            = 0x800328170701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE23_RX_GLBSM_PL_STAT1_O_PL            = 0x800328170701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE23_RX_GLBSM_PL_STAT1_O_PL            = 0x800328170701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE23_RX_GLBSM_PL_STAT1_O_PL           = 0x800328170801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE23_RX_GLBSM_PL_STAT1_O_PL           = 0x800328170801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE23_RX_GLBSM_PL_STAT1_O_PL           = 0x800328170801183Full;


static const uint64_t P9A_MC_RX0_SLICE23_RX_WORK_CNTL1_O_PL                   = 0x8003A8170701103Full;
//DUPS: 8003A8170701143F, 8003A8170701183F,
static const uint64_t P9A_MC_0_RX0_SLICE23_RX_WORK_CNTL1_O_PL                 = 0x8003A8170701103Full;
//DUPS: 8003A8170701143F, 8003A8170701183F,
static const uint64_t P9A_MC_1_RX0_SLICE23_RX_WORK_CNTL1_O_PL                 = 0x8003A8170801103Full;
//DUPS: 8003A8170801143F, 8003A8170801183F,

static const uint64_t P9A_OMIC_RX0_SLICE23_RX_WORK_CNTL1_O_PL                 = 0x8003A8170701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE23_RX_WORK_CNTL1_O_PL               = 0x8003A8170701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE23_RX_WORK_CNTL1_O_PL               = 0x8003A8170701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE23_RX_WORK_CNTL1_O_PL               = 0x8003A8170701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE23_RX_WORK_CNTL1_O_PL               = 0x8003A8170801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE23_RX_WORK_CNTL1_O_PL               = 0x8003A8170801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE23_RX_WORK_CNTL1_O_PL               = 0x8003A8170801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE23_RX_WORK_CNTL1_O_PL                = 0x8003A8170701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE23_RX_WORK_CNTL1_O_PL                = 0x8003A8170701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE23_RX_WORK_CNTL1_O_PL                = 0x8003A8170701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE23_RX_WORK_CNTL1_O_PL               = 0x8003A8170801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE23_RX_WORK_CNTL1_O_PL               = 0x8003A8170801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE23_RX_WORK_CNTL1_O_PL               = 0x8003A8170801183Full;


static const uint64_t P9A_MC_RX0_SLICE23_RX_WORK_STAT1_EO_PL                  = 0x800388170701103Full;
//DUPS: 800388170701143F, 800388170701183F,
static const uint64_t P9A_MC_0_RX0_SLICE23_RX_WORK_STAT1_EO_PL                = 0x800388170701103Full;
//DUPS: 800388170701143F, 800388170701183F,
static const uint64_t P9A_MC_1_RX0_SLICE23_RX_WORK_STAT1_EO_PL                = 0x800388170801103Full;
//DUPS: 800388170801143F, 800388170801183F,

static const uint64_t P9A_OMIC_RX0_SLICE23_RX_WORK_STAT1_EO_PL                = 0x800388170701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE23_RX_WORK_STAT1_EO_PL              = 0x800388170701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE23_RX_WORK_STAT1_EO_PL              = 0x800388170701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE23_RX_WORK_STAT1_EO_PL              = 0x800388170701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE23_RX_WORK_STAT1_EO_PL              = 0x800388170801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE23_RX_WORK_STAT1_EO_PL              = 0x800388170801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE23_RX_WORK_STAT1_EO_PL              = 0x800388170801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE23_RX_WORK_STAT1_EO_PL               = 0x800388170701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE23_RX_WORK_STAT1_EO_PL               = 0x800388170701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE23_RX_WORK_STAT1_EO_PL               = 0x800388170701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE23_RX_WORK_STAT1_EO_PL              = 0x800388170801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE23_RX_WORK_STAT1_EO_PL              = 0x800388170801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE23_RX_WORK_STAT1_EO_PL              = 0x800388170801183Full;


static const uint64_t P9A_MC_RX0_SLICE23_RX_WORK_STAT2_EO_PL                  = 0x800390170701103Full;
//DUPS: 800390170701143F, 800390170701183F,
static const uint64_t P9A_MC_0_RX0_SLICE23_RX_WORK_STAT2_EO_PL                = 0x800390170701103Full;
//DUPS: 800390170701143F, 800390170701183F,
static const uint64_t P9A_MC_1_RX0_SLICE23_RX_WORK_STAT2_EO_PL                = 0x800390170801103Full;
//DUPS: 800390170801143F, 800390170801183F,

static const uint64_t P9A_OMIC_RX0_SLICE23_RX_WORK_STAT2_EO_PL                = 0x800390170701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE23_RX_WORK_STAT2_EO_PL              = 0x800390170701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE23_RX_WORK_STAT2_EO_PL              = 0x800390170701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE23_RX_WORK_STAT2_EO_PL              = 0x800390170701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE23_RX_WORK_STAT2_EO_PL              = 0x800390170801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE23_RX_WORK_STAT2_EO_PL              = 0x800390170801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE23_RX_WORK_STAT2_EO_PL              = 0x800390170801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE23_RX_WORK_STAT2_EO_PL               = 0x800390170701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE23_RX_WORK_STAT2_EO_PL               = 0x800390170701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE23_RX_WORK_STAT2_EO_PL               = 0x800390170701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE23_RX_WORK_STAT2_EO_PL              = 0x800390170801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE23_RX_WORK_STAT2_EO_PL              = 0x800390170801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE23_RX_WORK_STAT2_EO_PL              = 0x800390170801183Full;


static const uint64_t P9A_MC_RX0_SLICE23_RX_WORK_STAT3_EO_PL                  = 0x800398170701103Full;
//DUPS: 800398170701143F, 800398170701183F,
static const uint64_t P9A_MC_0_RX0_SLICE23_RX_WORK_STAT3_EO_PL                = 0x800398170701103Full;
//DUPS: 800398170701143F, 800398170701183F,
static const uint64_t P9A_MC_1_RX0_SLICE23_RX_WORK_STAT3_EO_PL                = 0x800398170801103Full;
//DUPS: 800398170801143F, 800398170801183F,

static const uint64_t P9A_OMIC_RX0_SLICE23_RX_WORK_STAT3_EO_PL                = 0x800398170701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE23_RX_WORK_STAT3_EO_PL              = 0x800398170701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE23_RX_WORK_STAT3_EO_PL              = 0x800398170701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE23_RX_WORK_STAT3_EO_PL              = 0x800398170701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE23_RX_WORK_STAT3_EO_PL              = 0x800398170801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE23_RX_WORK_STAT3_EO_PL              = 0x800398170801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE23_RX_WORK_STAT3_EO_PL              = 0x800398170801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE23_RX_WORK_STAT3_EO_PL               = 0x800398170701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE23_RX_WORK_STAT3_EO_PL               = 0x800398170701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE23_RX_WORK_STAT3_EO_PL               = 0x800398170701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE23_RX_WORK_STAT3_EO_PL              = 0x800398170801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE23_RX_WORK_STAT3_EO_PL              = 0x800398170801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE23_RX_WORK_STAT3_EO_PL              = 0x800398170801183Full;


static const uint64_t P9A_MC_RX0_SLICE23_RX_WORK_STAT4_O_PL                   = 0x8003A0170701103Full;
//DUPS: 8003A0170701143F, 8003A0170701183F,
static const uint64_t P9A_MC_0_RX0_SLICE23_RX_WORK_STAT4_O_PL                 = 0x8003A0170701103Full;
//DUPS: 8003A0170701143F, 8003A0170701183F,
static const uint64_t P9A_MC_1_RX0_SLICE23_RX_WORK_STAT4_O_PL                 = 0x8003A0170801103Full;
//DUPS: 8003A0170801143F, 8003A0170801183F,

static const uint64_t P9A_OMIC_RX0_SLICE23_RX_WORK_STAT4_O_PL                 = 0x8003A0170701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE23_RX_WORK_STAT4_O_PL               = 0x8003A0170701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE23_RX_WORK_STAT4_O_PL               = 0x8003A0170701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE23_RX_WORK_STAT4_O_PL               = 0x8003A0170701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE23_RX_WORK_STAT4_O_PL               = 0x8003A0170801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE23_RX_WORK_STAT4_O_PL               = 0x8003A0170801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE23_RX_WORK_STAT4_O_PL               = 0x8003A0170801183Full;

static const uint64_t P9A_OMI_2_RX0_SLICE23_RX_WORK_STAT4_O_PL                = 0x8003A0170701103Full;

static const uint64_t P9A_OMI_5_RX0_SLICE23_RX_WORK_STAT4_O_PL                = 0x8003A0170701143Full;

static const uint64_t P9A_OMI_8_RX0_SLICE23_RX_WORK_STAT4_O_PL                = 0x8003A0170701183Full;

static const uint64_t P9A_OMI_10_RX0_SLICE23_RX_WORK_STAT4_O_PL               = 0x8003A0170801103Full;

static const uint64_t P9A_OMI_13_RX0_SLICE23_RX_WORK_STAT4_O_PL               = 0x8003A0170801143Full;

static const uint64_t P9A_OMI_16_RX0_SLICE23_RX_WORK_STAT4_O_PL               = 0x8003A0170801183Full;


static const uint64_t P9A_MC_RX0_SLICE2_RX_DATA_WORK_SPARE_MODE_PL            = 0x800380020701103Full;
//DUPS: 800380020701143F, 800380020701183F,
static const uint64_t P9A_MC_0_RX0_SLICE2_RX_DATA_WORK_SPARE_MODE_PL          = 0x800380020701103Full;
//DUPS: 800380020701143F, 800380020701183F,
static const uint64_t P9A_MC_1_RX0_SLICE2_RX_DATA_WORK_SPARE_MODE_PL          = 0x800380020801103Full;
//DUPS: 800380020801143F, 800380020801183F,

static const uint64_t P9A_OMI_RX0_SLICE2_RX_DATA_WORK_SPARE_MODE_PL           = 0x800380020701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE2_RX_DATA_WORK_SPARE_MODE_PL          = 0x800380020701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE2_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380020701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE2_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380020701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE2_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380020701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE2_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380020801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE2_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380020801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE2_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380020801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE2_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380020701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE2_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380020701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE2_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380020701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE2_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380020801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE2_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380020801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE2_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380020801183Full;


static const uint64_t P9A_MC_RX0_SLICE2_RX_GLBSM_PL_CNTL1X_O_PL               = 0x800330020701103Full;
//DUPS: 800330020701143F, 800330020701183F,
static const uint64_t P9A_MC_0_RX0_SLICE2_RX_GLBSM_PL_CNTL1X_O_PL             = 0x800330020701103Full;
//DUPS: 800330020701143F, 800330020701183F,
static const uint64_t P9A_MC_1_RX0_SLICE2_RX_GLBSM_PL_CNTL1X_O_PL             = 0x800330020801103Full;
//DUPS: 800330020801143F, 800330020801183F,

static const uint64_t P9A_OMI_RX0_SLICE2_RX_GLBSM_PL_CNTL1X_O_PL              = 0x800330020701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE2_RX_GLBSM_PL_CNTL1X_O_PL             = 0x800330020701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE2_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330020701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE2_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330020701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE2_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330020701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE2_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330020801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE2_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330020801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE2_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330020801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE2_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330020701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE2_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330020701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE2_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330020701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE2_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330020801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE2_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330020801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE2_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330020801183Full;


static const uint64_t P9A_MC_RX0_SLICE2_RX_GLBSM_PL_CNTL1_O_PL                = 0x800320020701103Full;
//DUPS: 800320020701143F, 800320020701183F,
static const uint64_t P9A_MC_0_RX0_SLICE2_RX_GLBSM_PL_CNTL1_O_PL              = 0x800320020701103Full;
//DUPS: 800320020701143F, 800320020701183F,
static const uint64_t P9A_MC_1_RX0_SLICE2_RX_GLBSM_PL_CNTL1_O_PL              = 0x800320020801103Full;
//DUPS: 800320020801143F, 800320020801183F,

static const uint64_t P9A_OMI_RX0_SLICE2_RX_GLBSM_PL_CNTL1_O_PL               = 0x800320020701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE2_RX_GLBSM_PL_CNTL1_O_PL              = 0x800320020701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE2_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320020701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE2_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320020701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE2_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320020701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE2_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320020801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE2_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320020801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE2_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320020801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE2_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320020701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE2_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320020701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE2_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320020701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE2_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320020801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE2_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320020801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE2_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320020801183Full;


static const uint64_t P9A_MC_RX0_SLICE2_RX_GLBSM_PL_STAT1_O_PL                = 0x800328020701103Full;
//DUPS: 800328020701143F, 800328020701183F,
static const uint64_t P9A_MC_0_RX0_SLICE2_RX_GLBSM_PL_STAT1_O_PL              = 0x800328020701103Full;
//DUPS: 800328020701143F, 800328020701183F,
static const uint64_t P9A_MC_1_RX0_SLICE2_RX_GLBSM_PL_STAT1_O_PL              = 0x800328020801103Full;
//DUPS: 800328020801143F, 800328020801183F,

static const uint64_t P9A_OMI_RX0_SLICE2_RX_GLBSM_PL_STAT1_O_PL               = 0x800328020701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE2_RX_GLBSM_PL_STAT1_O_PL              = 0x800328020701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE2_RX_GLBSM_PL_STAT1_O_PL            = 0x800328020701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE2_RX_GLBSM_PL_STAT1_O_PL            = 0x800328020701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE2_RX_GLBSM_PL_STAT1_O_PL            = 0x800328020701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE2_RX_GLBSM_PL_STAT1_O_PL            = 0x800328020801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE2_RX_GLBSM_PL_STAT1_O_PL            = 0x800328020801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE2_RX_GLBSM_PL_STAT1_O_PL            = 0x800328020801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE2_RX_GLBSM_PL_STAT1_O_PL             = 0x800328020701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE2_RX_GLBSM_PL_STAT1_O_PL             = 0x800328020701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE2_RX_GLBSM_PL_STAT1_O_PL             = 0x800328020701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE2_RX_GLBSM_PL_STAT1_O_PL             = 0x800328020801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE2_RX_GLBSM_PL_STAT1_O_PL            = 0x800328020801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE2_RX_GLBSM_PL_STAT1_O_PL            = 0x800328020801183Full;


static const uint64_t P9A_MC_RX0_SLICE2_RX_WORK_CNTL1_O_PL                    = 0x8003A8020701103Full;
//DUPS: 8003A8020701143F, 8003A8020701183F,
static const uint64_t P9A_MC_0_RX0_SLICE2_RX_WORK_CNTL1_O_PL                  = 0x8003A8020701103Full;
//DUPS: 8003A8020701143F, 8003A8020701183F,
static const uint64_t P9A_MC_1_RX0_SLICE2_RX_WORK_CNTL1_O_PL                  = 0x8003A8020801103Full;
//DUPS: 8003A8020801143F, 8003A8020801183F,

static const uint64_t P9A_OMI_RX0_SLICE2_RX_WORK_CNTL1_O_PL                   = 0x8003A8020701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE2_RX_WORK_CNTL1_O_PL                  = 0x8003A8020701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE2_RX_WORK_CNTL1_O_PL                = 0x8003A8020701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE2_RX_WORK_CNTL1_O_PL                = 0x8003A8020701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE2_RX_WORK_CNTL1_O_PL                = 0x8003A8020701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE2_RX_WORK_CNTL1_O_PL                = 0x8003A8020801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE2_RX_WORK_CNTL1_O_PL                = 0x8003A8020801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE2_RX_WORK_CNTL1_O_PL                = 0x8003A8020801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE2_RX_WORK_CNTL1_O_PL                 = 0x8003A8020701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE2_RX_WORK_CNTL1_O_PL                 = 0x8003A8020701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE2_RX_WORK_CNTL1_O_PL                 = 0x8003A8020701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE2_RX_WORK_CNTL1_O_PL                 = 0x8003A8020801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE2_RX_WORK_CNTL1_O_PL                = 0x8003A8020801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE2_RX_WORK_CNTL1_O_PL                = 0x8003A8020801183Full;


static const uint64_t P9A_MC_RX0_SLICE2_RX_WORK_STAT1_EO_PL                   = 0x800388020701103Full;
//DUPS: 800388020701143F, 800388020701183F,
static const uint64_t P9A_MC_0_RX0_SLICE2_RX_WORK_STAT1_EO_PL                 = 0x800388020701103Full;
//DUPS: 800388020701143F, 800388020701183F,
static const uint64_t P9A_MC_1_RX0_SLICE2_RX_WORK_STAT1_EO_PL                 = 0x800388020801103Full;
//DUPS: 800388020801143F, 800388020801183F,

static const uint64_t P9A_OMI_RX0_SLICE2_RX_WORK_STAT1_EO_PL                  = 0x800388020701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE2_RX_WORK_STAT1_EO_PL                 = 0x800388020701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE2_RX_WORK_STAT1_EO_PL               = 0x800388020701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE2_RX_WORK_STAT1_EO_PL               = 0x800388020701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE2_RX_WORK_STAT1_EO_PL               = 0x800388020701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE2_RX_WORK_STAT1_EO_PL               = 0x800388020801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE2_RX_WORK_STAT1_EO_PL               = 0x800388020801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE2_RX_WORK_STAT1_EO_PL               = 0x800388020801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE2_RX_WORK_STAT1_EO_PL                = 0x800388020701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE2_RX_WORK_STAT1_EO_PL                = 0x800388020701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE2_RX_WORK_STAT1_EO_PL                = 0x800388020701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE2_RX_WORK_STAT1_EO_PL                = 0x800388020801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE2_RX_WORK_STAT1_EO_PL               = 0x800388020801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE2_RX_WORK_STAT1_EO_PL               = 0x800388020801183Full;


static const uint64_t P9A_MC_RX0_SLICE2_RX_WORK_STAT2_EO_PL                   = 0x800390020701103Full;
//DUPS: 800390020701143F, 800390020701183F,
static const uint64_t P9A_MC_0_RX0_SLICE2_RX_WORK_STAT2_EO_PL                 = 0x800390020701103Full;
//DUPS: 800390020701143F, 800390020701183F,
static const uint64_t P9A_MC_1_RX0_SLICE2_RX_WORK_STAT2_EO_PL                 = 0x800390020801103Full;
//DUPS: 800390020801143F, 800390020801183F,

static const uint64_t P9A_OMI_RX0_SLICE2_RX_WORK_STAT2_EO_PL                  = 0x800390020701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE2_RX_WORK_STAT2_EO_PL                 = 0x800390020701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE2_RX_WORK_STAT2_EO_PL               = 0x800390020701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE2_RX_WORK_STAT2_EO_PL               = 0x800390020701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE2_RX_WORK_STAT2_EO_PL               = 0x800390020701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE2_RX_WORK_STAT2_EO_PL               = 0x800390020801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE2_RX_WORK_STAT2_EO_PL               = 0x800390020801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE2_RX_WORK_STAT2_EO_PL               = 0x800390020801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE2_RX_WORK_STAT2_EO_PL                = 0x800390020701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE2_RX_WORK_STAT2_EO_PL                = 0x800390020701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE2_RX_WORK_STAT2_EO_PL                = 0x800390020701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE2_RX_WORK_STAT2_EO_PL                = 0x800390020801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE2_RX_WORK_STAT2_EO_PL               = 0x800390020801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE2_RX_WORK_STAT2_EO_PL               = 0x800390020801183Full;


static const uint64_t P9A_MC_RX0_SLICE2_RX_WORK_STAT3_EO_PL                   = 0x800398020701103Full;
//DUPS: 800398020701143F, 800398020701183F,
static const uint64_t P9A_MC_0_RX0_SLICE2_RX_WORK_STAT3_EO_PL                 = 0x800398020701103Full;
//DUPS: 800398020701143F, 800398020701183F,
static const uint64_t P9A_MC_1_RX0_SLICE2_RX_WORK_STAT3_EO_PL                 = 0x800398020801103Full;
//DUPS: 800398020801143F, 800398020801183F,

static const uint64_t P9A_OMI_RX0_SLICE2_RX_WORK_STAT3_EO_PL                  = 0x800398020701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE2_RX_WORK_STAT3_EO_PL                 = 0x800398020701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE2_RX_WORK_STAT3_EO_PL               = 0x800398020701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE2_RX_WORK_STAT3_EO_PL               = 0x800398020701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE2_RX_WORK_STAT3_EO_PL               = 0x800398020701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE2_RX_WORK_STAT3_EO_PL               = 0x800398020801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE2_RX_WORK_STAT3_EO_PL               = 0x800398020801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE2_RX_WORK_STAT3_EO_PL               = 0x800398020801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE2_RX_WORK_STAT3_EO_PL                = 0x800398020701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE2_RX_WORK_STAT3_EO_PL                = 0x800398020701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE2_RX_WORK_STAT3_EO_PL                = 0x800398020701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE2_RX_WORK_STAT3_EO_PL                = 0x800398020801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE2_RX_WORK_STAT3_EO_PL               = 0x800398020801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE2_RX_WORK_STAT3_EO_PL               = 0x800398020801183Full;


static const uint64_t P9A_MC_RX0_SLICE2_RX_WORK_STAT4_O_PL                    = 0x8003A0020701103Full;
//DUPS: 8003A0020701143F, 8003A0020701183F,
static const uint64_t P9A_MC_0_RX0_SLICE2_RX_WORK_STAT4_O_PL                  = 0x8003A0020701103Full;
//DUPS: 8003A0020701143F, 8003A0020701183F,
static const uint64_t P9A_MC_1_RX0_SLICE2_RX_WORK_STAT4_O_PL                  = 0x8003A0020801103Full;
//DUPS: 8003A0020801143F, 8003A0020801183F,

static const uint64_t P9A_OMI_RX0_SLICE2_RX_WORK_STAT4_O_PL                   = 0x8003A0020701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE2_RX_WORK_STAT4_O_PL                  = 0x8003A0020701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE2_RX_WORK_STAT4_O_PL                = 0x8003A0020701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE2_RX_WORK_STAT4_O_PL                = 0x8003A0020701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE2_RX_WORK_STAT4_O_PL                = 0x8003A0020701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE2_RX_WORK_STAT4_O_PL                = 0x8003A0020801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE2_RX_WORK_STAT4_O_PL                = 0x8003A0020801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE2_RX_WORK_STAT4_O_PL                = 0x8003A0020801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE2_RX_WORK_STAT4_O_PL                 = 0x8003A0020701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE2_RX_WORK_STAT4_O_PL                 = 0x8003A0020701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE2_RX_WORK_STAT4_O_PL                 = 0x8003A0020701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE2_RX_WORK_STAT4_O_PL                 = 0x8003A0020801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE2_RX_WORK_STAT4_O_PL                = 0x8003A0020801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE2_RX_WORK_STAT4_O_PL                = 0x8003A0020801183Full;


static const uint64_t P9A_MC_RX0_SLICE3_RX_DATA_WORK_SPARE_MODE_PL            = 0x800380030701103Full;
//DUPS: 800380030701143F, 800380030701183F,
static const uint64_t P9A_MC_0_RX0_SLICE3_RX_DATA_WORK_SPARE_MODE_PL          = 0x800380030701103Full;
//DUPS: 800380030701143F, 800380030701183F,
static const uint64_t P9A_MC_1_RX0_SLICE3_RX_DATA_WORK_SPARE_MODE_PL          = 0x800380030801103Full;
//DUPS: 800380030801143F, 800380030801183F,

static const uint64_t P9A_OMI_RX0_SLICE3_RX_DATA_WORK_SPARE_MODE_PL           = 0x800380030701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE3_RX_DATA_WORK_SPARE_MODE_PL          = 0x800380030701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE3_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380030701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE3_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380030701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE3_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380030701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE3_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380030801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE3_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380030801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE3_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380030801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE3_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380030701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE3_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380030701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE3_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380030701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE3_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380030801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE3_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380030801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE3_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380030801183Full;


static const uint64_t P9A_MC_RX0_SLICE3_RX_GLBSM_PL_CNTL1X_O_PL               = 0x800330030701103Full;
//DUPS: 800330030701143F, 800330030701183F,
static const uint64_t P9A_MC_0_RX0_SLICE3_RX_GLBSM_PL_CNTL1X_O_PL             = 0x800330030701103Full;
//DUPS: 800330030701143F, 800330030701183F,
static const uint64_t P9A_MC_1_RX0_SLICE3_RX_GLBSM_PL_CNTL1X_O_PL             = 0x800330030801103Full;
//DUPS: 800330030801143F, 800330030801183F,

static const uint64_t P9A_OMI_RX0_SLICE3_RX_GLBSM_PL_CNTL1X_O_PL              = 0x800330030701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE3_RX_GLBSM_PL_CNTL1X_O_PL             = 0x800330030701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE3_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330030701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE3_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330030701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE3_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330030701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE3_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330030801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE3_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330030801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE3_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330030801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE3_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330030701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE3_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330030701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE3_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330030701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE3_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330030801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE3_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330030801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE3_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330030801183Full;


static const uint64_t P9A_MC_RX0_SLICE3_RX_GLBSM_PL_CNTL1_O_PL                = 0x800320030701103Full;
//DUPS: 800320030701143F, 800320030701183F,
static const uint64_t P9A_MC_0_RX0_SLICE3_RX_GLBSM_PL_CNTL1_O_PL              = 0x800320030701103Full;
//DUPS: 800320030701143F, 800320030701183F,
static const uint64_t P9A_MC_1_RX0_SLICE3_RX_GLBSM_PL_CNTL1_O_PL              = 0x800320030801103Full;
//DUPS: 800320030801143F, 800320030801183F,

static const uint64_t P9A_OMI_RX0_SLICE3_RX_GLBSM_PL_CNTL1_O_PL               = 0x800320030701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE3_RX_GLBSM_PL_CNTL1_O_PL              = 0x800320030701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE3_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320030701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE3_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320030701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE3_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320030701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE3_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320030801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE3_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320030801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE3_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320030801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE3_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320030701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE3_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320030701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE3_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320030701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE3_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320030801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE3_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320030801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE3_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320030801183Full;


static const uint64_t P9A_MC_RX0_SLICE3_RX_GLBSM_PL_STAT1_O_PL                = 0x800328030701103Full;
//DUPS: 800328030701143F, 800328030701183F,
static const uint64_t P9A_MC_0_RX0_SLICE3_RX_GLBSM_PL_STAT1_O_PL              = 0x800328030701103Full;
//DUPS: 800328030701143F, 800328030701183F,
static const uint64_t P9A_MC_1_RX0_SLICE3_RX_GLBSM_PL_STAT1_O_PL              = 0x800328030801103Full;
//DUPS: 800328030801143F, 800328030801183F,

static const uint64_t P9A_OMI_RX0_SLICE3_RX_GLBSM_PL_STAT1_O_PL               = 0x800328030701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE3_RX_GLBSM_PL_STAT1_O_PL              = 0x800328030701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE3_RX_GLBSM_PL_STAT1_O_PL            = 0x800328030701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE3_RX_GLBSM_PL_STAT1_O_PL            = 0x800328030701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE3_RX_GLBSM_PL_STAT1_O_PL            = 0x800328030701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE3_RX_GLBSM_PL_STAT1_O_PL            = 0x800328030801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE3_RX_GLBSM_PL_STAT1_O_PL            = 0x800328030801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE3_RX_GLBSM_PL_STAT1_O_PL            = 0x800328030801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE3_RX_GLBSM_PL_STAT1_O_PL             = 0x800328030701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE3_RX_GLBSM_PL_STAT1_O_PL             = 0x800328030701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE3_RX_GLBSM_PL_STAT1_O_PL             = 0x800328030701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE3_RX_GLBSM_PL_STAT1_O_PL             = 0x800328030801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE3_RX_GLBSM_PL_STAT1_O_PL            = 0x800328030801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE3_RX_GLBSM_PL_STAT1_O_PL            = 0x800328030801183Full;


static const uint64_t P9A_MC_RX0_SLICE3_RX_WORK_CNTL1_O_PL                    = 0x8003A8030701103Full;
//DUPS: 8003A8030701143F, 8003A8030701183F,
static const uint64_t P9A_MC_0_RX0_SLICE3_RX_WORK_CNTL1_O_PL                  = 0x8003A8030701103Full;
//DUPS: 8003A8030701143F, 8003A8030701183F,
static const uint64_t P9A_MC_1_RX0_SLICE3_RX_WORK_CNTL1_O_PL                  = 0x8003A8030801103Full;
//DUPS: 8003A8030801143F, 8003A8030801183F,

static const uint64_t P9A_OMI_RX0_SLICE3_RX_WORK_CNTL1_O_PL                   = 0x8003A8030701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE3_RX_WORK_CNTL1_O_PL                  = 0x8003A8030701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE3_RX_WORK_CNTL1_O_PL                = 0x8003A8030701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE3_RX_WORK_CNTL1_O_PL                = 0x8003A8030701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE3_RX_WORK_CNTL1_O_PL                = 0x8003A8030701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE3_RX_WORK_CNTL1_O_PL                = 0x8003A8030801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE3_RX_WORK_CNTL1_O_PL                = 0x8003A8030801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE3_RX_WORK_CNTL1_O_PL                = 0x8003A8030801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE3_RX_WORK_CNTL1_O_PL                 = 0x8003A8030701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE3_RX_WORK_CNTL1_O_PL                 = 0x8003A8030701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE3_RX_WORK_CNTL1_O_PL                 = 0x8003A8030701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE3_RX_WORK_CNTL1_O_PL                 = 0x8003A8030801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE3_RX_WORK_CNTL1_O_PL                = 0x8003A8030801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE3_RX_WORK_CNTL1_O_PL                = 0x8003A8030801183Full;


static const uint64_t P9A_MC_RX0_SLICE3_RX_WORK_STAT1_EO_PL                   = 0x800388030701103Full;
//DUPS: 800388030701143F, 800388030701183F,
static const uint64_t P9A_MC_0_RX0_SLICE3_RX_WORK_STAT1_EO_PL                 = 0x800388030701103Full;
//DUPS: 800388030701143F, 800388030701183F,
static const uint64_t P9A_MC_1_RX0_SLICE3_RX_WORK_STAT1_EO_PL                 = 0x800388030801103Full;
//DUPS: 800388030801143F, 800388030801183F,

static const uint64_t P9A_OMI_RX0_SLICE3_RX_WORK_STAT1_EO_PL                  = 0x800388030701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE3_RX_WORK_STAT1_EO_PL                 = 0x800388030701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE3_RX_WORK_STAT1_EO_PL               = 0x800388030701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE3_RX_WORK_STAT1_EO_PL               = 0x800388030701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE3_RX_WORK_STAT1_EO_PL               = 0x800388030701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE3_RX_WORK_STAT1_EO_PL               = 0x800388030801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE3_RX_WORK_STAT1_EO_PL               = 0x800388030801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE3_RX_WORK_STAT1_EO_PL               = 0x800388030801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE3_RX_WORK_STAT1_EO_PL                = 0x800388030701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE3_RX_WORK_STAT1_EO_PL                = 0x800388030701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE3_RX_WORK_STAT1_EO_PL                = 0x800388030701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE3_RX_WORK_STAT1_EO_PL                = 0x800388030801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE3_RX_WORK_STAT1_EO_PL               = 0x800388030801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE3_RX_WORK_STAT1_EO_PL               = 0x800388030801183Full;


static const uint64_t P9A_MC_RX0_SLICE3_RX_WORK_STAT2_EO_PL                   = 0x800390030701103Full;
//DUPS: 800390030701143F, 800390030701183F,
static const uint64_t P9A_MC_0_RX0_SLICE3_RX_WORK_STAT2_EO_PL                 = 0x800390030701103Full;
//DUPS: 800390030701143F, 800390030701183F,
static const uint64_t P9A_MC_1_RX0_SLICE3_RX_WORK_STAT2_EO_PL                 = 0x800390030801103Full;
//DUPS: 800390030801143F, 800390030801183F,

static const uint64_t P9A_OMI_RX0_SLICE3_RX_WORK_STAT2_EO_PL                  = 0x800390030701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE3_RX_WORK_STAT2_EO_PL                 = 0x800390030701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE3_RX_WORK_STAT2_EO_PL               = 0x800390030701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE3_RX_WORK_STAT2_EO_PL               = 0x800390030701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE3_RX_WORK_STAT2_EO_PL               = 0x800390030701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE3_RX_WORK_STAT2_EO_PL               = 0x800390030801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE3_RX_WORK_STAT2_EO_PL               = 0x800390030801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE3_RX_WORK_STAT2_EO_PL               = 0x800390030801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE3_RX_WORK_STAT2_EO_PL                = 0x800390030701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE3_RX_WORK_STAT2_EO_PL                = 0x800390030701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE3_RX_WORK_STAT2_EO_PL                = 0x800390030701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE3_RX_WORK_STAT2_EO_PL                = 0x800390030801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE3_RX_WORK_STAT2_EO_PL               = 0x800390030801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE3_RX_WORK_STAT2_EO_PL               = 0x800390030801183Full;


static const uint64_t P9A_MC_RX0_SLICE3_RX_WORK_STAT3_EO_PL                   = 0x800398030701103Full;
//DUPS: 800398030701143F, 800398030701183F,
static const uint64_t P9A_MC_0_RX0_SLICE3_RX_WORK_STAT3_EO_PL                 = 0x800398030701103Full;
//DUPS: 800398030701143F, 800398030701183F,
static const uint64_t P9A_MC_1_RX0_SLICE3_RX_WORK_STAT3_EO_PL                 = 0x800398030801103Full;
//DUPS: 800398030801143F, 800398030801183F,

static const uint64_t P9A_OMI_RX0_SLICE3_RX_WORK_STAT3_EO_PL                  = 0x800398030701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE3_RX_WORK_STAT3_EO_PL                 = 0x800398030701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE3_RX_WORK_STAT3_EO_PL               = 0x800398030701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE3_RX_WORK_STAT3_EO_PL               = 0x800398030701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE3_RX_WORK_STAT3_EO_PL               = 0x800398030701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE3_RX_WORK_STAT3_EO_PL               = 0x800398030801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE3_RX_WORK_STAT3_EO_PL               = 0x800398030801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE3_RX_WORK_STAT3_EO_PL               = 0x800398030801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE3_RX_WORK_STAT3_EO_PL                = 0x800398030701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE3_RX_WORK_STAT3_EO_PL                = 0x800398030701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE3_RX_WORK_STAT3_EO_PL                = 0x800398030701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE3_RX_WORK_STAT3_EO_PL                = 0x800398030801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE3_RX_WORK_STAT3_EO_PL               = 0x800398030801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE3_RX_WORK_STAT3_EO_PL               = 0x800398030801183Full;


static const uint64_t P9A_MC_RX0_SLICE3_RX_WORK_STAT4_O_PL                    = 0x8003A0030701103Full;
//DUPS: 8003A0030701143F, 8003A0030701183F,
static const uint64_t P9A_MC_0_RX0_SLICE3_RX_WORK_STAT4_O_PL                  = 0x8003A0030701103Full;
//DUPS: 8003A0030701143F, 8003A0030701183F,
static const uint64_t P9A_MC_1_RX0_SLICE3_RX_WORK_STAT4_O_PL                  = 0x8003A0030801103Full;
//DUPS: 8003A0030801143F, 8003A0030801183F,

static const uint64_t P9A_OMI_RX0_SLICE3_RX_WORK_STAT4_O_PL                   = 0x8003A0030701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE3_RX_WORK_STAT4_O_PL                  = 0x8003A0030701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE3_RX_WORK_STAT4_O_PL                = 0x8003A0030701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE3_RX_WORK_STAT4_O_PL                = 0x8003A0030701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE3_RX_WORK_STAT4_O_PL                = 0x8003A0030701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE3_RX_WORK_STAT4_O_PL                = 0x8003A0030801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE3_RX_WORK_STAT4_O_PL                = 0x8003A0030801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE3_RX_WORK_STAT4_O_PL                = 0x8003A0030801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE3_RX_WORK_STAT4_O_PL                 = 0x8003A0030701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE3_RX_WORK_STAT4_O_PL                 = 0x8003A0030701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE3_RX_WORK_STAT4_O_PL                 = 0x8003A0030701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE3_RX_WORK_STAT4_O_PL                 = 0x8003A0030801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE3_RX_WORK_STAT4_O_PL                = 0x8003A0030801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE3_RX_WORK_STAT4_O_PL                = 0x8003A0030801183Full;


static const uint64_t P9A_MC_RX0_SLICE4_RX_DATA_WORK_SPARE_MODE_PL            = 0x800380040701103Full;
//DUPS: 800380040701143F, 800380040701183F,
static const uint64_t P9A_MC_0_RX0_SLICE4_RX_DATA_WORK_SPARE_MODE_PL          = 0x800380040701103Full;
//DUPS: 800380040701143F, 800380040701183F,
static const uint64_t P9A_MC_1_RX0_SLICE4_RX_DATA_WORK_SPARE_MODE_PL          = 0x800380040801103Full;
//DUPS: 800380040801143F, 800380040801183F,

static const uint64_t P9A_OMI_RX0_SLICE4_RX_DATA_WORK_SPARE_MODE_PL           = 0x800380040701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE4_RX_DATA_WORK_SPARE_MODE_PL          = 0x800380040701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE4_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380040701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE4_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380040701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE4_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380040701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE4_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380040801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE4_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380040801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE4_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380040801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE4_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380040701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE4_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380040701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE4_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380040701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE4_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380040801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE4_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380040801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE4_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380040801183Full;


static const uint64_t P9A_MC_RX0_SLICE4_RX_GLBSM_PL_CNTL1X_O_PL               = 0x800330040701103Full;
//DUPS: 800330040701143F, 800330040701183F,
static const uint64_t P9A_MC_0_RX0_SLICE4_RX_GLBSM_PL_CNTL1X_O_PL             = 0x800330040701103Full;
//DUPS: 800330040701143F, 800330040701183F,
static const uint64_t P9A_MC_1_RX0_SLICE4_RX_GLBSM_PL_CNTL1X_O_PL             = 0x800330040801103Full;
//DUPS: 800330040801143F, 800330040801183F,

static const uint64_t P9A_OMI_RX0_SLICE4_RX_GLBSM_PL_CNTL1X_O_PL              = 0x800330040701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE4_RX_GLBSM_PL_CNTL1X_O_PL             = 0x800330040701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE4_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330040701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE4_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330040701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE4_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330040701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE4_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330040801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE4_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330040801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE4_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330040801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE4_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330040701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE4_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330040701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE4_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330040701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE4_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330040801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE4_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330040801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE4_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330040801183Full;


static const uint64_t P9A_MC_RX0_SLICE4_RX_GLBSM_PL_CNTL1_O_PL                = 0x800320040701103Full;
//DUPS: 800320040701143F, 800320040701183F,
static const uint64_t P9A_MC_0_RX0_SLICE4_RX_GLBSM_PL_CNTL1_O_PL              = 0x800320040701103Full;
//DUPS: 800320040701143F, 800320040701183F,
static const uint64_t P9A_MC_1_RX0_SLICE4_RX_GLBSM_PL_CNTL1_O_PL              = 0x800320040801103Full;
//DUPS: 800320040801143F, 800320040801183F,

static const uint64_t P9A_OMI_RX0_SLICE4_RX_GLBSM_PL_CNTL1_O_PL               = 0x800320040701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE4_RX_GLBSM_PL_CNTL1_O_PL              = 0x800320040701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE4_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320040701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE4_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320040701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE4_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320040701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE4_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320040801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE4_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320040801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE4_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320040801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE4_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320040701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE4_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320040701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE4_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320040701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE4_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320040801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE4_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320040801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE4_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320040801183Full;


static const uint64_t P9A_MC_RX0_SLICE4_RX_GLBSM_PL_STAT1_O_PL                = 0x800328040701103Full;
//DUPS: 800328040701143F, 800328040701183F,
static const uint64_t P9A_MC_0_RX0_SLICE4_RX_GLBSM_PL_STAT1_O_PL              = 0x800328040701103Full;
//DUPS: 800328040701143F, 800328040701183F,
static const uint64_t P9A_MC_1_RX0_SLICE4_RX_GLBSM_PL_STAT1_O_PL              = 0x800328040801103Full;
//DUPS: 800328040801143F, 800328040801183F,

static const uint64_t P9A_OMI_RX0_SLICE4_RX_GLBSM_PL_STAT1_O_PL               = 0x800328040701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE4_RX_GLBSM_PL_STAT1_O_PL              = 0x800328040701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE4_RX_GLBSM_PL_STAT1_O_PL            = 0x800328040701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE4_RX_GLBSM_PL_STAT1_O_PL            = 0x800328040701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE4_RX_GLBSM_PL_STAT1_O_PL            = 0x800328040701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE4_RX_GLBSM_PL_STAT1_O_PL            = 0x800328040801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE4_RX_GLBSM_PL_STAT1_O_PL            = 0x800328040801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE4_RX_GLBSM_PL_STAT1_O_PL            = 0x800328040801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE4_RX_GLBSM_PL_STAT1_O_PL             = 0x800328040701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE4_RX_GLBSM_PL_STAT1_O_PL             = 0x800328040701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE4_RX_GLBSM_PL_STAT1_O_PL             = 0x800328040701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE4_RX_GLBSM_PL_STAT1_O_PL             = 0x800328040801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE4_RX_GLBSM_PL_STAT1_O_PL            = 0x800328040801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE4_RX_GLBSM_PL_STAT1_O_PL            = 0x800328040801183Full;


static const uint64_t P9A_MC_RX0_SLICE4_RX_WORK_CNTL1_O_PL                    = 0x8003A8040701103Full;
//DUPS: 8003A8040701143F, 8003A8040701183F,
static const uint64_t P9A_MC_0_RX0_SLICE4_RX_WORK_CNTL1_O_PL                  = 0x8003A8040701103Full;
//DUPS: 8003A8040701143F, 8003A8040701183F,
static const uint64_t P9A_MC_1_RX0_SLICE4_RX_WORK_CNTL1_O_PL                  = 0x8003A8040801103Full;
//DUPS: 8003A8040801143F, 8003A8040801183F,

static const uint64_t P9A_OMI_RX0_SLICE4_RX_WORK_CNTL1_O_PL                   = 0x8003A8040701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE4_RX_WORK_CNTL1_O_PL                  = 0x8003A8040701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE4_RX_WORK_CNTL1_O_PL                = 0x8003A8040701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE4_RX_WORK_CNTL1_O_PL                = 0x8003A8040701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE4_RX_WORK_CNTL1_O_PL                = 0x8003A8040701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE4_RX_WORK_CNTL1_O_PL                = 0x8003A8040801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE4_RX_WORK_CNTL1_O_PL                = 0x8003A8040801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE4_RX_WORK_CNTL1_O_PL                = 0x8003A8040801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE4_RX_WORK_CNTL1_O_PL                 = 0x8003A8040701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE4_RX_WORK_CNTL1_O_PL                 = 0x8003A8040701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE4_RX_WORK_CNTL1_O_PL                 = 0x8003A8040701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE4_RX_WORK_CNTL1_O_PL                 = 0x8003A8040801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE4_RX_WORK_CNTL1_O_PL                = 0x8003A8040801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE4_RX_WORK_CNTL1_O_PL                = 0x8003A8040801183Full;


static const uint64_t P9A_MC_RX0_SLICE4_RX_WORK_STAT1_EO_PL                   = 0x800388040701103Full;
//DUPS: 800388040701143F, 800388040701183F,
static const uint64_t P9A_MC_0_RX0_SLICE4_RX_WORK_STAT1_EO_PL                 = 0x800388040701103Full;
//DUPS: 800388040701143F, 800388040701183F,
static const uint64_t P9A_MC_1_RX0_SLICE4_RX_WORK_STAT1_EO_PL                 = 0x800388040801103Full;
//DUPS: 800388040801143F, 800388040801183F,

static const uint64_t P9A_OMI_RX0_SLICE4_RX_WORK_STAT1_EO_PL                  = 0x800388040701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE4_RX_WORK_STAT1_EO_PL                 = 0x800388040701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE4_RX_WORK_STAT1_EO_PL               = 0x800388040701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE4_RX_WORK_STAT1_EO_PL               = 0x800388040701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE4_RX_WORK_STAT1_EO_PL               = 0x800388040701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE4_RX_WORK_STAT1_EO_PL               = 0x800388040801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE4_RX_WORK_STAT1_EO_PL               = 0x800388040801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE4_RX_WORK_STAT1_EO_PL               = 0x800388040801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE4_RX_WORK_STAT1_EO_PL                = 0x800388040701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE4_RX_WORK_STAT1_EO_PL                = 0x800388040701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE4_RX_WORK_STAT1_EO_PL                = 0x800388040701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE4_RX_WORK_STAT1_EO_PL                = 0x800388040801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE4_RX_WORK_STAT1_EO_PL               = 0x800388040801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE4_RX_WORK_STAT1_EO_PL               = 0x800388040801183Full;


static const uint64_t P9A_MC_RX0_SLICE4_RX_WORK_STAT2_EO_PL                   = 0x800390040701103Full;
//DUPS: 800390040701143F, 800390040701183F,
static const uint64_t P9A_MC_0_RX0_SLICE4_RX_WORK_STAT2_EO_PL                 = 0x800390040701103Full;
//DUPS: 800390040701143F, 800390040701183F,
static const uint64_t P9A_MC_1_RX0_SLICE4_RX_WORK_STAT2_EO_PL                 = 0x800390040801103Full;
//DUPS: 800390040801143F, 800390040801183F,

static const uint64_t P9A_OMI_RX0_SLICE4_RX_WORK_STAT2_EO_PL                  = 0x800390040701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE4_RX_WORK_STAT2_EO_PL                 = 0x800390040701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE4_RX_WORK_STAT2_EO_PL               = 0x800390040701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE4_RX_WORK_STAT2_EO_PL               = 0x800390040701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE4_RX_WORK_STAT2_EO_PL               = 0x800390040701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE4_RX_WORK_STAT2_EO_PL               = 0x800390040801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE4_RX_WORK_STAT2_EO_PL               = 0x800390040801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE4_RX_WORK_STAT2_EO_PL               = 0x800390040801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE4_RX_WORK_STAT2_EO_PL                = 0x800390040701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE4_RX_WORK_STAT2_EO_PL                = 0x800390040701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE4_RX_WORK_STAT2_EO_PL                = 0x800390040701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE4_RX_WORK_STAT2_EO_PL                = 0x800390040801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE4_RX_WORK_STAT2_EO_PL               = 0x800390040801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE4_RX_WORK_STAT2_EO_PL               = 0x800390040801183Full;


static const uint64_t P9A_MC_RX0_SLICE4_RX_WORK_STAT3_EO_PL                   = 0x800398040701103Full;
//DUPS: 800398040701143F, 800398040701183F,
static const uint64_t P9A_MC_0_RX0_SLICE4_RX_WORK_STAT3_EO_PL                 = 0x800398040701103Full;
//DUPS: 800398040701143F, 800398040701183F,
static const uint64_t P9A_MC_1_RX0_SLICE4_RX_WORK_STAT3_EO_PL                 = 0x800398040801103Full;
//DUPS: 800398040801143F, 800398040801183F,

static const uint64_t P9A_OMI_RX0_SLICE4_RX_WORK_STAT3_EO_PL                  = 0x800398040701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE4_RX_WORK_STAT3_EO_PL                 = 0x800398040701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE4_RX_WORK_STAT3_EO_PL               = 0x800398040701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE4_RX_WORK_STAT3_EO_PL               = 0x800398040701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE4_RX_WORK_STAT3_EO_PL               = 0x800398040701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE4_RX_WORK_STAT3_EO_PL               = 0x800398040801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE4_RX_WORK_STAT3_EO_PL               = 0x800398040801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE4_RX_WORK_STAT3_EO_PL               = 0x800398040801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE4_RX_WORK_STAT3_EO_PL                = 0x800398040701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE4_RX_WORK_STAT3_EO_PL                = 0x800398040701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE4_RX_WORK_STAT3_EO_PL                = 0x800398040701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE4_RX_WORK_STAT3_EO_PL                = 0x800398040801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE4_RX_WORK_STAT3_EO_PL               = 0x800398040801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE4_RX_WORK_STAT3_EO_PL               = 0x800398040801183Full;


static const uint64_t P9A_MC_RX0_SLICE4_RX_WORK_STAT4_O_PL                    = 0x8003A0040701103Full;
//DUPS: 8003A0040701143F, 8003A0040701183F,
static const uint64_t P9A_MC_0_RX0_SLICE4_RX_WORK_STAT4_O_PL                  = 0x8003A0040701103Full;
//DUPS: 8003A0040701143F, 8003A0040701183F,
static const uint64_t P9A_MC_1_RX0_SLICE4_RX_WORK_STAT4_O_PL                  = 0x8003A0040801103Full;
//DUPS: 8003A0040801143F, 8003A0040801183F,

static const uint64_t P9A_OMI_RX0_SLICE4_RX_WORK_STAT4_O_PL                   = 0x8003A0040701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE4_RX_WORK_STAT4_O_PL                  = 0x8003A0040701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE4_RX_WORK_STAT4_O_PL                = 0x8003A0040701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE4_RX_WORK_STAT4_O_PL                = 0x8003A0040701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE4_RX_WORK_STAT4_O_PL                = 0x8003A0040701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE4_RX_WORK_STAT4_O_PL                = 0x8003A0040801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE4_RX_WORK_STAT4_O_PL                = 0x8003A0040801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE4_RX_WORK_STAT4_O_PL                = 0x8003A0040801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE4_RX_WORK_STAT4_O_PL                 = 0x8003A0040701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE4_RX_WORK_STAT4_O_PL                 = 0x8003A0040701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE4_RX_WORK_STAT4_O_PL                 = 0x8003A0040701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE4_RX_WORK_STAT4_O_PL                 = 0x8003A0040801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE4_RX_WORK_STAT4_O_PL                = 0x8003A0040801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE4_RX_WORK_STAT4_O_PL                = 0x8003A0040801183Full;


static const uint64_t P9A_MC_RX0_SLICE5_RX_DATA_WORK_SPARE_MODE_PL            = 0x800380050701103Full;
//DUPS: 800380050701143F, 800380050701183F,
static const uint64_t P9A_MC_0_RX0_SLICE5_RX_DATA_WORK_SPARE_MODE_PL          = 0x800380050701103Full;
//DUPS: 800380050701143F, 800380050701183F,
static const uint64_t P9A_MC_1_RX0_SLICE5_RX_DATA_WORK_SPARE_MODE_PL          = 0x800380050801103Full;
//DUPS: 800380050801143F, 800380050801183F,

static const uint64_t P9A_OMI_RX0_SLICE5_RX_DATA_WORK_SPARE_MODE_PL           = 0x800380050701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE5_RX_DATA_WORK_SPARE_MODE_PL          = 0x800380050701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE5_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380050701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE5_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380050701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE5_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380050701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE5_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380050801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE5_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380050801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE5_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380050801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE5_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380050701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE5_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380050701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE5_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380050701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE5_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380050801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE5_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380050801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE5_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380050801183Full;


static const uint64_t P9A_MC_RX0_SLICE5_RX_GLBSM_PL_CNTL1X_O_PL               = 0x800330050701103Full;
//DUPS: 800330050701143F, 800330050701183F,
static const uint64_t P9A_MC_0_RX0_SLICE5_RX_GLBSM_PL_CNTL1X_O_PL             = 0x800330050701103Full;
//DUPS: 800330050701143F, 800330050701183F,
static const uint64_t P9A_MC_1_RX0_SLICE5_RX_GLBSM_PL_CNTL1X_O_PL             = 0x800330050801103Full;
//DUPS: 800330050801143F, 800330050801183F,

static const uint64_t P9A_OMI_RX0_SLICE5_RX_GLBSM_PL_CNTL1X_O_PL              = 0x800330050701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE5_RX_GLBSM_PL_CNTL1X_O_PL             = 0x800330050701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE5_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330050701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE5_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330050701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE5_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330050701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE5_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330050801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE5_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330050801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE5_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330050801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE5_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330050701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE5_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330050701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE5_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330050701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE5_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330050801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE5_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330050801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE5_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330050801183Full;


static const uint64_t P9A_MC_RX0_SLICE5_RX_GLBSM_PL_CNTL1_O_PL                = 0x800320050701103Full;
//DUPS: 800320050701143F, 800320050701183F,
static const uint64_t P9A_MC_0_RX0_SLICE5_RX_GLBSM_PL_CNTL1_O_PL              = 0x800320050701103Full;
//DUPS: 800320050701143F, 800320050701183F,
static const uint64_t P9A_MC_1_RX0_SLICE5_RX_GLBSM_PL_CNTL1_O_PL              = 0x800320050801103Full;
//DUPS: 800320050801143F, 800320050801183F,

static const uint64_t P9A_OMI_RX0_SLICE5_RX_GLBSM_PL_CNTL1_O_PL               = 0x800320050701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE5_RX_GLBSM_PL_CNTL1_O_PL              = 0x800320050701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE5_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320050701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE5_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320050701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE5_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320050701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE5_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320050801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE5_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320050801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE5_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320050801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE5_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320050701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE5_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320050701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE5_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320050701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE5_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320050801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE5_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320050801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE5_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320050801183Full;


static const uint64_t P9A_MC_RX0_SLICE5_RX_GLBSM_PL_STAT1_O_PL                = 0x800328050701103Full;
//DUPS: 800328050701143F, 800328050701183F,
static const uint64_t P9A_MC_0_RX0_SLICE5_RX_GLBSM_PL_STAT1_O_PL              = 0x800328050701103Full;
//DUPS: 800328050701143F, 800328050701183F,
static const uint64_t P9A_MC_1_RX0_SLICE5_RX_GLBSM_PL_STAT1_O_PL              = 0x800328050801103Full;
//DUPS: 800328050801143F, 800328050801183F,

static const uint64_t P9A_OMI_RX0_SLICE5_RX_GLBSM_PL_STAT1_O_PL               = 0x800328050701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE5_RX_GLBSM_PL_STAT1_O_PL              = 0x800328050701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE5_RX_GLBSM_PL_STAT1_O_PL            = 0x800328050701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE5_RX_GLBSM_PL_STAT1_O_PL            = 0x800328050701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE5_RX_GLBSM_PL_STAT1_O_PL            = 0x800328050701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE5_RX_GLBSM_PL_STAT1_O_PL            = 0x800328050801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE5_RX_GLBSM_PL_STAT1_O_PL            = 0x800328050801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE5_RX_GLBSM_PL_STAT1_O_PL            = 0x800328050801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE5_RX_GLBSM_PL_STAT1_O_PL             = 0x800328050701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE5_RX_GLBSM_PL_STAT1_O_PL             = 0x800328050701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE5_RX_GLBSM_PL_STAT1_O_PL             = 0x800328050701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE5_RX_GLBSM_PL_STAT1_O_PL             = 0x800328050801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE5_RX_GLBSM_PL_STAT1_O_PL            = 0x800328050801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE5_RX_GLBSM_PL_STAT1_O_PL            = 0x800328050801183Full;


static const uint64_t P9A_MC_RX0_SLICE5_RX_WORK_CNTL1_O_PL                    = 0x8003A8050701103Full;
//DUPS: 8003A8050701143F, 8003A8050701183F,
static const uint64_t P9A_MC_0_RX0_SLICE5_RX_WORK_CNTL1_O_PL                  = 0x8003A8050701103Full;
//DUPS: 8003A8050701143F, 8003A8050701183F,
static const uint64_t P9A_MC_1_RX0_SLICE5_RX_WORK_CNTL1_O_PL                  = 0x8003A8050801103Full;
//DUPS: 8003A8050801143F, 8003A8050801183F,

static const uint64_t P9A_OMI_RX0_SLICE5_RX_WORK_CNTL1_O_PL                   = 0x8003A8050701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE5_RX_WORK_CNTL1_O_PL                  = 0x8003A8050701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE5_RX_WORK_CNTL1_O_PL                = 0x8003A8050701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE5_RX_WORK_CNTL1_O_PL                = 0x8003A8050701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE5_RX_WORK_CNTL1_O_PL                = 0x8003A8050701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE5_RX_WORK_CNTL1_O_PL                = 0x8003A8050801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE5_RX_WORK_CNTL1_O_PL                = 0x8003A8050801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE5_RX_WORK_CNTL1_O_PL                = 0x8003A8050801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE5_RX_WORK_CNTL1_O_PL                 = 0x8003A8050701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE5_RX_WORK_CNTL1_O_PL                 = 0x8003A8050701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE5_RX_WORK_CNTL1_O_PL                 = 0x8003A8050701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE5_RX_WORK_CNTL1_O_PL                 = 0x8003A8050801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE5_RX_WORK_CNTL1_O_PL                = 0x8003A8050801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE5_RX_WORK_CNTL1_O_PL                = 0x8003A8050801183Full;


static const uint64_t P9A_MC_RX0_SLICE5_RX_WORK_STAT1_EO_PL                   = 0x800388050701103Full;
//DUPS: 800388050701143F, 800388050701183F,
static const uint64_t P9A_MC_0_RX0_SLICE5_RX_WORK_STAT1_EO_PL                 = 0x800388050701103Full;
//DUPS: 800388050701143F, 800388050701183F,
static const uint64_t P9A_MC_1_RX0_SLICE5_RX_WORK_STAT1_EO_PL                 = 0x800388050801103Full;
//DUPS: 800388050801143F, 800388050801183F,

static const uint64_t P9A_OMI_RX0_SLICE5_RX_WORK_STAT1_EO_PL                  = 0x800388050701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE5_RX_WORK_STAT1_EO_PL                 = 0x800388050701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE5_RX_WORK_STAT1_EO_PL               = 0x800388050701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE5_RX_WORK_STAT1_EO_PL               = 0x800388050701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE5_RX_WORK_STAT1_EO_PL               = 0x800388050701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE5_RX_WORK_STAT1_EO_PL               = 0x800388050801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE5_RX_WORK_STAT1_EO_PL               = 0x800388050801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE5_RX_WORK_STAT1_EO_PL               = 0x800388050801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE5_RX_WORK_STAT1_EO_PL                = 0x800388050701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE5_RX_WORK_STAT1_EO_PL                = 0x800388050701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE5_RX_WORK_STAT1_EO_PL                = 0x800388050701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE5_RX_WORK_STAT1_EO_PL                = 0x800388050801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE5_RX_WORK_STAT1_EO_PL               = 0x800388050801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE5_RX_WORK_STAT1_EO_PL               = 0x800388050801183Full;


static const uint64_t P9A_MC_RX0_SLICE5_RX_WORK_STAT2_EO_PL                   = 0x800390050701103Full;
//DUPS: 800390050701143F, 800390050701183F,
static const uint64_t P9A_MC_0_RX0_SLICE5_RX_WORK_STAT2_EO_PL                 = 0x800390050701103Full;
//DUPS: 800390050701143F, 800390050701183F,
static const uint64_t P9A_MC_1_RX0_SLICE5_RX_WORK_STAT2_EO_PL                 = 0x800390050801103Full;
//DUPS: 800390050801143F, 800390050801183F,

static const uint64_t P9A_OMI_RX0_SLICE5_RX_WORK_STAT2_EO_PL                  = 0x800390050701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE5_RX_WORK_STAT2_EO_PL                 = 0x800390050701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE5_RX_WORK_STAT2_EO_PL               = 0x800390050701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE5_RX_WORK_STAT2_EO_PL               = 0x800390050701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE5_RX_WORK_STAT2_EO_PL               = 0x800390050701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE5_RX_WORK_STAT2_EO_PL               = 0x800390050801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE5_RX_WORK_STAT2_EO_PL               = 0x800390050801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE5_RX_WORK_STAT2_EO_PL               = 0x800390050801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE5_RX_WORK_STAT2_EO_PL                = 0x800390050701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE5_RX_WORK_STAT2_EO_PL                = 0x800390050701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE5_RX_WORK_STAT2_EO_PL                = 0x800390050701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE5_RX_WORK_STAT2_EO_PL                = 0x800390050801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE5_RX_WORK_STAT2_EO_PL               = 0x800390050801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE5_RX_WORK_STAT2_EO_PL               = 0x800390050801183Full;


static const uint64_t P9A_MC_RX0_SLICE5_RX_WORK_STAT3_EO_PL                   = 0x800398050701103Full;
//DUPS: 800398050701143F, 800398050701183F,
static const uint64_t P9A_MC_0_RX0_SLICE5_RX_WORK_STAT3_EO_PL                 = 0x800398050701103Full;
//DUPS: 800398050701143F, 800398050701183F,
static const uint64_t P9A_MC_1_RX0_SLICE5_RX_WORK_STAT3_EO_PL                 = 0x800398050801103Full;
//DUPS: 800398050801143F, 800398050801183F,

static const uint64_t P9A_OMI_RX0_SLICE5_RX_WORK_STAT3_EO_PL                  = 0x800398050701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE5_RX_WORK_STAT3_EO_PL                 = 0x800398050701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE5_RX_WORK_STAT3_EO_PL               = 0x800398050701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE5_RX_WORK_STAT3_EO_PL               = 0x800398050701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE5_RX_WORK_STAT3_EO_PL               = 0x800398050701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE5_RX_WORK_STAT3_EO_PL               = 0x800398050801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE5_RX_WORK_STAT3_EO_PL               = 0x800398050801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE5_RX_WORK_STAT3_EO_PL               = 0x800398050801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE5_RX_WORK_STAT3_EO_PL                = 0x800398050701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE5_RX_WORK_STAT3_EO_PL                = 0x800398050701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE5_RX_WORK_STAT3_EO_PL                = 0x800398050701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE5_RX_WORK_STAT3_EO_PL                = 0x800398050801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE5_RX_WORK_STAT3_EO_PL               = 0x800398050801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE5_RX_WORK_STAT3_EO_PL               = 0x800398050801183Full;


static const uint64_t P9A_MC_RX0_SLICE5_RX_WORK_STAT4_O_PL                    = 0x8003A0050701103Full;
//DUPS: 8003A0050701143F, 8003A0050701183F,
static const uint64_t P9A_MC_0_RX0_SLICE5_RX_WORK_STAT4_O_PL                  = 0x8003A0050701103Full;
//DUPS: 8003A0050701143F, 8003A0050701183F,
static const uint64_t P9A_MC_1_RX0_SLICE5_RX_WORK_STAT4_O_PL                  = 0x8003A0050801103Full;
//DUPS: 8003A0050801143F, 8003A0050801183F,

static const uint64_t P9A_OMI_RX0_SLICE5_RX_WORK_STAT4_O_PL                   = 0x8003A0050701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE5_RX_WORK_STAT4_O_PL                  = 0x8003A0050701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE5_RX_WORK_STAT4_O_PL                = 0x8003A0050701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE5_RX_WORK_STAT4_O_PL                = 0x8003A0050701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE5_RX_WORK_STAT4_O_PL                = 0x8003A0050701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE5_RX_WORK_STAT4_O_PL                = 0x8003A0050801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE5_RX_WORK_STAT4_O_PL                = 0x8003A0050801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE5_RX_WORK_STAT4_O_PL                = 0x8003A0050801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE5_RX_WORK_STAT4_O_PL                 = 0x8003A0050701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE5_RX_WORK_STAT4_O_PL                 = 0x8003A0050701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE5_RX_WORK_STAT4_O_PL                 = 0x8003A0050701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE5_RX_WORK_STAT4_O_PL                 = 0x8003A0050801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE5_RX_WORK_STAT4_O_PL                = 0x8003A0050801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE5_RX_WORK_STAT4_O_PL                = 0x8003A0050801183Full;


static const uint64_t P9A_MC_RX0_SLICE6_RX_DATA_WORK_SPARE_MODE_PL            = 0x800380060701103Full;
//DUPS: 800380060701143F, 800380060701183F,
static const uint64_t P9A_MC_0_RX0_SLICE6_RX_DATA_WORK_SPARE_MODE_PL          = 0x800380060701103Full;
//DUPS: 800380060701143F, 800380060701183F,
static const uint64_t P9A_MC_1_RX0_SLICE6_RX_DATA_WORK_SPARE_MODE_PL          = 0x800380060801103Full;
//DUPS: 800380060801143F, 800380060801183F,

static const uint64_t P9A_OMI_RX0_SLICE6_RX_DATA_WORK_SPARE_MODE_PL           = 0x800380060701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE6_RX_DATA_WORK_SPARE_MODE_PL          = 0x800380060701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE6_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380060701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE6_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380060701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE6_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380060701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE6_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380060801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE6_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380060801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE6_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380060801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE6_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380060701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE6_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380060701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE6_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380060701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE6_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380060801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE6_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380060801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE6_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380060801183Full;


static const uint64_t P9A_MC_RX0_SLICE6_RX_GLBSM_PL_CNTL1X_O_PL               = 0x800330060701103Full;
//DUPS: 800330060701143F, 800330060701183F,
static const uint64_t P9A_MC_0_RX0_SLICE6_RX_GLBSM_PL_CNTL1X_O_PL             = 0x800330060701103Full;
//DUPS: 800330060701143F, 800330060701183F,
static const uint64_t P9A_MC_1_RX0_SLICE6_RX_GLBSM_PL_CNTL1X_O_PL             = 0x800330060801103Full;
//DUPS: 800330060801143F, 800330060801183F,

static const uint64_t P9A_OMI_RX0_SLICE6_RX_GLBSM_PL_CNTL1X_O_PL              = 0x800330060701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE6_RX_GLBSM_PL_CNTL1X_O_PL             = 0x800330060701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE6_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330060701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE6_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330060701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE6_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330060701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE6_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330060801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE6_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330060801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE6_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330060801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE6_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330060701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE6_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330060701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE6_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330060701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE6_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330060801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE6_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330060801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE6_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330060801183Full;


static const uint64_t P9A_MC_RX0_SLICE6_RX_GLBSM_PL_CNTL1_O_PL                = 0x800320060701103Full;
//DUPS: 800320060701143F, 800320060701183F,
static const uint64_t P9A_MC_0_RX0_SLICE6_RX_GLBSM_PL_CNTL1_O_PL              = 0x800320060701103Full;
//DUPS: 800320060701143F, 800320060701183F,
static const uint64_t P9A_MC_1_RX0_SLICE6_RX_GLBSM_PL_CNTL1_O_PL              = 0x800320060801103Full;
//DUPS: 800320060801143F, 800320060801183F,

static const uint64_t P9A_OMI_RX0_SLICE6_RX_GLBSM_PL_CNTL1_O_PL               = 0x800320060701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE6_RX_GLBSM_PL_CNTL1_O_PL              = 0x800320060701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE6_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320060701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE6_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320060701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE6_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320060701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE6_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320060801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE6_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320060801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE6_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320060801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE6_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320060701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE6_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320060701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE6_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320060701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE6_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320060801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE6_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320060801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE6_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320060801183Full;


static const uint64_t P9A_MC_RX0_SLICE6_RX_GLBSM_PL_STAT1_O_PL                = 0x800328060701103Full;
//DUPS: 800328060701143F, 800328060701183F,
static const uint64_t P9A_MC_0_RX0_SLICE6_RX_GLBSM_PL_STAT1_O_PL              = 0x800328060701103Full;
//DUPS: 800328060701143F, 800328060701183F,
static const uint64_t P9A_MC_1_RX0_SLICE6_RX_GLBSM_PL_STAT1_O_PL              = 0x800328060801103Full;
//DUPS: 800328060801143F, 800328060801183F,

static const uint64_t P9A_OMI_RX0_SLICE6_RX_GLBSM_PL_STAT1_O_PL               = 0x800328060701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE6_RX_GLBSM_PL_STAT1_O_PL              = 0x800328060701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE6_RX_GLBSM_PL_STAT1_O_PL            = 0x800328060701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE6_RX_GLBSM_PL_STAT1_O_PL            = 0x800328060701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE6_RX_GLBSM_PL_STAT1_O_PL            = 0x800328060701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE6_RX_GLBSM_PL_STAT1_O_PL            = 0x800328060801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE6_RX_GLBSM_PL_STAT1_O_PL            = 0x800328060801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE6_RX_GLBSM_PL_STAT1_O_PL            = 0x800328060801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE6_RX_GLBSM_PL_STAT1_O_PL             = 0x800328060701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE6_RX_GLBSM_PL_STAT1_O_PL             = 0x800328060701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE6_RX_GLBSM_PL_STAT1_O_PL             = 0x800328060701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE6_RX_GLBSM_PL_STAT1_O_PL             = 0x800328060801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE6_RX_GLBSM_PL_STAT1_O_PL            = 0x800328060801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE6_RX_GLBSM_PL_STAT1_O_PL            = 0x800328060801183Full;


static const uint64_t P9A_MC_RX0_SLICE6_RX_WORK_CNTL1_O_PL                    = 0x8003A8060701103Full;
//DUPS: 8003A8060701143F, 8003A8060701183F,
static const uint64_t P9A_MC_0_RX0_SLICE6_RX_WORK_CNTL1_O_PL                  = 0x8003A8060701103Full;
//DUPS: 8003A8060701143F, 8003A8060701183F,
static const uint64_t P9A_MC_1_RX0_SLICE6_RX_WORK_CNTL1_O_PL                  = 0x8003A8060801103Full;
//DUPS: 8003A8060801143F, 8003A8060801183F,

static const uint64_t P9A_OMI_RX0_SLICE6_RX_WORK_CNTL1_O_PL                   = 0x8003A8060701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE6_RX_WORK_CNTL1_O_PL                  = 0x8003A8060701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE6_RX_WORK_CNTL1_O_PL                = 0x8003A8060701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE6_RX_WORK_CNTL1_O_PL                = 0x8003A8060701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE6_RX_WORK_CNTL1_O_PL                = 0x8003A8060701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE6_RX_WORK_CNTL1_O_PL                = 0x8003A8060801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE6_RX_WORK_CNTL1_O_PL                = 0x8003A8060801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE6_RX_WORK_CNTL1_O_PL                = 0x8003A8060801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE6_RX_WORK_CNTL1_O_PL                 = 0x8003A8060701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE6_RX_WORK_CNTL1_O_PL                 = 0x8003A8060701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE6_RX_WORK_CNTL1_O_PL                 = 0x8003A8060701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE6_RX_WORK_CNTL1_O_PL                 = 0x8003A8060801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE6_RX_WORK_CNTL1_O_PL                = 0x8003A8060801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE6_RX_WORK_CNTL1_O_PL                = 0x8003A8060801183Full;


static const uint64_t P9A_MC_RX0_SLICE6_RX_WORK_STAT1_EO_PL                   = 0x800388060701103Full;
//DUPS: 800388060701143F, 800388060701183F,
static const uint64_t P9A_MC_0_RX0_SLICE6_RX_WORK_STAT1_EO_PL                 = 0x800388060701103Full;
//DUPS: 800388060701143F, 800388060701183F,
static const uint64_t P9A_MC_1_RX0_SLICE6_RX_WORK_STAT1_EO_PL                 = 0x800388060801103Full;
//DUPS: 800388060801143F, 800388060801183F,

static const uint64_t P9A_OMI_RX0_SLICE6_RX_WORK_STAT1_EO_PL                  = 0x800388060701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE6_RX_WORK_STAT1_EO_PL                 = 0x800388060701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE6_RX_WORK_STAT1_EO_PL               = 0x800388060701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE6_RX_WORK_STAT1_EO_PL               = 0x800388060701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE6_RX_WORK_STAT1_EO_PL               = 0x800388060701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE6_RX_WORK_STAT1_EO_PL               = 0x800388060801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE6_RX_WORK_STAT1_EO_PL               = 0x800388060801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE6_RX_WORK_STAT1_EO_PL               = 0x800388060801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE6_RX_WORK_STAT1_EO_PL                = 0x800388060701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE6_RX_WORK_STAT1_EO_PL                = 0x800388060701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE6_RX_WORK_STAT1_EO_PL                = 0x800388060701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE6_RX_WORK_STAT1_EO_PL                = 0x800388060801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE6_RX_WORK_STAT1_EO_PL               = 0x800388060801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE6_RX_WORK_STAT1_EO_PL               = 0x800388060801183Full;


static const uint64_t P9A_MC_RX0_SLICE6_RX_WORK_STAT2_EO_PL                   = 0x800390060701103Full;
//DUPS: 800390060701143F, 800390060701183F,
static const uint64_t P9A_MC_0_RX0_SLICE6_RX_WORK_STAT2_EO_PL                 = 0x800390060701103Full;
//DUPS: 800390060701143F, 800390060701183F,
static const uint64_t P9A_MC_1_RX0_SLICE6_RX_WORK_STAT2_EO_PL                 = 0x800390060801103Full;
//DUPS: 800390060801143F, 800390060801183F,

static const uint64_t P9A_OMI_RX0_SLICE6_RX_WORK_STAT2_EO_PL                  = 0x800390060701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE6_RX_WORK_STAT2_EO_PL                 = 0x800390060701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE6_RX_WORK_STAT2_EO_PL               = 0x800390060701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE6_RX_WORK_STAT2_EO_PL               = 0x800390060701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE6_RX_WORK_STAT2_EO_PL               = 0x800390060701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE6_RX_WORK_STAT2_EO_PL               = 0x800390060801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE6_RX_WORK_STAT2_EO_PL               = 0x800390060801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE6_RX_WORK_STAT2_EO_PL               = 0x800390060801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE6_RX_WORK_STAT2_EO_PL                = 0x800390060701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE6_RX_WORK_STAT2_EO_PL                = 0x800390060701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE6_RX_WORK_STAT2_EO_PL                = 0x800390060701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE6_RX_WORK_STAT2_EO_PL                = 0x800390060801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE6_RX_WORK_STAT2_EO_PL               = 0x800390060801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE6_RX_WORK_STAT2_EO_PL               = 0x800390060801183Full;


static const uint64_t P9A_MC_RX0_SLICE6_RX_WORK_STAT3_EO_PL                   = 0x800398060701103Full;
//DUPS: 800398060701143F, 800398060701183F,
static const uint64_t P9A_MC_0_RX0_SLICE6_RX_WORK_STAT3_EO_PL                 = 0x800398060701103Full;
//DUPS: 800398060701143F, 800398060701183F,
static const uint64_t P9A_MC_1_RX0_SLICE6_RX_WORK_STAT3_EO_PL                 = 0x800398060801103Full;
//DUPS: 800398060801143F, 800398060801183F,

static const uint64_t P9A_OMI_RX0_SLICE6_RX_WORK_STAT3_EO_PL                  = 0x800398060701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE6_RX_WORK_STAT3_EO_PL                 = 0x800398060701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE6_RX_WORK_STAT3_EO_PL               = 0x800398060701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE6_RX_WORK_STAT3_EO_PL               = 0x800398060701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE6_RX_WORK_STAT3_EO_PL               = 0x800398060701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE6_RX_WORK_STAT3_EO_PL               = 0x800398060801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE6_RX_WORK_STAT3_EO_PL               = 0x800398060801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE6_RX_WORK_STAT3_EO_PL               = 0x800398060801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE6_RX_WORK_STAT3_EO_PL                = 0x800398060701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE6_RX_WORK_STAT3_EO_PL                = 0x800398060701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE6_RX_WORK_STAT3_EO_PL                = 0x800398060701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE6_RX_WORK_STAT3_EO_PL                = 0x800398060801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE6_RX_WORK_STAT3_EO_PL               = 0x800398060801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE6_RX_WORK_STAT3_EO_PL               = 0x800398060801183Full;


static const uint64_t P9A_MC_RX0_SLICE6_RX_WORK_STAT4_O_PL                    = 0x8003A0060701103Full;
//DUPS: 8003A0060701143F, 8003A0060701183F,
static const uint64_t P9A_MC_0_RX0_SLICE6_RX_WORK_STAT4_O_PL                  = 0x8003A0060701103Full;
//DUPS: 8003A0060701143F, 8003A0060701183F,
static const uint64_t P9A_MC_1_RX0_SLICE6_RX_WORK_STAT4_O_PL                  = 0x8003A0060801103Full;
//DUPS: 8003A0060801143F, 8003A0060801183F,

static const uint64_t P9A_OMI_RX0_SLICE6_RX_WORK_STAT4_O_PL                   = 0x8003A0060701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE6_RX_WORK_STAT4_O_PL                  = 0x8003A0060701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE6_RX_WORK_STAT4_O_PL                = 0x8003A0060701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE6_RX_WORK_STAT4_O_PL                = 0x8003A0060701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE6_RX_WORK_STAT4_O_PL                = 0x8003A0060701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE6_RX_WORK_STAT4_O_PL                = 0x8003A0060801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE6_RX_WORK_STAT4_O_PL                = 0x8003A0060801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE6_RX_WORK_STAT4_O_PL                = 0x8003A0060801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE6_RX_WORK_STAT4_O_PL                 = 0x8003A0060701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE6_RX_WORK_STAT4_O_PL                 = 0x8003A0060701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE6_RX_WORK_STAT4_O_PL                 = 0x8003A0060701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE6_RX_WORK_STAT4_O_PL                 = 0x8003A0060801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE6_RX_WORK_STAT4_O_PL                = 0x8003A0060801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE6_RX_WORK_STAT4_O_PL                = 0x8003A0060801183Full;


static const uint64_t P9A_MC_RX0_SLICE7_RX_DATA_WORK_SPARE_MODE_PL            = 0x800380070701103Full;
//DUPS: 800380070701143F, 800380070701183F,
static const uint64_t P9A_MC_0_RX0_SLICE7_RX_DATA_WORK_SPARE_MODE_PL          = 0x800380070701103Full;
//DUPS: 800380070701143F, 800380070701183F,
static const uint64_t P9A_MC_1_RX0_SLICE7_RX_DATA_WORK_SPARE_MODE_PL          = 0x800380070801103Full;
//DUPS: 800380070801143F, 800380070801183F,

static const uint64_t P9A_OMI_RX0_SLICE7_RX_DATA_WORK_SPARE_MODE_PL           = 0x800380070701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE7_RX_DATA_WORK_SPARE_MODE_PL          = 0x800380070701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE7_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380070701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE7_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380070701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE7_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380070701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE7_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380070801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE7_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380070801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE7_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380070801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE7_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380070701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE7_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380070701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE7_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380070701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE7_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380070801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE7_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380070801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE7_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380070801183Full;


static const uint64_t P9A_MC_RX0_SLICE7_RX_GLBSM_PL_CNTL1X_O_PL               = 0x800330070701103Full;
//DUPS: 800330070701143F, 800330070701183F,
static const uint64_t P9A_MC_0_RX0_SLICE7_RX_GLBSM_PL_CNTL1X_O_PL             = 0x800330070701103Full;
//DUPS: 800330070701143F, 800330070701183F,
static const uint64_t P9A_MC_1_RX0_SLICE7_RX_GLBSM_PL_CNTL1X_O_PL             = 0x800330070801103Full;
//DUPS: 800330070801143F, 800330070801183F,

static const uint64_t P9A_OMI_RX0_SLICE7_RX_GLBSM_PL_CNTL1X_O_PL              = 0x800330070701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE7_RX_GLBSM_PL_CNTL1X_O_PL             = 0x800330070701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE7_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330070701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE7_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330070701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE7_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330070701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE7_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330070801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE7_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330070801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE7_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330070801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE7_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330070701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE7_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330070701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE7_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330070701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE7_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330070801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE7_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330070801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE7_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330070801183Full;


static const uint64_t P9A_MC_RX0_SLICE7_RX_GLBSM_PL_CNTL1_O_PL                = 0x800320070701103Full;
//DUPS: 800320070701143F, 800320070701183F,
static const uint64_t P9A_MC_0_RX0_SLICE7_RX_GLBSM_PL_CNTL1_O_PL              = 0x800320070701103Full;
//DUPS: 800320070701143F, 800320070701183F,
static const uint64_t P9A_MC_1_RX0_SLICE7_RX_GLBSM_PL_CNTL1_O_PL              = 0x800320070801103Full;
//DUPS: 800320070801143F, 800320070801183F,

static const uint64_t P9A_OMI_RX0_SLICE7_RX_GLBSM_PL_CNTL1_O_PL               = 0x800320070701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE7_RX_GLBSM_PL_CNTL1_O_PL              = 0x800320070701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE7_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320070701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE7_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320070701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE7_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320070701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE7_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320070801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE7_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320070801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE7_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320070801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE7_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320070701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE7_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320070701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE7_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320070701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE7_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320070801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE7_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320070801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE7_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320070801183Full;


static const uint64_t P9A_MC_RX0_SLICE7_RX_GLBSM_PL_STAT1_O_PL                = 0x800328070701103Full;
//DUPS: 800328070701143F, 800328070701183F,
static const uint64_t P9A_MC_0_RX0_SLICE7_RX_GLBSM_PL_STAT1_O_PL              = 0x800328070701103Full;
//DUPS: 800328070701143F, 800328070701183F,
static const uint64_t P9A_MC_1_RX0_SLICE7_RX_GLBSM_PL_STAT1_O_PL              = 0x800328070801103Full;
//DUPS: 800328070801143F, 800328070801183F,

static const uint64_t P9A_OMI_RX0_SLICE7_RX_GLBSM_PL_STAT1_O_PL               = 0x800328070701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE7_RX_GLBSM_PL_STAT1_O_PL              = 0x800328070701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE7_RX_GLBSM_PL_STAT1_O_PL            = 0x800328070701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE7_RX_GLBSM_PL_STAT1_O_PL            = 0x800328070701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE7_RX_GLBSM_PL_STAT1_O_PL            = 0x800328070701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE7_RX_GLBSM_PL_STAT1_O_PL            = 0x800328070801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE7_RX_GLBSM_PL_STAT1_O_PL            = 0x800328070801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE7_RX_GLBSM_PL_STAT1_O_PL            = 0x800328070801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE7_RX_GLBSM_PL_STAT1_O_PL             = 0x800328070701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE7_RX_GLBSM_PL_STAT1_O_PL             = 0x800328070701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE7_RX_GLBSM_PL_STAT1_O_PL             = 0x800328070701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE7_RX_GLBSM_PL_STAT1_O_PL             = 0x800328070801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE7_RX_GLBSM_PL_STAT1_O_PL            = 0x800328070801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE7_RX_GLBSM_PL_STAT1_O_PL            = 0x800328070801183Full;


static const uint64_t P9A_MC_RX0_SLICE7_RX_WORK_CNTL1_O_PL                    = 0x8003A8070701103Full;
//DUPS: 8003A8070701143F, 8003A8070701183F,
static const uint64_t P9A_MC_0_RX0_SLICE7_RX_WORK_CNTL1_O_PL                  = 0x8003A8070701103Full;
//DUPS: 8003A8070701143F, 8003A8070701183F,
static const uint64_t P9A_MC_1_RX0_SLICE7_RX_WORK_CNTL1_O_PL                  = 0x8003A8070801103Full;
//DUPS: 8003A8070801143F, 8003A8070801183F,

static const uint64_t P9A_OMI_RX0_SLICE7_RX_WORK_CNTL1_O_PL                   = 0x8003A8070701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE7_RX_WORK_CNTL1_O_PL                  = 0x8003A8070701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE7_RX_WORK_CNTL1_O_PL                = 0x8003A8070701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE7_RX_WORK_CNTL1_O_PL                = 0x8003A8070701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE7_RX_WORK_CNTL1_O_PL                = 0x8003A8070701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE7_RX_WORK_CNTL1_O_PL                = 0x8003A8070801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE7_RX_WORK_CNTL1_O_PL                = 0x8003A8070801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE7_RX_WORK_CNTL1_O_PL                = 0x8003A8070801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE7_RX_WORK_CNTL1_O_PL                 = 0x8003A8070701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE7_RX_WORK_CNTL1_O_PL                 = 0x8003A8070701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE7_RX_WORK_CNTL1_O_PL                 = 0x8003A8070701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE7_RX_WORK_CNTL1_O_PL                 = 0x8003A8070801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE7_RX_WORK_CNTL1_O_PL                = 0x8003A8070801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE7_RX_WORK_CNTL1_O_PL                = 0x8003A8070801183Full;


static const uint64_t P9A_MC_RX0_SLICE7_RX_WORK_STAT1_EO_PL                   = 0x800388070701103Full;
//DUPS: 800388070701143F, 800388070701183F,
static const uint64_t P9A_MC_0_RX0_SLICE7_RX_WORK_STAT1_EO_PL                 = 0x800388070701103Full;
//DUPS: 800388070701143F, 800388070701183F,
static const uint64_t P9A_MC_1_RX0_SLICE7_RX_WORK_STAT1_EO_PL                 = 0x800388070801103Full;
//DUPS: 800388070801143F, 800388070801183F,

static const uint64_t P9A_OMI_RX0_SLICE7_RX_WORK_STAT1_EO_PL                  = 0x800388070701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE7_RX_WORK_STAT1_EO_PL                 = 0x800388070701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE7_RX_WORK_STAT1_EO_PL               = 0x800388070701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE7_RX_WORK_STAT1_EO_PL               = 0x800388070701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE7_RX_WORK_STAT1_EO_PL               = 0x800388070701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE7_RX_WORK_STAT1_EO_PL               = 0x800388070801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE7_RX_WORK_STAT1_EO_PL               = 0x800388070801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE7_RX_WORK_STAT1_EO_PL               = 0x800388070801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE7_RX_WORK_STAT1_EO_PL                = 0x800388070701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE7_RX_WORK_STAT1_EO_PL                = 0x800388070701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE7_RX_WORK_STAT1_EO_PL                = 0x800388070701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE7_RX_WORK_STAT1_EO_PL                = 0x800388070801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE7_RX_WORK_STAT1_EO_PL               = 0x800388070801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE7_RX_WORK_STAT1_EO_PL               = 0x800388070801183Full;


static const uint64_t P9A_MC_RX0_SLICE7_RX_WORK_STAT2_EO_PL                   = 0x800390070701103Full;
//DUPS: 800390070701143F, 800390070701183F,
static const uint64_t P9A_MC_0_RX0_SLICE7_RX_WORK_STAT2_EO_PL                 = 0x800390070701103Full;
//DUPS: 800390070701143F, 800390070701183F,
static const uint64_t P9A_MC_1_RX0_SLICE7_RX_WORK_STAT2_EO_PL                 = 0x800390070801103Full;
//DUPS: 800390070801143F, 800390070801183F,

static const uint64_t P9A_OMI_RX0_SLICE7_RX_WORK_STAT2_EO_PL                  = 0x800390070701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE7_RX_WORK_STAT2_EO_PL                 = 0x800390070701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE7_RX_WORK_STAT2_EO_PL               = 0x800390070701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE7_RX_WORK_STAT2_EO_PL               = 0x800390070701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE7_RX_WORK_STAT2_EO_PL               = 0x800390070701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE7_RX_WORK_STAT2_EO_PL               = 0x800390070801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE7_RX_WORK_STAT2_EO_PL               = 0x800390070801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE7_RX_WORK_STAT2_EO_PL               = 0x800390070801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE7_RX_WORK_STAT2_EO_PL                = 0x800390070701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE7_RX_WORK_STAT2_EO_PL                = 0x800390070701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE7_RX_WORK_STAT2_EO_PL                = 0x800390070701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE7_RX_WORK_STAT2_EO_PL                = 0x800390070801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE7_RX_WORK_STAT2_EO_PL               = 0x800390070801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE7_RX_WORK_STAT2_EO_PL               = 0x800390070801183Full;


static const uint64_t P9A_MC_RX0_SLICE7_RX_WORK_STAT3_EO_PL                   = 0x800398070701103Full;
//DUPS: 800398070701143F, 800398070701183F,
static const uint64_t P9A_MC_0_RX0_SLICE7_RX_WORK_STAT3_EO_PL                 = 0x800398070701103Full;
//DUPS: 800398070701143F, 800398070701183F,
static const uint64_t P9A_MC_1_RX0_SLICE7_RX_WORK_STAT3_EO_PL                 = 0x800398070801103Full;
//DUPS: 800398070801143F, 800398070801183F,

static const uint64_t P9A_OMI_RX0_SLICE7_RX_WORK_STAT3_EO_PL                  = 0x800398070701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE7_RX_WORK_STAT3_EO_PL                 = 0x800398070701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE7_RX_WORK_STAT3_EO_PL               = 0x800398070701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE7_RX_WORK_STAT3_EO_PL               = 0x800398070701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE7_RX_WORK_STAT3_EO_PL               = 0x800398070701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE7_RX_WORK_STAT3_EO_PL               = 0x800398070801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE7_RX_WORK_STAT3_EO_PL               = 0x800398070801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE7_RX_WORK_STAT3_EO_PL               = 0x800398070801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE7_RX_WORK_STAT3_EO_PL                = 0x800398070701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE7_RX_WORK_STAT3_EO_PL                = 0x800398070701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE7_RX_WORK_STAT3_EO_PL                = 0x800398070701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE7_RX_WORK_STAT3_EO_PL                = 0x800398070801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE7_RX_WORK_STAT3_EO_PL               = 0x800398070801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE7_RX_WORK_STAT3_EO_PL               = 0x800398070801183Full;


static const uint64_t P9A_MC_RX0_SLICE7_RX_WORK_STAT4_O_PL                    = 0x8003A0070701103Full;
//DUPS: 8003A0070701143F, 8003A0070701183F,
static const uint64_t P9A_MC_0_RX0_SLICE7_RX_WORK_STAT4_O_PL                  = 0x8003A0070701103Full;
//DUPS: 8003A0070701143F, 8003A0070701183F,
static const uint64_t P9A_MC_1_RX0_SLICE7_RX_WORK_STAT4_O_PL                  = 0x8003A0070801103Full;
//DUPS: 8003A0070801143F, 8003A0070801183F,

static const uint64_t P9A_OMI_RX0_SLICE7_RX_WORK_STAT4_O_PL                   = 0x8003A0070701103Full;

static const uint64_t P9A_OMIC_RX0_SLICE7_RX_WORK_STAT4_O_PL                  = 0x8003A0070701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE7_RX_WORK_STAT4_O_PL                = 0x8003A0070701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE7_RX_WORK_STAT4_O_PL                = 0x8003A0070701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE7_RX_WORK_STAT4_O_PL                = 0x8003A0070701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE7_RX_WORK_STAT4_O_PL                = 0x8003A0070801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE7_RX_WORK_STAT4_O_PL                = 0x8003A0070801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE7_RX_WORK_STAT4_O_PL                = 0x8003A0070801183Full;

static const uint64_t P9A_OMI_0_RX0_SLICE7_RX_WORK_STAT4_O_PL                 = 0x8003A0070701103Full;

static const uint64_t P9A_OMI_3_RX0_SLICE7_RX_WORK_STAT4_O_PL                 = 0x8003A0070701143Full;

static const uint64_t P9A_OMI_6_RX0_SLICE7_RX_WORK_STAT4_O_PL                 = 0x8003A0070701183Full;

static const uint64_t P9A_OMI_8_RX0_SLICE7_RX_WORK_STAT4_O_PL                 = 0x8003A0070801103Full;

static const uint64_t P9A_OMI_11_RX0_SLICE7_RX_WORK_STAT4_O_PL                = 0x8003A0070801143Full;

static const uint64_t P9A_OMI_14_RX0_SLICE7_RX_WORK_STAT4_O_PL                = 0x8003A0070801183Full;


static const uint64_t P9A_MC_RX0_SLICE8_RX_DATA_WORK_SPARE_MODE_PL            = 0x800380080701103Full;
//DUPS: 800380080701143F, 800380080701183F,
static const uint64_t P9A_MC_0_RX0_SLICE8_RX_DATA_WORK_SPARE_MODE_PL          = 0x800380080701103Full;
//DUPS: 800380080701143F, 800380080701183F,
static const uint64_t P9A_MC_1_RX0_SLICE8_RX_DATA_WORK_SPARE_MODE_PL          = 0x800380080801103Full;
//DUPS: 800380080801143F, 800380080801183F,

static const uint64_t P9A_OMIC_RX0_SLICE8_RX_DATA_WORK_SPARE_MODE_PL          = 0x800380080701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE8_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380080701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE8_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380080701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE8_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380080701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE8_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380080801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE8_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380080801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE8_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380080801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE8_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380080701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE8_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380080701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE8_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380080701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE8_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380080801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE8_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380080801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE8_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380080801183Full;


static const uint64_t P9A_MC_RX0_SLICE8_RX_GLBSM_PL_CNTL1X_O_PL               = 0x800330080701103Full;
//DUPS: 800330080701143F, 800330080701183F,
static const uint64_t P9A_MC_0_RX0_SLICE8_RX_GLBSM_PL_CNTL1X_O_PL             = 0x800330080701103Full;
//DUPS: 800330080701143F, 800330080701183F,
static const uint64_t P9A_MC_1_RX0_SLICE8_RX_GLBSM_PL_CNTL1X_O_PL             = 0x800330080801103Full;
//DUPS: 800330080801143F, 800330080801183F,

static const uint64_t P9A_OMIC_RX0_SLICE8_RX_GLBSM_PL_CNTL1X_O_PL             = 0x800330080701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE8_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330080701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE8_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330080701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE8_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330080701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE8_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330080801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE8_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330080801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE8_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330080801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE8_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330080701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE8_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330080701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE8_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330080701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE8_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330080801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE8_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330080801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE8_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330080801183Full;


static const uint64_t P9A_MC_RX0_SLICE8_RX_GLBSM_PL_CNTL1_O_PL                = 0x800320080701103Full;
//DUPS: 800320080701143F, 800320080701183F,
static const uint64_t P9A_MC_0_RX0_SLICE8_RX_GLBSM_PL_CNTL1_O_PL              = 0x800320080701103Full;
//DUPS: 800320080701143F, 800320080701183F,
static const uint64_t P9A_MC_1_RX0_SLICE8_RX_GLBSM_PL_CNTL1_O_PL              = 0x800320080801103Full;
//DUPS: 800320080801143F, 800320080801183F,

static const uint64_t P9A_OMIC_RX0_SLICE8_RX_GLBSM_PL_CNTL1_O_PL              = 0x800320080701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE8_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320080701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE8_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320080701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE8_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320080701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE8_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320080801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE8_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320080801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE8_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320080801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE8_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320080701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE8_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320080701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE8_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320080701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE8_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320080801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE8_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320080801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE8_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320080801183Full;


static const uint64_t P9A_MC_RX0_SLICE8_RX_GLBSM_PL_STAT1_O_PL                = 0x800328080701103Full;
//DUPS: 800328080701143F, 800328080701183F,
static const uint64_t P9A_MC_0_RX0_SLICE8_RX_GLBSM_PL_STAT1_O_PL              = 0x800328080701103Full;
//DUPS: 800328080701143F, 800328080701183F,
static const uint64_t P9A_MC_1_RX0_SLICE8_RX_GLBSM_PL_STAT1_O_PL              = 0x800328080801103Full;
//DUPS: 800328080801143F, 800328080801183F,

static const uint64_t P9A_OMIC_RX0_SLICE8_RX_GLBSM_PL_STAT1_O_PL              = 0x800328080701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE8_RX_GLBSM_PL_STAT1_O_PL            = 0x800328080701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE8_RX_GLBSM_PL_STAT1_O_PL            = 0x800328080701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE8_RX_GLBSM_PL_STAT1_O_PL            = 0x800328080701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE8_RX_GLBSM_PL_STAT1_O_PL            = 0x800328080801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE8_RX_GLBSM_PL_STAT1_O_PL            = 0x800328080801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE8_RX_GLBSM_PL_STAT1_O_PL            = 0x800328080801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE8_RX_GLBSM_PL_STAT1_O_PL             = 0x800328080701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE8_RX_GLBSM_PL_STAT1_O_PL             = 0x800328080701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE8_RX_GLBSM_PL_STAT1_O_PL             = 0x800328080701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE8_RX_GLBSM_PL_STAT1_O_PL             = 0x800328080801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE8_RX_GLBSM_PL_STAT1_O_PL            = 0x800328080801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE8_RX_GLBSM_PL_STAT1_O_PL            = 0x800328080801183Full;


static const uint64_t P9A_MC_RX0_SLICE8_RX_WORK_CNTL1_O_PL                    = 0x8003A8080701103Full;
//DUPS: 8003A8080701143F, 8003A8080701183F,
static const uint64_t P9A_MC_0_RX0_SLICE8_RX_WORK_CNTL1_O_PL                  = 0x8003A8080701103Full;
//DUPS: 8003A8080701143F, 8003A8080701183F,
static const uint64_t P9A_MC_1_RX0_SLICE8_RX_WORK_CNTL1_O_PL                  = 0x8003A8080801103Full;
//DUPS: 8003A8080801143F, 8003A8080801183F,

static const uint64_t P9A_OMIC_RX0_SLICE8_RX_WORK_CNTL1_O_PL                  = 0x8003A8080701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE8_RX_WORK_CNTL1_O_PL                = 0x8003A8080701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE8_RX_WORK_CNTL1_O_PL                = 0x8003A8080701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE8_RX_WORK_CNTL1_O_PL                = 0x8003A8080701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE8_RX_WORK_CNTL1_O_PL                = 0x8003A8080801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE8_RX_WORK_CNTL1_O_PL                = 0x8003A8080801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE8_RX_WORK_CNTL1_O_PL                = 0x8003A8080801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE8_RX_WORK_CNTL1_O_PL                 = 0x8003A8080701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE8_RX_WORK_CNTL1_O_PL                 = 0x8003A8080701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE8_RX_WORK_CNTL1_O_PL                 = 0x8003A8080701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE8_RX_WORK_CNTL1_O_PL                 = 0x8003A8080801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE8_RX_WORK_CNTL1_O_PL                = 0x8003A8080801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE8_RX_WORK_CNTL1_O_PL                = 0x8003A8080801183Full;


static const uint64_t P9A_MC_RX0_SLICE8_RX_WORK_STAT1_EO_PL                   = 0x800388080701103Full;
//DUPS: 800388080701143F, 800388080701183F,
static const uint64_t P9A_MC_0_RX0_SLICE8_RX_WORK_STAT1_EO_PL                 = 0x800388080701103Full;
//DUPS: 800388080701143F, 800388080701183F,
static const uint64_t P9A_MC_1_RX0_SLICE8_RX_WORK_STAT1_EO_PL                 = 0x800388080801103Full;
//DUPS: 800388080801143F, 800388080801183F,

static const uint64_t P9A_OMIC_RX0_SLICE8_RX_WORK_STAT1_EO_PL                 = 0x800388080701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE8_RX_WORK_STAT1_EO_PL               = 0x800388080701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE8_RX_WORK_STAT1_EO_PL               = 0x800388080701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE8_RX_WORK_STAT1_EO_PL               = 0x800388080701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE8_RX_WORK_STAT1_EO_PL               = 0x800388080801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE8_RX_WORK_STAT1_EO_PL               = 0x800388080801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE8_RX_WORK_STAT1_EO_PL               = 0x800388080801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE8_RX_WORK_STAT1_EO_PL                = 0x800388080701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE8_RX_WORK_STAT1_EO_PL                = 0x800388080701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE8_RX_WORK_STAT1_EO_PL                = 0x800388080701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE8_RX_WORK_STAT1_EO_PL                = 0x800388080801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE8_RX_WORK_STAT1_EO_PL               = 0x800388080801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE8_RX_WORK_STAT1_EO_PL               = 0x800388080801183Full;


static const uint64_t P9A_MC_RX0_SLICE8_RX_WORK_STAT2_EO_PL                   = 0x800390080701103Full;
//DUPS: 800390080701143F, 800390080701183F,
static const uint64_t P9A_MC_0_RX0_SLICE8_RX_WORK_STAT2_EO_PL                 = 0x800390080701103Full;
//DUPS: 800390080701143F, 800390080701183F,
static const uint64_t P9A_MC_1_RX0_SLICE8_RX_WORK_STAT2_EO_PL                 = 0x800390080801103Full;
//DUPS: 800390080801143F, 800390080801183F,

static const uint64_t P9A_OMIC_RX0_SLICE8_RX_WORK_STAT2_EO_PL                 = 0x800390080701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE8_RX_WORK_STAT2_EO_PL               = 0x800390080701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE8_RX_WORK_STAT2_EO_PL               = 0x800390080701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE8_RX_WORK_STAT2_EO_PL               = 0x800390080701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE8_RX_WORK_STAT2_EO_PL               = 0x800390080801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE8_RX_WORK_STAT2_EO_PL               = 0x800390080801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE8_RX_WORK_STAT2_EO_PL               = 0x800390080801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE8_RX_WORK_STAT2_EO_PL                = 0x800390080701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE8_RX_WORK_STAT2_EO_PL                = 0x800390080701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE8_RX_WORK_STAT2_EO_PL                = 0x800390080701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE8_RX_WORK_STAT2_EO_PL                = 0x800390080801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE8_RX_WORK_STAT2_EO_PL               = 0x800390080801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE8_RX_WORK_STAT2_EO_PL               = 0x800390080801183Full;


static const uint64_t P9A_MC_RX0_SLICE8_RX_WORK_STAT3_EO_PL                   = 0x800398080701103Full;
//DUPS: 800398080701143F, 800398080701183F,
static const uint64_t P9A_MC_0_RX0_SLICE8_RX_WORK_STAT3_EO_PL                 = 0x800398080701103Full;
//DUPS: 800398080701143F, 800398080701183F,
static const uint64_t P9A_MC_1_RX0_SLICE8_RX_WORK_STAT3_EO_PL                 = 0x800398080801103Full;
//DUPS: 800398080801143F, 800398080801183F,

static const uint64_t P9A_OMIC_RX0_SLICE8_RX_WORK_STAT3_EO_PL                 = 0x800398080701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE8_RX_WORK_STAT3_EO_PL               = 0x800398080701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE8_RX_WORK_STAT3_EO_PL               = 0x800398080701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE8_RX_WORK_STAT3_EO_PL               = 0x800398080701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE8_RX_WORK_STAT3_EO_PL               = 0x800398080801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE8_RX_WORK_STAT3_EO_PL               = 0x800398080801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE8_RX_WORK_STAT3_EO_PL               = 0x800398080801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE8_RX_WORK_STAT3_EO_PL                = 0x800398080701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE8_RX_WORK_STAT3_EO_PL                = 0x800398080701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE8_RX_WORK_STAT3_EO_PL                = 0x800398080701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE8_RX_WORK_STAT3_EO_PL                = 0x800398080801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE8_RX_WORK_STAT3_EO_PL               = 0x800398080801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE8_RX_WORK_STAT3_EO_PL               = 0x800398080801183Full;


static const uint64_t P9A_MC_RX0_SLICE8_RX_WORK_STAT4_O_PL                    = 0x8003A0080701103Full;
//DUPS: 8003A0080701143F, 8003A0080701183F,
static const uint64_t P9A_MC_0_RX0_SLICE8_RX_WORK_STAT4_O_PL                  = 0x8003A0080701103Full;
//DUPS: 8003A0080701143F, 8003A0080701183F,
static const uint64_t P9A_MC_1_RX0_SLICE8_RX_WORK_STAT4_O_PL                  = 0x8003A0080801103Full;
//DUPS: 8003A0080801143F, 8003A0080801183F,

static const uint64_t P9A_OMIC_RX0_SLICE8_RX_WORK_STAT4_O_PL                  = 0x8003A0080701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE8_RX_WORK_STAT4_O_PL                = 0x8003A0080701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE8_RX_WORK_STAT4_O_PL                = 0x8003A0080701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE8_RX_WORK_STAT4_O_PL                = 0x8003A0080701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE8_RX_WORK_STAT4_O_PL                = 0x8003A0080801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE8_RX_WORK_STAT4_O_PL                = 0x8003A0080801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE8_RX_WORK_STAT4_O_PL                = 0x8003A0080801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE8_RX_WORK_STAT4_O_PL                 = 0x8003A0080701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE8_RX_WORK_STAT4_O_PL                 = 0x8003A0080701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE8_RX_WORK_STAT4_O_PL                 = 0x8003A0080701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE8_RX_WORK_STAT4_O_PL                 = 0x8003A0080801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE8_RX_WORK_STAT4_O_PL                = 0x8003A0080801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE8_RX_WORK_STAT4_O_PL                = 0x8003A0080801183Full;


static const uint64_t P9A_MC_RX0_SLICE9_RX_DATA_WORK_SPARE_MODE_PL            = 0x800380090701103Full;
//DUPS: 800380090701143F, 800380090701183F,
static const uint64_t P9A_MC_0_RX0_SLICE9_RX_DATA_WORK_SPARE_MODE_PL          = 0x800380090701103Full;
//DUPS: 800380090701143F, 800380090701183F,
static const uint64_t P9A_MC_1_RX0_SLICE9_RX_DATA_WORK_SPARE_MODE_PL          = 0x800380090801103Full;
//DUPS: 800380090801143F, 800380090801183F,

static const uint64_t P9A_OMIC_RX0_SLICE9_RX_DATA_WORK_SPARE_MODE_PL          = 0x800380090701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE9_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380090701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE9_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380090701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE9_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380090701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE9_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380090801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE9_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380090801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE9_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380090801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE9_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380090701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE9_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380090701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE9_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380090701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE9_RX_DATA_WORK_SPARE_MODE_PL         = 0x800380090801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE9_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380090801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE9_RX_DATA_WORK_SPARE_MODE_PL        = 0x800380090801183Full;


static const uint64_t P9A_MC_RX0_SLICE9_RX_GLBSM_PL_CNTL1X_O_PL               = 0x800330090701103Full;
//DUPS: 800330090701143F, 800330090701183F,
static const uint64_t P9A_MC_0_RX0_SLICE9_RX_GLBSM_PL_CNTL1X_O_PL             = 0x800330090701103Full;
//DUPS: 800330090701143F, 800330090701183F,
static const uint64_t P9A_MC_1_RX0_SLICE9_RX_GLBSM_PL_CNTL1X_O_PL             = 0x800330090801103Full;
//DUPS: 800330090801143F, 800330090801183F,

static const uint64_t P9A_OMIC_RX0_SLICE9_RX_GLBSM_PL_CNTL1X_O_PL             = 0x800330090701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE9_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330090701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE9_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330090701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE9_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330090701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE9_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330090801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE9_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330090801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE9_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330090801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE9_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330090701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE9_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330090701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE9_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330090701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE9_RX_GLBSM_PL_CNTL1X_O_PL            = 0x800330090801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE9_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330090801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE9_RX_GLBSM_PL_CNTL1X_O_PL           = 0x800330090801183Full;


static const uint64_t P9A_MC_RX0_SLICE9_RX_GLBSM_PL_CNTL1_O_PL                = 0x800320090701103Full;
//DUPS: 800320090701143F, 800320090701183F,
static const uint64_t P9A_MC_0_RX0_SLICE9_RX_GLBSM_PL_CNTL1_O_PL              = 0x800320090701103Full;
//DUPS: 800320090701143F, 800320090701183F,
static const uint64_t P9A_MC_1_RX0_SLICE9_RX_GLBSM_PL_CNTL1_O_PL              = 0x800320090801103Full;
//DUPS: 800320090801143F, 800320090801183F,

static const uint64_t P9A_OMIC_RX0_SLICE9_RX_GLBSM_PL_CNTL1_O_PL              = 0x800320090701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE9_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320090701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE9_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320090701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE9_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320090701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE9_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320090801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE9_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320090801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE9_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320090801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE9_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320090701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE9_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320090701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE9_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320090701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE9_RX_GLBSM_PL_CNTL1_O_PL             = 0x800320090801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE9_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320090801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE9_RX_GLBSM_PL_CNTL1_O_PL            = 0x800320090801183Full;


static const uint64_t P9A_MC_RX0_SLICE9_RX_GLBSM_PL_STAT1_O_PL                = 0x800328090701103Full;
//DUPS: 800328090701143F, 800328090701183F,
static const uint64_t P9A_MC_0_RX0_SLICE9_RX_GLBSM_PL_STAT1_O_PL              = 0x800328090701103Full;
//DUPS: 800328090701143F, 800328090701183F,
static const uint64_t P9A_MC_1_RX0_SLICE9_RX_GLBSM_PL_STAT1_O_PL              = 0x800328090801103Full;
//DUPS: 800328090801143F, 800328090801183F,

static const uint64_t P9A_OMIC_RX0_SLICE9_RX_GLBSM_PL_STAT1_O_PL              = 0x800328090701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE9_RX_GLBSM_PL_STAT1_O_PL            = 0x800328090701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE9_RX_GLBSM_PL_STAT1_O_PL            = 0x800328090701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE9_RX_GLBSM_PL_STAT1_O_PL            = 0x800328090701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE9_RX_GLBSM_PL_STAT1_O_PL            = 0x800328090801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE9_RX_GLBSM_PL_STAT1_O_PL            = 0x800328090801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE9_RX_GLBSM_PL_STAT1_O_PL            = 0x800328090801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE9_RX_GLBSM_PL_STAT1_O_PL             = 0x800328090701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE9_RX_GLBSM_PL_STAT1_O_PL             = 0x800328090701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE9_RX_GLBSM_PL_STAT1_O_PL             = 0x800328090701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE9_RX_GLBSM_PL_STAT1_O_PL             = 0x800328090801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE9_RX_GLBSM_PL_STAT1_O_PL            = 0x800328090801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE9_RX_GLBSM_PL_STAT1_O_PL            = 0x800328090801183Full;


static const uint64_t P9A_MC_RX0_SLICE9_RX_WORK_CNTL1_O_PL                    = 0x8003A8090701103Full;
//DUPS: 8003A8090701143F, 8003A8090701183F,
static const uint64_t P9A_MC_0_RX0_SLICE9_RX_WORK_CNTL1_O_PL                  = 0x8003A8090701103Full;
//DUPS: 8003A8090701143F, 8003A8090701183F,
static const uint64_t P9A_MC_1_RX0_SLICE9_RX_WORK_CNTL1_O_PL                  = 0x8003A8090801103Full;
//DUPS: 8003A8090801143F, 8003A8090801183F,

static const uint64_t P9A_OMIC_RX0_SLICE9_RX_WORK_CNTL1_O_PL                  = 0x8003A8090701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE9_RX_WORK_CNTL1_O_PL                = 0x8003A8090701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE9_RX_WORK_CNTL1_O_PL                = 0x8003A8090701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE9_RX_WORK_CNTL1_O_PL                = 0x8003A8090701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE9_RX_WORK_CNTL1_O_PL                = 0x8003A8090801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE9_RX_WORK_CNTL1_O_PL                = 0x8003A8090801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE9_RX_WORK_CNTL1_O_PL                = 0x8003A8090801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE9_RX_WORK_CNTL1_O_PL                 = 0x8003A8090701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE9_RX_WORK_CNTL1_O_PL                 = 0x8003A8090701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE9_RX_WORK_CNTL1_O_PL                 = 0x8003A8090701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE9_RX_WORK_CNTL1_O_PL                 = 0x8003A8090801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE9_RX_WORK_CNTL1_O_PL                = 0x8003A8090801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE9_RX_WORK_CNTL1_O_PL                = 0x8003A8090801183Full;


static const uint64_t P9A_MC_RX0_SLICE9_RX_WORK_STAT1_EO_PL                   = 0x800388090701103Full;
//DUPS: 800388090701143F, 800388090701183F,
static const uint64_t P9A_MC_0_RX0_SLICE9_RX_WORK_STAT1_EO_PL                 = 0x800388090701103Full;
//DUPS: 800388090701143F, 800388090701183F,
static const uint64_t P9A_MC_1_RX0_SLICE9_RX_WORK_STAT1_EO_PL                 = 0x800388090801103Full;
//DUPS: 800388090801143F, 800388090801183F,

static const uint64_t P9A_OMIC_RX0_SLICE9_RX_WORK_STAT1_EO_PL                 = 0x800388090701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE9_RX_WORK_STAT1_EO_PL               = 0x800388090701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE9_RX_WORK_STAT1_EO_PL               = 0x800388090701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE9_RX_WORK_STAT1_EO_PL               = 0x800388090701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE9_RX_WORK_STAT1_EO_PL               = 0x800388090801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE9_RX_WORK_STAT1_EO_PL               = 0x800388090801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE9_RX_WORK_STAT1_EO_PL               = 0x800388090801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE9_RX_WORK_STAT1_EO_PL                = 0x800388090701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE9_RX_WORK_STAT1_EO_PL                = 0x800388090701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE9_RX_WORK_STAT1_EO_PL                = 0x800388090701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE9_RX_WORK_STAT1_EO_PL                = 0x800388090801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE9_RX_WORK_STAT1_EO_PL               = 0x800388090801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE9_RX_WORK_STAT1_EO_PL               = 0x800388090801183Full;


static const uint64_t P9A_MC_RX0_SLICE9_RX_WORK_STAT2_EO_PL                   = 0x800390090701103Full;
//DUPS: 800390090701143F, 800390090701183F,
static const uint64_t P9A_MC_0_RX0_SLICE9_RX_WORK_STAT2_EO_PL                 = 0x800390090701103Full;
//DUPS: 800390090701143F, 800390090701183F,
static const uint64_t P9A_MC_1_RX0_SLICE9_RX_WORK_STAT2_EO_PL                 = 0x800390090801103Full;
//DUPS: 800390090801143F, 800390090801183F,

static const uint64_t P9A_OMIC_RX0_SLICE9_RX_WORK_STAT2_EO_PL                 = 0x800390090701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE9_RX_WORK_STAT2_EO_PL               = 0x800390090701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE9_RX_WORK_STAT2_EO_PL               = 0x800390090701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE9_RX_WORK_STAT2_EO_PL               = 0x800390090701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE9_RX_WORK_STAT2_EO_PL               = 0x800390090801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE9_RX_WORK_STAT2_EO_PL               = 0x800390090801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE9_RX_WORK_STAT2_EO_PL               = 0x800390090801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE9_RX_WORK_STAT2_EO_PL                = 0x800390090701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE9_RX_WORK_STAT2_EO_PL                = 0x800390090701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE9_RX_WORK_STAT2_EO_PL                = 0x800390090701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE9_RX_WORK_STAT2_EO_PL                = 0x800390090801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE9_RX_WORK_STAT2_EO_PL               = 0x800390090801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE9_RX_WORK_STAT2_EO_PL               = 0x800390090801183Full;


static const uint64_t P9A_MC_RX0_SLICE9_RX_WORK_STAT3_EO_PL                   = 0x800398090701103Full;
//DUPS: 800398090701143F, 800398090701183F,
static const uint64_t P9A_MC_0_RX0_SLICE9_RX_WORK_STAT3_EO_PL                 = 0x800398090701103Full;
//DUPS: 800398090701143F, 800398090701183F,
static const uint64_t P9A_MC_1_RX0_SLICE9_RX_WORK_STAT3_EO_PL                 = 0x800398090801103Full;
//DUPS: 800398090801143F, 800398090801183F,

static const uint64_t P9A_OMIC_RX0_SLICE9_RX_WORK_STAT3_EO_PL                 = 0x800398090701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE9_RX_WORK_STAT3_EO_PL               = 0x800398090701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE9_RX_WORK_STAT3_EO_PL               = 0x800398090701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE9_RX_WORK_STAT3_EO_PL               = 0x800398090701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE9_RX_WORK_STAT3_EO_PL               = 0x800398090801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE9_RX_WORK_STAT3_EO_PL               = 0x800398090801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE9_RX_WORK_STAT3_EO_PL               = 0x800398090801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE9_RX_WORK_STAT3_EO_PL                = 0x800398090701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE9_RX_WORK_STAT3_EO_PL                = 0x800398090701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE9_RX_WORK_STAT3_EO_PL                = 0x800398090701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE9_RX_WORK_STAT3_EO_PL                = 0x800398090801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE9_RX_WORK_STAT3_EO_PL               = 0x800398090801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE9_RX_WORK_STAT3_EO_PL               = 0x800398090801183Full;


static const uint64_t P9A_MC_RX0_SLICE9_RX_WORK_STAT4_O_PL                    = 0x8003A0090701103Full;
//DUPS: 8003A0090701143F, 8003A0090701183F,
static const uint64_t P9A_MC_0_RX0_SLICE9_RX_WORK_STAT4_O_PL                  = 0x8003A0090701103Full;
//DUPS: 8003A0090701143F, 8003A0090701183F,
static const uint64_t P9A_MC_1_RX0_SLICE9_RX_WORK_STAT4_O_PL                  = 0x8003A0090801103Full;
//DUPS: 8003A0090801143F, 8003A0090801183F,

static const uint64_t P9A_OMIC_RX0_SLICE9_RX_WORK_STAT4_O_PL                  = 0x8003A0090701103Full;

static const uint64_t P9A_OMIC_0_RX0_SLICE9_RX_WORK_STAT4_O_PL                = 0x8003A0090701103Full;

static const uint64_t P9A_OMIC_1_RX0_SLICE9_RX_WORK_STAT4_O_PL                = 0x8003A0090701143Full;

static const uint64_t P9A_OMIC_2_RX0_SLICE9_RX_WORK_STAT4_O_PL                = 0x8003A0090701183Full;

static const uint64_t P9A_OMIC_3_RX0_SLICE9_RX_WORK_STAT4_O_PL                = 0x8003A0090801103Full;

static const uint64_t P9A_OMIC_4_RX0_SLICE9_RX_WORK_STAT4_O_PL                = 0x8003A0090801143Full;

static const uint64_t P9A_OMIC_5_RX0_SLICE9_RX_WORK_STAT4_O_PL                = 0x8003A0090801183Full;

static const uint64_t P9A_OMI_1_RX0_SLICE9_RX_WORK_STAT4_O_PL                 = 0x8003A0090701103Full;

static const uint64_t P9A_OMI_4_RX0_SLICE9_RX_WORK_STAT4_O_PL                 = 0x8003A0090701143Full;

static const uint64_t P9A_OMI_7_RX0_SLICE9_RX_WORK_STAT4_O_PL                 = 0x8003A0090701183Full;

static const uint64_t P9A_OMI_9_RX0_SLICE9_RX_WORK_STAT4_O_PL                 = 0x8003A0090801103Full;

static const uint64_t P9A_OMI_12_RX0_SLICE9_RX_WORK_STAT4_O_PL                = 0x8003A0090801143Full;

static const uint64_t P9A_OMI_15_RX0_SLICE9_RX_WORK_STAT4_O_PL                = 0x8003A0090801183Full;


static const uint64_t P9A_MC_RX_CTL_CNTL10_EO_PG                              = 0x800920000701103Full;
//DUPS: 800920000701143F, 800920000701183F,
static const uint64_t P9A_MC_0_RX_CTL_CNTL10_EO_PG                            = 0x800920000701103Full;
//DUPS: 800920000701143F, 800920000701183F,
static const uint64_t P9A_MC_1_RX_CTL_CNTL10_EO_PG                            = 0x800920000801103Full;
//DUPS: 800920000801143F, 800920000801183F,
static const uint64_t P9A_OMIC_RX_CTL_CNTL10_EO_PG                            = 0x800920000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_CNTL10_EO_PG                          = 0x800920000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_CNTL10_EO_PG                          = 0x800920000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_CNTL10_EO_PG                          = 0x800920000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_CNTL10_EO_PG                          = 0x800920000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_CNTL10_EO_PG                          = 0x800920000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_CNTL10_EO_PG                          = 0x800920000801183Full;


static const uint64_t P9A_MC_RX_CTL_CNTL11_EO_PG                              = 0x800928000701103Full;
//DUPS: 800928000701143F, 800928000701183F,
static const uint64_t P9A_MC_0_RX_CTL_CNTL11_EO_PG                            = 0x800928000701103Full;
//DUPS: 800928000701143F, 800928000701183F,
static const uint64_t P9A_MC_1_RX_CTL_CNTL11_EO_PG                            = 0x800928000801103Full;
//DUPS: 800928000801143F, 800928000801183F,
static const uint64_t P9A_OMIC_RX_CTL_CNTL11_EO_PG                            = 0x800928000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_CNTL11_EO_PG                          = 0x800928000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_CNTL11_EO_PG                          = 0x800928000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_CNTL11_EO_PG                          = 0x800928000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_CNTL11_EO_PG                          = 0x800928000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_CNTL11_EO_PG                          = 0x800928000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_CNTL11_EO_PG                          = 0x800928000801183Full;


static const uint64_t P9A_MC_RX_CTL_CNTL12_EO_PG                              = 0x800930000701103Full;
//DUPS: 800930000701143F, 800930000701183F,
static const uint64_t P9A_MC_0_RX_CTL_CNTL12_EO_PG                            = 0x800930000701103Full;
//DUPS: 800930000701143F, 800930000701183F,
static const uint64_t P9A_MC_1_RX_CTL_CNTL12_EO_PG                            = 0x800930000801103Full;
//DUPS: 800930000801143F, 800930000801183F,
static const uint64_t P9A_OMIC_RX_CTL_CNTL12_EO_PG                            = 0x800930000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_CNTL12_EO_PG                          = 0x800930000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_CNTL12_EO_PG                          = 0x800930000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_CNTL12_EO_PG                          = 0x800930000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_CNTL12_EO_PG                          = 0x800930000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_CNTL12_EO_PG                          = 0x800930000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_CNTL12_EO_PG                          = 0x800930000801183Full;


static const uint64_t P9A_MC_RX_CTL_CNTL13_EO_PG                              = 0x800938000701103Full;
//DUPS: 800938000701143F, 800938000701183F,
static const uint64_t P9A_MC_0_RX_CTL_CNTL13_EO_PG                            = 0x800938000701103Full;
//DUPS: 800938000701143F, 800938000701183F,
static const uint64_t P9A_MC_1_RX_CTL_CNTL13_EO_PG                            = 0x800938000801103Full;
//DUPS: 800938000801143F, 800938000801183F,
static const uint64_t P9A_OMIC_RX_CTL_CNTL13_EO_PG                            = 0x800938000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_CNTL13_EO_PG                          = 0x800938000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_CNTL13_EO_PG                          = 0x800938000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_CNTL13_EO_PG                          = 0x800938000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_CNTL13_EO_PG                          = 0x800938000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_CNTL13_EO_PG                          = 0x800938000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_CNTL13_EO_PG                          = 0x800938000801183Full;


static const uint64_t P9A_MC_RX_CTL_CNTL14_EO_PG                              = 0x800940000701103Full;
//DUPS: 800940000701143F, 800940000701183F,
static const uint64_t P9A_MC_0_RX_CTL_CNTL14_EO_PG                            = 0x800940000701103Full;
//DUPS: 800940000701143F, 800940000701183F,
static const uint64_t P9A_MC_1_RX_CTL_CNTL14_EO_PG                            = 0x800940000801103Full;
//DUPS: 800940000801143F, 800940000801183F,
static const uint64_t P9A_OMIC_RX_CTL_CNTL14_EO_PG                            = 0x800940000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_CNTL14_EO_PG                          = 0x800940000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_CNTL14_EO_PG                          = 0x800940000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_CNTL14_EO_PG                          = 0x800940000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_CNTL14_EO_PG                          = 0x800940000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_CNTL14_EO_PG                          = 0x800940000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_CNTL14_EO_PG                          = 0x800940000801183Full;


static const uint64_t P9A_MC_RX_CTL_CNTL1_EO_PG                               = 0x8008D8000701103Full;
//DUPS: 8008D8000701143F, 8008D8000701183F,
static const uint64_t P9A_MC_0_RX_CTL_CNTL1_EO_PG                             = 0x8008D8000701103Full;
//DUPS: 8008D8000701143F, 8008D8000701183F,
static const uint64_t P9A_MC_1_RX_CTL_CNTL1_EO_PG                             = 0x8008D8000801103Full;
//DUPS: 8008D8000801143F, 8008D8000801183F,
static const uint64_t P9A_OMIC_RX_CTL_CNTL1_EO_PG                             = 0x8008D8000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_CNTL1_EO_PG                           = 0x8008D8000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_CNTL1_EO_PG                           = 0x8008D8000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_CNTL1_EO_PG                           = 0x8008D8000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_CNTL1_EO_PG                           = 0x8008D8000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_CNTL1_EO_PG                           = 0x8008D8000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_CNTL1_EO_PG                           = 0x8008D8000801183Full;


static const uint64_t P9A_MC_RX_CTL_CNTL2_EO_PG                               = 0x8008E0000701103Full;
//DUPS: 8008E0000701143F, 8008E0000701183F,
static const uint64_t P9A_MC_0_RX_CTL_CNTL2_EO_PG                             = 0x8008E0000701103Full;
//DUPS: 8008E0000701143F, 8008E0000701183F,
static const uint64_t P9A_MC_1_RX_CTL_CNTL2_EO_PG                             = 0x8008E0000801103Full;
//DUPS: 8008E0000801143F, 8008E0000801183F,
static const uint64_t P9A_OMIC_RX_CTL_CNTL2_EO_PG                             = 0x8008E0000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_CNTL2_EO_PG                           = 0x8008E0000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_CNTL2_EO_PG                           = 0x8008E0000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_CNTL2_EO_PG                           = 0x8008E0000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_CNTL2_EO_PG                           = 0x8008E0000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_CNTL2_EO_PG                           = 0x8008E0000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_CNTL2_EO_PG                           = 0x8008E0000801183Full;


static const uint64_t P9A_MC_RX_CTL_CNTL3_EO_PG                               = 0x8008E8000701103Full;
//DUPS: 8008E8000701143F, 8008E8000701183F,
static const uint64_t P9A_MC_0_RX_CTL_CNTL3_EO_PG                             = 0x8008E8000701103Full;
//DUPS: 8008E8000701143F, 8008E8000701183F,
static const uint64_t P9A_MC_1_RX_CTL_CNTL3_EO_PG                             = 0x8008E8000801103Full;
//DUPS: 8008E8000801143F, 8008E8000801183F,
static const uint64_t P9A_OMIC_RX_CTL_CNTL3_EO_PG                             = 0x8008E8000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_CNTL3_EO_PG                           = 0x8008E8000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_CNTL3_EO_PG                           = 0x8008E8000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_CNTL3_EO_PG                           = 0x8008E8000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_CNTL3_EO_PG                           = 0x8008E8000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_CNTL3_EO_PG                           = 0x8008E8000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_CNTL3_EO_PG                           = 0x8008E8000801183Full;


static const uint64_t P9A_MC_RX_CTL_CNTL4_EO_PG                               = 0x8008F0000701103Full;
//DUPS: 8008F0000701143F, 8008F0000701183F,
static const uint64_t P9A_MC_0_RX_CTL_CNTL4_EO_PG                             = 0x8008F0000701103Full;
//DUPS: 8008F0000701143F, 8008F0000701183F,
static const uint64_t P9A_MC_1_RX_CTL_CNTL4_EO_PG                             = 0x8008F0000801103Full;
//DUPS: 8008F0000801143F, 8008F0000801183F,
static const uint64_t P9A_OMIC_RX_CTL_CNTL4_EO_PG                             = 0x8008F0000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_CNTL4_EO_PG                           = 0x8008F0000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_CNTL4_EO_PG                           = 0x8008F0000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_CNTL4_EO_PG                           = 0x8008F0000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_CNTL4_EO_PG                           = 0x8008F0000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_CNTL4_EO_PG                           = 0x8008F0000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_CNTL4_EO_PG                           = 0x8008F0000801183Full;


static const uint64_t P9A_MC_RX_CTL_CNTL5_EO_PG                               = 0x8008F8000701103Full;
//DUPS: 8008F8000701143F, 8008F8000701183F,
static const uint64_t P9A_MC_0_RX_CTL_CNTL5_EO_PG                             = 0x8008F8000701103Full;
//DUPS: 8008F8000701143F, 8008F8000701183F,
static const uint64_t P9A_MC_1_RX_CTL_CNTL5_EO_PG                             = 0x8008F8000801103Full;
//DUPS: 8008F8000801143F, 8008F8000801183F,
static const uint64_t P9A_OMIC_RX_CTL_CNTL5_EO_PG                             = 0x8008F8000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_CNTL5_EO_PG                           = 0x8008F8000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_CNTL5_EO_PG                           = 0x8008F8000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_CNTL5_EO_PG                           = 0x8008F8000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_CNTL5_EO_PG                           = 0x8008F8000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_CNTL5_EO_PG                           = 0x8008F8000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_CNTL5_EO_PG                           = 0x8008F8000801183Full;


static const uint64_t P9A_MC_RX_CTL_CNTL6_EO_PG                               = 0x800900000701103Full;
//DUPS: 800900000701143F, 800900000701183F,
static const uint64_t P9A_MC_0_RX_CTL_CNTL6_EO_PG                             = 0x800900000701103Full;
//DUPS: 800900000701143F, 800900000701183F,
static const uint64_t P9A_MC_1_RX_CTL_CNTL6_EO_PG                             = 0x800900000801103Full;
//DUPS: 800900000801143F, 800900000801183F,
static const uint64_t P9A_OMIC_RX_CTL_CNTL6_EO_PG                             = 0x800900000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_CNTL6_EO_PG                           = 0x800900000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_CNTL6_EO_PG                           = 0x800900000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_CNTL6_EO_PG                           = 0x800900000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_CNTL6_EO_PG                           = 0x800900000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_CNTL6_EO_PG                           = 0x800900000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_CNTL6_EO_PG                           = 0x800900000801183Full;


static const uint64_t P9A_MC_RX_CTL_CNTL8_EO_PG                               = 0x800910000701103Full;
//DUPS: 800910000701143F, 800910000701183F,
static const uint64_t P9A_MC_0_RX_CTL_CNTL8_EO_PG                             = 0x800910000701103Full;
//DUPS: 800910000701143F, 800910000701183F,
static const uint64_t P9A_MC_1_RX_CTL_CNTL8_EO_PG                             = 0x800910000801103Full;
//DUPS: 800910000801143F, 800910000801183F,
static const uint64_t P9A_OMIC_RX_CTL_CNTL8_EO_PG                             = 0x800910000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_CNTL8_EO_PG                           = 0x800910000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_CNTL8_EO_PG                           = 0x800910000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_CNTL8_EO_PG                           = 0x800910000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_CNTL8_EO_PG                           = 0x800910000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_CNTL8_EO_PG                           = 0x800910000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_CNTL8_EO_PG                           = 0x800910000801183Full;


static const uint64_t P9A_MC_RX_CTL_CNTL9_EO_PG                               = 0x800918000701103Full;
//DUPS: 800918000701143F, 800918000701183F,
static const uint64_t P9A_MC_0_RX_CTL_CNTL9_EO_PG                             = 0x800918000701103Full;
//DUPS: 800918000701143F, 800918000701183F,
static const uint64_t P9A_MC_1_RX_CTL_CNTL9_EO_PG                             = 0x800918000801103Full;
//DUPS: 800918000801143F, 800918000801183F,
static const uint64_t P9A_OMIC_RX_CTL_CNTL9_EO_PG                             = 0x800918000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_CNTL9_EO_PG                           = 0x800918000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_CNTL9_EO_PG                           = 0x800918000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_CNTL9_EO_PG                           = 0x800918000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_CNTL9_EO_PG                           = 0x800918000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_CNTL9_EO_PG                           = 0x800918000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_CNTL9_EO_PG                           = 0x800918000801183Full;


static const uint64_t P9A_MC_RX_CTL_CNTLX7_EO_PG                              = 0x800908000701103Full;
//DUPS: 800908000701143F, 800908000701183F,
static const uint64_t P9A_MC_0_RX_CTL_CNTLX7_EO_PG                            = 0x800908000701103Full;
//DUPS: 800908000701143F, 800908000701183F,
static const uint64_t P9A_MC_1_RX_CTL_CNTLX7_EO_PG                            = 0x800908000801103Full;
//DUPS: 800908000801143F, 800908000801183F,
static const uint64_t P9A_OMIC_RX_CTL_CNTLX7_EO_PG                            = 0x800908000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_CNTLX7_EO_PG                          = 0x800908000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_CNTLX7_EO_PG                          = 0x800908000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_CNTLX7_EO_PG                          = 0x800908000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_CNTLX7_EO_PG                          = 0x800908000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_CNTLX7_EO_PG                          = 0x800908000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_CNTLX7_EO_PG                          = 0x800908000801183Full;


static const uint64_t P9A_MC_RX_CTL_MODE10_EO_PG                              = 0x800858000701103Full;
//DUPS: 800858000701143F, 800858000701183F,
static const uint64_t P9A_MC_0_RX_CTL_MODE10_EO_PG                            = 0x800858000701103Full;
//DUPS: 800858000701143F, 800858000701183F,
static const uint64_t P9A_MC_1_RX_CTL_MODE10_EO_PG                            = 0x800858000801103Full;
//DUPS: 800858000801143F, 800858000801183F,
static const uint64_t P9A_OMIC_RX_CTL_MODE10_EO_PG                            = 0x800858000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_MODE10_EO_PG                          = 0x800858000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_MODE10_EO_PG                          = 0x800858000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_MODE10_EO_PG                          = 0x800858000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_MODE10_EO_PG                          = 0x800858000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_MODE10_EO_PG                          = 0x800858000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_MODE10_EO_PG                          = 0x800858000801183Full;


static const uint64_t P9A_MC_RX_CTL_MODE11_EO_PG                              = 0x800860000701103Full;
//DUPS: 800860000701143F, 800860000701183F,
static const uint64_t P9A_MC_0_RX_CTL_MODE11_EO_PG                            = 0x800860000701103Full;
//DUPS: 800860000701143F, 800860000701183F,
static const uint64_t P9A_MC_1_RX_CTL_MODE11_EO_PG                            = 0x800860000801103Full;
//DUPS: 800860000801143F, 800860000801183F,
static const uint64_t P9A_OMIC_RX_CTL_MODE11_EO_PG                            = 0x800860000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_MODE11_EO_PG                          = 0x800860000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_MODE11_EO_PG                          = 0x800860000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_MODE11_EO_PG                          = 0x800860000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_MODE11_EO_PG                          = 0x800860000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_MODE11_EO_PG                          = 0x800860000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_MODE11_EO_PG                          = 0x800860000801183Full;


static const uint64_t P9A_MC_RX_CTL_MODE12_EO_PG                              = 0x800868000701103Full;
//DUPS: 800868000701143F, 800868000701183F,
static const uint64_t P9A_MC_0_RX_CTL_MODE12_EO_PG                            = 0x800868000701103Full;
//DUPS: 800868000701143F, 800868000701183F,
static const uint64_t P9A_MC_1_RX_CTL_MODE12_EO_PG                            = 0x800868000801103Full;
//DUPS: 800868000801143F, 800868000801183F,
static const uint64_t P9A_OMIC_RX_CTL_MODE12_EO_PG                            = 0x800868000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_MODE12_EO_PG                          = 0x800868000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_MODE12_EO_PG                          = 0x800868000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_MODE12_EO_PG                          = 0x800868000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_MODE12_EO_PG                          = 0x800868000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_MODE12_EO_PG                          = 0x800868000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_MODE12_EO_PG                          = 0x800868000801183Full;


static const uint64_t P9A_MC_RX_CTL_MODE13_EO_PG                              = 0x800870000701103Full;
//DUPS: 800870000701143F, 800870000701183F,
static const uint64_t P9A_MC_0_RX_CTL_MODE13_EO_PG                            = 0x800870000701103Full;
//DUPS: 800870000701143F, 800870000701183F,
static const uint64_t P9A_MC_1_RX_CTL_MODE13_EO_PG                            = 0x800870000801103Full;
//DUPS: 800870000801143F, 800870000801183F,
static const uint64_t P9A_OMIC_RX_CTL_MODE13_EO_PG                            = 0x800870000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_MODE13_EO_PG                          = 0x800870000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_MODE13_EO_PG                          = 0x800870000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_MODE13_EO_PG                          = 0x800870000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_MODE13_EO_PG                          = 0x800870000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_MODE13_EO_PG                          = 0x800870000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_MODE13_EO_PG                          = 0x800870000801183Full;


static const uint64_t P9A_MC_RX_CTL_MODE14_EO_PG                              = 0x800878000701103Full;
//DUPS: 800878000701143F, 800878000701183F,
static const uint64_t P9A_MC_0_RX_CTL_MODE14_EO_PG                            = 0x800878000701103Full;
//DUPS: 800878000701143F, 800878000701183F,
static const uint64_t P9A_MC_1_RX_CTL_MODE14_EO_PG                            = 0x800878000801103Full;
//DUPS: 800878000801143F, 800878000801183F,
static const uint64_t P9A_OMIC_RX_CTL_MODE14_EO_PG                            = 0x800878000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_MODE14_EO_PG                          = 0x800878000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_MODE14_EO_PG                          = 0x800878000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_MODE14_EO_PG                          = 0x800878000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_MODE14_EO_PG                          = 0x800878000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_MODE14_EO_PG                          = 0x800878000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_MODE14_EO_PG                          = 0x800878000801183Full;


static const uint64_t P9A_MC_RX_CTL_MODE15_EO_PG                              = 0x800880000701103Full;
//DUPS: 800880000701143F, 800880000701183F,
static const uint64_t P9A_MC_0_RX_CTL_MODE15_EO_PG                            = 0x800880000701103Full;
//DUPS: 800880000701143F, 800880000701183F,
static const uint64_t P9A_MC_1_RX_CTL_MODE15_EO_PG                            = 0x800880000801103Full;
//DUPS: 800880000801143F, 800880000801183F,
static const uint64_t P9A_OMIC_RX_CTL_MODE15_EO_PG                            = 0x800880000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_MODE15_EO_PG                          = 0x800880000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_MODE15_EO_PG                          = 0x800880000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_MODE15_EO_PG                          = 0x800880000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_MODE15_EO_PG                          = 0x800880000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_MODE15_EO_PG                          = 0x800880000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_MODE15_EO_PG                          = 0x800880000801183Full;


static const uint64_t P9A_MC_RX_CTL_MODE16_EO_PG                              = 0x800888000701103Full;
//DUPS: 800888000701143F, 800888000701183F,
static const uint64_t P9A_MC_0_RX_CTL_MODE16_EO_PG                            = 0x800888000701103Full;
//DUPS: 800888000701143F, 800888000701183F,
static const uint64_t P9A_MC_1_RX_CTL_MODE16_EO_PG                            = 0x800888000801103Full;
//DUPS: 800888000801143F, 800888000801183F,
static const uint64_t P9A_OMIC_RX_CTL_MODE16_EO_PG                            = 0x800888000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_MODE16_EO_PG                          = 0x800888000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_MODE16_EO_PG                          = 0x800888000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_MODE16_EO_PG                          = 0x800888000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_MODE16_EO_PG                          = 0x800888000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_MODE16_EO_PG                          = 0x800888000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_MODE16_EO_PG                          = 0x800888000801183Full;


static const uint64_t P9A_MC_RX_CTL_MODE17_EO_PG                              = 0x800890000701103Full;
//DUPS: 800890000701143F, 800890000701183F,
static const uint64_t P9A_MC_0_RX_CTL_MODE17_EO_PG                            = 0x800890000701103Full;
//DUPS: 800890000701143F, 800890000701183F,
static const uint64_t P9A_MC_1_RX_CTL_MODE17_EO_PG                            = 0x800890000801103Full;
//DUPS: 800890000801143F, 800890000801183F,
static const uint64_t P9A_OMIC_RX_CTL_MODE17_EO_PG                            = 0x800890000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_MODE17_EO_PG                          = 0x800890000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_MODE17_EO_PG                          = 0x800890000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_MODE17_EO_PG                          = 0x800890000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_MODE17_EO_PG                          = 0x800890000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_MODE17_EO_PG                          = 0x800890000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_MODE17_EO_PG                          = 0x800890000801183Full;


static const uint64_t P9A_MC_RX_CTL_MODE18_EO_PG                              = 0x800898000701103Full;
//DUPS: 800898000701143F, 800898000701183F,
static const uint64_t P9A_MC_0_RX_CTL_MODE18_EO_PG                            = 0x800898000701103Full;
//DUPS: 800898000701143F, 800898000701183F,
static const uint64_t P9A_MC_1_RX_CTL_MODE18_EO_PG                            = 0x800898000801103Full;
//DUPS: 800898000801143F, 800898000801183F,
static const uint64_t P9A_OMIC_RX_CTL_MODE18_EO_PG                            = 0x800898000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_MODE18_EO_PG                          = 0x800898000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_MODE18_EO_PG                          = 0x800898000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_MODE18_EO_PG                          = 0x800898000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_MODE18_EO_PG                          = 0x800898000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_MODE18_EO_PG                          = 0x800898000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_MODE18_EO_PG                          = 0x800898000801183Full;


static const uint64_t P9A_MC_RX_CTL_MODE19_EO_PG                              = 0x8008A0000701103Full;
//DUPS: 8008A0000701143F, 8008A0000701183F,
static const uint64_t P9A_MC_0_RX_CTL_MODE19_EO_PG                            = 0x8008A0000701103Full;
//DUPS: 8008A0000701143F, 8008A0000701183F,
static const uint64_t P9A_MC_1_RX_CTL_MODE19_EO_PG                            = 0x8008A0000801103Full;
//DUPS: 8008A0000801143F, 8008A0000801183F,
static const uint64_t P9A_OMIC_RX_CTL_MODE19_EO_PG                            = 0x8008A0000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_MODE19_EO_PG                          = 0x8008A0000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_MODE19_EO_PG                          = 0x8008A0000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_MODE19_EO_PG                          = 0x8008A0000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_MODE19_EO_PG                          = 0x8008A0000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_MODE19_EO_PG                          = 0x8008A0000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_MODE19_EO_PG                          = 0x8008A0000801183Full;


static const uint64_t P9A_MC_RX_CTL_MODE1_EO_PG                               = 0x800810000701103Full;
//DUPS: 800810000701143F, 800810000701183F,
static const uint64_t P9A_MC_0_RX_CTL_MODE1_EO_PG                             = 0x800810000701103Full;
//DUPS: 800810000701143F, 800810000701183F,
static const uint64_t P9A_MC_1_RX_CTL_MODE1_EO_PG                             = 0x800810000801103Full;
//DUPS: 800810000801143F, 800810000801183F,
static const uint64_t P9A_OMIC_RX_CTL_MODE1_EO_PG                             = 0x800810000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_MODE1_EO_PG                           = 0x800810000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_MODE1_EO_PG                           = 0x800810000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_MODE1_EO_PG                           = 0x800810000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_MODE1_EO_PG                           = 0x800810000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_MODE1_EO_PG                           = 0x800810000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_MODE1_EO_PG                           = 0x800810000801183Full;


static const uint64_t P9A_MC_RX_CTL_MODE1_O_PG                                = 0x8009B0000701103Full;
//DUPS: 8009B0000701143F, 8009B0000701183F,
static const uint64_t P9A_MC_0_RX_CTL_MODE1_O_PG                              = 0x8009B0000701103Full;
//DUPS: 8009B0000701143F, 8009B0000701183F,
static const uint64_t P9A_MC_1_RX_CTL_MODE1_O_PG                              = 0x8009B0000801103Full;
//DUPS: 8009B0000801143F, 8009B0000801183F,
static const uint64_t P9A_OMIC_RX_CTL_MODE1_O_PG                              = 0x8009B0000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_MODE1_O_PG                            = 0x8009B0000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_MODE1_O_PG                            = 0x8009B0000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_MODE1_O_PG                            = 0x8009B0000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_MODE1_O_PG                            = 0x8009B0000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_MODE1_O_PG                            = 0x8009B0000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_MODE1_O_PG                            = 0x8009B0000801183Full;


static const uint64_t P9A_MC_RX_CTL_MODE20_EO_PG                              = 0x8008A8000701103Full;
//DUPS: 8008A8000701143F, 8008A8000701183F,
static const uint64_t P9A_MC_0_RX_CTL_MODE20_EO_PG                            = 0x8008A8000701103Full;
//DUPS: 8008A8000701143F, 8008A8000701183F,
static const uint64_t P9A_MC_1_RX_CTL_MODE20_EO_PG                            = 0x8008A8000801103Full;
//DUPS: 8008A8000801143F, 8008A8000801183F,
static const uint64_t P9A_OMIC_RX_CTL_MODE20_EO_PG                            = 0x8008A8000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_MODE20_EO_PG                          = 0x8008A8000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_MODE20_EO_PG                          = 0x8008A8000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_MODE20_EO_PG                          = 0x8008A8000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_MODE20_EO_PG                          = 0x8008A8000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_MODE20_EO_PG                          = 0x8008A8000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_MODE20_EO_PG                          = 0x8008A8000801183Full;


static const uint64_t P9A_MC_RX_CTL_MODE21_EO_PG                              = 0x8008B0000701103Full;
//DUPS: 8008B0000701143F, 8008B0000701183F,
static const uint64_t P9A_MC_0_RX_CTL_MODE21_EO_PG                            = 0x8008B0000701103Full;
//DUPS: 8008B0000701143F, 8008B0000701183F,
static const uint64_t P9A_MC_1_RX_CTL_MODE21_EO_PG                            = 0x8008B0000801103Full;
//DUPS: 8008B0000801143F, 8008B0000801183F,
static const uint64_t P9A_OMIC_RX_CTL_MODE21_EO_PG                            = 0x8008B0000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_MODE21_EO_PG                          = 0x8008B0000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_MODE21_EO_PG                          = 0x8008B0000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_MODE21_EO_PG                          = 0x8008B0000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_MODE21_EO_PG                          = 0x8008B0000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_MODE21_EO_PG                          = 0x8008B0000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_MODE21_EO_PG                          = 0x8008B0000801183Full;


static const uint64_t P9A_MC_RX_CTL_MODE22_EO_PG                              = 0x8008B8000701103Full;
//DUPS: 8008B8000701143F, 8008B8000701183F,
static const uint64_t P9A_MC_0_RX_CTL_MODE22_EO_PG                            = 0x8008B8000701103Full;
//DUPS: 8008B8000701143F, 8008B8000701183F,
static const uint64_t P9A_MC_1_RX_CTL_MODE22_EO_PG                            = 0x8008B8000801103Full;
//DUPS: 8008B8000801143F, 8008B8000801183F,
static const uint64_t P9A_OMIC_RX_CTL_MODE22_EO_PG                            = 0x8008B8000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_MODE22_EO_PG                          = 0x8008B8000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_MODE22_EO_PG                          = 0x8008B8000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_MODE22_EO_PG                          = 0x8008B8000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_MODE22_EO_PG                          = 0x8008B8000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_MODE22_EO_PG                          = 0x8008B8000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_MODE22_EO_PG                          = 0x8008B8000801183Full;


static const uint64_t P9A_MC_RX_CTL_MODE23_EO_PG                              = 0x8008C0000701103Full;
//DUPS: 8008C0000701143F, 8008C0000701183F,
static const uint64_t P9A_MC_0_RX_CTL_MODE23_EO_PG                            = 0x8008C0000701103Full;
//DUPS: 8008C0000701143F, 8008C0000701183F,
static const uint64_t P9A_MC_1_RX_CTL_MODE23_EO_PG                            = 0x8008C0000801103Full;
//DUPS: 8008C0000801143F, 8008C0000801183F,
static const uint64_t P9A_OMIC_RX_CTL_MODE23_EO_PG                            = 0x8008C0000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_MODE23_EO_PG                          = 0x8008C0000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_MODE23_EO_PG                          = 0x8008C0000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_MODE23_EO_PG                          = 0x8008C0000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_MODE23_EO_PG                          = 0x8008C0000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_MODE23_EO_PG                          = 0x8008C0000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_MODE23_EO_PG                          = 0x8008C0000801183Full;


static const uint64_t P9A_MC_RX_CTL_MODE24_EO_PG                              = 0x8008C8000701103Full;
//DUPS: 8008C8000701143F, 8008C8000701183F,
static const uint64_t P9A_MC_0_RX_CTL_MODE24_EO_PG                            = 0x8008C8000701103Full;
//DUPS: 8008C8000701143F, 8008C8000701183F,
static const uint64_t P9A_MC_1_RX_CTL_MODE24_EO_PG                            = 0x8008C8000801103Full;
//DUPS: 8008C8000801143F, 8008C8000801183F,
static const uint64_t P9A_OMIC_RX_CTL_MODE24_EO_PG                            = 0x8008C8000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_MODE24_EO_PG                          = 0x8008C8000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_MODE24_EO_PG                          = 0x8008C8000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_MODE24_EO_PG                          = 0x8008C8000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_MODE24_EO_PG                          = 0x8008C8000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_MODE24_EO_PG                          = 0x8008C8000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_MODE24_EO_PG                          = 0x8008C8000801183Full;


static const uint64_t P9A_MC_RX_CTL_MODE26_EO_PG                              = 0x800968000701103Full;
//DUPS: 800968000701143F, 800968000701183F,
static const uint64_t P9A_MC_0_RX_CTL_MODE26_EO_PG                            = 0x800968000701103Full;
//DUPS: 800968000701143F, 800968000701183F,
static const uint64_t P9A_MC_1_RX_CTL_MODE26_EO_PG                            = 0x800968000801103Full;
//DUPS: 800968000801143F, 800968000801183F,
static const uint64_t P9A_OMIC_RX_CTL_MODE26_EO_PG                            = 0x800968000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_MODE26_EO_PG                          = 0x800968000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_MODE26_EO_PG                          = 0x800968000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_MODE26_EO_PG                          = 0x800968000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_MODE26_EO_PG                          = 0x800968000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_MODE26_EO_PG                          = 0x800968000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_MODE26_EO_PG                          = 0x800968000801183Full;


static const uint64_t P9A_MC_RX_CTL_MODE27_EO_PG                              = 0x800970000701103Full;
//DUPS: 800970000701143F, 800970000701183F,
static const uint64_t P9A_MC_0_RX_CTL_MODE27_EO_PG                            = 0x800970000701103Full;
//DUPS: 800970000701143F, 800970000701183F,
static const uint64_t P9A_MC_1_RX_CTL_MODE27_EO_PG                            = 0x800970000801103Full;
//DUPS: 800970000801143F, 800970000801183F,
static const uint64_t P9A_OMIC_RX_CTL_MODE27_EO_PG                            = 0x800970000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_MODE27_EO_PG                          = 0x800970000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_MODE27_EO_PG                          = 0x800970000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_MODE27_EO_PG                          = 0x800970000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_MODE27_EO_PG                          = 0x800970000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_MODE27_EO_PG                          = 0x800970000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_MODE27_EO_PG                          = 0x800970000801183Full;


static const uint64_t P9A_MC_RX_CTL_MODE28_EO_PG                              = 0x800978000701103Full;
//DUPS: 800978000701143F, 800978000701183F,
static const uint64_t P9A_MC_0_RX_CTL_MODE28_EO_PG                            = 0x800978000701103Full;
//DUPS: 800978000701143F, 800978000701183F,
static const uint64_t P9A_MC_1_RX_CTL_MODE28_EO_PG                            = 0x800978000801103Full;
//DUPS: 800978000801143F, 800978000801183F,
static const uint64_t P9A_OMIC_RX_CTL_MODE28_EO_PG                            = 0x800978000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_MODE28_EO_PG                          = 0x800978000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_MODE28_EO_PG                          = 0x800978000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_MODE28_EO_PG                          = 0x800978000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_MODE28_EO_PG                          = 0x800978000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_MODE28_EO_PG                          = 0x800978000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_MODE28_EO_PG                          = 0x800978000801183Full;


static const uint64_t P9A_MC_RX_CTL_MODE29_EO_PG                              = 0x8008D0000701103Full;
//DUPS: 8008D0000701143F, 8008D0000701183F,
static const uint64_t P9A_MC_0_RX_CTL_MODE29_EO_PG                            = 0x8008D0000701103Full;
//DUPS: 8008D0000701143F, 8008D0000701183F,
static const uint64_t P9A_MC_1_RX_CTL_MODE29_EO_PG                            = 0x8008D0000801103Full;
//DUPS: 8008D0000801143F, 8008D0000801183F,
static const uint64_t P9A_OMIC_RX_CTL_MODE29_EO_PG                            = 0x8008D0000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_MODE29_EO_PG                          = 0x8008D0000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_MODE29_EO_PG                          = 0x8008D0000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_MODE29_EO_PG                          = 0x8008D0000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_MODE29_EO_PG                          = 0x8008D0000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_MODE29_EO_PG                          = 0x8008D0000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_MODE29_EO_PG                          = 0x8008D0000801183Full;


static const uint64_t P9A_MC_RX_CTL_MODE2_EO_PG                               = 0x800818000701103Full;
//DUPS: 800818000701143F, 800818000701183F,
static const uint64_t P9A_MC_0_RX_CTL_MODE2_EO_PG                             = 0x800818000701103Full;
//DUPS: 800818000701143F, 800818000701183F,
static const uint64_t P9A_MC_1_RX_CTL_MODE2_EO_PG                             = 0x800818000801103Full;
//DUPS: 800818000801143F, 800818000801183F,
static const uint64_t P9A_OMIC_RX_CTL_MODE2_EO_PG                             = 0x800818000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_MODE2_EO_PG                           = 0x800818000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_MODE2_EO_PG                           = 0x800818000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_MODE2_EO_PG                           = 0x800818000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_MODE2_EO_PG                           = 0x800818000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_MODE2_EO_PG                           = 0x800818000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_MODE2_EO_PG                           = 0x800818000801183Full;


static const uint64_t P9A_MC_RX_CTL_MODE2_O_PG                                = 0x800988000701103Full;
//DUPS: 800988000701143F, 800988000701183F,
static const uint64_t P9A_MC_0_RX_CTL_MODE2_O_PG                              = 0x800988000701103Full;
//DUPS: 800988000701143F, 800988000701183F,
static const uint64_t P9A_MC_1_RX_CTL_MODE2_O_PG                              = 0x800988000801103Full;
//DUPS: 800988000801143F, 800988000801183F,
static const uint64_t P9A_OMIC_RX_CTL_MODE2_O_PG                              = 0x800988000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_MODE2_O_PG                            = 0x800988000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_MODE2_O_PG                            = 0x800988000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_MODE2_O_PG                            = 0x800988000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_MODE2_O_PG                            = 0x800988000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_MODE2_O_PG                            = 0x800988000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_MODE2_O_PG                            = 0x800988000801183Full;


static const uint64_t P9A_MC_RX_CTL_MODE5_EO_PG                               = 0x800830000701103Full;
//DUPS: 800830000701143F, 800830000701183F,
static const uint64_t P9A_MC_0_RX_CTL_MODE5_EO_PG                             = 0x800830000701103Full;
//DUPS: 800830000701143F, 800830000701183F,
static const uint64_t P9A_MC_1_RX_CTL_MODE5_EO_PG                             = 0x800830000801103Full;
//DUPS: 800830000801143F, 800830000801183F,
static const uint64_t P9A_OMIC_RX_CTL_MODE5_EO_PG                             = 0x800830000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_MODE5_EO_PG                           = 0x800830000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_MODE5_EO_PG                           = 0x800830000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_MODE5_EO_PG                           = 0x800830000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_MODE5_EO_PG                           = 0x800830000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_MODE5_EO_PG                           = 0x800830000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_MODE5_EO_PG                           = 0x800830000801183Full;


static const uint64_t P9A_MC_RX_CTL_MODE6_EO_PG                               = 0x800838000701103Full;
//DUPS: 800838000701143F, 800838000701183F,
static const uint64_t P9A_MC_0_RX_CTL_MODE6_EO_PG                             = 0x800838000701103Full;
//DUPS: 800838000701143F, 800838000701183F,
static const uint64_t P9A_MC_1_RX_CTL_MODE6_EO_PG                             = 0x800838000801103Full;
//DUPS: 800838000801143F, 800838000801183F,
static const uint64_t P9A_OMIC_RX_CTL_MODE6_EO_PG                             = 0x800838000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_MODE6_EO_PG                           = 0x800838000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_MODE6_EO_PG                           = 0x800838000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_MODE6_EO_PG                           = 0x800838000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_MODE6_EO_PG                           = 0x800838000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_MODE6_EO_PG                           = 0x800838000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_MODE6_EO_PG                           = 0x800838000801183Full;


static const uint64_t P9A_MC_RX_CTL_MODE7_EO_PG                               = 0x800840000701103Full;
//DUPS: 800840000701143F, 800840000701183F,
static const uint64_t P9A_MC_0_RX_CTL_MODE7_EO_PG                             = 0x800840000701103Full;
//DUPS: 800840000701143F, 800840000701183F,
static const uint64_t P9A_MC_1_RX_CTL_MODE7_EO_PG                             = 0x800840000801103Full;
//DUPS: 800840000801143F, 800840000801183F,
static const uint64_t P9A_OMIC_RX_CTL_MODE7_EO_PG                             = 0x800840000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_MODE7_EO_PG                           = 0x800840000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_MODE7_EO_PG                           = 0x800840000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_MODE7_EO_PG                           = 0x800840000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_MODE7_EO_PG                           = 0x800840000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_MODE7_EO_PG                           = 0x800840000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_MODE7_EO_PG                           = 0x800840000801183Full;


static const uint64_t P9A_MC_RX_CTL_MODE9_EO_PG                               = 0x800850000701103Full;
//DUPS: 800850000701143F, 800850000701183F,
static const uint64_t P9A_MC_0_RX_CTL_MODE9_EO_PG                             = 0x800850000701103Full;
//DUPS: 800850000701143F, 800850000701183F,
static const uint64_t P9A_MC_1_RX_CTL_MODE9_EO_PG                             = 0x800850000801103Full;
//DUPS: 800850000801143F, 800850000801183F,
static const uint64_t P9A_OMIC_RX_CTL_MODE9_EO_PG                             = 0x800850000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_MODE9_EO_PG                           = 0x800850000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_MODE9_EO_PG                           = 0x800850000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_MODE9_EO_PG                           = 0x800850000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_MODE9_EO_PG                           = 0x800850000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_MODE9_EO_PG                           = 0x800850000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_MODE9_EO_PG                           = 0x800850000801183Full;


static const uint64_t P9A_MC_RX_CTL_STAT1_EO_PG                               = 0x800950000701103Full;
//DUPS: 800950000701143F, 800950000701183F,
static const uint64_t P9A_MC_0_RX_CTL_STAT1_EO_PG                             = 0x800950000701103Full;
//DUPS: 800950000701143F, 800950000701183F,
static const uint64_t P9A_MC_1_RX_CTL_STAT1_EO_PG                             = 0x800950000801103Full;
//DUPS: 800950000801143F, 800950000801183F,
static const uint64_t P9A_OMIC_RX_CTL_STAT1_EO_PG                             = 0x800950000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_STAT1_EO_PG                           = 0x800950000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_STAT1_EO_PG                           = 0x800950000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_STAT1_EO_PG                           = 0x800950000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_STAT1_EO_PG                           = 0x800950000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_STAT1_EO_PG                           = 0x800950000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_STAT1_EO_PG                           = 0x800950000801183Full;


static const uint64_t P9A_MC_RX_CTL_STAT2_EO_PG                               = 0x800958000701103Full;
//DUPS: 800958000701143F, 800958000701183F,
static const uint64_t P9A_MC_0_RX_CTL_STAT2_EO_PG                             = 0x800958000701103Full;
//DUPS: 800958000701143F, 800958000701183F,
static const uint64_t P9A_MC_1_RX_CTL_STAT2_EO_PG                             = 0x800958000801103Full;
//DUPS: 800958000801143F, 800958000801183F,
static const uint64_t P9A_OMIC_RX_CTL_STAT2_EO_PG                             = 0x800958000701103Full;

static const uint64_t P9A_OMIC_0_RX_CTL_STAT2_EO_PG                           = 0x800958000701103Full;

static const uint64_t P9A_OMIC_1_RX_CTL_STAT2_EO_PG                           = 0x800958000701143Full;

static const uint64_t P9A_OMIC_2_RX_CTL_STAT2_EO_PG                           = 0x800958000701183Full;

static const uint64_t P9A_OMIC_3_RX_CTL_STAT2_EO_PG                           = 0x800958000801103Full;

static const uint64_t P9A_OMIC_4_RX_CTL_STAT2_EO_PG                           = 0x800958000801143Full;

static const uint64_t P9A_OMIC_5_RX_CTL_STAT2_EO_PG                           = 0x800958000801183Full;


static const uint64_t P9A_MC_RX_DATASM_CNTLX1_EO_PG                           = 0x800B88000701103Full;
//DUPS: 800B88000701143F, 800B88000701183F,
static const uint64_t P9A_MC_0_RX_DATASM_CNTLX1_EO_PG                         = 0x800B88000701103Full;
//DUPS: 800B88000701143F, 800B88000701183F,
static const uint64_t P9A_MC_1_RX_DATASM_CNTLX1_EO_PG                         = 0x800B88000801103Full;
//DUPS: 800B88000801143F, 800B88000801183F,
static const uint64_t P9A_OMIC_RX_DATASM_CNTLX1_EO_PG                         = 0x800B88000701103Full;

static const uint64_t P9A_OMIC_0_RX_DATASM_CNTLX1_EO_PG                       = 0x800B88000701103Full;

static const uint64_t P9A_OMIC_1_RX_DATASM_CNTLX1_EO_PG                       = 0x800B88000701143Full;

static const uint64_t P9A_OMIC_2_RX_DATASM_CNTLX1_EO_PG                       = 0x800B88000701183Full;

static const uint64_t P9A_OMIC_3_RX_DATASM_CNTLX1_EO_PG                       = 0x800B88000801103Full;

static const uint64_t P9A_OMIC_4_RX_DATASM_CNTLX1_EO_PG                       = 0x800B88000801143Full;

static const uint64_t P9A_OMIC_5_RX_DATASM_CNTLX1_EO_PG                       = 0x800B88000801183Full;


static const uint64_t P9A_MC_RX_DATASM_SPARE_MODE_PG                          = 0x800B80000701103Full;
//DUPS: 800B80000701143F, 800B80000701183F,
static const uint64_t P9A_MC_0_RX_DATASM_SPARE_MODE_PG                        = 0x800B80000701103Full;
//DUPS: 800B80000701143F, 800B80000701183F,
static const uint64_t P9A_MC_1_RX_DATASM_SPARE_MODE_PG                        = 0x800B80000801103Full;
//DUPS: 800B80000801143F, 800B80000801183F,
static const uint64_t P9A_OMIC_RX_DATASM_SPARE_MODE_PG                        = 0x800B80000701103Full;

static const uint64_t P9A_OMIC_0_RX_DATASM_SPARE_MODE_PG                      = 0x800B80000701103Full;

static const uint64_t P9A_OMIC_1_RX_DATASM_SPARE_MODE_PG                      = 0x800B80000701143Full;

static const uint64_t P9A_OMIC_2_RX_DATASM_SPARE_MODE_PG                      = 0x800B80000701183Full;

static const uint64_t P9A_OMIC_3_RX_DATASM_SPARE_MODE_PG                      = 0x800B80000801103Full;

static const uint64_t P9A_OMIC_4_RX_DATASM_SPARE_MODE_PG                      = 0x800B80000801143Full;

static const uint64_t P9A_OMIC_5_RX_DATASM_SPARE_MODE_PG                      = 0x800B80000801183Full;


static const uint64_t P9A_MC_RX_DATASM_STAT10_EO_PG                           = 0x800BD8000701103Full;
//DUPS: 800BD8000701143F, 800BD8000701183F,
static const uint64_t P9A_MC_0_RX_DATASM_STAT10_EO_PG                         = 0x800BD8000701103Full;
//DUPS: 800BD8000701143F, 800BD8000701183F,
static const uint64_t P9A_MC_1_RX_DATASM_STAT10_EO_PG                         = 0x800BD8000801103Full;
//DUPS: 800BD8000801143F, 800BD8000801183F,
static const uint64_t P9A_OMIC_RX_DATASM_STAT10_EO_PG                         = 0x800BD8000701103Full;

static const uint64_t P9A_OMIC_0_RX_DATASM_STAT10_EO_PG                       = 0x800BD8000701103Full;

static const uint64_t P9A_OMIC_1_RX_DATASM_STAT10_EO_PG                       = 0x800BD8000701143Full;

static const uint64_t P9A_OMIC_2_RX_DATASM_STAT10_EO_PG                       = 0x800BD8000701183Full;

static const uint64_t P9A_OMIC_3_RX_DATASM_STAT10_EO_PG                       = 0x800BD8000801103Full;

static const uint64_t P9A_OMIC_4_RX_DATASM_STAT10_EO_PG                       = 0x800BD8000801143Full;

static const uint64_t P9A_OMIC_5_RX_DATASM_STAT10_EO_PG                       = 0x800BD8000801183Full;


static const uint64_t P9A_MC_RX_DATASM_STAT11_EO_PG                           = 0x800BE0000701103Full;
//DUPS: 800BE0000701143F, 800BE0000701183F,
static const uint64_t P9A_MC_0_RX_DATASM_STAT11_EO_PG                         = 0x800BE0000701103Full;
//DUPS: 800BE0000701143F, 800BE0000701183F,
static const uint64_t P9A_MC_1_RX_DATASM_STAT11_EO_PG                         = 0x800BE0000801103Full;
//DUPS: 800BE0000801143F, 800BE0000801183F,
static const uint64_t P9A_OMIC_RX_DATASM_STAT11_EO_PG                         = 0x800BE0000701103Full;

static const uint64_t P9A_OMIC_0_RX_DATASM_STAT11_EO_PG                       = 0x800BE0000701103Full;

static const uint64_t P9A_OMIC_1_RX_DATASM_STAT11_EO_PG                       = 0x800BE0000701143Full;

static const uint64_t P9A_OMIC_2_RX_DATASM_STAT11_EO_PG                       = 0x800BE0000701183Full;

static const uint64_t P9A_OMIC_3_RX_DATASM_STAT11_EO_PG                       = 0x800BE0000801103Full;

static const uint64_t P9A_OMIC_4_RX_DATASM_STAT11_EO_PG                       = 0x800BE0000801143Full;

static const uint64_t P9A_OMIC_5_RX_DATASM_STAT11_EO_PG                       = 0x800BE0000801183Full;


static const uint64_t P9A_MC_RX_DATASM_STAT1_EO_PG                            = 0x800B90000701103Full;
//DUPS: 800B90000701143F, 800B90000701183F,
static const uint64_t P9A_MC_0_RX_DATASM_STAT1_EO_PG                          = 0x800B90000701103Full;
//DUPS: 800B90000701143F, 800B90000701183F,
static const uint64_t P9A_MC_1_RX_DATASM_STAT1_EO_PG                          = 0x800B90000801103Full;
//DUPS: 800B90000801143F, 800B90000801183F,
static const uint64_t P9A_OMIC_RX_DATASM_STAT1_EO_PG                          = 0x800B90000701103Full;

static const uint64_t P9A_OMIC_0_RX_DATASM_STAT1_EO_PG                        = 0x800B90000701103Full;

static const uint64_t P9A_OMIC_1_RX_DATASM_STAT1_EO_PG                        = 0x800B90000701143Full;

static const uint64_t P9A_OMIC_2_RX_DATASM_STAT1_EO_PG                        = 0x800B90000701183Full;

static const uint64_t P9A_OMIC_3_RX_DATASM_STAT1_EO_PG                        = 0x800B90000801103Full;

static const uint64_t P9A_OMIC_4_RX_DATASM_STAT1_EO_PG                        = 0x800B90000801143Full;

static const uint64_t P9A_OMIC_5_RX_DATASM_STAT1_EO_PG                        = 0x800B90000801183Full;


static const uint64_t P9A_MC_RX_DATASM_STAT2_EO_PG                            = 0x800B98000701103Full;
//DUPS: 800B98000701143F, 800B98000701183F,
static const uint64_t P9A_MC_0_RX_DATASM_STAT2_EO_PG                          = 0x800B98000701103Full;
//DUPS: 800B98000701143F, 800B98000701183F,
static const uint64_t P9A_MC_1_RX_DATASM_STAT2_EO_PG                          = 0x800B98000801103Full;
//DUPS: 800B98000801143F, 800B98000801183F,
static const uint64_t P9A_OMIC_RX_DATASM_STAT2_EO_PG                          = 0x800B98000701103Full;

static const uint64_t P9A_OMIC_0_RX_DATASM_STAT2_EO_PG                        = 0x800B98000701103Full;

static const uint64_t P9A_OMIC_1_RX_DATASM_STAT2_EO_PG                        = 0x800B98000701143Full;

static const uint64_t P9A_OMIC_2_RX_DATASM_STAT2_EO_PG                        = 0x800B98000701183Full;

static const uint64_t P9A_OMIC_3_RX_DATASM_STAT2_EO_PG                        = 0x800B98000801103Full;

static const uint64_t P9A_OMIC_4_RX_DATASM_STAT2_EO_PG                        = 0x800B98000801143Full;

static const uint64_t P9A_OMIC_5_RX_DATASM_STAT2_EO_PG                        = 0x800B98000801183Full;


static const uint64_t P9A_MC_RX_DATASM_STAT3_EO_PG                            = 0x800BA0000701103Full;
//DUPS: 800BA0000701143F, 800BA0000701183F,
static const uint64_t P9A_MC_0_RX_DATASM_STAT3_EO_PG                          = 0x800BA0000701103Full;
//DUPS: 800BA0000701143F, 800BA0000701183F,
static const uint64_t P9A_MC_1_RX_DATASM_STAT3_EO_PG                          = 0x800BA0000801103Full;
//DUPS: 800BA0000801143F, 800BA0000801183F,
static const uint64_t P9A_OMIC_RX_DATASM_STAT3_EO_PG                          = 0x800BA0000701103Full;

static const uint64_t P9A_OMIC_0_RX_DATASM_STAT3_EO_PG                        = 0x800BA0000701103Full;

static const uint64_t P9A_OMIC_1_RX_DATASM_STAT3_EO_PG                        = 0x800BA0000701143Full;

static const uint64_t P9A_OMIC_2_RX_DATASM_STAT3_EO_PG                        = 0x800BA0000701183Full;

static const uint64_t P9A_OMIC_3_RX_DATASM_STAT3_EO_PG                        = 0x800BA0000801103Full;

static const uint64_t P9A_OMIC_4_RX_DATASM_STAT3_EO_PG                        = 0x800BA0000801143Full;

static const uint64_t P9A_OMIC_5_RX_DATASM_STAT3_EO_PG                        = 0x800BA0000801183Full;


static const uint64_t P9A_MC_RX_DATASM_STAT4_EO_PG                            = 0x800BA8000701103Full;
//DUPS: 800BA8000701143F, 800BA8000701183F,
static const uint64_t P9A_MC_0_RX_DATASM_STAT4_EO_PG                          = 0x800BA8000701103Full;
//DUPS: 800BA8000701143F, 800BA8000701183F,
static const uint64_t P9A_MC_1_RX_DATASM_STAT4_EO_PG                          = 0x800BA8000801103Full;
//DUPS: 800BA8000801143F, 800BA8000801183F,
static const uint64_t P9A_OMIC_RX_DATASM_STAT4_EO_PG                          = 0x800BA8000701103Full;

static const uint64_t P9A_OMIC_0_RX_DATASM_STAT4_EO_PG                        = 0x800BA8000701103Full;

static const uint64_t P9A_OMIC_1_RX_DATASM_STAT4_EO_PG                        = 0x800BA8000701143Full;

static const uint64_t P9A_OMIC_2_RX_DATASM_STAT4_EO_PG                        = 0x800BA8000701183Full;

static const uint64_t P9A_OMIC_3_RX_DATASM_STAT4_EO_PG                        = 0x800BA8000801103Full;

static const uint64_t P9A_OMIC_4_RX_DATASM_STAT4_EO_PG                        = 0x800BA8000801143Full;

static const uint64_t P9A_OMIC_5_RX_DATASM_STAT4_EO_PG                        = 0x800BA8000801183Full;


static const uint64_t P9A_MC_RX_DATASM_STAT5_EO_PG                            = 0x800BB0000701103Full;
//DUPS: 800BB0000701143F, 800BB0000701183F,
static const uint64_t P9A_MC_0_RX_DATASM_STAT5_EO_PG                          = 0x800BB0000701103Full;
//DUPS: 800BB0000701143F, 800BB0000701183F,
static const uint64_t P9A_MC_1_RX_DATASM_STAT5_EO_PG                          = 0x800BB0000801103Full;
//DUPS: 800BB0000801143F, 800BB0000801183F,
static const uint64_t P9A_OMIC_RX_DATASM_STAT5_EO_PG                          = 0x800BB0000701103Full;

static const uint64_t P9A_OMIC_0_RX_DATASM_STAT5_EO_PG                        = 0x800BB0000701103Full;

static const uint64_t P9A_OMIC_1_RX_DATASM_STAT5_EO_PG                        = 0x800BB0000701143Full;

static const uint64_t P9A_OMIC_2_RX_DATASM_STAT5_EO_PG                        = 0x800BB0000701183Full;

static const uint64_t P9A_OMIC_3_RX_DATASM_STAT5_EO_PG                        = 0x800BB0000801103Full;

static const uint64_t P9A_OMIC_4_RX_DATASM_STAT5_EO_PG                        = 0x800BB0000801143Full;

static const uint64_t P9A_OMIC_5_RX_DATASM_STAT5_EO_PG                        = 0x800BB0000801183Full;


static const uint64_t P9A_MC_RX_DATASM_STAT6_EO_PG                            = 0x800BB8000701103Full;
//DUPS: 800BB8000701143F, 800BB8000701183F,
static const uint64_t P9A_MC_0_RX_DATASM_STAT6_EO_PG                          = 0x800BB8000701103Full;
//DUPS: 800BB8000701143F, 800BB8000701183F,
static const uint64_t P9A_MC_1_RX_DATASM_STAT6_EO_PG                          = 0x800BB8000801103Full;
//DUPS: 800BB8000801143F, 800BB8000801183F,
static const uint64_t P9A_OMIC_RX_DATASM_STAT6_EO_PG                          = 0x800BB8000701103Full;

static const uint64_t P9A_OMIC_0_RX_DATASM_STAT6_EO_PG                        = 0x800BB8000701103Full;

static const uint64_t P9A_OMIC_1_RX_DATASM_STAT6_EO_PG                        = 0x800BB8000701143Full;

static const uint64_t P9A_OMIC_2_RX_DATASM_STAT6_EO_PG                        = 0x800BB8000701183Full;

static const uint64_t P9A_OMIC_3_RX_DATASM_STAT6_EO_PG                        = 0x800BB8000801103Full;

static const uint64_t P9A_OMIC_4_RX_DATASM_STAT6_EO_PG                        = 0x800BB8000801143Full;

static const uint64_t P9A_OMIC_5_RX_DATASM_STAT6_EO_PG                        = 0x800BB8000801183Full;


static const uint64_t P9A_MC_RX_DATASM_STAT7_EO_PG                            = 0x800BC0000701103Full;
//DUPS: 800BC0000701143F, 800BC0000701183F,
static const uint64_t P9A_MC_0_RX_DATASM_STAT7_EO_PG                          = 0x800BC0000701103Full;
//DUPS: 800BC0000701143F, 800BC0000701183F,
static const uint64_t P9A_MC_1_RX_DATASM_STAT7_EO_PG                          = 0x800BC0000801103Full;
//DUPS: 800BC0000801143F, 800BC0000801183F,
static const uint64_t P9A_OMIC_RX_DATASM_STAT7_EO_PG                          = 0x800BC0000701103Full;

static const uint64_t P9A_OMIC_0_RX_DATASM_STAT7_EO_PG                        = 0x800BC0000701103Full;

static const uint64_t P9A_OMIC_1_RX_DATASM_STAT7_EO_PG                        = 0x800BC0000701143Full;

static const uint64_t P9A_OMIC_2_RX_DATASM_STAT7_EO_PG                        = 0x800BC0000701183Full;

static const uint64_t P9A_OMIC_3_RX_DATASM_STAT7_EO_PG                        = 0x800BC0000801103Full;

static const uint64_t P9A_OMIC_4_RX_DATASM_STAT7_EO_PG                        = 0x800BC0000801143Full;

static const uint64_t P9A_OMIC_5_RX_DATASM_STAT7_EO_PG                        = 0x800BC0000801183Full;


static const uint64_t P9A_MC_RX_DATASM_STAT8_EO_PG                            = 0x800BC8000701103Full;
//DUPS: 800BC8000701143F, 800BC8000701183F,
static const uint64_t P9A_MC_0_RX_DATASM_STAT8_EO_PG                          = 0x800BC8000701103Full;
//DUPS: 800BC8000701143F, 800BC8000701183F,
static const uint64_t P9A_MC_1_RX_DATASM_STAT8_EO_PG                          = 0x800BC8000801103Full;
//DUPS: 800BC8000801143F, 800BC8000801183F,
static const uint64_t P9A_OMIC_RX_DATASM_STAT8_EO_PG                          = 0x800BC8000701103Full;

static const uint64_t P9A_OMIC_0_RX_DATASM_STAT8_EO_PG                        = 0x800BC8000701103Full;

static const uint64_t P9A_OMIC_1_RX_DATASM_STAT8_EO_PG                        = 0x800BC8000701143Full;

static const uint64_t P9A_OMIC_2_RX_DATASM_STAT8_EO_PG                        = 0x800BC8000701183Full;

static const uint64_t P9A_OMIC_3_RX_DATASM_STAT8_EO_PG                        = 0x800BC8000801103Full;

static const uint64_t P9A_OMIC_4_RX_DATASM_STAT8_EO_PG                        = 0x800BC8000801143Full;

static const uint64_t P9A_OMIC_5_RX_DATASM_STAT8_EO_PG                        = 0x800BC8000801183Full;


static const uint64_t P9A_MC_RX_DATASM_STAT9_EO_PG                            = 0x800BD0000701103Full;
//DUPS: 800BD0000701143F, 800BD0000701183F,
static const uint64_t P9A_MC_0_RX_DATASM_STAT9_EO_PG                          = 0x800BD0000701103Full;
//DUPS: 800BD0000701143F, 800BD0000701183F,
static const uint64_t P9A_MC_1_RX_DATASM_STAT9_EO_PG                          = 0x800BD0000801103Full;
//DUPS: 800BD0000801143F, 800BD0000801183F,
static const uint64_t P9A_OMIC_RX_DATASM_STAT9_EO_PG                          = 0x800BD0000701103Full;

static const uint64_t P9A_OMIC_0_RX_DATASM_STAT9_EO_PG                        = 0x800BD0000701103Full;

static const uint64_t P9A_OMIC_1_RX_DATASM_STAT9_EO_PG                        = 0x800BD0000701143Full;

static const uint64_t P9A_OMIC_2_RX_DATASM_STAT9_EO_PG                        = 0x800BD0000701183Full;

static const uint64_t P9A_OMIC_3_RX_DATASM_STAT9_EO_PG                        = 0x800BD0000801103Full;

static const uint64_t P9A_OMIC_4_RX_DATASM_STAT9_EO_PG                        = 0x800BD0000801143Full;

static const uint64_t P9A_OMIC_5_RX_DATASM_STAT9_EO_PG                        = 0x800BD0000801183Full;


static const uint64_t P9A_MC_RX_FIR1_ERROR_INJECT_PG                          = 0x800A98000701103Full;
//DUPS: 800A98000701143F, 800A98000701183F,
static const uint64_t P9A_MC_0_RX_FIR1_ERROR_INJECT_PG                        = 0x800A98000701103Full;
//DUPS: 800A98000701143F, 800A98000701183F,
static const uint64_t P9A_MC_1_RX_FIR1_ERROR_INJECT_PG                        = 0x800A98000801103Full;
//DUPS: 800A98000801143F, 800A98000801183F,
static const uint64_t P9A_OMIC_RX_FIR1_ERROR_INJECT_PG                        = 0x800A98000701103Full;

static const uint64_t P9A_OMIC_0_RX_FIR1_ERROR_INJECT_PG                      = 0x800A98000701103Full;

static const uint64_t P9A_OMIC_1_RX_FIR1_ERROR_INJECT_PG                      = 0x800A98000701143Full;

static const uint64_t P9A_OMIC_2_RX_FIR1_ERROR_INJECT_PG                      = 0x800A98000701183Full;

static const uint64_t P9A_OMIC_3_RX_FIR1_ERROR_INJECT_PG                      = 0x800A98000801103Full;

static const uint64_t P9A_OMIC_4_RX_FIR1_ERROR_INJECT_PG                      = 0x800A98000801143Full;

static const uint64_t P9A_OMIC_5_RX_FIR1_ERROR_INJECT_PG                      = 0x800A98000801183Full;


static const uint64_t P9A_MC_RX_FIR1_MASK_PG                                  = 0x800A90000701103Full;
//DUPS: 800A90000701143F, 800A90000701183F,
static const uint64_t P9A_MC_0_RX_FIR1_MASK_PG                                = 0x800A90000701103Full;
//DUPS: 800A90000701143F, 800A90000701183F,
static const uint64_t P9A_MC_1_RX_FIR1_MASK_PG                                = 0x800A90000801103Full;
//DUPS: 800A90000801143F, 800A90000801183F,
static const uint64_t P9A_OMIC_RX_FIR1_MASK_PG                                = 0x800A90000701103Full;

static const uint64_t P9A_OMIC_0_RX_FIR1_MASK_PG                              = 0x800A90000701103Full;

static const uint64_t P9A_OMIC_1_RX_FIR1_MASK_PG                              = 0x800A90000701143Full;

static const uint64_t P9A_OMIC_2_RX_FIR1_MASK_PG                              = 0x800A90000701183Full;

static const uint64_t P9A_OMIC_3_RX_FIR1_MASK_PG                              = 0x800A90000801103Full;

static const uint64_t P9A_OMIC_4_RX_FIR1_MASK_PG                              = 0x800A90000801143Full;

static const uint64_t P9A_OMIC_5_RX_FIR1_MASK_PG                              = 0x800A90000801183Full;


static const uint64_t P9A_MC_RX_FIR1_PG                                       = 0x800A88000701103Full;
//DUPS: 800A88000701143F, 800A88000701183F,
static const uint64_t P9A_MC_0_RX_FIR1_PG                                     = 0x800A88000701103Full;
//DUPS: 800A88000701143F, 800A88000701183F,
static const uint64_t P9A_MC_1_RX_FIR1_PG                                     = 0x800A88000801103Full;
//DUPS: 800A88000801143F, 800A88000801183F,
static const uint64_t P9A_OMIC_RX_FIR1_PG                                     = 0x800A88000701103Full;

static const uint64_t P9A_OMIC_0_RX_FIR1_PG                                   = 0x800A88000701103Full;

static const uint64_t P9A_OMIC_1_RX_FIR1_PG                                   = 0x800A88000701143Full;

static const uint64_t P9A_OMIC_2_RX_FIR1_PG                                   = 0x800A88000701183Full;

static const uint64_t P9A_OMIC_3_RX_FIR1_PG                                   = 0x800A88000801103Full;

static const uint64_t P9A_OMIC_4_RX_FIR1_PG                                   = 0x800A88000801143Full;

static const uint64_t P9A_OMIC_5_RX_FIR1_PG                                   = 0x800A88000801183Full;


static const uint64_t P9A_MC_RX_FIR_ERROR_INJECT_PB                           = 0x800F98000701103Full;
//DUPS: 800F98000701143F, 800F98000701183F,
static const uint64_t P9A_MC_0_RX_FIR_ERROR_INJECT_PB                         = 0x800F98000701103Full;
//DUPS: 800F98000701143F, 800F98000701183F,
static const uint64_t P9A_MC_1_RX_FIR_ERROR_INJECT_PB                         = 0x800F98000801103Full;
//DUPS: 800F98000801143F, 800F98000801183F,

static const uint64_t P9A_OMIC_RX_FIR_ERROR_INJECT_PB                         = 0x800F98000701103Full;

static const uint64_t P9A_OMIC_0_RX_FIR_ERROR_INJECT_PB                       = 0x800F98000701103Full;

static const uint64_t P9A_OMIC_1_RX_FIR_ERROR_INJECT_PB                       = 0x800F98000701143Full;

static const uint64_t P9A_OMIC_2_RX_FIR_ERROR_INJECT_PB                       = 0x800F98000701183Full;

static const uint64_t P9A_OMIC_3_RX_FIR_ERROR_INJECT_PB                       = 0x800F98000801103Full;

static const uint64_t P9A_OMIC_4_RX_FIR_ERROR_INJECT_PB                       = 0x800F98000801143Full;

static const uint64_t P9A_OMIC_5_RX_FIR_ERROR_INJECT_PB                       = 0x800F98000801183Full;


static const uint64_t P9A_MC_RX_FIR_MASK_PB                                   = 0x800F90000701103Full;
//DUPS: 800F90000701143F, 800F90000701183F,
static const uint64_t P9A_MC_0_RX_FIR_MASK_PB                                 = 0x800F90000701103Full;
//DUPS: 800F90000701143F, 800F90000701183F,
static const uint64_t P9A_MC_1_RX_FIR_MASK_PB                                 = 0x800F90000801103Full;
//DUPS: 800F90000801143F, 800F90000801183F,

static const uint64_t P9A_OMIC_RX_FIR_MASK_PB                                 = 0x800F90000701103Full;

static const uint64_t P9A_OMIC_0_RX_FIR_MASK_PB                               = 0x800F90000701103Full;

static const uint64_t P9A_OMIC_1_RX_FIR_MASK_PB                               = 0x800F90000701143Full;

static const uint64_t P9A_OMIC_2_RX_FIR_MASK_PB                               = 0x800F90000701183Full;

static const uint64_t P9A_OMIC_3_RX_FIR_MASK_PB                               = 0x800F90000801103Full;

static const uint64_t P9A_OMIC_4_RX_FIR_MASK_PB                               = 0x800F90000801143Full;

static const uint64_t P9A_OMIC_5_RX_FIR_MASK_PB                               = 0x800F90000801183Full;


static const uint64_t P9A_MC_RX_FIR_PB                                        = 0x800F88000701103Full;
//DUPS: 800F88000701143F, 800F88000701183F,
static const uint64_t P9A_MC_0_RX_FIR_PB                                      = 0x800F88000701103Full;
//DUPS: 800F88000701143F, 800F88000701183F,
static const uint64_t P9A_MC_1_RX_FIR_PB                                      = 0x800F88000801103Full;
//DUPS: 800F88000801143F, 800F88000801183F,

static const uint64_t P9A_OMIC_RX_FIR_PB                                      = 0x800F88000701103Full;

static const uint64_t P9A_OMIC_0_RX_FIR_PB                                    = 0x800F88000701103Full;

static const uint64_t P9A_OMIC_1_RX_FIR_PB                                    = 0x800F88000701143Full;

static const uint64_t P9A_OMIC_2_RX_FIR_PB                                    = 0x800F88000701183Full;

static const uint64_t P9A_OMIC_3_RX_FIR_PB                                    = 0x800F88000801103Full;

static const uint64_t P9A_OMIC_4_RX_FIR_PB                                    = 0x800F88000801143Full;

static const uint64_t P9A_OMIC_5_RX_FIR_PB                                    = 0x800F88000801183Full;


static const uint64_t P9A_MC_RX_FIR_RESET_PB                                  = 0x800F80000701103Full;
//DUPS: 800F80000701143F, 800F80000701183F,
static const uint64_t P9A_MC_0_RX_FIR_RESET_PB                                = 0x800F80000701103Full;
//DUPS: 800F80000701143F, 800F80000701183F,
static const uint64_t P9A_MC_1_RX_FIR_RESET_PB                                = 0x800F80000801103Full;
//DUPS: 800F80000801143F, 800F80000801183F,

static const uint64_t P9A_OMIC_RX_FIR_RESET_PB                                = 0x800F80000701103Full;

static const uint64_t P9A_OMIC_0_RX_FIR_RESET_PB                              = 0x800F80000701103Full;

static const uint64_t P9A_OMIC_1_RX_FIR_RESET_PB                              = 0x800F80000701143Full;

static const uint64_t P9A_OMIC_2_RX_FIR_RESET_PB                              = 0x800F80000701183Full;

static const uint64_t P9A_OMIC_3_RX_FIR_RESET_PB                              = 0x800F80000801103Full;

static const uint64_t P9A_OMIC_4_RX_FIR_RESET_PB                              = 0x800F80000801143Full;

static const uint64_t P9A_OMIC_5_RX_FIR_RESET_PB                              = 0x800F80000801183Full;


static const uint64_t P9A_MC_RX_GLBSM_CNTL2_EO_PG                             = 0x800AE0000701103Full;
//DUPS: 800AE0000701143F, 800AE0000701183F,
static const uint64_t P9A_MC_0_RX_GLBSM_CNTL2_EO_PG                           = 0x800AE0000701103Full;
//DUPS: 800AE0000701143F, 800AE0000701183F,
static const uint64_t P9A_MC_1_RX_GLBSM_CNTL2_EO_PG                           = 0x800AE0000801103Full;
//DUPS: 800AE0000801143F, 800AE0000801183F,
static const uint64_t P9A_OMIC_RX_GLBSM_CNTL2_EO_PG                           = 0x800AE0000701103Full;

static const uint64_t P9A_OMIC_0_RX_GLBSM_CNTL2_EO_PG                         = 0x800AE0000701103Full;

static const uint64_t P9A_OMIC_1_RX_GLBSM_CNTL2_EO_PG                         = 0x800AE0000701143Full;

static const uint64_t P9A_OMIC_2_RX_GLBSM_CNTL2_EO_PG                         = 0x800AE0000701183Full;

static const uint64_t P9A_OMIC_3_RX_GLBSM_CNTL2_EO_PG                         = 0x800AE0000801103Full;

static const uint64_t P9A_OMIC_4_RX_GLBSM_CNTL2_EO_PG                         = 0x800AE0000801143Full;

static const uint64_t P9A_OMIC_5_RX_GLBSM_CNTL2_EO_PG                         = 0x800AE0000801183Full;


static const uint64_t P9A_MC_RX_GLBSM_CNTL3_EO_PG                             = 0x800AE8000701103Full;
//DUPS: 800AE8000701143F, 800AE8000701183F,
static const uint64_t P9A_MC_0_RX_GLBSM_CNTL3_EO_PG                           = 0x800AE8000701103Full;
//DUPS: 800AE8000701143F, 800AE8000701183F,
static const uint64_t P9A_MC_1_RX_GLBSM_CNTL3_EO_PG                           = 0x800AE8000801103Full;
//DUPS: 800AE8000801143F, 800AE8000801183F,
static const uint64_t P9A_OMIC_RX_GLBSM_CNTL3_EO_PG                           = 0x800AE8000701103Full;

static const uint64_t P9A_OMIC_0_RX_GLBSM_CNTL3_EO_PG                         = 0x800AE8000701103Full;

static const uint64_t P9A_OMIC_1_RX_GLBSM_CNTL3_EO_PG                         = 0x800AE8000701143Full;

static const uint64_t P9A_OMIC_2_RX_GLBSM_CNTL3_EO_PG                         = 0x800AE8000701183Full;

static const uint64_t P9A_OMIC_3_RX_GLBSM_CNTL3_EO_PG                         = 0x800AE8000801103Full;

static const uint64_t P9A_OMIC_4_RX_GLBSM_CNTL3_EO_PG                         = 0x800AE8000801143Full;

static const uint64_t P9A_OMIC_5_RX_GLBSM_CNTL3_EO_PG                         = 0x800AE8000801183Full;


static const uint64_t P9A_MC_RX_GLBSM_CNTL4_EO_PG                             = 0x800AF0000701103Full;
//DUPS: 800AF0000701143F, 800AF0000701183F,
static const uint64_t P9A_MC_0_RX_GLBSM_CNTL4_EO_PG                           = 0x800AF0000701103Full;
//DUPS: 800AF0000701143F, 800AF0000701183F,
static const uint64_t P9A_MC_1_RX_GLBSM_CNTL4_EO_PG                           = 0x800AF0000801103Full;
//DUPS: 800AF0000801143F, 800AF0000801183F,
static const uint64_t P9A_OMIC_RX_GLBSM_CNTL4_EO_PG                           = 0x800AF0000701103Full;

static const uint64_t P9A_OMIC_0_RX_GLBSM_CNTL4_EO_PG                         = 0x800AF0000701103Full;

static const uint64_t P9A_OMIC_1_RX_GLBSM_CNTL4_EO_PG                         = 0x800AF0000701143Full;

static const uint64_t P9A_OMIC_2_RX_GLBSM_CNTL4_EO_PG                         = 0x800AF0000701183Full;

static const uint64_t P9A_OMIC_3_RX_GLBSM_CNTL4_EO_PG                         = 0x800AF0000801103Full;

static const uint64_t P9A_OMIC_4_RX_GLBSM_CNTL4_EO_PG                         = 0x800AF0000801143Full;

static const uint64_t P9A_OMIC_5_RX_GLBSM_CNTL4_EO_PG                         = 0x800AF0000801183Full;


static const uint64_t P9A_MC_RX_GLBSM_CNTLX1_EO_PG                            = 0x800AB0000701103Full;
//DUPS: 800AB0000701143F, 800AB0000701183F,
static const uint64_t P9A_MC_0_RX_GLBSM_CNTLX1_EO_PG                          = 0x800AB0000701103Full;
//DUPS: 800AB0000701143F, 800AB0000701183F,
static const uint64_t P9A_MC_1_RX_GLBSM_CNTLX1_EO_PG                          = 0x800AB0000801103Full;
//DUPS: 800AB0000801143F, 800AB0000801183F,
static const uint64_t P9A_OMIC_RX_GLBSM_CNTLX1_EO_PG                          = 0x800AB0000701103Full;

static const uint64_t P9A_OMIC_0_RX_GLBSM_CNTLX1_EO_PG                        = 0x800AB0000701103Full;

static const uint64_t P9A_OMIC_1_RX_GLBSM_CNTLX1_EO_PG                        = 0x800AB0000701143Full;

static const uint64_t P9A_OMIC_2_RX_GLBSM_CNTLX1_EO_PG                        = 0x800AB0000701183Full;

static const uint64_t P9A_OMIC_3_RX_GLBSM_CNTLX1_EO_PG                        = 0x800AB0000801103Full;

static const uint64_t P9A_OMIC_4_RX_GLBSM_CNTLX1_EO_PG                        = 0x800AB0000801143Full;

static const uint64_t P9A_OMIC_5_RX_GLBSM_CNTLX1_EO_PG                        = 0x800AB0000801183Full;


static const uint64_t P9A_MC_RX_GLBSM_MODE1_EO_PG                             = 0x800AF8000701103Full;
//DUPS: 800AF8000701143F, 800AF8000701183F,
static const uint64_t P9A_MC_0_RX_GLBSM_MODE1_EO_PG                           = 0x800AF8000701103Full;
//DUPS: 800AF8000701143F, 800AF8000701183F,
static const uint64_t P9A_MC_1_RX_GLBSM_MODE1_EO_PG                           = 0x800AF8000801103Full;
//DUPS: 800AF8000801143F, 800AF8000801183F,
static const uint64_t P9A_OMIC_RX_GLBSM_MODE1_EO_PG                           = 0x800AF8000701103Full;

static const uint64_t P9A_OMIC_0_RX_GLBSM_MODE1_EO_PG                         = 0x800AF8000701103Full;

static const uint64_t P9A_OMIC_1_RX_GLBSM_MODE1_EO_PG                         = 0x800AF8000701143Full;

static const uint64_t P9A_OMIC_2_RX_GLBSM_MODE1_EO_PG                         = 0x800AF8000701183Full;

static const uint64_t P9A_OMIC_3_RX_GLBSM_MODE1_EO_PG                         = 0x800AF8000801103Full;

static const uint64_t P9A_OMIC_4_RX_GLBSM_MODE1_EO_PG                         = 0x800AF8000801143Full;

static const uint64_t P9A_OMIC_5_RX_GLBSM_MODE1_EO_PG                         = 0x800AF8000801183Full;


static const uint64_t P9A_MC_RX_GLBSM_SPARE_MODE_PG                           = 0x800A80000701103Full;
//DUPS: 800A80000701143F, 800A80000701183F,
static const uint64_t P9A_MC_0_RX_GLBSM_SPARE_MODE_PG                         = 0x800A80000701103Full;
//DUPS: 800A80000701143F, 800A80000701183F,
static const uint64_t P9A_MC_1_RX_GLBSM_SPARE_MODE_PG                         = 0x800A80000801103Full;
//DUPS: 800A80000801143F, 800A80000801183F,
static const uint64_t P9A_OMIC_RX_GLBSM_SPARE_MODE_PG                         = 0x800A80000701103Full;

static const uint64_t P9A_OMIC_0_RX_GLBSM_SPARE_MODE_PG                       = 0x800A80000701103Full;

static const uint64_t P9A_OMIC_1_RX_GLBSM_SPARE_MODE_PG                       = 0x800A80000701143Full;

static const uint64_t P9A_OMIC_2_RX_GLBSM_SPARE_MODE_PG                       = 0x800A80000701183Full;

static const uint64_t P9A_OMIC_3_RX_GLBSM_SPARE_MODE_PG                       = 0x800A80000801103Full;

static const uint64_t P9A_OMIC_4_RX_GLBSM_SPARE_MODE_PG                       = 0x800A80000801143Full;

static const uint64_t P9A_OMIC_5_RX_GLBSM_SPARE_MODE_PG                       = 0x800A80000801183Full;


static const uint64_t P9A_MC_RX_GLBSM_STAT1_EO_PG                             = 0x800AB8000701103Full;
//DUPS: 800AB8000701143F, 800AB8000701183F,
static const uint64_t P9A_MC_0_RX_GLBSM_STAT1_EO_PG                           = 0x800AB8000701103Full;
//DUPS: 800AB8000701143F, 800AB8000701183F,
static const uint64_t P9A_MC_1_RX_GLBSM_STAT1_EO_PG                           = 0x800AB8000801103Full;
//DUPS: 800AB8000801143F, 800AB8000801183F,
static const uint64_t P9A_OMIC_RX_GLBSM_STAT1_EO_PG                           = 0x800AB8000701103Full;

static const uint64_t P9A_OMIC_0_RX_GLBSM_STAT1_EO_PG                         = 0x800AB8000701103Full;

static const uint64_t P9A_OMIC_1_RX_GLBSM_STAT1_EO_PG                         = 0x800AB8000701143Full;

static const uint64_t P9A_OMIC_2_RX_GLBSM_STAT1_EO_PG                         = 0x800AB8000701183Full;

static const uint64_t P9A_OMIC_3_RX_GLBSM_STAT1_EO_PG                         = 0x800AB8000801103Full;

static const uint64_t P9A_OMIC_4_RX_GLBSM_STAT1_EO_PG                         = 0x800AB8000801143Full;

static const uint64_t P9A_OMIC_5_RX_GLBSM_STAT1_EO_PG                         = 0x800AB8000801183Full;


static const uint64_t P9A_MC_RX_GLBSM_STAT2_EO_PG                             = 0x800AC0000701103Full;
//DUPS: 800AC0000701143F, 800AC0000701183F,
static const uint64_t P9A_MC_0_RX_GLBSM_STAT2_EO_PG                           = 0x800AC0000701103Full;
//DUPS: 800AC0000701143F, 800AC0000701183F,
static const uint64_t P9A_MC_1_RX_GLBSM_STAT2_EO_PG                           = 0x800AC0000801103Full;
//DUPS: 800AC0000801143F, 800AC0000801183F,
static const uint64_t P9A_OMIC_RX_GLBSM_STAT2_EO_PG                           = 0x800AC0000701103Full;

static const uint64_t P9A_OMIC_0_RX_GLBSM_STAT2_EO_PG                         = 0x800AC0000701103Full;

static const uint64_t P9A_OMIC_1_RX_GLBSM_STAT2_EO_PG                         = 0x800AC0000701143Full;

static const uint64_t P9A_OMIC_2_RX_GLBSM_STAT2_EO_PG                         = 0x800AC0000701183Full;

static const uint64_t P9A_OMIC_3_RX_GLBSM_STAT2_EO_PG                         = 0x800AC0000801103Full;

static const uint64_t P9A_OMIC_4_RX_GLBSM_STAT2_EO_PG                         = 0x800AC0000801143Full;

static const uint64_t P9A_OMIC_5_RX_GLBSM_STAT2_EO_PG                         = 0x800AC0000801183Full;


static const uint64_t P9A_MC_RX_GLBSM_STAT3_EO_PG                             = 0x800AC8000701103Full;
//DUPS: 800AC8000701143F, 800AC8000701183F,
static const uint64_t P9A_MC_0_RX_GLBSM_STAT3_EO_PG                           = 0x800AC8000701103Full;
//DUPS: 800AC8000701143F, 800AC8000701183F,
static const uint64_t P9A_MC_1_RX_GLBSM_STAT3_EO_PG                           = 0x800AC8000801103Full;
//DUPS: 800AC8000801143F, 800AC8000801183F,
static const uint64_t P9A_OMIC_RX_GLBSM_STAT3_EO_PG                           = 0x800AC8000701103Full;

static const uint64_t P9A_OMIC_0_RX_GLBSM_STAT3_EO_PG                         = 0x800AC8000701103Full;

static const uint64_t P9A_OMIC_1_RX_GLBSM_STAT3_EO_PG                         = 0x800AC8000701143Full;

static const uint64_t P9A_OMIC_2_RX_GLBSM_STAT3_EO_PG                         = 0x800AC8000701183Full;

static const uint64_t P9A_OMIC_3_RX_GLBSM_STAT3_EO_PG                         = 0x800AC8000801103Full;

static const uint64_t P9A_OMIC_4_RX_GLBSM_STAT3_EO_PG                         = 0x800AC8000801143Full;

static const uint64_t P9A_OMIC_5_RX_GLBSM_STAT3_EO_PG                         = 0x800AC8000801183Full;


static const uint64_t P9A_MC_RX_GLBSM_STAT4_EO_PG                             = 0x800AD0000701103Full;
//DUPS: 800AD0000701143F, 800AD0000701183F,
static const uint64_t P9A_MC_0_RX_GLBSM_STAT4_EO_PG                           = 0x800AD0000701103Full;
//DUPS: 800AD0000701143F, 800AD0000701183F,
static const uint64_t P9A_MC_1_RX_GLBSM_STAT4_EO_PG                           = 0x800AD0000801103Full;
//DUPS: 800AD0000801143F, 800AD0000801183F,
static const uint64_t P9A_OMIC_RX_GLBSM_STAT4_EO_PG                           = 0x800AD0000701103Full;

static const uint64_t P9A_OMIC_0_RX_GLBSM_STAT4_EO_PG                         = 0x800AD0000701103Full;

static const uint64_t P9A_OMIC_1_RX_GLBSM_STAT4_EO_PG                         = 0x800AD0000701143Full;

static const uint64_t P9A_OMIC_2_RX_GLBSM_STAT4_EO_PG                         = 0x800AD0000701183Full;

static const uint64_t P9A_OMIC_3_RX_GLBSM_STAT4_EO_PG                         = 0x800AD0000801103Full;

static const uint64_t P9A_OMIC_4_RX_GLBSM_STAT4_EO_PG                         = 0x800AD0000801143Full;

static const uint64_t P9A_OMIC_5_RX_GLBSM_STAT4_EO_PG                         = 0x800AD0000801183Full;


static const uint64_t P9A_MC_RX_GLBSM_STAT5_EO_PG                             = 0x800AD8000701103Full;
//DUPS: 800AD8000701143F, 800AD8000701183F,
static const uint64_t P9A_MC_0_RX_GLBSM_STAT5_EO_PG                           = 0x800AD8000701103Full;
//DUPS: 800AD8000701143F, 800AD8000701183F,
static const uint64_t P9A_MC_1_RX_GLBSM_STAT5_EO_PG                           = 0x800AD8000801103Full;
//DUPS: 800AD8000801143F, 800AD8000801183F,
static const uint64_t P9A_OMIC_RX_GLBSM_STAT5_EO_PG                           = 0x800AD8000701103Full;

static const uint64_t P9A_OMIC_0_RX_GLBSM_STAT5_EO_PG                         = 0x800AD8000701103Full;

static const uint64_t P9A_OMIC_1_RX_GLBSM_STAT5_EO_PG                         = 0x800AD8000701143Full;

static const uint64_t P9A_OMIC_2_RX_GLBSM_STAT5_EO_PG                         = 0x800AD8000701183Full;

static const uint64_t P9A_OMIC_3_RX_GLBSM_STAT5_EO_PG                         = 0x800AD8000801103Full;

static const uint64_t P9A_OMIC_4_RX_GLBSM_STAT5_EO_PG                         = 0x800AD8000801143Full;

static const uint64_t P9A_OMIC_5_RX_GLBSM_STAT5_EO_PG                         = 0x800AD8000801183Full;


static const uint64_t P9A_MC_RX_ID1_PG                                        = 0x800808000701103Full;
//DUPS: 800808000701143F, 800808000701183F,
static const uint64_t P9A_MC_0_RX_ID1_PG                                      = 0x800808000701103Full;
//DUPS: 800808000701143F, 800808000701183F,
static const uint64_t P9A_MC_1_RX_ID1_PG                                      = 0x800808000801103Full;
//DUPS: 800808000801143F, 800808000801183F,
static const uint64_t P9A_OMIC_RX_ID1_PG                                      = 0x800808000701103Full;

static const uint64_t P9A_OMIC_0_RX_ID1_PG                                    = 0x800808000701103Full;

static const uint64_t P9A_OMIC_1_RX_ID1_PG                                    = 0x800808000701143Full;

static const uint64_t P9A_OMIC_2_RX_ID1_PG                                    = 0x800808000701183Full;

static const uint64_t P9A_OMIC_3_RX_ID1_PG                                    = 0x800808000801103Full;

static const uint64_t P9A_OMIC_4_RX_ID1_PG                                    = 0x800808000801143Full;

static const uint64_t P9A_OMIC_5_RX_ID1_PG                                    = 0x800808000801183Full;


static const uint64_t P9A_MC_RX_SPARE_MODE_PG                                 = 0x800800000701103Full;
//DUPS: 800800000701143F, 800800000701183F,
static const uint64_t P9A_MC_0_RX_SPARE_MODE_PG                               = 0x800800000701103Full;
//DUPS: 800800000701143F, 800800000701183F,
static const uint64_t P9A_MC_1_RX_SPARE_MODE_PG                               = 0x800800000801103Full;
//DUPS: 800800000801143F, 800800000801183F,
static const uint64_t P9A_OMIC_RX_SPARE_MODE_PG                               = 0x800800000701103Full;

static const uint64_t P9A_OMIC_0_RX_SPARE_MODE_PG                             = 0x800800000701103Full;

static const uint64_t P9A_OMIC_1_RX_SPARE_MODE_PG                             = 0x800800000701143Full;

static const uint64_t P9A_OMIC_2_RX_SPARE_MODE_PG                             = 0x800800000701183Full;

static const uint64_t P9A_OMIC_3_RX_SPARE_MODE_PG                             = 0x800800000801103Full;

static const uint64_t P9A_OMIC_4_RX_SPARE_MODE_PG                             = 0x800800000801143Full;

static const uint64_t P9A_OMIC_5_RX_SPARE_MODE_PG                             = 0x800800000801183Full;


static const uint64_t P9A_MC_SCAN32                                           = 0x07038000ull;

static const uint64_t P9A_MC_0_SCAN32                                         = 0x07038000ull;

static const uint64_t P9A_MC_1_SCAN32                                         = 0x08038000ull;


static const uint64_t P9A_MC_SCAN64                                           = 0x0703E000ull;

static const uint64_t P9A_MC_0_SCAN64                                         = 0x0703E000ull;

static const uint64_t P9A_MC_1_SCAN64                                         = 0x0803E000ull;


static const uint64_t P9A_MC_SCAN_CAPTUREDR                                   = 0x0703C000ull;

static const uint64_t P9A_MC_0_SCAN_CAPTUREDR                                 = 0x0703C000ull;

static const uint64_t P9A_MC_1_SCAN_CAPTUREDR                                 = 0x0803C000ull;


static const uint64_t P9A_MC_SCAN_CAPTUREDR_LONG                              = 0x0703D000ull;

static const uint64_t P9A_MC_0_SCAN_CAPTUREDR_LONG                            = 0x0703D000ull;

static const uint64_t P9A_MC_1_SCAN_CAPTUREDR_LONG                            = 0x0803D000ull;


static const uint64_t P9A_MC_SCAN_LONG_ROTATE                                 = 0x07039000ull;

static const uint64_t P9A_MC_0_SCAN_LONG_ROTATE                               = 0x07039000ull;

static const uint64_t P9A_MC_1_SCAN_LONG_ROTATE                               = 0x08039000ull;


static const uint64_t P9A_MC_SCAN_REGION_TYPE                                 = 0x07030005ull;

static const uint64_t P9A_MC_0_SCAN_REGION_TYPE                               = 0x07030005ull;

static const uint64_t P9A_MC_1_SCAN_REGION_TYPE                               = 0x08030005ull;


static const uint64_t P9A_MC_SCAN_UPDATEDR                                    = 0x0703A000ull;

static const uint64_t P9A_MC_0_SCAN_UPDATEDR                                  = 0x0703A000ull;

static const uint64_t P9A_MC_1_SCAN_UPDATEDR                                  = 0x0803A000ull;


static const uint64_t P9A_MC_SCAN_UPDATEDR_LONG                               = 0x0703B000ull;

static const uint64_t P9A_MC_0_SCAN_UPDATEDR_LONG                             = 0x0703B000ull;

static const uint64_t P9A_MC_1_SCAN_UPDATEDR_LONG                             = 0x0803B000ull;


static const uint64_t P9A_MC_SCOM_MODE_PB                                     = 0x07011020ull;
//DUPS: 07011420, 07011820,
static const uint64_t P9A_MC_0_SCOM_MODE_PB                                   = 0x07011020ull;
//DUPS: 07011420, 07011820,
static const uint64_t P9A_MC_1_SCOM_MODE_PB                                   = 0x08011020ull;
//DUPS: 08011420, 08011820,

static const uint64_t P9A_OMIC_SCOM_MODE_PB                                   = 0x07011020ull;

static const uint64_t P9A_OMIC_0_SCOM_MODE_PB                                 = 0x07011020ull;

static const uint64_t P9A_OMIC_1_SCOM_MODE_PB                                 = 0x07011420ull;

static const uint64_t P9A_OMIC_2_SCOM_MODE_PB                                 = 0x07011820ull;

static const uint64_t P9A_OMIC_3_SCOM_MODE_PB                                 = 0x08011020ull;

static const uint64_t P9A_OMIC_4_SCOM_MODE_PB                                 = 0x08011420ull;

static const uint64_t P9A_OMIC_5_SCOM_MODE_PB                                 = 0x08011820ull;


static const uint64_t P9A_MC_SCOM_PPE_CNTL                                    = 0x07012C60ull;
//DUPS: 07012860, 07012C60,
static const uint64_t P9A_MC_0_SCOM_PPE_CNTL                                  = 0x07012C60ull;
//DUPS: 07012860, 07012C60,
static const uint64_t P9A_MC_1_SCOM_PPE_CNTL                                  = 0x08012C60ull;
//DUPS: 08012860, 08012C60,

static const uint64_t P9A_OMIC_SCOM_PPE_CNTL                                  = 0x07012460ull;

static const uint64_t P9A_OMIC_0_SCOM_PPE_CNTL                                = 0x07012460ull;

static const uint64_t P9A_OMIC_1_SCOM_PPE_CNTL                                = 0x07012860ull;

static const uint64_t P9A_OMIC_3_SCOM_PPE_CNTL                                = 0x08012460ull;

static const uint64_t P9A_OMIC_4_SCOM_PPE_CNTL                                = 0x08012860ull;


static const uint64_t P9A_MC_SCOM_PPE_FLAGS                                   = 0x07012C63ull;
//DUPS: 07012863, 07012C63,
static const uint64_t P9A_MC_SCOM_PPE_FLAGS_OR                                = 0x07012C64ull;
//DUPS: 07012864, 07012C64,
static const uint64_t P9A_MC_SCOM_PPE_FLAGS_CLEAR                             = 0x07012C65ull;
//DUPS: 07012865, 07012C65,
static const uint64_t P9A_MC_0_SCOM_PPE_FLAGS                                 = 0x07012C63ull;
//DUPS: 07012863, 07012C63,
static const uint64_t P9A_MC_0_SCOM_PPE_FLAGS_OR                              = 0x07012C64ull;
//DUPS: 07012864, 07012C64,
static const uint64_t P9A_MC_0_SCOM_PPE_FLAGS_CLEAR                           = 0x07012C65ull;
//DUPS: 07012865, 07012C65,
static const uint64_t P9A_MC_1_SCOM_PPE_FLAGS                                 = 0x08012C63ull;
//DUPS: 08012863, 08012C63,
static const uint64_t P9A_MC_1_SCOM_PPE_FLAGS_OR                              = 0x08012C64ull;
//DUPS: 08012864, 08012C64,
static const uint64_t P9A_MC_1_SCOM_PPE_FLAGS_CLEAR                           = 0x08012C65ull;
//DUPS: 08012865, 08012C65,

static const uint64_t P9A_OMIC_SCOM_PPE_FLAGS                                 = 0x07012463ull;

static const uint64_t P9A_OMIC_SCOM_PPE_FLAGS_OR                              = 0x07012464ull;

static const uint64_t P9A_OMIC_SCOM_PPE_FLAGS_CLEAR                           = 0x07012465ull;

static const uint64_t P9A_OMIC_0_SCOM_PPE_FLAGS                               = 0x07012463ull;

static const uint64_t P9A_OMIC_0_SCOM_PPE_FLAGS_OR                            = 0x07012464ull;

static const uint64_t P9A_OMIC_0_SCOM_PPE_FLAGS_CLEAR                         = 0x07012465ull;

static const uint64_t P9A_OMIC_1_SCOM_PPE_FLAGS                               = 0x07012863ull;

static const uint64_t P9A_OMIC_1_SCOM_PPE_FLAGS_OR                            = 0x07012864ull;

static const uint64_t P9A_OMIC_1_SCOM_PPE_FLAGS_CLEAR                         = 0x07012865ull;

static const uint64_t P9A_OMIC_3_SCOM_PPE_FLAGS                               = 0x08012463ull;

static const uint64_t P9A_OMIC_3_SCOM_PPE_FLAGS_OR                            = 0x08012464ull;

static const uint64_t P9A_OMIC_3_SCOM_PPE_FLAGS_CLEAR                         = 0x08012465ull;

static const uint64_t P9A_OMIC_4_SCOM_PPE_FLAGS                               = 0x08012863ull;

static const uint64_t P9A_OMIC_4_SCOM_PPE_FLAGS_OR                            = 0x08012864ull;

static const uint64_t P9A_OMIC_4_SCOM_PPE_FLAGS_CLEAR                         = 0x08012865ull;


static const uint64_t P9A_MC_SCOM_PPE_WORK_REG1                               = 0x07012C61ull;
//DUPS: 07012861, 07012C61,
static const uint64_t P9A_MC_0_SCOM_PPE_WORK_REG1                             = 0x07012C61ull;
//DUPS: 07012861, 07012C61,
static const uint64_t P9A_MC_1_SCOM_PPE_WORK_REG1                             = 0x08012C61ull;
//DUPS: 08012861, 08012C61,

static const uint64_t P9A_OMIC_SCOM_PPE_WORK_REG1                             = 0x07012461ull;

static const uint64_t P9A_OMIC_0_SCOM_PPE_WORK_REG1                           = 0x07012461ull;

static const uint64_t P9A_OMIC_1_SCOM_PPE_WORK_REG1                           = 0x07012861ull;

static const uint64_t P9A_OMIC_3_SCOM_PPE_WORK_REG1                           = 0x08012461ull;

static const uint64_t P9A_OMIC_4_SCOM_PPE_WORK_REG1                           = 0x08012861ull;


static const uint64_t P9A_MC_SCOM_PPE_WORK_REG2                               = 0x07012C62ull;
//DUPS: 07012862, 07012C62,
static const uint64_t P9A_MC_0_SCOM_PPE_WORK_REG2                             = 0x07012C62ull;
//DUPS: 07012862, 07012C62,
static const uint64_t P9A_MC_1_SCOM_PPE_WORK_REG2                             = 0x08012C62ull;
//DUPS: 08012862, 08012C62,

static const uint64_t P9A_OMIC_SCOM_PPE_WORK_REG2                             = 0x07012462ull;

static const uint64_t P9A_OMIC_0_SCOM_PPE_WORK_REG2                           = 0x07012462ull;

static const uint64_t P9A_OMIC_1_SCOM_PPE_WORK_REG2                           = 0x07012862ull;

static const uint64_t P9A_OMIC_3_SCOM_PPE_WORK_REG2                           = 0x08012462ull;

static const uint64_t P9A_OMIC_4_SCOM_PPE_WORK_REG2                           = 0x08012862ull;


static const uint64_t P9A_MC_SKITTER_CLKSRC_REG                               = 0x07050016ull;

static const uint64_t P9A_MC_0_SKITTER_CLKSRC_REG                             = 0x07050016ull;

static const uint64_t P9A_MC_1_SKITTER_CLKSRC_REG                             = 0x08050016ull;


static const uint64_t P9A_MC_SKITTER_DATA0                                    = 0x07050019ull;

static const uint64_t P9A_MC_0_SKITTER_DATA0                                  = 0x07050019ull;

static const uint64_t P9A_MC_1_SKITTER_DATA0                                  = 0x08050019ull;


static const uint64_t P9A_MC_SKITTER_DATA1                                    = 0x0705001Aull;

static const uint64_t P9A_MC_0_SKITTER_DATA1                                  = 0x0705001Aull;

static const uint64_t P9A_MC_1_SKITTER_DATA1                                  = 0x0805001Aull;


static const uint64_t P9A_MC_SKITTER_DATA2                                    = 0x0705001Bull;

static const uint64_t P9A_MC_0_SKITTER_DATA2                                  = 0x0705001Bull;

static const uint64_t P9A_MC_1_SKITTER_DATA2                                  = 0x0805001Bull;


static const uint64_t P9A_MC_SKITTER_FORCE_REG                                = 0x07050014ull;

static const uint64_t P9A_MC_0_SKITTER_FORCE_REG                              = 0x07050014ull;

static const uint64_t P9A_MC_1_SKITTER_FORCE_REG                              = 0x08050014ull;


static const uint64_t P9A_MC_SKITTER_MODE_REG                                 = 0x07050010ull;

static const uint64_t P9A_MC_0_SKITTER_MODE_REG                               = 0x07050010ull;

static const uint64_t P9A_MC_1_SKITTER_MODE_REG                               = 0x08050010ull;


static const uint64_t P9A_MC_SLAVE_CONFIG_REG                                 = 0x070F001Eull;

static const uint64_t P9A_MC_0_SLAVE_CONFIG_REG                               = 0x070F001Eull;

static const uint64_t P9A_MC_1_SLAVE_CONFIG_REG                               = 0x080F001Eull;


static const uint64_t P9A_MC_SPARE                                            = 0x0701230Full;

static const uint64_t P9A_MC_0_SPARE                                          = 0x0701230Full;

static const uint64_t P9A_MC_1_SPARE                                          = 0x0801230Full;


static const uint64_t P9A_MC_SPARE_MODE_PB                                    = 0x800F34000701103Full;
//DUPS: 800F34000701143F, 800F34000701183F,
static const uint64_t P9A_MC_0_SPARE_MODE_PB                                  = 0x800F34000701103Full;
//DUPS: 800F34000701143F, 800F34000701183F,
static const uint64_t P9A_MC_1_SPARE_MODE_PB                                  = 0x800F34000801103Full;
//DUPS: 800F34000801143F, 800F34000801183F,

static const uint64_t P9A_OMIC_SPARE_MODE_PB                                  = 0x800F34000701103Full;

static const uint64_t P9A_OMIC_0_SPARE_MODE_PB                                = 0x800F34000701103Full;

static const uint64_t P9A_OMIC_1_SPARE_MODE_PB                                = 0x800F34000701143Full;

static const uint64_t P9A_OMIC_2_SPARE_MODE_PB                                = 0x800F34000701183Full;

static const uint64_t P9A_OMIC_3_SPARE_MODE_PB                                = 0x800F34000801103Full;

static const uint64_t P9A_OMIC_4_SPARE_MODE_PB                                = 0x800F34000801143Full;

static const uint64_t P9A_OMIC_5_SPARE_MODE_PB                                = 0x800F34000801183Full;


static const uint64_t P9A_MC_SPATTN_SCOM                                      = 0x07040004ull;

static const uint64_t P9A_MC_SPATTN_SCOM1                                     = 0x07040005ull;

static const uint64_t P9A_MC_SPATTN_SCOM2                                     = 0x07040006ull;

static const uint64_t P9A_MC_0_SPATTN_SCOM                                    = 0x07040004ull;

static const uint64_t P9A_MC_0_SPATTN_SCOM1                                   = 0x07040005ull;

static const uint64_t P9A_MC_0_SPATTN_SCOM2                                   = 0x07040006ull;

static const uint64_t P9A_MC_1_SPATTN_SCOM                                    = 0x08040004ull;

static const uint64_t P9A_MC_1_SPATTN_SCOM1                                   = 0x08040005ull;

static const uint64_t P9A_MC_1_SPATTN_SCOM2                                   = 0x08040006ull;


static const uint64_t P9A_MC_SPA_MASK                                         = 0x07040007ull;

static const uint64_t P9A_MC_0_SPA_MASK                                       = 0x07040007ull;

static const uint64_t P9A_MC_1_SPA_MASK                                       = 0x08040007ull;


static const uint64_t P9A_MC_SUM_MASK_REG                                     = 0x07040017ull;

static const uint64_t P9A_MC_0_SUM_MASK_REG                                   = 0x07040017ull;

static const uint64_t P9A_MC_1_SUM_MASK_REG                                   = 0x08040017ull;


static const uint64_t P9A_MC_SYNC_CONFIG                                      = 0x07030000ull;

static const uint64_t P9A_MC_0_SYNC_CONFIG                                    = 0x07030000ull;

static const uint64_t P9A_MC_1_SYNC_CONFIG                                    = 0x08030000ull;


static const uint64_t P9A_MC_TCMC01_TRA0_TR0_TRACE_HI_DATA_REG                = 0x07010400ull;

static const uint64_t P9A_MC_0_TCMC01_TRA0_TR0_TRACE_HI_DATA_REG              = 0x07010400ull;


static const uint64_t P9A_MC_TCMC01_TRA0_TR0_TRACE_LO_DATA_REG                = 0x07010401ull;

static const uint64_t P9A_MC_0_TCMC01_TRA0_TR0_TRACE_LO_DATA_REG              = 0x07010401ull;


static const uint64_t P9A_MC_TCMC01_TRA0_TR0_TRACE_TRCTRL_CONFIG              = 0x07010402ull;

static const uint64_t P9A_MC_0_TCMC01_TRA0_TR0_TRACE_TRCTRL_CONFIG            = 0x07010402ull;


static const uint64_t P9A_MC_TCMC01_TRA0_TR0_TRACE_TRDATA_CONFIG_0            = 0x07010403ull;

static const uint64_t P9A_MC_0_TCMC01_TRA0_TR0_TRACE_TRDATA_CONFIG_0          = 0x07010403ull;


static const uint64_t P9A_MC_TCMC01_TRA0_TR0_TRACE_TRDATA_CONFIG_1            = 0x07010404ull;

static const uint64_t P9A_MC_0_TCMC01_TRA0_TR0_TRACE_TRDATA_CONFIG_1          = 0x07010404ull;


static const uint64_t P9A_MC_TCMC01_TRA0_TR0_TRACE_TRDATA_CONFIG_2            = 0x07010405ull;

static const uint64_t P9A_MC_0_TCMC01_TRA0_TR0_TRACE_TRDATA_CONFIG_2          = 0x07010405ull;


static const uint64_t P9A_MC_TCMC01_TRA0_TR0_TRACE_TRDATA_CONFIG_3            = 0x07010406ull;

static const uint64_t P9A_MC_0_TCMC01_TRA0_TR0_TRACE_TRDATA_CONFIG_3          = 0x07010406ull;


static const uint64_t P9A_MC_TCMC01_TRA0_TR0_TRACE_TRDATA_CONFIG_4            = 0x07010407ull;

static const uint64_t P9A_MC_0_TCMC01_TRA0_TR0_TRACE_TRDATA_CONFIG_4          = 0x07010407ull;


static const uint64_t P9A_MC_TCMC01_TRA0_TR0_TRACE_TRDATA_CONFIG_5            = 0x07010408ull;

static const uint64_t P9A_MC_0_TCMC01_TRA0_TR0_TRACE_TRDATA_CONFIG_5          = 0x07010408ull;


static const uint64_t P9A_MC_TCMC01_TRA0_TR0_TRACE_TRDATA_CONFIG_9            = 0x07010409ull;

static const uint64_t P9A_MC_0_TCMC01_TRA0_TR0_TRACE_TRDATA_CONFIG_9          = 0x07010409ull;


static const uint64_t P9A_MC_TCMC01_TRA1_TR0_TRACE_HI_DATA_REG                = 0x07010440ull;

static const uint64_t P9A_MC_0_TCMC01_TRA1_TR0_TRACE_HI_DATA_REG              = 0x07010440ull;


static const uint64_t P9A_MC_TCMC01_TRA1_TR0_TRACE_LO_DATA_REG                = 0x07010441ull;

static const uint64_t P9A_MC_0_TCMC01_TRA1_TR0_TRACE_LO_DATA_REG              = 0x07010441ull;


static const uint64_t P9A_MC_TCMC01_TRA1_TR0_TRACE_TRCTRL_CONFIG              = 0x07010442ull;

static const uint64_t P9A_MC_0_TCMC01_TRA1_TR0_TRACE_TRCTRL_CONFIG            = 0x07010442ull;


static const uint64_t P9A_MC_TCMC01_TRA1_TR0_TRACE_TRDATA_CONFIG_0            = 0x07010443ull;

static const uint64_t P9A_MC_0_TCMC01_TRA1_TR0_TRACE_TRDATA_CONFIG_0          = 0x07010443ull;


static const uint64_t P9A_MC_TCMC01_TRA1_TR0_TRACE_TRDATA_CONFIG_1            = 0x07010444ull;

static const uint64_t P9A_MC_0_TCMC01_TRA1_TR0_TRACE_TRDATA_CONFIG_1          = 0x07010444ull;


static const uint64_t P9A_MC_TCMC01_TRA1_TR0_TRACE_TRDATA_CONFIG_2            = 0x07010445ull;

static const uint64_t P9A_MC_0_TCMC01_TRA1_TR0_TRACE_TRDATA_CONFIG_2          = 0x07010445ull;


static const uint64_t P9A_MC_TCMC01_TRA1_TR0_TRACE_TRDATA_CONFIG_3            = 0x07010446ull;

static const uint64_t P9A_MC_0_TCMC01_TRA1_TR0_TRACE_TRDATA_CONFIG_3          = 0x07010446ull;


static const uint64_t P9A_MC_TCMC01_TRA1_TR0_TRACE_TRDATA_CONFIG_4            = 0x07010447ull;

static const uint64_t P9A_MC_0_TCMC01_TRA1_TR0_TRACE_TRDATA_CONFIG_4          = 0x07010447ull;


static const uint64_t P9A_MC_TCMC01_TRA1_TR0_TRACE_TRDATA_CONFIG_5            = 0x07010448ull;

static const uint64_t P9A_MC_0_TCMC01_TRA1_TR0_TRACE_TRDATA_CONFIG_5          = 0x07010448ull;


static const uint64_t P9A_MC_TCMC01_TRA1_TR0_TRACE_TRDATA_CONFIG_9            = 0x07010449ull;

static const uint64_t P9A_MC_0_TCMC01_TRA1_TR0_TRACE_TRDATA_CONFIG_9          = 0x07010449ull;


static const uint64_t P9A_MC_TCMC01_TRA2_TR0_TRACE_HI_DATA_REG                = 0x07010480ull;

static const uint64_t P9A_MC_0_TCMC01_TRA2_TR0_TRACE_HI_DATA_REG              = 0x07010480ull;


static const uint64_t P9A_MC_TCMC01_TRA2_TR0_TRACE_LO_DATA_REG                = 0x07010481ull;

static const uint64_t P9A_MC_0_TCMC01_TRA2_TR0_TRACE_LO_DATA_REG              = 0x07010481ull;


static const uint64_t P9A_MC_TCMC01_TRA2_TR0_TRACE_TRCTRL_CONFIG              = 0x07010482ull;

static const uint64_t P9A_MC_0_TCMC01_TRA2_TR0_TRACE_TRCTRL_CONFIG            = 0x07010482ull;


static const uint64_t P9A_MC_TCMC01_TRA2_TR0_TRACE_TRDATA_CONFIG_0            = 0x07010483ull;

static const uint64_t P9A_MC_0_TCMC01_TRA2_TR0_TRACE_TRDATA_CONFIG_0          = 0x07010483ull;


static const uint64_t P9A_MC_TCMC01_TRA2_TR0_TRACE_TRDATA_CONFIG_1            = 0x07010484ull;

static const uint64_t P9A_MC_0_TCMC01_TRA2_TR0_TRACE_TRDATA_CONFIG_1          = 0x07010484ull;


static const uint64_t P9A_MC_TCMC01_TRA2_TR0_TRACE_TRDATA_CONFIG_2            = 0x07010485ull;

static const uint64_t P9A_MC_0_TCMC01_TRA2_TR0_TRACE_TRDATA_CONFIG_2          = 0x07010485ull;


static const uint64_t P9A_MC_TCMC01_TRA2_TR0_TRACE_TRDATA_CONFIG_3            = 0x07010486ull;

static const uint64_t P9A_MC_0_TCMC01_TRA2_TR0_TRACE_TRDATA_CONFIG_3          = 0x07010486ull;


static const uint64_t P9A_MC_TCMC01_TRA2_TR0_TRACE_TRDATA_CONFIG_4            = 0x07010487ull;

static const uint64_t P9A_MC_0_TCMC01_TRA2_TR0_TRACE_TRDATA_CONFIG_4          = 0x07010487ull;


static const uint64_t P9A_MC_TCMC01_TRA2_TR0_TRACE_TRDATA_CONFIG_5            = 0x07010488ull;

static const uint64_t P9A_MC_0_TCMC01_TRA2_TR0_TRACE_TRDATA_CONFIG_5          = 0x07010488ull;


static const uint64_t P9A_MC_TCMC01_TRA2_TR0_TRACE_TRDATA_CONFIG_9            = 0x07010489ull;

static const uint64_t P9A_MC_0_TCMC01_TRA2_TR0_TRACE_TRDATA_CONFIG_9          = 0x07010489ull;


static const uint64_t P9A_MC_1_TCMC23_TRA0_TR0_TRACE_HI_DATA_REG              = 0x08010400ull;


static const uint64_t P9A_MC_1_TCMC23_TRA0_TR0_TRACE_LO_DATA_REG              = 0x08010401ull;


static const uint64_t P9A_MC_1_TCMC23_TRA0_TR0_TRACE_TRCTRL_CONFIG            = 0x08010402ull;


static const uint64_t P9A_MC_1_TCMC23_TRA0_TR0_TRACE_TRDATA_CONFIG_0          = 0x08010403ull;


static const uint64_t P9A_MC_1_TCMC23_TRA0_TR0_TRACE_TRDATA_CONFIG_1          = 0x08010404ull;


static const uint64_t P9A_MC_1_TCMC23_TRA0_TR0_TRACE_TRDATA_CONFIG_2          = 0x08010405ull;


static const uint64_t P9A_MC_1_TCMC23_TRA0_TR0_TRACE_TRDATA_CONFIG_3          = 0x08010406ull;


static const uint64_t P9A_MC_1_TCMC23_TRA0_TR0_TRACE_TRDATA_CONFIG_4          = 0x08010407ull;


static const uint64_t P9A_MC_1_TCMC23_TRA0_TR0_TRACE_TRDATA_CONFIG_5          = 0x08010408ull;


static const uint64_t P9A_MC_1_TCMC23_TRA0_TR0_TRACE_TRDATA_CONFIG_9          = 0x08010409ull;


static const uint64_t P9A_MC_1_TCMC23_TRA1_TR0_TRACE_HI_DATA_REG              = 0x08010440ull;


static const uint64_t P9A_MC_1_TCMC23_TRA1_TR0_TRACE_LO_DATA_REG              = 0x08010441ull;


static const uint64_t P9A_MC_1_TCMC23_TRA1_TR0_TRACE_TRCTRL_CONFIG            = 0x08010442ull;


static const uint64_t P9A_MC_1_TCMC23_TRA1_TR0_TRACE_TRDATA_CONFIG_0          = 0x08010443ull;


static const uint64_t P9A_MC_1_TCMC23_TRA1_TR0_TRACE_TRDATA_CONFIG_1          = 0x08010444ull;


static const uint64_t P9A_MC_1_TCMC23_TRA1_TR0_TRACE_TRDATA_CONFIG_2          = 0x08010445ull;


static const uint64_t P9A_MC_1_TCMC23_TRA1_TR0_TRACE_TRDATA_CONFIG_3          = 0x08010446ull;


static const uint64_t P9A_MC_1_TCMC23_TRA1_TR0_TRACE_TRDATA_CONFIG_4          = 0x08010447ull;


static const uint64_t P9A_MC_1_TCMC23_TRA1_TR0_TRACE_TRDATA_CONFIG_5          = 0x08010448ull;


static const uint64_t P9A_MC_1_TCMC23_TRA1_TR0_TRACE_TRDATA_CONFIG_9          = 0x08010449ull;


static const uint64_t P9A_MC_1_TCMC23_TRA2_TR0_TRACE_HI_DATA_REG              = 0x08010480ull;


static const uint64_t P9A_MC_1_TCMC23_TRA2_TR0_TRACE_LO_DATA_REG              = 0x08010481ull;


static const uint64_t P9A_MC_1_TCMC23_TRA2_TR0_TRACE_TRCTRL_CONFIG            = 0x08010482ull;


static const uint64_t P9A_MC_1_TCMC23_TRA2_TR0_TRACE_TRDATA_CONFIG_0          = 0x08010483ull;


static const uint64_t P9A_MC_1_TCMC23_TRA2_TR0_TRACE_TRDATA_CONFIG_1          = 0x08010484ull;


static const uint64_t P9A_MC_1_TCMC23_TRA2_TR0_TRACE_TRDATA_CONFIG_2          = 0x08010485ull;


static const uint64_t P9A_MC_1_TCMC23_TRA2_TR0_TRACE_TRDATA_CONFIG_3          = 0x08010486ull;


static const uint64_t P9A_MC_1_TCMC23_TRA2_TR0_TRACE_TRDATA_CONFIG_4          = 0x08010487ull;


static const uint64_t P9A_MC_1_TCMC23_TRA2_TR0_TRACE_TRDATA_CONFIG_5          = 0x08010488ull;


static const uint64_t P9A_MC_1_TCMC23_TRA2_TR0_TRACE_TRDATA_CONFIG_9          = 0x08010489ull;


static const uint64_t P9A_MC_THERM_MODE_REG                                   = 0x0705000Full;

static const uint64_t P9A_MC_0_THERM_MODE_REG                                 = 0x0705000Full;

static const uint64_t P9A_MC_1_THERM_MODE_REG                                 = 0x0805000Full;


static const uint64_t P9A_MC_TIMEOUT_REG                                      = 0x070F0010ull;

static const uint64_t P9A_MC_0_TIMEOUT_REG                                    = 0x070F0010ull;

static const uint64_t P9A_MC_1_TIMEOUT_REG                                    = 0x080F0010ull;


static const uint64_t P9A_MC_TIMESTAMP_COUNTER_READ                           = 0x0705001Cull;

static const uint64_t P9A_MC_0_TIMESTAMP_COUNTER_READ                         = 0x0705001Cull;

static const uint64_t P9A_MC_1_TIMESTAMP_COUNTER_READ                         = 0x0805001Cull;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE0_TX_CNTL1G_PL                 = 0x800414000701103Full;
//DUPS: 800414000701143F, 800414000701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE0_TX_CNTL1G_PL               = 0x800414000701103Full;
//DUPS: 800414000701143F, 800414000701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE0_TX_CNTL1G_PL               = 0x800414000801103Full;
//DUPS: 800414000801143F, 800414000801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE0_TX_CNTL1G_PL                = 0x800414000701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE0_TX_CNTL1G_PL               = 0x800414000701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE0_TX_CNTL1G_PL             = 0x800414000701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE0_TX_CNTL1G_PL             = 0x800414000701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE0_TX_CNTL1G_PL             = 0x800414000701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE0_TX_CNTL1G_PL             = 0x800414000801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE0_TX_CNTL1G_PL             = 0x800414000801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE0_TX_CNTL1G_PL             = 0x800414000801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE0_TX_CNTL1G_PL              = 0x800414000701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE0_TX_CNTL1G_PL              = 0x800414000701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE0_TX_CNTL1G_PL              = 0x800414000701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE0_TX_CNTL1G_PL              = 0x800414000801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE0_TX_CNTL1G_PL             = 0x800414000801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE0_TX_CNTL1G_PL             = 0x800414000801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE0_TX_CNTL2_O_PL                = 0x80044C000701103Full;
//DUPS: 80044C000701143F, 80044C000701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE0_TX_CNTL2_O_PL              = 0x80044C000701103Full;
//DUPS: 80044C000701143F, 80044C000701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE0_TX_CNTL2_O_PL              = 0x80044C000801103Full;
//DUPS: 80044C000801143F, 80044C000801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE0_TX_CNTL2_O_PL               = 0x80044C000701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE0_TX_CNTL2_O_PL              = 0x80044C000701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE0_TX_CNTL2_O_PL            = 0x80044C000701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE0_TX_CNTL2_O_PL            = 0x80044C000701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE0_TX_CNTL2_O_PL            = 0x80044C000701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE0_TX_CNTL2_O_PL            = 0x80044C000801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE0_TX_CNTL2_O_PL            = 0x80044C000801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE0_TX_CNTL2_O_PL            = 0x80044C000801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE0_TX_CNTL2_O_PL             = 0x80044C000701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE0_TX_CNTL2_O_PL             = 0x80044C000701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE0_TX_CNTL2_O_PL             = 0x80044C000701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE0_TX_CNTL2_O_PL             = 0x80044C000801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE0_TX_CNTL2_O_PL            = 0x80044C000801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE0_TX_CNTL2_O_PL            = 0x80044C000801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE0_TX_CNTL3_EO_PL               = 0x800454000701103Full;
//DUPS: 800454000701143F, 800454000701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE0_TX_CNTL3_EO_PL             = 0x800454000701103Full;
//DUPS: 800454000701143F, 800454000701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE0_TX_CNTL3_EO_PL             = 0x800454000801103Full;
//DUPS: 800454000801143F, 800454000801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE0_TX_CNTL3_EO_PL              = 0x800454000701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE0_TX_CNTL3_EO_PL             = 0x800454000701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE0_TX_CNTL3_EO_PL           = 0x800454000701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE0_TX_CNTL3_EO_PL           = 0x800454000701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE0_TX_CNTL3_EO_PL           = 0x800454000701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE0_TX_CNTL3_EO_PL           = 0x800454000801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE0_TX_CNTL3_EO_PL           = 0x800454000801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE0_TX_CNTL3_EO_PL           = 0x800454000801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE0_TX_CNTL3_EO_PL            = 0x800454000701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE0_TX_CNTL3_EO_PL            = 0x800454000701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE0_TX_CNTL3_EO_PL            = 0x800454000701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE0_TX_CNTL3_EO_PL            = 0x800454000801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE0_TX_CNTL3_EO_PL           = 0x800454000801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE0_TX_CNTL3_EO_PL           = 0x800454000801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE0_TX_FIR_ERROR_INJECT_PL       = 0x800434000701103Full;
//DUPS: 800434000701143F, 800434000701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE0_TX_FIR_ERROR_INJECT_PL     = 0x800434000701103Full;
//DUPS: 800434000701143F, 800434000701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE0_TX_FIR_ERROR_INJECT_PL     = 0x800434000801103Full;
//DUPS: 800434000801143F, 800434000801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE0_TX_FIR_ERROR_INJECT_PL      = 0x800434000701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE0_TX_FIR_ERROR_INJECT_PL     = 0x800434000701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x800434000701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x800434000701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x800434000701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x800434000801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x800434000801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x800434000801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE0_TX_FIR_ERROR_INJECT_PL    = 0x800434000701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE0_TX_FIR_ERROR_INJECT_PL    = 0x800434000701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE0_TX_FIR_ERROR_INJECT_PL    = 0x800434000701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE0_TX_FIR_ERROR_INJECT_PL    = 0x800434000801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x800434000801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x800434000801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE0_TX_FIR_MASK_PL               = 0x80042C000701103Full;
//DUPS: 80042C000701143F, 80042C000701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE0_TX_FIR_MASK_PL             = 0x80042C000701103Full;
//DUPS: 80042C000701143F, 80042C000701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE0_TX_FIR_MASK_PL             = 0x80042C000801103Full;
//DUPS: 80042C000801143F, 80042C000801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE0_TX_FIR_MASK_PL              = 0x80042C000701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE0_TX_FIR_MASK_PL             = 0x80042C000701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE0_TX_FIR_MASK_PL           = 0x80042C000701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE0_TX_FIR_MASK_PL           = 0x80042C000701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE0_TX_FIR_MASK_PL           = 0x80042C000701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE0_TX_FIR_MASK_PL           = 0x80042C000801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE0_TX_FIR_MASK_PL           = 0x80042C000801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE0_TX_FIR_MASK_PL           = 0x80042C000801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE0_TX_FIR_MASK_PL            = 0x80042C000701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE0_TX_FIR_MASK_PL            = 0x80042C000701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE0_TX_FIR_MASK_PL            = 0x80042C000701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE0_TX_FIR_MASK_PL            = 0x80042C000801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE0_TX_FIR_MASK_PL           = 0x80042C000801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE0_TX_FIR_MASK_PL           = 0x80042C000801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE0_TX_FIR_PL                    = 0x800424000701103Full;
//DUPS: 800424000701143F, 800424000701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE0_TX_FIR_PL                  = 0x800424000701103Full;
//DUPS: 800424000701143F, 800424000701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE0_TX_FIR_PL                  = 0x800424000801103Full;
//DUPS: 800424000801143F, 800424000801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE0_TX_FIR_PL                   = 0x800424000701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE0_TX_FIR_PL                  = 0x800424000701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE0_TX_FIR_PL                = 0x800424000701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE0_TX_FIR_PL                = 0x800424000701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE0_TX_FIR_PL                = 0x800424000701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE0_TX_FIR_PL                = 0x800424000801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE0_TX_FIR_PL                = 0x800424000801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE0_TX_FIR_PL                = 0x800424000801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE0_TX_FIR_PL                 = 0x800424000701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE0_TX_FIR_PL                 = 0x800424000701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE0_TX_FIR_PL                 = 0x800424000701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE0_TX_FIR_PL                 = 0x800424000801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE0_TX_FIR_PL                = 0x800424000801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE0_TX_FIR_PL                = 0x800424000801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE0_TX_MODE1_PL                  = 0x800404000701103Full;
//DUPS: 800404000701143F, 800404000701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE0_TX_MODE1_PL                = 0x800404000701103Full;
//DUPS: 800404000701143F, 800404000701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE0_TX_MODE1_PL                = 0x800404000801103Full;
//DUPS: 800404000801143F, 800404000801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE0_TX_MODE1_PL                 = 0x800404000701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE0_TX_MODE1_PL                = 0x800404000701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE0_TX_MODE1_PL              = 0x800404000701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE0_TX_MODE1_PL              = 0x800404000701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE0_TX_MODE1_PL              = 0x800404000701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE0_TX_MODE1_PL              = 0x800404000801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE0_TX_MODE1_PL              = 0x800404000801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE0_TX_MODE1_PL              = 0x800404000801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE0_TX_MODE1_PL               = 0x800404000701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE0_TX_MODE1_PL               = 0x800404000701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE0_TX_MODE1_PL               = 0x800404000701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE0_TX_MODE1_PL               = 0x800404000801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE0_TX_MODE1_PL              = 0x800404000801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE0_TX_MODE1_PL              = 0x800404000801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE0_TX_MODE2_PL                  = 0x80040C000701103Full;
//DUPS: 80040C000701143F, 80040C000701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE0_TX_MODE2_PL                = 0x80040C000701103Full;
//DUPS: 80040C000701143F, 80040C000701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE0_TX_MODE2_PL                = 0x80040C000801103Full;
//DUPS: 80040C000801143F, 80040C000801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE0_TX_MODE2_PL                 = 0x80040C000701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE0_TX_MODE2_PL                = 0x80040C000701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE0_TX_MODE2_PL              = 0x80040C000701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE0_TX_MODE2_PL              = 0x80040C000701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE0_TX_MODE2_PL              = 0x80040C000701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE0_TX_MODE2_PL              = 0x80040C000801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE0_TX_MODE2_PL              = 0x80040C000801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE0_TX_MODE2_PL              = 0x80040C000801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE0_TX_MODE2_PL               = 0x80040C000701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE0_TX_MODE2_PL               = 0x80040C000701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE0_TX_MODE2_PL               = 0x80040C000701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE0_TX_MODE2_PL               = 0x80040C000801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE0_TX_MODE2_PL              = 0x80040C000801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE0_TX_MODE2_PL              = 0x80040C000801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE0_TX_STAT1_PL                  = 0x80041C000701103Full;
//DUPS: 80041C000701143F, 80041C000701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE0_TX_STAT1_PL                = 0x80041C000701103Full;
//DUPS: 80041C000701143F, 80041C000701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE0_TX_STAT1_PL                = 0x80041C000801103Full;
//DUPS: 80041C000801143F, 80041C000801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE0_TX_STAT1_PL                 = 0x80041C000701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE0_TX_STAT1_PL                = 0x80041C000701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE0_TX_STAT1_PL              = 0x80041C000701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE0_TX_STAT1_PL              = 0x80041C000701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE0_TX_STAT1_PL              = 0x80041C000701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE0_TX_STAT1_PL              = 0x80041C000801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE0_TX_STAT1_PL              = 0x80041C000801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE0_TX_STAT1_PL              = 0x80041C000801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE0_TX_STAT1_PL               = 0x80041C000701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE0_TX_STAT1_PL               = 0x80041C000701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE0_TX_STAT1_PL               = 0x80041C000701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE0_TX_STAT1_PL               = 0x80041C000801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE0_TX_STAT1_PL              = 0x80041C000801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE0_TX_STAT1_PL              = 0x80041C000801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE1_TX_CNTL1G_PL                 = 0x800414010701103Full;
//DUPS: 800414010701143F, 800414010701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE1_TX_CNTL1G_PL               = 0x800414010701103Full;
//DUPS: 800414010701143F, 800414010701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE1_TX_CNTL1G_PL               = 0x800414010801103Full;
//DUPS: 800414010801143F, 800414010801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE1_TX_CNTL1G_PL                = 0x800414010701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE1_TX_CNTL1G_PL               = 0x800414010701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE1_TX_CNTL1G_PL             = 0x800414010701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE1_TX_CNTL1G_PL             = 0x800414010701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE1_TX_CNTL1G_PL             = 0x800414010701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE1_TX_CNTL1G_PL             = 0x800414010801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE1_TX_CNTL1G_PL             = 0x800414010801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE1_TX_CNTL1G_PL             = 0x800414010801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE1_TX_CNTL1G_PL              = 0x800414010701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE1_TX_CNTL1G_PL              = 0x800414010701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE1_TX_CNTL1G_PL              = 0x800414010701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE1_TX_CNTL1G_PL              = 0x800414010801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE1_TX_CNTL1G_PL             = 0x800414010801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE1_TX_CNTL1G_PL             = 0x800414010801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE1_TX_CNTL2_O_PL                = 0x80044C010701103Full;
//DUPS: 80044C010701143F, 80044C010701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE1_TX_CNTL2_O_PL              = 0x80044C010701103Full;
//DUPS: 80044C010701143F, 80044C010701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE1_TX_CNTL2_O_PL              = 0x80044C010801103Full;
//DUPS: 80044C010801143F, 80044C010801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE1_TX_CNTL2_O_PL               = 0x80044C010701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE1_TX_CNTL2_O_PL              = 0x80044C010701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE1_TX_CNTL2_O_PL            = 0x80044C010701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE1_TX_CNTL2_O_PL            = 0x80044C010701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE1_TX_CNTL2_O_PL            = 0x80044C010701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE1_TX_CNTL2_O_PL            = 0x80044C010801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE1_TX_CNTL2_O_PL            = 0x80044C010801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE1_TX_CNTL2_O_PL            = 0x80044C010801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE1_TX_CNTL2_O_PL             = 0x80044C010701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE1_TX_CNTL2_O_PL             = 0x80044C010701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE1_TX_CNTL2_O_PL             = 0x80044C010701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE1_TX_CNTL2_O_PL             = 0x80044C010801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE1_TX_CNTL2_O_PL            = 0x80044C010801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE1_TX_CNTL2_O_PL            = 0x80044C010801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE1_TX_CNTL3_EO_PL               = 0x800454010701103Full;
//DUPS: 800454010701143F, 800454010701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE1_TX_CNTL3_EO_PL             = 0x800454010701103Full;
//DUPS: 800454010701143F, 800454010701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE1_TX_CNTL3_EO_PL             = 0x800454010801103Full;
//DUPS: 800454010801143F, 800454010801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE1_TX_CNTL3_EO_PL              = 0x800454010701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE1_TX_CNTL3_EO_PL             = 0x800454010701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE1_TX_CNTL3_EO_PL           = 0x800454010701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE1_TX_CNTL3_EO_PL           = 0x800454010701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE1_TX_CNTL3_EO_PL           = 0x800454010701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE1_TX_CNTL3_EO_PL           = 0x800454010801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE1_TX_CNTL3_EO_PL           = 0x800454010801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE1_TX_CNTL3_EO_PL           = 0x800454010801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE1_TX_CNTL3_EO_PL            = 0x800454010701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE1_TX_CNTL3_EO_PL            = 0x800454010701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE1_TX_CNTL3_EO_PL            = 0x800454010701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE1_TX_CNTL3_EO_PL            = 0x800454010801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE1_TX_CNTL3_EO_PL           = 0x800454010801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE1_TX_CNTL3_EO_PL           = 0x800454010801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE1_TX_FIR_ERROR_INJECT_PL       = 0x800434010701103Full;
//DUPS: 800434010701143F, 800434010701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE1_TX_FIR_ERROR_INJECT_PL     = 0x800434010701103Full;
//DUPS: 800434010701143F, 800434010701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE1_TX_FIR_ERROR_INJECT_PL     = 0x800434010801103Full;
//DUPS: 800434010801143F, 800434010801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE1_TX_FIR_ERROR_INJECT_PL      = 0x800434010701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE1_TX_FIR_ERROR_INJECT_PL     = 0x800434010701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x800434010701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x800434010701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x800434010701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x800434010801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x800434010801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x800434010801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE1_TX_FIR_ERROR_INJECT_PL    = 0x800434010701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE1_TX_FIR_ERROR_INJECT_PL    = 0x800434010701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE1_TX_FIR_ERROR_INJECT_PL    = 0x800434010701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE1_TX_FIR_ERROR_INJECT_PL    = 0x800434010801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x800434010801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x800434010801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE1_TX_FIR_MASK_PL               = 0x80042C010701103Full;
//DUPS: 80042C010701143F, 80042C010701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE1_TX_FIR_MASK_PL             = 0x80042C010701103Full;
//DUPS: 80042C010701143F, 80042C010701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE1_TX_FIR_MASK_PL             = 0x80042C010801103Full;
//DUPS: 80042C010801143F, 80042C010801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE1_TX_FIR_MASK_PL              = 0x80042C010701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE1_TX_FIR_MASK_PL             = 0x80042C010701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE1_TX_FIR_MASK_PL           = 0x80042C010701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE1_TX_FIR_MASK_PL           = 0x80042C010701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE1_TX_FIR_MASK_PL           = 0x80042C010701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE1_TX_FIR_MASK_PL           = 0x80042C010801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE1_TX_FIR_MASK_PL           = 0x80042C010801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE1_TX_FIR_MASK_PL           = 0x80042C010801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE1_TX_FIR_MASK_PL            = 0x80042C010701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE1_TX_FIR_MASK_PL            = 0x80042C010701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE1_TX_FIR_MASK_PL            = 0x80042C010701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE1_TX_FIR_MASK_PL            = 0x80042C010801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE1_TX_FIR_MASK_PL           = 0x80042C010801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE1_TX_FIR_MASK_PL           = 0x80042C010801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE1_TX_FIR_PL                    = 0x800424010701103Full;
//DUPS: 800424010701143F, 800424010701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE1_TX_FIR_PL                  = 0x800424010701103Full;
//DUPS: 800424010701143F, 800424010701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE1_TX_FIR_PL                  = 0x800424010801103Full;
//DUPS: 800424010801143F, 800424010801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE1_TX_FIR_PL                   = 0x800424010701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE1_TX_FIR_PL                  = 0x800424010701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE1_TX_FIR_PL                = 0x800424010701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE1_TX_FIR_PL                = 0x800424010701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE1_TX_FIR_PL                = 0x800424010701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE1_TX_FIR_PL                = 0x800424010801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE1_TX_FIR_PL                = 0x800424010801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE1_TX_FIR_PL                = 0x800424010801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE1_TX_FIR_PL                 = 0x800424010701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE1_TX_FIR_PL                 = 0x800424010701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE1_TX_FIR_PL                 = 0x800424010701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE1_TX_FIR_PL                 = 0x800424010801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE1_TX_FIR_PL                = 0x800424010801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE1_TX_FIR_PL                = 0x800424010801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE1_TX_MODE1_PL                  = 0x800404010701103Full;
//DUPS: 800404010701143F, 800404010701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE1_TX_MODE1_PL                = 0x800404010701103Full;
//DUPS: 800404010701143F, 800404010701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE1_TX_MODE1_PL                = 0x800404010801103Full;
//DUPS: 800404010801143F, 800404010801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE1_TX_MODE1_PL                 = 0x800404010701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE1_TX_MODE1_PL                = 0x800404010701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE1_TX_MODE1_PL              = 0x800404010701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE1_TX_MODE1_PL              = 0x800404010701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE1_TX_MODE1_PL              = 0x800404010701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE1_TX_MODE1_PL              = 0x800404010801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE1_TX_MODE1_PL              = 0x800404010801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE1_TX_MODE1_PL              = 0x800404010801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE1_TX_MODE1_PL               = 0x800404010701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE1_TX_MODE1_PL               = 0x800404010701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE1_TX_MODE1_PL               = 0x800404010701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE1_TX_MODE1_PL               = 0x800404010801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE1_TX_MODE1_PL              = 0x800404010801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE1_TX_MODE1_PL              = 0x800404010801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE1_TX_MODE2_PL                  = 0x80040C010701103Full;
//DUPS: 80040C010701143F, 80040C010701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE1_TX_MODE2_PL                = 0x80040C010701103Full;
//DUPS: 80040C010701143F, 80040C010701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE1_TX_MODE2_PL                = 0x80040C010801103Full;
//DUPS: 80040C010801143F, 80040C010801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE1_TX_MODE2_PL                 = 0x80040C010701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE1_TX_MODE2_PL                = 0x80040C010701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE1_TX_MODE2_PL              = 0x80040C010701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE1_TX_MODE2_PL              = 0x80040C010701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE1_TX_MODE2_PL              = 0x80040C010701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE1_TX_MODE2_PL              = 0x80040C010801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE1_TX_MODE2_PL              = 0x80040C010801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE1_TX_MODE2_PL              = 0x80040C010801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE1_TX_MODE2_PL               = 0x80040C010701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE1_TX_MODE2_PL               = 0x80040C010701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE1_TX_MODE2_PL               = 0x80040C010701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE1_TX_MODE2_PL               = 0x80040C010801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE1_TX_MODE2_PL              = 0x80040C010801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE1_TX_MODE2_PL              = 0x80040C010801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE1_TX_STAT1_PL                  = 0x80041C010701103Full;
//DUPS: 80041C010701143F, 80041C010701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE1_TX_STAT1_PL                = 0x80041C010701103Full;
//DUPS: 80041C010701143F, 80041C010701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE1_TX_STAT1_PL                = 0x80041C010801103Full;
//DUPS: 80041C010801143F, 80041C010801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE1_TX_STAT1_PL                 = 0x80041C010701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE1_TX_STAT1_PL                = 0x80041C010701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE1_TX_STAT1_PL              = 0x80041C010701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE1_TX_STAT1_PL              = 0x80041C010701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE1_TX_STAT1_PL              = 0x80041C010701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE1_TX_STAT1_PL              = 0x80041C010801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE1_TX_STAT1_PL              = 0x80041C010801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE1_TX_STAT1_PL              = 0x80041C010801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE1_TX_STAT1_PL               = 0x80041C010701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE1_TX_STAT1_PL               = 0x80041C010701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE1_TX_STAT1_PL               = 0x80041C010701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE1_TX_STAT1_PL               = 0x80041C010801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE1_TX_STAT1_PL              = 0x80041C010801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE1_TX_STAT1_PL              = 0x80041C010801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE2_TX_CNTL1G_PL                 = 0x800414020701103Full;
//DUPS: 800414020701143F, 800414020701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE2_TX_CNTL1G_PL               = 0x800414020701103Full;
//DUPS: 800414020701143F, 800414020701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE2_TX_CNTL1G_PL               = 0x800414020801103Full;
//DUPS: 800414020801143F, 800414020801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE2_TX_CNTL1G_PL                = 0x800414020701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE2_TX_CNTL1G_PL               = 0x800414020701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE2_TX_CNTL1G_PL             = 0x800414020701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE2_TX_CNTL1G_PL             = 0x800414020701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE2_TX_CNTL1G_PL             = 0x800414020701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE2_TX_CNTL1G_PL             = 0x800414020801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE2_TX_CNTL1G_PL             = 0x800414020801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE2_TX_CNTL1G_PL             = 0x800414020801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE2_TX_CNTL1G_PL              = 0x800414020701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE2_TX_CNTL1G_PL              = 0x800414020701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE2_TX_CNTL1G_PL              = 0x800414020701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE2_TX_CNTL1G_PL              = 0x800414020801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE2_TX_CNTL1G_PL             = 0x800414020801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE2_TX_CNTL1G_PL             = 0x800414020801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE2_TX_CNTL2_O_PL                = 0x80044C020701103Full;
//DUPS: 80044C020701143F, 80044C020701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE2_TX_CNTL2_O_PL              = 0x80044C020701103Full;
//DUPS: 80044C020701143F, 80044C020701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE2_TX_CNTL2_O_PL              = 0x80044C020801103Full;
//DUPS: 80044C020801143F, 80044C020801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE2_TX_CNTL2_O_PL               = 0x80044C020701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE2_TX_CNTL2_O_PL              = 0x80044C020701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE2_TX_CNTL2_O_PL            = 0x80044C020701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE2_TX_CNTL2_O_PL            = 0x80044C020701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE2_TX_CNTL2_O_PL            = 0x80044C020701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE2_TX_CNTL2_O_PL            = 0x80044C020801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE2_TX_CNTL2_O_PL            = 0x80044C020801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE2_TX_CNTL2_O_PL            = 0x80044C020801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE2_TX_CNTL2_O_PL             = 0x80044C020701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE2_TX_CNTL2_O_PL             = 0x80044C020701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE2_TX_CNTL2_O_PL             = 0x80044C020701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE2_TX_CNTL2_O_PL             = 0x80044C020801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE2_TX_CNTL2_O_PL            = 0x80044C020801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE2_TX_CNTL2_O_PL            = 0x80044C020801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE2_TX_CNTL3_EO_PL               = 0x800454020701103Full;
//DUPS: 800454020701143F, 800454020701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE2_TX_CNTL3_EO_PL             = 0x800454020701103Full;
//DUPS: 800454020701143F, 800454020701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE2_TX_CNTL3_EO_PL             = 0x800454020801103Full;
//DUPS: 800454020801143F, 800454020801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE2_TX_CNTL3_EO_PL              = 0x800454020701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE2_TX_CNTL3_EO_PL             = 0x800454020701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE2_TX_CNTL3_EO_PL           = 0x800454020701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE2_TX_CNTL3_EO_PL           = 0x800454020701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE2_TX_CNTL3_EO_PL           = 0x800454020701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE2_TX_CNTL3_EO_PL           = 0x800454020801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE2_TX_CNTL3_EO_PL           = 0x800454020801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE2_TX_CNTL3_EO_PL           = 0x800454020801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE2_TX_CNTL3_EO_PL            = 0x800454020701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE2_TX_CNTL3_EO_PL            = 0x800454020701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE2_TX_CNTL3_EO_PL            = 0x800454020701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE2_TX_CNTL3_EO_PL            = 0x800454020801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE2_TX_CNTL3_EO_PL           = 0x800454020801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE2_TX_CNTL3_EO_PL           = 0x800454020801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE2_TX_FIR_ERROR_INJECT_PL       = 0x800434020701103Full;
//DUPS: 800434020701143F, 800434020701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE2_TX_FIR_ERROR_INJECT_PL     = 0x800434020701103Full;
//DUPS: 800434020701143F, 800434020701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE2_TX_FIR_ERROR_INJECT_PL     = 0x800434020801103Full;
//DUPS: 800434020801143F, 800434020801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE2_TX_FIR_ERROR_INJECT_PL      = 0x800434020701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE2_TX_FIR_ERROR_INJECT_PL     = 0x800434020701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x800434020701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x800434020701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x800434020701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x800434020801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x800434020801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x800434020801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE2_TX_FIR_ERROR_INJECT_PL    = 0x800434020701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE2_TX_FIR_ERROR_INJECT_PL    = 0x800434020701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE2_TX_FIR_ERROR_INJECT_PL    = 0x800434020701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE2_TX_FIR_ERROR_INJECT_PL    = 0x800434020801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x800434020801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x800434020801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE2_TX_FIR_MASK_PL               = 0x80042C020701103Full;
//DUPS: 80042C020701143F, 80042C020701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE2_TX_FIR_MASK_PL             = 0x80042C020701103Full;
//DUPS: 80042C020701143F, 80042C020701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE2_TX_FIR_MASK_PL             = 0x80042C020801103Full;
//DUPS: 80042C020801143F, 80042C020801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE2_TX_FIR_MASK_PL              = 0x80042C020701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE2_TX_FIR_MASK_PL             = 0x80042C020701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE2_TX_FIR_MASK_PL           = 0x80042C020701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE2_TX_FIR_MASK_PL           = 0x80042C020701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE2_TX_FIR_MASK_PL           = 0x80042C020701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE2_TX_FIR_MASK_PL           = 0x80042C020801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE2_TX_FIR_MASK_PL           = 0x80042C020801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE2_TX_FIR_MASK_PL           = 0x80042C020801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE2_TX_FIR_MASK_PL            = 0x80042C020701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE2_TX_FIR_MASK_PL            = 0x80042C020701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE2_TX_FIR_MASK_PL            = 0x80042C020701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE2_TX_FIR_MASK_PL            = 0x80042C020801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE2_TX_FIR_MASK_PL           = 0x80042C020801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE2_TX_FIR_MASK_PL           = 0x80042C020801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE2_TX_FIR_PL                    = 0x800424020701103Full;
//DUPS: 800424020701143F, 800424020701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE2_TX_FIR_PL                  = 0x800424020701103Full;
//DUPS: 800424020701143F, 800424020701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE2_TX_FIR_PL                  = 0x800424020801103Full;
//DUPS: 800424020801143F, 800424020801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE2_TX_FIR_PL                   = 0x800424020701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE2_TX_FIR_PL                  = 0x800424020701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE2_TX_FIR_PL                = 0x800424020701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE2_TX_FIR_PL                = 0x800424020701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE2_TX_FIR_PL                = 0x800424020701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE2_TX_FIR_PL                = 0x800424020801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE2_TX_FIR_PL                = 0x800424020801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE2_TX_FIR_PL                = 0x800424020801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE2_TX_FIR_PL                 = 0x800424020701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE2_TX_FIR_PL                 = 0x800424020701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE2_TX_FIR_PL                 = 0x800424020701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE2_TX_FIR_PL                 = 0x800424020801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE2_TX_FIR_PL                = 0x800424020801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE2_TX_FIR_PL                = 0x800424020801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE2_TX_MODE1_PL                  = 0x800404020701103Full;
//DUPS: 800404020701143F, 800404020701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE2_TX_MODE1_PL                = 0x800404020701103Full;
//DUPS: 800404020701143F, 800404020701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE2_TX_MODE1_PL                = 0x800404020801103Full;
//DUPS: 800404020801143F, 800404020801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE2_TX_MODE1_PL                 = 0x800404020701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE2_TX_MODE1_PL                = 0x800404020701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE2_TX_MODE1_PL              = 0x800404020701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE2_TX_MODE1_PL              = 0x800404020701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE2_TX_MODE1_PL              = 0x800404020701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE2_TX_MODE1_PL              = 0x800404020801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE2_TX_MODE1_PL              = 0x800404020801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE2_TX_MODE1_PL              = 0x800404020801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE2_TX_MODE1_PL               = 0x800404020701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE2_TX_MODE1_PL               = 0x800404020701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE2_TX_MODE1_PL               = 0x800404020701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE2_TX_MODE1_PL               = 0x800404020801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE2_TX_MODE1_PL              = 0x800404020801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE2_TX_MODE1_PL              = 0x800404020801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE2_TX_MODE2_PL                  = 0x80040C020701103Full;
//DUPS: 80040C020701143F, 80040C020701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE2_TX_MODE2_PL                = 0x80040C020701103Full;
//DUPS: 80040C020701143F, 80040C020701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE2_TX_MODE2_PL                = 0x80040C020801103Full;
//DUPS: 80040C020801143F, 80040C020801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE2_TX_MODE2_PL                 = 0x80040C020701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE2_TX_MODE2_PL                = 0x80040C020701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE2_TX_MODE2_PL              = 0x80040C020701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE2_TX_MODE2_PL              = 0x80040C020701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE2_TX_MODE2_PL              = 0x80040C020701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE2_TX_MODE2_PL              = 0x80040C020801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE2_TX_MODE2_PL              = 0x80040C020801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE2_TX_MODE2_PL              = 0x80040C020801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE2_TX_MODE2_PL               = 0x80040C020701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE2_TX_MODE2_PL               = 0x80040C020701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE2_TX_MODE2_PL               = 0x80040C020701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE2_TX_MODE2_PL               = 0x80040C020801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE2_TX_MODE2_PL              = 0x80040C020801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE2_TX_MODE2_PL              = 0x80040C020801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE2_TX_STAT1_PL                  = 0x80041C020701103Full;
//DUPS: 80041C020701143F, 80041C020701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE2_TX_STAT1_PL                = 0x80041C020701103Full;
//DUPS: 80041C020701143F, 80041C020701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE2_TX_STAT1_PL                = 0x80041C020801103Full;
//DUPS: 80041C020801143F, 80041C020801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE2_TX_STAT1_PL                 = 0x80041C020701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE2_TX_STAT1_PL                = 0x80041C020701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE2_TX_STAT1_PL              = 0x80041C020701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE2_TX_STAT1_PL              = 0x80041C020701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE2_TX_STAT1_PL              = 0x80041C020701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE2_TX_STAT1_PL              = 0x80041C020801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE2_TX_STAT1_PL              = 0x80041C020801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE2_TX_STAT1_PL              = 0x80041C020801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE2_TX_STAT1_PL               = 0x80041C020701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE2_TX_STAT1_PL               = 0x80041C020701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE2_TX_STAT1_PL               = 0x80041C020701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE2_TX_STAT1_PL               = 0x80041C020801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE2_TX_STAT1_PL              = 0x80041C020801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE2_TX_STAT1_PL              = 0x80041C020801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE3_TX_CNTL1G_PL                 = 0x800414030701103Full;
//DUPS: 800414030701143F, 800414030701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE3_TX_CNTL1G_PL               = 0x800414030701103Full;
//DUPS: 800414030701143F, 800414030701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE3_TX_CNTL1G_PL               = 0x800414030801103Full;
//DUPS: 800414030801143F, 800414030801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE3_TX_CNTL1G_PL                = 0x800414030701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE3_TX_CNTL1G_PL               = 0x800414030701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE3_TX_CNTL1G_PL             = 0x800414030701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE3_TX_CNTL1G_PL             = 0x800414030701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE3_TX_CNTL1G_PL             = 0x800414030701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE3_TX_CNTL1G_PL             = 0x800414030801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE3_TX_CNTL1G_PL             = 0x800414030801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE3_TX_CNTL1G_PL             = 0x800414030801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE3_TX_CNTL1G_PL              = 0x800414030701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE3_TX_CNTL1G_PL              = 0x800414030701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE3_TX_CNTL1G_PL              = 0x800414030701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE3_TX_CNTL1G_PL              = 0x800414030801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE3_TX_CNTL1G_PL             = 0x800414030801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE3_TX_CNTL1G_PL             = 0x800414030801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE3_TX_CNTL2_O_PL                = 0x80044C030701103Full;
//DUPS: 80044C030701143F, 80044C030701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE3_TX_CNTL2_O_PL              = 0x80044C030701103Full;
//DUPS: 80044C030701143F, 80044C030701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE3_TX_CNTL2_O_PL              = 0x80044C030801103Full;
//DUPS: 80044C030801143F, 80044C030801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE3_TX_CNTL2_O_PL               = 0x80044C030701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE3_TX_CNTL2_O_PL              = 0x80044C030701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE3_TX_CNTL2_O_PL            = 0x80044C030701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE3_TX_CNTL2_O_PL            = 0x80044C030701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE3_TX_CNTL2_O_PL            = 0x80044C030701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE3_TX_CNTL2_O_PL            = 0x80044C030801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE3_TX_CNTL2_O_PL            = 0x80044C030801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE3_TX_CNTL2_O_PL            = 0x80044C030801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE3_TX_CNTL2_O_PL             = 0x80044C030701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE3_TX_CNTL2_O_PL             = 0x80044C030701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE3_TX_CNTL2_O_PL             = 0x80044C030701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE3_TX_CNTL2_O_PL             = 0x80044C030801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE3_TX_CNTL2_O_PL            = 0x80044C030801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE3_TX_CNTL2_O_PL            = 0x80044C030801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE3_TX_CNTL3_EO_PL               = 0x800454030701103Full;
//DUPS: 800454030701143F, 800454030701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE3_TX_CNTL3_EO_PL             = 0x800454030701103Full;
//DUPS: 800454030701143F, 800454030701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE3_TX_CNTL3_EO_PL             = 0x800454030801103Full;
//DUPS: 800454030801143F, 800454030801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE3_TX_CNTL3_EO_PL              = 0x800454030701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE3_TX_CNTL3_EO_PL             = 0x800454030701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE3_TX_CNTL3_EO_PL           = 0x800454030701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE3_TX_CNTL3_EO_PL           = 0x800454030701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE3_TX_CNTL3_EO_PL           = 0x800454030701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE3_TX_CNTL3_EO_PL           = 0x800454030801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE3_TX_CNTL3_EO_PL           = 0x800454030801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE3_TX_CNTL3_EO_PL           = 0x800454030801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE3_TX_CNTL3_EO_PL            = 0x800454030701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE3_TX_CNTL3_EO_PL            = 0x800454030701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE3_TX_CNTL3_EO_PL            = 0x800454030701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE3_TX_CNTL3_EO_PL            = 0x800454030801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE3_TX_CNTL3_EO_PL           = 0x800454030801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE3_TX_CNTL3_EO_PL           = 0x800454030801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE3_TX_FIR_ERROR_INJECT_PL       = 0x800434030701103Full;
//DUPS: 800434030701143F, 800434030701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE3_TX_FIR_ERROR_INJECT_PL     = 0x800434030701103Full;
//DUPS: 800434030701143F, 800434030701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE3_TX_FIR_ERROR_INJECT_PL     = 0x800434030801103Full;
//DUPS: 800434030801143F, 800434030801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE3_TX_FIR_ERROR_INJECT_PL      = 0x800434030701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE3_TX_FIR_ERROR_INJECT_PL     = 0x800434030701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x800434030701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x800434030701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x800434030701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x800434030801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x800434030801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x800434030801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE3_TX_FIR_ERROR_INJECT_PL    = 0x800434030701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE3_TX_FIR_ERROR_INJECT_PL    = 0x800434030701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE3_TX_FIR_ERROR_INJECT_PL    = 0x800434030701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE3_TX_FIR_ERROR_INJECT_PL    = 0x800434030801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x800434030801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x800434030801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE3_TX_FIR_MASK_PL               = 0x80042C030701103Full;
//DUPS: 80042C030701143F, 80042C030701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE3_TX_FIR_MASK_PL             = 0x80042C030701103Full;
//DUPS: 80042C030701143F, 80042C030701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE3_TX_FIR_MASK_PL             = 0x80042C030801103Full;
//DUPS: 80042C030801143F, 80042C030801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE3_TX_FIR_MASK_PL              = 0x80042C030701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE3_TX_FIR_MASK_PL             = 0x80042C030701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE3_TX_FIR_MASK_PL           = 0x80042C030701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE3_TX_FIR_MASK_PL           = 0x80042C030701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE3_TX_FIR_MASK_PL           = 0x80042C030701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE3_TX_FIR_MASK_PL           = 0x80042C030801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE3_TX_FIR_MASK_PL           = 0x80042C030801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE3_TX_FIR_MASK_PL           = 0x80042C030801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE3_TX_FIR_MASK_PL            = 0x80042C030701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE3_TX_FIR_MASK_PL            = 0x80042C030701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE3_TX_FIR_MASK_PL            = 0x80042C030701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE3_TX_FIR_MASK_PL            = 0x80042C030801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE3_TX_FIR_MASK_PL           = 0x80042C030801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE3_TX_FIR_MASK_PL           = 0x80042C030801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE3_TX_FIR_PL                    = 0x800424030701103Full;
//DUPS: 800424030701143F, 800424030701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE3_TX_FIR_PL                  = 0x800424030701103Full;
//DUPS: 800424030701143F, 800424030701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE3_TX_FIR_PL                  = 0x800424030801103Full;
//DUPS: 800424030801143F, 800424030801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE3_TX_FIR_PL                   = 0x800424030701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE3_TX_FIR_PL                  = 0x800424030701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE3_TX_FIR_PL                = 0x800424030701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE3_TX_FIR_PL                = 0x800424030701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE3_TX_FIR_PL                = 0x800424030701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE3_TX_FIR_PL                = 0x800424030801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE3_TX_FIR_PL                = 0x800424030801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE3_TX_FIR_PL                = 0x800424030801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE3_TX_FIR_PL                 = 0x800424030701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE3_TX_FIR_PL                 = 0x800424030701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE3_TX_FIR_PL                 = 0x800424030701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE3_TX_FIR_PL                 = 0x800424030801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE3_TX_FIR_PL                = 0x800424030801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE3_TX_FIR_PL                = 0x800424030801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE3_TX_MODE1_PL                  = 0x800404030701103Full;
//DUPS: 800404030701143F, 800404030701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE3_TX_MODE1_PL                = 0x800404030701103Full;
//DUPS: 800404030701143F, 800404030701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE3_TX_MODE1_PL                = 0x800404030801103Full;
//DUPS: 800404030801143F, 800404030801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE3_TX_MODE1_PL                 = 0x800404030701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE3_TX_MODE1_PL                = 0x800404030701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE3_TX_MODE1_PL              = 0x800404030701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE3_TX_MODE1_PL              = 0x800404030701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE3_TX_MODE1_PL              = 0x800404030701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE3_TX_MODE1_PL              = 0x800404030801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE3_TX_MODE1_PL              = 0x800404030801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE3_TX_MODE1_PL              = 0x800404030801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE3_TX_MODE1_PL               = 0x800404030701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE3_TX_MODE1_PL               = 0x800404030701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE3_TX_MODE1_PL               = 0x800404030701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE3_TX_MODE1_PL               = 0x800404030801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE3_TX_MODE1_PL              = 0x800404030801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE3_TX_MODE1_PL              = 0x800404030801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE3_TX_MODE2_PL                  = 0x80040C030701103Full;
//DUPS: 80040C030701143F, 80040C030701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE3_TX_MODE2_PL                = 0x80040C030701103Full;
//DUPS: 80040C030701143F, 80040C030701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE3_TX_MODE2_PL                = 0x80040C030801103Full;
//DUPS: 80040C030801143F, 80040C030801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE3_TX_MODE2_PL                 = 0x80040C030701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE3_TX_MODE2_PL                = 0x80040C030701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE3_TX_MODE2_PL              = 0x80040C030701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE3_TX_MODE2_PL              = 0x80040C030701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE3_TX_MODE2_PL              = 0x80040C030701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE3_TX_MODE2_PL              = 0x80040C030801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE3_TX_MODE2_PL              = 0x80040C030801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE3_TX_MODE2_PL              = 0x80040C030801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE3_TX_MODE2_PL               = 0x80040C030701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE3_TX_MODE2_PL               = 0x80040C030701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE3_TX_MODE2_PL               = 0x80040C030701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE3_TX_MODE2_PL               = 0x80040C030801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE3_TX_MODE2_PL              = 0x80040C030801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE3_TX_MODE2_PL              = 0x80040C030801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE3_TX_STAT1_PL                  = 0x80041C030701103Full;
//DUPS: 80041C030701143F, 80041C030701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE3_TX_STAT1_PL                = 0x80041C030701103Full;
//DUPS: 80041C030701143F, 80041C030701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE3_TX_STAT1_PL                = 0x80041C030801103Full;
//DUPS: 80041C030801143F, 80041C030801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS0_SLICE3_TX_STAT1_PL                 = 0x80041C030701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS0_SLICE3_TX_STAT1_PL                = 0x80041C030701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS0_SLICE3_TX_STAT1_PL              = 0x80041C030701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS0_SLICE3_TX_STAT1_PL              = 0x80041C030701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE3_TX_STAT1_PL              = 0x80041C030701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS0_SLICE3_TX_STAT1_PL              = 0x80041C030801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS0_SLICE3_TX_STAT1_PL              = 0x80041C030801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE3_TX_STAT1_PL              = 0x80041C030801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS0_SLICE3_TX_STAT1_PL               = 0x80041C030701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS0_SLICE3_TX_STAT1_PL               = 0x80041C030701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE3_TX_STAT1_PL               = 0x80041C030701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS0_SLICE3_TX_STAT1_PL               = 0x80041C030801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS0_SLICE3_TX_STAT1_PL              = 0x80041C030801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE3_TX_STAT1_PL              = 0x80041C030801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE4_TX_CNTL1G_PL                 = 0x800414040701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE4_TX_CNTL1G_PL               = 0x800414040701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE4_TX_CNTL1G_PL               = 0x800414040801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE4_TX_CNTL1G_PL             = 0x800414040701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE4_TX_CNTL1G_PL             = 0x800414040801183Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE4_TX_CNTL1G_PL              = 0x800414040701183Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE4_TX_CNTL1G_PL             = 0x800414040801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE4_TX_CNTL2_O_PL                = 0x80044C040701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE4_TX_CNTL2_O_PL              = 0x80044C040701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE4_TX_CNTL2_O_PL              = 0x80044C040801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE4_TX_CNTL2_O_PL            = 0x80044C040701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE4_TX_CNTL2_O_PL            = 0x80044C040801183Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE4_TX_CNTL2_O_PL             = 0x80044C040701183Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE4_TX_CNTL2_O_PL            = 0x80044C040801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE4_TX_CNTL3_EO_PL               = 0x800454040701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE4_TX_CNTL3_EO_PL             = 0x800454040701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE4_TX_CNTL3_EO_PL             = 0x800454040801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE4_TX_CNTL3_EO_PL           = 0x800454040701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE4_TX_CNTL3_EO_PL           = 0x800454040801183Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE4_TX_CNTL3_EO_PL            = 0x800454040701183Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE4_TX_CNTL3_EO_PL           = 0x800454040801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE4_TX_FIR_ERROR_INJECT_PL       = 0x800434040701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE4_TX_FIR_ERROR_INJECT_PL     = 0x800434040701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE4_TX_FIR_ERROR_INJECT_PL     = 0x800434040801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE4_TX_FIR_ERROR_INJECT_PL   = 0x800434040701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE4_TX_FIR_ERROR_INJECT_PL   = 0x800434040801183Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE4_TX_FIR_ERROR_INJECT_PL    = 0x800434040701183Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE4_TX_FIR_ERROR_INJECT_PL   = 0x800434040801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE4_TX_FIR_MASK_PL               = 0x80042C040701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE4_TX_FIR_MASK_PL             = 0x80042C040701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE4_TX_FIR_MASK_PL             = 0x80042C040801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE4_TX_FIR_MASK_PL           = 0x80042C040701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE4_TX_FIR_MASK_PL           = 0x80042C040801183Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE4_TX_FIR_MASK_PL            = 0x80042C040701183Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE4_TX_FIR_MASK_PL           = 0x80042C040801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE4_TX_FIR_PL                    = 0x800424040701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE4_TX_FIR_PL                  = 0x800424040701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE4_TX_FIR_PL                  = 0x800424040801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE4_TX_FIR_PL                = 0x800424040701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE4_TX_FIR_PL                = 0x800424040801183Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE4_TX_FIR_PL                 = 0x800424040701183Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE4_TX_FIR_PL                = 0x800424040801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE4_TX_MODE1_PL                  = 0x800404040701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE4_TX_MODE1_PL                = 0x800404040701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE4_TX_MODE1_PL                = 0x800404040801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE4_TX_MODE1_PL              = 0x800404040701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE4_TX_MODE1_PL              = 0x800404040801183Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE4_TX_MODE1_PL               = 0x800404040701183Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE4_TX_MODE1_PL              = 0x800404040801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE4_TX_MODE2_PL                  = 0x80040C040701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE4_TX_MODE2_PL                = 0x80040C040701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE4_TX_MODE2_PL                = 0x80040C040801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE4_TX_MODE2_PL              = 0x80040C040701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE4_TX_MODE2_PL              = 0x80040C040801183Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE4_TX_MODE2_PL               = 0x80040C040701183Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE4_TX_MODE2_PL              = 0x80040C040801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE4_TX_STAT1_PL                  = 0x80041C040701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE4_TX_STAT1_PL                = 0x80041C040701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE4_TX_STAT1_PL                = 0x80041C040801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE4_TX_STAT1_PL              = 0x80041C040701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE4_TX_STAT1_PL              = 0x80041C040801183Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE4_TX_STAT1_PL               = 0x80041C040701183Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE4_TX_STAT1_PL              = 0x80041C040801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE5_TX_CNTL1G_PL                 = 0x800414050701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE5_TX_CNTL1G_PL               = 0x800414050701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE5_TX_CNTL1G_PL               = 0x800414050801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE5_TX_CNTL1G_PL             = 0x800414050701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE5_TX_CNTL1G_PL             = 0x800414050801183Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE5_TX_CNTL1G_PL              = 0x800414050701183Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE5_TX_CNTL1G_PL             = 0x800414050801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE5_TX_CNTL2_O_PL                = 0x80044C050701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE5_TX_CNTL2_O_PL              = 0x80044C050701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE5_TX_CNTL2_O_PL              = 0x80044C050801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE5_TX_CNTL2_O_PL            = 0x80044C050701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE5_TX_CNTL2_O_PL            = 0x80044C050801183Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE5_TX_CNTL2_O_PL             = 0x80044C050701183Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE5_TX_CNTL2_O_PL            = 0x80044C050801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE5_TX_CNTL3_EO_PL               = 0x800454050701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE5_TX_CNTL3_EO_PL             = 0x800454050701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE5_TX_CNTL3_EO_PL             = 0x800454050801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE5_TX_CNTL3_EO_PL           = 0x800454050701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE5_TX_CNTL3_EO_PL           = 0x800454050801183Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE5_TX_CNTL3_EO_PL            = 0x800454050701183Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE5_TX_CNTL3_EO_PL           = 0x800454050801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE5_TX_FIR_ERROR_INJECT_PL       = 0x800434050701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE5_TX_FIR_ERROR_INJECT_PL     = 0x800434050701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE5_TX_FIR_ERROR_INJECT_PL     = 0x800434050801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE5_TX_FIR_ERROR_INJECT_PL   = 0x800434050701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE5_TX_FIR_ERROR_INJECT_PL   = 0x800434050801183Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE5_TX_FIR_ERROR_INJECT_PL    = 0x800434050701183Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE5_TX_FIR_ERROR_INJECT_PL   = 0x800434050801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE5_TX_FIR_MASK_PL               = 0x80042C050701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE5_TX_FIR_MASK_PL             = 0x80042C050701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE5_TX_FIR_MASK_PL             = 0x80042C050801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE5_TX_FIR_MASK_PL           = 0x80042C050701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE5_TX_FIR_MASK_PL           = 0x80042C050801183Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE5_TX_FIR_MASK_PL            = 0x80042C050701183Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE5_TX_FIR_MASK_PL           = 0x80042C050801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE5_TX_FIR_PL                    = 0x800424050701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE5_TX_FIR_PL                  = 0x800424050701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE5_TX_FIR_PL                  = 0x800424050801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE5_TX_FIR_PL                = 0x800424050701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE5_TX_FIR_PL                = 0x800424050801183Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE5_TX_FIR_PL                 = 0x800424050701183Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE5_TX_FIR_PL                = 0x800424050801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE5_TX_MODE1_PL                  = 0x800404050701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE5_TX_MODE1_PL                = 0x800404050701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE5_TX_MODE1_PL                = 0x800404050801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE5_TX_MODE1_PL              = 0x800404050701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE5_TX_MODE1_PL              = 0x800404050801183Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE5_TX_MODE1_PL               = 0x800404050701183Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE5_TX_MODE1_PL              = 0x800404050801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE5_TX_MODE2_PL                  = 0x80040C050701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE5_TX_MODE2_PL                = 0x80040C050701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE5_TX_MODE2_PL                = 0x80040C050801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE5_TX_MODE2_PL              = 0x80040C050701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE5_TX_MODE2_PL              = 0x80040C050801183Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE5_TX_MODE2_PL               = 0x80040C050701183Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE5_TX_MODE2_PL              = 0x80040C050801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS0_SLICE5_TX_STAT1_PL                  = 0x80041C050701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS0_SLICE5_TX_STAT1_PL                = 0x80041C050701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS0_SLICE5_TX_STAT1_PL                = 0x80041C050801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS0_SLICE5_TX_STAT1_PL              = 0x80041C050701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS0_SLICE5_TX_STAT1_PL              = 0x80041C050801183Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS0_SLICE5_TX_STAT1_PL               = 0x80041C050701183Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS0_SLICE5_TX_STAT1_PL              = 0x80041C050801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE0_TX_CNTL1G_PL                 = 0x800414040701103Full;
//DUPS: 800414040701143F, 800414060701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE0_TX_CNTL1G_PL               = 0x800414040701103Full;
//DUPS: 800414040701143F, 800414060701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE0_TX_CNTL1G_PL               = 0x800414040801103Full;
//DUPS: 800414040801143F, 800414060801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE0_TX_CNTL1G_PL                = 0x800414040701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE0_TX_CNTL1G_PL               = 0x800414040701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE0_TX_CNTL1G_PL             = 0x800414040701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE0_TX_CNTL1G_PL             = 0x800414040701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE0_TX_CNTL1G_PL             = 0x800414060701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE0_TX_CNTL1G_PL             = 0x800414040801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE0_TX_CNTL1G_PL             = 0x800414040801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE0_TX_CNTL1G_PL             = 0x800414060801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE0_TX_CNTL1G_PL              = 0x800414040701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE0_TX_CNTL1G_PL              = 0x800414040701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS1_SLICE0_TX_CNTL1G_PL              = 0x800414060701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE0_TX_CNTL1G_PL              = 0x800414040801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE0_TX_CNTL1G_PL             = 0x800414040801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS1_SLICE0_TX_CNTL1G_PL             = 0x800414060801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE0_TX_CNTL2_O_PL                = 0x80044C040701103Full;
//DUPS: 80044C040701143F, 80044C060701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE0_TX_CNTL2_O_PL              = 0x80044C040701103Full;
//DUPS: 80044C040701143F, 80044C060701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE0_TX_CNTL2_O_PL              = 0x80044C040801103Full;
//DUPS: 80044C040801143F, 80044C060801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE0_TX_CNTL2_O_PL               = 0x80044C040701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE0_TX_CNTL2_O_PL              = 0x80044C040701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE0_TX_CNTL2_O_PL            = 0x80044C040701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE0_TX_CNTL2_O_PL            = 0x80044C040701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE0_TX_CNTL2_O_PL            = 0x80044C060701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE0_TX_CNTL2_O_PL            = 0x80044C040801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE0_TX_CNTL2_O_PL            = 0x80044C040801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE0_TX_CNTL2_O_PL            = 0x80044C060801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE0_TX_CNTL2_O_PL             = 0x80044C040701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE0_TX_CNTL2_O_PL             = 0x80044C040701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS1_SLICE0_TX_CNTL2_O_PL             = 0x80044C060701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE0_TX_CNTL2_O_PL             = 0x80044C040801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE0_TX_CNTL2_O_PL            = 0x80044C040801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS1_SLICE0_TX_CNTL2_O_PL            = 0x80044C060801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE0_TX_CNTL3_EO_PL               = 0x800454040701103Full;
//DUPS: 800454040701143F, 800454060701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE0_TX_CNTL3_EO_PL             = 0x800454040701103Full;
//DUPS: 800454040701143F, 800454060701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE0_TX_CNTL3_EO_PL             = 0x800454040801103Full;
//DUPS: 800454040801143F, 800454060801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE0_TX_CNTL3_EO_PL              = 0x800454040701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE0_TX_CNTL3_EO_PL             = 0x800454040701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE0_TX_CNTL3_EO_PL           = 0x800454040701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE0_TX_CNTL3_EO_PL           = 0x800454040701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE0_TX_CNTL3_EO_PL           = 0x800454060701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE0_TX_CNTL3_EO_PL           = 0x800454040801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE0_TX_CNTL3_EO_PL           = 0x800454040801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE0_TX_CNTL3_EO_PL           = 0x800454060801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE0_TX_CNTL3_EO_PL            = 0x800454040701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE0_TX_CNTL3_EO_PL            = 0x800454040701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS1_SLICE0_TX_CNTL3_EO_PL            = 0x800454060701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE0_TX_CNTL3_EO_PL            = 0x800454040801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE0_TX_CNTL3_EO_PL           = 0x800454040801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS1_SLICE0_TX_CNTL3_EO_PL           = 0x800454060801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE0_TX_FIR_ERROR_INJECT_PL       = 0x800434040701103Full;
//DUPS: 800434040701143F, 800434060701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE0_TX_FIR_ERROR_INJECT_PL     = 0x800434040701103Full;
//DUPS: 800434040701143F, 800434060701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE0_TX_FIR_ERROR_INJECT_PL     = 0x800434040801103Full;
//DUPS: 800434040801143F, 800434060801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE0_TX_FIR_ERROR_INJECT_PL      = 0x800434040701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE0_TX_FIR_ERROR_INJECT_PL     = 0x800434040701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x800434040701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x800434040701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x800434060701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x800434040801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x800434040801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x800434060801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE0_TX_FIR_ERROR_INJECT_PL    = 0x800434040701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE0_TX_FIR_ERROR_INJECT_PL    = 0x800434040701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS1_SLICE0_TX_FIR_ERROR_INJECT_PL    = 0x800434060701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE0_TX_FIR_ERROR_INJECT_PL    = 0x800434040801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x800434040801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS1_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x800434060801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE0_TX_FIR_MASK_PL               = 0x80042C040701103Full;
//DUPS: 80042C040701143F, 80042C060701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE0_TX_FIR_MASK_PL             = 0x80042C040701103Full;
//DUPS: 80042C040701143F, 80042C060701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE0_TX_FIR_MASK_PL             = 0x80042C040801103Full;
//DUPS: 80042C040801143F, 80042C060801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE0_TX_FIR_MASK_PL              = 0x80042C040701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE0_TX_FIR_MASK_PL             = 0x80042C040701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE0_TX_FIR_MASK_PL           = 0x80042C040701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE0_TX_FIR_MASK_PL           = 0x80042C040701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE0_TX_FIR_MASK_PL           = 0x80042C060701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE0_TX_FIR_MASK_PL           = 0x80042C040801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE0_TX_FIR_MASK_PL           = 0x80042C040801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE0_TX_FIR_MASK_PL           = 0x80042C060801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE0_TX_FIR_MASK_PL            = 0x80042C040701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE0_TX_FIR_MASK_PL            = 0x80042C040701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS1_SLICE0_TX_FIR_MASK_PL            = 0x80042C060701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE0_TX_FIR_MASK_PL            = 0x80042C040801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE0_TX_FIR_MASK_PL           = 0x80042C040801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS1_SLICE0_TX_FIR_MASK_PL           = 0x80042C060801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE0_TX_FIR_PL                    = 0x800424040701103Full;
//DUPS: 800424040701143F, 800424060701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE0_TX_FIR_PL                  = 0x800424040701103Full;
//DUPS: 800424040701143F, 800424060701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE0_TX_FIR_PL                  = 0x800424040801103Full;
//DUPS: 800424040801143F, 800424060801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE0_TX_FIR_PL                   = 0x800424040701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE0_TX_FIR_PL                  = 0x800424040701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE0_TX_FIR_PL                = 0x800424040701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE0_TX_FIR_PL                = 0x800424040701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE0_TX_FIR_PL                = 0x800424060701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE0_TX_FIR_PL                = 0x800424040801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE0_TX_FIR_PL                = 0x800424040801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE0_TX_FIR_PL                = 0x800424060801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE0_TX_FIR_PL                 = 0x800424040701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE0_TX_FIR_PL                 = 0x800424040701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS1_SLICE0_TX_FIR_PL                 = 0x800424060701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE0_TX_FIR_PL                 = 0x800424040801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE0_TX_FIR_PL                = 0x800424040801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS1_SLICE0_TX_FIR_PL                = 0x800424060801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE0_TX_MODE1_PL                  = 0x800404040701103Full;
//DUPS: 800404040701143F, 800404060701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE0_TX_MODE1_PL                = 0x800404040701103Full;
//DUPS: 800404040701143F, 800404060701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE0_TX_MODE1_PL                = 0x800404040801103Full;
//DUPS: 800404040801143F, 800404060801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE0_TX_MODE1_PL                 = 0x800404040701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE0_TX_MODE1_PL                = 0x800404040701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE0_TX_MODE1_PL              = 0x800404040701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE0_TX_MODE1_PL              = 0x800404040701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE0_TX_MODE1_PL              = 0x800404060701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE0_TX_MODE1_PL              = 0x800404040801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE0_TX_MODE1_PL              = 0x800404040801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE0_TX_MODE1_PL              = 0x800404060801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE0_TX_MODE1_PL               = 0x800404040701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE0_TX_MODE1_PL               = 0x800404040701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS1_SLICE0_TX_MODE1_PL               = 0x800404060701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE0_TX_MODE1_PL               = 0x800404040801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE0_TX_MODE1_PL              = 0x800404040801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS1_SLICE0_TX_MODE1_PL              = 0x800404060801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE0_TX_MODE2_PL                  = 0x80040C040701103Full;
//DUPS: 80040C040701143F, 80040C060701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE0_TX_MODE2_PL                = 0x80040C040701103Full;
//DUPS: 80040C040701143F, 80040C060701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE0_TX_MODE2_PL                = 0x80040C040801103Full;
//DUPS: 80040C040801143F, 80040C060801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE0_TX_MODE2_PL                 = 0x80040C040701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE0_TX_MODE2_PL                = 0x80040C040701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE0_TX_MODE2_PL              = 0x80040C040701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE0_TX_MODE2_PL              = 0x80040C040701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE0_TX_MODE2_PL              = 0x80040C060701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE0_TX_MODE2_PL              = 0x80040C040801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE0_TX_MODE2_PL              = 0x80040C040801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE0_TX_MODE2_PL              = 0x80040C060801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE0_TX_MODE2_PL               = 0x80040C040701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE0_TX_MODE2_PL               = 0x80040C040701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS1_SLICE0_TX_MODE2_PL               = 0x80040C060701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE0_TX_MODE2_PL               = 0x80040C040801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE0_TX_MODE2_PL              = 0x80040C040801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS1_SLICE0_TX_MODE2_PL              = 0x80040C060801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE0_TX_STAT1_PL                  = 0x80041C040701103Full;
//DUPS: 80041C040701143F, 80041C060701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE0_TX_STAT1_PL                = 0x80041C040701103Full;
//DUPS: 80041C040701143F, 80041C060701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE0_TX_STAT1_PL                = 0x80041C040801103Full;
//DUPS: 80041C040801143F, 80041C060801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE0_TX_STAT1_PL                 = 0x80041C040701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE0_TX_STAT1_PL                = 0x80041C040701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE0_TX_STAT1_PL              = 0x80041C040701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE0_TX_STAT1_PL              = 0x80041C040701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE0_TX_STAT1_PL              = 0x80041C060701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE0_TX_STAT1_PL              = 0x80041C040801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE0_TX_STAT1_PL              = 0x80041C040801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE0_TX_STAT1_PL              = 0x80041C060801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE0_TX_STAT1_PL               = 0x80041C040701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE0_TX_STAT1_PL               = 0x80041C040701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS1_SLICE0_TX_STAT1_PL               = 0x80041C060701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE0_TX_STAT1_PL               = 0x80041C040801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE0_TX_STAT1_PL              = 0x80041C040801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS1_SLICE0_TX_STAT1_PL              = 0x80041C060801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE1_TX_CNTL1G_PL                 = 0x800414050701103Full;
//DUPS: 800414050701143F, 800414070701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE1_TX_CNTL1G_PL               = 0x800414050701103Full;
//DUPS: 800414050701143F, 800414070701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE1_TX_CNTL1G_PL               = 0x800414050801103Full;
//DUPS: 800414050801143F, 800414070801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE1_TX_CNTL1G_PL                = 0x800414050701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE1_TX_CNTL1G_PL               = 0x800414050701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE1_TX_CNTL1G_PL             = 0x800414050701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE1_TX_CNTL1G_PL             = 0x800414050701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE1_TX_CNTL1G_PL             = 0x800414070701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE1_TX_CNTL1G_PL             = 0x800414050801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE1_TX_CNTL1G_PL             = 0x800414050801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE1_TX_CNTL1G_PL             = 0x800414070801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE1_TX_CNTL1G_PL              = 0x800414050701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE1_TX_CNTL1G_PL              = 0x800414050701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS1_SLICE1_TX_CNTL1G_PL              = 0x800414070701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE1_TX_CNTL1G_PL              = 0x800414050801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE1_TX_CNTL1G_PL             = 0x800414050801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS1_SLICE1_TX_CNTL1G_PL             = 0x800414070801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE1_TX_CNTL2_O_PL                = 0x80044C050701103Full;
//DUPS: 80044C050701143F, 80044C070701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE1_TX_CNTL2_O_PL              = 0x80044C050701103Full;
//DUPS: 80044C050701143F, 80044C070701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE1_TX_CNTL2_O_PL              = 0x80044C050801103Full;
//DUPS: 80044C050801143F, 80044C070801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE1_TX_CNTL2_O_PL               = 0x80044C050701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE1_TX_CNTL2_O_PL              = 0x80044C050701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE1_TX_CNTL2_O_PL            = 0x80044C050701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE1_TX_CNTL2_O_PL            = 0x80044C050701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE1_TX_CNTL2_O_PL            = 0x80044C070701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE1_TX_CNTL2_O_PL            = 0x80044C050801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE1_TX_CNTL2_O_PL            = 0x80044C050801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE1_TX_CNTL2_O_PL            = 0x80044C070801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE1_TX_CNTL2_O_PL             = 0x80044C050701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE1_TX_CNTL2_O_PL             = 0x80044C050701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS1_SLICE1_TX_CNTL2_O_PL             = 0x80044C070701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE1_TX_CNTL2_O_PL             = 0x80044C050801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE1_TX_CNTL2_O_PL            = 0x80044C050801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS1_SLICE1_TX_CNTL2_O_PL            = 0x80044C070801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE1_TX_CNTL3_EO_PL               = 0x800454050701103Full;
//DUPS: 800454050701143F, 800454070701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE1_TX_CNTL3_EO_PL             = 0x800454050701103Full;
//DUPS: 800454050701143F, 800454070701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE1_TX_CNTL3_EO_PL             = 0x800454050801103Full;
//DUPS: 800454050801143F, 800454070801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE1_TX_CNTL3_EO_PL              = 0x800454050701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE1_TX_CNTL3_EO_PL             = 0x800454050701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE1_TX_CNTL3_EO_PL           = 0x800454050701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE1_TX_CNTL3_EO_PL           = 0x800454050701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE1_TX_CNTL3_EO_PL           = 0x800454070701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE1_TX_CNTL3_EO_PL           = 0x800454050801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE1_TX_CNTL3_EO_PL           = 0x800454050801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE1_TX_CNTL3_EO_PL           = 0x800454070801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE1_TX_CNTL3_EO_PL            = 0x800454050701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE1_TX_CNTL3_EO_PL            = 0x800454050701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS1_SLICE1_TX_CNTL3_EO_PL            = 0x800454070701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE1_TX_CNTL3_EO_PL            = 0x800454050801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE1_TX_CNTL3_EO_PL           = 0x800454050801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS1_SLICE1_TX_CNTL3_EO_PL           = 0x800454070801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE1_TX_FIR_ERROR_INJECT_PL       = 0x800434050701103Full;
//DUPS: 800434050701143F, 800434070701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE1_TX_FIR_ERROR_INJECT_PL     = 0x800434050701103Full;
//DUPS: 800434050701143F, 800434070701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE1_TX_FIR_ERROR_INJECT_PL     = 0x800434050801103Full;
//DUPS: 800434050801143F, 800434070801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE1_TX_FIR_ERROR_INJECT_PL      = 0x800434050701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE1_TX_FIR_ERROR_INJECT_PL     = 0x800434050701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x800434050701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x800434050701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x800434070701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x800434050801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x800434050801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x800434070801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE1_TX_FIR_ERROR_INJECT_PL    = 0x800434050701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE1_TX_FIR_ERROR_INJECT_PL    = 0x800434050701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS1_SLICE1_TX_FIR_ERROR_INJECT_PL    = 0x800434070701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE1_TX_FIR_ERROR_INJECT_PL    = 0x800434050801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x800434050801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS1_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x800434070801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE1_TX_FIR_MASK_PL               = 0x80042C050701103Full;
//DUPS: 80042C050701143F, 80042C070701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE1_TX_FIR_MASK_PL             = 0x80042C050701103Full;
//DUPS: 80042C050701143F, 80042C070701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE1_TX_FIR_MASK_PL             = 0x80042C050801103Full;
//DUPS: 80042C050801143F, 80042C070801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE1_TX_FIR_MASK_PL              = 0x80042C050701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE1_TX_FIR_MASK_PL             = 0x80042C050701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE1_TX_FIR_MASK_PL           = 0x80042C050701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE1_TX_FIR_MASK_PL           = 0x80042C050701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE1_TX_FIR_MASK_PL           = 0x80042C070701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE1_TX_FIR_MASK_PL           = 0x80042C050801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE1_TX_FIR_MASK_PL           = 0x80042C050801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE1_TX_FIR_MASK_PL           = 0x80042C070801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE1_TX_FIR_MASK_PL            = 0x80042C050701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE1_TX_FIR_MASK_PL            = 0x80042C050701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS1_SLICE1_TX_FIR_MASK_PL            = 0x80042C070701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE1_TX_FIR_MASK_PL            = 0x80042C050801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE1_TX_FIR_MASK_PL           = 0x80042C050801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS1_SLICE1_TX_FIR_MASK_PL           = 0x80042C070801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE1_TX_FIR_PL                    = 0x800424050701103Full;
//DUPS: 800424050701143F, 800424070701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE1_TX_FIR_PL                  = 0x800424050701103Full;
//DUPS: 800424050701143F, 800424070701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE1_TX_FIR_PL                  = 0x800424050801103Full;
//DUPS: 800424050801143F, 800424070801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE1_TX_FIR_PL                   = 0x800424050701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE1_TX_FIR_PL                  = 0x800424050701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE1_TX_FIR_PL                = 0x800424050701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE1_TX_FIR_PL                = 0x800424050701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE1_TX_FIR_PL                = 0x800424070701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE1_TX_FIR_PL                = 0x800424050801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE1_TX_FIR_PL                = 0x800424050801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE1_TX_FIR_PL                = 0x800424070801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE1_TX_FIR_PL                 = 0x800424050701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE1_TX_FIR_PL                 = 0x800424050701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS1_SLICE1_TX_FIR_PL                 = 0x800424070701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE1_TX_FIR_PL                 = 0x800424050801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE1_TX_FIR_PL                = 0x800424050801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS1_SLICE1_TX_FIR_PL                = 0x800424070801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE1_TX_MODE1_PL                  = 0x800404050701103Full;
//DUPS: 800404050701143F, 800404070701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE1_TX_MODE1_PL                = 0x800404050701103Full;
//DUPS: 800404050701143F, 800404070701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE1_TX_MODE1_PL                = 0x800404050801103Full;
//DUPS: 800404050801143F, 800404070801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE1_TX_MODE1_PL                 = 0x800404050701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE1_TX_MODE1_PL                = 0x800404050701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE1_TX_MODE1_PL              = 0x800404050701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE1_TX_MODE1_PL              = 0x800404050701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE1_TX_MODE1_PL              = 0x800404070701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE1_TX_MODE1_PL              = 0x800404050801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE1_TX_MODE1_PL              = 0x800404050801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE1_TX_MODE1_PL              = 0x800404070801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE1_TX_MODE1_PL               = 0x800404050701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE1_TX_MODE1_PL               = 0x800404050701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS1_SLICE1_TX_MODE1_PL               = 0x800404070701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE1_TX_MODE1_PL               = 0x800404050801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE1_TX_MODE1_PL              = 0x800404050801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS1_SLICE1_TX_MODE1_PL              = 0x800404070801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE1_TX_MODE2_PL                  = 0x80040C050701103Full;
//DUPS: 80040C050701143F, 80040C070701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE1_TX_MODE2_PL                = 0x80040C050701103Full;
//DUPS: 80040C050701143F, 80040C070701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE1_TX_MODE2_PL                = 0x80040C050801103Full;
//DUPS: 80040C050801143F, 80040C070801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE1_TX_MODE2_PL                 = 0x80040C050701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE1_TX_MODE2_PL                = 0x80040C050701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE1_TX_MODE2_PL              = 0x80040C050701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE1_TX_MODE2_PL              = 0x80040C050701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE1_TX_MODE2_PL              = 0x80040C070701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE1_TX_MODE2_PL              = 0x80040C050801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE1_TX_MODE2_PL              = 0x80040C050801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE1_TX_MODE2_PL              = 0x80040C070801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE1_TX_MODE2_PL               = 0x80040C050701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE1_TX_MODE2_PL               = 0x80040C050701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS1_SLICE1_TX_MODE2_PL               = 0x80040C070701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE1_TX_MODE2_PL               = 0x80040C050801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE1_TX_MODE2_PL              = 0x80040C050801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS1_SLICE1_TX_MODE2_PL              = 0x80040C070801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE1_TX_STAT1_PL                  = 0x80041C050701103Full;
//DUPS: 80041C050701143F, 80041C070701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE1_TX_STAT1_PL                = 0x80041C050701103Full;
//DUPS: 80041C050701143F, 80041C070701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE1_TX_STAT1_PL                = 0x80041C050801103Full;
//DUPS: 80041C050801143F, 80041C070801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE1_TX_STAT1_PL                 = 0x80041C050701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE1_TX_STAT1_PL                = 0x80041C050701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE1_TX_STAT1_PL              = 0x80041C050701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE1_TX_STAT1_PL              = 0x80041C050701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE1_TX_STAT1_PL              = 0x80041C070701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE1_TX_STAT1_PL              = 0x80041C050801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE1_TX_STAT1_PL              = 0x80041C050801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE1_TX_STAT1_PL              = 0x80041C070801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE1_TX_STAT1_PL               = 0x80041C050701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE1_TX_STAT1_PL               = 0x80041C050701143Full;

static const uint64_t P9A_OMI_6_TX0_TXPACKS1_SLICE1_TX_STAT1_PL               = 0x80041C070701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE1_TX_STAT1_PL               = 0x80041C050801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE1_TX_STAT1_PL              = 0x80041C050801143Full;

static const uint64_t P9A_OMI_14_TX0_TXPACKS1_SLICE1_TX_STAT1_PL              = 0x80041C070801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE2_TX_CNTL1G_PL                 = 0x800414060701103Full;
//DUPS: 800414060701143F, 800414080701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE2_TX_CNTL1G_PL               = 0x800414060701103Full;
//DUPS: 800414060701143F, 800414080701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE2_TX_CNTL1G_PL               = 0x800414060801103Full;
//DUPS: 800414060801143F, 800414080801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE2_TX_CNTL1G_PL                = 0x800414060701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE2_TX_CNTL1G_PL               = 0x800414060701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE2_TX_CNTL1G_PL             = 0x800414060701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE2_TX_CNTL1G_PL             = 0x800414060701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE2_TX_CNTL1G_PL             = 0x800414080701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE2_TX_CNTL1G_PL             = 0x800414060801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE2_TX_CNTL1G_PL             = 0x800414060801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE2_TX_CNTL1G_PL             = 0x800414080801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE2_TX_CNTL1G_PL              = 0x800414060701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE2_TX_CNTL1G_PL              = 0x800414060701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE2_TX_CNTL1G_PL              = 0x800414080701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE2_TX_CNTL1G_PL              = 0x800414060801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE2_TX_CNTL1G_PL             = 0x800414060801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE2_TX_CNTL1G_PL             = 0x800414080801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE2_TX_CNTL2_O_PL                = 0x80044C060701103Full;
//DUPS: 80044C060701143F, 80044C080701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE2_TX_CNTL2_O_PL              = 0x80044C060701103Full;
//DUPS: 80044C060701143F, 80044C080701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE2_TX_CNTL2_O_PL              = 0x80044C060801103Full;
//DUPS: 80044C060801143F, 80044C080801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE2_TX_CNTL2_O_PL               = 0x80044C060701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE2_TX_CNTL2_O_PL              = 0x80044C060701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE2_TX_CNTL2_O_PL            = 0x80044C060701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE2_TX_CNTL2_O_PL            = 0x80044C060701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE2_TX_CNTL2_O_PL            = 0x80044C080701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE2_TX_CNTL2_O_PL            = 0x80044C060801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE2_TX_CNTL2_O_PL            = 0x80044C060801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE2_TX_CNTL2_O_PL            = 0x80044C080801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE2_TX_CNTL2_O_PL             = 0x80044C060701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE2_TX_CNTL2_O_PL             = 0x80044C060701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE2_TX_CNTL2_O_PL             = 0x80044C080701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE2_TX_CNTL2_O_PL             = 0x80044C060801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE2_TX_CNTL2_O_PL            = 0x80044C060801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE2_TX_CNTL2_O_PL            = 0x80044C080801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE2_TX_CNTL3_EO_PL               = 0x800454060701103Full;
//DUPS: 800454060701143F, 800454080701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE2_TX_CNTL3_EO_PL             = 0x800454060701103Full;
//DUPS: 800454060701143F, 800454080701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE2_TX_CNTL3_EO_PL             = 0x800454060801103Full;
//DUPS: 800454060801143F, 800454080801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE2_TX_CNTL3_EO_PL              = 0x800454060701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE2_TX_CNTL3_EO_PL             = 0x800454060701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE2_TX_CNTL3_EO_PL           = 0x800454060701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE2_TX_CNTL3_EO_PL           = 0x800454060701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE2_TX_CNTL3_EO_PL           = 0x800454080701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE2_TX_CNTL3_EO_PL           = 0x800454060801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE2_TX_CNTL3_EO_PL           = 0x800454060801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE2_TX_CNTL3_EO_PL           = 0x800454080801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE2_TX_CNTL3_EO_PL            = 0x800454060701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE2_TX_CNTL3_EO_PL            = 0x800454060701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE2_TX_CNTL3_EO_PL            = 0x800454080701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE2_TX_CNTL3_EO_PL            = 0x800454060801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE2_TX_CNTL3_EO_PL           = 0x800454060801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE2_TX_CNTL3_EO_PL           = 0x800454080801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE2_TX_FIR_ERROR_INJECT_PL       = 0x800434060701103Full;
//DUPS: 800434060701143F, 800434080701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE2_TX_FIR_ERROR_INJECT_PL     = 0x800434060701103Full;
//DUPS: 800434060701143F, 800434080701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE2_TX_FIR_ERROR_INJECT_PL     = 0x800434060801103Full;
//DUPS: 800434060801143F, 800434080801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE2_TX_FIR_ERROR_INJECT_PL      = 0x800434060701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE2_TX_FIR_ERROR_INJECT_PL     = 0x800434060701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x800434060701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x800434060701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x800434080701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x800434060801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x800434060801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x800434080801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE2_TX_FIR_ERROR_INJECT_PL    = 0x800434060701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE2_TX_FIR_ERROR_INJECT_PL    = 0x800434060701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE2_TX_FIR_ERROR_INJECT_PL    = 0x800434080701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE2_TX_FIR_ERROR_INJECT_PL    = 0x800434060801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x800434060801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x800434080801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE2_TX_FIR_MASK_PL               = 0x80042C060701103Full;
//DUPS: 80042C060701143F, 80042C080701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE2_TX_FIR_MASK_PL             = 0x80042C060701103Full;
//DUPS: 80042C060701143F, 80042C080701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE2_TX_FIR_MASK_PL             = 0x80042C060801103Full;
//DUPS: 80042C060801143F, 80042C080801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE2_TX_FIR_MASK_PL              = 0x80042C060701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE2_TX_FIR_MASK_PL             = 0x80042C060701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE2_TX_FIR_MASK_PL           = 0x80042C060701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE2_TX_FIR_MASK_PL           = 0x80042C060701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE2_TX_FIR_MASK_PL           = 0x80042C080701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE2_TX_FIR_MASK_PL           = 0x80042C060801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE2_TX_FIR_MASK_PL           = 0x80042C060801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE2_TX_FIR_MASK_PL           = 0x80042C080801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE2_TX_FIR_MASK_PL            = 0x80042C060701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE2_TX_FIR_MASK_PL            = 0x80042C060701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE2_TX_FIR_MASK_PL            = 0x80042C080701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE2_TX_FIR_MASK_PL            = 0x80042C060801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE2_TX_FIR_MASK_PL           = 0x80042C060801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE2_TX_FIR_MASK_PL           = 0x80042C080801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE2_TX_FIR_PL                    = 0x800424060701103Full;
//DUPS: 800424060701143F, 800424080701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE2_TX_FIR_PL                  = 0x800424060701103Full;
//DUPS: 800424060701143F, 800424080701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE2_TX_FIR_PL                  = 0x800424060801103Full;
//DUPS: 800424060801143F, 800424080801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE2_TX_FIR_PL                   = 0x800424060701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE2_TX_FIR_PL                  = 0x800424060701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE2_TX_FIR_PL                = 0x800424060701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE2_TX_FIR_PL                = 0x800424060701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE2_TX_FIR_PL                = 0x800424080701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE2_TX_FIR_PL                = 0x800424060801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE2_TX_FIR_PL                = 0x800424060801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE2_TX_FIR_PL                = 0x800424080801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE2_TX_FIR_PL                 = 0x800424060701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE2_TX_FIR_PL                 = 0x800424060701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE2_TX_FIR_PL                 = 0x800424080701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE2_TX_FIR_PL                 = 0x800424060801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE2_TX_FIR_PL                = 0x800424060801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE2_TX_FIR_PL                = 0x800424080801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE2_TX_MODE1_PL                  = 0x800404060701103Full;
//DUPS: 800404060701143F, 800404080701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE2_TX_MODE1_PL                = 0x800404060701103Full;
//DUPS: 800404060701143F, 800404080701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE2_TX_MODE1_PL                = 0x800404060801103Full;
//DUPS: 800404060801143F, 800404080801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE2_TX_MODE1_PL                 = 0x800404060701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE2_TX_MODE1_PL                = 0x800404060701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE2_TX_MODE1_PL              = 0x800404060701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE2_TX_MODE1_PL              = 0x800404060701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE2_TX_MODE1_PL              = 0x800404080701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE2_TX_MODE1_PL              = 0x800404060801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE2_TX_MODE1_PL              = 0x800404060801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE2_TX_MODE1_PL              = 0x800404080801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE2_TX_MODE1_PL               = 0x800404060701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE2_TX_MODE1_PL               = 0x800404060701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE2_TX_MODE1_PL               = 0x800404080701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE2_TX_MODE1_PL               = 0x800404060801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE2_TX_MODE1_PL              = 0x800404060801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE2_TX_MODE1_PL              = 0x800404080801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE2_TX_MODE2_PL                  = 0x80040C060701103Full;
//DUPS: 80040C060701143F, 80040C080701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE2_TX_MODE2_PL                = 0x80040C060701103Full;
//DUPS: 80040C060701143F, 80040C080701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE2_TX_MODE2_PL                = 0x80040C060801103Full;
//DUPS: 80040C060801143F, 80040C080801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE2_TX_MODE2_PL                 = 0x80040C060701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE2_TX_MODE2_PL                = 0x80040C060701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE2_TX_MODE2_PL              = 0x80040C060701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE2_TX_MODE2_PL              = 0x80040C060701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE2_TX_MODE2_PL              = 0x80040C080701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE2_TX_MODE2_PL              = 0x80040C060801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE2_TX_MODE2_PL              = 0x80040C060801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE2_TX_MODE2_PL              = 0x80040C080801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE2_TX_MODE2_PL               = 0x80040C060701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE2_TX_MODE2_PL               = 0x80040C060701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE2_TX_MODE2_PL               = 0x80040C080701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE2_TX_MODE2_PL               = 0x80040C060801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE2_TX_MODE2_PL              = 0x80040C060801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE2_TX_MODE2_PL              = 0x80040C080801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE2_TX_STAT1_PL                  = 0x80041C060701103Full;
//DUPS: 80041C060701143F, 80041C080701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE2_TX_STAT1_PL                = 0x80041C060701103Full;
//DUPS: 80041C060701143F, 80041C080701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE2_TX_STAT1_PL                = 0x80041C060801103Full;
//DUPS: 80041C060801143F, 80041C080801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE2_TX_STAT1_PL                 = 0x80041C060701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE2_TX_STAT1_PL                = 0x80041C060701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE2_TX_STAT1_PL              = 0x80041C060701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE2_TX_STAT1_PL              = 0x80041C060701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE2_TX_STAT1_PL              = 0x80041C080701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE2_TX_STAT1_PL              = 0x80041C060801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE2_TX_STAT1_PL              = 0x80041C060801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE2_TX_STAT1_PL              = 0x80041C080801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE2_TX_STAT1_PL               = 0x80041C060701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE2_TX_STAT1_PL               = 0x80041C060701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE2_TX_STAT1_PL               = 0x80041C080701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE2_TX_STAT1_PL               = 0x80041C060801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE2_TX_STAT1_PL              = 0x80041C060801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE2_TX_STAT1_PL              = 0x80041C080801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE3_TX_CNTL1G_PL                 = 0x800414070701103Full;
//DUPS: 800414070701143F, 800414090701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE3_TX_CNTL1G_PL               = 0x800414070701103Full;
//DUPS: 800414070701143F, 800414090701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE3_TX_CNTL1G_PL               = 0x800414070801103Full;
//DUPS: 800414070801143F, 800414090801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE3_TX_CNTL1G_PL                = 0x800414070701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE3_TX_CNTL1G_PL               = 0x800414070701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE3_TX_CNTL1G_PL             = 0x800414070701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE3_TX_CNTL1G_PL             = 0x800414070701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE3_TX_CNTL1G_PL             = 0x800414090701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE3_TX_CNTL1G_PL             = 0x800414070801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE3_TX_CNTL1G_PL             = 0x800414070801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE3_TX_CNTL1G_PL             = 0x800414090801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE3_TX_CNTL1G_PL              = 0x800414070701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE3_TX_CNTL1G_PL              = 0x800414070701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE3_TX_CNTL1G_PL              = 0x800414090701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE3_TX_CNTL1G_PL              = 0x800414070801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE3_TX_CNTL1G_PL             = 0x800414070801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE3_TX_CNTL1G_PL             = 0x800414090801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE3_TX_CNTL2_O_PL                = 0x80044C070701103Full;
//DUPS: 80044C070701143F, 80044C090701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE3_TX_CNTL2_O_PL              = 0x80044C070701103Full;
//DUPS: 80044C070701143F, 80044C090701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE3_TX_CNTL2_O_PL              = 0x80044C070801103Full;
//DUPS: 80044C070801143F, 80044C090801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE3_TX_CNTL2_O_PL               = 0x80044C070701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE3_TX_CNTL2_O_PL              = 0x80044C070701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE3_TX_CNTL2_O_PL            = 0x80044C070701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE3_TX_CNTL2_O_PL            = 0x80044C070701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE3_TX_CNTL2_O_PL            = 0x80044C090701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE3_TX_CNTL2_O_PL            = 0x80044C070801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE3_TX_CNTL2_O_PL            = 0x80044C070801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE3_TX_CNTL2_O_PL            = 0x80044C090801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE3_TX_CNTL2_O_PL             = 0x80044C070701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE3_TX_CNTL2_O_PL             = 0x80044C070701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE3_TX_CNTL2_O_PL             = 0x80044C090701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE3_TX_CNTL2_O_PL             = 0x80044C070801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE3_TX_CNTL2_O_PL            = 0x80044C070801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE3_TX_CNTL2_O_PL            = 0x80044C090801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE3_TX_CNTL3_EO_PL               = 0x800454070701103Full;
//DUPS: 800454070701143F, 800454090701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE3_TX_CNTL3_EO_PL             = 0x800454070701103Full;
//DUPS: 800454070701143F, 800454090701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE3_TX_CNTL3_EO_PL             = 0x800454070801103Full;
//DUPS: 800454070801143F, 800454090801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE3_TX_CNTL3_EO_PL              = 0x800454070701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE3_TX_CNTL3_EO_PL             = 0x800454070701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE3_TX_CNTL3_EO_PL           = 0x800454070701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE3_TX_CNTL3_EO_PL           = 0x800454070701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE3_TX_CNTL3_EO_PL           = 0x800454090701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE3_TX_CNTL3_EO_PL           = 0x800454070801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE3_TX_CNTL3_EO_PL           = 0x800454070801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE3_TX_CNTL3_EO_PL           = 0x800454090801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE3_TX_CNTL3_EO_PL            = 0x800454070701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE3_TX_CNTL3_EO_PL            = 0x800454070701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE3_TX_CNTL3_EO_PL            = 0x800454090701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE3_TX_CNTL3_EO_PL            = 0x800454070801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE3_TX_CNTL3_EO_PL           = 0x800454070801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE3_TX_CNTL3_EO_PL           = 0x800454090801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE3_TX_FIR_ERROR_INJECT_PL       = 0x800434070701103Full;
//DUPS: 800434070701143F, 800434090701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE3_TX_FIR_ERROR_INJECT_PL     = 0x800434070701103Full;
//DUPS: 800434070701143F, 800434090701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE3_TX_FIR_ERROR_INJECT_PL     = 0x800434070801103Full;
//DUPS: 800434070801143F, 800434090801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE3_TX_FIR_ERROR_INJECT_PL      = 0x800434070701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE3_TX_FIR_ERROR_INJECT_PL     = 0x800434070701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x800434070701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x800434070701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x800434090701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x800434070801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x800434070801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x800434090801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE3_TX_FIR_ERROR_INJECT_PL    = 0x800434070701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE3_TX_FIR_ERROR_INJECT_PL    = 0x800434070701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE3_TX_FIR_ERROR_INJECT_PL    = 0x800434090701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE3_TX_FIR_ERROR_INJECT_PL    = 0x800434070801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x800434070801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x800434090801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE3_TX_FIR_MASK_PL               = 0x80042C070701103Full;
//DUPS: 80042C070701143F, 80042C090701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE3_TX_FIR_MASK_PL             = 0x80042C070701103Full;
//DUPS: 80042C070701143F, 80042C090701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE3_TX_FIR_MASK_PL             = 0x80042C070801103Full;
//DUPS: 80042C070801143F, 80042C090801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE3_TX_FIR_MASK_PL              = 0x80042C070701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE3_TX_FIR_MASK_PL             = 0x80042C070701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE3_TX_FIR_MASK_PL           = 0x80042C070701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE3_TX_FIR_MASK_PL           = 0x80042C070701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE3_TX_FIR_MASK_PL           = 0x80042C090701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE3_TX_FIR_MASK_PL           = 0x80042C070801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE3_TX_FIR_MASK_PL           = 0x80042C070801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE3_TX_FIR_MASK_PL           = 0x80042C090801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE3_TX_FIR_MASK_PL            = 0x80042C070701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE3_TX_FIR_MASK_PL            = 0x80042C070701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE3_TX_FIR_MASK_PL            = 0x80042C090701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE3_TX_FIR_MASK_PL            = 0x80042C070801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE3_TX_FIR_MASK_PL           = 0x80042C070801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE3_TX_FIR_MASK_PL           = 0x80042C090801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE3_TX_FIR_PL                    = 0x800424070701103Full;
//DUPS: 800424070701143F, 800424090701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE3_TX_FIR_PL                  = 0x800424070701103Full;
//DUPS: 800424070701143F, 800424090701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE3_TX_FIR_PL                  = 0x800424070801103Full;
//DUPS: 800424070801143F, 800424090801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE3_TX_FIR_PL                   = 0x800424070701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE3_TX_FIR_PL                  = 0x800424070701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE3_TX_FIR_PL                = 0x800424070701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE3_TX_FIR_PL                = 0x800424070701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE3_TX_FIR_PL                = 0x800424090701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE3_TX_FIR_PL                = 0x800424070801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE3_TX_FIR_PL                = 0x800424070801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE3_TX_FIR_PL                = 0x800424090801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE3_TX_FIR_PL                 = 0x800424070701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE3_TX_FIR_PL                 = 0x800424070701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE3_TX_FIR_PL                 = 0x800424090701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE3_TX_FIR_PL                 = 0x800424070801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE3_TX_FIR_PL                = 0x800424070801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE3_TX_FIR_PL                = 0x800424090801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE3_TX_MODE1_PL                  = 0x800404070701103Full;
//DUPS: 800404070701143F, 800404090701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE3_TX_MODE1_PL                = 0x800404070701103Full;
//DUPS: 800404070701143F, 800404090701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE3_TX_MODE1_PL                = 0x800404070801103Full;
//DUPS: 800404070801143F, 800404090801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE3_TX_MODE1_PL                 = 0x800404070701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE3_TX_MODE1_PL                = 0x800404070701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE3_TX_MODE1_PL              = 0x800404070701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE3_TX_MODE1_PL              = 0x800404070701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE3_TX_MODE1_PL              = 0x800404090701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE3_TX_MODE1_PL              = 0x800404070801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE3_TX_MODE1_PL              = 0x800404070801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE3_TX_MODE1_PL              = 0x800404090801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE3_TX_MODE1_PL               = 0x800404070701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE3_TX_MODE1_PL               = 0x800404070701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE3_TX_MODE1_PL               = 0x800404090701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE3_TX_MODE1_PL               = 0x800404070801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE3_TX_MODE1_PL              = 0x800404070801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE3_TX_MODE1_PL              = 0x800404090801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE3_TX_MODE2_PL                  = 0x80040C070701103Full;
//DUPS: 80040C070701143F, 80040C090701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE3_TX_MODE2_PL                = 0x80040C070701103Full;
//DUPS: 80040C070701143F, 80040C090701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE3_TX_MODE2_PL                = 0x80040C070801103Full;
//DUPS: 80040C070801143F, 80040C090801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE3_TX_MODE2_PL                 = 0x80040C070701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE3_TX_MODE2_PL                = 0x80040C070701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE3_TX_MODE2_PL              = 0x80040C070701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE3_TX_MODE2_PL              = 0x80040C070701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE3_TX_MODE2_PL              = 0x80040C090701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE3_TX_MODE2_PL              = 0x80040C070801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE3_TX_MODE2_PL              = 0x80040C070801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE3_TX_MODE2_PL              = 0x80040C090801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE3_TX_MODE2_PL               = 0x80040C070701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE3_TX_MODE2_PL               = 0x80040C070701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE3_TX_MODE2_PL               = 0x80040C090701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE3_TX_MODE2_PL               = 0x80040C070801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE3_TX_MODE2_PL              = 0x80040C070801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE3_TX_MODE2_PL              = 0x80040C090801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE3_TX_STAT1_PL                  = 0x80041C070701103Full;
//DUPS: 80041C070701143F, 80041C090701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE3_TX_STAT1_PL                = 0x80041C070701103Full;
//DUPS: 80041C070701143F, 80041C090701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE3_TX_STAT1_PL                = 0x80041C070801103Full;
//DUPS: 80041C070801143F, 80041C090801183F,

static const uint64_t P9A_OMI_TX0_TXPACKS1_SLICE3_TX_STAT1_PL                 = 0x80041C070701103Full;

static const uint64_t P9A_OMIC_TX0_TXPACKS1_SLICE3_TX_STAT1_PL                = 0x80041C070701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS1_SLICE3_TX_STAT1_PL              = 0x80041C070701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS1_SLICE3_TX_STAT1_PL              = 0x80041C070701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE3_TX_STAT1_PL              = 0x80041C090701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS1_SLICE3_TX_STAT1_PL              = 0x80041C070801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS1_SLICE3_TX_STAT1_PL              = 0x80041C070801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE3_TX_STAT1_PL              = 0x80041C090801183Full;

static const uint64_t P9A_OMI_0_TX0_TXPACKS1_SLICE3_TX_STAT1_PL               = 0x80041C070701103Full;

static const uint64_t P9A_OMI_3_TX0_TXPACKS1_SLICE3_TX_STAT1_PL               = 0x80041C070701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE3_TX_STAT1_PL               = 0x80041C090701183Full;

static const uint64_t P9A_OMI_8_TX0_TXPACKS1_SLICE3_TX_STAT1_PL               = 0x80041C070801103Full;

static const uint64_t P9A_OMI_11_TX0_TXPACKS1_SLICE3_TX_STAT1_PL              = 0x80041C070801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE3_TX_STAT1_PL              = 0x80041C090801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE4_TX_CNTL1G_PL                 = 0x8004140A0701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE4_TX_CNTL1G_PL               = 0x8004140A0701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE4_TX_CNTL1G_PL               = 0x8004140A0801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE4_TX_CNTL1G_PL             = 0x8004140A0701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE4_TX_CNTL1G_PL             = 0x8004140A0801183Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE4_TX_CNTL1G_PL              = 0x8004140A0701183Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE4_TX_CNTL1G_PL             = 0x8004140A0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE4_TX_CNTL2_O_PL                = 0x80044C0A0701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE4_TX_CNTL2_O_PL              = 0x80044C0A0701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE4_TX_CNTL2_O_PL              = 0x80044C0A0801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE4_TX_CNTL2_O_PL            = 0x80044C0A0701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE4_TX_CNTL2_O_PL            = 0x80044C0A0801183Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE4_TX_CNTL2_O_PL             = 0x80044C0A0701183Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE4_TX_CNTL2_O_PL            = 0x80044C0A0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE4_TX_CNTL3_EO_PL               = 0x8004540A0701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE4_TX_CNTL3_EO_PL             = 0x8004540A0701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE4_TX_CNTL3_EO_PL             = 0x8004540A0801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE4_TX_CNTL3_EO_PL           = 0x8004540A0701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE4_TX_CNTL3_EO_PL           = 0x8004540A0801183Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE4_TX_CNTL3_EO_PL            = 0x8004540A0701183Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE4_TX_CNTL3_EO_PL           = 0x8004540A0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE4_TX_FIR_ERROR_INJECT_PL       = 0x8004340A0701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE4_TX_FIR_ERROR_INJECT_PL     = 0x8004340A0701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE4_TX_FIR_ERROR_INJECT_PL     = 0x8004340A0801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE4_TX_FIR_ERROR_INJECT_PL   = 0x8004340A0701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE4_TX_FIR_ERROR_INJECT_PL   = 0x8004340A0801183Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE4_TX_FIR_ERROR_INJECT_PL    = 0x8004340A0701183Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE4_TX_FIR_ERROR_INJECT_PL   = 0x8004340A0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE4_TX_FIR_MASK_PL               = 0x80042C0A0701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE4_TX_FIR_MASK_PL             = 0x80042C0A0701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE4_TX_FIR_MASK_PL             = 0x80042C0A0801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE4_TX_FIR_MASK_PL           = 0x80042C0A0701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE4_TX_FIR_MASK_PL           = 0x80042C0A0801183Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE4_TX_FIR_MASK_PL            = 0x80042C0A0701183Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE4_TX_FIR_MASK_PL           = 0x80042C0A0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE4_TX_FIR_PL                    = 0x8004240A0701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE4_TX_FIR_PL                  = 0x8004240A0701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE4_TX_FIR_PL                  = 0x8004240A0801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE4_TX_FIR_PL                = 0x8004240A0701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE4_TX_FIR_PL                = 0x8004240A0801183Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE4_TX_FIR_PL                 = 0x8004240A0701183Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE4_TX_FIR_PL                = 0x8004240A0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE4_TX_MODE1_PL                  = 0x8004040A0701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE4_TX_MODE1_PL                = 0x8004040A0701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE4_TX_MODE1_PL                = 0x8004040A0801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE4_TX_MODE1_PL              = 0x8004040A0701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE4_TX_MODE1_PL              = 0x8004040A0801183Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE4_TX_MODE1_PL               = 0x8004040A0701183Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE4_TX_MODE1_PL              = 0x8004040A0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE4_TX_MODE2_PL                  = 0x80040C0A0701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE4_TX_MODE2_PL                = 0x80040C0A0701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE4_TX_MODE2_PL                = 0x80040C0A0801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE4_TX_MODE2_PL              = 0x80040C0A0701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE4_TX_MODE2_PL              = 0x80040C0A0801183Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE4_TX_MODE2_PL               = 0x80040C0A0701183Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE4_TX_MODE2_PL              = 0x80040C0A0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE4_TX_STAT1_PL                  = 0x80041C0A0701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE4_TX_STAT1_PL                = 0x80041C0A0701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE4_TX_STAT1_PL                = 0x80041C0A0801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE4_TX_STAT1_PL              = 0x80041C0A0701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE4_TX_STAT1_PL              = 0x80041C0A0801183Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE4_TX_STAT1_PL               = 0x80041C0A0701183Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE4_TX_STAT1_PL              = 0x80041C0A0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE5_TX_CNTL1G_PL                 = 0x8004140B0701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE5_TX_CNTL1G_PL               = 0x8004140B0701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE5_TX_CNTL1G_PL               = 0x8004140B0801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE5_TX_CNTL1G_PL             = 0x8004140B0701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE5_TX_CNTL1G_PL             = 0x8004140B0801183Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE5_TX_CNTL1G_PL              = 0x8004140B0701183Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE5_TX_CNTL1G_PL             = 0x8004140B0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE5_TX_CNTL2_O_PL                = 0x80044C0B0701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE5_TX_CNTL2_O_PL              = 0x80044C0B0701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE5_TX_CNTL2_O_PL              = 0x80044C0B0801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE5_TX_CNTL2_O_PL            = 0x80044C0B0701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE5_TX_CNTL2_O_PL            = 0x80044C0B0801183Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE5_TX_CNTL2_O_PL             = 0x80044C0B0701183Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE5_TX_CNTL2_O_PL            = 0x80044C0B0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE5_TX_CNTL3_EO_PL               = 0x8004540B0701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE5_TX_CNTL3_EO_PL             = 0x8004540B0701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE5_TX_CNTL3_EO_PL             = 0x8004540B0801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE5_TX_CNTL3_EO_PL           = 0x8004540B0701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE5_TX_CNTL3_EO_PL           = 0x8004540B0801183Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE5_TX_CNTL3_EO_PL            = 0x8004540B0701183Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE5_TX_CNTL3_EO_PL           = 0x8004540B0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE5_TX_FIR_ERROR_INJECT_PL       = 0x8004340B0701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE5_TX_FIR_ERROR_INJECT_PL     = 0x8004340B0701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE5_TX_FIR_ERROR_INJECT_PL     = 0x8004340B0801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE5_TX_FIR_ERROR_INJECT_PL   = 0x8004340B0701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE5_TX_FIR_ERROR_INJECT_PL   = 0x8004340B0801183Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE5_TX_FIR_ERROR_INJECT_PL    = 0x8004340B0701183Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE5_TX_FIR_ERROR_INJECT_PL   = 0x8004340B0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE5_TX_FIR_MASK_PL               = 0x80042C0B0701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE5_TX_FIR_MASK_PL             = 0x80042C0B0701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE5_TX_FIR_MASK_PL             = 0x80042C0B0801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE5_TX_FIR_MASK_PL           = 0x80042C0B0701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE5_TX_FIR_MASK_PL           = 0x80042C0B0801183Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE5_TX_FIR_MASK_PL            = 0x80042C0B0701183Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE5_TX_FIR_MASK_PL           = 0x80042C0B0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE5_TX_FIR_PL                    = 0x8004240B0701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE5_TX_FIR_PL                  = 0x8004240B0701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE5_TX_FIR_PL                  = 0x8004240B0801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE5_TX_FIR_PL                = 0x8004240B0701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE5_TX_FIR_PL                = 0x8004240B0801183Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE5_TX_FIR_PL                 = 0x8004240B0701183Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE5_TX_FIR_PL                = 0x8004240B0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE5_TX_MODE1_PL                  = 0x8004040B0701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE5_TX_MODE1_PL                = 0x8004040B0701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE5_TX_MODE1_PL                = 0x8004040B0801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE5_TX_MODE1_PL              = 0x8004040B0701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE5_TX_MODE1_PL              = 0x8004040B0801183Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE5_TX_MODE1_PL               = 0x8004040B0701183Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE5_TX_MODE1_PL              = 0x8004040B0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE5_TX_MODE2_PL                  = 0x80040C0B0701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE5_TX_MODE2_PL                = 0x80040C0B0701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE5_TX_MODE2_PL                = 0x80040C0B0801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE5_TX_MODE2_PL              = 0x80040C0B0701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE5_TX_MODE2_PL              = 0x80040C0B0801183Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE5_TX_MODE2_PL               = 0x80040C0B0701183Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE5_TX_MODE2_PL              = 0x80040C0B0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS1_SLICE5_TX_STAT1_PL                  = 0x80041C0B0701183Full;

static const uint64_t P9A_MC_0_TX0_TXPACKS1_SLICE5_TX_STAT1_PL                = 0x80041C0B0701183Full;

static const uint64_t P9A_MC_1_TX0_TXPACKS1_SLICE5_TX_STAT1_PL                = 0x80041C0B0801183Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS1_SLICE5_TX_STAT1_PL              = 0x80041C0B0701183Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS1_SLICE5_TX_STAT1_PL              = 0x80041C0B0801183Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS1_SLICE5_TX_STAT1_PL               = 0x80041C0B0701183Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS1_SLICE5_TX_STAT1_PL              = 0x80041C0B0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE0_TX_CNTL1G_PL                 = 0x8004140C0701183Full;
//DUPS: 800414080701143F, 8004140C0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE0_TX_CNTL1G_PL               = 0x8004140C0701183Full;
//DUPS: 800414080701143F, 8004140C0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE0_TX_CNTL1G_PL               = 0x8004140C0801183Full;
//DUPS: 800414080801143F, 8004140C0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE0_TX_CNTL1G_PL               = 0x800414080701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE0_TX_CNTL1G_PL             = 0x800414080701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE0_TX_CNTL1G_PL             = 0x800414080701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE0_TX_CNTL1G_PL             = 0x8004140C0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE0_TX_CNTL1G_PL             = 0x800414080801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE0_TX_CNTL1G_PL             = 0x800414080801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE0_TX_CNTL1G_PL             = 0x8004140C0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE0_TX_CNTL1G_PL              = 0x800414080701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE0_TX_CNTL1G_PL              = 0x800414080701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE0_TX_CNTL1G_PL              = 0x8004140C0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE0_TX_CNTL1G_PL              = 0x800414080801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE0_TX_CNTL1G_PL             = 0x800414080801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE0_TX_CNTL1G_PL             = 0x8004140C0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE0_TX_CNTL2_O_PL                = 0x80044C080701103Full;
//DUPS: 80044C080701143F, 80044C0C0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE0_TX_CNTL2_O_PL              = 0x80044C080701103Full;
//DUPS: 80044C080701143F, 80044C0C0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE0_TX_CNTL2_O_PL              = 0x80044C080801103Full;
//DUPS: 80044C080801143F, 80044C0C0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE0_TX_CNTL2_O_PL              = 0x80044C080701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE0_TX_CNTL2_O_PL            = 0x80044C080701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE0_TX_CNTL2_O_PL            = 0x80044C080701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE0_TX_CNTL2_O_PL            = 0x80044C0C0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE0_TX_CNTL2_O_PL            = 0x80044C080801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE0_TX_CNTL2_O_PL            = 0x80044C080801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE0_TX_CNTL2_O_PL            = 0x80044C0C0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE0_TX_CNTL2_O_PL             = 0x80044C080701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE0_TX_CNTL2_O_PL             = 0x80044C080701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE0_TX_CNTL2_O_PL             = 0x80044C0C0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE0_TX_CNTL2_O_PL             = 0x80044C080801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE0_TX_CNTL2_O_PL            = 0x80044C080801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE0_TX_CNTL2_O_PL            = 0x80044C0C0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE0_TX_CNTL3_EO_PL               = 0x8004540C0701183Full;
//DUPS: 800454080701143F, 8004540C0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE0_TX_CNTL3_EO_PL             = 0x8004540C0701183Full;
//DUPS: 800454080701143F, 8004540C0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE0_TX_CNTL3_EO_PL             = 0x8004540C0801183Full;
//DUPS: 800454080801143F, 8004540C0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE0_TX_CNTL3_EO_PL             = 0x800454080701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE0_TX_CNTL3_EO_PL           = 0x800454080701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE0_TX_CNTL3_EO_PL           = 0x800454080701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE0_TX_CNTL3_EO_PL           = 0x8004540C0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE0_TX_CNTL3_EO_PL           = 0x800454080801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE0_TX_CNTL3_EO_PL           = 0x800454080801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE0_TX_CNTL3_EO_PL           = 0x8004540C0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE0_TX_CNTL3_EO_PL            = 0x800454080701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE0_TX_CNTL3_EO_PL            = 0x800454080701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE0_TX_CNTL3_EO_PL            = 0x8004540C0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE0_TX_CNTL3_EO_PL            = 0x800454080801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE0_TX_CNTL3_EO_PL           = 0x800454080801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE0_TX_CNTL3_EO_PL           = 0x8004540C0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE0_TX_FIR_ERROR_INJECT_PL       = 0x8004340C0701183Full;
//DUPS: 800434080701143F, 8004340C0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE0_TX_FIR_ERROR_INJECT_PL     = 0x8004340C0701183Full;
//DUPS: 800434080701143F, 8004340C0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE0_TX_FIR_ERROR_INJECT_PL     = 0x8004340C0801183Full;
//DUPS: 800434080801143F, 8004340C0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE0_TX_FIR_ERROR_INJECT_PL     = 0x800434080701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x800434080701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x800434080701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x8004340C0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x800434080801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x800434080801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x8004340C0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE0_TX_FIR_ERROR_INJECT_PL    = 0x800434080701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE0_TX_FIR_ERROR_INJECT_PL    = 0x800434080701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE0_TX_FIR_ERROR_INJECT_PL    = 0x8004340C0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE0_TX_FIR_ERROR_INJECT_PL    = 0x800434080801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x800434080801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x8004340C0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE0_TX_FIR_MASK_PL               = 0x80042C080701103Full;
//DUPS: 80042C080701143F, 80042C0C0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE0_TX_FIR_MASK_PL             = 0x80042C080701103Full;
//DUPS: 80042C080701143F, 80042C0C0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE0_TX_FIR_MASK_PL             = 0x80042C080801103Full;
//DUPS: 80042C080801143F, 80042C0C0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE0_TX_FIR_MASK_PL             = 0x80042C080701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE0_TX_FIR_MASK_PL           = 0x80042C080701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE0_TX_FIR_MASK_PL           = 0x80042C080701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE0_TX_FIR_MASK_PL           = 0x80042C0C0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE0_TX_FIR_MASK_PL           = 0x80042C080801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE0_TX_FIR_MASK_PL           = 0x80042C080801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE0_TX_FIR_MASK_PL           = 0x80042C0C0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE0_TX_FIR_MASK_PL            = 0x80042C080701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE0_TX_FIR_MASK_PL            = 0x80042C080701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE0_TX_FIR_MASK_PL            = 0x80042C0C0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE0_TX_FIR_MASK_PL            = 0x80042C080801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE0_TX_FIR_MASK_PL           = 0x80042C080801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE0_TX_FIR_MASK_PL           = 0x80042C0C0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE0_TX_FIR_PL                    = 0x8004240C0701183Full;
//DUPS: 800424080701143F, 8004240C0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE0_TX_FIR_PL                  = 0x8004240C0701183Full;
//DUPS: 800424080701143F, 8004240C0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE0_TX_FIR_PL                  = 0x8004240C0801183Full;
//DUPS: 800424080801143F, 8004240C0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE0_TX_FIR_PL                  = 0x800424080701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE0_TX_FIR_PL                = 0x800424080701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE0_TX_FIR_PL                = 0x800424080701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE0_TX_FIR_PL                = 0x8004240C0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE0_TX_FIR_PL                = 0x800424080801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE0_TX_FIR_PL                = 0x800424080801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE0_TX_FIR_PL                = 0x8004240C0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE0_TX_FIR_PL                 = 0x800424080701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE0_TX_FIR_PL                 = 0x800424080701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE0_TX_FIR_PL                 = 0x8004240C0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE0_TX_FIR_PL                 = 0x800424080801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE0_TX_FIR_PL                = 0x800424080801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE0_TX_FIR_PL                = 0x8004240C0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE0_TX_MODE1_PL                  = 0x8004040C0701183Full;
//DUPS: 800404080701143F, 8004040C0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE0_TX_MODE1_PL                = 0x8004040C0701183Full;
//DUPS: 800404080701143F, 8004040C0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE0_TX_MODE1_PL                = 0x8004040C0801183Full;
//DUPS: 800404080801143F, 8004040C0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE0_TX_MODE1_PL                = 0x800404080701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE0_TX_MODE1_PL              = 0x800404080701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE0_TX_MODE1_PL              = 0x800404080701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE0_TX_MODE1_PL              = 0x8004040C0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE0_TX_MODE1_PL              = 0x800404080801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE0_TX_MODE1_PL              = 0x800404080801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE0_TX_MODE1_PL              = 0x8004040C0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE0_TX_MODE1_PL               = 0x800404080701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE0_TX_MODE1_PL               = 0x800404080701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE0_TX_MODE1_PL               = 0x8004040C0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE0_TX_MODE1_PL               = 0x800404080801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE0_TX_MODE1_PL              = 0x800404080801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE0_TX_MODE1_PL              = 0x8004040C0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE0_TX_MODE2_PL                  = 0x80040C080701103Full;
//DUPS: 80040C080701143F, 80040C0C0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE0_TX_MODE2_PL                = 0x80040C080701103Full;
//DUPS: 80040C080701143F, 80040C0C0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE0_TX_MODE2_PL                = 0x80040C080801103Full;
//DUPS: 80040C080801143F, 80040C0C0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE0_TX_MODE2_PL                = 0x80040C080701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE0_TX_MODE2_PL              = 0x80040C080701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE0_TX_MODE2_PL              = 0x80040C080701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE0_TX_MODE2_PL              = 0x80040C0C0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE0_TX_MODE2_PL              = 0x80040C080801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE0_TX_MODE2_PL              = 0x80040C080801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE0_TX_MODE2_PL              = 0x80040C0C0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE0_TX_MODE2_PL               = 0x80040C080701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE0_TX_MODE2_PL               = 0x80040C080701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE0_TX_MODE2_PL               = 0x80040C0C0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE0_TX_MODE2_PL               = 0x80040C080801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE0_TX_MODE2_PL              = 0x80040C080801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE0_TX_MODE2_PL              = 0x80040C0C0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE0_TX_STAT1_PL                  = 0x80041C080701103Full;
//DUPS: 80041C080701143F, 80041C0C0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE0_TX_STAT1_PL                = 0x80041C080701103Full;
//DUPS: 80041C080701143F, 80041C0C0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE0_TX_STAT1_PL                = 0x80041C080801103Full;
//DUPS: 80041C080801143F, 80041C0C0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE0_TX_STAT1_PL                = 0x80041C080701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE0_TX_STAT1_PL              = 0x80041C080701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE0_TX_STAT1_PL              = 0x80041C080701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE0_TX_STAT1_PL              = 0x80041C0C0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE0_TX_STAT1_PL              = 0x80041C080801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE0_TX_STAT1_PL              = 0x80041C080801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE0_TX_STAT1_PL              = 0x80041C0C0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE0_TX_STAT1_PL               = 0x80041C080701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE0_TX_STAT1_PL               = 0x80041C080701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE0_TX_STAT1_PL               = 0x80041C0C0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE0_TX_STAT1_PL               = 0x80041C080801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE0_TX_STAT1_PL              = 0x80041C080801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE0_TX_STAT1_PL              = 0x80041C0C0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE1_TX_CNTL1G_PL                 = 0x8004140D0701183Full;
//DUPS: 800414090701143F, 8004140D0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE1_TX_CNTL1G_PL               = 0x8004140D0701183Full;
//DUPS: 800414090701143F, 8004140D0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE1_TX_CNTL1G_PL               = 0x8004140D0801183Full;
//DUPS: 800414090801143F, 8004140D0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE1_TX_CNTL1G_PL               = 0x800414090701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE1_TX_CNTL1G_PL             = 0x800414090701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE1_TX_CNTL1G_PL             = 0x800414090701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE1_TX_CNTL1G_PL             = 0x8004140D0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE1_TX_CNTL1G_PL             = 0x800414090801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE1_TX_CNTL1G_PL             = 0x800414090801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE1_TX_CNTL1G_PL             = 0x8004140D0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE1_TX_CNTL1G_PL              = 0x800414090701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE1_TX_CNTL1G_PL              = 0x800414090701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE1_TX_CNTL1G_PL              = 0x8004140D0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE1_TX_CNTL1G_PL              = 0x800414090801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE1_TX_CNTL1G_PL             = 0x800414090801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE1_TX_CNTL1G_PL             = 0x8004140D0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE1_TX_CNTL2_O_PL                = 0x80044C090701103Full;
//DUPS: 80044C090701143F, 80044C0D0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE1_TX_CNTL2_O_PL              = 0x80044C090701103Full;
//DUPS: 80044C090701143F, 80044C0D0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE1_TX_CNTL2_O_PL              = 0x80044C090801103Full;
//DUPS: 80044C090801143F, 80044C0D0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE1_TX_CNTL2_O_PL              = 0x80044C090701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE1_TX_CNTL2_O_PL            = 0x80044C090701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE1_TX_CNTL2_O_PL            = 0x80044C090701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE1_TX_CNTL2_O_PL            = 0x80044C0D0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE1_TX_CNTL2_O_PL            = 0x80044C090801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE1_TX_CNTL2_O_PL            = 0x80044C090801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE1_TX_CNTL2_O_PL            = 0x80044C0D0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE1_TX_CNTL2_O_PL             = 0x80044C090701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE1_TX_CNTL2_O_PL             = 0x80044C090701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE1_TX_CNTL2_O_PL             = 0x80044C0D0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE1_TX_CNTL2_O_PL             = 0x80044C090801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE1_TX_CNTL2_O_PL            = 0x80044C090801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE1_TX_CNTL2_O_PL            = 0x80044C0D0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE1_TX_CNTL3_EO_PL               = 0x8004540D0701183Full;
//DUPS: 800454090701143F, 8004540D0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE1_TX_CNTL3_EO_PL             = 0x8004540D0701183Full;
//DUPS: 800454090701143F, 8004540D0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE1_TX_CNTL3_EO_PL             = 0x8004540D0801183Full;
//DUPS: 800454090801143F, 8004540D0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE1_TX_CNTL3_EO_PL             = 0x800454090701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE1_TX_CNTL3_EO_PL           = 0x800454090701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE1_TX_CNTL3_EO_PL           = 0x800454090701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE1_TX_CNTL3_EO_PL           = 0x8004540D0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE1_TX_CNTL3_EO_PL           = 0x800454090801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE1_TX_CNTL3_EO_PL           = 0x800454090801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE1_TX_CNTL3_EO_PL           = 0x8004540D0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE1_TX_CNTL3_EO_PL            = 0x800454090701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE1_TX_CNTL3_EO_PL            = 0x800454090701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE1_TX_CNTL3_EO_PL            = 0x8004540D0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE1_TX_CNTL3_EO_PL            = 0x800454090801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE1_TX_CNTL3_EO_PL           = 0x800454090801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE1_TX_CNTL3_EO_PL           = 0x8004540D0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE1_TX_FIR_ERROR_INJECT_PL       = 0x8004340D0701183Full;
//DUPS: 800434090701143F, 8004340D0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE1_TX_FIR_ERROR_INJECT_PL     = 0x8004340D0701183Full;
//DUPS: 800434090701143F, 8004340D0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE1_TX_FIR_ERROR_INJECT_PL     = 0x8004340D0801183Full;
//DUPS: 800434090801143F, 8004340D0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE1_TX_FIR_ERROR_INJECT_PL     = 0x800434090701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x800434090701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x800434090701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x8004340D0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x800434090801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x800434090801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x8004340D0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE1_TX_FIR_ERROR_INJECT_PL    = 0x800434090701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE1_TX_FIR_ERROR_INJECT_PL    = 0x800434090701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE1_TX_FIR_ERROR_INJECT_PL    = 0x8004340D0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE1_TX_FIR_ERROR_INJECT_PL    = 0x800434090801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x800434090801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x8004340D0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE1_TX_FIR_MASK_PL               = 0x80042C090701103Full;
//DUPS: 80042C090701143F, 80042C0D0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE1_TX_FIR_MASK_PL             = 0x80042C090701103Full;
//DUPS: 80042C090701143F, 80042C0D0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE1_TX_FIR_MASK_PL             = 0x80042C090801103Full;
//DUPS: 80042C090801143F, 80042C0D0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE1_TX_FIR_MASK_PL             = 0x80042C090701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE1_TX_FIR_MASK_PL           = 0x80042C090701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE1_TX_FIR_MASK_PL           = 0x80042C090701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE1_TX_FIR_MASK_PL           = 0x80042C0D0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE1_TX_FIR_MASK_PL           = 0x80042C090801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE1_TX_FIR_MASK_PL           = 0x80042C090801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE1_TX_FIR_MASK_PL           = 0x80042C0D0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE1_TX_FIR_MASK_PL            = 0x80042C090701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE1_TX_FIR_MASK_PL            = 0x80042C090701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE1_TX_FIR_MASK_PL            = 0x80042C0D0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE1_TX_FIR_MASK_PL            = 0x80042C090801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE1_TX_FIR_MASK_PL           = 0x80042C090801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE1_TX_FIR_MASK_PL           = 0x80042C0D0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE1_TX_FIR_PL                    = 0x8004240D0701183Full;
//DUPS: 800424090701143F, 8004240D0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE1_TX_FIR_PL                  = 0x8004240D0701183Full;
//DUPS: 800424090701143F, 8004240D0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE1_TX_FIR_PL                  = 0x8004240D0801183Full;
//DUPS: 800424090801143F, 8004240D0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE1_TX_FIR_PL                  = 0x800424090701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE1_TX_FIR_PL                = 0x800424090701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE1_TX_FIR_PL                = 0x800424090701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE1_TX_FIR_PL                = 0x8004240D0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE1_TX_FIR_PL                = 0x800424090801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE1_TX_FIR_PL                = 0x800424090801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE1_TX_FIR_PL                = 0x8004240D0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE1_TX_FIR_PL                 = 0x800424090701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE1_TX_FIR_PL                 = 0x800424090701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE1_TX_FIR_PL                 = 0x8004240D0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE1_TX_FIR_PL                 = 0x800424090801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE1_TX_FIR_PL                = 0x800424090801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE1_TX_FIR_PL                = 0x8004240D0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE1_TX_MODE1_PL                  = 0x8004040D0701183Full;
//DUPS: 800404090701143F, 8004040D0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE1_TX_MODE1_PL                = 0x8004040D0701183Full;
//DUPS: 800404090701143F, 8004040D0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE1_TX_MODE1_PL                = 0x8004040D0801183Full;
//DUPS: 800404090801143F, 8004040D0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE1_TX_MODE1_PL                = 0x800404090701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE1_TX_MODE1_PL              = 0x800404090701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE1_TX_MODE1_PL              = 0x800404090701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE1_TX_MODE1_PL              = 0x8004040D0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE1_TX_MODE1_PL              = 0x800404090801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE1_TX_MODE1_PL              = 0x800404090801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE1_TX_MODE1_PL              = 0x8004040D0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE1_TX_MODE1_PL               = 0x800404090701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE1_TX_MODE1_PL               = 0x800404090701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE1_TX_MODE1_PL               = 0x8004040D0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE1_TX_MODE1_PL               = 0x800404090801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE1_TX_MODE1_PL              = 0x800404090801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE1_TX_MODE1_PL              = 0x8004040D0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE1_TX_MODE2_PL                  = 0x80040C090701103Full;
//DUPS: 80040C090701143F, 80040C0D0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE1_TX_MODE2_PL                = 0x80040C090701103Full;
//DUPS: 80040C090701143F, 80040C0D0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE1_TX_MODE2_PL                = 0x80040C090801103Full;
//DUPS: 80040C090801143F, 80040C0D0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE1_TX_MODE2_PL                = 0x80040C090701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE1_TX_MODE2_PL              = 0x80040C090701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE1_TX_MODE2_PL              = 0x80040C090701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE1_TX_MODE2_PL              = 0x80040C0D0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE1_TX_MODE2_PL              = 0x80040C090801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE1_TX_MODE2_PL              = 0x80040C090801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE1_TX_MODE2_PL              = 0x80040C0D0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE1_TX_MODE2_PL               = 0x80040C090701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE1_TX_MODE2_PL               = 0x80040C090701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE1_TX_MODE2_PL               = 0x80040C0D0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE1_TX_MODE2_PL               = 0x80040C090801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE1_TX_MODE2_PL              = 0x80040C090801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE1_TX_MODE2_PL              = 0x80040C0D0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE1_TX_STAT1_PL                  = 0x80041C090701103Full;
//DUPS: 80041C090701143F, 80041C0D0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE1_TX_STAT1_PL                = 0x80041C090701103Full;
//DUPS: 80041C090701143F, 80041C0D0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE1_TX_STAT1_PL                = 0x80041C090801103Full;
//DUPS: 80041C090801143F, 80041C0D0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE1_TX_STAT1_PL                = 0x80041C090701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE1_TX_STAT1_PL              = 0x80041C090701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE1_TX_STAT1_PL              = 0x80041C090701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE1_TX_STAT1_PL              = 0x80041C0D0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE1_TX_STAT1_PL              = 0x80041C090801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE1_TX_STAT1_PL              = 0x80041C090801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE1_TX_STAT1_PL              = 0x80041C0D0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE1_TX_STAT1_PL               = 0x80041C090701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE1_TX_STAT1_PL               = 0x80041C090701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE1_TX_STAT1_PL               = 0x80041C0D0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE1_TX_STAT1_PL               = 0x80041C090801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE1_TX_STAT1_PL              = 0x80041C090801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE1_TX_STAT1_PL              = 0x80041C0D0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE2_TX_CNTL1G_PL                 = 0x8004140A0701103Full;
//DUPS: 8004140A0701143F, 8004140E0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE2_TX_CNTL1G_PL               = 0x8004140A0701103Full;
//DUPS: 8004140A0701143F, 8004140E0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE2_TX_CNTL1G_PL               = 0x8004140A0801103Full;
//DUPS: 8004140A0801143F, 8004140E0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE2_TX_CNTL1G_PL               = 0x8004140A0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE2_TX_CNTL1G_PL             = 0x8004140A0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE2_TX_CNTL1G_PL             = 0x8004140A0701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE2_TX_CNTL1G_PL             = 0x8004140E0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE2_TX_CNTL1G_PL             = 0x8004140A0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE2_TX_CNTL1G_PL             = 0x8004140A0801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE2_TX_CNTL1G_PL             = 0x8004140E0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE2_TX_CNTL1G_PL              = 0x8004140A0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE2_TX_CNTL1G_PL              = 0x8004140A0701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE2_TX_CNTL1G_PL              = 0x8004140E0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE2_TX_CNTL1G_PL              = 0x8004140A0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE2_TX_CNTL1G_PL             = 0x8004140A0801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE2_TX_CNTL1G_PL             = 0x8004140E0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE2_TX_CNTL2_O_PL                = 0x80044C0A0701103Full;
//DUPS: 80044C0A0701143F, 80044C0E0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE2_TX_CNTL2_O_PL              = 0x80044C0A0701103Full;
//DUPS: 80044C0A0701143F, 80044C0E0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE2_TX_CNTL2_O_PL              = 0x80044C0A0801103Full;
//DUPS: 80044C0A0801143F, 80044C0E0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE2_TX_CNTL2_O_PL              = 0x80044C0A0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE2_TX_CNTL2_O_PL            = 0x80044C0A0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE2_TX_CNTL2_O_PL            = 0x80044C0A0701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE2_TX_CNTL2_O_PL            = 0x80044C0E0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE2_TX_CNTL2_O_PL            = 0x80044C0A0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE2_TX_CNTL2_O_PL            = 0x80044C0A0801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE2_TX_CNTL2_O_PL            = 0x80044C0E0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE2_TX_CNTL2_O_PL             = 0x80044C0A0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE2_TX_CNTL2_O_PL             = 0x80044C0A0701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE2_TX_CNTL2_O_PL             = 0x80044C0E0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE2_TX_CNTL2_O_PL             = 0x80044C0A0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE2_TX_CNTL2_O_PL            = 0x80044C0A0801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE2_TX_CNTL2_O_PL            = 0x80044C0E0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE2_TX_CNTL3_EO_PL               = 0x8004540A0701103Full;
//DUPS: 8004540A0701143F, 8004540E0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE2_TX_CNTL3_EO_PL             = 0x8004540A0701103Full;
//DUPS: 8004540A0701143F, 8004540E0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE2_TX_CNTL3_EO_PL             = 0x8004540A0801103Full;
//DUPS: 8004540A0801143F, 8004540E0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE2_TX_CNTL3_EO_PL             = 0x8004540A0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE2_TX_CNTL3_EO_PL           = 0x8004540A0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE2_TX_CNTL3_EO_PL           = 0x8004540A0701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE2_TX_CNTL3_EO_PL           = 0x8004540E0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE2_TX_CNTL3_EO_PL           = 0x8004540A0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE2_TX_CNTL3_EO_PL           = 0x8004540A0801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE2_TX_CNTL3_EO_PL           = 0x8004540E0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE2_TX_CNTL3_EO_PL            = 0x8004540A0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE2_TX_CNTL3_EO_PL            = 0x8004540A0701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE2_TX_CNTL3_EO_PL            = 0x8004540E0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE2_TX_CNTL3_EO_PL            = 0x8004540A0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE2_TX_CNTL3_EO_PL           = 0x8004540A0801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE2_TX_CNTL3_EO_PL           = 0x8004540E0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE2_TX_FIR_ERROR_INJECT_PL       = 0x8004340A0701103Full;
//DUPS: 8004340A0701143F, 8004340E0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE2_TX_FIR_ERROR_INJECT_PL     = 0x8004340A0701103Full;
//DUPS: 8004340A0701143F, 8004340E0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE2_TX_FIR_ERROR_INJECT_PL     = 0x8004340A0801103Full;
//DUPS: 8004340A0801143F, 8004340E0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE2_TX_FIR_ERROR_INJECT_PL     = 0x8004340A0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x8004340A0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x8004340A0701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x8004340E0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x8004340A0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x8004340A0801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x8004340E0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE2_TX_FIR_ERROR_INJECT_PL    = 0x8004340A0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE2_TX_FIR_ERROR_INJECT_PL    = 0x8004340A0701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE2_TX_FIR_ERROR_INJECT_PL    = 0x8004340E0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE2_TX_FIR_ERROR_INJECT_PL    = 0x8004340A0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x8004340A0801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x8004340E0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE2_TX_FIR_MASK_PL               = 0x80042C0A0701103Full;
//DUPS: 80042C0A0701143F, 80042C0E0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE2_TX_FIR_MASK_PL             = 0x80042C0A0701103Full;
//DUPS: 80042C0A0701143F, 80042C0E0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE2_TX_FIR_MASK_PL             = 0x80042C0A0801103Full;
//DUPS: 80042C0A0801143F, 80042C0E0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE2_TX_FIR_MASK_PL             = 0x80042C0A0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE2_TX_FIR_MASK_PL           = 0x80042C0A0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE2_TX_FIR_MASK_PL           = 0x80042C0A0701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE2_TX_FIR_MASK_PL           = 0x80042C0E0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE2_TX_FIR_MASK_PL           = 0x80042C0A0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE2_TX_FIR_MASK_PL           = 0x80042C0A0801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE2_TX_FIR_MASK_PL           = 0x80042C0E0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE2_TX_FIR_MASK_PL            = 0x80042C0A0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE2_TX_FIR_MASK_PL            = 0x80042C0A0701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE2_TX_FIR_MASK_PL            = 0x80042C0E0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE2_TX_FIR_MASK_PL            = 0x80042C0A0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE2_TX_FIR_MASK_PL           = 0x80042C0A0801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE2_TX_FIR_MASK_PL           = 0x80042C0E0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE2_TX_FIR_PL                    = 0x8004240A0701103Full;
//DUPS: 8004240A0701143F, 8004240E0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE2_TX_FIR_PL                  = 0x8004240A0701103Full;
//DUPS: 8004240A0701143F, 8004240E0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE2_TX_FIR_PL                  = 0x8004240A0801103Full;
//DUPS: 8004240A0801143F, 8004240E0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE2_TX_FIR_PL                  = 0x8004240A0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE2_TX_FIR_PL                = 0x8004240A0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE2_TX_FIR_PL                = 0x8004240A0701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE2_TX_FIR_PL                = 0x8004240E0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE2_TX_FIR_PL                = 0x8004240A0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE2_TX_FIR_PL                = 0x8004240A0801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE2_TX_FIR_PL                = 0x8004240E0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE2_TX_FIR_PL                 = 0x8004240A0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE2_TX_FIR_PL                 = 0x8004240A0701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE2_TX_FIR_PL                 = 0x8004240E0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE2_TX_FIR_PL                 = 0x8004240A0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE2_TX_FIR_PL                = 0x8004240A0801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE2_TX_FIR_PL                = 0x8004240E0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE2_TX_MODE1_PL                  = 0x8004040A0701103Full;
//DUPS: 8004040A0701143F, 8004040E0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE2_TX_MODE1_PL                = 0x8004040A0701103Full;
//DUPS: 8004040A0701143F, 8004040E0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE2_TX_MODE1_PL                = 0x8004040A0801103Full;
//DUPS: 8004040A0801143F, 8004040E0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE2_TX_MODE1_PL                = 0x8004040A0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE2_TX_MODE1_PL              = 0x8004040A0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE2_TX_MODE1_PL              = 0x8004040A0701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE2_TX_MODE1_PL              = 0x8004040E0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE2_TX_MODE1_PL              = 0x8004040A0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE2_TX_MODE1_PL              = 0x8004040A0801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE2_TX_MODE1_PL              = 0x8004040E0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE2_TX_MODE1_PL               = 0x8004040A0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE2_TX_MODE1_PL               = 0x8004040A0701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE2_TX_MODE1_PL               = 0x8004040E0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE2_TX_MODE1_PL               = 0x8004040A0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE2_TX_MODE1_PL              = 0x8004040A0801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE2_TX_MODE1_PL              = 0x8004040E0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE2_TX_MODE2_PL                  = 0x80040C0A0701103Full;
//DUPS: 80040C0A0701143F, 80040C0E0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE2_TX_MODE2_PL                = 0x80040C0A0701103Full;
//DUPS: 80040C0A0701143F, 80040C0E0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE2_TX_MODE2_PL                = 0x80040C0A0801103Full;
//DUPS: 80040C0A0801143F, 80040C0E0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE2_TX_MODE2_PL                = 0x80040C0A0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE2_TX_MODE2_PL              = 0x80040C0A0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE2_TX_MODE2_PL              = 0x80040C0A0701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE2_TX_MODE2_PL              = 0x80040C0E0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE2_TX_MODE2_PL              = 0x80040C0A0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE2_TX_MODE2_PL              = 0x80040C0A0801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE2_TX_MODE2_PL              = 0x80040C0E0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE2_TX_MODE2_PL               = 0x80040C0A0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE2_TX_MODE2_PL               = 0x80040C0A0701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE2_TX_MODE2_PL               = 0x80040C0E0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE2_TX_MODE2_PL               = 0x80040C0A0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE2_TX_MODE2_PL              = 0x80040C0A0801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE2_TX_MODE2_PL              = 0x80040C0E0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE2_TX_STAT1_PL                  = 0x80041C0A0701103Full;
//DUPS: 80041C0A0701143F, 80041C0E0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE2_TX_STAT1_PL                = 0x80041C0A0701103Full;
//DUPS: 80041C0A0701143F, 80041C0E0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE2_TX_STAT1_PL                = 0x80041C0A0801103Full;
//DUPS: 80041C0A0801143F, 80041C0E0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE2_TX_STAT1_PL                = 0x80041C0A0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE2_TX_STAT1_PL              = 0x80041C0A0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE2_TX_STAT1_PL              = 0x80041C0A0701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE2_TX_STAT1_PL              = 0x80041C0E0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE2_TX_STAT1_PL              = 0x80041C0A0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE2_TX_STAT1_PL              = 0x80041C0A0801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE2_TX_STAT1_PL              = 0x80041C0E0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE2_TX_STAT1_PL               = 0x80041C0A0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE2_TX_STAT1_PL               = 0x80041C0A0701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE2_TX_STAT1_PL               = 0x80041C0E0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE2_TX_STAT1_PL               = 0x80041C0A0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE2_TX_STAT1_PL              = 0x80041C0A0801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE2_TX_STAT1_PL              = 0x80041C0E0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE3_TX_CNTL1G_PL                 = 0x8004140B0701103Full;
//DUPS: 8004140B0701143F, 8004140F0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE3_TX_CNTL1G_PL               = 0x8004140B0701103Full;
//DUPS: 8004140B0701143F, 8004140F0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE3_TX_CNTL1G_PL               = 0x8004140B0801103Full;
//DUPS: 8004140B0801143F, 8004140F0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE3_TX_CNTL1G_PL               = 0x8004140B0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE3_TX_CNTL1G_PL             = 0x8004140B0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE3_TX_CNTL1G_PL             = 0x8004140B0701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE3_TX_CNTL1G_PL             = 0x8004140F0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE3_TX_CNTL1G_PL             = 0x8004140B0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE3_TX_CNTL1G_PL             = 0x8004140B0801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE3_TX_CNTL1G_PL             = 0x8004140F0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE3_TX_CNTL1G_PL              = 0x8004140B0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE3_TX_CNTL1G_PL              = 0x8004140B0701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE3_TX_CNTL1G_PL              = 0x8004140F0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE3_TX_CNTL1G_PL              = 0x8004140B0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE3_TX_CNTL1G_PL             = 0x8004140B0801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE3_TX_CNTL1G_PL             = 0x8004140F0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE3_TX_CNTL2_O_PL                = 0x80044C0B0701103Full;
//DUPS: 80044C0B0701143F, 80044C0F0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE3_TX_CNTL2_O_PL              = 0x80044C0B0701103Full;
//DUPS: 80044C0B0701143F, 80044C0F0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE3_TX_CNTL2_O_PL              = 0x80044C0B0801103Full;
//DUPS: 80044C0B0801143F, 80044C0F0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE3_TX_CNTL2_O_PL              = 0x80044C0B0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE3_TX_CNTL2_O_PL            = 0x80044C0B0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE3_TX_CNTL2_O_PL            = 0x80044C0B0701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE3_TX_CNTL2_O_PL            = 0x80044C0F0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE3_TX_CNTL2_O_PL            = 0x80044C0B0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE3_TX_CNTL2_O_PL            = 0x80044C0B0801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE3_TX_CNTL2_O_PL            = 0x80044C0F0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE3_TX_CNTL2_O_PL             = 0x80044C0B0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE3_TX_CNTL2_O_PL             = 0x80044C0B0701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE3_TX_CNTL2_O_PL             = 0x80044C0F0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE3_TX_CNTL2_O_PL             = 0x80044C0B0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE3_TX_CNTL2_O_PL            = 0x80044C0B0801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE3_TX_CNTL2_O_PL            = 0x80044C0F0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE3_TX_CNTL3_EO_PL               = 0x8004540B0701103Full;
//DUPS: 8004540B0701143F, 8004540F0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE3_TX_CNTL3_EO_PL             = 0x8004540B0701103Full;
//DUPS: 8004540B0701143F, 8004540F0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE3_TX_CNTL3_EO_PL             = 0x8004540B0801103Full;
//DUPS: 8004540B0801143F, 8004540F0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE3_TX_CNTL3_EO_PL             = 0x8004540B0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE3_TX_CNTL3_EO_PL           = 0x8004540B0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE3_TX_CNTL3_EO_PL           = 0x8004540B0701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE3_TX_CNTL3_EO_PL           = 0x8004540F0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE3_TX_CNTL3_EO_PL           = 0x8004540B0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE3_TX_CNTL3_EO_PL           = 0x8004540B0801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE3_TX_CNTL3_EO_PL           = 0x8004540F0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE3_TX_CNTL3_EO_PL            = 0x8004540B0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE3_TX_CNTL3_EO_PL            = 0x8004540B0701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE3_TX_CNTL3_EO_PL            = 0x8004540F0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE3_TX_CNTL3_EO_PL            = 0x8004540B0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE3_TX_CNTL3_EO_PL           = 0x8004540B0801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE3_TX_CNTL3_EO_PL           = 0x8004540F0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE3_TX_FIR_ERROR_INJECT_PL       = 0x8004340B0701103Full;
//DUPS: 8004340B0701143F, 8004340F0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE3_TX_FIR_ERROR_INJECT_PL     = 0x8004340B0701103Full;
//DUPS: 8004340B0701143F, 8004340F0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE3_TX_FIR_ERROR_INJECT_PL     = 0x8004340B0801103Full;
//DUPS: 8004340B0801143F, 8004340F0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE3_TX_FIR_ERROR_INJECT_PL     = 0x8004340B0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x8004340B0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x8004340B0701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x8004340F0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x8004340B0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x8004340B0801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x8004340F0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE3_TX_FIR_ERROR_INJECT_PL    = 0x8004340B0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE3_TX_FIR_ERROR_INJECT_PL    = 0x8004340B0701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE3_TX_FIR_ERROR_INJECT_PL    = 0x8004340F0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE3_TX_FIR_ERROR_INJECT_PL    = 0x8004340B0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x8004340B0801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x8004340F0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE3_TX_FIR_MASK_PL               = 0x80042C0B0701103Full;
//DUPS: 80042C0B0701143F, 80042C0F0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE3_TX_FIR_MASK_PL             = 0x80042C0B0701103Full;
//DUPS: 80042C0B0701143F, 80042C0F0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE3_TX_FIR_MASK_PL             = 0x80042C0B0801103Full;
//DUPS: 80042C0B0801143F, 80042C0F0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE3_TX_FIR_MASK_PL             = 0x80042C0B0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE3_TX_FIR_MASK_PL           = 0x80042C0B0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE3_TX_FIR_MASK_PL           = 0x80042C0B0701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE3_TX_FIR_MASK_PL           = 0x80042C0F0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE3_TX_FIR_MASK_PL           = 0x80042C0B0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE3_TX_FIR_MASK_PL           = 0x80042C0B0801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE3_TX_FIR_MASK_PL           = 0x80042C0F0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE3_TX_FIR_MASK_PL            = 0x80042C0B0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE3_TX_FIR_MASK_PL            = 0x80042C0B0701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE3_TX_FIR_MASK_PL            = 0x80042C0F0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE3_TX_FIR_MASK_PL            = 0x80042C0B0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE3_TX_FIR_MASK_PL           = 0x80042C0B0801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE3_TX_FIR_MASK_PL           = 0x80042C0F0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE3_TX_FIR_PL                    = 0x8004240B0701103Full;
//DUPS: 8004240B0701143F, 8004240F0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE3_TX_FIR_PL                  = 0x8004240B0701103Full;
//DUPS: 8004240B0701143F, 8004240F0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE3_TX_FIR_PL                  = 0x8004240B0801103Full;
//DUPS: 8004240B0801143F, 8004240F0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE3_TX_FIR_PL                  = 0x8004240B0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE3_TX_FIR_PL                = 0x8004240B0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE3_TX_FIR_PL                = 0x8004240B0701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE3_TX_FIR_PL                = 0x8004240F0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE3_TX_FIR_PL                = 0x8004240B0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE3_TX_FIR_PL                = 0x8004240B0801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE3_TX_FIR_PL                = 0x8004240F0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE3_TX_FIR_PL                 = 0x8004240B0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE3_TX_FIR_PL                 = 0x8004240B0701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE3_TX_FIR_PL                 = 0x8004240F0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE3_TX_FIR_PL                 = 0x8004240B0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE3_TX_FIR_PL                = 0x8004240B0801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE3_TX_FIR_PL                = 0x8004240F0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE3_TX_MODE1_PL                  = 0x8004040B0701103Full;
//DUPS: 8004040B0701143F, 8004040F0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE3_TX_MODE1_PL                = 0x8004040B0701103Full;
//DUPS: 8004040B0701143F, 8004040F0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE3_TX_MODE1_PL                = 0x8004040B0801103Full;
//DUPS: 8004040B0801143F, 8004040F0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE3_TX_MODE1_PL                = 0x8004040B0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE3_TX_MODE1_PL              = 0x8004040B0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE3_TX_MODE1_PL              = 0x8004040B0701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE3_TX_MODE1_PL              = 0x8004040F0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE3_TX_MODE1_PL              = 0x8004040B0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE3_TX_MODE1_PL              = 0x8004040B0801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE3_TX_MODE1_PL              = 0x8004040F0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE3_TX_MODE1_PL               = 0x8004040B0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE3_TX_MODE1_PL               = 0x8004040B0701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE3_TX_MODE1_PL               = 0x8004040F0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE3_TX_MODE1_PL               = 0x8004040B0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE3_TX_MODE1_PL              = 0x8004040B0801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE3_TX_MODE1_PL              = 0x8004040F0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE3_TX_MODE2_PL                  = 0x80040C0B0701103Full;
//DUPS: 80040C0B0701143F, 80040C0F0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE3_TX_MODE2_PL                = 0x80040C0B0701103Full;
//DUPS: 80040C0B0701143F, 80040C0F0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE3_TX_MODE2_PL                = 0x80040C0B0801103Full;
//DUPS: 80040C0B0801143F, 80040C0F0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE3_TX_MODE2_PL                = 0x80040C0B0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE3_TX_MODE2_PL              = 0x80040C0B0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE3_TX_MODE2_PL              = 0x80040C0B0701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE3_TX_MODE2_PL              = 0x80040C0F0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE3_TX_MODE2_PL              = 0x80040C0B0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE3_TX_MODE2_PL              = 0x80040C0B0801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE3_TX_MODE2_PL              = 0x80040C0F0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE3_TX_MODE2_PL               = 0x80040C0B0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE3_TX_MODE2_PL               = 0x80040C0B0701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE3_TX_MODE2_PL               = 0x80040C0F0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE3_TX_MODE2_PL               = 0x80040C0B0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE3_TX_MODE2_PL              = 0x80040C0B0801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE3_TX_MODE2_PL              = 0x80040C0F0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS2_SLICE3_TX_STAT1_PL                  = 0x80041C0B0701103Full;
//DUPS: 80041C0B0701143F, 80041C0F0701183F,
static const uint64_t P9A_MC_0_TX0_TXPACKS2_SLICE3_TX_STAT1_PL                = 0x80041C0B0701103Full;
//DUPS: 80041C0B0701143F, 80041C0F0701183F,
static const uint64_t P9A_MC_1_TX0_TXPACKS2_SLICE3_TX_STAT1_PL                = 0x80041C0B0801103Full;
//DUPS: 80041C0B0801143F, 80041C0F0801183F,

static const uint64_t P9A_OMIC_TX0_TXPACKS2_SLICE3_TX_STAT1_PL                = 0x80041C0B0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS2_SLICE3_TX_STAT1_PL              = 0x80041C0B0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS2_SLICE3_TX_STAT1_PL              = 0x80041C0B0701143Full;

static const uint64_t P9A_OMIC_2_TX0_TXPACKS2_SLICE3_TX_STAT1_PL              = 0x80041C0F0701183Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS2_SLICE3_TX_STAT1_PL              = 0x80041C0B0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS2_SLICE3_TX_STAT1_PL              = 0x80041C0B0801143Full;

static const uint64_t P9A_OMIC_5_TX0_TXPACKS2_SLICE3_TX_STAT1_PL              = 0x80041C0F0801183Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS2_SLICE3_TX_STAT1_PL               = 0x80041C0B0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS2_SLICE3_TX_STAT1_PL               = 0x80041C0B0701143Full;

static const uint64_t P9A_OMI_7_TX0_TXPACKS2_SLICE3_TX_STAT1_PL               = 0x80041C0F0701183Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS2_SLICE3_TX_STAT1_PL               = 0x80041C0B0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS2_SLICE3_TX_STAT1_PL              = 0x80041C0B0801143Full;

static const uint64_t P9A_OMI_15_TX0_TXPACKS2_SLICE3_TX_STAT1_PL              = 0x80041C0F0801183Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE0_TX_CNTL1G_PL                 = 0x8004140C0701103Full;
//DUPS: 8004140C0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE0_TX_CNTL1G_PL               = 0x8004140C0701103Full;
//DUPS: 8004140C0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE0_TX_CNTL1G_PL               = 0x8004140C0801103Full;
//DUPS: 8004140C0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE0_TX_CNTL1G_PL               = 0x8004140C0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE0_TX_CNTL1G_PL             = 0x8004140C0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE0_TX_CNTL1G_PL             = 0x8004140C0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE0_TX_CNTL1G_PL             = 0x8004140C0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE0_TX_CNTL1G_PL             = 0x8004140C0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE0_TX_CNTL1G_PL              = 0x8004140C0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE0_TX_CNTL1G_PL              = 0x8004140C0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE0_TX_CNTL1G_PL              = 0x8004140C0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE0_TX_CNTL1G_PL             = 0x8004140C0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE0_TX_CNTL2_O_PL                = 0x80044C0C0701103Full;
//DUPS: 80044C0C0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE0_TX_CNTL2_O_PL              = 0x80044C0C0701103Full;
//DUPS: 80044C0C0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE0_TX_CNTL2_O_PL              = 0x80044C0C0801103Full;
//DUPS: 80044C0C0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE0_TX_CNTL2_O_PL              = 0x80044C0C0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE0_TX_CNTL2_O_PL            = 0x80044C0C0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE0_TX_CNTL2_O_PL            = 0x80044C0C0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE0_TX_CNTL2_O_PL            = 0x80044C0C0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE0_TX_CNTL2_O_PL            = 0x80044C0C0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE0_TX_CNTL2_O_PL             = 0x80044C0C0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE0_TX_CNTL2_O_PL             = 0x80044C0C0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE0_TX_CNTL2_O_PL             = 0x80044C0C0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE0_TX_CNTL2_O_PL            = 0x80044C0C0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE0_TX_CNTL3_EO_PL               = 0x8004540C0701103Full;
//DUPS: 8004540C0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE0_TX_CNTL3_EO_PL             = 0x8004540C0701103Full;
//DUPS: 8004540C0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE0_TX_CNTL3_EO_PL             = 0x8004540C0801103Full;
//DUPS: 8004540C0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE0_TX_CNTL3_EO_PL             = 0x8004540C0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE0_TX_CNTL3_EO_PL           = 0x8004540C0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE0_TX_CNTL3_EO_PL           = 0x8004540C0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE0_TX_CNTL3_EO_PL           = 0x8004540C0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE0_TX_CNTL3_EO_PL           = 0x8004540C0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE0_TX_CNTL3_EO_PL            = 0x8004540C0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE0_TX_CNTL3_EO_PL            = 0x8004540C0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE0_TX_CNTL3_EO_PL            = 0x8004540C0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE0_TX_CNTL3_EO_PL           = 0x8004540C0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE0_TX_FIR_ERROR_INJECT_PL       = 0x8004340C0701103Full;
//DUPS: 8004340C0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE0_TX_FIR_ERROR_INJECT_PL     = 0x8004340C0701103Full;
//DUPS: 8004340C0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE0_TX_FIR_ERROR_INJECT_PL     = 0x8004340C0801103Full;
//DUPS: 8004340C0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE0_TX_FIR_ERROR_INJECT_PL     = 0x8004340C0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x8004340C0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x8004340C0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x8004340C0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x8004340C0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE0_TX_FIR_ERROR_INJECT_PL    = 0x8004340C0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE0_TX_FIR_ERROR_INJECT_PL    = 0x8004340C0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE0_TX_FIR_ERROR_INJECT_PL    = 0x8004340C0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x8004340C0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE0_TX_FIR_MASK_PL               = 0x80042C0C0701103Full;
//DUPS: 80042C0C0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE0_TX_FIR_MASK_PL             = 0x80042C0C0701103Full;
//DUPS: 80042C0C0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE0_TX_FIR_MASK_PL             = 0x80042C0C0801103Full;
//DUPS: 80042C0C0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE0_TX_FIR_MASK_PL             = 0x80042C0C0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE0_TX_FIR_MASK_PL           = 0x80042C0C0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE0_TX_FIR_MASK_PL           = 0x80042C0C0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE0_TX_FIR_MASK_PL           = 0x80042C0C0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE0_TX_FIR_MASK_PL           = 0x80042C0C0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE0_TX_FIR_MASK_PL            = 0x80042C0C0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE0_TX_FIR_MASK_PL            = 0x80042C0C0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE0_TX_FIR_MASK_PL            = 0x80042C0C0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE0_TX_FIR_MASK_PL           = 0x80042C0C0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE0_TX_FIR_PL                    = 0x8004240C0701103Full;
//DUPS: 8004240C0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE0_TX_FIR_PL                  = 0x8004240C0701103Full;
//DUPS: 8004240C0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE0_TX_FIR_PL                  = 0x8004240C0801103Full;
//DUPS: 8004240C0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE0_TX_FIR_PL                  = 0x8004240C0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE0_TX_FIR_PL                = 0x8004240C0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE0_TX_FIR_PL                = 0x8004240C0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE0_TX_FIR_PL                = 0x8004240C0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE0_TX_FIR_PL                = 0x8004240C0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE0_TX_FIR_PL                 = 0x8004240C0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE0_TX_FIR_PL                 = 0x8004240C0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE0_TX_FIR_PL                 = 0x8004240C0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE0_TX_FIR_PL                = 0x8004240C0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE0_TX_MODE1_PL                  = 0x8004040C0701103Full;
//DUPS: 8004040C0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE0_TX_MODE1_PL                = 0x8004040C0701103Full;
//DUPS: 8004040C0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE0_TX_MODE1_PL                = 0x8004040C0801103Full;
//DUPS: 8004040C0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE0_TX_MODE1_PL                = 0x8004040C0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE0_TX_MODE1_PL              = 0x8004040C0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE0_TX_MODE1_PL              = 0x8004040C0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE0_TX_MODE1_PL              = 0x8004040C0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE0_TX_MODE1_PL              = 0x8004040C0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE0_TX_MODE1_PL               = 0x8004040C0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE0_TX_MODE1_PL               = 0x8004040C0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE0_TX_MODE1_PL               = 0x8004040C0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE0_TX_MODE1_PL              = 0x8004040C0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE0_TX_MODE2_PL                  = 0x80040C0C0701103Full;
//DUPS: 80040C0C0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE0_TX_MODE2_PL                = 0x80040C0C0701103Full;
//DUPS: 80040C0C0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE0_TX_MODE2_PL                = 0x80040C0C0801103Full;
//DUPS: 80040C0C0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE0_TX_MODE2_PL                = 0x80040C0C0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE0_TX_MODE2_PL              = 0x80040C0C0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE0_TX_MODE2_PL              = 0x80040C0C0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE0_TX_MODE2_PL              = 0x80040C0C0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE0_TX_MODE2_PL              = 0x80040C0C0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE0_TX_MODE2_PL               = 0x80040C0C0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE0_TX_MODE2_PL               = 0x80040C0C0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE0_TX_MODE2_PL               = 0x80040C0C0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE0_TX_MODE2_PL              = 0x80040C0C0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE0_TX_STAT1_PL                  = 0x80041C0C0701103Full;
//DUPS: 80041C0C0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE0_TX_STAT1_PL                = 0x80041C0C0701103Full;
//DUPS: 80041C0C0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE0_TX_STAT1_PL                = 0x80041C0C0801103Full;
//DUPS: 80041C0C0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE0_TX_STAT1_PL                = 0x80041C0C0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE0_TX_STAT1_PL              = 0x80041C0C0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE0_TX_STAT1_PL              = 0x80041C0C0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE0_TX_STAT1_PL              = 0x80041C0C0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE0_TX_STAT1_PL              = 0x80041C0C0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE0_TX_STAT1_PL               = 0x80041C0C0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE0_TX_STAT1_PL               = 0x80041C0C0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE0_TX_STAT1_PL               = 0x80041C0C0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE0_TX_STAT1_PL              = 0x80041C0C0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE1_TX_CNTL1G_PL                 = 0x8004140D0701103Full;
//DUPS: 8004140D0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE1_TX_CNTL1G_PL               = 0x8004140D0701103Full;
//DUPS: 8004140D0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE1_TX_CNTL1G_PL               = 0x8004140D0801103Full;
//DUPS: 8004140D0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE1_TX_CNTL1G_PL               = 0x8004140D0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE1_TX_CNTL1G_PL             = 0x8004140D0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE1_TX_CNTL1G_PL             = 0x8004140D0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE1_TX_CNTL1G_PL             = 0x8004140D0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE1_TX_CNTL1G_PL             = 0x8004140D0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE1_TX_CNTL1G_PL              = 0x8004140D0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE1_TX_CNTL1G_PL              = 0x8004140D0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE1_TX_CNTL1G_PL              = 0x8004140D0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE1_TX_CNTL1G_PL             = 0x8004140D0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE1_TX_CNTL2_O_PL                = 0x80044C0D0701103Full;
//DUPS: 80044C0D0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE1_TX_CNTL2_O_PL              = 0x80044C0D0701103Full;
//DUPS: 80044C0D0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE1_TX_CNTL2_O_PL              = 0x80044C0D0801103Full;
//DUPS: 80044C0D0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE1_TX_CNTL2_O_PL              = 0x80044C0D0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE1_TX_CNTL2_O_PL            = 0x80044C0D0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE1_TX_CNTL2_O_PL            = 0x80044C0D0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE1_TX_CNTL2_O_PL            = 0x80044C0D0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE1_TX_CNTL2_O_PL            = 0x80044C0D0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE1_TX_CNTL2_O_PL             = 0x80044C0D0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE1_TX_CNTL2_O_PL             = 0x80044C0D0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE1_TX_CNTL2_O_PL             = 0x80044C0D0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE1_TX_CNTL2_O_PL            = 0x80044C0D0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE1_TX_CNTL3_EO_PL               = 0x8004540D0701103Full;
//DUPS: 8004540D0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE1_TX_CNTL3_EO_PL             = 0x8004540D0701103Full;
//DUPS: 8004540D0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE1_TX_CNTL3_EO_PL             = 0x8004540D0801103Full;
//DUPS: 8004540D0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE1_TX_CNTL3_EO_PL             = 0x8004540D0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE1_TX_CNTL3_EO_PL           = 0x8004540D0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE1_TX_CNTL3_EO_PL           = 0x8004540D0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE1_TX_CNTL3_EO_PL           = 0x8004540D0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE1_TX_CNTL3_EO_PL           = 0x8004540D0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE1_TX_CNTL3_EO_PL            = 0x8004540D0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE1_TX_CNTL3_EO_PL            = 0x8004540D0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE1_TX_CNTL3_EO_PL            = 0x8004540D0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE1_TX_CNTL3_EO_PL           = 0x8004540D0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE1_TX_FIR_ERROR_INJECT_PL       = 0x8004340D0701103Full;
//DUPS: 8004340D0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE1_TX_FIR_ERROR_INJECT_PL     = 0x8004340D0701103Full;
//DUPS: 8004340D0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE1_TX_FIR_ERROR_INJECT_PL     = 0x8004340D0801103Full;
//DUPS: 8004340D0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE1_TX_FIR_ERROR_INJECT_PL     = 0x8004340D0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x8004340D0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x8004340D0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x8004340D0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x8004340D0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE1_TX_FIR_ERROR_INJECT_PL    = 0x8004340D0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE1_TX_FIR_ERROR_INJECT_PL    = 0x8004340D0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE1_TX_FIR_ERROR_INJECT_PL    = 0x8004340D0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x8004340D0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE1_TX_FIR_MASK_PL               = 0x80042C0D0701103Full;
//DUPS: 80042C0D0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE1_TX_FIR_MASK_PL             = 0x80042C0D0701103Full;
//DUPS: 80042C0D0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE1_TX_FIR_MASK_PL             = 0x80042C0D0801103Full;
//DUPS: 80042C0D0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE1_TX_FIR_MASK_PL             = 0x80042C0D0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE1_TX_FIR_MASK_PL           = 0x80042C0D0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE1_TX_FIR_MASK_PL           = 0x80042C0D0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE1_TX_FIR_MASK_PL           = 0x80042C0D0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE1_TX_FIR_MASK_PL           = 0x80042C0D0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE1_TX_FIR_MASK_PL            = 0x80042C0D0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE1_TX_FIR_MASK_PL            = 0x80042C0D0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE1_TX_FIR_MASK_PL            = 0x80042C0D0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE1_TX_FIR_MASK_PL           = 0x80042C0D0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE1_TX_FIR_PL                    = 0x8004240D0701103Full;
//DUPS: 8004240D0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE1_TX_FIR_PL                  = 0x8004240D0701103Full;
//DUPS: 8004240D0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE1_TX_FIR_PL                  = 0x8004240D0801103Full;
//DUPS: 8004240D0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE1_TX_FIR_PL                  = 0x8004240D0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE1_TX_FIR_PL                = 0x8004240D0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE1_TX_FIR_PL                = 0x8004240D0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE1_TX_FIR_PL                = 0x8004240D0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE1_TX_FIR_PL                = 0x8004240D0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE1_TX_FIR_PL                 = 0x8004240D0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE1_TX_FIR_PL                 = 0x8004240D0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE1_TX_FIR_PL                 = 0x8004240D0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE1_TX_FIR_PL                = 0x8004240D0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE1_TX_MODE1_PL                  = 0x8004040D0701103Full;
//DUPS: 8004040D0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE1_TX_MODE1_PL                = 0x8004040D0701103Full;
//DUPS: 8004040D0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE1_TX_MODE1_PL                = 0x8004040D0801103Full;
//DUPS: 8004040D0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE1_TX_MODE1_PL                = 0x8004040D0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE1_TX_MODE1_PL              = 0x8004040D0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE1_TX_MODE1_PL              = 0x8004040D0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE1_TX_MODE1_PL              = 0x8004040D0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE1_TX_MODE1_PL              = 0x8004040D0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE1_TX_MODE1_PL               = 0x8004040D0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE1_TX_MODE1_PL               = 0x8004040D0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE1_TX_MODE1_PL               = 0x8004040D0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE1_TX_MODE1_PL              = 0x8004040D0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE1_TX_MODE2_PL                  = 0x80040C0D0701103Full;
//DUPS: 80040C0D0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE1_TX_MODE2_PL                = 0x80040C0D0701103Full;
//DUPS: 80040C0D0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE1_TX_MODE2_PL                = 0x80040C0D0801103Full;
//DUPS: 80040C0D0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE1_TX_MODE2_PL                = 0x80040C0D0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE1_TX_MODE2_PL              = 0x80040C0D0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE1_TX_MODE2_PL              = 0x80040C0D0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE1_TX_MODE2_PL              = 0x80040C0D0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE1_TX_MODE2_PL              = 0x80040C0D0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE1_TX_MODE2_PL               = 0x80040C0D0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE1_TX_MODE2_PL               = 0x80040C0D0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE1_TX_MODE2_PL               = 0x80040C0D0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE1_TX_MODE2_PL              = 0x80040C0D0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE1_TX_STAT1_PL                  = 0x80041C0D0701103Full;
//DUPS: 80041C0D0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE1_TX_STAT1_PL                = 0x80041C0D0701103Full;
//DUPS: 80041C0D0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE1_TX_STAT1_PL                = 0x80041C0D0801103Full;
//DUPS: 80041C0D0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE1_TX_STAT1_PL                = 0x80041C0D0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE1_TX_STAT1_PL              = 0x80041C0D0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE1_TX_STAT1_PL              = 0x80041C0D0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE1_TX_STAT1_PL              = 0x80041C0D0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE1_TX_STAT1_PL              = 0x80041C0D0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE1_TX_STAT1_PL               = 0x80041C0D0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE1_TX_STAT1_PL               = 0x80041C0D0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE1_TX_STAT1_PL               = 0x80041C0D0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE1_TX_STAT1_PL              = 0x80041C0D0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE2_TX_CNTL1G_PL                 = 0x8004140E0701103Full;
//DUPS: 8004140E0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE2_TX_CNTL1G_PL               = 0x8004140E0701103Full;
//DUPS: 8004140E0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE2_TX_CNTL1G_PL               = 0x8004140E0801103Full;
//DUPS: 8004140E0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE2_TX_CNTL1G_PL               = 0x8004140E0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE2_TX_CNTL1G_PL             = 0x8004140E0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE2_TX_CNTL1G_PL             = 0x8004140E0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE2_TX_CNTL1G_PL             = 0x8004140E0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE2_TX_CNTL1G_PL             = 0x8004140E0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE2_TX_CNTL1G_PL              = 0x8004140E0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE2_TX_CNTL1G_PL              = 0x8004140E0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE2_TX_CNTL1G_PL              = 0x8004140E0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE2_TX_CNTL1G_PL             = 0x8004140E0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE2_TX_CNTL2_O_PL                = 0x80044C0E0701103Full;
//DUPS: 80044C0E0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE2_TX_CNTL2_O_PL              = 0x80044C0E0701103Full;
//DUPS: 80044C0E0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE2_TX_CNTL2_O_PL              = 0x80044C0E0801103Full;
//DUPS: 80044C0E0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE2_TX_CNTL2_O_PL              = 0x80044C0E0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE2_TX_CNTL2_O_PL            = 0x80044C0E0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE2_TX_CNTL2_O_PL            = 0x80044C0E0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE2_TX_CNTL2_O_PL            = 0x80044C0E0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE2_TX_CNTL2_O_PL            = 0x80044C0E0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE2_TX_CNTL2_O_PL             = 0x80044C0E0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE2_TX_CNTL2_O_PL             = 0x80044C0E0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE2_TX_CNTL2_O_PL             = 0x80044C0E0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE2_TX_CNTL2_O_PL            = 0x80044C0E0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE2_TX_CNTL3_EO_PL               = 0x8004540E0701103Full;
//DUPS: 8004540E0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE2_TX_CNTL3_EO_PL             = 0x8004540E0701103Full;
//DUPS: 8004540E0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE2_TX_CNTL3_EO_PL             = 0x8004540E0801103Full;
//DUPS: 8004540E0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE2_TX_CNTL3_EO_PL             = 0x8004540E0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE2_TX_CNTL3_EO_PL           = 0x8004540E0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE2_TX_CNTL3_EO_PL           = 0x8004540E0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE2_TX_CNTL3_EO_PL           = 0x8004540E0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE2_TX_CNTL3_EO_PL           = 0x8004540E0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE2_TX_CNTL3_EO_PL            = 0x8004540E0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE2_TX_CNTL3_EO_PL            = 0x8004540E0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE2_TX_CNTL3_EO_PL            = 0x8004540E0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE2_TX_CNTL3_EO_PL           = 0x8004540E0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE2_TX_FIR_ERROR_INJECT_PL       = 0x8004340E0701103Full;
//DUPS: 8004340E0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE2_TX_FIR_ERROR_INJECT_PL     = 0x8004340E0701103Full;
//DUPS: 8004340E0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE2_TX_FIR_ERROR_INJECT_PL     = 0x8004340E0801103Full;
//DUPS: 8004340E0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE2_TX_FIR_ERROR_INJECT_PL     = 0x8004340E0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x8004340E0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x8004340E0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x8004340E0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x8004340E0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE2_TX_FIR_ERROR_INJECT_PL    = 0x8004340E0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE2_TX_FIR_ERROR_INJECT_PL    = 0x8004340E0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE2_TX_FIR_ERROR_INJECT_PL    = 0x8004340E0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x8004340E0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE2_TX_FIR_MASK_PL               = 0x80042C0E0701103Full;
//DUPS: 80042C0E0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE2_TX_FIR_MASK_PL             = 0x80042C0E0701103Full;
//DUPS: 80042C0E0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE2_TX_FIR_MASK_PL             = 0x80042C0E0801103Full;
//DUPS: 80042C0E0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE2_TX_FIR_MASK_PL             = 0x80042C0E0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE2_TX_FIR_MASK_PL           = 0x80042C0E0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE2_TX_FIR_MASK_PL           = 0x80042C0E0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE2_TX_FIR_MASK_PL           = 0x80042C0E0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE2_TX_FIR_MASK_PL           = 0x80042C0E0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE2_TX_FIR_MASK_PL            = 0x80042C0E0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE2_TX_FIR_MASK_PL            = 0x80042C0E0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE2_TX_FIR_MASK_PL            = 0x80042C0E0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE2_TX_FIR_MASK_PL           = 0x80042C0E0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE2_TX_FIR_PL                    = 0x8004240E0701103Full;
//DUPS: 8004240E0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE2_TX_FIR_PL                  = 0x8004240E0701103Full;
//DUPS: 8004240E0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE2_TX_FIR_PL                  = 0x8004240E0801103Full;
//DUPS: 8004240E0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE2_TX_FIR_PL                  = 0x8004240E0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE2_TX_FIR_PL                = 0x8004240E0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE2_TX_FIR_PL                = 0x8004240E0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE2_TX_FIR_PL                = 0x8004240E0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE2_TX_FIR_PL                = 0x8004240E0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE2_TX_FIR_PL                 = 0x8004240E0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE2_TX_FIR_PL                 = 0x8004240E0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE2_TX_FIR_PL                 = 0x8004240E0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE2_TX_FIR_PL                = 0x8004240E0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE2_TX_MODE1_PL                  = 0x8004040E0701103Full;
//DUPS: 8004040E0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE2_TX_MODE1_PL                = 0x8004040E0701103Full;
//DUPS: 8004040E0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE2_TX_MODE1_PL                = 0x8004040E0801103Full;
//DUPS: 8004040E0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE2_TX_MODE1_PL                = 0x8004040E0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE2_TX_MODE1_PL              = 0x8004040E0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE2_TX_MODE1_PL              = 0x8004040E0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE2_TX_MODE1_PL              = 0x8004040E0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE2_TX_MODE1_PL              = 0x8004040E0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE2_TX_MODE1_PL               = 0x8004040E0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE2_TX_MODE1_PL               = 0x8004040E0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE2_TX_MODE1_PL               = 0x8004040E0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE2_TX_MODE1_PL              = 0x8004040E0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE2_TX_MODE2_PL                  = 0x80040C0E0701103Full;
//DUPS: 80040C0E0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE2_TX_MODE2_PL                = 0x80040C0E0701103Full;
//DUPS: 80040C0E0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE2_TX_MODE2_PL                = 0x80040C0E0801103Full;
//DUPS: 80040C0E0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE2_TX_MODE2_PL                = 0x80040C0E0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE2_TX_MODE2_PL              = 0x80040C0E0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE2_TX_MODE2_PL              = 0x80040C0E0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE2_TX_MODE2_PL              = 0x80040C0E0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE2_TX_MODE2_PL              = 0x80040C0E0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE2_TX_MODE2_PL               = 0x80040C0E0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE2_TX_MODE2_PL               = 0x80040C0E0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE2_TX_MODE2_PL               = 0x80040C0E0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE2_TX_MODE2_PL              = 0x80040C0E0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE2_TX_STAT1_PL                  = 0x80041C0E0701103Full;
//DUPS: 80041C0E0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE2_TX_STAT1_PL                = 0x80041C0E0701103Full;
//DUPS: 80041C0E0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE2_TX_STAT1_PL                = 0x80041C0E0801103Full;
//DUPS: 80041C0E0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE2_TX_STAT1_PL                = 0x80041C0E0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE2_TX_STAT1_PL              = 0x80041C0E0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE2_TX_STAT1_PL              = 0x80041C0E0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE2_TX_STAT1_PL              = 0x80041C0E0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE2_TX_STAT1_PL              = 0x80041C0E0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE2_TX_STAT1_PL               = 0x80041C0E0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE2_TX_STAT1_PL               = 0x80041C0E0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE2_TX_STAT1_PL               = 0x80041C0E0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE2_TX_STAT1_PL              = 0x80041C0E0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE3_TX_CNTL1G_PL                 = 0x8004140F0701103Full;
//DUPS: 8004140F0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE3_TX_CNTL1G_PL               = 0x8004140F0701103Full;
//DUPS: 8004140F0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE3_TX_CNTL1G_PL               = 0x8004140F0801103Full;
//DUPS: 8004140F0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE3_TX_CNTL1G_PL               = 0x8004140F0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE3_TX_CNTL1G_PL             = 0x8004140F0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE3_TX_CNTL1G_PL             = 0x8004140F0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE3_TX_CNTL1G_PL             = 0x8004140F0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE3_TX_CNTL1G_PL             = 0x8004140F0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE3_TX_CNTL1G_PL              = 0x8004140F0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE3_TX_CNTL1G_PL              = 0x8004140F0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE3_TX_CNTL1G_PL              = 0x8004140F0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE3_TX_CNTL1G_PL             = 0x8004140F0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE3_TX_CNTL2_O_PL                = 0x80044C0F0701103Full;
//DUPS: 80044C0F0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE3_TX_CNTL2_O_PL              = 0x80044C0F0701103Full;
//DUPS: 80044C0F0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE3_TX_CNTL2_O_PL              = 0x80044C0F0801103Full;
//DUPS: 80044C0F0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE3_TX_CNTL2_O_PL              = 0x80044C0F0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE3_TX_CNTL2_O_PL            = 0x80044C0F0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE3_TX_CNTL2_O_PL            = 0x80044C0F0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE3_TX_CNTL2_O_PL            = 0x80044C0F0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE3_TX_CNTL2_O_PL            = 0x80044C0F0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE3_TX_CNTL2_O_PL             = 0x80044C0F0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE3_TX_CNTL2_O_PL             = 0x80044C0F0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE3_TX_CNTL2_O_PL             = 0x80044C0F0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE3_TX_CNTL2_O_PL            = 0x80044C0F0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE3_TX_CNTL3_EO_PL               = 0x8004540F0701103Full;
//DUPS: 8004540F0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE3_TX_CNTL3_EO_PL             = 0x8004540F0701103Full;
//DUPS: 8004540F0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE3_TX_CNTL3_EO_PL             = 0x8004540F0801103Full;
//DUPS: 8004540F0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE3_TX_CNTL3_EO_PL             = 0x8004540F0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE3_TX_CNTL3_EO_PL           = 0x8004540F0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE3_TX_CNTL3_EO_PL           = 0x8004540F0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE3_TX_CNTL3_EO_PL           = 0x8004540F0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE3_TX_CNTL3_EO_PL           = 0x8004540F0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE3_TX_CNTL3_EO_PL            = 0x8004540F0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE3_TX_CNTL3_EO_PL            = 0x8004540F0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE3_TX_CNTL3_EO_PL            = 0x8004540F0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE3_TX_CNTL3_EO_PL           = 0x8004540F0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE3_TX_FIR_ERROR_INJECT_PL       = 0x8004340F0701103Full;
//DUPS: 8004340F0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE3_TX_FIR_ERROR_INJECT_PL     = 0x8004340F0701103Full;
//DUPS: 8004340F0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE3_TX_FIR_ERROR_INJECT_PL     = 0x8004340F0801103Full;
//DUPS: 8004340F0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE3_TX_FIR_ERROR_INJECT_PL     = 0x8004340F0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x8004340F0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x8004340F0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x8004340F0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x8004340F0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE3_TX_FIR_ERROR_INJECT_PL    = 0x8004340F0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE3_TX_FIR_ERROR_INJECT_PL    = 0x8004340F0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE3_TX_FIR_ERROR_INJECT_PL    = 0x8004340F0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x8004340F0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE3_TX_FIR_MASK_PL               = 0x80042C0F0701103Full;
//DUPS: 80042C0F0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE3_TX_FIR_MASK_PL             = 0x80042C0F0701103Full;
//DUPS: 80042C0F0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE3_TX_FIR_MASK_PL             = 0x80042C0F0801103Full;
//DUPS: 80042C0F0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE3_TX_FIR_MASK_PL             = 0x80042C0F0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE3_TX_FIR_MASK_PL           = 0x80042C0F0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE3_TX_FIR_MASK_PL           = 0x80042C0F0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE3_TX_FIR_MASK_PL           = 0x80042C0F0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE3_TX_FIR_MASK_PL           = 0x80042C0F0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE3_TX_FIR_MASK_PL            = 0x80042C0F0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE3_TX_FIR_MASK_PL            = 0x80042C0F0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE3_TX_FIR_MASK_PL            = 0x80042C0F0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE3_TX_FIR_MASK_PL           = 0x80042C0F0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE3_TX_FIR_PL                    = 0x8004240F0701103Full;
//DUPS: 8004240F0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE3_TX_FIR_PL                  = 0x8004240F0701103Full;
//DUPS: 8004240F0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE3_TX_FIR_PL                  = 0x8004240F0801103Full;
//DUPS: 8004240F0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE3_TX_FIR_PL                  = 0x8004240F0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE3_TX_FIR_PL                = 0x8004240F0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE3_TX_FIR_PL                = 0x8004240F0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE3_TX_FIR_PL                = 0x8004240F0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE3_TX_FIR_PL                = 0x8004240F0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE3_TX_FIR_PL                 = 0x8004240F0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE3_TX_FIR_PL                 = 0x8004240F0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE3_TX_FIR_PL                 = 0x8004240F0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE3_TX_FIR_PL                = 0x8004240F0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE3_TX_MODE1_PL                  = 0x8004040F0701103Full;
//DUPS: 8004040F0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE3_TX_MODE1_PL                = 0x8004040F0701103Full;
//DUPS: 8004040F0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE3_TX_MODE1_PL                = 0x8004040F0801103Full;
//DUPS: 8004040F0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE3_TX_MODE1_PL                = 0x8004040F0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE3_TX_MODE1_PL              = 0x8004040F0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE3_TX_MODE1_PL              = 0x8004040F0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE3_TX_MODE1_PL              = 0x8004040F0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE3_TX_MODE1_PL              = 0x8004040F0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE3_TX_MODE1_PL               = 0x8004040F0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE3_TX_MODE1_PL               = 0x8004040F0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE3_TX_MODE1_PL               = 0x8004040F0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE3_TX_MODE1_PL              = 0x8004040F0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE3_TX_MODE2_PL                  = 0x80040C0F0701103Full;
//DUPS: 80040C0F0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE3_TX_MODE2_PL                = 0x80040C0F0701103Full;
//DUPS: 80040C0F0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE3_TX_MODE2_PL                = 0x80040C0F0801103Full;
//DUPS: 80040C0F0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE3_TX_MODE2_PL                = 0x80040C0F0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE3_TX_MODE2_PL              = 0x80040C0F0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE3_TX_MODE2_PL              = 0x80040C0F0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE3_TX_MODE2_PL              = 0x80040C0F0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE3_TX_MODE2_PL              = 0x80040C0F0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE3_TX_MODE2_PL               = 0x80040C0F0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE3_TX_MODE2_PL               = 0x80040C0F0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE3_TX_MODE2_PL               = 0x80040C0F0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE3_TX_MODE2_PL              = 0x80040C0F0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS3_SLICE3_TX_STAT1_PL                  = 0x80041C0F0701103Full;
//DUPS: 80041C0F0701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS3_SLICE3_TX_STAT1_PL                = 0x80041C0F0701103Full;
//DUPS: 80041C0F0701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS3_SLICE3_TX_STAT1_PL                = 0x80041C0F0801103Full;
//DUPS: 80041C0F0801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS3_SLICE3_TX_STAT1_PL                = 0x80041C0F0701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS3_SLICE3_TX_STAT1_PL              = 0x80041C0F0701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS3_SLICE3_TX_STAT1_PL              = 0x80041C0F0701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS3_SLICE3_TX_STAT1_PL              = 0x80041C0F0801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS3_SLICE3_TX_STAT1_PL              = 0x80041C0F0801143Full;

static const uint64_t P9A_OMI_1_TX0_TXPACKS3_SLICE3_TX_STAT1_PL               = 0x80041C0F0701103Full;

static const uint64_t P9A_OMI_4_TX0_TXPACKS3_SLICE3_TX_STAT1_PL               = 0x80041C0F0701143Full;

static const uint64_t P9A_OMI_9_TX0_TXPACKS3_SLICE3_TX_STAT1_PL               = 0x80041C0F0801103Full;

static const uint64_t P9A_OMI_12_TX0_TXPACKS3_SLICE3_TX_STAT1_PL              = 0x80041C0F0801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE0_TX_CNTL1G_PL                 = 0x800414100701103Full;
//DUPS: 800414100701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE0_TX_CNTL1G_PL               = 0x800414100701103Full;
//DUPS: 800414100701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE0_TX_CNTL1G_PL               = 0x800414100801103Full;
//DUPS: 800414100801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE0_TX_CNTL1G_PL               = 0x800414100701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE0_TX_CNTL1G_PL             = 0x800414100701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE0_TX_CNTL1G_PL             = 0x800414100701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE0_TX_CNTL1G_PL             = 0x800414100801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE0_TX_CNTL1G_PL             = 0x800414100801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE0_TX_CNTL1G_PL              = 0x800414100701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE0_TX_CNTL1G_PL              = 0x800414100701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE0_TX_CNTL1G_PL             = 0x800414100801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE0_TX_CNTL1G_PL             = 0x800414100801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE0_TX_CNTL2_O_PL                = 0x80044C100701103Full;
//DUPS: 80044C100701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE0_TX_CNTL2_O_PL              = 0x80044C100701103Full;
//DUPS: 80044C100701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE0_TX_CNTL2_O_PL              = 0x80044C100801103Full;
//DUPS: 80044C100801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE0_TX_CNTL2_O_PL              = 0x80044C100701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE0_TX_CNTL2_O_PL            = 0x80044C100701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE0_TX_CNTL2_O_PL            = 0x80044C100701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE0_TX_CNTL2_O_PL            = 0x80044C100801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE0_TX_CNTL2_O_PL            = 0x80044C100801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE0_TX_CNTL2_O_PL             = 0x80044C100701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE0_TX_CNTL2_O_PL             = 0x80044C100701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE0_TX_CNTL2_O_PL            = 0x80044C100801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE0_TX_CNTL2_O_PL            = 0x80044C100801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE0_TX_CNTL3_EO_PL               = 0x800454100701103Full;
//DUPS: 800454100701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE0_TX_CNTL3_EO_PL             = 0x800454100701103Full;
//DUPS: 800454100701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE0_TX_CNTL3_EO_PL             = 0x800454100801103Full;
//DUPS: 800454100801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE0_TX_CNTL3_EO_PL             = 0x800454100701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE0_TX_CNTL3_EO_PL           = 0x800454100701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE0_TX_CNTL3_EO_PL           = 0x800454100701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE0_TX_CNTL3_EO_PL           = 0x800454100801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE0_TX_CNTL3_EO_PL           = 0x800454100801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE0_TX_CNTL3_EO_PL            = 0x800454100701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE0_TX_CNTL3_EO_PL            = 0x800454100701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE0_TX_CNTL3_EO_PL           = 0x800454100801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE0_TX_CNTL3_EO_PL           = 0x800454100801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE0_TX_FIR_ERROR_INJECT_PL       = 0x800434100701103Full;
//DUPS: 800434100701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE0_TX_FIR_ERROR_INJECT_PL     = 0x800434100701103Full;
//DUPS: 800434100701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE0_TX_FIR_ERROR_INJECT_PL     = 0x800434100801103Full;
//DUPS: 800434100801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE0_TX_FIR_ERROR_INJECT_PL     = 0x800434100701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x800434100701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x800434100701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x800434100801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x800434100801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE0_TX_FIR_ERROR_INJECT_PL    = 0x800434100701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE0_TX_FIR_ERROR_INJECT_PL    = 0x800434100701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x800434100801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x800434100801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE0_TX_FIR_MASK_PL               = 0x80042C100701103Full;
//DUPS: 80042C100701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE0_TX_FIR_MASK_PL             = 0x80042C100701103Full;
//DUPS: 80042C100701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE0_TX_FIR_MASK_PL             = 0x80042C100801103Full;
//DUPS: 80042C100801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE0_TX_FIR_MASK_PL             = 0x80042C100701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE0_TX_FIR_MASK_PL           = 0x80042C100701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE0_TX_FIR_MASK_PL           = 0x80042C100701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE0_TX_FIR_MASK_PL           = 0x80042C100801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE0_TX_FIR_MASK_PL           = 0x80042C100801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE0_TX_FIR_MASK_PL            = 0x80042C100701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE0_TX_FIR_MASK_PL            = 0x80042C100701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE0_TX_FIR_MASK_PL           = 0x80042C100801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE0_TX_FIR_MASK_PL           = 0x80042C100801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE0_TX_FIR_PL                    = 0x800424100701103Full;
//DUPS: 800424100701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE0_TX_FIR_PL                  = 0x800424100701103Full;
//DUPS: 800424100701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE0_TX_FIR_PL                  = 0x800424100801103Full;
//DUPS: 800424100801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE0_TX_FIR_PL                  = 0x800424100701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE0_TX_FIR_PL                = 0x800424100701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE0_TX_FIR_PL                = 0x800424100701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE0_TX_FIR_PL                = 0x800424100801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE0_TX_FIR_PL                = 0x800424100801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE0_TX_FIR_PL                 = 0x800424100701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE0_TX_FIR_PL                 = 0x800424100701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE0_TX_FIR_PL                = 0x800424100801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE0_TX_FIR_PL                = 0x800424100801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE0_TX_MODE1_PL                  = 0x800404100701103Full;
//DUPS: 800404100701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE0_TX_MODE1_PL                = 0x800404100701103Full;
//DUPS: 800404100701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE0_TX_MODE1_PL                = 0x800404100801103Full;
//DUPS: 800404100801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE0_TX_MODE1_PL                = 0x800404100701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE0_TX_MODE1_PL              = 0x800404100701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE0_TX_MODE1_PL              = 0x800404100701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE0_TX_MODE1_PL              = 0x800404100801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE0_TX_MODE1_PL              = 0x800404100801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE0_TX_MODE1_PL               = 0x800404100701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE0_TX_MODE1_PL               = 0x800404100701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE0_TX_MODE1_PL              = 0x800404100801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE0_TX_MODE1_PL              = 0x800404100801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE0_TX_MODE2_PL                  = 0x80040C100701103Full;
//DUPS: 80040C100701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE0_TX_MODE2_PL                = 0x80040C100701103Full;
//DUPS: 80040C100701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE0_TX_MODE2_PL                = 0x80040C100801103Full;
//DUPS: 80040C100801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE0_TX_MODE2_PL                = 0x80040C100701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE0_TX_MODE2_PL              = 0x80040C100701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE0_TX_MODE2_PL              = 0x80040C100701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE0_TX_MODE2_PL              = 0x80040C100801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE0_TX_MODE2_PL              = 0x80040C100801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE0_TX_MODE2_PL               = 0x80040C100701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE0_TX_MODE2_PL               = 0x80040C100701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE0_TX_MODE2_PL              = 0x80040C100801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE0_TX_MODE2_PL              = 0x80040C100801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE0_TX_STAT1_PL                  = 0x80041C100701103Full;
//DUPS: 80041C100701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE0_TX_STAT1_PL                = 0x80041C100701103Full;
//DUPS: 80041C100701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE0_TX_STAT1_PL                = 0x80041C100801103Full;
//DUPS: 80041C100801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE0_TX_STAT1_PL                = 0x80041C100701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE0_TX_STAT1_PL              = 0x80041C100701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE0_TX_STAT1_PL              = 0x80041C100701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE0_TX_STAT1_PL              = 0x80041C100801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE0_TX_STAT1_PL              = 0x80041C100801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE0_TX_STAT1_PL               = 0x80041C100701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE0_TX_STAT1_PL               = 0x80041C100701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE0_TX_STAT1_PL              = 0x80041C100801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE0_TX_STAT1_PL              = 0x80041C100801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE1_TX_CNTL1G_PL                 = 0x800414110701103Full;
//DUPS: 800414110701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE1_TX_CNTL1G_PL               = 0x800414110701103Full;
//DUPS: 800414110701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE1_TX_CNTL1G_PL               = 0x800414110801103Full;
//DUPS: 800414110801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE1_TX_CNTL1G_PL               = 0x800414110701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE1_TX_CNTL1G_PL             = 0x800414110701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE1_TX_CNTL1G_PL             = 0x800414110701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE1_TX_CNTL1G_PL             = 0x800414110801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE1_TX_CNTL1G_PL             = 0x800414110801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE1_TX_CNTL1G_PL              = 0x800414110701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE1_TX_CNTL1G_PL              = 0x800414110701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE1_TX_CNTL1G_PL             = 0x800414110801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE1_TX_CNTL1G_PL             = 0x800414110801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE1_TX_CNTL2_O_PL                = 0x80044C110701103Full;
//DUPS: 80044C110701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE1_TX_CNTL2_O_PL              = 0x80044C110701103Full;
//DUPS: 80044C110701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE1_TX_CNTL2_O_PL              = 0x80044C110801103Full;
//DUPS: 80044C110801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE1_TX_CNTL2_O_PL              = 0x80044C110701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE1_TX_CNTL2_O_PL            = 0x80044C110701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE1_TX_CNTL2_O_PL            = 0x80044C110701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE1_TX_CNTL2_O_PL            = 0x80044C110801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE1_TX_CNTL2_O_PL            = 0x80044C110801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE1_TX_CNTL2_O_PL             = 0x80044C110701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE1_TX_CNTL2_O_PL             = 0x80044C110701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE1_TX_CNTL2_O_PL            = 0x80044C110801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE1_TX_CNTL2_O_PL            = 0x80044C110801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE1_TX_CNTL3_EO_PL               = 0x800454110701103Full;
//DUPS: 800454110701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE1_TX_CNTL3_EO_PL             = 0x800454110701103Full;
//DUPS: 800454110701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE1_TX_CNTL3_EO_PL             = 0x800454110801103Full;
//DUPS: 800454110801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE1_TX_CNTL3_EO_PL             = 0x800454110701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE1_TX_CNTL3_EO_PL           = 0x800454110701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE1_TX_CNTL3_EO_PL           = 0x800454110701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE1_TX_CNTL3_EO_PL           = 0x800454110801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE1_TX_CNTL3_EO_PL           = 0x800454110801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE1_TX_CNTL3_EO_PL            = 0x800454110701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE1_TX_CNTL3_EO_PL            = 0x800454110701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE1_TX_CNTL3_EO_PL           = 0x800454110801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE1_TX_CNTL3_EO_PL           = 0x800454110801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE1_TX_FIR_ERROR_INJECT_PL       = 0x800434110701103Full;
//DUPS: 800434110701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE1_TX_FIR_ERROR_INJECT_PL     = 0x800434110701103Full;
//DUPS: 800434110701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE1_TX_FIR_ERROR_INJECT_PL     = 0x800434110801103Full;
//DUPS: 800434110801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE1_TX_FIR_ERROR_INJECT_PL     = 0x800434110701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x800434110701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x800434110701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x800434110801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x800434110801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE1_TX_FIR_ERROR_INJECT_PL    = 0x800434110701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE1_TX_FIR_ERROR_INJECT_PL    = 0x800434110701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x800434110801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x800434110801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE1_TX_FIR_MASK_PL               = 0x80042C110701103Full;
//DUPS: 80042C110701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE1_TX_FIR_MASK_PL             = 0x80042C110701103Full;
//DUPS: 80042C110701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE1_TX_FIR_MASK_PL             = 0x80042C110801103Full;
//DUPS: 80042C110801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE1_TX_FIR_MASK_PL             = 0x80042C110701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE1_TX_FIR_MASK_PL           = 0x80042C110701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE1_TX_FIR_MASK_PL           = 0x80042C110701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE1_TX_FIR_MASK_PL           = 0x80042C110801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE1_TX_FIR_MASK_PL           = 0x80042C110801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE1_TX_FIR_MASK_PL            = 0x80042C110701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE1_TX_FIR_MASK_PL            = 0x80042C110701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE1_TX_FIR_MASK_PL           = 0x80042C110801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE1_TX_FIR_MASK_PL           = 0x80042C110801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE1_TX_FIR_PL                    = 0x800424110701103Full;
//DUPS: 800424110701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE1_TX_FIR_PL                  = 0x800424110701103Full;
//DUPS: 800424110701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE1_TX_FIR_PL                  = 0x800424110801103Full;
//DUPS: 800424110801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE1_TX_FIR_PL                  = 0x800424110701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE1_TX_FIR_PL                = 0x800424110701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE1_TX_FIR_PL                = 0x800424110701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE1_TX_FIR_PL                = 0x800424110801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE1_TX_FIR_PL                = 0x800424110801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE1_TX_FIR_PL                 = 0x800424110701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE1_TX_FIR_PL                 = 0x800424110701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE1_TX_FIR_PL                = 0x800424110801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE1_TX_FIR_PL                = 0x800424110801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE1_TX_MODE1_PL                  = 0x800404110701103Full;
//DUPS: 800404110701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE1_TX_MODE1_PL                = 0x800404110701103Full;
//DUPS: 800404110701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE1_TX_MODE1_PL                = 0x800404110801103Full;
//DUPS: 800404110801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE1_TX_MODE1_PL                = 0x800404110701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE1_TX_MODE1_PL              = 0x800404110701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE1_TX_MODE1_PL              = 0x800404110701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE1_TX_MODE1_PL              = 0x800404110801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE1_TX_MODE1_PL              = 0x800404110801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE1_TX_MODE1_PL               = 0x800404110701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE1_TX_MODE1_PL               = 0x800404110701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE1_TX_MODE1_PL              = 0x800404110801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE1_TX_MODE1_PL              = 0x800404110801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE1_TX_MODE2_PL                  = 0x80040C110701103Full;
//DUPS: 80040C110701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE1_TX_MODE2_PL                = 0x80040C110701103Full;
//DUPS: 80040C110701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE1_TX_MODE2_PL                = 0x80040C110801103Full;
//DUPS: 80040C110801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE1_TX_MODE2_PL                = 0x80040C110701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE1_TX_MODE2_PL              = 0x80040C110701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE1_TX_MODE2_PL              = 0x80040C110701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE1_TX_MODE2_PL              = 0x80040C110801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE1_TX_MODE2_PL              = 0x80040C110801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE1_TX_MODE2_PL               = 0x80040C110701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE1_TX_MODE2_PL               = 0x80040C110701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE1_TX_MODE2_PL              = 0x80040C110801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE1_TX_MODE2_PL              = 0x80040C110801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE1_TX_STAT1_PL                  = 0x80041C110701103Full;
//DUPS: 80041C110701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE1_TX_STAT1_PL                = 0x80041C110701103Full;
//DUPS: 80041C110701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE1_TX_STAT1_PL                = 0x80041C110801103Full;
//DUPS: 80041C110801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE1_TX_STAT1_PL                = 0x80041C110701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE1_TX_STAT1_PL              = 0x80041C110701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE1_TX_STAT1_PL              = 0x80041C110701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE1_TX_STAT1_PL              = 0x80041C110801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE1_TX_STAT1_PL              = 0x80041C110801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE1_TX_STAT1_PL               = 0x80041C110701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE1_TX_STAT1_PL               = 0x80041C110701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE1_TX_STAT1_PL              = 0x80041C110801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE1_TX_STAT1_PL              = 0x80041C110801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE2_TX_CNTL1G_PL                 = 0x800414120701103Full;
//DUPS: 800414120701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE2_TX_CNTL1G_PL               = 0x800414120701103Full;
//DUPS: 800414120701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE2_TX_CNTL1G_PL               = 0x800414120801103Full;
//DUPS: 800414120801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE2_TX_CNTL1G_PL               = 0x800414120701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE2_TX_CNTL1G_PL             = 0x800414120701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE2_TX_CNTL1G_PL             = 0x800414120701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE2_TX_CNTL1G_PL             = 0x800414120801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE2_TX_CNTL1G_PL             = 0x800414120801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE2_TX_CNTL1G_PL              = 0x800414120701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE2_TX_CNTL1G_PL              = 0x800414120701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE2_TX_CNTL1G_PL             = 0x800414120801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE2_TX_CNTL1G_PL             = 0x800414120801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE2_TX_CNTL2_O_PL                = 0x80044C120701103Full;
//DUPS: 80044C120701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE2_TX_CNTL2_O_PL              = 0x80044C120701103Full;
//DUPS: 80044C120701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE2_TX_CNTL2_O_PL              = 0x80044C120801103Full;
//DUPS: 80044C120801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE2_TX_CNTL2_O_PL              = 0x80044C120701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE2_TX_CNTL2_O_PL            = 0x80044C120701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE2_TX_CNTL2_O_PL            = 0x80044C120701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE2_TX_CNTL2_O_PL            = 0x80044C120801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE2_TX_CNTL2_O_PL            = 0x80044C120801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE2_TX_CNTL2_O_PL             = 0x80044C120701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE2_TX_CNTL2_O_PL             = 0x80044C120701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE2_TX_CNTL2_O_PL            = 0x80044C120801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE2_TX_CNTL2_O_PL            = 0x80044C120801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE2_TX_CNTL3_EO_PL               = 0x800454120701103Full;
//DUPS: 800454120701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE2_TX_CNTL3_EO_PL             = 0x800454120701103Full;
//DUPS: 800454120701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE2_TX_CNTL3_EO_PL             = 0x800454120801103Full;
//DUPS: 800454120801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE2_TX_CNTL3_EO_PL             = 0x800454120701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE2_TX_CNTL3_EO_PL           = 0x800454120701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE2_TX_CNTL3_EO_PL           = 0x800454120701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE2_TX_CNTL3_EO_PL           = 0x800454120801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE2_TX_CNTL3_EO_PL           = 0x800454120801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE2_TX_CNTL3_EO_PL            = 0x800454120701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE2_TX_CNTL3_EO_PL            = 0x800454120701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE2_TX_CNTL3_EO_PL           = 0x800454120801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE2_TX_CNTL3_EO_PL           = 0x800454120801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE2_TX_FIR_ERROR_INJECT_PL       = 0x800434120701103Full;
//DUPS: 800434120701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE2_TX_FIR_ERROR_INJECT_PL     = 0x800434120701103Full;
//DUPS: 800434120701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE2_TX_FIR_ERROR_INJECT_PL     = 0x800434120801103Full;
//DUPS: 800434120801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE2_TX_FIR_ERROR_INJECT_PL     = 0x800434120701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x800434120701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x800434120701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x800434120801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x800434120801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE2_TX_FIR_ERROR_INJECT_PL    = 0x800434120701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE2_TX_FIR_ERROR_INJECT_PL    = 0x800434120701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x800434120801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x800434120801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE2_TX_FIR_MASK_PL               = 0x80042C120701103Full;
//DUPS: 80042C120701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE2_TX_FIR_MASK_PL             = 0x80042C120701103Full;
//DUPS: 80042C120701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE2_TX_FIR_MASK_PL             = 0x80042C120801103Full;
//DUPS: 80042C120801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE2_TX_FIR_MASK_PL             = 0x80042C120701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE2_TX_FIR_MASK_PL           = 0x80042C120701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE2_TX_FIR_MASK_PL           = 0x80042C120701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE2_TX_FIR_MASK_PL           = 0x80042C120801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE2_TX_FIR_MASK_PL           = 0x80042C120801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE2_TX_FIR_MASK_PL            = 0x80042C120701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE2_TX_FIR_MASK_PL            = 0x80042C120701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE2_TX_FIR_MASK_PL           = 0x80042C120801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE2_TX_FIR_MASK_PL           = 0x80042C120801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE2_TX_FIR_PL                    = 0x800424120701103Full;
//DUPS: 800424120701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE2_TX_FIR_PL                  = 0x800424120701103Full;
//DUPS: 800424120701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE2_TX_FIR_PL                  = 0x800424120801103Full;
//DUPS: 800424120801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE2_TX_FIR_PL                  = 0x800424120701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE2_TX_FIR_PL                = 0x800424120701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE2_TX_FIR_PL                = 0x800424120701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE2_TX_FIR_PL                = 0x800424120801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE2_TX_FIR_PL                = 0x800424120801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE2_TX_FIR_PL                 = 0x800424120701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE2_TX_FIR_PL                 = 0x800424120701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE2_TX_FIR_PL                = 0x800424120801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE2_TX_FIR_PL                = 0x800424120801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE2_TX_MODE1_PL                  = 0x800404120701103Full;
//DUPS: 800404120701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE2_TX_MODE1_PL                = 0x800404120701103Full;
//DUPS: 800404120701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE2_TX_MODE1_PL                = 0x800404120801103Full;
//DUPS: 800404120801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE2_TX_MODE1_PL                = 0x800404120701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE2_TX_MODE1_PL              = 0x800404120701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE2_TX_MODE1_PL              = 0x800404120701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE2_TX_MODE1_PL              = 0x800404120801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE2_TX_MODE1_PL              = 0x800404120801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE2_TX_MODE1_PL               = 0x800404120701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE2_TX_MODE1_PL               = 0x800404120701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE2_TX_MODE1_PL              = 0x800404120801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE2_TX_MODE1_PL              = 0x800404120801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE2_TX_MODE2_PL                  = 0x80040C120701103Full;
//DUPS: 80040C120701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE2_TX_MODE2_PL                = 0x80040C120701103Full;
//DUPS: 80040C120701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE2_TX_MODE2_PL                = 0x80040C120801103Full;
//DUPS: 80040C120801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE2_TX_MODE2_PL                = 0x80040C120701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE2_TX_MODE2_PL              = 0x80040C120701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE2_TX_MODE2_PL              = 0x80040C120701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE2_TX_MODE2_PL              = 0x80040C120801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE2_TX_MODE2_PL              = 0x80040C120801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE2_TX_MODE2_PL               = 0x80040C120701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE2_TX_MODE2_PL               = 0x80040C120701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE2_TX_MODE2_PL              = 0x80040C120801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE2_TX_MODE2_PL              = 0x80040C120801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE2_TX_STAT1_PL                  = 0x80041C120701103Full;
//DUPS: 80041C120701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE2_TX_STAT1_PL                = 0x80041C120701103Full;
//DUPS: 80041C120701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE2_TX_STAT1_PL                = 0x80041C120801103Full;
//DUPS: 80041C120801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE2_TX_STAT1_PL                = 0x80041C120701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE2_TX_STAT1_PL              = 0x80041C120701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE2_TX_STAT1_PL              = 0x80041C120701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE2_TX_STAT1_PL              = 0x80041C120801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE2_TX_STAT1_PL              = 0x80041C120801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE2_TX_STAT1_PL               = 0x80041C120701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE2_TX_STAT1_PL               = 0x80041C120701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE2_TX_STAT1_PL              = 0x80041C120801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE2_TX_STAT1_PL              = 0x80041C120801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE3_TX_CNTL1G_PL                 = 0x800414130701103Full;
//DUPS: 800414130701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE3_TX_CNTL1G_PL               = 0x800414130701103Full;
//DUPS: 800414130701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE3_TX_CNTL1G_PL               = 0x800414130801103Full;
//DUPS: 800414130801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE3_TX_CNTL1G_PL               = 0x800414130701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE3_TX_CNTL1G_PL             = 0x800414130701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE3_TX_CNTL1G_PL             = 0x800414130701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE3_TX_CNTL1G_PL             = 0x800414130801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE3_TX_CNTL1G_PL             = 0x800414130801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE3_TX_CNTL1G_PL              = 0x800414130701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE3_TX_CNTL1G_PL              = 0x800414130701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE3_TX_CNTL1G_PL             = 0x800414130801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE3_TX_CNTL1G_PL             = 0x800414130801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE3_TX_CNTL2_O_PL                = 0x80044C130701103Full;
//DUPS: 80044C130701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE3_TX_CNTL2_O_PL              = 0x80044C130701103Full;
//DUPS: 80044C130701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE3_TX_CNTL2_O_PL              = 0x80044C130801103Full;
//DUPS: 80044C130801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE3_TX_CNTL2_O_PL              = 0x80044C130701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE3_TX_CNTL2_O_PL            = 0x80044C130701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE3_TX_CNTL2_O_PL            = 0x80044C130701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE3_TX_CNTL2_O_PL            = 0x80044C130801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE3_TX_CNTL2_O_PL            = 0x80044C130801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE3_TX_CNTL2_O_PL             = 0x80044C130701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE3_TX_CNTL2_O_PL             = 0x80044C130701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE3_TX_CNTL2_O_PL            = 0x80044C130801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE3_TX_CNTL2_O_PL            = 0x80044C130801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE3_TX_CNTL3_EO_PL               = 0x800454130701103Full;
//DUPS: 800454130701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE3_TX_CNTL3_EO_PL             = 0x800454130701103Full;
//DUPS: 800454130701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE3_TX_CNTL3_EO_PL             = 0x800454130801103Full;
//DUPS: 800454130801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE3_TX_CNTL3_EO_PL             = 0x800454130701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE3_TX_CNTL3_EO_PL           = 0x800454130701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE3_TX_CNTL3_EO_PL           = 0x800454130701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE3_TX_CNTL3_EO_PL           = 0x800454130801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE3_TX_CNTL3_EO_PL           = 0x800454130801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE3_TX_CNTL3_EO_PL            = 0x800454130701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE3_TX_CNTL3_EO_PL            = 0x800454130701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE3_TX_CNTL3_EO_PL           = 0x800454130801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE3_TX_CNTL3_EO_PL           = 0x800454130801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE3_TX_FIR_ERROR_INJECT_PL       = 0x800434130701103Full;
//DUPS: 800434130701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE3_TX_FIR_ERROR_INJECT_PL     = 0x800434130701103Full;
//DUPS: 800434130701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE3_TX_FIR_ERROR_INJECT_PL     = 0x800434130801103Full;
//DUPS: 800434130801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE3_TX_FIR_ERROR_INJECT_PL     = 0x800434130701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x800434130701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x800434130701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x800434130801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x800434130801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE3_TX_FIR_ERROR_INJECT_PL    = 0x800434130701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE3_TX_FIR_ERROR_INJECT_PL    = 0x800434130701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x800434130801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x800434130801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE3_TX_FIR_MASK_PL               = 0x80042C130701103Full;
//DUPS: 80042C130701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE3_TX_FIR_MASK_PL             = 0x80042C130701103Full;
//DUPS: 80042C130701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE3_TX_FIR_MASK_PL             = 0x80042C130801103Full;
//DUPS: 80042C130801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE3_TX_FIR_MASK_PL             = 0x80042C130701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE3_TX_FIR_MASK_PL           = 0x80042C130701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE3_TX_FIR_MASK_PL           = 0x80042C130701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE3_TX_FIR_MASK_PL           = 0x80042C130801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE3_TX_FIR_MASK_PL           = 0x80042C130801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE3_TX_FIR_MASK_PL            = 0x80042C130701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE3_TX_FIR_MASK_PL            = 0x80042C130701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE3_TX_FIR_MASK_PL           = 0x80042C130801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE3_TX_FIR_MASK_PL           = 0x80042C130801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE3_TX_FIR_PL                    = 0x800424130701103Full;
//DUPS: 800424130701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE3_TX_FIR_PL                  = 0x800424130701103Full;
//DUPS: 800424130701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE3_TX_FIR_PL                  = 0x800424130801103Full;
//DUPS: 800424130801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE3_TX_FIR_PL                  = 0x800424130701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE3_TX_FIR_PL                = 0x800424130701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE3_TX_FIR_PL                = 0x800424130701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE3_TX_FIR_PL                = 0x800424130801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE3_TX_FIR_PL                = 0x800424130801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE3_TX_FIR_PL                 = 0x800424130701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE3_TX_FIR_PL                 = 0x800424130701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE3_TX_FIR_PL                = 0x800424130801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE3_TX_FIR_PL                = 0x800424130801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE3_TX_MODE1_PL                  = 0x800404130701103Full;
//DUPS: 800404130701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE3_TX_MODE1_PL                = 0x800404130701103Full;
//DUPS: 800404130701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE3_TX_MODE1_PL                = 0x800404130801103Full;
//DUPS: 800404130801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE3_TX_MODE1_PL                = 0x800404130701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE3_TX_MODE1_PL              = 0x800404130701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE3_TX_MODE1_PL              = 0x800404130701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE3_TX_MODE1_PL              = 0x800404130801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE3_TX_MODE1_PL              = 0x800404130801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE3_TX_MODE1_PL               = 0x800404130701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE3_TX_MODE1_PL               = 0x800404130701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE3_TX_MODE1_PL              = 0x800404130801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE3_TX_MODE1_PL              = 0x800404130801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE3_TX_MODE2_PL                  = 0x80040C130701103Full;
//DUPS: 80040C130701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE3_TX_MODE2_PL                = 0x80040C130701103Full;
//DUPS: 80040C130701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE3_TX_MODE2_PL                = 0x80040C130801103Full;
//DUPS: 80040C130801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE3_TX_MODE2_PL                = 0x80040C130701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE3_TX_MODE2_PL              = 0x80040C130701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE3_TX_MODE2_PL              = 0x80040C130701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE3_TX_MODE2_PL              = 0x80040C130801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE3_TX_MODE2_PL              = 0x80040C130801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE3_TX_MODE2_PL               = 0x80040C130701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE3_TX_MODE2_PL               = 0x80040C130701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE3_TX_MODE2_PL              = 0x80040C130801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE3_TX_MODE2_PL              = 0x80040C130801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS4_SLICE3_TX_STAT1_PL                  = 0x80041C130701103Full;
//DUPS: 80041C130701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS4_SLICE3_TX_STAT1_PL                = 0x80041C130701103Full;
//DUPS: 80041C130701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS4_SLICE3_TX_STAT1_PL                = 0x80041C130801103Full;
//DUPS: 80041C130801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS4_SLICE3_TX_STAT1_PL                = 0x80041C130701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS4_SLICE3_TX_STAT1_PL              = 0x80041C130701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS4_SLICE3_TX_STAT1_PL              = 0x80041C130701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS4_SLICE3_TX_STAT1_PL              = 0x80041C130801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS4_SLICE3_TX_STAT1_PL              = 0x80041C130801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS4_SLICE3_TX_STAT1_PL               = 0x80041C130701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS4_SLICE3_TX_STAT1_PL               = 0x80041C130701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS4_SLICE3_TX_STAT1_PL              = 0x80041C130801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS4_SLICE3_TX_STAT1_PL              = 0x80041C130801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE0_TX_CNTL1G_PL                 = 0x800414140701103Full;
//DUPS: 800414140701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE0_TX_CNTL1G_PL               = 0x800414140701103Full;
//DUPS: 800414140701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE0_TX_CNTL1G_PL               = 0x800414140801103Full;
//DUPS: 800414140801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE0_TX_CNTL1G_PL               = 0x800414140701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE0_TX_CNTL1G_PL             = 0x800414140701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE0_TX_CNTL1G_PL             = 0x800414140701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE0_TX_CNTL1G_PL             = 0x800414140801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE0_TX_CNTL1G_PL             = 0x800414140801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE0_TX_CNTL1G_PL              = 0x800414140701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE0_TX_CNTL1G_PL              = 0x800414140701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE0_TX_CNTL1G_PL             = 0x800414140801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE0_TX_CNTL1G_PL             = 0x800414140801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE0_TX_CNTL2_O_PL                = 0x80044C140701103Full;
//DUPS: 80044C140701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE0_TX_CNTL2_O_PL              = 0x80044C140701103Full;
//DUPS: 80044C140701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE0_TX_CNTL2_O_PL              = 0x80044C140801103Full;
//DUPS: 80044C140801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE0_TX_CNTL2_O_PL              = 0x80044C140701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE0_TX_CNTL2_O_PL            = 0x80044C140701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE0_TX_CNTL2_O_PL            = 0x80044C140701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE0_TX_CNTL2_O_PL            = 0x80044C140801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE0_TX_CNTL2_O_PL            = 0x80044C140801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE0_TX_CNTL2_O_PL             = 0x80044C140701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE0_TX_CNTL2_O_PL             = 0x80044C140701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE0_TX_CNTL2_O_PL            = 0x80044C140801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE0_TX_CNTL2_O_PL            = 0x80044C140801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE0_TX_CNTL3_EO_PL               = 0x800454140701103Full;
//DUPS: 800454140701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE0_TX_CNTL3_EO_PL             = 0x800454140701103Full;
//DUPS: 800454140701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE0_TX_CNTL3_EO_PL             = 0x800454140801103Full;
//DUPS: 800454140801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE0_TX_CNTL3_EO_PL             = 0x800454140701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE0_TX_CNTL3_EO_PL           = 0x800454140701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE0_TX_CNTL3_EO_PL           = 0x800454140701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE0_TX_CNTL3_EO_PL           = 0x800454140801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE0_TX_CNTL3_EO_PL           = 0x800454140801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE0_TX_CNTL3_EO_PL            = 0x800454140701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE0_TX_CNTL3_EO_PL            = 0x800454140701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE0_TX_CNTL3_EO_PL           = 0x800454140801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE0_TX_CNTL3_EO_PL           = 0x800454140801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE0_TX_FIR_ERROR_INJECT_PL       = 0x800434140701103Full;
//DUPS: 800434140701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE0_TX_FIR_ERROR_INJECT_PL     = 0x800434140701103Full;
//DUPS: 800434140701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE0_TX_FIR_ERROR_INJECT_PL     = 0x800434140801103Full;
//DUPS: 800434140801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE0_TX_FIR_ERROR_INJECT_PL     = 0x800434140701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x800434140701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x800434140701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x800434140801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x800434140801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE0_TX_FIR_ERROR_INJECT_PL    = 0x800434140701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE0_TX_FIR_ERROR_INJECT_PL    = 0x800434140701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x800434140801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE0_TX_FIR_ERROR_INJECT_PL   = 0x800434140801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE0_TX_FIR_MASK_PL               = 0x80042C140701103Full;
//DUPS: 80042C140701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE0_TX_FIR_MASK_PL             = 0x80042C140701103Full;
//DUPS: 80042C140701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE0_TX_FIR_MASK_PL             = 0x80042C140801103Full;
//DUPS: 80042C140801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE0_TX_FIR_MASK_PL             = 0x80042C140701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE0_TX_FIR_MASK_PL           = 0x80042C140701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE0_TX_FIR_MASK_PL           = 0x80042C140701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE0_TX_FIR_MASK_PL           = 0x80042C140801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE0_TX_FIR_MASK_PL           = 0x80042C140801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE0_TX_FIR_MASK_PL            = 0x80042C140701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE0_TX_FIR_MASK_PL            = 0x80042C140701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE0_TX_FIR_MASK_PL           = 0x80042C140801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE0_TX_FIR_MASK_PL           = 0x80042C140801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE0_TX_FIR_PL                    = 0x800424140701103Full;
//DUPS: 800424140701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE0_TX_FIR_PL                  = 0x800424140701103Full;
//DUPS: 800424140701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE0_TX_FIR_PL                  = 0x800424140801103Full;
//DUPS: 800424140801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE0_TX_FIR_PL                  = 0x800424140701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE0_TX_FIR_PL                = 0x800424140701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE0_TX_FIR_PL                = 0x800424140701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE0_TX_FIR_PL                = 0x800424140801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE0_TX_FIR_PL                = 0x800424140801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE0_TX_FIR_PL                 = 0x800424140701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE0_TX_FIR_PL                 = 0x800424140701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE0_TX_FIR_PL                = 0x800424140801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE0_TX_FIR_PL                = 0x800424140801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE0_TX_MODE1_PL                  = 0x800404140701103Full;
//DUPS: 800404140701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE0_TX_MODE1_PL                = 0x800404140701103Full;
//DUPS: 800404140701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE0_TX_MODE1_PL                = 0x800404140801103Full;
//DUPS: 800404140801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE0_TX_MODE1_PL                = 0x800404140701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE0_TX_MODE1_PL              = 0x800404140701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE0_TX_MODE1_PL              = 0x800404140701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE0_TX_MODE1_PL              = 0x800404140801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE0_TX_MODE1_PL              = 0x800404140801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE0_TX_MODE1_PL               = 0x800404140701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE0_TX_MODE1_PL               = 0x800404140701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE0_TX_MODE1_PL              = 0x800404140801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE0_TX_MODE1_PL              = 0x800404140801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE0_TX_MODE2_PL                  = 0x80040C140701103Full;
//DUPS: 80040C140701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE0_TX_MODE2_PL                = 0x80040C140701103Full;
//DUPS: 80040C140701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE0_TX_MODE2_PL                = 0x80040C140801103Full;
//DUPS: 80040C140801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE0_TX_MODE2_PL                = 0x80040C140701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE0_TX_MODE2_PL              = 0x80040C140701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE0_TX_MODE2_PL              = 0x80040C140701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE0_TX_MODE2_PL              = 0x80040C140801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE0_TX_MODE2_PL              = 0x80040C140801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE0_TX_MODE2_PL               = 0x80040C140701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE0_TX_MODE2_PL               = 0x80040C140701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE0_TX_MODE2_PL              = 0x80040C140801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE0_TX_MODE2_PL              = 0x80040C140801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE0_TX_STAT1_PL                  = 0x80041C140701103Full;
//DUPS: 80041C140701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE0_TX_STAT1_PL                = 0x80041C140701103Full;
//DUPS: 80041C140701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE0_TX_STAT1_PL                = 0x80041C140801103Full;
//DUPS: 80041C140801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE0_TX_STAT1_PL                = 0x80041C140701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE0_TX_STAT1_PL              = 0x80041C140701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE0_TX_STAT1_PL              = 0x80041C140701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE0_TX_STAT1_PL              = 0x80041C140801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE0_TX_STAT1_PL              = 0x80041C140801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE0_TX_STAT1_PL               = 0x80041C140701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE0_TX_STAT1_PL               = 0x80041C140701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE0_TX_STAT1_PL              = 0x80041C140801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE0_TX_STAT1_PL              = 0x80041C140801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE1_TX_CNTL1G_PL                 = 0x800414150701103Full;
//DUPS: 800414150701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE1_TX_CNTL1G_PL               = 0x800414150701103Full;
//DUPS: 800414150701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE1_TX_CNTL1G_PL               = 0x800414150801103Full;
//DUPS: 800414150801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE1_TX_CNTL1G_PL               = 0x800414150701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE1_TX_CNTL1G_PL             = 0x800414150701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE1_TX_CNTL1G_PL             = 0x800414150701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE1_TX_CNTL1G_PL             = 0x800414150801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE1_TX_CNTL1G_PL             = 0x800414150801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE1_TX_CNTL1G_PL              = 0x800414150701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE1_TX_CNTL1G_PL              = 0x800414150701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE1_TX_CNTL1G_PL             = 0x800414150801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE1_TX_CNTL1G_PL             = 0x800414150801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE1_TX_CNTL2_O_PL                = 0x80044C150701103Full;
//DUPS: 80044C150701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE1_TX_CNTL2_O_PL              = 0x80044C150701103Full;
//DUPS: 80044C150701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE1_TX_CNTL2_O_PL              = 0x80044C150801103Full;
//DUPS: 80044C150801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE1_TX_CNTL2_O_PL              = 0x80044C150701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE1_TX_CNTL2_O_PL            = 0x80044C150701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE1_TX_CNTL2_O_PL            = 0x80044C150701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE1_TX_CNTL2_O_PL            = 0x80044C150801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE1_TX_CNTL2_O_PL            = 0x80044C150801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE1_TX_CNTL2_O_PL             = 0x80044C150701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE1_TX_CNTL2_O_PL             = 0x80044C150701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE1_TX_CNTL2_O_PL            = 0x80044C150801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE1_TX_CNTL2_O_PL            = 0x80044C150801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE1_TX_CNTL3_EO_PL               = 0x800454150701103Full;
//DUPS: 800454150701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE1_TX_CNTL3_EO_PL             = 0x800454150701103Full;
//DUPS: 800454150701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE1_TX_CNTL3_EO_PL             = 0x800454150801103Full;
//DUPS: 800454150801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE1_TX_CNTL3_EO_PL             = 0x800454150701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE1_TX_CNTL3_EO_PL           = 0x800454150701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE1_TX_CNTL3_EO_PL           = 0x800454150701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE1_TX_CNTL3_EO_PL           = 0x800454150801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE1_TX_CNTL3_EO_PL           = 0x800454150801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE1_TX_CNTL3_EO_PL            = 0x800454150701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE1_TX_CNTL3_EO_PL            = 0x800454150701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE1_TX_CNTL3_EO_PL           = 0x800454150801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE1_TX_CNTL3_EO_PL           = 0x800454150801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE1_TX_FIR_ERROR_INJECT_PL       = 0x800434150701103Full;
//DUPS: 800434150701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE1_TX_FIR_ERROR_INJECT_PL     = 0x800434150701103Full;
//DUPS: 800434150701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE1_TX_FIR_ERROR_INJECT_PL     = 0x800434150801103Full;
//DUPS: 800434150801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE1_TX_FIR_ERROR_INJECT_PL     = 0x800434150701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x800434150701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x800434150701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x800434150801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x800434150801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE1_TX_FIR_ERROR_INJECT_PL    = 0x800434150701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE1_TX_FIR_ERROR_INJECT_PL    = 0x800434150701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x800434150801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE1_TX_FIR_ERROR_INJECT_PL   = 0x800434150801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE1_TX_FIR_MASK_PL               = 0x80042C150701103Full;
//DUPS: 80042C150701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE1_TX_FIR_MASK_PL             = 0x80042C150701103Full;
//DUPS: 80042C150701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE1_TX_FIR_MASK_PL             = 0x80042C150801103Full;
//DUPS: 80042C150801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE1_TX_FIR_MASK_PL             = 0x80042C150701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE1_TX_FIR_MASK_PL           = 0x80042C150701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE1_TX_FIR_MASK_PL           = 0x80042C150701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE1_TX_FIR_MASK_PL           = 0x80042C150801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE1_TX_FIR_MASK_PL           = 0x80042C150801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE1_TX_FIR_MASK_PL            = 0x80042C150701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE1_TX_FIR_MASK_PL            = 0x80042C150701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE1_TX_FIR_MASK_PL           = 0x80042C150801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE1_TX_FIR_MASK_PL           = 0x80042C150801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE1_TX_FIR_PL                    = 0x800424150701103Full;
//DUPS: 800424150701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE1_TX_FIR_PL                  = 0x800424150701103Full;
//DUPS: 800424150701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE1_TX_FIR_PL                  = 0x800424150801103Full;
//DUPS: 800424150801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE1_TX_FIR_PL                  = 0x800424150701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE1_TX_FIR_PL                = 0x800424150701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE1_TX_FIR_PL                = 0x800424150701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE1_TX_FIR_PL                = 0x800424150801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE1_TX_FIR_PL                = 0x800424150801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE1_TX_FIR_PL                 = 0x800424150701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE1_TX_FIR_PL                 = 0x800424150701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE1_TX_FIR_PL                = 0x800424150801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE1_TX_FIR_PL                = 0x800424150801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE1_TX_MODE1_PL                  = 0x800404150701103Full;
//DUPS: 800404150701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE1_TX_MODE1_PL                = 0x800404150701103Full;
//DUPS: 800404150701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE1_TX_MODE1_PL                = 0x800404150801103Full;
//DUPS: 800404150801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE1_TX_MODE1_PL                = 0x800404150701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE1_TX_MODE1_PL              = 0x800404150701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE1_TX_MODE1_PL              = 0x800404150701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE1_TX_MODE1_PL              = 0x800404150801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE1_TX_MODE1_PL              = 0x800404150801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE1_TX_MODE1_PL               = 0x800404150701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE1_TX_MODE1_PL               = 0x800404150701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE1_TX_MODE1_PL              = 0x800404150801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE1_TX_MODE1_PL              = 0x800404150801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE1_TX_MODE2_PL                  = 0x80040C150701103Full;
//DUPS: 80040C150701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE1_TX_MODE2_PL                = 0x80040C150701103Full;
//DUPS: 80040C150701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE1_TX_MODE2_PL                = 0x80040C150801103Full;
//DUPS: 80040C150801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE1_TX_MODE2_PL                = 0x80040C150701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE1_TX_MODE2_PL              = 0x80040C150701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE1_TX_MODE2_PL              = 0x80040C150701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE1_TX_MODE2_PL              = 0x80040C150801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE1_TX_MODE2_PL              = 0x80040C150801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE1_TX_MODE2_PL               = 0x80040C150701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE1_TX_MODE2_PL               = 0x80040C150701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE1_TX_MODE2_PL              = 0x80040C150801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE1_TX_MODE2_PL              = 0x80040C150801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE1_TX_STAT1_PL                  = 0x80041C150701103Full;
//DUPS: 80041C150701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE1_TX_STAT1_PL                = 0x80041C150701103Full;
//DUPS: 80041C150701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE1_TX_STAT1_PL                = 0x80041C150801103Full;
//DUPS: 80041C150801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE1_TX_STAT1_PL                = 0x80041C150701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE1_TX_STAT1_PL              = 0x80041C150701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE1_TX_STAT1_PL              = 0x80041C150701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE1_TX_STAT1_PL              = 0x80041C150801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE1_TX_STAT1_PL              = 0x80041C150801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE1_TX_STAT1_PL               = 0x80041C150701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE1_TX_STAT1_PL               = 0x80041C150701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE1_TX_STAT1_PL              = 0x80041C150801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE1_TX_STAT1_PL              = 0x80041C150801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE2_TX_CNTL1G_PL                 = 0x800414160701103Full;
//DUPS: 800414160701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE2_TX_CNTL1G_PL               = 0x800414160701103Full;
//DUPS: 800414160701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE2_TX_CNTL1G_PL               = 0x800414160801103Full;
//DUPS: 800414160801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE2_TX_CNTL1G_PL               = 0x800414160701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE2_TX_CNTL1G_PL             = 0x800414160701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE2_TX_CNTL1G_PL             = 0x800414160701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE2_TX_CNTL1G_PL             = 0x800414160801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE2_TX_CNTL1G_PL             = 0x800414160801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE2_TX_CNTL1G_PL              = 0x800414160701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE2_TX_CNTL1G_PL              = 0x800414160701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE2_TX_CNTL1G_PL             = 0x800414160801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE2_TX_CNTL1G_PL             = 0x800414160801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE2_TX_CNTL2_O_PL                = 0x80044C160701103Full;
//DUPS: 80044C160701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE2_TX_CNTL2_O_PL              = 0x80044C160701103Full;
//DUPS: 80044C160701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE2_TX_CNTL2_O_PL              = 0x80044C160801103Full;
//DUPS: 80044C160801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE2_TX_CNTL2_O_PL              = 0x80044C160701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE2_TX_CNTL2_O_PL            = 0x80044C160701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE2_TX_CNTL2_O_PL            = 0x80044C160701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE2_TX_CNTL2_O_PL            = 0x80044C160801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE2_TX_CNTL2_O_PL            = 0x80044C160801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE2_TX_CNTL2_O_PL             = 0x80044C160701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE2_TX_CNTL2_O_PL             = 0x80044C160701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE2_TX_CNTL2_O_PL            = 0x80044C160801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE2_TX_CNTL2_O_PL            = 0x80044C160801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE2_TX_CNTL3_EO_PL               = 0x800454160701103Full;
//DUPS: 800454160701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE2_TX_CNTL3_EO_PL             = 0x800454160701103Full;
//DUPS: 800454160701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE2_TX_CNTL3_EO_PL             = 0x800454160801103Full;
//DUPS: 800454160801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE2_TX_CNTL3_EO_PL             = 0x800454160701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE2_TX_CNTL3_EO_PL           = 0x800454160701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE2_TX_CNTL3_EO_PL           = 0x800454160701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE2_TX_CNTL3_EO_PL           = 0x800454160801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE2_TX_CNTL3_EO_PL           = 0x800454160801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE2_TX_CNTL3_EO_PL            = 0x800454160701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE2_TX_CNTL3_EO_PL            = 0x800454160701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE2_TX_CNTL3_EO_PL           = 0x800454160801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE2_TX_CNTL3_EO_PL           = 0x800454160801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE2_TX_FIR_ERROR_INJECT_PL       = 0x800434160701103Full;
//DUPS: 800434160701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE2_TX_FIR_ERROR_INJECT_PL     = 0x800434160701103Full;
//DUPS: 800434160701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE2_TX_FIR_ERROR_INJECT_PL     = 0x800434160801103Full;
//DUPS: 800434160801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE2_TX_FIR_ERROR_INJECT_PL     = 0x800434160701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x800434160701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x800434160701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x800434160801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x800434160801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE2_TX_FIR_ERROR_INJECT_PL    = 0x800434160701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE2_TX_FIR_ERROR_INJECT_PL    = 0x800434160701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x800434160801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE2_TX_FIR_ERROR_INJECT_PL   = 0x800434160801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE2_TX_FIR_MASK_PL               = 0x80042C160701103Full;
//DUPS: 80042C160701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE2_TX_FIR_MASK_PL             = 0x80042C160701103Full;
//DUPS: 80042C160701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE2_TX_FIR_MASK_PL             = 0x80042C160801103Full;
//DUPS: 80042C160801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE2_TX_FIR_MASK_PL             = 0x80042C160701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE2_TX_FIR_MASK_PL           = 0x80042C160701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE2_TX_FIR_MASK_PL           = 0x80042C160701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE2_TX_FIR_MASK_PL           = 0x80042C160801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE2_TX_FIR_MASK_PL           = 0x80042C160801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE2_TX_FIR_MASK_PL            = 0x80042C160701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE2_TX_FIR_MASK_PL            = 0x80042C160701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE2_TX_FIR_MASK_PL           = 0x80042C160801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE2_TX_FIR_MASK_PL           = 0x80042C160801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE2_TX_FIR_PL                    = 0x800424160701103Full;
//DUPS: 800424160701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE2_TX_FIR_PL                  = 0x800424160701103Full;
//DUPS: 800424160701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE2_TX_FIR_PL                  = 0x800424160801103Full;
//DUPS: 800424160801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE2_TX_FIR_PL                  = 0x800424160701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE2_TX_FIR_PL                = 0x800424160701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE2_TX_FIR_PL                = 0x800424160701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE2_TX_FIR_PL                = 0x800424160801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE2_TX_FIR_PL                = 0x800424160801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE2_TX_FIR_PL                 = 0x800424160701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE2_TX_FIR_PL                 = 0x800424160701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE2_TX_FIR_PL                = 0x800424160801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE2_TX_FIR_PL                = 0x800424160801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE2_TX_MODE1_PL                  = 0x800404160701103Full;
//DUPS: 800404160701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE2_TX_MODE1_PL                = 0x800404160701103Full;
//DUPS: 800404160701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE2_TX_MODE1_PL                = 0x800404160801103Full;
//DUPS: 800404160801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE2_TX_MODE1_PL                = 0x800404160701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE2_TX_MODE1_PL              = 0x800404160701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE2_TX_MODE1_PL              = 0x800404160701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE2_TX_MODE1_PL              = 0x800404160801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE2_TX_MODE1_PL              = 0x800404160801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE2_TX_MODE1_PL               = 0x800404160701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE2_TX_MODE1_PL               = 0x800404160701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE2_TX_MODE1_PL              = 0x800404160801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE2_TX_MODE1_PL              = 0x800404160801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE2_TX_MODE2_PL                  = 0x80040C160701103Full;
//DUPS: 80040C160701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE2_TX_MODE2_PL                = 0x80040C160701103Full;
//DUPS: 80040C160701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE2_TX_MODE2_PL                = 0x80040C160801103Full;
//DUPS: 80040C160801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE2_TX_MODE2_PL                = 0x80040C160701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE2_TX_MODE2_PL              = 0x80040C160701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE2_TX_MODE2_PL              = 0x80040C160701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE2_TX_MODE2_PL              = 0x80040C160801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE2_TX_MODE2_PL              = 0x80040C160801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE2_TX_MODE2_PL               = 0x80040C160701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE2_TX_MODE2_PL               = 0x80040C160701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE2_TX_MODE2_PL              = 0x80040C160801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE2_TX_MODE2_PL              = 0x80040C160801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE2_TX_STAT1_PL                  = 0x80041C160701103Full;
//DUPS: 80041C160701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE2_TX_STAT1_PL                = 0x80041C160701103Full;
//DUPS: 80041C160701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE2_TX_STAT1_PL                = 0x80041C160801103Full;
//DUPS: 80041C160801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE2_TX_STAT1_PL                = 0x80041C160701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE2_TX_STAT1_PL              = 0x80041C160701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE2_TX_STAT1_PL              = 0x80041C160701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE2_TX_STAT1_PL              = 0x80041C160801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE2_TX_STAT1_PL              = 0x80041C160801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE2_TX_STAT1_PL               = 0x80041C160701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE2_TX_STAT1_PL               = 0x80041C160701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE2_TX_STAT1_PL              = 0x80041C160801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE2_TX_STAT1_PL              = 0x80041C160801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE3_TX_CNTL1G_PL                 = 0x800414170701103Full;
//DUPS: 800414170701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE3_TX_CNTL1G_PL               = 0x800414170701103Full;
//DUPS: 800414170701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE3_TX_CNTL1G_PL               = 0x800414170801103Full;
//DUPS: 800414170801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE3_TX_CNTL1G_PL               = 0x800414170701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE3_TX_CNTL1G_PL             = 0x800414170701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE3_TX_CNTL1G_PL             = 0x800414170701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE3_TX_CNTL1G_PL             = 0x800414170801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE3_TX_CNTL1G_PL             = 0x800414170801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE3_TX_CNTL1G_PL              = 0x800414170701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE3_TX_CNTL1G_PL              = 0x800414170701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE3_TX_CNTL1G_PL             = 0x800414170801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE3_TX_CNTL1G_PL             = 0x800414170801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE3_TX_CNTL2_O_PL                = 0x80044C170701103Full;
//DUPS: 80044C170701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE3_TX_CNTL2_O_PL              = 0x80044C170701103Full;
//DUPS: 80044C170701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE3_TX_CNTL2_O_PL              = 0x80044C170801103Full;
//DUPS: 80044C170801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE3_TX_CNTL2_O_PL              = 0x80044C170701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE3_TX_CNTL2_O_PL            = 0x80044C170701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE3_TX_CNTL2_O_PL            = 0x80044C170701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE3_TX_CNTL2_O_PL            = 0x80044C170801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE3_TX_CNTL2_O_PL            = 0x80044C170801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE3_TX_CNTL2_O_PL             = 0x80044C170701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE3_TX_CNTL2_O_PL             = 0x80044C170701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE3_TX_CNTL2_O_PL            = 0x80044C170801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE3_TX_CNTL2_O_PL            = 0x80044C170801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE3_TX_CNTL3_EO_PL               = 0x800454170701103Full;
//DUPS: 800454170701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE3_TX_CNTL3_EO_PL             = 0x800454170701103Full;
//DUPS: 800454170701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE3_TX_CNTL3_EO_PL             = 0x800454170801103Full;
//DUPS: 800454170801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE3_TX_CNTL3_EO_PL             = 0x800454170701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE3_TX_CNTL3_EO_PL           = 0x800454170701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE3_TX_CNTL3_EO_PL           = 0x800454170701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE3_TX_CNTL3_EO_PL           = 0x800454170801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE3_TX_CNTL3_EO_PL           = 0x800454170801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE3_TX_CNTL3_EO_PL            = 0x800454170701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE3_TX_CNTL3_EO_PL            = 0x800454170701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE3_TX_CNTL3_EO_PL           = 0x800454170801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE3_TX_CNTL3_EO_PL           = 0x800454170801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE3_TX_FIR_ERROR_INJECT_PL       = 0x800434170701103Full;
//DUPS: 800434170701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE3_TX_FIR_ERROR_INJECT_PL     = 0x800434170701103Full;
//DUPS: 800434170701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE3_TX_FIR_ERROR_INJECT_PL     = 0x800434170801103Full;
//DUPS: 800434170801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE3_TX_FIR_ERROR_INJECT_PL     = 0x800434170701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x800434170701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x800434170701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x800434170801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x800434170801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE3_TX_FIR_ERROR_INJECT_PL    = 0x800434170701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE3_TX_FIR_ERROR_INJECT_PL    = 0x800434170701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x800434170801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE3_TX_FIR_ERROR_INJECT_PL   = 0x800434170801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE3_TX_FIR_MASK_PL               = 0x80042C170701103Full;
//DUPS: 80042C170701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE3_TX_FIR_MASK_PL             = 0x80042C170701103Full;
//DUPS: 80042C170701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE3_TX_FIR_MASK_PL             = 0x80042C170801103Full;
//DUPS: 80042C170801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE3_TX_FIR_MASK_PL             = 0x80042C170701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE3_TX_FIR_MASK_PL           = 0x80042C170701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE3_TX_FIR_MASK_PL           = 0x80042C170701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE3_TX_FIR_MASK_PL           = 0x80042C170801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE3_TX_FIR_MASK_PL           = 0x80042C170801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE3_TX_FIR_MASK_PL            = 0x80042C170701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE3_TX_FIR_MASK_PL            = 0x80042C170701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE3_TX_FIR_MASK_PL           = 0x80042C170801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE3_TX_FIR_MASK_PL           = 0x80042C170801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE3_TX_FIR_PL                    = 0x800424170701103Full;
//DUPS: 800424170701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE3_TX_FIR_PL                  = 0x800424170701103Full;
//DUPS: 800424170701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE3_TX_FIR_PL                  = 0x800424170801103Full;
//DUPS: 800424170801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE3_TX_FIR_PL                  = 0x800424170701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE3_TX_FIR_PL                = 0x800424170701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE3_TX_FIR_PL                = 0x800424170701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE3_TX_FIR_PL                = 0x800424170801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE3_TX_FIR_PL                = 0x800424170801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE3_TX_FIR_PL                 = 0x800424170701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE3_TX_FIR_PL                 = 0x800424170701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE3_TX_FIR_PL                = 0x800424170801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE3_TX_FIR_PL                = 0x800424170801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE3_TX_MODE1_PL                  = 0x800404170701103Full;
//DUPS: 800404170701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE3_TX_MODE1_PL                = 0x800404170701103Full;
//DUPS: 800404170701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE3_TX_MODE1_PL                = 0x800404170801103Full;
//DUPS: 800404170801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE3_TX_MODE1_PL                = 0x800404170701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE3_TX_MODE1_PL              = 0x800404170701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE3_TX_MODE1_PL              = 0x800404170701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE3_TX_MODE1_PL              = 0x800404170801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE3_TX_MODE1_PL              = 0x800404170801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE3_TX_MODE1_PL               = 0x800404170701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE3_TX_MODE1_PL               = 0x800404170701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE3_TX_MODE1_PL              = 0x800404170801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE3_TX_MODE1_PL              = 0x800404170801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE3_TX_MODE2_PL                  = 0x80040C170701103Full;
//DUPS: 80040C170701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE3_TX_MODE2_PL                = 0x80040C170701103Full;
//DUPS: 80040C170701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE3_TX_MODE2_PL                = 0x80040C170801103Full;
//DUPS: 80040C170801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE3_TX_MODE2_PL                = 0x80040C170701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE3_TX_MODE2_PL              = 0x80040C170701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE3_TX_MODE2_PL              = 0x80040C170701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE3_TX_MODE2_PL              = 0x80040C170801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE3_TX_MODE2_PL              = 0x80040C170801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE3_TX_MODE2_PL               = 0x80040C170701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE3_TX_MODE2_PL               = 0x80040C170701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE3_TX_MODE2_PL              = 0x80040C170801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE3_TX_MODE2_PL              = 0x80040C170801143Full;


static const uint64_t P9A_MC_TX0_TXPACKS5_SLICE3_TX_STAT1_PL                  = 0x80041C170701103Full;
//DUPS: 80041C170701143F,
static const uint64_t P9A_MC_0_TX0_TXPACKS5_SLICE3_TX_STAT1_PL                = 0x80041C170701103Full;
//DUPS: 80041C170701143F,
static const uint64_t P9A_MC_1_TX0_TXPACKS5_SLICE3_TX_STAT1_PL                = 0x80041C170801103Full;
//DUPS: 80041C170801143F,

static const uint64_t P9A_OMIC_TX0_TXPACKS5_SLICE3_TX_STAT1_PL                = 0x80041C170701103Full;

static const uint64_t P9A_OMIC_0_TX0_TXPACKS5_SLICE3_TX_STAT1_PL              = 0x80041C170701103Full;

static const uint64_t P9A_OMIC_1_TX0_TXPACKS5_SLICE3_TX_STAT1_PL              = 0x80041C170701143Full;

static const uint64_t P9A_OMIC_3_TX0_TXPACKS5_SLICE3_TX_STAT1_PL              = 0x80041C170801103Full;

static const uint64_t P9A_OMIC_4_TX0_TXPACKS5_SLICE3_TX_STAT1_PL              = 0x80041C170801143Full;

static const uint64_t P9A_OMI_2_TX0_TXPACKS5_SLICE3_TX_STAT1_PL               = 0x80041C170701103Full;

static const uint64_t P9A_OMI_5_TX0_TXPACKS5_SLICE3_TX_STAT1_PL               = 0x80041C170701143Full;

static const uint64_t P9A_OMI_10_TX0_TXPACKS5_SLICE3_TX_STAT1_PL              = 0x80041C170801103Full;

static const uint64_t P9A_OMI_13_TX0_TXPACKS5_SLICE3_TX_STAT1_PL              = 0x80041C170801143Full;


static const uint64_t P9A_MC_TX_CTLSM_CNTL1_EO_PG                             = 0x800D34000701103Full;
//DUPS: 800D34000701143F, 800D34000701183F,
static const uint64_t P9A_MC_0_TX_CTLSM_CNTL1_EO_PG                           = 0x800D34000701103Full;
//DUPS: 800D34000701143F, 800D34000701183F,
static const uint64_t P9A_MC_1_TX_CTLSM_CNTL1_EO_PG                           = 0x800D34000801103Full;
//DUPS: 800D34000801143F, 800D34000801183F,
static const uint64_t P9A_OMIC_TX_CTLSM_CNTL1_EO_PG                           = 0x800D34000701103Full;

static const uint64_t P9A_OMIC_0_TX_CTLSM_CNTL1_EO_PG                         = 0x800D34000701103Full;

static const uint64_t P9A_OMIC_1_TX_CTLSM_CNTL1_EO_PG                         = 0x800D34000701143Full;

static const uint64_t P9A_OMIC_2_TX_CTLSM_CNTL1_EO_PG                         = 0x800D34000701183Full;

static const uint64_t P9A_OMIC_3_TX_CTLSM_CNTL1_EO_PG                         = 0x800D34000801103Full;

static const uint64_t P9A_OMIC_4_TX_CTLSM_CNTL1_EO_PG                         = 0x800D34000801143Full;

static const uint64_t P9A_OMIC_5_TX_CTLSM_CNTL1_EO_PG                         = 0x800D34000801183Full;


static const uint64_t P9A_MC_TX_CTLSM_CNTL1_O_PG                              = 0x800D84000701103Full;
//DUPS: 800D84000701143F, 800D84000701183F,
static const uint64_t P9A_MC_0_TX_CTLSM_CNTL1_O_PG                            = 0x800D84000701103Full;
//DUPS: 800D84000701143F, 800D84000701183F,
static const uint64_t P9A_MC_1_TX_CTLSM_CNTL1_O_PG                            = 0x800D84000801103Full;
//DUPS: 800D84000801143F, 800D84000801183F,
static const uint64_t P9A_OMIC_TX_CTLSM_CNTL1_O_PG                            = 0x800D84000701103Full;

static const uint64_t P9A_OMIC_0_TX_CTLSM_CNTL1_O_PG                          = 0x800D84000701103Full;

static const uint64_t P9A_OMIC_1_TX_CTLSM_CNTL1_O_PG                          = 0x800D84000701143Full;

static const uint64_t P9A_OMIC_2_TX_CTLSM_CNTL1_O_PG                          = 0x800D84000701183Full;

static const uint64_t P9A_OMIC_3_TX_CTLSM_CNTL1_O_PG                          = 0x800D84000801103Full;

static const uint64_t P9A_OMIC_4_TX_CTLSM_CNTL1_O_PG                          = 0x800D84000801143Full;

static const uint64_t P9A_OMIC_5_TX_CTLSM_CNTL1_O_PG                          = 0x800D84000801183Full;


static const uint64_t P9A_MC_TX_CTLSM_CNTL2_EO_PG                             = 0x800D3C000701103Full;
//DUPS: 800D3C000701143F, 800D3C000701183F,
static const uint64_t P9A_MC_0_TX_CTLSM_CNTL2_EO_PG                           = 0x800D3C000701103Full;
//DUPS: 800D3C000701143F, 800D3C000701183F,
static const uint64_t P9A_MC_1_TX_CTLSM_CNTL2_EO_PG                           = 0x800D3C000801103Full;
//DUPS: 800D3C000801143F, 800D3C000801183F,
static const uint64_t P9A_OMIC_TX_CTLSM_CNTL2_EO_PG                           = 0x800D3C000701103Full;

static const uint64_t P9A_OMIC_0_TX_CTLSM_CNTL2_EO_PG                         = 0x800D3C000701103Full;

static const uint64_t P9A_OMIC_1_TX_CTLSM_CNTL2_EO_PG                         = 0x800D3C000701143Full;

static const uint64_t P9A_OMIC_2_TX_CTLSM_CNTL2_EO_PG                         = 0x800D3C000701183Full;

static const uint64_t P9A_OMIC_3_TX_CTLSM_CNTL2_EO_PG                         = 0x800D3C000801103Full;

static const uint64_t P9A_OMIC_4_TX_CTLSM_CNTL2_EO_PG                         = 0x800D3C000801143Full;

static const uint64_t P9A_OMIC_5_TX_CTLSM_CNTL2_EO_PG                         = 0x800D3C000801183Full;


static const uint64_t P9A_MC_TX_CTLSM_CNTL2_O_PG                              = 0x800D8C000701103Full;
//DUPS: 800D8C000701143F, 800D8C000701183F,
static const uint64_t P9A_MC_0_TX_CTLSM_CNTL2_O_PG                            = 0x800D8C000701103Full;
//DUPS: 800D8C000701143F, 800D8C000701183F,
static const uint64_t P9A_MC_1_TX_CTLSM_CNTL2_O_PG                            = 0x800D8C000801103Full;
//DUPS: 800D8C000801143F, 800D8C000801183F,
static const uint64_t P9A_OMIC_TX_CTLSM_CNTL2_O_PG                            = 0x800D8C000701103Full;

static const uint64_t P9A_OMIC_0_TX_CTLSM_CNTL2_O_PG                          = 0x800D8C000701103Full;

static const uint64_t P9A_OMIC_1_TX_CTLSM_CNTL2_O_PG                          = 0x800D8C000701143Full;

static const uint64_t P9A_OMIC_2_TX_CTLSM_CNTL2_O_PG                          = 0x800D8C000701183Full;

static const uint64_t P9A_OMIC_3_TX_CTLSM_CNTL2_O_PG                          = 0x800D8C000801103Full;

static const uint64_t P9A_OMIC_4_TX_CTLSM_CNTL2_O_PG                          = 0x800D8C000801143Full;

static const uint64_t P9A_OMIC_5_TX_CTLSM_CNTL2_O_PG                          = 0x800D8C000801183Full;


static const uint64_t P9A_MC_TX_CTLSM_CNTL3_EO_PG                             = 0x800D44000701103Full;
//DUPS: 800D44000701143F, 800D44000701183F,
static const uint64_t P9A_MC_0_TX_CTLSM_CNTL3_EO_PG                           = 0x800D44000701103Full;
//DUPS: 800D44000701143F, 800D44000701183F,
static const uint64_t P9A_MC_1_TX_CTLSM_CNTL3_EO_PG                           = 0x800D44000801103Full;
//DUPS: 800D44000801143F, 800D44000801183F,
static const uint64_t P9A_OMIC_TX_CTLSM_CNTL3_EO_PG                           = 0x800D44000701103Full;

static const uint64_t P9A_OMIC_0_TX_CTLSM_CNTL3_EO_PG                         = 0x800D44000701103Full;

static const uint64_t P9A_OMIC_1_TX_CTLSM_CNTL3_EO_PG                         = 0x800D44000701143Full;

static const uint64_t P9A_OMIC_2_TX_CTLSM_CNTL3_EO_PG                         = 0x800D44000701183Full;

static const uint64_t P9A_OMIC_3_TX_CTLSM_CNTL3_EO_PG                         = 0x800D44000801103Full;

static const uint64_t P9A_OMIC_4_TX_CTLSM_CNTL3_EO_PG                         = 0x800D44000801143Full;

static const uint64_t P9A_OMIC_5_TX_CTLSM_CNTL3_EO_PG                         = 0x800D44000801183Full;


static const uint64_t P9A_MC_TX_CTLSM_CNTL4_EO_PG                             = 0x800D4C000701103Full;
//DUPS: 800D4C000701143F, 800D4C000701183F,
static const uint64_t P9A_MC_0_TX_CTLSM_CNTL4_EO_PG                           = 0x800D4C000701103Full;
//DUPS: 800D4C000701143F, 800D4C000701183F,
static const uint64_t P9A_MC_1_TX_CTLSM_CNTL4_EO_PG                           = 0x800D4C000801103Full;
//DUPS: 800D4C000801143F, 800D4C000801183F,
static const uint64_t P9A_OMIC_TX_CTLSM_CNTL4_EO_PG                           = 0x800D4C000701103Full;

static const uint64_t P9A_OMIC_0_TX_CTLSM_CNTL4_EO_PG                         = 0x800D4C000701103Full;

static const uint64_t P9A_OMIC_1_TX_CTLSM_CNTL4_EO_PG                         = 0x800D4C000701143Full;

static const uint64_t P9A_OMIC_2_TX_CTLSM_CNTL4_EO_PG                         = 0x800D4C000701183Full;

static const uint64_t P9A_OMIC_3_TX_CTLSM_CNTL4_EO_PG                         = 0x800D4C000801103Full;

static const uint64_t P9A_OMIC_4_TX_CTLSM_CNTL4_EO_PG                         = 0x800D4C000801143Full;

static const uint64_t P9A_OMIC_5_TX_CTLSM_CNTL4_EO_PG                         = 0x800D4C000801183Full;


static const uint64_t P9A_MC_TX_CTLSM_CNTL5_EO_PG                             = 0x800D54000701103Full;
//DUPS: 800D54000701143F, 800D54000701183F,
static const uint64_t P9A_MC_0_TX_CTLSM_CNTL5_EO_PG                           = 0x800D54000701103Full;
//DUPS: 800D54000701143F, 800D54000701183F,
static const uint64_t P9A_MC_1_TX_CTLSM_CNTL5_EO_PG                           = 0x800D54000801103Full;
//DUPS: 800D54000801143F, 800D54000801183F,
static const uint64_t P9A_OMIC_TX_CTLSM_CNTL5_EO_PG                           = 0x800D54000701103Full;

static const uint64_t P9A_OMIC_0_TX_CTLSM_CNTL5_EO_PG                         = 0x800D54000701103Full;

static const uint64_t P9A_OMIC_1_TX_CTLSM_CNTL5_EO_PG                         = 0x800D54000701143Full;

static const uint64_t P9A_OMIC_2_TX_CTLSM_CNTL5_EO_PG                         = 0x800D54000701183Full;

static const uint64_t P9A_OMIC_3_TX_CTLSM_CNTL5_EO_PG                         = 0x800D54000801103Full;

static const uint64_t P9A_OMIC_4_TX_CTLSM_CNTL5_EO_PG                         = 0x800D54000801143Full;

static const uint64_t P9A_OMIC_5_TX_CTLSM_CNTL5_EO_PG                         = 0x800D54000801183Full;


static const uint64_t P9A_MC_TX_CTLSM_CNTL6_EO_PG                             = 0x800D5C000701103Full;
//DUPS: 800D5C000701143F, 800D5C000701183F,
static const uint64_t P9A_MC_0_TX_CTLSM_CNTL6_EO_PG                           = 0x800D5C000701103Full;
//DUPS: 800D5C000701143F, 800D5C000701183F,
static const uint64_t P9A_MC_1_TX_CTLSM_CNTL6_EO_PG                           = 0x800D5C000801103Full;
//DUPS: 800D5C000801143F, 800D5C000801183F,
static const uint64_t P9A_OMIC_TX_CTLSM_CNTL6_EO_PG                           = 0x800D5C000701103Full;

static const uint64_t P9A_OMIC_0_TX_CTLSM_CNTL6_EO_PG                         = 0x800D5C000701103Full;

static const uint64_t P9A_OMIC_1_TX_CTLSM_CNTL6_EO_PG                         = 0x800D5C000701143Full;

static const uint64_t P9A_OMIC_2_TX_CTLSM_CNTL6_EO_PG                         = 0x800D5C000701183Full;

static const uint64_t P9A_OMIC_3_TX_CTLSM_CNTL6_EO_PG                         = 0x800D5C000801103Full;

static const uint64_t P9A_OMIC_4_TX_CTLSM_CNTL6_EO_PG                         = 0x800D5C000801143Full;

static const uint64_t P9A_OMIC_5_TX_CTLSM_CNTL6_EO_PG                         = 0x800D5C000801183Full;


static const uint64_t P9A_MC_TX_CTLSM_CNTL7_EO_PG                             = 0x800D64000701103Full;
//DUPS: 800D64000701143F, 800D64000701183F,
static const uint64_t P9A_MC_0_TX_CTLSM_CNTL7_EO_PG                           = 0x800D64000701103Full;
//DUPS: 800D64000701143F, 800D64000701183F,
static const uint64_t P9A_MC_1_TX_CTLSM_CNTL7_EO_PG                           = 0x800D64000801103Full;
//DUPS: 800D64000801143F, 800D64000801183F,
static const uint64_t P9A_OMIC_TX_CTLSM_CNTL7_EO_PG                           = 0x800D64000701103Full;

static const uint64_t P9A_OMIC_0_TX_CTLSM_CNTL7_EO_PG                         = 0x800D64000701103Full;

static const uint64_t P9A_OMIC_1_TX_CTLSM_CNTL7_EO_PG                         = 0x800D64000701143Full;

static const uint64_t P9A_OMIC_2_TX_CTLSM_CNTL7_EO_PG                         = 0x800D64000701183Full;

static const uint64_t P9A_OMIC_3_TX_CTLSM_CNTL7_EO_PG                         = 0x800D64000801103Full;

static const uint64_t P9A_OMIC_4_TX_CTLSM_CNTL7_EO_PG                         = 0x800D64000801143Full;

static const uint64_t P9A_OMIC_5_TX_CTLSM_CNTL7_EO_PG                         = 0x800D64000801183Full;


static const uint64_t P9A_MC_TX_CTLSM_MODE1_EO_PG                             = 0x800D2C000701103Full;
//DUPS: 800D2C000701143F, 800D2C000701183F,
static const uint64_t P9A_MC_0_TX_CTLSM_MODE1_EO_PG                           = 0x800D2C000701103Full;
//DUPS: 800D2C000701143F, 800D2C000701183F,
static const uint64_t P9A_MC_1_TX_CTLSM_MODE1_EO_PG                           = 0x800D2C000801103Full;
//DUPS: 800D2C000801143F, 800D2C000801183F,
static const uint64_t P9A_OMIC_TX_CTLSM_MODE1_EO_PG                           = 0x800D2C000701103Full;

static const uint64_t P9A_OMIC_0_TX_CTLSM_MODE1_EO_PG                         = 0x800D2C000701103Full;

static const uint64_t P9A_OMIC_1_TX_CTLSM_MODE1_EO_PG                         = 0x800D2C000701143Full;

static const uint64_t P9A_OMIC_2_TX_CTLSM_MODE1_EO_PG                         = 0x800D2C000701183Full;

static const uint64_t P9A_OMIC_3_TX_CTLSM_MODE1_EO_PG                         = 0x800D2C000801103Full;

static const uint64_t P9A_OMIC_4_TX_CTLSM_MODE1_EO_PG                         = 0x800D2C000801143Full;

static const uint64_t P9A_OMIC_5_TX_CTLSM_MODE1_EO_PG                         = 0x800D2C000801183Full;


static const uint64_t P9A_MC_TX_CTLSM_SPARE_MODE_PG                           = 0x800D24000701103Full;
//DUPS: 800D24000701143F, 800D24000701183F,
static const uint64_t P9A_MC_0_TX_CTLSM_SPARE_MODE_PG                         = 0x800D24000701103Full;
//DUPS: 800D24000701143F, 800D24000701183F,
static const uint64_t P9A_MC_1_TX_CTLSM_SPARE_MODE_PG                         = 0x800D24000801103Full;
//DUPS: 800D24000801143F, 800D24000801183F,
static const uint64_t P9A_OMIC_TX_CTLSM_SPARE_MODE_PG                         = 0x800D24000701103Full;

static const uint64_t P9A_OMIC_0_TX_CTLSM_SPARE_MODE_PG                       = 0x800D24000701103Full;

static const uint64_t P9A_OMIC_1_TX_CTLSM_SPARE_MODE_PG                       = 0x800D24000701143Full;

static const uint64_t P9A_OMIC_2_TX_CTLSM_SPARE_MODE_PG                       = 0x800D24000701183Full;

static const uint64_t P9A_OMIC_3_TX_CTLSM_SPARE_MODE_PG                       = 0x800D24000801103Full;

static const uint64_t P9A_OMIC_4_TX_CTLSM_SPARE_MODE_PG                       = 0x800D24000801143Full;

static const uint64_t P9A_OMIC_5_TX_CTLSM_SPARE_MODE_PG                       = 0x800D24000801183Full;


static const uint64_t P9A_MC_TX_CTLSM_STAT1_EO_PG                             = 0x800D6C000701103Full;
//DUPS: 800D6C000701143F, 800D6C000701183F,
static const uint64_t P9A_MC_0_TX_CTLSM_STAT1_EO_PG                           = 0x800D6C000701103Full;
//DUPS: 800D6C000701143F, 800D6C000701183F,
static const uint64_t P9A_MC_1_TX_CTLSM_STAT1_EO_PG                           = 0x800D6C000801103Full;
//DUPS: 800D6C000801143F, 800D6C000801183F,
static const uint64_t P9A_OMIC_TX_CTLSM_STAT1_EO_PG                           = 0x800D6C000701103Full;

static const uint64_t P9A_OMIC_0_TX_CTLSM_STAT1_EO_PG                         = 0x800D6C000701103Full;

static const uint64_t P9A_OMIC_1_TX_CTLSM_STAT1_EO_PG                         = 0x800D6C000701143Full;

static const uint64_t P9A_OMIC_2_TX_CTLSM_STAT1_EO_PG                         = 0x800D6C000701183Full;

static const uint64_t P9A_OMIC_3_TX_CTLSM_STAT1_EO_PG                         = 0x800D6C000801103Full;

static const uint64_t P9A_OMIC_4_TX_CTLSM_STAT1_EO_PG                         = 0x800D6C000801143Full;

static const uint64_t P9A_OMIC_5_TX_CTLSM_STAT1_EO_PG                         = 0x800D6C000801183Full;


static const uint64_t P9A_MC_TX_CTL_CNTL10_EO_PG                              = 0x800CDC000701103Full;
//DUPS: 800CDC000701143F, 800CDC000701183F,
static const uint64_t P9A_MC_0_TX_CTL_CNTL10_EO_PG                            = 0x800CDC000701103Full;
//DUPS: 800CDC000701143F, 800CDC000701183F,
static const uint64_t P9A_MC_1_TX_CTL_CNTL10_EO_PG                            = 0x800CDC000801103Full;
//DUPS: 800CDC000801143F, 800CDC000801183F,
static const uint64_t P9A_OMIC_TX_CTL_CNTL10_EO_PG                            = 0x800CDC000701103Full;

static const uint64_t P9A_OMIC_0_TX_CTL_CNTL10_EO_PG                          = 0x800CDC000701103Full;

static const uint64_t P9A_OMIC_1_TX_CTL_CNTL10_EO_PG                          = 0x800CDC000701143Full;

static const uint64_t P9A_OMIC_2_TX_CTL_CNTL10_EO_PG                          = 0x800CDC000701183Full;

static const uint64_t P9A_OMIC_3_TX_CTL_CNTL10_EO_PG                          = 0x800CDC000801103Full;

static const uint64_t P9A_OMIC_4_TX_CTL_CNTL10_EO_PG                          = 0x800CDC000801143Full;

static const uint64_t P9A_OMIC_5_TX_CTL_CNTL10_EO_PG                          = 0x800CDC000801183Full;


static const uint64_t P9A_MC_TX_CTL_CNTL2_EO_PG                               = 0x800C2C000701103Full;
//DUPS: 800C2C000701143F, 800C2C000701183F,
static const uint64_t P9A_MC_0_TX_CTL_CNTL2_EO_PG                             = 0x800C2C000701103Full;
//DUPS: 800C2C000701143F, 800C2C000701183F,
static const uint64_t P9A_MC_1_TX_CTL_CNTL2_EO_PG                             = 0x800C2C000801103Full;
//DUPS: 800C2C000801143F, 800C2C000801183F,
static const uint64_t P9A_OMIC_TX_CTL_CNTL2_EO_PG                             = 0x800C2C000701103Full;

static const uint64_t P9A_OMIC_0_TX_CTL_CNTL2_EO_PG                           = 0x800C2C000701103Full;

static const uint64_t P9A_OMIC_1_TX_CTL_CNTL2_EO_PG                           = 0x800C2C000701143Full;

static const uint64_t P9A_OMIC_2_TX_CTL_CNTL2_EO_PG                           = 0x800C2C000701183Full;

static const uint64_t P9A_OMIC_3_TX_CTL_CNTL2_EO_PG                           = 0x800C2C000801103Full;

static const uint64_t P9A_OMIC_4_TX_CTL_CNTL2_EO_PG                           = 0x800C2C000801143Full;

static const uint64_t P9A_OMIC_5_TX_CTL_CNTL2_EO_PG                           = 0x800C2C000801183Full;


static const uint64_t P9A_MC_TX_CTL_CNTL3_EO_PG                               = 0x800C34000701103Full;
//DUPS: 800C34000701143F, 800C34000701183F,
static const uint64_t P9A_MC_0_TX_CTL_CNTL3_EO_PG                             = 0x800C34000701103Full;
//DUPS: 800C34000701143F, 800C34000701183F,
static const uint64_t P9A_MC_1_TX_CTL_CNTL3_EO_PG                             = 0x800C34000801103Full;
//DUPS: 800C34000801143F, 800C34000801183F,
static const uint64_t P9A_OMIC_TX_CTL_CNTL3_EO_PG                             = 0x800C34000701103Full;

static const uint64_t P9A_OMIC_0_TX_CTL_CNTL3_EO_PG                           = 0x800C34000701103Full;

static const uint64_t P9A_OMIC_1_TX_CTL_CNTL3_EO_PG                           = 0x800C34000701143Full;

static const uint64_t P9A_OMIC_2_TX_CTL_CNTL3_EO_PG                           = 0x800C34000701183Full;

static const uint64_t P9A_OMIC_3_TX_CTL_CNTL3_EO_PG                           = 0x800C34000801103Full;

static const uint64_t P9A_OMIC_4_TX_CTL_CNTL3_EO_PG                           = 0x800C34000801143Full;

static const uint64_t P9A_OMIC_5_TX_CTL_CNTL3_EO_PG                           = 0x800C34000801183Full;


static const uint64_t P9A_MC_TX_CTL_CNTL8_EO_PG                               = 0x800CCC000701103Full;
//DUPS: 800CCC000701143F, 800CCC000701183F,
static const uint64_t P9A_MC_0_TX_CTL_CNTL8_EO_PG                             = 0x800CCC000701103Full;
//DUPS: 800CCC000701143F, 800CCC000701183F,
static const uint64_t P9A_MC_1_TX_CTL_CNTL8_EO_PG                             = 0x800CCC000801103Full;
//DUPS: 800CCC000801143F, 800CCC000801183F,
static const uint64_t P9A_OMIC_TX_CTL_CNTL8_EO_PG                             = 0x800CCC000701103Full;

static const uint64_t P9A_OMIC_0_TX_CTL_CNTL8_EO_PG                           = 0x800CCC000701103Full;

static const uint64_t P9A_OMIC_1_TX_CTL_CNTL8_EO_PG                           = 0x800CCC000701143Full;

static const uint64_t P9A_OMIC_2_TX_CTL_CNTL8_EO_PG                           = 0x800CCC000701183Full;

static const uint64_t P9A_OMIC_3_TX_CTL_CNTL8_EO_PG                           = 0x800CCC000801103Full;

static const uint64_t P9A_OMIC_4_TX_CTL_CNTL8_EO_PG                           = 0x800CCC000801143Full;

static const uint64_t P9A_OMIC_5_TX_CTL_CNTL8_EO_PG                           = 0x800CCC000801183Full;


static const uint64_t P9A_MC_TX_CTL_CNTL9_EO_PG                               = 0x800CD4000701103Full;
//DUPS: 800CD4000701143F, 800CD4000701183F,
static const uint64_t P9A_MC_0_TX_CTL_CNTL9_EO_PG                             = 0x800CD4000701103Full;
//DUPS: 800CD4000701143F, 800CD4000701183F,
static const uint64_t P9A_MC_1_TX_CTL_CNTL9_EO_PG                             = 0x800CD4000801103Full;
//DUPS: 800CD4000801143F, 800CD4000801183F,
static const uint64_t P9A_OMIC_TX_CTL_CNTL9_EO_PG                             = 0x800CD4000701103Full;

static const uint64_t P9A_OMIC_0_TX_CTL_CNTL9_EO_PG                           = 0x800CD4000701103Full;

static const uint64_t P9A_OMIC_1_TX_CTL_CNTL9_EO_PG                           = 0x800CD4000701143Full;

static const uint64_t P9A_OMIC_2_TX_CTL_CNTL9_EO_PG                           = 0x800CD4000701183Full;

static const uint64_t P9A_OMIC_3_TX_CTL_CNTL9_EO_PG                           = 0x800CD4000801103Full;

static const uint64_t P9A_OMIC_4_TX_CTL_CNTL9_EO_PG                           = 0x800CD4000801143Full;

static const uint64_t P9A_OMIC_5_TX_CTL_CNTL9_EO_PG                           = 0x800CD4000801183Full;


static const uint64_t P9A_MC_TX_CTL_CNTLG1_EO_PG                              = 0x800C24000701103Full;
//DUPS: 800C24000701143F, 800C24000701183F,
static const uint64_t P9A_MC_0_TX_CTL_CNTLG1_EO_PG                            = 0x800C24000701103Full;
//DUPS: 800C24000701143F, 800C24000701183F,
static const uint64_t P9A_MC_1_TX_CTL_CNTLG1_EO_PG                            = 0x800C24000801103Full;
//DUPS: 800C24000801143F, 800C24000801183F,
static const uint64_t P9A_OMIC_TX_CTL_CNTLG1_EO_PG                            = 0x800C24000701103Full;

static const uint64_t P9A_OMIC_0_TX_CTL_CNTLG1_EO_PG                          = 0x800C24000701103Full;

static const uint64_t P9A_OMIC_1_TX_CTL_CNTLG1_EO_PG                          = 0x800C24000701143Full;

static const uint64_t P9A_OMIC_2_TX_CTL_CNTLG1_EO_PG                          = 0x800C24000701183Full;

static const uint64_t P9A_OMIC_3_TX_CTL_CNTLG1_EO_PG                          = 0x800C24000801103Full;

static const uint64_t P9A_OMIC_4_TX_CTL_CNTLG1_EO_PG                          = 0x800C24000801143Full;

static const uint64_t P9A_OMIC_5_TX_CTL_CNTLG1_EO_PG                          = 0x800C24000801183Full;


static const uint64_t P9A_MC_TX_CTL_MODE1_EO_PG                               = 0x800C14000701103Full;
//DUPS: 800C14000701143F, 800C14000701183F,
static const uint64_t P9A_MC_0_TX_CTL_MODE1_EO_PG                             = 0x800C14000701103Full;
//DUPS: 800C14000701143F, 800C14000701183F,
static const uint64_t P9A_MC_1_TX_CTL_MODE1_EO_PG                             = 0x800C14000801103Full;
//DUPS: 800C14000801143F, 800C14000801183F,
static const uint64_t P9A_OMIC_TX_CTL_MODE1_EO_PG                             = 0x800C14000701103Full;

static const uint64_t P9A_OMIC_0_TX_CTL_MODE1_EO_PG                           = 0x800C14000701103Full;

static const uint64_t P9A_OMIC_1_TX_CTL_MODE1_EO_PG                           = 0x800C14000701143Full;

static const uint64_t P9A_OMIC_2_TX_CTL_MODE1_EO_PG                           = 0x800C14000701183Full;

static const uint64_t P9A_OMIC_3_TX_CTL_MODE1_EO_PG                           = 0x800C14000801103Full;

static const uint64_t P9A_OMIC_4_TX_CTL_MODE1_EO_PG                           = 0x800C14000801143Full;

static const uint64_t P9A_OMIC_5_TX_CTL_MODE1_EO_PG                           = 0x800C14000801183Full;


static const uint64_t P9A_MC_TX_FIR_ERROR_INJECT_PG                           = 0x800D1C000701103Full;
//DUPS: 800D1C000701143F, 800D1C000701183F,
static const uint64_t P9A_MC_0_TX_FIR_ERROR_INJECT_PG                         = 0x800D1C000701103Full;
//DUPS: 800D1C000701143F, 800D1C000701183F,
static const uint64_t P9A_MC_1_TX_FIR_ERROR_INJECT_PG                         = 0x800D1C000801103Full;
//DUPS: 800D1C000801143F, 800D1C000801183F,
static const uint64_t P9A_OMIC_TX_FIR_ERROR_INJECT_PG                         = 0x800D1C000701103Full;

static const uint64_t P9A_OMIC_0_TX_FIR_ERROR_INJECT_PG                       = 0x800D1C000701103Full;

static const uint64_t P9A_OMIC_1_TX_FIR_ERROR_INJECT_PG                       = 0x800D1C000701143Full;

static const uint64_t P9A_OMIC_2_TX_FIR_ERROR_INJECT_PG                       = 0x800D1C000701183Full;

static const uint64_t P9A_OMIC_3_TX_FIR_ERROR_INJECT_PG                       = 0x800D1C000801103Full;

static const uint64_t P9A_OMIC_4_TX_FIR_ERROR_INJECT_PG                       = 0x800D1C000801143Full;

static const uint64_t P9A_OMIC_5_TX_FIR_ERROR_INJECT_PG                       = 0x800D1C000801183Full;


static const uint64_t P9A_MC_TX_FIR_MASK_PG                                   = 0x800D0C000701103Full;
//DUPS: 800D0C000701143F, 800D0C000701183F,
static const uint64_t P9A_MC_0_TX_FIR_MASK_PG                                 = 0x800D0C000701103Full;
//DUPS: 800D0C000701143F, 800D0C000701183F,
static const uint64_t P9A_MC_1_TX_FIR_MASK_PG                                 = 0x800D0C000801103Full;
//DUPS: 800D0C000801143F, 800D0C000801183F,
static const uint64_t P9A_OMIC_TX_FIR_MASK_PG                                 = 0x800D0C000701103Full;

static const uint64_t P9A_OMIC_0_TX_FIR_MASK_PG                               = 0x800D0C000701103Full;

static const uint64_t P9A_OMIC_1_TX_FIR_MASK_PG                               = 0x800D0C000701143Full;

static const uint64_t P9A_OMIC_2_TX_FIR_MASK_PG                               = 0x800D0C000701183Full;

static const uint64_t P9A_OMIC_3_TX_FIR_MASK_PG                               = 0x800D0C000801103Full;

static const uint64_t P9A_OMIC_4_TX_FIR_MASK_PG                               = 0x800D0C000801143Full;

static const uint64_t P9A_OMIC_5_TX_FIR_MASK_PG                               = 0x800D0C000801183Full;


static const uint64_t P9A_MC_TX_FIR_PG                                        = 0x800D04000701103Full;
//DUPS: 800D04000701143F, 800D04000701183F,
static const uint64_t P9A_MC_0_TX_FIR_PG                                      = 0x800D04000701103Full;
//DUPS: 800D04000701143F, 800D04000701183F,
static const uint64_t P9A_MC_1_TX_FIR_PG                                      = 0x800D04000801103Full;
//DUPS: 800D04000801143F, 800D04000801183F,
static const uint64_t P9A_OMIC_TX_FIR_PG                                      = 0x800D04000701103Full;

static const uint64_t P9A_OMIC_0_TX_FIR_PG                                    = 0x800D04000701103Full;

static const uint64_t P9A_OMIC_1_TX_FIR_PG                                    = 0x800D04000701143Full;

static const uint64_t P9A_OMIC_2_TX_FIR_PG                                    = 0x800D04000701183Full;

static const uint64_t P9A_OMIC_3_TX_FIR_PG                                    = 0x800D04000801103Full;

static const uint64_t P9A_OMIC_4_TX_FIR_PG                                    = 0x800D04000801143Full;

static const uint64_t P9A_OMIC_5_TX_FIR_PG                                    = 0x800D04000801183Full;


static const uint64_t P9A_MC_TX_FIR_RESET_PG                                  = 0x800D14000701103Full;
//DUPS: 800D14000701143F, 800D14000701183F,
static const uint64_t P9A_MC_0_TX_FIR_RESET_PG                                = 0x800D14000701103Full;
//DUPS: 800D14000701143F, 800D14000701183F,
static const uint64_t P9A_MC_1_TX_FIR_RESET_PG                                = 0x800D14000801103Full;
//DUPS: 800D14000801143F, 800D14000801183F,
static const uint64_t P9A_OMIC_TX_FIR_RESET_PG                                = 0x800D14000701103Full;

static const uint64_t P9A_OMIC_0_TX_FIR_RESET_PG                              = 0x800D14000701103Full;

static const uint64_t P9A_OMIC_1_TX_FIR_RESET_PG                              = 0x800D14000701143Full;

static const uint64_t P9A_OMIC_2_TX_FIR_RESET_PG                              = 0x800D14000701183Full;

static const uint64_t P9A_OMIC_3_TX_FIR_RESET_PG                              = 0x800D14000801103Full;

static const uint64_t P9A_OMIC_4_TX_FIR_RESET_PG                              = 0x800D14000801143Full;

static const uint64_t P9A_OMIC_5_TX_FIR_RESET_PG                              = 0x800D14000801183Full;


static const uint64_t P9A_MC_TX_ID1_PG                                        = 0x800C0C000701103Full;
//DUPS: 800C0C000701143F, 800C0C000701183F,
static const uint64_t P9A_MC_0_TX_ID1_PG                                      = 0x800C0C000701103Full;
//DUPS: 800C0C000701143F, 800C0C000701183F,
static const uint64_t P9A_MC_1_TX_ID1_PG                                      = 0x800C0C000801103Full;
//DUPS: 800C0C000801143F, 800C0C000801183F,
static const uint64_t P9A_OMIC_TX_ID1_PG                                      = 0x800C0C000701103Full;

static const uint64_t P9A_OMIC_0_TX_ID1_PG                                    = 0x800C0C000701103Full;

static const uint64_t P9A_OMIC_1_TX_ID1_PG                                    = 0x800C0C000701143Full;

static const uint64_t P9A_OMIC_2_TX_ID1_PG                                    = 0x800C0C000701183Full;

static const uint64_t P9A_OMIC_3_TX_ID1_PG                                    = 0x800C0C000801103Full;

static const uint64_t P9A_OMIC_4_TX_ID1_PG                                    = 0x800C0C000801143Full;

static const uint64_t P9A_OMIC_5_TX_ID1_PG                                    = 0x800C0C000801183Full;


static const uint64_t P9A_MC_TX_IMPCAL2_PB                                    = 0x800F3C000701103Full;
//DUPS: 800F3C000701143F, 800F3C000701183F,
static const uint64_t P9A_MC_0_TX_IMPCAL2_PB                                  = 0x800F3C000701103Full;
//DUPS: 800F3C000701143F, 800F3C000701183F,
static const uint64_t P9A_MC_1_TX_IMPCAL2_PB                                  = 0x800F3C000801103Full;
//DUPS: 800F3C000801143F, 800F3C000801183F,

static const uint64_t P9A_OMIC_TX_IMPCAL2_PB                                  = 0x800F3C000701103Full;

static const uint64_t P9A_OMIC_0_TX_IMPCAL2_PB                                = 0x800F3C000701103Full;

static const uint64_t P9A_OMIC_1_TX_IMPCAL2_PB                                = 0x800F3C000701143Full;

static const uint64_t P9A_OMIC_2_TX_IMPCAL2_PB                                = 0x800F3C000701183Full;

static const uint64_t P9A_OMIC_3_TX_IMPCAL2_PB                                = 0x800F3C000801103Full;

static const uint64_t P9A_OMIC_4_TX_IMPCAL2_PB                                = 0x800F3C000801143Full;

static const uint64_t P9A_OMIC_5_TX_IMPCAL2_PB                                = 0x800F3C000801183Full;


static const uint64_t P9A_MC_TX_IMPCAL_NVAL_PB                                = 0x800F0C000701103Full;
//DUPS: 800F0C000701143F, 800F0C000701183F,
static const uint64_t P9A_MC_0_TX_IMPCAL_NVAL_PB                              = 0x800F0C000701103Full;
//DUPS: 800F0C000701143F, 800F0C000701183F,
static const uint64_t P9A_MC_1_TX_IMPCAL_NVAL_PB                              = 0x800F0C000801103Full;
//DUPS: 800F0C000801143F, 800F0C000801183F,

static const uint64_t P9A_OMIC_TX_IMPCAL_NVAL_PB                              = 0x800F0C000701103Full;

static const uint64_t P9A_OMIC_0_TX_IMPCAL_NVAL_PB                            = 0x800F0C000701103Full;

static const uint64_t P9A_OMIC_1_TX_IMPCAL_NVAL_PB                            = 0x800F0C000701143Full;

static const uint64_t P9A_OMIC_2_TX_IMPCAL_NVAL_PB                            = 0x800F0C000701183Full;

static const uint64_t P9A_OMIC_3_TX_IMPCAL_NVAL_PB                            = 0x800F0C000801103Full;

static const uint64_t P9A_OMIC_4_TX_IMPCAL_NVAL_PB                            = 0x800F0C000801143Full;

static const uint64_t P9A_OMIC_5_TX_IMPCAL_NVAL_PB                            = 0x800F0C000801183Full;


static const uint64_t P9A_MC_TX_IMPCAL_PB                                     = 0x800F04000701103Full;
//DUPS: 800F04000701143F, 800F04000701183F,
static const uint64_t P9A_MC_0_TX_IMPCAL_PB                                   = 0x800F04000701103Full;
//DUPS: 800F04000701143F, 800F04000701183F,
static const uint64_t P9A_MC_1_TX_IMPCAL_PB                                   = 0x800F04000801103Full;
//DUPS: 800F04000801143F, 800F04000801183F,

static const uint64_t P9A_OMIC_TX_IMPCAL_PB                                   = 0x800F04000701103Full;

static const uint64_t P9A_OMIC_0_TX_IMPCAL_PB                                 = 0x800F04000701103Full;

static const uint64_t P9A_OMIC_1_TX_IMPCAL_PB                                 = 0x800F04000701143Full;

static const uint64_t P9A_OMIC_2_TX_IMPCAL_PB                                 = 0x800F04000701183Full;

static const uint64_t P9A_OMIC_3_TX_IMPCAL_PB                                 = 0x800F04000801103Full;

static const uint64_t P9A_OMIC_4_TX_IMPCAL_PB                                 = 0x800F04000801143Full;

static const uint64_t P9A_OMIC_5_TX_IMPCAL_PB                                 = 0x800F04000801183Full;


static const uint64_t P9A_MC_TX_IMPCAL_PVAL_PB                                = 0x800F14000701103Full;
//DUPS: 800F14000701143F, 800F14000701183F,
static const uint64_t P9A_MC_0_TX_IMPCAL_PVAL_PB                              = 0x800F14000701103Full;
//DUPS: 800F14000701143F, 800F14000701183F,
static const uint64_t P9A_MC_1_TX_IMPCAL_PVAL_PB                              = 0x800F14000801103Full;
//DUPS: 800F14000801143F, 800F14000801183F,

static const uint64_t P9A_OMIC_TX_IMPCAL_PVAL_PB                              = 0x800F14000701103Full;

static const uint64_t P9A_OMIC_0_TX_IMPCAL_PVAL_PB                            = 0x800F14000701103Full;

static const uint64_t P9A_OMIC_1_TX_IMPCAL_PVAL_PB                            = 0x800F14000701143Full;

static const uint64_t P9A_OMIC_2_TX_IMPCAL_PVAL_PB                            = 0x800F14000701183Full;

static const uint64_t P9A_OMIC_3_TX_IMPCAL_PVAL_PB                            = 0x800F14000801103Full;

static const uint64_t P9A_OMIC_4_TX_IMPCAL_PVAL_PB                            = 0x800F14000801143Full;

static const uint64_t P9A_OMIC_5_TX_IMPCAL_PVAL_PB                            = 0x800F14000801183Full;


static const uint64_t P9A_MC_TX_IMPCAL_P_4X_PB                                = 0x800F1C000701103Full;
//DUPS: 800F1C000701143F, 800F1C000701183F,
static const uint64_t P9A_MC_0_TX_IMPCAL_P_4X_PB                              = 0x800F1C000701103Full;
//DUPS: 800F1C000701143F, 800F1C000701183F,
static const uint64_t P9A_MC_1_TX_IMPCAL_P_4X_PB                              = 0x800F1C000801103Full;
//DUPS: 800F1C000801143F, 800F1C000801183F,

static const uint64_t P9A_OMIC_TX_IMPCAL_P_4X_PB                              = 0x800F1C000701103Full;

static const uint64_t P9A_OMIC_0_TX_IMPCAL_P_4X_PB                            = 0x800F1C000701103Full;

static const uint64_t P9A_OMIC_1_TX_IMPCAL_P_4X_PB                            = 0x800F1C000701143Full;

static const uint64_t P9A_OMIC_2_TX_IMPCAL_P_4X_PB                            = 0x800F1C000701183Full;

static const uint64_t P9A_OMIC_3_TX_IMPCAL_P_4X_PB                            = 0x800F1C000801103Full;

static const uint64_t P9A_OMIC_4_TX_IMPCAL_P_4X_PB                            = 0x800F1C000801143Full;

static const uint64_t P9A_OMIC_5_TX_IMPCAL_P_4X_PB                            = 0x800F1C000801183Full;


static const uint64_t P9A_MC_TX_IMPCAL_SWO1_PB                                = 0x800F24000701103Full;
//DUPS: 800F24000701143F, 800F24000701183F,
static const uint64_t P9A_MC_0_TX_IMPCAL_SWO1_PB                              = 0x800F24000701103Full;
//DUPS: 800F24000701143F, 800F24000701183F,
static const uint64_t P9A_MC_1_TX_IMPCAL_SWO1_PB                              = 0x800F24000801103Full;
//DUPS: 800F24000801143F, 800F24000801183F,

static const uint64_t P9A_OMIC_TX_IMPCAL_SWO1_PB                              = 0x800F24000701103Full;

static const uint64_t P9A_OMIC_0_TX_IMPCAL_SWO1_PB                            = 0x800F24000701103Full;

static const uint64_t P9A_OMIC_1_TX_IMPCAL_SWO1_PB                            = 0x800F24000701143Full;

static const uint64_t P9A_OMIC_2_TX_IMPCAL_SWO1_PB                            = 0x800F24000701183Full;

static const uint64_t P9A_OMIC_3_TX_IMPCAL_SWO1_PB                            = 0x800F24000801103Full;

static const uint64_t P9A_OMIC_4_TX_IMPCAL_SWO1_PB                            = 0x800F24000801143Full;

static const uint64_t P9A_OMIC_5_TX_IMPCAL_SWO1_PB                            = 0x800F24000801183Full;


static const uint64_t P9A_MC_TX_IMPCAL_SWO2_PB                                = 0x800F2C000701103Full;
//DUPS: 800F2C000701143F, 800F2C000701183F,
static const uint64_t P9A_MC_0_TX_IMPCAL_SWO2_PB                              = 0x800F2C000701103Full;
//DUPS: 800F2C000701143F, 800F2C000701183F,
static const uint64_t P9A_MC_1_TX_IMPCAL_SWO2_PB                              = 0x800F2C000801103Full;
//DUPS: 800F2C000801143F, 800F2C000801183F,

static const uint64_t P9A_OMIC_TX_IMPCAL_SWO2_PB                              = 0x800F2C000701103Full;

static const uint64_t P9A_OMIC_0_TX_IMPCAL_SWO2_PB                            = 0x800F2C000701103Full;

static const uint64_t P9A_OMIC_1_TX_IMPCAL_SWO2_PB                            = 0x800F2C000701143Full;

static const uint64_t P9A_OMIC_2_TX_IMPCAL_SWO2_PB                            = 0x800F2C000701183Full;

static const uint64_t P9A_OMIC_3_TX_IMPCAL_SWO2_PB                            = 0x800F2C000801103Full;

static const uint64_t P9A_OMIC_4_TX_IMPCAL_SWO2_PB                            = 0x800F2C000801143Full;

static const uint64_t P9A_OMIC_5_TX_IMPCAL_SWO2_PB                            = 0x800F2C000801183Full;


static const uint64_t P9A_MC_TX_SPARE_MODE_PG                                 = 0x800C04000701103Full;
//DUPS: 800C04000701143F, 800C04000701183F,
static const uint64_t P9A_MC_0_TX_SPARE_MODE_PG                               = 0x800C04000701103Full;
//DUPS: 800C04000701143F, 800C04000701183F,
static const uint64_t P9A_MC_1_TX_SPARE_MODE_PG                               = 0x800C04000801103Full;
//DUPS: 800C04000801143F, 800C04000801183F,
static const uint64_t P9A_OMIC_TX_SPARE_MODE_PG                               = 0x800C04000701103Full;

static const uint64_t P9A_OMIC_0_TX_SPARE_MODE_PG                             = 0x800C04000701103Full;

static const uint64_t P9A_OMIC_1_TX_SPARE_MODE_PG                             = 0x800C04000701143Full;

static const uint64_t P9A_OMIC_2_TX_SPARE_MODE_PG                             = 0x800C04000701183Full;

static const uint64_t P9A_OMIC_3_TX_SPARE_MODE_PG                             = 0x800C04000801103Full;

static const uint64_t P9A_OMIC_4_TX_SPARE_MODE_PG                             = 0x800C04000801143Full;

static const uint64_t P9A_OMIC_5_TX_SPARE_MODE_PG                             = 0x800C04000801183Full;


static const uint64_t P9A_MC_USTLCFG                                          = 0x07010A8Bull;
//DUPS: 07010ACB, 07010A0B, 07010A4B,
static const uint64_t P9A_MCC_USTLCFG                                         = 0x07010A0Bull;

static const uint64_t P9A_MCC_0_USTLCFG                                       = 0x07010A0Bull;

static const uint64_t P9A_MCC_1_USTLCFG                                       = 0x07010A4Bull;

static const uint64_t P9A_MCC_2_USTLCFG                                       = 0x07010A8Bull;

static const uint64_t P9A_MCC_3_USTLCFG                                       = 0x07010ACBull;

static const uint64_t P9A_MCC_4_USTLCFG                                       = 0x08010A0Bull;

static const uint64_t P9A_MCC_5_USTLCFG                                       = 0x08010A4Bull;

static const uint64_t P9A_MCC_6_USTLCFG                                       = 0x08010A8Bull;

static const uint64_t P9A_MCC_7_USTLCFG                                       = 0x08010ACBull;

static const uint64_t P9A_MC_0_USTLCFG                                        = 0x07010A8Bull;
//DUPS: 07010ACB, 07010A0B, 07010A4B,
static const uint64_t P9A_MC_1_USTLCFG                                        = 0x08010A8Bull;
//DUPS: 08010ACB, 08010A0B, 08010A4B,

static const uint64_t P9A_MC_USTLDBG                                          = 0x07010A8Dull;
//DUPS: 07010ACD, 07010A0D, 07010A4D,
static const uint64_t P9A_MCC_USTLDBG                                         = 0x07010A0Dull;

static const uint64_t P9A_MCC_0_USTLDBG                                       = 0x07010A0Dull;

static const uint64_t P9A_MCC_1_USTLDBG                                       = 0x07010A4Dull;

static const uint64_t P9A_MCC_2_USTLDBG                                       = 0x07010A8Dull;

static const uint64_t P9A_MCC_3_USTLDBG                                       = 0x07010ACDull;

static const uint64_t P9A_MCC_4_USTLDBG                                       = 0x08010A0Dull;

static const uint64_t P9A_MCC_5_USTLDBG                                       = 0x08010A4Dull;

static const uint64_t P9A_MCC_6_USTLDBG                                       = 0x08010A8Dull;

static const uint64_t P9A_MCC_7_USTLDBG                                       = 0x08010ACDull;

static const uint64_t P9A_MC_0_USTLDBG                                        = 0x07010A8Dull;
//DUPS: 07010ACD, 07010A0D, 07010A4D,
static const uint64_t P9A_MC_1_USTLDBG                                        = 0x08010A8Dull;
//DUPS: 08010ACD, 08010A0D, 08010A4D,

static const uint64_t P9A_MC_USTLFIR                                          = 0x07010A80ull;
//DUPS: 07010AC0, 07010A00, 07010A40,
static const uint64_t P9A_MC_USTLFIR_AND                                      = 0x07010A81ull;
//DUPS: 07010AC1, 07010A01, 07010A41,
static const uint64_t P9A_MC_USTLFIR_OR                                       = 0x07010A82ull;
//DUPS: 07010AC2, 07010A02, 07010A42,
static const uint64_t P9A_MCC_USTLFIR                                         = 0x07010A00ull;

static const uint64_t P9A_MCC_USTLFIR_AND                                     = 0x07010A01ull;

static const uint64_t P9A_MCC_USTLFIR_OR                                      = 0x07010A02ull;

static const uint64_t P9A_MCC_0_USTLFIR                                       = 0x07010A00ull;

static const uint64_t P9A_MCC_0_USTLFIR_AND                                   = 0x07010A01ull;

static const uint64_t P9A_MCC_0_USTLFIR_OR                                    = 0x07010A02ull;

static const uint64_t P9A_MCC_1_USTLFIR                                       = 0x07010A40ull;

static const uint64_t P9A_MCC_1_USTLFIR_AND                                   = 0x07010A41ull;

static const uint64_t P9A_MCC_1_USTLFIR_OR                                    = 0x07010A42ull;

static const uint64_t P9A_MCC_2_USTLFIR                                       = 0x07010A80ull;

static const uint64_t P9A_MCC_2_USTLFIR_AND                                   = 0x07010A81ull;

static const uint64_t P9A_MCC_2_USTLFIR_OR                                    = 0x07010A82ull;

static const uint64_t P9A_MCC_3_USTLFIR                                       = 0x07010AC0ull;

static const uint64_t P9A_MCC_3_USTLFIR_AND                                   = 0x07010AC1ull;

static const uint64_t P9A_MCC_3_USTLFIR_OR                                    = 0x07010AC2ull;

static const uint64_t P9A_MCC_4_USTLFIR                                       = 0x08010A00ull;

static const uint64_t P9A_MCC_4_USTLFIR_AND                                   = 0x08010A01ull;

static const uint64_t P9A_MCC_4_USTLFIR_OR                                    = 0x08010A02ull;

static const uint64_t P9A_MCC_5_USTLFIR                                       = 0x08010A40ull;

static const uint64_t P9A_MCC_5_USTLFIR_AND                                   = 0x08010A41ull;

static const uint64_t P9A_MCC_5_USTLFIR_OR                                    = 0x08010A42ull;

static const uint64_t P9A_MCC_6_USTLFIR                                       = 0x08010A80ull;

static const uint64_t P9A_MCC_6_USTLFIR_AND                                   = 0x08010A81ull;

static const uint64_t P9A_MCC_6_USTLFIR_OR                                    = 0x08010A82ull;

static const uint64_t P9A_MCC_7_USTLFIR                                       = 0x08010AC0ull;

static const uint64_t P9A_MCC_7_USTLFIR_AND                                   = 0x08010AC1ull;

static const uint64_t P9A_MCC_7_USTLFIR_OR                                    = 0x08010AC2ull;

static const uint64_t P9A_MC_0_USTLFIR                                        = 0x07010A80ull;
//DUPS: 07010AC0, 07010A00, 07010A40,
static const uint64_t P9A_MC_0_USTLFIR_AND                                    = 0x07010A81ull;
//DUPS: 07010AC1, 07010A01, 07010A41,
static const uint64_t P9A_MC_0_USTLFIR_OR                                     = 0x07010A82ull;
//DUPS: 07010AC2, 07010A02, 07010A42,
static const uint64_t P9A_MC_1_USTLFIR                                        = 0x08010A80ull;
//DUPS: 08010AC0, 08010A00, 08010A40,
static const uint64_t P9A_MC_1_USTLFIR_AND                                    = 0x08010A81ull;
//DUPS: 08010AC1, 08010A01, 08010A41,
static const uint64_t P9A_MC_1_USTLFIR_OR                                     = 0x08010A82ull;
//DUPS: 08010AC2, 08010A02, 08010A42,

static const uint64_t P9A_MC_USTLFIRACT0                                      = 0x07010A86ull;
//DUPS: 07010AC6, 07010A06, 07010A46,
static const uint64_t P9A_MCC_USTLFIRACT0                                     = 0x07010A06ull;

static const uint64_t P9A_MCC_0_USTLFIRACT0                                   = 0x07010A06ull;

static const uint64_t P9A_MCC_1_USTLFIRACT0                                   = 0x07010A46ull;

static const uint64_t P9A_MCC_2_USTLFIRACT0                                   = 0x07010A86ull;

static const uint64_t P9A_MCC_3_USTLFIRACT0                                   = 0x07010AC6ull;

static const uint64_t P9A_MCC_4_USTLFIRACT0                                   = 0x08010A06ull;

static const uint64_t P9A_MCC_5_USTLFIRACT0                                   = 0x08010A46ull;

static const uint64_t P9A_MCC_6_USTLFIRACT0                                   = 0x08010A86ull;

static const uint64_t P9A_MCC_7_USTLFIRACT0                                   = 0x08010AC6ull;

static const uint64_t P9A_MC_0_USTLFIRACT0                                    = 0x07010A86ull;
//DUPS: 07010AC6, 07010A06, 07010A46,
static const uint64_t P9A_MC_1_USTLFIRACT0                                    = 0x08010A86ull;
//DUPS: 08010AC6, 08010A06, 08010A46,

static const uint64_t P9A_MC_USTLFIRACT1                                      = 0x07010A87ull;
//DUPS: 07010AC7, 07010A07, 07010A47,
static const uint64_t P9A_MCC_USTLFIRACT1                                     = 0x07010A07ull;

static const uint64_t P9A_MCC_0_USTLFIRACT1                                   = 0x07010A07ull;

static const uint64_t P9A_MCC_1_USTLFIRACT1                                   = 0x07010A47ull;

static const uint64_t P9A_MCC_2_USTLFIRACT1                                   = 0x07010A87ull;

static const uint64_t P9A_MCC_3_USTLFIRACT1                                   = 0x07010AC7ull;

static const uint64_t P9A_MCC_4_USTLFIRACT1                                   = 0x08010A07ull;

static const uint64_t P9A_MCC_5_USTLFIRACT1                                   = 0x08010A47ull;

static const uint64_t P9A_MCC_6_USTLFIRACT1                                   = 0x08010A87ull;

static const uint64_t P9A_MCC_7_USTLFIRACT1                                   = 0x08010AC7ull;

static const uint64_t P9A_MC_0_USTLFIRACT1                                    = 0x07010A87ull;
//DUPS: 07010AC7, 07010A07, 07010A47,
static const uint64_t P9A_MC_1_USTLFIRACT1                                    = 0x08010A87ull;
//DUPS: 08010AC7, 08010A07, 08010A47,

static const uint64_t P9A_MC_USTLFIRMASK                                      = 0x07010A83ull;
//DUPS: 07010AC3, 07010A03, 07010A43,
static const uint64_t P9A_MC_USTLFIRMASK_AND                                  = 0x07010A84ull;
//DUPS: 07010AC4, 07010A04, 07010A44,
static const uint64_t P9A_MC_USTLFIRMASK_OR                                   = 0x07010A85ull;
//DUPS: 07010AC5, 07010A05, 07010A45,
static const uint64_t P9A_MCC_USTLFIRMASK                                     = 0x07010A03ull;

static const uint64_t P9A_MCC_USTLFIRMASK_AND                                 = 0x07010A04ull;

static const uint64_t P9A_MCC_USTLFIRMASK_OR                                  = 0x07010A05ull;

static const uint64_t P9A_MCC_0_USTLFIRMASK                                   = 0x07010A03ull;

static const uint64_t P9A_MCC_0_USTLFIRMASK_AND                               = 0x07010A04ull;

static const uint64_t P9A_MCC_0_USTLFIRMASK_OR                                = 0x07010A05ull;

static const uint64_t P9A_MCC_1_USTLFIRMASK                                   = 0x07010A43ull;

static const uint64_t P9A_MCC_1_USTLFIRMASK_AND                               = 0x07010A44ull;

static const uint64_t P9A_MCC_1_USTLFIRMASK_OR                                = 0x07010A45ull;

static const uint64_t P9A_MCC_2_USTLFIRMASK                                   = 0x07010A83ull;

static const uint64_t P9A_MCC_2_USTLFIRMASK_AND                               = 0x07010A84ull;

static const uint64_t P9A_MCC_2_USTLFIRMASK_OR                                = 0x07010A85ull;

static const uint64_t P9A_MCC_3_USTLFIRMASK                                   = 0x07010AC3ull;

static const uint64_t P9A_MCC_3_USTLFIRMASK_AND                               = 0x07010AC4ull;

static const uint64_t P9A_MCC_3_USTLFIRMASK_OR                                = 0x07010AC5ull;

static const uint64_t P9A_MCC_4_USTLFIRMASK                                   = 0x08010A03ull;

static const uint64_t P9A_MCC_4_USTLFIRMASK_AND                               = 0x08010A04ull;

static const uint64_t P9A_MCC_4_USTLFIRMASK_OR                                = 0x08010A05ull;

static const uint64_t P9A_MCC_5_USTLFIRMASK                                   = 0x08010A43ull;

static const uint64_t P9A_MCC_5_USTLFIRMASK_AND                               = 0x08010A44ull;

static const uint64_t P9A_MCC_5_USTLFIRMASK_OR                                = 0x08010A45ull;

static const uint64_t P9A_MCC_6_USTLFIRMASK                                   = 0x08010A83ull;

static const uint64_t P9A_MCC_6_USTLFIRMASK_AND                               = 0x08010A84ull;

static const uint64_t P9A_MCC_6_USTLFIRMASK_OR                                = 0x08010A85ull;

static const uint64_t P9A_MCC_7_USTLFIRMASK                                   = 0x08010AC3ull;

static const uint64_t P9A_MCC_7_USTLFIRMASK_AND                               = 0x08010AC4ull;

static const uint64_t P9A_MCC_7_USTLFIRMASK_OR                                = 0x08010AC5ull;

static const uint64_t P9A_MC_0_USTLFIRMASK                                    = 0x07010A83ull;
//DUPS: 07010AC3, 07010A03, 07010A43,
static const uint64_t P9A_MC_0_USTLFIRMASK_AND                                = 0x07010A84ull;
//DUPS: 07010AC4, 07010A04, 07010A44,
static const uint64_t P9A_MC_0_USTLFIRMASK_OR                                 = 0x07010A85ull;
//DUPS: 07010AC5, 07010A05, 07010A45,
static const uint64_t P9A_MC_1_USTLFIRMASK                                    = 0x08010A83ull;
//DUPS: 08010AC3, 08010A03, 08010A43,
static const uint64_t P9A_MC_1_USTLFIRMASK_AND                                = 0x08010A84ull;
//DUPS: 08010AC4, 08010A04, 08010A44,
static const uint64_t P9A_MC_1_USTLFIRMASK_OR                                 = 0x08010A85ull;
//DUPS: 08010AC5, 08010A05, 08010A45,

static const uint64_t P9A_MC_USTLFIRWOF                                       = 0x07010A88ull;
//DUPS: 07010AC8, 07010A08, 07010A48,
static const uint64_t P9A_MCC_USTLFIRWOF                                      = 0x07010A08ull;

static const uint64_t P9A_MCC_0_USTLFIRWOF                                    = 0x07010A08ull;

static const uint64_t P9A_MCC_1_USTLFIRWOF                                    = 0x07010A48ull;

static const uint64_t P9A_MCC_2_USTLFIRWOF                                    = 0x07010A88ull;

static const uint64_t P9A_MCC_3_USTLFIRWOF                                    = 0x07010AC8ull;

static const uint64_t P9A_MCC_4_USTLFIRWOF                                    = 0x08010A08ull;

static const uint64_t P9A_MCC_5_USTLFIRWOF                                    = 0x08010A48ull;

static const uint64_t P9A_MCC_6_USTLFIRWOF                                    = 0x08010A88ull;

static const uint64_t P9A_MCC_7_USTLFIRWOF                                    = 0x08010AC8ull;

static const uint64_t P9A_MC_0_USTLFIRWOF                                     = 0x07010A88ull;
//DUPS: 07010AC8, 07010A08, 07010A48,
static const uint64_t P9A_MC_1_USTLFIRWOF                                     = 0x08010A88ull;
//DUPS: 08010AC8, 08010A08, 08010A48,

static const uint64_t P9A_MC_USTLINJ                                          = 0x07010A8Cull;
//DUPS: 07010ACC, 07010A0C, 07010A4C,
static const uint64_t P9A_MCC_USTLINJ                                         = 0x07010A0Cull;

static const uint64_t P9A_MCC_0_USTLINJ                                       = 0x07010A0Cull;

static const uint64_t P9A_MCC_1_USTLINJ                                       = 0x07010A4Cull;

static const uint64_t P9A_MCC_2_USTLINJ                                       = 0x07010A8Cull;

static const uint64_t P9A_MCC_3_USTLINJ                                       = 0x07010ACCull;

static const uint64_t P9A_MCC_4_USTLINJ                                       = 0x08010A0Cull;

static const uint64_t P9A_MCC_5_USTLINJ                                       = 0x08010A4Cull;

static const uint64_t P9A_MCC_6_USTLINJ                                       = 0x08010A8Cull;

static const uint64_t P9A_MCC_7_USTLINJ                                       = 0x08010ACCull;

static const uint64_t P9A_MC_0_USTLINJ                                        = 0x07010A8Cull;
//DUPS: 07010ACC, 07010A0C, 07010A4C,
static const uint64_t P9A_MC_1_USTLINJ                                        = 0x08010A8Cull;
//DUPS: 08010ACC, 08010A0C, 08010A4C,

static const uint64_t P9A_MC_USTLSTATUS                                       = 0x07010A8Aull;
//DUPS: 07010ACA, 07010A0A, 07010A4A,
static const uint64_t P9A_MCC_USTLSTATUS                                      = 0x07010A0Aull;

static const uint64_t P9A_MCC_0_USTLSTATUS                                    = 0x07010A0Aull;

static const uint64_t P9A_MCC_1_USTLSTATUS                                    = 0x07010A4Aull;

static const uint64_t P9A_MCC_2_USTLSTATUS                                    = 0x07010A8Aull;

static const uint64_t P9A_MCC_3_USTLSTATUS                                    = 0x07010ACAull;

static const uint64_t P9A_MCC_4_USTLSTATUS                                    = 0x08010A0Aull;

static const uint64_t P9A_MCC_5_USTLSTATUS                                    = 0x08010A4Aull;

static const uint64_t P9A_MCC_6_USTLSTATUS                                    = 0x08010A8Aull;

static const uint64_t P9A_MCC_7_USTLSTATUS                                    = 0x08010ACAull;

static const uint64_t P9A_MC_0_USTLSTATUS                                     = 0x07010A8Aull;
//DUPS: 07010ACA, 07010A0A, 07010A4A,
static const uint64_t P9A_MC_1_USTLSTATUS                                     = 0x08010A8Aull;
//DUPS: 08010ACA, 08010A0A, 08010A4A,

static const uint64_t P9A_MC_VITAL_SCAN_OUT                                   = 0x070F0017ull;

static const uint64_t P9A_MC_0_VITAL_SCAN_OUT                                 = 0x070F0017ull;

static const uint64_t P9A_MC_1_VITAL_SCAN_OUT                                 = 0x080F0017ull;


static const uint64_t P9A_MC_WATCFG0AQ                                        = 0x07012380ull;

static const uint64_t P9A_MC_0_WATCFG0AQ                                      = 0x07012380ull;

static const uint64_t P9A_MC_1_WATCFG0AQ                                      = 0x08012380ull;


static const uint64_t P9A_MC_WATCFG0BQ                                        = 0x07012381ull;

static const uint64_t P9A_MC_0_WATCFG0BQ                                      = 0x07012381ull;

static const uint64_t P9A_MC_1_WATCFG0BQ                                      = 0x08012381ull;


static const uint64_t P9A_MC_WATCFG0CQ                                        = 0x07012382ull;

static const uint64_t P9A_MC_0_WATCFG0CQ                                      = 0x07012382ull;

static const uint64_t P9A_MC_1_WATCFG0CQ                                      = 0x08012382ull;


static const uint64_t P9A_MC_WATCFG0DQ                                        = 0x07012383ull;

static const uint64_t P9A_MC_0_WATCFG0DQ                                      = 0x07012383ull;

static const uint64_t P9A_MC_1_WATCFG0DQ                                      = 0x08012383ull;


static const uint64_t P9A_MC_WATCFG0EQ                                        = 0x07012384ull;

static const uint64_t P9A_MC_0_WATCFG0EQ                                      = 0x07012384ull;

static const uint64_t P9A_MC_1_WATCFG0EQ                                      = 0x08012384ull;


static const uint64_t P9A_MC_WATCFG1AQ                                        = 0x07012385ull;

static const uint64_t P9A_MC_0_WATCFG1AQ                                      = 0x07012385ull;

static const uint64_t P9A_MC_1_WATCFG1AQ                                      = 0x08012385ull;


static const uint64_t P9A_MC_WATCFG1BQ                                        = 0x07012386ull;

static const uint64_t P9A_MC_0_WATCFG1BQ                                      = 0x07012386ull;

static const uint64_t P9A_MC_1_WATCFG1BQ                                      = 0x08012386ull;


static const uint64_t P9A_MC_WATCFG1CQ                                        = 0x07012387ull;

static const uint64_t P9A_MC_0_WATCFG1CQ                                      = 0x07012387ull;

static const uint64_t P9A_MC_1_WATCFG1CQ                                      = 0x08012387ull;


static const uint64_t P9A_MC_WATCFG1DQ                                        = 0x07012388ull;

static const uint64_t P9A_MC_0_WATCFG1DQ                                      = 0x07012388ull;

static const uint64_t P9A_MC_1_WATCFG1DQ                                      = 0x08012388ull;


static const uint64_t P9A_MC_WATCFG1EQ                                        = 0x07012389ull;

static const uint64_t P9A_MC_0_WATCFG1EQ                                      = 0x07012389ull;

static const uint64_t P9A_MC_1_WATCFG1EQ                                      = 0x08012389ull;


static const uint64_t P9A_MC_WATCFG2AQ                                        = 0x0701238Aull;

static const uint64_t P9A_MC_0_WATCFG2AQ                                      = 0x0701238Aull;

static const uint64_t P9A_MC_1_WATCFG2AQ                                      = 0x0801238Aull;


static const uint64_t P9A_MC_WATCFG2BQ                                        = 0x0701238Bull;

static const uint64_t P9A_MC_0_WATCFG2BQ                                      = 0x0701238Bull;

static const uint64_t P9A_MC_1_WATCFG2BQ                                      = 0x0801238Bull;


static const uint64_t P9A_MC_WATCFG2CQ                                        = 0x0701238Cull;

static const uint64_t P9A_MC_0_WATCFG2CQ                                      = 0x0701238Cull;

static const uint64_t P9A_MC_1_WATCFG2CQ                                      = 0x0801238Cull;


static const uint64_t P9A_MC_WATCFG2DQ                                        = 0x0701238Dull;

static const uint64_t P9A_MC_0_WATCFG2DQ                                      = 0x0701238Dull;

static const uint64_t P9A_MC_1_WATCFG2DQ                                      = 0x0801238Dull;


static const uint64_t P9A_MC_WATCFG2EQ                                        = 0x0701238Eull;

static const uint64_t P9A_MC_0_WATCFG2EQ                                      = 0x0701238Eull;

static const uint64_t P9A_MC_1_WATCFG2EQ                                      = 0x0801238Eull;


static const uint64_t P9A_MC_WATCFG3AQ                                        = 0x0701238Full;

static const uint64_t P9A_MC_0_WATCFG3AQ                                      = 0x0701238Full;

static const uint64_t P9A_MC_1_WATCFG3AQ                                      = 0x0801238Full;


static const uint64_t P9A_MC_WATCFG3BQ                                        = 0x07012390ull;

static const uint64_t P9A_MC_0_WATCFG3BQ                                      = 0x07012390ull;

static const uint64_t P9A_MC_1_WATCFG3BQ                                      = 0x08012390ull;


static const uint64_t P9A_MC_WATCFG3CQ                                        = 0x07012391ull;

static const uint64_t P9A_MC_0_WATCFG3CQ                                      = 0x07012391ull;

static const uint64_t P9A_MC_1_WATCFG3CQ                                      = 0x08012391ull;


static const uint64_t P9A_MC_WATCFG3DQ                                        = 0x07012392ull;

static const uint64_t P9A_MC_0_WATCFG3DQ                                      = 0x07012392ull;

static const uint64_t P9A_MC_1_WATCFG3DQ                                      = 0x08012392ull;


static const uint64_t P9A_MC_WATCFG3EQ                                        = 0x07012393ull;

static const uint64_t P9A_MC_0_WATCFG3EQ                                      = 0x07012393ull;

static const uint64_t P9A_MC_1_WATCFG3EQ                                      = 0x08012393ull;


static const uint64_t P9A_MC_WBMGR_TAG_INFO                                   = 0x07012343ull;
//DUPS: 07012373, 07012343, 07012353,
static const uint64_t P9A_MC_0_WBMGR_TAG_INFO                                 = 0x07012343ull;
//DUPS: 07012373, 07012343, 07012353,
static const uint64_t P9A_MC_1_WBMGR_TAG_INFO                                 = 0x08012343ull;
//DUPS: 08012373, 08012343, 08012353,

static const uint64_t P9A_MC_WDFCFG                                           = 0x07012340ull;
//DUPS: 07012370, 07012340, 07012350,
static const uint64_t P9A_MC_0_WDFCFG                                         = 0x07012340ull;
//DUPS: 07012370, 07012340, 07012350,
static const uint64_t P9A_MC_1_WDFCFG                                         = 0x08012340ull;
//DUPS: 08012370, 08012340, 08012350,

static const uint64_t P9A_MC_WDFDBG                                           = 0x07012344ull;
//DUPS: 07012374, 07012344, 07012354,
static const uint64_t P9A_MC_0_WDFDBG                                         = 0x07012344ull;
//DUPS: 07012374, 07012344, 07012354,
static const uint64_t P9A_MC_1_WDFDBG                                         = 0x08012344ull;
//DUPS: 08012374, 08012344, 08012354,

static const uint64_t P9A_MC_WDF_DSTL_ECC                                     = 0x07012345ull;
//DUPS: 07012375, 07012345, 07012355,
static const uint64_t P9A_MC_0_WDF_DSTL_ECC                                   = 0x07012345ull;
//DUPS: 07012375, 07012345, 07012355,
static const uint64_t P9A_MC_1_WDF_DSTL_ECC                                   = 0x08012345ull;
//DUPS: 08012375, 08012345, 08012355,

static const uint64_t P9A_MC_WDF_ECC                                          = 0x07012341ull;
//DUPS: 07012371, 07012341, 07012351,
static const uint64_t P9A_MC_0_WDF_ECC                                        = 0x07012341ull;
//DUPS: 07012371, 07012341, 07012351,
static const uint64_t P9A_MC_1_WDF_ECC                                        = 0x08012341ull;
//DUPS: 08012371, 08012341, 08012351,

static const uint64_t P9A_MC_WRITE_PROTECT_ENABLE_REG                         = 0x07010005ull;

static const uint64_t P9A_MC_0_WRITE_PROTECT_ENABLE_REG                       = 0x07010005ull;

static const uint64_t P9A_MC_1_WRITE_PROTECT_ENABLE_REG                       = 0x08010005ull;


static const uint64_t P9A_MC_WRITE_PROTECT_RINGS_REG                          = 0x07010006ull;

static const uint64_t P9A_MC_0_WRITE_PROTECT_RINGS_REG                        = 0x07010006ull;

static const uint64_t P9A_MC_1_WRITE_PROTECT_RINGS_REG                        = 0x08010006ull;


static const uint64_t P9A_MC_WRTCFG                                           = 0x07012348ull;
//DUPS: 07012378, 07012348, 07012358,
static const uint64_t P9A_MC_0_WRTCFG                                         = 0x07012348ull;
//DUPS: 07012378, 07012348, 07012358,
static const uint64_t P9A_MC_1_WRTCFG                                         = 0x08012348ull;
//DUPS: 08012378, 08012348, 08012358,

static const uint64_t P9A_MC_WRTDBGMCA                                        = 0x0701234Aull;
//DUPS: 0701237A, 0701234A, 0701235A,
static const uint64_t P9A_MC_0_WRTDBGMCA                                      = 0x0701234Aull;
//DUPS: 0701237A, 0701234A, 0701235A,
static const uint64_t P9A_MC_1_WRTDBGMCA                                      = 0x0801234Aull;
//DUPS: 0801237A, 0801234A, 0801235A,

static const uint64_t P9A_MC_WRTDBGNEST                                       = 0x0701234Bull;
//DUPS: 0701237B, 0701234B, 0701235B,
static const uint64_t P9A_MC_0_WRTDBGNEST                                     = 0x0701234Bull;
//DUPS: 0701237B, 0701234B, 0701235B,
static const uint64_t P9A_MC_1_WRTDBGNEST                                     = 0x0801234Bull;
//DUPS: 0801237B, 0801234B, 0801235B,

static const uint64_t P9A_MC_WRT_DSTL_ECC                                     = 0x0701234Cull;
//DUPS: 0701237C, 0701234C, 0701235C,
static const uint64_t P9A_MC_0_WRT_DSTL_ECC                                   = 0x0701234Cull;
//DUPS: 0701237C, 0701234C, 0701235C,
static const uint64_t P9A_MC_1_WRT_DSTL_ECC                                   = 0x0801234Cull;
//DUPS: 0801237C, 0801234C, 0801235C,

static const uint64_t P9A_MC_WRT_WDF_ECC                                      = 0x07012349ull;
//DUPS: 07012379, 07012349, 07012359,
static const uint64_t P9A_MC_0_WRT_WDF_ECC                                    = 0x07012349ull;
//DUPS: 07012379, 07012349, 07012359,
static const uint64_t P9A_MC_1_WRT_WDF_ECC                                    = 0x08012349ull;
//DUPS: 08012379, 08012349, 08012359,

static const uint64_t P9A_MC_XFIR                                             = 0x07040000ull;

static const uint64_t P9A_MC_0_XFIR                                           = 0x07040000ull;

static const uint64_t P9A_MC_1_XFIR                                           = 0x08040000ull;


static const uint64_t P9A_MC_XSTOP1                                           = 0x0703000Cull;

static const uint64_t P9A_MC_0_XSTOP1                                         = 0x0703000Cull;

static const uint64_t P9A_MC_1_XSTOP1                                         = 0x0803000Cull;


static const uint64_t P9A_MC_XSTOP2                                           = 0x0703000Dull;

static const uint64_t P9A_MC_0_XSTOP2                                         = 0x0703000Dull;

static const uint64_t P9A_MC_1_XSTOP2                                         = 0x0803000Dull;


static const uint64_t P9A_MC_XSTOP3                                           = 0x0703000Eull;

static const uint64_t P9A_MC_0_XSTOP3                                         = 0x0703000Eull;

static const uint64_t P9A_MC_1_XSTOP3                                         = 0x0803000Eull;


static const uint64_t P9A_MC_XSTOP_INTERRUPT_REG                              = 0x070F001Cull;

static const uint64_t P9A_MC_0_XSTOP_INTERRUPT_REG                            = 0x070F001Cull;

static const uint64_t P9A_MC_1_XSTOP_INTERRUPT_REG                            = 0x080F001Cull;


static const uint64_t P9A_MC_XTRA_TRACE_MODE                                  = 0x070107D1ull;

static const uint64_t P9A_MC_0_XTRA_TRACE_MODE                                = 0x070107D1ull;

static const uint64_t P9A_MC_1_XTRA_TRACE_MODE                                = 0x080107D1ull;

#endif
